﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using Nabu.IO;

namespace CompressionExample.FF
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
		}

		private void ZipArchiveReader_FileLocalHeader(object sender, ZipArchiveReaderFileLocalHeaderEventArgs e)
		{
			if (e.DecompressedSize == 0)
			{
				Directory.CreateDirectory(
					this._folderBrowserDialog.SelectedPath + Path.DirectorySeparatorChar + e.FilePath);
			}
			else
			{
				using (Stream fileStream = File.Create(this._folderBrowserDialog.SelectedPath + Path.DirectorySeparatorChar + e.FilePath))
				{
					using (Stream zipStream = e.GetDecompressedStream())
					{
						byte[] buffer = new byte[65536];

						for (int position = 0; position < e.DecompressedSize; )
						{
							int read =
								zipStream.Read(
									buffer,
									0,
									(int)Math.Min(e.DecompressedSize - position, buffer.Length));

							fileStream.Write(buffer, 0, read);

							position += read;
						}
					}
				}
			}
		}

		private void MSCabArchiveReader_File(object sender, MSCabFileEventArgs e)
		{
			using (Stream fileStream = File.Create(this._folderBrowserDialog.SelectedPath + Path.DirectorySeparatorChar + e.FilePath))
			{
				using (Stream cabStream = e.GetDecompressedStream())
				{
					byte[] buffer = new byte[65536];

					for (int position = 0; position < e.DecompressedSize; )
					{
						int read =
							cabStream.Read(
								buffer,
								0,
								(int)Math.Min(e.DecompressedSize - position, buffer.Length));

						fileStream.Write(buffer, 0, read);

						position += read;
					}
				}
			}
		}

		private void ButtonCreateZip_Click(object sender, EventArgs e)
		{
			this._textLog.Clear();

			List<string> paths = new List<string>();

			while (this._openAnyFileDialog.ShowDialog(this) == DialogResult.OK)
			{
				paths.AddRange(this._openAnyFileDialog.FileNames);
			}

			if ((paths.Count > 0) && (this._saveZipFileDialog.ShowDialog(this) == DialogResult.OK))
			{
				using (ZipArchiveWriter zipArchiveWriter = new ZipArchiveWriter())
				{
					foreach (string path in paths)
					{
						zipArchiveWriter.Entries.Add(
							new ZipArchiveWriterEntry(
								path, Path.GetFileName(path)));
					}

					using (Stream zipStream = File.Create(this._saveZipFileDialog.FileName))
					{
						zipArchiveWriter.Write(zipStream);
					}
				}
			}
		}

		private void ButtonExtractZIP_Click(object sender, EventArgs e)
		{
			if (
				(this._openZipFileDialog.ShowDialog(this) == DialogResult.OK) &&
				(this._folderBrowserDialog.ShowDialog(this) == DialogResult.OK))
			{
				ZipArchiveReader zipArchiveReader = new ZipArchiveReader();

				zipArchiveReader.FileLocalHeader += new EventHandler<ZipArchiveReaderFileLocalHeaderEventArgs>(ZipArchiveReader_FileLocalHeader);

				using (Stream zipStream = File.OpenRead(this._openZipFileDialog.FileName))
				{
					zipArchiveReader.Read(zipStream);
				}
			}
		}

		private void ButtonExtractMSCab_Click(object sender, EventArgs e)
		{
			if (
				(this._openCabFileDialog.ShowDialog(this) == DialogResult.OK) &&
				(this._folderBrowserDialog.ShowDialog(this) == DialogResult.OK))
			{
				MSCabArchiveReader msCabArchiveReader = new MSCabArchiveReader();

				msCabArchiveReader.File += new EventHandler<MSCabFileEventArgs>(MSCabArchiveReader_File);

				using (Stream cabStream = File.OpenRead(this._openCabFileDialog.FileName))
				{
					msCabArchiveReader.Read(cabStream);
				}
			}
		}
	}
}
