﻿using System;
using System.Drawing;
using System.Windows.Forms;

using Nabu.Data;
using Nabu.Forms.TextObjectModel;

namespace DiffExample
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
			UpdateView();
		}

		private void UpdateView()
		{
			string[] linesLeft = this._richTextLeft.Lines;
			string[] linesRight = this._richTextRight.Lines;

			DiffHirschbergText diff =
				new DiffHirschbergText(
					linesLeft,
					linesRight,
					StringComparison.OrdinalIgnoreCase);
			PatchOperationList<string> poList = new PatchOperationList<string>();

			diff.BuildPatch(
				poList,
				0,
				linesLeft.Length,
				0,
				linesRight.Length);

			IRichTextDocument textDocumentLeft = this._richTextLeft.TextDocument;
			IRichTextDocument textDocumentRight = this._richTextRight.TextDocument;
			IRichTextRange rangeLeft = textDocumentLeft.Range(0, this._richTextLeft.TextLength);
			IRichTextRange rangeRight = textDocumentRight.Range(0, this._richTextRight.TextLength);

			rangeLeft.Font.BackColor = ColorRTF.FromColor(Color.White);
			rangeRight.Font.BackColor = ColorRTF.FromColor(Color.White);

			int lineIndexLeft = 0;
			int lineIndexRight = 0;
			int charIndexLeft = 0;
			int charIndexRight = 0;

			foreach (PatchOperation<string> po in poList)
			{
				switch (po.Type)
				{
					case PatchOperationType.Copy:
						{
							for (int index = 0; index < po.Length; index++)
							{
								int lineLengthLeft = linesLeft[lineIndexLeft].Length + 1;
								int lineLengthRight = linesRight[lineIndexRight].Length + 1;

								lineIndexLeft++;
								lineIndexRight++;
								charIndexLeft += lineLengthLeft;
								charIndexRight += lineLengthRight;
							}
						}
						break;
					case PatchOperationType.Insert:
						{
							int rangeFrom = charIndexRight;
							int rangeTo = rangeFrom;

							for (int index = 0; index < po.Length; index++)
							{
								int lineLengthRight = linesRight[lineIndexRight].Length + 1;

								rangeTo = charIndexRight + lineLengthRight;
								lineIndexRight++;
								charIndexRight += lineLengthRight;
							}

							rangeRight = textDocumentRight.Range(rangeFrom, rangeTo);
							rangeRight.Font.BackColor = ColorRTF.FromColor(Color.LightSteelBlue);
						}
						break;
					case PatchOperationType.Delete:
						{
							int rangeFrom = charIndexLeft;
							int rangeTo = rangeFrom;

							for (int index = 0; index < po.Length; index++)
							{
								int lineLengthLeft = linesLeft[lineIndexLeft].Length + 1;

								rangeTo = charIndexLeft + lineLengthLeft;
								lineIndexLeft++;
								charIndexLeft += lineLengthLeft;
							}

							rangeLeft = textDocumentLeft.Range(rangeFrom, rangeTo);
							rangeLeft.Font.BackColor = ColorRTF.FromColor(Color.Pink);
						}
						break;
				}
			}
		}

		private void RichText_TextChanged(object sender, EventArgs e)
		{
			UpdateView();
		}

		private void ButtonClose_Click(object sender, EventArgs e)
		{
			base.Close();
		}
	}
}