﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Nabu.Collections;
using Nabu.Forms.Html;

namespace HtmlControlsExample
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
		}

		private void ButtonReadValues_Click(object sender, EventArgs e)
		{
			StringBuilder message = new StringBuilder();

			using (DisposableList<HtmlTag> inputs = this._htmlView.RootTag.Select("[id]"))
			{
				foreach (HtmlTag input in inputs)
				{
					switch (input.ControlType)
					{
						case HtmlTagControlType.Form:
							using (HtmlControl control = new HtmlControl(input))
							{
								object value = control.Value;
							}
							break;
						case HtmlTagControlType.TextBox_SingleLine:
						case HtmlTagControlType.TextBox_Numeric:
						case HtmlTagControlType.TextBox_MultiLine:
						case HtmlTagControlType.TextBox_Password:
						case HtmlTagControlType.TextBox_Decimal:
						case HtmlTagControlType.TextBox_Currency:
						case HtmlTagControlType.TextBox_Hyperlink:
						case HtmlTagControlType.TextBox_Url:
							using (HtmlControl control = new HtmlControl(input))
							{
								object value = control.Value;

								if (value is string)
								{
									value = ((string)value).TrimEnd('\0');
								}

								message.AppendFormat(
									"{0}({1}) = '{2}'/'{3}'",
									input.Attributes["id"],
									input.ControlType,
									(value == null) ? "null" : value.GetType().Name,
									value);
								message.AppendLine();
							}
							break;
						case HtmlTagControlType.RichText:
							using (HtmlControl control = new HtmlControl(input))
							{
								object value = control.Value;

								if (value is string)
								{
									value = ((string)value).TrimEnd('\0');
								}

								message.AppendFormat(
									"{0}({1}) = ({2}) '{3}'",
									input.Attributes["id"],
									input.ControlType,
									(value == null) ? "null" : value.GetType().Name,
									value);
								message.AppendLine();
							}
							break;
						case HtmlTagControlType.Button_Radio:
						case HtmlTagControlType.Button_Check:
							using (HtmlControl control = new HtmlControl(input))
							{
								object value = control.Value;

								message.AppendFormat(
									"{0}={1}({2}) = {3}/({4}) '{5}'",
									input.Attributes["name"],
									input.Attributes["value"],
									input.ControlType,
									input.GetState(HtmlTagState.Checked),
									(value == null) ? "null" : value.GetType().Name,
									value);
								message.AppendLine();
							}
							break;
						case HtmlTagControlType.Button_Push:
							message.AppendLine("Button_Push");
							break;
						case HtmlTagControlType.Select_Single:
						case HtmlTagControlType.Select_Multiple:
						case HtmlTagControlType.Select_DropDown:
							using (HtmlControl control = new HtmlControl(input))
							{
								using (HtmlSelect select = new HtmlSelect(input))
								{
									object value = control.Value;

									message.AppendFormat(
										"{0}({1})=",
										input.Attributes["id"],
										input.ControlType);

									foreach (HtmlTag option in select.SelectedOptions)
									{
										message.AppendFormat(
											"{0};",
											option.Attributes["value"]);
									}

									message.AppendFormat(
										"/({0}) '{1}'",
										(value == null) ? "null" : value.GetType().Name,
										value);

									message.AppendLine();
								}
							}
							break;
						case HtmlTagControlType.DateTime_Calendar:
						case HtmlTagControlType.DateTime_Date:
						case HtmlTagControlType.DateTime_Time:
							using (HtmlControl control = new HtmlControl(input))
							{
								object value = control.Value;

								message.AppendFormat(
									"{0}({1}) = ({2}) '{3}'",
									input.Attributes["id"],
									input.ControlType,
									(value == null) ? "null" : value.GetType().Name,
									value);
								message.AppendLine();
							}
							break;
						case HtmlTagControlType.Progress:
							using (HtmlControl control = new HtmlControl(input))
							{
								object value = control.Value;

								message.AppendFormat(
									"{0}({1}) = ({2}) '{3}'",
									input.Attributes["id"],
									input.ControlType,
									(value == null) ? "null" : value.GetType().Name,
									value);
								message.AppendLine();
							}
							break;
						/*
						case HtmlTagControlType.Menu_MenuBar:
						case HtmlTagControlType.Menu_Menu:
						case HtmlTagControlType.Menu_MenuButton:
						case HtmlTagControlType.Frame_Frame:
						case HtmlTagControlType.Frame_FrameSet:
							break;
						*/
						case HtmlTagControlType.Slider:
							using (HtmlControl control = new HtmlControl(input))
							{
								object value = control.Value;

								message.AppendFormat(
									"{0}({1}) = ({2}) '{3}'",
									input.Attributes["id"],
									input.ControlType,
									(value == null) ? "null" : value.GetType().Name,
									value);
								message.AppendLine();
							}
							break;
						case HtmlTagControlType.None:
						case HtmlTagControlType.Unknown:
						case HtmlTagControlType.HtmlBox:
						case HtmlTagControlType.Scrollbar:
						case HtmlTagControlType.Graphics:
						case HtmlTagControlType.Sprite:
						case HtmlTagControlType.List:
						case HtmlTagControlType.Tooltip:
						case HtmlTagControlType.Hidden:
						default:
							message.AppendFormat(
								"UNKNOWN: {0} = {1}",
								input.Attributes["id"],
								input.ControlType);
							message.AppendLine();
							break;
					}
				}
			}

			MessageBox.Show(message.ToString());
		}

		private void ButtonWriteValues_Click(object sender, EventArgs e)
		{
			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='input-text']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value = Guid.NewGuid().ToString();
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='input-password']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value = Guid.NewGuid().ToString();
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='input-decimal']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value = (new Random()).NextDouble();
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='input-number']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value = (new Random()).Next();
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='input-progress']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value = 100 * (new Random()).NextDouble();
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='input-hslider']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value = (new Random()).Next(0, 100);
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='textarea']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value = Guid.NewGuid().ToString();
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='richtext']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value =
						string.Format(
							"<b>{0}</b><i>{1}</i>",
							Guid.NewGuid().ToString(),
							Guid.NewGuid().ToString());
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='input-radio1']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value = ((new Random()).Next() % 2 == 0);
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='input-checkbox1']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value = ((new Random()).Next() % 2 == 0);
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='select-1']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value =
						string.Format(
							"DD{0}",
							(new Random()).Next(1, 3));
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='select-3']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value =
						string.Format(
							"LB{0}",
							(new Random()).Next(1, 3));
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='select-3']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value = new object[]
					{
						string.Format("MLB{0}", (new Random()).Next(1, 3)),
						string.Format("MLB{0}", (new Random()).Next(1, 3))
					};
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='input-date']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value =
						new DateTime(
							(new Random()).Next(1900, 2100),
							(new Random()).Next(1, 12),
							(new Random()).Next(1, 28));
				}
			}

			using (HtmlTag input = this._htmlView.RootTag.SelectFirst("[id='input-calendar']"))
			{
				using (HtmlControl control = new HtmlControl(input))
				{
					control.Value =
						new DateTime(
							(new Random()).Next(1900, 2100),
							(new Random()).Next(1, 12),
							(new Random()).Next(1, 28));
				}
			}
		}
	}
}
