﻿using System;
using System.Windows.Forms;

using Nabu.Forms.Html;

namespace HtmlDomExample
{
	public partial class MainForm : Form
	{
		private void BuildDom(HtmlTag parent, TreeNodeCollection nodes)
		{
			foreach (HtmlTag element in parent.Children)
			{
				if (element == null)
				{
					TreeNode node = nodes.Add("[null]");
				}
				else
				{
					TreeNode node = nodes.Add(element.Type);

					node.Tag = element;
					node.ToolTipText = element.InnerText;

					BuildDom(element, node.Nodes);
				}
			}
		}

		public MainForm()
		{
			InitializeComponent();
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			//this._htmlView.Subscribe("#add_row", AddRow_OnMouse);
			//this._htmlView.Subscribe("#remove_row", RemoveRow_OnMouse);
			this._htmlView.RootTag.SelectFirst("#add_row").Behavior += new EventHandler<HtmlBehaviorEventArgs>(AddRow_OnMouse);
			this._htmlView.RootTag.SelectFirst("#remove_row").Behavior += new EventHandler<HtmlBehaviorEventArgs>(RemoveRow_OnMouse);
		}

		public void AddRow_OnMouse(
			object sender,
			HtmlBehaviorEventArgs e)
		{
			// While this application is an example, it can be used as test of event dispatching system.
			// The following lines make you sure that Nabu.Forms.Html keeps managed references to delegates
			// internally not allowing them to be collected.
			GC.Collect();
			GC.WaitForPendingFinalizers();

			switch (e.Command)
			{
				case HtmlBehaviorEventCommand.ButtonClick:
					{
						using (HtmlTag data = this._htmlView.RootTag.SelectFirst("#data"))
						{
							using (HtmlTag tr = HtmlTag.Create("tr"))
							{
								tr.Children.Add(HtmlTag.Create("td", Guid.NewGuid().ToString().Substring(24)));
								tr.Children.Add(HtmlTag.Create("td", DateTime.Now.ToString()));
								tr.Children.Add(HtmlTag.Create("td", (new Random()).Next(1000).ToString()));

								data.Children.Add(tr);
							}

							data.Update(true);
						}
					}
					break;
			}
		}

		public void RemoveRow_OnMouse(
			object sender,
			HtmlBehaviorEventArgs e)
		{
			// While this application is an example, it can be used as test of event dispatching system.
			// The following lines make you sure that Nabu.Forms.Html keeps managed references to delegates
			// internally not allowing them to be collected.
			GC.Collect();
			GC.WaitForPendingFinalizers();

			switch (e.Command)
			{
				case HtmlBehaviorEventCommand.ButtonClick:
					{
						HtmlTag data = this._htmlView.RootTag.SelectFirst("#data");

						if (data.Children.Count > 0)
						{
							data.Children.RemoveAt(data.Children.Count - 1);
						}

						data.Update(true);
					}
					break;
			}
		}

		private void ButtonBuildDomTree_Click(object sender, EventArgs e)
		{
			this._treeDom.Nodes.Clear();

			BuildDom(this._htmlView.RootTag, this._treeDom.Nodes);

			this._treeDom.ExpandAll();
		}
	}
}