﻿using System;
using System.Windows.Forms;

using Nabu.Collections;
using Nabu.Forms.Html;

namespace HtmlListViewExample
{
	public partial class MainForm : Form
	{
		#region Types

		public class ListItem
		{
			public string Name { get; set; }
			public string Value { get; set; }

			public ListItem()
			{
			}

			public ListItem(string name, string value)
			{
				this.Name = name;
				this.Value = value;
			}
		}

		#endregion

		#region Fields

		private BindingListEx<ListItem> _list = new BindingListEx<ListItem>();
		private Random _random = new Random();

		#endregion

		#region Methods

		private void UpdateButtons()
		{
			this._buttonChange.Enabled = this._list.Count > 0;
			this._buttonRemove.Enabled = this._list.Count > 0;
		}

		public MainForm()
		{
			InitializeComponent();
			UpdateButtons();
		}

		private void MainForm_Shown(object sender, EventArgs e)
		{
			using (HtmlTag root = this._htmlView.RootTag)
			{
				using (HtmlTag table = root.SelectFirst("table.list"))
				{
					table.ManagedProc(
						"SetBindingList",
						this._list);
				}
			}
		}

		private void ButtonAdd_Click(object sender, EventArgs e)
		{
			int digits = this._random.Next(14);

			this._list.Add(
				new ListItem(
					Math.Round(this._random.NextDouble(), digits).ToString(),
					Math.Round(this._random.NextDouble(), 14 - digits).ToString()));

			UpdateButtons();
		}

		private void ButtonChange_Click(object sender, EventArgs e)
		{
			this._list[this._random.Next(this._list.Count)] =
				new ListItem(
					this._random.Next().ToString(),
					this._random.NextDouble().ToString());
		}

		private void ButtonRemove_Click(object sender, EventArgs e)
		{
			this._list.RemoveAt(this._random.Next(this._list.Count));

			UpdateButtons();
		}

		#endregion
	}
}