﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Nabu.Logging;
using Nabu.Logging.LogWriters;

namespace LoggingExample
{
	public partial class MainForm : Form
	{
		#region Fields

		ILogWriter _logWriter = null;

		#endregion

		#region Methods

		public MainForm()
		{
			InitializeComponent();

			foreach (Type type in new Type[]
			{
				typeof(MicrosoftAccessDatabaseLogWriter),
				typeof(MicrosoftSqlServerDatabaseLogWriter),
				typeof(TextFileLogWriter),
				typeof(TextMultiFileLogWriter),
				typeof(DebugOutputLogWriter),
				typeof(EventLogLogWriter),
				typeof(XmlFileLogWriter),
				typeof(XmlMultiFileLogWriter)
			})
			{
				this._comboLogWriterType.Items.Add(type);
			}

			this._comboLogWriterType.SelectedIndex = 0;

			foreach (LogSource source in LogSource.DefaultList)
			{
				this._comboLogSource.Items.Add(source);
			}

			this._comboLogSource.SelectedIndex = 0;

			foreach (LogLevel level in LogLevel.DefaultList)
			{
				this._comboLogLevel.Items.Add(level);
			}

			this._comboLogLevel.SelectedIndex = 0;
		}

		private void ComboLogWriterType_SelectedIndexChanged(object sender, EventArgs e)
		{
			this._logWriter = (ILogWriter)Activator.CreateInstance((Type)this._comboLogWriterType.SelectedItem);
			this._propertyLogWriter.SelectedObject = this._logWriter;
		}

		private void ButtonLog_Click(object sender, EventArgs e)
		{
			LogManager.AddWriter(
				this._logWriter,
				new LogLevel[] { LogLevel.Sink },
				new LogSource[] { LogSource.Sink });
			LogManager.Log(
				(LogLevel)this._comboLogLevel.SelectedItem,
				(LogSource)this._comboLogSource.SelectedItem,
				this._textText.Text);
			LogManager.RemoveWriter(
				this._logWriter,
				new LogLevel[] { LogLevel.Sink },
				new LogSource[] { LogSource.Sink });
		}

		private void ButtonClose_Click(object sender, EventArgs e)
		{
			base.Close();
		}

		#endregion
	}
}