﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Nabu;
using Nabu.Text;

namespace NumeralFormatterExample
{
	public partial class MainForm : Form
	{
		#region Fields

		private NumeralFormatter _nf_enUS = NumeralFormatter.Create(1033);
		private NumeralFormatter _nf_enGB = NumeralFormatter.Create(2057);
		private NumeralFormatter _nf_ruRU = NumeralFormatter.Create(1049);

		#endregion

		#region Method

		public MainForm()
		{
			InitializeComponent();

			this._comboType.SelectedIndex = 0;
			this._comboRUCase.SelectedIndex = 0;
			this._comboRUGender.SelectedIndex = 0;
		}

		private void UpdateTexts()
		{
			NumeralType numeralType = NumeralType.Cardinal;
			WordForms numeralForm_ruRU = WordForms.RU_Default;
			WordGender numeralGender_ruRU = WordGender.Masculine;

			switch (this._comboType.SelectedIndex)
			{
				case 0:
					numeralType = NumeralType.Cardinal;
					break;
				case 1:
					numeralType = NumeralType.Ordinal;
					break;
			}

			switch (this._comboRUCase.SelectedIndex)
			{
				case 0:
					numeralForm_ruRU = WordForms.RU_SubjectiveCase;
					break;
				case 1:
					numeralForm_ruRU = WordForms.RU_GenitiveCase;
					break;
				case 2:
					numeralForm_ruRU = WordForms.RU_DativeCase;
					break;
				case 3:
					numeralForm_ruRU = WordForms.RU_AccusativeCase;
					break;
				case 4:
					numeralForm_ruRU = WordForms.RU_InstrumentalCase;
					break;
				case 5:
					numeralForm_ruRU = WordForms.RU_PrepositionalCase;
					break;
			}

			switch (this._comboRUGender.SelectedIndex)
			{
				case 0:
					numeralGender_ruRU = WordGender.Masculine;
					break;
				case 1:
					numeralGender_ruRU = WordGender.Neuter;
					break;
				case 2:
					numeralGender_ruRU = WordGender.Feminine;
					break;
			}

			long numeralValue = (long)this._numericValue.Value;

			this._text_enUS.Text = this._nf_enUS.NumeralNameFromValue(
				numeralValue,
				numeralType,
				WordGender.Masculine,
				WordForms.EN_Default,
				false);

			this._text_enGB.Text = this._nf_enGB.NumeralNameFromValue(
				numeralValue,
				numeralType,
				WordGender.Masculine,
				WordForms.EN_Default,
				false);

			this._text_ruRU.Text = this._nf_ruRU.NumeralNameFromValue(
				numeralValue,
				numeralType,
				numeralGender_ruRU,
				numeralForm_ruRU,
				false);
		}

		private void NumericValue_ValueChanged(object sender, EventArgs e)
		{
			UpdateTexts();
		}

		private void ComboType_SelectedIndexChanged(object sender, EventArgs e)
		{
			UpdateTexts();
		}

		private void ComboRUCase_SelectedIndexChanged(object sender, EventArgs e)
		{
			UpdateTexts();
		}

		private void ComboRUGender_SelectedIndexChanged(object sender, EventArgs e)
		{
			UpdateTexts();
		}

		private void ButtonClose_Click(object sender, EventArgs e)
		{
			base.Close();
		}

		#endregion
	}
}