﻿using System;
using System.Collections.Generic;
using System.Net.Security;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using System.Text;

using Nabu.Net;

namespace Pop3ServerExample
{
	internal sealed class Pop3ServerCallback : IPop3ServerCallback<TcpServerSocketIO>
	{
		#region Pop3ServerCallback

		#region Fields

		private int _messageCount;
		private bool[] _messageAvailability;
		private byte[][] _messageContent;
		private Guid _uid = Guid.NewGuid();

		#endregion

		#region Methods

		public Pop3ServerCallback()
		{
			this._messageCount = 5;
			this._messageAvailability =
			new bool[5]
				{
					true, true, true, true, true
				};

			string[] messages =
				new string[5]
					{
						"1st Message! ",
						"Second Message! ",
						"Third Message! ",
						"Forth Message Content! ",
						"Wow, Fifth Message! "
					};

			this._messageContent =
			new byte[5][]
				{
					Encoding.ASCII.GetBytes(messages[0]),
					Encoding.ASCII.GetBytes(messages[1]),
					Encoding.ASCII.GetBytes(messages[2]),
					Encoding.ASCII.GetBytes(messages[3]),
					Encoding.ASCII.GetBytes(messages[4])
				};
		}

		#endregion

		#endregion

		#region IPop3ServerCallback<TcpServerSocketIO>

		public void OnError(
			Pop3ServerSession<TcpServerSocketIO> session,
			Pop3ServerState state,
			string text)
		{
			Console.WriteLine("OnError: {0}, '{1}'", state, text);
		}

		public void OnCredentials(
			Pop3ServerSession<TcpServerSocketIO> session,
			out bool isValid)
		{
			Console.WriteLine("OnCredentials");

			isValid = session.Username == session.Password;
		}

		public void OnStat(
			Pop3ServerSession<TcpServerSocketIO> session,
			out int totalCount,
			out int totalSize)
		{
			Console.WriteLine("OnStat");

			totalCount = 0;
			totalSize = 0;

			for (int index = 0; index < this._messageContent.Length; index++)
			{
				if (this._messageAvailability[index])
				{
					totalCount++;
					totalSize += _messageContent[index].Length;
				}
			}
		}

		public void OnList(
			Pop3ServerSession<TcpServerSocketIO> session,
			int index,
			out bool isFound,
			out int size)
		{
			Console.WriteLine("OnList");

			if (index < this._messageCount)
			{
				isFound = true;
				size = this._messageContent[index].Length;
			}
			else
			{
				isFound = false;
				size = 0;
			}
		}

		public void OnList(
			Pop3ServerSession<TcpServerSocketIO> session,
			out IDictionary<int, int> messages)
		{
			Console.WriteLine("OnList");

			messages = new Dictionary<int, int>();

			for (int index = 0; index < this._messageContent.Length; index++)
			{
				if (this._messageAvailability[index])
				{
					messages[index] = this._messageContent[index].Length;
				}
			}
		}

		public void OnUidl(
			Pop3ServerSession<TcpServerSocketIO> session,
			int index,
			out bool isFound,
			out string uid)
		{
			Console.WriteLine("OnUidl");

			if (index < this._messageCount)
			{
				isFound = true;
				uid = "ID" + this._uid + "-" + index;
			}
			else
			{
				isFound = false;
				uid = null;
			}
		}

		public void OnUidl(
			Pop3ServerSession<TcpServerSocketIO> session,
			out IDictionary<int, string> messages)
		{
			Console.WriteLine("OnUidl");

			messages = new Dictionary<int, string>();

			for (int index = 0; index < this._messageContent.Length; index++)
			{
				if (this._messageAvailability[index])
				{
					messages[index] = "ID" + this._uid + "-" + index;
				}
			}
		}

		public void OnRetr(
			Pop3ServerSession<TcpServerSocketIO> session,
			int index,
			out bool isFound,
			out ArraySegment<byte> segment)
		{
			Console.WriteLine("OnRetr");

			if (index < this._messageCount)
			{
				isFound = true;
				segment = new ArraySegment<byte>(this._messageContent[index]);
			}
			else
			{
				isFound = false;
				segment = new ArraySegment<byte>(new byte[0]);
			}
		}

		public void OnTop(
			Pop3ServerSession<TcpServerSocketIO> session,
			int index,
			out bool isFound,
			out ArraySegment<byte> segment)
		{
			Console.WriteLine("OnTop");

			if (index < this._messageCount)
			{
				isFound = true;
				segment = new ArraySegment<byte>(new byte[0]);
			}
			else
			{
				isFound = false;
				segment = new ArraySegment<byte>(new byte[0]);
			}
		}

		public void OnDele(
			Pop3ServerSession<TcpServerSocketIO> session,
			int index,
			out bool isFound)
		{
			Console.WriteLine("OnDele");

			if (
				(index <= this._messageCount) &&
				(this._messageAvailability[index]))
			{
				isFound = true;
				this._messageAvailability[index] = false;
			}
			else
			{
				isFound = false;
			}
		}

		public void OnRset(
			Pop3ServerSession<TcpServerSocketIO> session)
		{
			Console.WriteLine("OnRset");

			for (int index = 0; index < this._messageAvailability.Length; index++)
			{
				this._messageAvailability[index] = true;
			}
		}

		public void OnQuit(
			Pop3ServerSession<TcpServerSocketIO> session)
		{
			Console.WriteLine("OnQuit");
		}

		#endregion

		#region ITcpNodeCallback<Pop3ServerSession<TcpServerSocketIO>> Members

		public void OnOpenSession(
			Pop3ServerSession<TcpServerSocketIO> session)
		{
			Console.WriteLine("CONNECTED.");
		}

		public void OnCloseSession(
			Pop3ServerSession<TcpServerSocketIO> session)
		{
			Console.WriteLine("DISCONNECTED.");
		}

		public void OnSslAuthenticate(Pop3ServerSession<TcpServerSocketIO> session, out X509CertificateCollection certificates, out SslProtocols enabledSslProtocols, out bool mutualAuthenticationRequired, out bool checkCertificateRevocation)
		{
			certificates = null;
			mutualAuthenticationRequired = false;
			enabledSslProtocols = SslProtocols.Default;
			checkCertificateRevocation = false;
		}

		public bool OnSslValidateCertificate(
			Pop3ServerSession<TcpServerSocketIO> session,
			X509Certificate certificate,
			X509Chain chain,
			SslPolicyErrors sslPolicyErrors)
		{
			return true;
		}

		public X509Certificate OnSslSelectCertificate(
			Pop3ServerSession<TcpServerSocketIO> session,
			string targetHost,
			X509CertificateCollection localCertificates,
			X509Certificate remoteCertificate,
			string[] acceptableIssuers)
		{
			return null;
		}

		#endregion
	}

	public sealed class Program
	{
		public static void Main(string[] args)
		{
			Console.WriteLine("POP3 Server Example");
			Console.WriteLine();

			using (Pop3Server server = new Pop3Server(new Pop3ServerCallback()))
			{
				server.Start("*", 110);
				Console.WriteLine("Press <Enter> to stop accepting connections");
				Console.ReadLine();
				// You can terminate existing sessions by setting session.IsClosing
				// property to true in IPop3ServerCallback methods.
				// Note, this is not recommended since not expected by clients.
				Console.WriteLine("Shutting down, waiting for existing sessions to complete");
				server.Stop();
			}
		}
	}
}
