﻿using System;
using System.Collections.Generic;
using System.Net.Security;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using System.Text;

using Nabu.Net;

namespace SmtpClientExample
{
	internal sealed class SmtpClientCallback : ISmtpClientCallback<TcpClientSocket>
	{
		#region SmtpClientCallback

		private string _hostname;
		private SmtpClientRequestType _request = SmtpClientRequestType.Unknown;

		public SmtpClientCallback(string hostname)
		{
			this._hostname = hostname;
		}

		#endregion

		#region ISmtpClientCallback

		public void OnError(
			SmtpClientSession<TcpClientSocket> session,
			SmtpClientState state,
			string text,
			int code)
		{
			Console.WriteLine("OnError: {0}, {1}, '{2}'", state, code, text);
		}

		public void OnCredentials(
			SmtpClientSession<TcpClientSocket> session,
			out string hostname,
			out string username,
			out string password)
		{
			Console.WriteLine("GetCredentials");
			hostname = this._hostname;
			Console.Write("\tUsername: ");
			username = Console.ReadLine();
			Console.Write("\tPassword: ");
			password = Console.ReadLine();
		}

		public void OnRequest(
			SmtpClientSession<TcpClientSocket> session,
			out SmtpClientRequest request)
		{
			Console.WriteLine("GetRequest");

			string sourceAddress = null;
			IEnumerable<string> destinationAddresses = null;

			switch (this._request)
			{
				case SmtpClientRequestType.Unknown:
					this._request = SmtpClientRequestType.Send;

					Console.WriteLine("\t{0}", this._request);
					Console.Write("\tSource Address: ");
					sourceAddress = Console.ReadLine();
					Console.Write("\tDestination Addresses (separate with spaces): ");
					destinationAddresses = Console.ReadLine().Split(' ');
					break;
				case SmtpClientRequestType.Send:
					this._request = SmtpClientRequestType.Quit;
					break;
			}

			request = new SmtpClientRequest(this._request, sourceAddress, destinationAddresses);
		}

		public void OnMessage(
			SmtpClientSession<TcpClientSocket> session,
			out ArraySegment<byte> segment,
			out bool isLast)
		{
			Console.WriteLine("GetMessage");

			segment =
				new ArraySegment<byte>(
					Encoding.ASCII.GetBytes("\r\nHi!\r\n.\r\n"));
			isLast = true;
		}

		public void OnVerify(
			SmtpClientSession<TcpClientSocket> session,
			bool isValid,
			bool isLocal)
		{
			Console.WriteLine("OnVerify");
			Console.WriteLine("\tIsValid={0}, IsLocal={1}", isValid, isLocal);
		}

		public void OnExpand(
			SmtpClientSession<TcpClientSocket> session,
			IEnumerable<string> addresses)
		{
			Console.WriteLine("OnExpand");
		}

		#endregion

		#region ITcpNodeCallback

		public void OnOpenSession(SmtpClientSession<TcpClientSocket> session)
		{
			Console.WriteLine("CONNECTED.");
		}

		public void OnCloseSession(SmtpClientSession<TcpClientSocket> session)
		{
			Console.WriteLine("DISCONNECTED.");
		}

		public void OnSslAuthenticate(SmtpClientSession<TcpClientSocket> session, out X509CertificateCollection certificates, out SslProtocols enabledSslProtocols, out bool mutualAuthenticationRequired, out bool checkCertificateRevocation)
		{
			certificates = null;
			mutualAuthenticationRequired = false;
			enabledSslProtocols = SslProtocols.Default;
			checkCertificateRevocation = false;
		}

		public bool OnSslValidateCertificate(
			SmtpClientSession<TcpClientSocket> session,
			X509Certificate certificate,
			X509Chain chain,
			SslPolicyErrors sslPolicyErrors)
		{
			return true;
		}

		public X509Certificate OnSslSelectCertificate(
			SmtpClientSession<TcpClientSocket> session,
			string targetHost,
			X509CertificateCollection localCertificates,
			X509Certificate remoteCertificate,
			string[] acceptableIssuers)
		{
			return null;
		}

		#endregion
	}

	public sealed class Program
	{
		public static void Main(string[] args)
		{
			Console.WriteLine("SMTP Client Example");
			Console.WriteLine();
			Console.WriteLine("Connection Parameters:");
			Console.Write("\tHost: ");

			string host = Console.ReadLine();

			Console.Write("\tPort: ");

			string port = Console.ReadLine();

			using (SmtpClient smtpClient =
				new SmtpClient(
					new SmtpClientCallback(
						host)))
			{
				smtpClient.OpenSession(
					host,
					ushort.Parse(port),
					TcpConnectionWrapper.None,
					null);
				smtpClient.WaitForSessions();
			}
		}
	}
}
