﻿using System;
using System.Collections.Generic;
using System.Net.Security;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using System.Text;

using Nabu.Net;

namespace SmtpServerExample
{
	internal sealed class SmtpServerCallback : ISmtpServerCallback<TcpServerSocketIO>
	{
		#region ISmtpServerCallback<TcpServerSocketIO>

		public void OnError(
			SmtpServerSession<TcpServerSocketIO> session,
			SmtpServerState state,
			string text)
		{
			Console.WriteLine("OnError: '{0}, {1}'", state, text);
		}

		public void OnCredentials(
			SmtpServerSession<TcpServerSocketIO> session,
			out bool isValid)
		{
			Console.WriteLine("OnCredentials");

			isValid = session.Username == session.Password;
		}

		public void OnSend(
			SmtpServerSession<TcpServerSocketIO> session,
			IEnumerable<string> senders,
			IEnumerable<string> recipients,
			ArraySegment<byte> data)
		{
			Console.WriteLine("OnSend");
			Console.Write("\tSenders:");

			foreach (string sender in senders)
			{
				Console.Write(" {0}", sender);
			}

			Console.WriteLine();
			Console.Write("\tRecipients:");

			foreach (string recipient in recipients)
			{
				Console.Write(" {0}", recipient);
			}

			Console.WriteLine();

			Console.WriteLine("\tData:");
			Console.WriteLine(Encoding.ASCII.GetString(data.Array, data.Offset, data.Count));
			Console.WriteLine();
		}

		public void OnRset(
			SmtpServerSession<TcpServerSocketIO> session)
		{
			Console.WriteLine("OnRset");
		}

		public void OnQuit(
			SmtpServerSession<TcpServerSocketIO> session)
		{
			Console.WriteLine("OnQuit");
		}

		#endregion

		#region ITcpNodeCallback<SmtpServerSession<TcpServerSocketIO>> Members

		public void OnOpenSession(
			SmtpServerSession<TcpServerSocketIO> session)
		{
			Console.WriteLine("CONNECTED.");
		}

		public void OnCloseSession(
			SmtpServerSession<TcpServerSocketIO> session)
		{
			Console.WriteLine("DISCONNECTED.");
		}

		public void OnSslAuthenticate(SmtpServerSession<TcpServerSocketIO> session, out X509CertificateCollection certificates, out SslProtocols enabledSslProtocols, out bool mutualAuthenticationRequired, out bool checkCertificateRevocation)
		{
			certificates = null;
			mutualAuthenticationRequired = false;
			enabledSslProtocols = SslProtocols.Default;
			checkCertificateRevocation = false;
		}

		public bool OnSslValidateCertificate(
			SmtpServerSession<TcpServerSocketIO> session,
			X509Certificate certificate,
			X509Chain chain,
			SslPolicyErrors sslPolicyErrors)
		{
			return true;
		}

		public X509Certificate OnSslSelectCertificate(
			SmtpServerSession<TcpServerSocketIO> session,
			string targetHost,
			X509CertificateCollection localCertificates,
			X509Certificate remoteCertificate,
			string[] acceptableIssuers)
		{
			return null;
		}

		#endregion
	}

	public sealed class Program
	{
		public static void Main(string[] args)
		{
			Console.WriteLine("Smtp Server Example");
			Console.WriteLine();

			using (SmtpServer server =
				new SmtpServer(
					new SmtpServerCallback(),
					"mail.example.com"))
			{
				server.Start("*", 25);
				Console.WriteLine("Press <Enter> to stop accepting connections");
				Console.ReadLine();
				// You can terminate existing sessions by setting session.IsClosing
				// property to true in ISmtpServerCallback methods.
				// Note, this is not recommended since not expected by clients.
				Console.WriteLine("Shutting down, waiting for existing sessions to complete");
				server.Stop();
			}
		}
	}
}
