﻿using System;
using System.Net;

using Nabu.Net;

namespace SysLogServerExample
{
	internal sealed class SysLogServerCallback : ISysLogServerCallback<UdpServerSocket>
	{
		#region SysLogServerCallback

		private int _counter = 0;

		#endregion

		#region ISysLogServerCallback

		public bool OnMessage(
			IPEndPoint endPoint,
			SysLogFacility facility,
			SysLogSeverity severity,
			DateTime timestamp,
			string hostname,
			string message)
		{
			this._counter++;

			Console.WriteLine(
				"[{0}]; {1}/{2}; {3}; {4} ({5})",
				this._counter,
				facility,
				severity,
				timestamp,
				hostname,
				endPoint);
			Console.WriteLine("\t{0}", message);

			return true;
		}

		#endregion

		#region IUdpNodeCallback

		public void OnOpenSession(SysLogServerSession<UdpServerSocket> session)
		{
			Console.WriteLine("CONNECTED.");
		}

		public void OnCloseSession(SysLogServerSession<UdpServerSocket> session)
		{
			Console.WriteLine("DISCONNECTED.");
		}

		#endregion
	}

	public sealed class Program
	{
		public static void Main(string[] args)
		{
			SysLogServer sysLogServer = new SysLogServer(new SysLogServerCallback());

			sysLogServer.Start("*", 514, null);

			Console.WriteLine("Press any key to exit...");
			Console.ReadKey();
		}
	}
}
