﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Collections Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;

namespace Nabu.Collections
{
	public sealed class BindingListExAddingRemovingEventArgs<T> : EventArgs
	{
		#region Fields

		private readonly int _index;
		private readonly T _value;
		private bool _cancel;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public BindingListExAddingRemovingEventArgs(int index, T value)
		{
			this._index = index;
			this._value = value;
		}

		#endregion

		#region Properties

		public int Index
		{
			[DebuggerStepThrough]
			get { return this._index; }
		}

		public T Value
		{
			[DebuggerStepThrough]
			get { return this._value; }
		}

		public bool Cancel
		{
			[DebuggerStepThrough]
			get { return this._cancel; }
			[DebuggerStepThrough]
			set { this._cancel = value; }
		}

		#endregion
	};

	public sealed class BindingListExAddedRemovedEventArgs<T> : EventArgs
	{
		#region Fields

		private readonly int _index;
		private readonly T _value;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public BindingListExAddedRemovedEventArgs(int index, T value)
		{
			this._index = index;
			this._value = value;
		}

		#endregion

		#region Properties

		public int Index
		{
			[DebuggerStepThrough]
			get { return this._index; }
		}

		public T Value
		{
			[DebuggerStepThrough]
			get { return this._value; }
		}

		#endregion
	};

	public sealed class BindingListExChangingEventArgs<T> : EventArgs
	{
		#region Fields

		private readonly int _index;
		private readonly T _oldValue;
		private readonly T _newValue;
		private bool _cancel;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public BindingListExChangingEventArgs(int index, T oldValue, T newValue)
		{
			this._index = index;
			this._oldValue = oldValue;
			this._newValue = newValue;
		}

		#endregion

		#region Properties

		public int Index
		{
			[DebuggerStepThrough]
			get { return this._index; }
		}

		public T OldValue
		{
			[DebuggerStepThrough]
			get { return this._oldValue; }
		}

		public T NewValue
		{
			[DebuggerStepThrough]
			get { return this._newValue; }
		}

		public bool Cancel
		{
			[DebuggerStepThrough]
			get { return this._cancel; }
			[DebuggerStepThrough]
			set { this._cancel = value; }
		}

		#endregion
	};

	public sealed class BindingListExChangedEventArgs<T> : EventArgs
	{
		#region Fields

		private readonly int _index;
		private readonly T _oldValue;
		private readonly T _newValue;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public BindingListExChangedEventArgs(int index, T oldValue, T newValue)
		{
			this._index = index;
			this._oldValue = oldValue;
			this._newValue = newValue;
		}

		#endregion

		#region Properties

		public int Index
		{
			[DebuggerStepThrough]
			get { return this._index; }
		}

		public T OldValue
		{
			[DebuggerStepThrough]
			get { return this._oldValue; }
		}

		public T NewValue
		{
			[DebuggerStepThrough]
			get { return this._newValue; }
		}

		#endregion
	};

	public sealed class BindingListExMovingEventArgs<T> : EventArgs
	{
		#region Fields

		private readonly int _index1;
		private readonly int _index2;
		private bool _cancel;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public BindingListExMovingEventArgs(int index1, int index2)
		{
			this._index1 = index1;
			this._index2 = index2;
		}

		#endregion

		#region Properties

		public int Index1
		{
			[DebuggerStepThrough]
			get { return this._index1; }
		}

		public int Index2
		{
			[DebuggerStepThrough]
			get { return this._index2; }
		}

		public bool Cancel
		{
			[DebuggerStepThrough]
			get { return this._cancel; }
			[DebuggerStepThrough]
			set { this._cancel = value; }
		}

		#endregion
	};

	public sealed class BindingListExMovedEventArgs<T> : EventArgs
	{
		#region Fields

		private readonly int _index1;
		private readonly int _index2;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public BindingListExMovedEventArgs(int index1, int index2)
		{
			this._index1 = index1;
			this._index2 = index2;
		}

		#endregion

		#region Properties

		public int Index1
		{
			[DebuggerStepThrough]
			get { return this._index1; }
		}

		public int Index2
		{
			[DebuggerStepThrough]
			get { return this._index2; }
		}

		#endregion
	};

#if COMPACT_FRAMEWORK

	public sealed class PropertyChangingEventArgs : EventArgs
	{
	#region Fields

		private readonly string _propertyName;

		#endregion

	#region Methods

		public PropertyChangingEventArgs(string propertyName)
		{
			this._propertyName = propertyName;
		}

		#endregion

	#region Properties

		public string PropertyName
		{
			get { return this._propertyName; }
		}

		#endregion
	}

	public delegate void PropertyChangingEventHandler(object sender, PropertyChangingEventArgs e);

	public interface INotifyPropertyChanging
	{
		event PropertyChangingEventHandler PropertyChanging;
	}

#endif
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
