﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Collections Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;
using System.Xml;
using System.Xml.Serialization;

namespace Nabu.Collections
{
	[SuppressMessage("Microsoft.Design", "CA1005:AvoidExcessiveParametersOnGenericTypes")]
	public abstract class DictionaryBaseEx<TDictionary, TKey, TValue> :
		IDictionary<TKey, TValue>,
		IDictionary,
		IXmlSerializable
		where TDictionary : new()
	{
		#region DictionaryBaseEx<TDictionary, TKey, TValue>

		#region Constants

		private string TagExpectedExceptionMessage1 = "<{0}> tag expected.";
		private string TagExpectedExceptionMessage2 = "<{0}> or <{1}> tag expected.";

		#endregion

		#region Fields

		private TDictionary _container;
		private IDictionary<TKey, TValue> _genericDictionary;
		private IDictionary _dictionary;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		protected DictionaryBaseEx()
		{
			this._container = new TDictionary();
			this._genericDictionary = (IDictionary<TKey, TValue>)this._container;
			this._dictionary = (IDictionary)this._container;
		}

		[DebuggerStepThrough]
		protected DictionaryBaseEx(IDictionary<TKey, TValue> dictionary)
			: this()
		{
			foreach (KeyValuePair<TKey, TValue> item in dictionary)
			{
				this._genericDictionary.Add(item);
			}
		}

		#endregion

		#endregion

		#region IDictionary<TKey,TValue>

		#region Methods

		[DebuggerStepThrough]
		public void Add(TKey key, TValue value)
		{
			this._genericDictionary.Add(key, value);
		}

		[DebuggerStepThrough]
		public bool Remove(TKey key)
		{
			return this._genericDictionary.Remove(key);
		}

		[DebuggerStepThrough]
		public bool ContainsKey(TKey key)
		{
			return this._genericDictionary.ContainsKey(key);
		}

		[DebuggerStepThrough]
		public bool TryGetValue(TKey key, out TValue value)
		{
			return this._genericDictionary.TryGetValue(key, out value);
		}

		#endregion

		#region Properties

		public ICollection<TKey> Keys
		{
			[DebuggerStepThrough]
			get { return this._genericDictionary.Keys; }
		}

		public ICollection<TValue> Values
		{
			[DebuggerStepThrough]
			get { return this._genericDictionary.Values; }
		}

		public TValue this[TKey key]
		{
			[DebuggerStepThrough]
			get { return this._genericDictionary[key]; }
			[DebuggerStepThrough]
			set { this._genericDictionary[key] = value; }
		}

		#endregion

		#endregion

		#region ICollection<KeyValuePair<TKey,TValue>>

		#region Methods

		[DebuggerStepThrough]
		public void Add(KeyValuePair<TKey, TValue> item)
		{
			this._genericDictionary.Add(item);
		}

		[DebuggerStepThrough]
		public bool Remove(KeyValuePair<TKey, TValue> item)
		{
			return this._genericDictionary.Remove(item);
		}

		[DebuggerStepThrough]
		public void Clear()
		{
			this._genericDictionary.Clear();
		}

		[DebuggerStepThrough]
		public bool Contains(KeyValuePair<TKey, TValue> item)
		{
			return this._genericDictionary.Contains(item);
		}

		[DebuggerStepThrough]
		public void CopyTo(KeyValuePair<TKey, TValue>[] array, int arrayIndex)
		{
			this._genericDictionary.CopyTo(array, arrayIndex);
		}

		#endregion

		#region Properties

		public int Count
		{
			[DebuggerStepThrough]
			get { return this._genericDictionary.Count; }
		}

		public bool IsReadOnly
		{
			[DebuggerStepThrough]
			get { return this._genericDictionary.IsReadOnly; }
		}

		#endregion

		#endregion

		#region IEnumerable<KeyValuePair<TKey,TValue>>

		[DebuggerStepThrough]
		public IEnumerator<KeyValuePair<TKey, TValue>> GetEnumerator()
		{
			return this._genericDictionary.GetEnumerator();
		}

		#endregion

		#region IDictionary

		#region Methods

		[DebuggerStepThrough]
		public void Add(object key, object value)
		{
			this._dictionary.Add(key, value);
		}

		[DebuggerStepThrough]
		public void Remove(object key)
		{
			this._dictionary.Remove(key);
		}

		[DebuggerStepThrough]
		public bool Contains(object key)
		{
			return this._dictionary.Contains(key);
		}

		[DebuggerStepThrough]
		IDictionaryEnumerator IDictionary.GetEnumerator()
		{
			return this._dictionary.GetEnumerator();
		}

		#endregion

		#region Properties

		public bool IsFixedSize
		{
			[DebuggerStepThrough]
			get { return this._dictionary.IsFixedSize; }
		}

		ICollection IDictionary.Keys
		{
			[DebuggerStepThrough]
			get { return this._dictionary.Keys; }
		}

		ICollection IDictionary.Values
		{
			[DebuggerStepThrough]
			get { return this._dictionary.Values; }
		}

		public object this[object key]
		{
			[DebuggerStepThrough]
			get { return this._dictionary[key]; }
			[DebuggerStepThrough]
			set { this._dictionary[key] = value; }
		}

		#endregion

		#endregion

		#region ICollection

		#region Methods

		[DebuggerStepThrough]
		public void CopyTo(Array array, int index)
		{
			this._dictionary.CopyTo(array, index);
		}

		#endregion

		#region Properties

		public bool IsSynchronized
		{
			[DebuggerStepThrough]
			get { return this._dictionary.IsSynchronized; }
		}

		public object SyncRoot
		{
			[DebuggerStepThrough]
			get { return this._dictionary.SyncRoot; }
		}

		#endregion

		#endregion

		#region IEnumerable

		[DebuggerStepThrough]
		IEnumerator IEnumerable.GetEnumerator()
		{
			return this._dictionary.GetEnumerator();
		}

		#endregion

		#region IXmlSerializable

		[DebuggerStepThrough]
		public System.Xml.Schema.XmlSchema GetSchema()
		{
			return null;
		}

		[DebuggerStepThrough]
		public void ReadXml(XmlReader reader)
		{
			if (reader.IsEmptyElement)
			{
				reader.Read();
			}
			else
			{
				string itemName = "Item";
				string keyName = "Key";
				string valueName = "Value";

				foreach (Attribute attribute in TypeDescriptor.GetAttributes(this))
				{
					XmlDictionaryItemAttribute itemAttribute = attribute as XmlDictionaryItemAttribute;
					XmlDictionaryKeyAttribute keyAttribute = attribute as XmlDictionaryKeyAttribute;
					XmlDictionaryValueAttribute valueAttribute = attribute as XmlDictionaryValueAttribute;

					if (itemAttribute != null)
					{
						itemName = itemAttribute.Name;
					}

					if (keyAttribute != null)
					{
						keyName = keyAttribute.Name;
					}

					if (valueAttribute != null)
					{
						valueName = valueAttribute.Name;
					}
				}

				XmlSerializer keySerializer = new XmlSerializer(typeof(TKey));
				XmlSerializer valueSerializer = new XmlSerializer(typeof(TValue));

				string name = reader.Name;

				reader.ReadStartElement();
				reader.MoveToContent();

				for (; name != reader.Name; )
				{
					if (reader.Name != itemName)
					{
						throw new XmlException(
							string.Format(
								CultureInfo.InvariantCulture,
								TagExpectedExceptionMessage1,
								itemName));
					}

					reader.ReadStartElement();
					reader.MoveToContent();

					if (reader.Name == keyName)
					{
						reader.ReadStartElement();
						reader.MoveToContent();

						TKey key = (TKey)keySerializer.Deserialize(reader);

						reader.ReadEndElement();

						if (reader.Name != valueName)
						{
							throw new XmlException(
								string.Format(
									CultureInfo.InvariantCulture,
									TagExpectedExceptionMessage1,
									valueName));
						}

						reader.ReadStartElement();
						reader.MoveToContent();

						TValue value = (TValue)valueSerializer.Deserialize(reader);

						reader.ReadEndElement();

						Add(key, value);
					}
					else if (reader.Name == valueName)
					{
						reader.ReadStartElement();
						reader.MoveToContent();

						TValue value = (TValue)valueSerializer.Deserialize(reader);

						reader.ReadEndElement();

						if (reader.Name != keyName)
						{
							throw new XmlException(
								string.Format(
									CultureInfo.InvariantCulture,
									TagExpectedExceptionMessage1,
									keyName));
						}

						reader.ReadStartElement();
						reader.MoveToContent();

						TKey key = (TKey)keySerializer.Deserialize(reader);

						reader.ReadEndElement();

						Add(key, value);
					}
					else
					{
						throw new XmlException(
							string.Format(
								CultureInfo.InvariantCulture,
								TagExpectedExceptionMessage2,
								keyName,
								valueName));
					}

					reader.ReadEndElement();
				}

				reader.ReadEndElement();
			}
		}

		[DebuggerStepThrough]
		public void WriteXml(XmlWriter writer)
		{
			string itemName = "Item";
			string keyName = "Key";
			string valueName = "Value";

			foreach (Attribute attribute in TypeDescriptor.GetAttributes(this))
			{
				XmlDictionaryItemAttribute itemAttribute = attribute as XmlDictionaryItemAttribute;
				XmlDictionaryKeyAttribute keyAttribute = attribute as XmlDictionaryKeyAttribute;
				XmlDictionaryValueAttribute valueAttribute = attribute as XmlDictionaryValueAttribute;

				if (itemAttribute != null)
				{
					itemName = itemAttribute.Name;
				}

				if (keyAttribute != null)
				{
					keyName = keyAttribute.Name;
				}

				if (valueAttribute != null)
				{
					valueName = valueAttribute.Name;
				}
			}

			XmlSerializer keySerializer = new XmlSerializer(typeof(TKey));
			XmlSerializer valueSerializer = new XmlSerializer(typeof(TValue));

			foreach (KeyValuePair<TKey, TValue> item in this._genericDictionary)
			{
				writer.WriteStartElement(itemName);
				writer.WriteStartElement(keyName);
				keySerializer.Serialize(writer, item.Key);
				writer.WriteEndElement();
				writer.WriteStartElement(valueName);
				valueSerializer.Serialize(writer, item.Value);
				writer.WriteEndElement();
				writer.WriteEndElement();
			}
		}

		#endregion
	}

	[AttributeUsage(AttributeTargets.Class | AttributeTargets.Struct, AllowMultiple = false, Inherited = true)]
	public sealed class XmlDictionaryItemAttribute : Attribute
	{
		#region Fields

		private readonly string _name;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public XmlDictionaryItemAttribute(string name)
		{
			this._name = name;
		}

		#endregion

		#region Properties

		public string Name
		{
			[DebuggerStepThrough]
			get { return this._name; }
		}

		#endregion
	}

	[AttributeUsage(AttributeTargets.Class | AttributeTargets.Struct, AllowMultiple = false, Inherited = true)]
	public sealed class XmlDictionaryKeyAttribute : Attribute
	{
		#region Fields

		private readonly string _name;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public XmlDictionaryKeyAttribute(string name)
		{
			this._name = name;
		}

		#endregion

		#region Properties

		public string Name
		{
			[DebuggerStepThrough]
			get { return this._name; }
		}

		#endregion
	}

	[AttributeUsage(AttributeTargets.Class | AttributeTargets.Struct, AllowMultiple = false, Inherited = true)]
	public sealed class XmlDictionaryValueAttribute : Attribute
	{
		#region Fields

		private readonly string _name;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public XmlDictionaryValueAttribute(string name)
		{
			this._name = name;
		}

		#endregion

		#region Properties

		public string Name
		{
			[DebuggerStepThrough]
			get { return this._name; }
		}

		#endregion
	}

	public sealed class DictionaryEx<TKey, TValue> :
		DictionaryBaseEx<Dictionary<TKey, TValue>, TKey, TValue>
	{
		public DictionaryEx()
		{
		}

		public DictionaryEx(IDictionary<TKey, TValue> dictionary)
			: base(dictionary)
		{
		}
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
