﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Collections Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Diagnostics;

namespace Nabu.Collections
{
	[Serializable]
	public sealed class DisposableList<T> : ListEx<T>, IDisposable
	{
		#region DisposableList

		#region Constructors

		[DebuggerStepThrough]
		public DisposableList()
		{
		}

		[DebuggerStepThrough]
		public DisposableList(int capacity)
			: base(capacity)
		{
		}

		[DebuggerStepThrough]
		public DisposableList(IEnumerable<T> collection)
			: base(collection)
		{
		}

		[DebuggerStepThrough]
		public DisposableList(IEnumerable<T> collection, int capacity)
			: base(collection, capacity)
		{
		}

		[DebuggerStepThrough]
		public DisposableList(T[] array)
			: base(array)
		{
		}

		[DebuggerStepThrough]
		public DisposableList(T[] array, bool attach)
			: base(array, attach)
		{
		}

		#endregion

		#region Finalizers

		~DisposableList()
		{
			DisposeElements();
		}

		private void DisposeElements()
		{
			int count = base.Count;

			for (int index = 0; index < count; index++)
			{
				IDisposable disposable = this[index] as IDisposable;

				if (disposable != null)
				{
					disposable.Dispose();

					if (count != base.Count)
					{
						count = base.Count;
						index = 0;
					}
				}

				this[index] = default(T);
			}
		}

		#endregion

		#endregion

		#region IDisposable

		public void Dispose()
		{
			DisposeElements();
			GC.SuppressFinalize(this);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
