﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Collections Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;

namespace Nabu.Collections
{
	public interface IListEx<T> : IList<T>
	{
		#region Methods

		#region Capacity

		void EnsureCapacity(int capacity);
		void TrimExcess(float ratio);
		void TrimExcess();

		#endregion

		#region Range

		IListEx<T> GetRange(int startIndex, int count);
		void AddRange(IEnumerable<T> collection);
		void AddRange(IListEx<T> list, int offset, int count);
		void AddRange(IListEx<T> list);
		void AddRange(T[] array, int offset, int count);
		void AddRange(T[] array);
		void InsertRange(int startIndex, IEnumerable<T> collection);
		void InsertRange(int startIndex, IListEx<T> list, int offset, int count);
		void InsertRange(int startIndex, IListEx<T> list);
		void InsertRange(int startIndex, T[] array, int offset, int count);
		void InsertRange(int startIndex, T[] array);
		void RemoveRange(int startIndex, int count);

		#endregion

		#region Copy and Convert

		void FromArray(T[] array, bool attach);
		void FromArray(T[] array);
		T[] ToArray();
		T[] DangerousGetArray();

		ArraySegment<T> ToArraySegment();

		void CopyTo(T[] array);
		void CopyTo(int startIndex, T[] array, int arrayIndex, int count);
		IListEx<TOutput> ConvertAll<TOutput>(Converter<T, TOutput> converter);

		#endregion

		#region Reverse

		void Reverse(int startIndex, int count);
		void Reverse();

		#endregion

		#region Sort

		void Sort(int startIndex, int count);
		void Sort();
		void Sort(IComparer<T> comparer, int startIndex, int count);
		void Sort(IComparer<T> comparer);
		void Sort(Comparison<T> comparer, int startIndex, int count);
		void Sort(Comparison<T> comparer);

		#endregion

		#region Search

		int IndexOf(T item, int startIndex, int count);
		int IndexOf(T item, int startIndex);
		//int IndexOf(T item);
		int IndexOf(T item, IComparer<T> comparer, int startIndex, int count);
		int IndexOf(T item, IComparer<T> comparer, int startIndex);
		int IndexOf(T item, IComparer<T> comparer);
		int IndexOf(T item, Comparison<T> comparer, int startIndex, int count);
		int IndexOf(T item, Comparison<T> comparer, int startIndex);
		int IndexOf(T item, Comparison<T> comparer);

		int LastIndexOf(T item, int startIndex, int count);
		int LastIndexOf(T item, int startIndex);
		int LastIndexOf(T item);
		int LastIndexOf(T item, IComparer<T> comparer, int startIndex, int count);
		int LastIndexOf(T item, IComparer<T> comparer, int startIndex);
		int LastIndexOf(T item, IComparer<T> comparer);
		int LastIndexOf(T item, Comparison<T> comparer, int startIndex, int count);
		int LastIndexOf(T item, Comparison<T> comparer, int startIndex);
		int LastIndexOf(T item, Comparison<T> comparer);

		int BinarySearch(T item, int startIndex, int count);
		int BinarySearch(T item, int startIndex);
		int BinarySearch(T item);
		int BinarySearch(T item, IComparer<T> comparer, int startIndex, int count);
		int BinarySearch(T item, IComparer<T> comparer, int startIndex);
		int BinarySearch(T item, IComparer<T> comparer);
		int BinarySearch(T item, Comparison<T> comparer, int startIndex, int count);
		int BinarySearch(T item, Comparison<T> comparer, int startIndex);
		int BinarySearch(T item, Comparison<T> comparer);

		#endregion

		#region Predicates

		#region Exists

		bool Exists(Predicate<T> match);

		#endregion

		#region FindAll

		IListEx<T> FindAll(Predicate<T> match, int startIndex, int count);
		IListEx<T> FindAll(Predicate<T> match, int startIndex);
		IListEx<T> FindAll(Predicate<T> match);

		IListEx<int> FindAllIndex(Predicate<T> match, int startIndex, int count);
		IListEx<int> FindAllIndex(Predicate<T> match, int startIndex);
		IListEx<int> FindAllIndex(Predicate<T> match);

		#endregion

		#region FindFirst

		int FindFirstIndex(Predicate<T> match, int startIndex, int count);
		int FindFirstIndex(Predicate<T> match, int startIndex);
		int FindFirstIndex(Predicate<T> match);

		#endregion

		#region FindLast

		int FindLastIndex(Predicate<T> match, int startIndex, int count);
		int FindLastIndex(Predicate<T> match, int startIndex);
		int FindLastIndex(Predicate<T> match);

		#endregion

		#region RemoveAll

		int RemoveAll(Predicate<T> match, int startIndex, int count);
		int RemoveAll(Predicate<T> match, int startIndex);
		int RemoveAll(Predicate<T> match);

		#endregion

		#region TrueForAll

		bool TrueForAll(Predicate<T> match, int startIndex, int count);
		bool TrueForAll(Predicate<T> match, int startIndex);
		bool TrueForAll(Predicate<T> match);

		#endregion

		#region FalseForAll

		bool FalseForAll(Predicate<T> match, int startIndex, int count);
		bool FalseForAll(Predicate<T> match, int startIndex);
		bool FalseForAll(Predicate<T> match);

		#endregion

		#endregion

		#region Actions

		void ForEach(Action<T> action, int startIndex, int count);
		void ForEach(Action<T> action, int startIndex);
		void ForEach(Action<T> action);

		void ForEach(Action<T> action, Predicate<T> match, int startIndex, int count);
		void ForEach(Action<T> action, Predicate<T> match, int startIndex);
		void ForEach(Action<T> action, Predicate<T> match);

		#endregion

		#endregion

		#region Properties

		int Capacity { get; set; }
		T First { get; }
		T Last { get; }
		bool IsAddAllowed { get; set; }
		bool IsRemoveAllowed { get; set; }
		bool IsChangeAllowed { get; set; }

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
