﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Data Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;
using System.Reflection;

namespace Nabu.Data
{
	public sealed class Singleton<T> where T : class
	{
		#region Fields

		private static volatile T _instance = default(T);
		private static object _instanceSync = new object();

		#endregion

		#region Methods

		private Singleton()
		{
		}

		#endregion

		#region Properties

		[SuppressMessage("Microsoft.Design", "CA1000:DoNotDeclareStaticMembersOnGenericTypes")]
		public static T Instance
		{
			get
			{
				if (_instance == null)
				{
					lock (_instanceSync)
					{
						if (_instance == null)
						{
							Type type = typeof(T);

							_instance =
								type.InvokeMember(
									type.Name,
									BindingFlags.CreateInstance | BindingFlags.Instance | BindingFlags.NonPublic,
									null,
									null,
									null,
									null,
									CultureInfo.InvariantCulture,
									null) as T;
						}
					}
				}

				return _instance;
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
