﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Data Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Collections.Generic;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;

namespace Nabu.Data
{
	public abstract class DiffBase<T> : IDiff<T>
	{
		#region DiffBase

		#region Fields

		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		protected T[] OldContent;
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		protected T[] NewContent;

		private PatchOperationType _poType;
		private List<T> _poData;
		private int _poLength;

		#endregion

		#region Methods

		protected void AppendPatchOperation(PatchOperationList<T> poList, PatchOperationType poType, T poData)
		{
			if (this._poType != poType)
			{
				switch (this._poType)
				{
					case PatchOperationType.Copy:
						poList.Add(new PatchOperation<T>(this._poType, this._poLength, null));
						break;
					case PatchOperationType.Insert:
						poList.Add(new PatchOperation<T>(this._poType, this._poLength, this._poData.ToArray()));
						break;
					case PatchOperationType.Delete:
						poList.Add(new PatchOperation<T>(this._poType, this._poLength, null));
						break;
				}

				this._poType = poType;
				this._poLength = 0;
				this._poData.Clear();
			}

			this._poLength++;

			switch (this._poType)
			{
				case PatchOperationType.Copy:
					break;
				case PatchOperationType.Insert:
					this._poData.Add(poData);
					break;
				case PatchOperationType.Delete:
					break;
			}
		}

		[DebuggerStepThrough]
		protected DiffBase(
			T[] oldContent,
			T[] newContent)
		{
			this.OldContent = oldContent;
			this.NewContent = newContent;
		}

		protected abstract void Calculate(
			PatchOperationList<T> patchOperationList,
			int oldContentOffset,
			int oldContentLength,
			int newContentOffset,
			int newContentLength);

		#endregion

		#endregion

		#region IDiff<T>

		public void BuildPatch(
			PatchOperationList<T> poList,
			int oldContentOffset,
			int oldContentLength,
			int newContentOffset,
			int newContentLength)
		{
			this._poType = PatchOperationType.Unknown;
			this._poData = new List<T>();
			this._poLength = 0;

			Calculate(
				poList,
				oldContentOffset,
				oldContentLength,
				newContentOffset,
				newContentLength);
			AppendPatchOperation(
				poList,
				PatchOperationType.Unknown,
				default(T));
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
