﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Data Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;

namespace Nabu.Data
{
	public sealed class DiffHirschbergText : DiffBase<string>
	{
		#region DiffHirschbergText

		#region Fields

		private int[][] _forward;
		private int[][] _reverse;
		private int _oldOffset;
		private int _newOffset;
		private StringComparison _comparison;

		#endregion

		#region Methods

		private void Forward(
			int oldContentOffset,
			int oldContentLength,
			int newContentOffset,
			int newContentLength)
		{
			this._forward[oldContentOffset % 2][newContentOffset] = 0;

			for (
				int indexNew = newContentOffset + 1;
				indexNew <= (newContentOffset + newContentLength);
				indexNew++)
			{
				this._forward[oldContentOffset % 2][indexNew] =
					this._forward[oldContentOffset % 2][indexNew - 1] + 1;
			}

			for (
				int indexOld = oldContentOffset + 1;
				indexOld <= (oldContentOffset + oldContentLength);
				indexOld++)
			{
				this._forward[indexOld % 2][newContentOffset] =
					this._forward[(indexOld - 1) % 2][newContentOffset] + 1;

				for (
					int indexNew = newContentOffset + 1;
					indexNew <= (newContentOffset + newContentLength);
					indexNew++)
				{
					int diagonal = this._forward[(indexOld - 1) % 2][indexNew - 1];

					if (base.OldContent[indexOld - 1] != base.NewContent[indexNew - 1])
					{
						diagonal++;
					}

					this._forward[indexOld % 2][indexNew] =
						Math.Min(
							diagonal,
							Math.Min(
								this._forward[(indexOld - 1) % 2][indexNew],
								this._forward[indexOld % 2][indexNew - 1]) + 1);
				}
			}
		}

		private void Reverse(
			int oldContentOffset,
			int oldContentLength,
			int newContentOffset,
			int newContentLength)
		{
			this._reverse[(oldContentOffset + oldContentLength) % 2][newContentOffset + newContentLength] = 0;

			for (
				int indexNew = (newContentOffset + newContentLength) - 1;
				indexNew >= newContentOffset;
				indexNew--)
			{
				this._reverse[(oldContentOffset + oldContentLength) % 2][indexNew] =
					this._reverse[(oldContentOffset + oldContentLength) % 2][indexNew + 1] + 1;
			}

			for (
				int indexOld = (oldContentOffset + oldContentLength) - 1;
				indexOld >= oldContentOffset;
				indexOld--)
			{
				this._reverse[indexOld % 2][(newContentOffset + newContentLength)] =
					this._reverse[(indexOld + 1) % 2][(newContentOffset + newContentLength)] + 1;

				for (
					int indexNew = (newContentOffset + newContentLength) - 1;
					indexNew >= newContentOffset;
					indexNew--)
				{
					int diagonal = this._reverse[(indexOld + 1) % 2][indexNew + 1];

					if (base.OldContent[indexOld] != base.NewContent[indexNew])
					{
						diagonal++;
					}

					this._reverse[indexOld % 2][indexNew] =
						Math.Min(
							diagonal,
							Math.Min(
								this._reverse[(indexOld + 1) % 2][indexNew],
								this._reverse[indexOld % 2][indexNew + 1]) + 1);
				}
			}
		}

		private void InternalCalculate(
			PatchOperationList<string> poList,
			int oldContentOffset,
			int oldContentLength,
			int newContentOffset,
			int newContentLength)
		{
			if (oldContentLength <= 0)
			{
				for (
					int index = newContentOffset;
					index < (newContentOffset + newContentLength);
					index++)
				{
					AppendPatchOperation(poList, PatchOperationType.Insert, base.NewContent[index]);
				}
			}
			else if (newContentLength <= 0)
			{
				for (
					int index = oldContentOffset;
					index < (oldContentOffset + oldContentLength);
					index++)
				{
					AppendPatchOperation(poList, PatchOperationType.Delete, base.OldContent[index]);
				}
			}
			else if (oldContentLength == 1)
			{
				string value = base.OldContent[oldContentOffset];
				int valueIndex = newContentOffset;

				for (
					int index = newContentOffset + 1;
					index < (newContentOffset + newContentLength);
					index++)
				{
					if (string.Compare(base.NewContent[index], value, this._comparison) == 0)
					{
						valueIndex = index;
					}
				}

				for (
					int index = newContentOffset;
					index < (newContentOffset + newContentLength);
					index++)
				{
					if (index == valueIndex)
					{
						if (string.Compare(base.NewContent[index], value, this._comparison) == 0)
						{
							AppendPatchOperation(poList, PatchOperationType.Copy, null);
						}
						else
						{
							AppendPatchOperation(poList, PatchOperationType.Delete, base.OldContent[index]);
							AppendPatchOperation(poList, PatchOperationType.Insert, base.NewContent[index]);
						}
					}
					else
					{
						AppendPatchOperation(poList, PatchOperationType.Insert, base.NewContent[index]);
					}
				}
			}
			else
			{
				int oldMiddle = oldContentOffset + (oldContentLength / 2);

				Forward(
					oldContentOffset - this._oldOffset,
					oldMiddle - oldContentOffset,
					newContentOffset - this._newOffset,
					newContentLength);
				Reverse(
					oldMiddle - this._oldOffset,
					(oldContentOffset + oldContentLength) - oldMiddle,
					newContentOffset - this._newOffset,
					newContentLength);

				int newMiddle = newContentOffset;
				int newMiddlePrice = int.MaxValue;

				for (
					int index = newContentOffset;
					index <= (newContentOffset + newContentLength);
					index++)
				{
					int price = this._forward[(oldMiddle - this._oldOffset) % 2][index] + this._reverse[(oldMiddle - this._oldOffset) % 2][index];

					if (price < newMiddlePrice)
					{
						newMiddlePrice = price;
						newMiddle = index;
					}
				}

				InternalCalculate(
					poList,
					oldContentOffset,
					oldMiddle - oldContentOffset,
					newContentOffset,
					newMiddle - newContentOffset);
				InternalCalculate(
					poList,
					oldMiddle,
					(oldContentOffset + oldContentLength) - oldMiddle,
					newMiddle,
					(newContentOffset + newContentLength) - newMiddle);
			}
		}

		public DiffHirschbergText(
			string[] oldContent,
			string[] newContent,
			StringComparison comparison)
			: base(oldContent, newContent)
		{
			this._forward = new int[2][]
			{
				new int[0],
				new int[0]
			};

			this._reverse = new int[2][]
			{
				new int[0],
				new int[0]
			};

			this._comparison = comparison;
		}

		#endregion

		#endregion

		#region DiffBase

		protected override void Calculate(
			PatchOperationList<string> poList,
			int oldContentOffset,
			int oldContentLength,
			int newContentOffset,
			int newContentLength)
		{
			int contentLength = Math.Max(oldContentOffset + oldContentLength, newContentOffset + newContentLength) + 2;

			for (int index = 0; index < 2; index++)
			{
				if (this._forward[index].Length < contentLength)
				{
					this._forward[index] = new int[contentLength];
				}
				else
				{
					Array.Clear(this._forward[index], 0, this._forward[index].Length);
				}

				if (this._reverse[index].Length < contentLength)
				{
					this._reverse[index] = new int[contentLength];
				}
				else
				{
					Array.Clear(this._reverse[index], 0, this._reverse[index].Length);
				}
			}

			this._oldOffset = oldContentOffset;
			this._newOffset = newContentOffset;

			InternalCalculate(
				poList,
				oldContentOffset,
				oldContentLength,
				newContentOffset,
				newContentLength);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
