﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Data Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;

namespace Nabu.Data
{
	public class Patch<T> : IPatch<T>
	{
		#region Patch

		#endregion

		#region IPatch

		public void Apply(
			ICollection<T> newContent,
			T[] oldContent,
			int oldContentOffset,
			int oldContentLength,
			PatchOperationList<T> poList)
		{
			int oldIndex = oldContentOffset;

			foreach (PatchOperation<T> po in poList)
			{
				switch (po.Type)
				{
					case PatchOperationType.Copy:
						for (int index = oldIndex; index < oldIndex + po.Length; index++)
						{
							newContent.Add(oldContent[index]);
						}

						oldIndex += po.Length;
						break;
					case PatchOperationType.Insert:
						for (int index = 0; index < po.Length; index++)
						{
							newContent.Add(po.Data[index]);
						}
						break;
					case PatchOperationType.Delete:
						oldIndex += po.Length;
						break;
				}
			}
		}

		public void Apply(
			T[] newContent,
			int newContentOffset,
			T[] oldContent,
			int oldContentOffset,
			int oldContentLength,
			PatchOperationList<T> poList)
		{
			int oldIndex = oldContentOffset;
			int newIndex = newContentOffset;

			foreach (PatchOperation<T> po in poList)
			{
				switch (po.Type)
				{
					case PatchOperationType.Copy:
						Array.Copy(oldContent, oldIndex, newContent, newIndex, po.Length);
						oldIndex += po.Length;
						newIndex += po.Length;
						break;
					case PatchOperationType.Insert:
						Array.Copy(po.Data, 0, newContent, newIndex, po.Length);
						newIndex += po.Length;
						break;
					case PatchOperationType.Delete:
						oldIndex += po.Length;
						break;
				}
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
