﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Data Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Collections.Generic;
using System.Diagnostics;

namespace Nabu.Data
{
	public class PatchOperationList<T> : IList<PatchOperation<T>>
	{
		#region PatchOperationList

		#region Fields

		private IList<PatchOperation<T>> _patchOperations = new List<PatchOperation<T>>();

		#endregion

		#region Methods

		public PatchOperationList()
		{
			this._patchOperations = new List<PatchOperation<T>>();
		}

		public PatchOperationList(int capacity)
		{
			this._patchOperations = new List<PatchOperation<T>>(capacity);
		}

		public void Optimize()
		{
			while ((this._patchOperations.Count > 0) && (this._patchOperations[this._patchOperations.Count - 1].Type == PatchOperationType.Delete))
			{
				this._patchOperations.RemoveAt(this._patchOperations.Count - 1);
			}
		}

		#endregion

		#endregion

		#region IList<PatchOperation>

		public int IndexOf(PatchOperation<T> item)
		{
			return this._patchOperations.IndexOf(item);
		}

		public void Insert(int index, PatchOperation<T> item)
		{
			this._patchOperations.Insert(index, item);
		}

		public void RemoveAt(int index)
		{
			this._patchOperations.RemoveAt(index);
		}

		public PatchOperation<T> this[int index]
		{
			[DebuggerStepThrough]
			get { return this._patchOperations[index]; }
			[DebuggerStepThrough]
			set { this._patchOperations[index] = value; }
		}

		#endregion

		#region ICollection<PatchOperation>

		public void Add(PatchOperation<T> item)
		{
			this._patchOperations.Add(item);
		}

		public void Clear()
		{
			this._patchOperations.Clear();
		}

		public bool Contains(PatchOperation<T> item)
		{
			return this._patchOperations.Contains(item);
		}

		public void CopyTo(PatchOperation<T>[] array, int arrayIndex)
		{
			this._patchOperations.CopyTo(array, arrayIndex);
		}

		public int Count
		{
			[DebuggerStepThrough]
			get { return this._patchOperations.Count; }
		}

		public bool IsReadOnly
		{
			[DebuggerStepThrough]
			get { return false; }
		}

		public bool Remove(PatchOperation<T> item)
		{
			return this._patchOperations.Remove(item);
		}

		#endregion

		#region IEnumerable<PatchOperation>

		public IEnumerator<PatchOperation<T>> GetEnumerator()
		{
			return this._patchOperations.GetEnumerator();
		}

		#endregion

		#region IEnumerable

		System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
		{
			return ((System.Collections.IEnumerable)this._patchOperations).GetEnumerator();
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
