﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.DebuggerVisualizers Library              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Reflection;
using System.Reflection.Emit;

namespace Nabu.DebuggerVisualizers
{
	[Serializable]
	public sealed class DynamicMetadataTokenResolver : IMetadataTokenResolver
	{
		#region MetadataTokenResolver

		#region Fields

		private static Type _genericMethodInfoType = Type.GetType("System.Reflection.Emit.GenericMethodInfo");
		private static FieldInfo _genericMethodInfoMethod = _genericMethodInfoType.GetField("m_method", BindingFlags.NonPublic | BindingFlags.Instance);
		private static FieldInfo _genericMethodInfoContext = _genericMethodInfoType.GetField("m_context", BindingFlags.NonPublic | BindingFlags.Instance);
		private static Type _varArgMethodType = Type.GetType("System.Reflection.Emit.VarArgMethod");

		private object[] _tokens;

		#endregion

		#region Methods

		public DynamicMetadataTokenResolver(object[] tokens)
		{
			this._tokens = tokens;
		}

		#endregion

		#endregion

		#region IMetadataTokenResolver

		public MethodBase ResolveMethod(int metadataToken)
		{
			object token = this._tokens[metadataToken & 0x00FFFFFF];
			DynamicMethod dynamicMethod = token as DynamicMethod;

			if (dynamicMethod != null)
			{
				return dynamicMethod;
			}

			if (token is RuntimeMethodHandle)
			{
				return MethodBase.GetMethodFromHandle((RuntimeMethodHandle)token);
			}

			if (token.GetType() == _genericMethodInfoType)
			{
				return MethodBase.GetMethodFromHandle(
					(RuntimeMethodHandle)_genericMethodInfoMethod.GetValue(token),
					(RuntimeTypeHandle)_genericMethodInfoContext.GetValue(token));
			}

			if (token.GetType() == _varArgMethodType)
			{
				return MethodBase.GetMethodFromHandle(
					(RuntimeMethodHandle)_genericMethodInfoMethod.GetValue(token));
			}

			return null;
		}

		public FieldInfo ResolveField(int metadataToken)
		{
			return FieldInfo.GetFieldFromHandle((RuntimeFieldHandle)this._tokens[metadataToken & 0x00FFFFFF]);
		}

		public Type ResolveType(int metadataToken)
		{
			return Type.GetTypeFromHandle((RuntimeTypeHandle)this._tokens[metadataToken & 0x00FFFFFF]);
		}

		public String ResolveString(int metadataToken)
		{
			return this._tokens[metadataToken & 0x00FFFFFF] as string;
		}

		public MemberInfo ResolveMember(int metadataToken)
		{
			if ((metadataToken & 0x02000000) == 0x02000000)
			{
				return ResolveType(metadataToken);
			}

			if ((metadataToken & 0x04000000) == 0x04000000)
			{
				return ResolveField(metadataToken);
			}

			if ((metadataToken & 0x06000000) == 0x06000000)
			{
				return ResolveMethod(metadataToken);
			}

			return null;
		}

		public byte[] ResolveSignature(int metadataToken)
		{
			return this._tokens[metadataToken & 0x00FFFFFF] as byte[];
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
