﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.DebuggerVisualizers Library              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Reflection.Emit;
using System.Runtime.Serialization.Formatters.Binary;

using Microsoft.VisualStudio.DebuggerVisualizers;

namespace Nabu.DebuggerVisualizers
{
	public sealed class DynamicMethodObjectSource : VisualizerObjectSource
	{
		#region Fields

		private static FieldInfo _ilLengthField = typeof(ILGenerator).GetField("m_length", BindingFlags.NonPublic | BindingFlags.Instance);
		private static FieldInfo _ilStreamField = typeof(ILGenerator).GetField("m_ILStream", BindingFlags.NonPublic | BindingFlags.Instance);
		private static FieldInfo _scopeField = Type.GetType("System.Reflection.Emit.DynamicILGenerator").GetField("m_scope", BindingFlags.NonPublic | BindingFlags.Instance);
		private static FieldInfo _tokensField = Type.GetType("System.Reflection.Emit.DynamicScope").GetField("m_tokens", BindingFlags.NonPublic | BindingFlags.Instance);

		#endregion

		#region Methods

		public override void GetData(object target, Stream outgoingData)
		{
			DynamicMethod dynamicMethod = target as DynamicMethod;
			ILGenerator ilGenerator = dynamicMethod.GetILGenerator();
			byte[] il = new byte[(int)_ilLengthField.GetValue(ilGenerator)];

			Array.Copy((byte[])_ilStreamField.GetValue(ilGenerator), il, il.Length);

			object scope = _scopeField.GetValue(ilGenerator);
			ArrayList tokens = (ArrayList)_tokensField.GetValue(scope);
			List<KeyValuePair<int, Type>> localVariables = new List<KeyValuePair<int, Type>>();

			DynamicMethodInfo dynamicMethodInfo = new DynamicMethodInfo(
				il,
				localVariables,
				tokens.ToArray());

			BinaryFormatter binaryFormatter = new BinaryFormatter();

			binaryFormatter.Serialize(outgoingData, dynamicMethodInfo);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
