﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.DebuggerVisualizers Library              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Reflection;
using System.Reflection.Emit;
using System.Text;
using System.Windows.Forms;

using Nabu.Forms.Html;
using Nabu.Reflection;

namespace Nabu.DebuggerVisualizers
{
	public partial class ILViewerForm : Form
	{
		#region Fields

		private IMetadataTokenResolver _metadataTokenResolver;
		private IList<KeyValuePair<int, Type>> _localVariables;
		private byte[] _il;
		private bool _isViewOutdated;

		#endregion

		#region Methods

		private static bool GetBool(string valueName)
		{
			object value = Microsoft.Win32.Registry.GetValue(
				"HKEY_CURRENT_USER\\Software\\TrifleSoft\\Nabu\\DebuggerVisualizer",
				valueName,
				false);

			return value.Equals("True");
		}

		private static void SetBool(string valueName, bool value)
		{
			Microsoft.Win32.Registry.SetValue(
				"HKEY_CURRENT_USER\\Software\\TrifleSoft\\Nabu\\DebuggerVisualizer",
				valueName,
				value);
		}

		/*
				public string GetMemberUrl(MethodBase methodBase)
				{
					StringBuilder result = new StringBuilder();

					result.AppendFormat(
						"code://{0}/{1}{2}/{3}{4}(",
						System.IO.Path.GetFileNameWithoutExtension(methodBase.Module.Name),
						methodBase.ReflectedType.FullName,
						methodBase.ReflectedType.IsGenericType ? "<>" : "",
						methodBase.Name,
						methodBase.IsGenericMethod ? "<>" : "");

					ParameterInfo[] parameters = methodBase.GetParameters();

					if (parameters.Length > 0)
					{
						result.Append(parameters[0].ParameterType.Name);

						for (int index = 1; index < parameters.Length; index++)
						{
							result.AppendFormat(
								", {0}",
								parameters[index].ParameterType);
						}
					}

					if (methodBase.IsConstructor)
					{
						result.Append(")");
					}
					else
					{
						result.AppendFormat(
							"):{0}",
							(methodBase as MethodInfo).ReturnType);
					}

					return result.ToString();
				}
		*/

		public ILViewerForm(
			IMetadataTokenResolver metadataTokenResolver,
			IList<KeyValuePair<int, Type>> localVariables,
			byte[] il)
		{
			InitializeComponent();
			this._metadataTokenResolver = metadataTokenResolver;
			this._localVariables = localVariables;
			this._il = il;
		}

		private void ILViewerForm_Shown(object sender, EventArgs e)
		{
			this._comboLanguage.SelectedIndex = 0;
			this._checkOffset.Checked = GetBool("OffsetVisible");
			this._checkRawBytes.Checked = GetBool("RawBytesVisible");
			InvalidateView();
			UpdateView();
		}

		private void ILViewerForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			SetBool("OffsetVisible", this._checkOffset.Checked);
			SetBool("RawBytesVisible", this._checkRawBytes.Checked);
		}

		private void Check_CheckedChanged(object sender, EventArgs e)
		{
			InvalidateView();
			UpdateView();
		}

		private void ComboLanguage_SelectedIndexChanged(object sender, EventArgs e)
		{
			InvalidateView();
			UpdateView();
		}

		private void InvalidateView()
		{
			this._isViewOutdated = true;
		}

		private void UpdateView()
		{
			if (this._isViewOutdated)
			{
				StringBuilder ilHtml = new StringBuilder();

				bool isOffsetVisible = this._checkOffset.Checked;
				bool isRawBytesVisible = this._checkRawBytes.Checked;
				ProgrammingLanguage language = (ProgrammingLanguage)this._comboLanguage.SelectedIndex;

				try
				{
					ILParser parser = new ILParser(this._il);

					ilHtml.Append("<table cellspacing='1px'>");
					ilHtml.AppendLine();

					foreach (KeyValuePair<int, Type> localVariable in this._localVariables)
					{
						ilHtml.AppendFormat(
							"<tr><td class='opcode'>local_{0}</td><td class='operand'>{1}</td></tr>",
							localVariable.Key,
							localVariable.Value);
					}

					ilHtml.Append("</table><br/>");
					ilHtml.AppendLine();

					ilHtml.Append("<table cellspacing='1px'>");
					ilHtml.AppendLine();

					foreach (ILInstruction ilInstruction in parser)
					{
						ilHtml.AppendFormat("<tr name='{0}'>", ilInstruction.Offset);

						if (isOffsetVisible)
						{
							ilHtml.AppendFormat("<td class='offset'>0x{0:x8}</td>", ilInstruction.Offset);
						}

						if (isRawBytesVisible)
						{
							ilHtml.Append("<td class='rawbytes'>");

							for (int index = 0; index < ilInstruction.Length; index++)
							{
								ilHtml.AppendFormat("0x{0:x2} ", this._il[ilInstruction.Offset + index]);
							}

							ilHtml.Append("</td>");
						}

						ilHtml.AppendFormat("<td class='opcode'>{0}</td>", ilInstruction.OpCode.Name);

						switch (ilInstruction.OpCode.OperandType)
						{
							// The operand is a 32-bit integer branch target.
							case OperandType.InlineBrTarget:
								ilHtml.AppendFormat(
									"<td class='operand'><a href='#{0}'>0x{0:x8}</a></td>",
									ilInstruction.Offset + (int)ilInstruction.Operands[0]);
								break;
							// The operand is a 32-bit metadata token.
							case OperandType.InlineField:
								{
									FieldInfo fieldInfo = this._metadataTokenResolver.ResolveField((int)ilInstruction.Operands[0]);

									ilHtml.AppendFormat(
										"<td class='operand'>{0}</td>",
										MemberDeclaration.GetDeclaration(fieldInfo, language));
								}
								break;
							// The operand is a 32-bit integer.
							case OperandType.InlineI:
								ilHtml.AppendFormat(
									"<td class='operand'>0x{0:x8} ({0})</td>",
									(int)ilInstruction.Operands[0]);
								break;
							// The operand is a 64-bit integer.
							case OperandType.InlineI8:
								ilHtml.AppendFormat(
									"<td class='operand'>0x{0:x16} ({0})</td>",
									(long)ilInstruction.Operands[0]);
								break;
							// The operand is a 32-bit metadata token.
							case OperandType.InlineMethod:
								{
									MethodBase methodBase = this._metadataTokenResolver.ResolveMethod((int)ilInstruction.Operands[0]);

									ilHtml.AppendFormat(
										"<td class='operand'>{0}</td>",
										MemberDeclaration.GetDeclaration(methodBase, language));
								}
								break;
							// No operand.
							case OperandType.InlineNone:
								break;
							// The operand is reserved and should not be used.
							//case OperandType.InlinePhi:
							//break;
							// The operand is a 64-bit IEEE floating point number.
							case OperandType.InlineR:
								ilHtml.AppendFormat(
									"<td class='operand'>{0}</td>",
									(double)ilInstruction.Operands[0]);
								break;
							// The operand is a 32-bit metadata signature token.
							case OperandType.InlineSig:
								ilHtml.AppendFormat(
									"<td class='operand'>InlineSig {0}</td>",
									ilInstruction.Operands[0]);
								break;
							// The operand is a 32-bit metadata string token.
							case OperandType.InlineString:
								ilHtml.AppendFormat(
									"<td class='operand'>InlineString {0}</td>",
									ilInstruction.Operands[0]);
								break;
							// The operand is the 32-bit integer argument to a switch instruction.
							case OperandType.InlineSwitch:
								ilHtml.AppendFormat(
									"<td class='operand'>InlineSwitch</td>");
								break;
							// The operand is a FieldRef, MethodRef, or TypeRef token.
							case OperandType.InlineTok:
								ilHtml.AppendFormat(
									"<td class='operand'>InlineTok {0}</td>",
									ilInstruction.Operands[0]);
								break;
							// The operand is a 32-bit metadata token.
							case OperandType.InlineType:
								{
									Type type = this._metadataTokenResolver.ResolveType((int)ilInstruction.Operands[0]);

									ilHtml.AppendFormat(
										"<td class='operand'>{0}</td>",
										type);
								}
								break;
							// The operand is 16-bit integer containing the ordinal of a local variable or an argument.
							case OperandType.InlineVar:
								ilHtml.AppendFormat(
									"<td class='operand'>local_{0}</td>",
									(ushort)ilInstruction.Operands[0]);
								break;
							// The operand is an 8-bit integer branch target.
							case OperandType.ShortInlineBrTarget:
								ilHtml.AppendFormat(
									"<td class='operand'><a href='#{0}'>0x{0:x8}</a></td>",
									ilInstruction.Offset + (sbyte)ilInstruction.Operands[0]);
								break;
							// The operand is an 8-bit integer.
							case OperandType.ShortInlineI:
								ilHtml.AppendFormat(
									"<td class='operand'>0x{0:x2} ({0})</td>",
									(byte)ilInstruction.Operands[0]);
								break;
							// The operand is a 32-bit IEEE floating point number.
							case OperandType.ShortInlineR:
								ilHtml.AppendFormat(
									"<td class='operand'>{0}</td>",
									(float)ilInstruction.Operands[0]);
								break;
							// The operand is an 8-bit integer containing the ordinal of a local variable or an argument.
							case OperandType.ShortInlineVar:
								ilHtml.AppendFormat(
									"<td class='operand'>local_{0}</td>",
									(byte)ilInstruction.Operands[0]);
								break;
						}

						ilHtml.Append("</tr>");
						ilHtml.AppendLine();
					}

					ilHtml.Append("</table>");
					ilHtml.AppendLine();
				}
				catch (BadImageFormatException exception)
				{
					ilHtml.Append("<br/>");
					ilHtml.Append(exception.Message);
				}

				HtmlTag body = this._htmlCode.RootTag.SelectFirst("body");

				body.InnerHtml = ilHtml.ToString();
				body.Update(true);

				this._isViewOutdated = false;
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
