﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.DebuggerVisualizers Library              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Reflection;

using Microsoft.VisualStudio.DebuggerVisualizers;

[assembly:
	DebuggerVisualizer(
		typeof(Nabu.DebuggerVisualizers.MethodInfoVisualizer),
		typeof(VisualizerObjectSource),
		Target = typeof(System.Reflection.MethodBase),
		Description = "IL Visualizer")]

namespace Nabu.DebuggerVisualizers
{
	public sealed class MethodInfoVisualizer : DialogDebuggerVisualizer
	{
		protected override void Show(IDialogVisualizerService windowService, IVisualizerObjectProvider objectProvider)
		{
			MethodBase methodInfo = (MethodBase)objectProvider.GetObject();
			MethodBody methodBody = methodInfo.GetMethodBody();
			List<KeyValuePair<int, Type>> localVariables = new List<KeyValuePair<int, Type>>();

			foreach (LocalVariableInfo localVariableInfo in methodBody.LocalVariables)
			{
				localVariables.Add(
					new KeyValuePair<int, Type>(
						localVariableInfo.LocalIndex,
						localVariableInfo.LocalType));
			}

			ILViewerForm ilViewerForm = new ILViewerForm(
				new MetadataTokenResolver(methodInfo.Module),
				localVariables,
				methodBody.GetILAsByteArray());

			ilViewerForm.ShowDialog();
		}
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
