﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Diagnostics Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Text;

namespace Nabu.Diagnostics
{
	public sealed class DeviceClass
	{
		#region Fields

		private Guid _guid;
		private string _name;
		private string _description;

		#endregion

		#region Methods

		internal DeviceClass(Guid guid)
		{
			this._guid = guid;
		}

		public DeviceInstanceCollection GetInstances(bool onlyPresent, bool onlyProfile)
		{
			return new DeviceInstanceCollection(this._guid, onlyPresent, onlyProfile);
		}

		public DeviceInstanceCollection GetInstances()
		{
			return GetInstances(true, true);
		}

		#endregion

		#region Properties

		public string Name
		{
			get
			{
				if (this._name == null)
				{
					StringBuilder name = new StringBuilder(64);
					int requiredSize;

					if (NativeMethods.SetupDiClassNameFromGuidW(
						ref this._guid,
						name,
						name.Capacity,
						out requiredSize))
					{
						this._name = name.ToString();
					}
				}

				return this._name;
			}
		}

		public string Description
		{
			get
			{
				if (this._description == null)
				{
					StringBuilder description = new StringBuilder(1024);
					int requiredSize;

					if (NativeMethods.SetupDiGetClassDescriptionW(
						ref this._guid,
						description,
						description.Capacity,
						out requiredSize))
					{
						this._description = description.ToString();
					}
				}

				return this._description;
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
