﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Diagnostics Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Diagnostics;
using System.Globalization;

namespace Nabu.Diagnostics
{
	[System.Diagnostics.DebuggerDisplay("Channel: {Channel}; Min: {Min}; Max: {Max}; Flags: {Flags}")]
	public sealed class DeviceDmaRange
	{
		#region DeviceDmaRange

		#region Fields

		private int _min;
		private int _max;
		private DeviceDmaFlags _flags;
		private int _channel;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		internal DeviceDmaRange(int min, int max, DeviceDmaFlags flags, int channel)
		{
			this._min = min;
			this._max = max;
			this._flags = flags;
			this._channel = channel;
		}

		#endregion

		#region Properties

		public int Min
		{
			[DebuggerStepThrough]
			get { return this._min; }
		}

		public int Max
		{
			[DebuggerStepThrough]
			get { return this._max; }
		}

		public DeviceDmaFlags Flags
		{
			[DebuggerStepThrough]
			get { return this._flags; }
		}

		public int Channel
		{
			[DebuggerStepThrough]
			get { return this._channel; }
		}

		public bool IsReadable
		{
			[DebuggerStepThrough]
			get { return (this._flags & DeviceDmaFlags.Is16Bit) != 0; }
		}

		public bool IsWritable
		{
			[DebuggerStepThrough]
			get { return (this._flags & DeviceDmaFlags.Is32Bit) != 0; }
		}

		public bool IsCacheable
		{
			[DebuggerStepThrough]
			get { return (this._flags & DeviceDmaFlags.IsBusMaster) != 0; }
		}

		public char Type
		{
			[DebuggerStepThrough]
			get
			{
				switch (((int)this._flags) & 0x18)
				{
					case 0x08: return 'A';
					case 0x10: return 'B';
					case 0x18: return 'F';
					default: return '?';
				}
			}
		}

		#endregion

		#endregion

		#region Object

		public override string ToString()
		{
			return
				string.Format(
					CultureInfo.InvariantCulture,
					"Channel: {0}; Min: {1}; Max: {2}; Flags: {3}",
					this._channel,
					this._min,
					this._max,
					this._flags);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
