﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Diagnostics Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Security.Permissions;

namespace Nabu.Diagnostics
{
	public sealed class DeviceDriver
	{
		#region Fields

		private string _manufacturer;
		private string _provider;
		private string _description;
		private DateTime _releaseDate;
		private Version _version;
		private IList<string> _files;

		#endregion

		#region Methods

		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.UnmanagedCode)]
		[DebuggerStepThrough]
		internal DeviceDriver(
			IntPtr infoSet,
			DeviceInstanceInfo instanceInfo,
			DeviceDriverInfo driverInfo)
		{
			this._manufacturer = driverInfo.Manufacturer;
			this._provider = driverInfo.Provider;
			this._description = driverInfo.Description;
			this._releaseDate = DateTime.FromFileTime(driverInfo.ReleaseDate);
			this._version =
				new Version(
					(int)((driverInfo.Version >> 48) & 0xFFFF),
					(int)((driverInfo.Version >> 32) & 0xFFFF),
					(int)((driverInfo.Version >> 16) & 0xFFFF),
					(int)((driverInfo.Version >> 0) & 0xFFFF));
			this._files = new List<string>();

			IntPtr hQueue = NativeMethods.SetupOpenFileQueue();

			if (hQueue != new IntPtr(-1))
			{
				DeviceInstallParameters installParameters = new DeviceInstallParameters();

				installParameters.Size = Marshal.SizeOf(typeof(DeviceInstallParameters));

				if (NativeMethods.SetupDiGetDeviceInstallParamsW(
					infoSet,
					ref instanceInfo,
					ref installParameters))
				{
					installParameters.Flags |= DeviceInstallFlags.NOVCP;
					installParameters.hFileQueue = hQueue;

					if (NativeMethods.SetupDiSetDeviceInstallParamsW(
						infoSet,
						ref instanceInfo,
						ref installParameters))
					{
						if (NativeMethods.SetupDiSetSelectedDriverW(
						infoSet,
						ref instanceInfo,
						ref driverInfo))
						{
							if (NativeMethods.SetupDiCallClassInstaller(
								DeviceDriverInstallFunction.InstallDeviceFiles,
								infoSet,
								ref instanceInfo))
							{
								DeviceDriverFileCallback fileCallback = new DeviceDriverFileCallback(OnDriverFile);
								int result;

								NativeMethods.SetupScanFileQueueW(
									hQueue,
									DeviceDriverFileQueueFlags.UseCallback,
									IntPtr.Zero,
									fileCallback,
									IntPtr.Zero,
									out result);

								GC.KeepAlive(fileCallback);
							}
						}
					}
				}

				NativeMethods.SetupCloseFileQueue(hQueue);
			}
		}

		private int OnDriverFile(
			IntPtr context,
			int notification,
			IntPtr param1,
			IntPtr param2)
		{
			this._files.Add(Marshal.PtrToStringUni(param1));

			return 0;
		}

		#endregion

		#region Properties

		public string Manufacturer
		{
			[DebuggerStepThrough]
			get { return this._manufacturer; }
		}

		public string Provider
		{
			[DebuggerStepThrough]
			get { return this._provider; }
		}

		public string Description
		{
			[DebuggerStepThrough]
			get { return this._description; }
		}

		public DateTime ReleaseDate
		{
			[DebuggerStepThrough]
			get { return this._releaseDate; }
		}

		public Version Version
		{
			[DebuggerStepThrough]
			get { return this._version; }
		}

		public ICollection<string> Files
		{
			[DebuggerStepThrough]
			get { return this._files; }
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
