﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Diagnostics Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Security.Permissions;

namespace Nabu.Diagnostics
{
	public sealed class DeviceInstanceCollection :
		DisposableBase,
		ICollection<DeviceInstance>
	{
		#region DeviceInstanceCollection

		#region Constants

		private const string ModificationExceptionMessage = "Device collection is read-only.";

		#endregion

		#region Fields

		private IntPtr _hInfoSet;
		private List<DeviceInstance> _instances;

		#endregion

		#region Methods

		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.UnmanagedCode)]
		internal DeviceInstanceCollection(Guid classGuid, bool onlyPresent, bool onlyProfile)
		{
			this._hInfoSet =
				NativeMethods.SetupDiGetClassDevsW(
					ref classGuid,
					null,
					IntPtr.Zero,
					(onlyPresent ? 2 : 0) | (onlyProfile ? 8 : 0));

			if (!this._hInfoSet.Equals(new IntPtr(-1)))
			{
				DeviceInstanceInfo instanceInfo = new DeviceInstanceInfo();

				instanceInfo.Size = Marshal.SizeOf(typeof(DeviceInstanceInfo));

				this._instances = new List<DeviceInstance>();

				for (
					int index = 0;
					NativeMethods.SetupDiEnumDeviceInfo(
						this._hInfoSet,
						index,
						out instanceInfo);
					index++)
				{
					this._instances.Add(
						new DeviceInstance(
							this._hInfoSet,
							instanceInfo));
					instanceInfo.Size = Marshal.SizeOf(typeof(DeviceInstanceInfo));
				}
			}
		}

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			if (this._hInfoSet != IntPtr.Zero)
			{
				NativeMethods.SetupDiDestroyDeviceInfoList(this._hInfoSet);
				this._hInfoSet = IntPtr.Zero;
			}

			base.Dispose(disposing);
		}

		#endregion

		#region ICollection<DeviceInstance>

		#region Methods

		public void Add(DeviceInstance item)
		{
			throw new NotImplementedException(ModificationExceptionMessage);
		}

		public bool Remove(DeviceInstance item)
		{
			throw new NotImplementedException(ModificationExceptionMessage);
		}

		public void Clear()
		{
			throw new NotImplementedException(ModificationExceptionMessage);
		}

		public bool Contains(DeviceInstance item)
		{
			base.CheckDisposed();

			return this._instances.Contains(item);
		}

		public void CopyTo(DeviceInstance[] array, int arrayIndex)
		{
			base.CheckDisposed();

			this._instances.CopyTo(array, arrayIndex);
		}

		#endregion

		#region Properties

		public int Count
		{
			get
			{
				base.CheckDisposed();

				return this._instances.Count;
			}
		}

		public bool IsReadOnly
		{
			get
			{
				base.CheckDisposed();

				return true;
			}
		}

		#endregion

		#endregion

		#region IEnumerable<DeviceInstance>

		public IEnumerator<DeviceInstance> GetEnumerator()
		{
			base.CheckDisposed();

			return this._instances.GetEnumerator();
		}

		#endregion

		#region IEnumerable

		System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
		{
			base.CheckDisposed();

			return this._instances.GetEnumerator();
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
