﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Diagnostics Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Diagnostics;
using System.Globalization;

namespace Nabu.Diagnostics
{
	[System.Diagnostics.DebuggerDisplay("Number: {Number}; Min: {Min}; Max: {Max}; Flags: {Flags}")]
	public sealed class DeviceIrqRange
	{
		#region DeviceIrqRange

		#region Fields

		private int _min;
		private int _max;
		private DeviceIrqFlags _flags;
		private int _number;
		private long _cpuAffinity;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		internal DeviceIrqRange(int min, int max, DeviceIrqFlags flags, int number, long cpuAffinity)
		{
			this._min = min;
			this._max = max;
			this._flags = flags;
			this._number = number;
			this._cpuAffinity = cpuAffinity;
		}

		#endregion

		#region Properties

		public int Min
		{
			[DebuggerStepThrough]
			get { return this._min; }
		}

		public int Max
		{
			[DebuggerStepThrough]
			get { return this._max; }
		}

		public DeviceIrqFlags Flags
		{
			[DebuggerStepThrough]
			get { return this._flags; }
		}

		public int Number
		{
			[DebuggerStepThrough]
			get { return this._number; }
		}

		public long CpuAffinity
		{
			[DebuggerStepThrough]
			get { return this._cpuAffinity; }
		}

		public bool IsShared
		{
			[DebuggerStepThrough]
			get { return (this._flags & DeviceIrqFlags.IsShared) != 0; }
		}

		public bool IsEdgeSensitive
		{
			[DebuggerStepThrough]
			get { return (this._flags & DeviceIrqFlags.IsEdgeSensitive) != 0; }
		}

		#endregion

		#endregion

		#region Object

		public override string ToString()
		{
			return
				string.Format(
					CultureInfo.InvariantCulture,
					"Number: {Number}; Min: {Min}; Max: {Max}; Flags: {Flags}",
					this._number,
					this._min,
					this._max,
					this._flags);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
