﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Diagnostics Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Diagnostics;
using System.Globalization;

namespace Nabu.Diagnostics
{
	[System.Diagnostics.DebuggerDisplay("Min: {Min}; Max: {Max}; Flags: {Flags}")]
	public sealed class DeviceMemoryRange
	{
		#region DeviceMemoryRange

		#region Fields

		private long _min;
		private long _max;
		private DeviceMemoryFlags _flags;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		internal DeviceMemoryRange(long min, long max, DeviceMemoryFlags flags)
		{
			this._min = min;
			this._max = max;
			this._flags = flags;
		}

		#endregion

		#region Properties

		public long Min
		{
			[DebuggerStepThrough]
			get { return this._min; }
		}

		public long Max
		{
			[DebuggerStepThrough]
			get { return this._max; }
		}

		public DeviceMemoryFlags Flags
		{
			[DebuggerStepThrough]
			get { return this._flags; }
		}

		public bool IsReadable
		{
			[DebuggerStepThrough]
			get { return (this._flags & DeviceMemoryFlags.IsReadable) != 0; }
		}

		public bool IsWritable
		{
			[DebuggerStepThrough]
			get { return (this._flags & DeviceMemoryFlags.IsWritable) != 0; }
		}

		public bool IsCacheable
		{
			[DebuggerStepThrough]
			get { return (this._flags & DeviceMemoryFlags.IsCacheable) != 0; }
		}

		public bool IsWriteCombined
		{
			[DebuggerStepThrough]
			get { return (this._flags & DeviceMemoryFlags.IsWriteCombined) != 0; }
		}

		public bool Is32Bit
		{
			[DebuggerStepThrough]
			get { return (this._flags & DeviceMemoryFlags.Is32Bit) != 0; }
		}

		#endregion

		#endregion

		#region Object

		public override string ToString()
		{
			return
				string.Format(
					CultureInfo.InvariantCulture,
					"Min: {1}; Max: {2}; Flags: {3}",
					this._min,
					this._max,
					this._flags);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
