﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Diagnostics Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Diagnostics;
using System.Globalization;

namespace Nabu.Diagnostics
{
	[System.Diagnostics.DebuggerDisplay("Min: {Min}; Max: {Max}; Flags: {Flags}")]
	public sealed class DevicePortRange
	{
		#region DevicePortRange

		#region Fields

		private long _min;
		private long _max;
		private DevicePortFlags _flags;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		internal DevicePortRange(long min, long max, DevicePortFlags flags)
		{
			this._min = min;
			this._max = max;
			this._flags = flags;
		}

		#endregion

		#region Properties

		public long Min
		{
			[DebuggerStepThrough]
			get { return this._min; }
		}

		public long Max
		{
			[DebuggerStepThrough]
			get { return this._max; }
		}

		public DevicePortFlags Flags
		{
			[DebuggerStepThrough]
			get { return this._flags; }
		}

		public bool IsIoPort
		{
			[DebuggerStepThrough]
			get { return (this._flags & DevicePortFlags.IsIoPort) != 0; }
		}

		public bool IsDecode10bit
		{
			[DebuggerStepThrough]
			get { return (this._flags & DevicePortFlags.IsDecode10Bit) != 0; }
		}

		public bool IsDecode12bit
		{
			[DebuggerStepThrough]
			get { return (this._flags & DevicePortFlags.IsDecode12Bit) != 0; }
		}

		public bool IsDecode16bit
		{
			[DebuggerStepThrough]
			get { return (this._flags & DevicePortFlags.IsDecode16Bit) != 0; }
		}

		public bool IsDecodePositive
		{
			[DebuggerStepThrough]
			get { return (this._flags & DevicePortFlags.IsDecodePositive) != 0; }
		}

		public bool IsDecodePassive
		{
			[DebuggerStepThrough]
			get { return (this._flags & DevicePortFlags.IsDecodePassive) != 0; }
		}

		#endregion

		#endregion

		#region Object

		public override string ToString()
		{
			return
				string.Format(
					CultureInfo.InvariantCulture,
					"Min: {1}; Max: {2}; Flags: {3}",
					this._min,
					this._max,
					this._flags);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
