﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu Library                                  */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.InteropServices;
using System.Runtime.InteropServices.ComTypes;
using System.Text;

namespace Nabu.Diagnostics
{
	[ComImport]
	[Guid("e707dcde-d1cd-11d2-bab9-00c04f8eceae")]
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[CLSCompliant(false)]
	public interface IAssemblyCache
	{
		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "3#")]
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult UninstallAssembly(
			uint flags,
			[MarshalAs(UnmanagedType.LPWStr)] string assemblyName,
			[MarshalAs(UnmanagedType.LPArray)] FusionInstallReference[] references,
			[MarshalAs(UnmanagedType.U4)] out AssemblyCacheUninstallDisposition disposition);

		[SuppressMessage("Microsoft.Design", "CA1045", MessageId = "2#")]
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult QueryAssemblyInfo(
			uint flags,
			[MarshalAs(UnmanagedType.LPWStr)] string assemblyName,
			ref AssemblyInfo assemblyInfo);

		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "2#")]
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult CreateAssemblyCacheItem(
			uint flags,
			IntPtr reserved,
			out IAssemblyCacheItem assemblyCacheItem,
			[MarshalAs(UnmanagedType.LPWStr)] string assemblyName);

		[SuppressMessage("Microsoft.Design", "CA1007")]
		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "0#")]
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult CreateAssemblyScavenger(
			[MarshalAs(UnmanagedType.IUnknown)] out object assemblyScavenger);

		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult InstallAssembly(
			uint flags,
			[MarshalAs(UnmanagedType.LPWStr)] string manifestFilePath,
			[MarshalAs(UnmanagedType.LPArray)] FusionInstallReference[] references);
	}

	[ComImport]
	[Guid("CD193BC0-B4BC-11d2-9833-00C04FC31D2E")]
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[CLSCompliant(false)]
	public interface IAssemblyName
	{
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult SetProperty(
			AssemblyNamePart propertyId,
			IntPtr propertyPtr,
			int propertySize);

		[SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference", MessageId = "2#")]
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult GetProperty(
			AssemblyNamePart propertyId,
			IntPtr propertyPtr,
			ref int propertySize);

		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult Finalize();

		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "0#")]
		[SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference", MessageId = "1#")]
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult GetDisplayName(
			[Out][MarshalAs(UnmanagedType.LPWStr)] StringBuilder displayName,
			ref int displayNameSize,
			AssemblyDisplayModes flags);

		[SuppressMessage("Microsoft.Design", "CA1045", MessageId = "0#")]
		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "7#")]
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult BindToObject(
			ref Guid refIID,
			[MarshalAs(UnmanagedType.IUnknown)] object unknownSink,
			[MarshalAs(UnmanagedType.IUnknown)] object unknownContext,
			[MarshalAs(UnmanagedType.LPWStr)] string codeBase,
			long flags,
			IntPtr reserved1,
			uint reserved2,
			out IntPtr obj);

		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "1#")]
		[SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference", MessageId = "0#")]
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult GetName(
			ref int nameSize,
			[Out][MarshalAs(UnmanagedType.LPWStr)] StringBuilder name);

		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "0#")]
		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "1#")]
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult GetVersion(
			out uint versionHigh,
			out uint versionLow);

		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult IsEqual(
			IAssemblyName assemblyName,
			AssemblyCompareOptions flags);

		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "0#")]
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult Clone(
			out IAssemblyName assemblyName);
	}

	[ComImport]
	[Guid("21b8916c-f28e-11d2-a473-00c04f8ef448")]
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[CLSCompliant(false)]
	public interface IAssemblyEnum
	{
		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "1#")]
		[PreserveSig()]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult GetNextAssembly(
			IntPtr reserved,
			out IAssemblyName assemblyName,
			uint flags);

		[PreserveSig()]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult Reset();

		[PreserveSig()]
		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "0#")]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult Clone(
			out IAssemblyEnum assemblyEnum);
	}

	[ComImport]
	[Guid("582dac66-e678-449f-aba6-6faaec8a9394")]
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[CLSCompliant(false)]
	public interface IInstallReferenceItem
	{
		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "0#")]
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult GetReference(
			[MarshalAs(UnmanagedType.LPArray)] out FusionInstallReference[] references,
			uint flags,
			IntPtr reserved);
	}

	[ComImport]
	[Guid("56b1a988-7c0c-4aa2-8639-c3eb5a90226f")]
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[CLSCompliant(false)]
	public interface IInstallReferenceEnum
	{
		[PreserveSig()]
		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "0#")]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult GetNextInstallReferenceItem(
			out IInstallReferenceItem installReferenceItem,
			uint flags,
			IntPtr reserved);
	}

	[ComImport]
	[Guid("9E3AAEB4-D1CD-11D2-BAB9-00C04F8ECEAE")]
	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[CLSCompliant(false)]
	public interface IAssemblyCacheItem
	{
		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "4#")]
		[SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference", MessageId = "5#")]
		void CreateStream(
			uint flags,
			[MarshalAs(UnmanagedType.LPWStr)] string streamName,
			uint format,
			uint formatFlags,
			out IStream stream,
			ref long maxSize);

		[SuppressMessage("Microsoft.Design", "CA1021", MessageId = "1#")]
		[SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference", MessageId = "5#")]
		void Commit(
			uint flags,
			out long disposition);

		void AbortItem();
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
