﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Diagnostics Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.ConstrainedExecution;
using System.Runtime.InteropServices;
using System.Text;

using Microsoft.Win32;

namespace Nabu.Diagnostics
{
	internal static class NativeMethods
	{
		#region Fusion

#if !COMPACT_FRAMEWORK
		[SuppressMessage("Microsoft.Design", "CA1021:AvoidOutParameters", MessageId = "0#")]
		[DllImport("fusion.dll", SetLastError = true, PreserveSig = false)]
		public static extern void CreateAssemblyCache(
			out IAssemblyCache assemblyCache,
			uint reserved);

		[SuppressMessage("Microsoft.Design", "CA1021:AvoidOutParameters", MessageId = "0#")]
		[DllImport("fusion.dll", SetLastError = true, CharSet = CharSet.Unicode, PreserveSig = false)]
		public static extern void CreateAssemblyNameObject(
			out IAssemblyName assemblyNameObject,
			string assemblyName,
			uint flags,
			IntPtr reserved);

		[SuppressMessage("Microsoft.Design", "CA1021:AvoidOutParameters", MessageId = "0#")]
		[DllImport("fusion.dll", SetLastError = true, PreserveSig = false)]
		public static extern void CreateAssemblyEnum(
			out IAssemblyEnum assemblyEnum,
			IntPtr reserved1,
			IAssemblyName assemblyName,
			AssemblyCacheModes flags,
			IntPtr reserved2);

		//	[SuppressMessage("Microsoft.Design", "CA1021:AvoidOutParameters", MessageId = "0#")]
		//	[DllImport("fusion.dll", SetLastError = true, PreserveSig = false)]
		//	public static extern void CreateInstallReferenceEnum(
		//		out IInstallReferenceEnum installReferenceEnum,
		//		IAssemblyName assemblyName,
		//		uint flags,
		//		IntPtr reserved);

		[DllImport("fusion.dll", SetLastError = true, CharSet = CharSet.Unicode, PreserveSig = false)]
		public static extern void GetCachePath(
			AssemblyCacheModes assemblyCacheFlags,
			[MarshalAs(UnmanagedType.LPWStr)] StringBuilder cachePath,
			ref int cachePathSize);
#endif

		#endregion

		#region Device Manager

		[DllImport("SetupAPI.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiBuildClassInfoList(
			[In] int flags,
			[Out] Guid[] guidList,
			[In] int guidListSize,
			[Out] out int requiredSize);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiClassNameFromGuidW(
			[In] ref Guid classGuid,
			[Out][MarshalAs(UnmanagedType.LPWStr)] StringBuilder className,
			[In] int classNameSize,
			[Out] out int requiredSize);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiGetClassDescriptionW(
			[In] ref Guid classGuid,
			[Out][MarshalAs(UnmanagedType.LPWStr)] StringBuilder classDescription,
			[In] int classDescriptionSize,
			[Out] out int requiredSize);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		public static extern IntPtr SetupDiGetClassDevsW(
			[In] ref Guid classGuid,
			[In][MarshalAs(UnmanagedType.LPWStr)] string enumerator,
			[In] IntPtr hWindow,
			[In] int flags);

		[DllImport("SetupAPI.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiDestroyDeviceInfoList(
			[In] IntPtr hInfoSet);

		[DllImport("SetupAPI.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiEnumDeviceInfo(
			[In] IntPtr hInfoSet,
			[In] int index,
			[Out] out DeviceInstanceInfo instanceInfo);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiGetDeviceRegistryPropertyW(
			[In] IntPtr hInfoSet,
			[In] ref DeviceInstanceInfo instanceInfo,
			[In][MarshalAs(UnmanagedType.U4)] DevicePropertyID propertyID,
			[Out][MarshalAs(UnmanagedType.U4)] out RegistryValueKind dataType,
			[Out] byte[] dataValue,
			[In] int dataSize,
			[Out] out int requiredSize);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiGetDeviceRegistryPropertyW(
			[In] IntPtr hInfoSet,
			[In] ref DeviceInstanceInfo instanceInfo,
			[In][MarshalAs(UnmanagedType.U4)] DevicePropertyID propertyID,
			[Out][MarshalAs(UnmanagedType.U4)] out RegistryValueKind dataType,
			[Out] out int dataValue,
			[In] int dataSize,
			[Out] out int requiredSize);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiGetDeviceRegistryPropertyW(
			[In] IntPtr hInfoSet,
			[In] ref DeviceInstanceInfo instanceInfo,
			[In][MarshalAs(UnmanagedType.U4)] DevicePropertyID propertyID,
			[Out][MarshalAs(UnmanagedType.U4)] out RegistryValueKind dataType,
			[Out][MarshalAs(UnmanagedType.LPWStr)] StringBuilder dataValue,
			[In] int dataSize,
			[Out] out int requiredSize);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiGetDeviceRegistryPropertyW(
			[In] IntPtr hInfoSet,
			[In] ref DeviceInstanceInfo instanceInfo,
			[In][MarshalAs(UnmanagedType.U4)] DevicePropertyID propertyID,
			[Out][MarshalAs(UnmanagedType.U4)] out RegistryValueKind dataType,
			[Out][MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 5, ArraySubType = UnmanagedType.U2)] char[] dataValue,
			[In] int dataSize,
			[Out] out int requiredSize);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiGetDeviceInstallParamsW(
			[In] IntPtr hInfoSet,
			[In] ref DeviceInstanceInfo instanceInfo,
			[In][Out] ref DeviceInstallParameters installParams);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiSetDeviceInstallParamsW(
			[In] IntPtr hInfoSet,
			[In] ref DeviceInstanceInfo instanceInfo,
			[In] ref DeviceInstallParameters installParams);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiBuildDriverInfoList(
			[In] IntPtr hInfoSet,
			[In][Out] ref DeviceInstanceInfo instanceInfo,
			[In][MarshalAs(UnmanagedType.U4)] DeviceDriverType driverType);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiEnumDriverInfoW(
			[In] IntPtr hInfoSet,
			[In] ref DeviceInstanceInfo instanceInfo,
			[In][MarshalAs(UnmanagedType.U4)] DeviceDriverType driverType,
			[In] int driverIndex,
			[In][Out] ref DeviceDriverInfo driverInfo);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiSetSelectedDriverW(
			[In] IntPtr hInfoSet,
			[In] ref DeviceInstanceInfo instanceInfo,
			[In][Out] ref DeviceDriverInfo driverInfo);

		[DllImport("SetupAPI.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		public static extern IntPtr SetupOpenFileQueue();

		[DllImport("SetupAPI.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		public static extern void SetupCloseFileQueue(
			[In] IntPtr hQueue);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupDiCallClassInstaller(
			[In][MarshalAs(UnmanagedType.U4)] DeviceDriverInstallFunction function,
			[In] IntPtr hInfoSet,
			[In] ref DeviceInstanceInfo instanceInfo);

		[DllImport("SetupAPI.dll", CharSet = CharSet.Unicode)]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool SetupScanFileQueueW(
			[In] IntPtr hQueue,
			[In][MarshalAs(UnmanagedType.U4)] DeviceDriverFileQueueFlags flags,
			[In] IntPtr hWindow,
			[In][MarshalAs(UnmanagedType.FunctionPtr)] DeviceDriverFileCallback callback,
			[In] IntPtr context,
			[Out] out int result);

		[DllImport("CfgMgr32.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.U4)]
		public static extern ConfigResult CM_Get_First_Log_Conf(
			[Out] out IntPtr hConfiguration,
			[In] int InstanceID,
			[In] int flags);

		[DllImport("CfgMgr32.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.U4)]
		public static extern ConfigResult CM_Get_Next_Log_Conf(
			[Out] out IntPtr hNextConfiguration,
			[In] IntPtr hConfiguration,
			[In] int flags);

		[DllImport("CfgMgr32.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.U4)]
		public static extern ConfigResult CM_Free_Log_Conf_Handle(
			[In] IntPtr hConfiguration);

		[DllImport("CfgMgr32.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.U4)]
		public static extern ConfigResult CM_Get_Next_Res_Des(
			[Out] out IntPtr hResource,
			[In] IntPtr hConfiguration,
			[In][MarshalAs(UnmanagedType.U4)] DeviceResourceType resourceTypeRequested,
			[Out][MarshalAs(UnmanagedType.U4)] out DeviceResourceType resourceType,
			[In] int flags);

		[DllImport("CfgMgr32.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.U4)]
		public static extern ConfigResult CM_Get_Res_Des_Data_Size(
			[Out] out int size,
			[In] IntPtr hResource,
			[In] int flags);

		[DllImport("CfgMgr32.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.U4)]
		public static extern ConfigResult CM_Get_Res_Des_Data(
			[In] IntPtr hResource,
			[In] IntPtr lpBuffer,
			[In] int size,
			[In] int flags);

		[DllImport("CfgMgr32.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.U4)]
		public static extern ConfigResult CM_Free_Res_Des_Handle(
			[In] IntPtr hResource);

		[DllImport("CfgMgr32.dll")]
		[ReliabilityContract(Consistency.WillNotCorruptState, Cer.None)]
		[return: MarshalAs(UnmanagedType.U4)]
		public static extern ConfigResult CM_Get_DevNode_Status(
			[Out][MarshalAs(UnmanagedType.U4)] out DeviceInstanceFlags status,
			[Out][MarshalAs(UnmanagedType.U4)] out DeviceInstanceProblem problemID,
			[In] int InstanceID,
			[In] int flags);

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
