﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Diagnostics Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.InteropServices;
using System.Security.Permissions;

namespace Nabu.Diagnostics
{
	#region Setup API

	#region Constants

	#region Return Codes

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	internal enum ConfigResult : int
	{
		SUCCESS /*                        */ = 0x00000000,
		DEFAULT /*                        */ = 0x00000001,
		OUT_OF_MEMORY /*                  */ = 0x00000002,
		INVALID_POINTER /*                */ = 0x00000003,
		INVALID_FLAG /*                   */ = 0x00000004,
		INVALID_DEVNODE /*                */ = 0x00000005,
		INVALID_RES_DES /*                */ = 0x00000006,
		INVALID_LOG_CONF /*               */ = 0x00000007,
		INVALID_ARBITRATOR /*             */ = 0x00000008,
		INVALID_NODELIST /*               */ = 0x00000009,
		DEVNODE_HAS_REQS /*               */ = 0x0000000A,
		INVALID_RESOURCEID /*             */ = 0x0000000B,
		DLVXD_NOT_FOUND /*                */ = 0x0000000C,
		NO_SUCH_DEVNODE /*                */ = 0x0000000D,
		NO_MORE_LOG_CONF /*               */ = 0x0000000E,
		NO_MORE_RES_DES /*                */ = 0x0000000F,
		ALREADY_SUCH_DEVNODE /*           */ = 0x00000010,
		INVALID_RANGE_LIST /*             */ = 0x00000011,
		INVALID_RANGE /*                  */ = 0x00000012,
		FAILURE /*                        */ = 0x00000013,
		NO_SUCH_LOGICAL_DEV /*            */ = 0x00000014,
		CREATE_BLOCKED /*                 */ = 0x00000015,
		NOT_SYSTEM_VM /*                  */ = 0x00000016,
		REMOVE_VETOED /*                  */ = 0x00000017,
		APM_VETOED /*                     */ = 0x00000018,
		INVALID_LOAD_TYPE /*              */ = 0x00000019,
		BUFFER_SMALL /*                   */ = 0x0000001A,
		NO_ARBITRATOR /*                  */ = 0x0000001B,
		NO_REGISTRY_HANDLE /*             */ = 0x0000001C,
		REGISTRY_ERROR /*                 */ = 0x0000001D,
		INVALID_DEVICE_ID /*              */ = 0x0000001E,
		INVALID_DATA /*                   */ = 0x0000001F,
		INVALID_API /*                    */ = 0x00000020,
		DEVLOADER_NOT_READY /*            */ = 0x00000021,
		NEED_RESTART /*                   */ = 0x00000022,
		NO_MORE_HW_PROFILES /*            */ = 0x00000023,
		DEVICE_NOT_THERE /*               */ = 0x00000024,
		NO_SUCH_VALUE /*                  */ = 0x00000025,
		WRONG_TYPE  /*                    */ = 0x00000026,
		INVALID_PRIORITY /*               */ = 0x00000027,
		NOT_DISABLEABLE /*                */ = 0x00000028,
		FREE_RESOURCES /*                 */ = 0x00000029,
		QUERY_VETOED /*                   */ = 0x0000002A,
		CANT_SHARE_IRQ /*                 */ = 0x0000002B,
		NO_DEPENDENT /*                   */ = 0x0000002C,
		SAME_RESOURCES /*                 */ = 0x0000002D,
		NO_SUCH_REGISTRY_KEY /*           */ = 0x0000002E,
		INVALID_MACHINENAME /*            */ = 0x0000002F,
		REMOTE_COMM_FAILURE /*            */ = 0x00000030,
		MACHINE_UNAVAILABLE /*            */ = 0x00000031,
		NO_CM_SERVICES /*                 */ = 0x00000032,
		ACCESS_DENIED /*                  */ = 0x00000033,
		CALL_NOT_IMPLEMENTED /*           */ = 0x00000034,
		INVALID_PROPERTY /*               */ = 0x00000035,
		DEVICE_INTERFACE_ACTIVE /*        */ = 0x00000036,
		NO_SUCH_DEVICE_INTERFACE /*       */ = 0x00000037,
		INVALID_REFERENCE_STRING /*       */ = 0x00000038,
		INVALID_CONFLICT_LIST /*          */ = 0x00000039,
		INVALID_INDEX /*                  */ = 0x0000003A,
		INVALID_STRUCTURE_SIZE /*         */ = 0x0000003B,
	}

	#endregion

	#region Info

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum DeviceCapabilityFlags : int
	{
		LOCKSUPPORTED /*       */ = 0x00000001,
		EJECTSUPPORTED /*      */ = 0x00000002,
		REMOVABLE /*           */ = 0x00000004,
		DOCKDEVICE /*          */ = 0x00000008,
		UNIQUEID /*            */ = 0x00000010,
		SILENTINSTALL /*       */ = 0x00000020,
		RAWDEVICEOK /*         */ = 0x00000040,
		SURPRISEREMOVALOK /*   */ = 0x00000080,
		HARDWAREDISABLED /*    */ = 0x00000100,
		NONDYNAMIC /*          */ = 0x00000200
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum DeviceConfigFlags : int
	{
		/// <summary>
		/// Set if disabled
		/// </summary>
		DISABLED /*             */ = 0x00000001,
		/// <summary>
		/// Set if a present hardware enum device deleted
		/// </summary>
		REMOVED /*              */ = 0x00000002,
		/// <summary>
		/// Set if the devnode was manually installed
		/// </summary>
		MANUAL_INSTALL /*       */ = 0x00000004,
		/// <summary>
		/// Set if skip the boot config
		/// </summary>
		IGNORE_BOOT_LC /*       */ = 0x00000008,
		/// <summary>
		/// Load this devnode when in net boot
		/// </summary>
		NET_BOOT /*             */ = 0x00000010,
		/// <summary>
		/// Redo install
		/// </summary>
		REINSTALL /*            */ = 0x00000020,
		/// <summary>
		/// Failed the install
		/// </summary>
		FAILEDINSTALL /*        */ = 0x00000040,
		/// <summary>
		/// Can't stop/remove a single child
		/// </summary>
		CANTSTOPACHILD /*       */ = 0x00000080,
		/// <summary>
		/// Can remove even if rom.
		/// </summary>
		OKREMOVEROM /*          */ = 0x00000100,
		/// <summary>
		/// Don't remove at exit.
		/// </summary>
		NOREMOVEEXIT /*         */ = 0x00000200,
		/// <summary>
		/// Complete install for devnode running 'raw'
		/// </summary>
		FINISH_INSTALL /*       */ = 0x00000400,
		/// <summary>
		/// This devnode requires a forced config
		/// </summary>
		NEEDS_FORCED_CONFIG /*  */ = 0x00000800,
		/// <summary>
		/// This is the remote boot network card
		/// </summary>
		NETBOOT_CARD /*         */ = 0x00001000,
		/// <summary>
		/// This device has a partial logconfig
		/// </summary>
		PARTIAL_LOG_CONF /*     */ = 0x00002000,
		/// <summary>
		/// Set if unsafe removals should be ignored
		/// </summary>
		SUPPRESS_SURPRISE /*    */ = 0x00004000,
		/// <summary>
		/// Set if hardware should be tested for logo failures
		/// </summary>
		VERIFY_HARDWARE /*      */ = 0x00008000,
		/// <summary>
		/// Show the finish install wizard pages for the installed device.
		/// </summary>
		FINISHINSTALL_UI /*     */ = 0x00010000,
		/// <summary>
		/// Call installer with DIF_FINISHINSTALL_ACTION in client context.
		/// </summary>
		FINISHINSTALL_ACTION /* */ = 0x00020000,
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Usage", "CA2217:DoNotMarkEnumsWithFlags")]
	[Flags]
	public enum DeviceInstanceFlags : int
	{
		/// <summary>
		/// Was enumerated by ROOT
		/// </summary>
		ROOT_ENUMERATED /*  */ = 0x00000001,
		/// <summary>
		/// Has Register_Device_Driver
		/// </summary>
		DRIVER_LOADED /*    */ = 0x00000002,
		/// <summary>
		/// Has Register_Enumerator
		/// </summary>
		ENUM_LOADED /*      */ = 0x00000004,
		/// <summary>
		/// Is currently configured
		/// </summary>
		STARTED /*          */ = 0x00000008,
		/// <summary>
		/// Manually installed
		/// </summary>
		MANUAL /*           */ = 0x00000010,
		/// <summary>
		/// May need reenumeration
		/// </summary>
		NEED_TO_ENUM /*     */ = 0x00000020,
		/// <summary>
		/// Has received a config
		/// </summary>
		NOT_FIRST_TIME /*   */ = 0x00000040,
		/// <summary>
		/// Enum generates hardware ID
		/// </summary>
		HARDWARE_ENUM /*    */ = 0x00000080,
		/// <summary>
		/// Lied about can reconfig once
		/// </summary>
		LIAR /*             */ = 0x00000100,
		/// <summary>
		/// Not CM_Create_DevInst lately
		/// </summary>
		HAS_MARK /*         */ = 0x00000200,
		/// <summary>
		/// Need device installer
		/// </summary>
		HAS_PROBLEM /*      */ = 0x00000400,
		/// <summary>
		/// Is filtered
		/// </summary>
		FILTERED /*         */ = 0x00000800,
		/// <summary>
		/// Has been moved
		/// </summary>
		MOVED /*            */ = 0x00001000,
		/// <summary>
		/// Can be disabled
		/// </summary>
		DISABLEABLE /*      */ = 0x00002000,
		/// <summary>
		/// Can be removed
		/// </summary>
		REMOVABLE /*        */ = 0x00004000,
		/// <summary>
		/// Has a private problem
		/// </summary>
		PRIVATE_PROBLEM /*  */ = 0x00008000,
		/// <summary>
		/// Multi function parent
		/// </summary>
		MF_PARENT /*        */ = 0x00010000,
		/// <summary>
		/// Multi function child
		/// </summary>
		MF_CHILD /*         */ = 0x00020000,
		/// <summary>
		/// DevInst is being removed
		/// </summary>
		WILL_BE_REMOVED /*  */ = 0x00040000,
		/// <summary>
		/// Has received a config enumerate
		/// </summary>
		NOT_FIRST_TIMEE /*  */ = 0x00080000,
		/// <summary>
		/// When child is stopped, free resources
		/// </summary>
		STOP_FREE_RES /*    */ = 0x00100000,
		/// <summary>
		/// Don't skip during rebalance
		/// </summary>
		REBAL_CANDIDATE /*  */ = 0x00200000,
		/// <summary>
		/// This devnode's log_confs do not have same resources
		/// </summary>
		BAD_PARTIAL /*      */ = 0x00400000,
		/// <summary>
		/// This devnode's is an NT enumerator
		/// </summary>
		NT_ENUMERATOR /*    */ = 0x00800000,
		/// <summary>
		/// This devnode's is an NT driver
		/// </summary>
		NT_DRIVER /*        */ = 0x01000000,
		/// <summary>
		/// Devnode need lock resume processing
		/// </summary>
		NEEDS_LOCKING /*    */ = 0x02000000,
		/// <summary>
		/// Devnode can be the wakeup device
		/// </summary>
		ARM_WAKEUP /*       */ = 0x04000000,
		/// <summary>
		/// APM aware enumerator
		/// </summary>
		APM_ENUMERATOR /*   */ = 0x08000000,
		/// <summary>
		/// APM aware driver
		/// </summary>
		APM_DRIVER /*       */ = 0x10000000,
		/// <summary>
		/// Silent install
		/// </summary>
		SILENT_INSTALL /*   */ = 0x20000000,
		/// <summary>
		/// No show in device manager
		/// </summary>
		NO_SHOW_IN_DM /*    */ = 0x40000000,
		/// <summary>
		/// Had a problem during preassignment of boot log conf
		/// </summary>
		BOOT_LOG_PROB /*    */ = (int)(0x80000000 - 0x100000000),
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum DeviceInstanceProblem : int
	{
		NO_PROBLEM /*                 */ = 0x00000000,
		/// <summary>
		/// No config for device
		/// </summary>
		NOT_CONFIGURED /*             */ = 0x00000001,
		/// <summary>
		/// Service load failed
		/// </summary>
		DEVLOADER_FAILED /*           */ = 0x00000002,
		/// <summary>
		/// Out of memory
		/// </summary>
		OUT_OF_MEMORY /*              */ = 0x00000003,
		/// <summary>
		/// 
		/// </summary>
		ENTRY_IS_WRONG_TYPE /*        */ = 0x00000004,
		/// <summary>
		/// 
		/// </summary>
		LACKED_ARBITRATOR /*          */ = 0x00000005,
		/// <summary>
		/// Boot config conflict
		/// </summary>
		BOOT_CONFIG_CONFLICT /*       */ = 0x00000006,
		/// <summary>
		/// 
		/// </summary>
		FAILED_FILTER /*              */ = 0x00000007,
		/// <summary>
		/// Devloader not found
		/// </summary>
		DEVLOADER_NOT_FOUND /*        */ = 0x00000008,
		/// <summary>
		/// Invalid ID
		/// </summary>
		INVALID_DATA /*               */ = 0x00000009,
		/// <summary>
		/// 
		/// </summary>
		FAILED_START /*               */ = 0x0000000A,
		/// <summary>
		/// 
		/// </summary>
		LIAR /*                       */ = 0x0000000B,
		/// <summary>
		/// Config conflict
		/// </summary>
		NORMAL_CONFLICT /*            */ = 0x0000000C,
		/// <summary>
		/// 
		/// </summary>
		NOT_VERIFIED /*               */ = 0x0000000D,
		/// <summary>
		/// Requires restart
		/// </summary>
		NEED_RESTART /*               */ = 0x0000000E,
		/// <summary>
		/// 
		/// </summary>
		REENUMERATION /*              */ = 0x0000000F,
		/// <summary>
		/// 
		/// </summary>
		PARTIAL_LOG_CONF /*           */ = 0x00000010,
		/// <summary>
		/// Unknown res type
		/// </summary>
		UNKNOWN_RESOURCE /*           */ = 0x00000011,
		/// <summary>
		/// 
		/// </summary>
		REINSTALL /*                  */ = 0x00000012,
		/// <summary>
		/// 
		/// </summary>
		REGISTRY /*                   */ = 0x00000013,
		/// <summary>
		/// WINDOWS 95 ONLY
		/// </summary>
		VXDLDR /*                     */ = 0x00000014,
		/// <summary>
		/// Devinst will remove
		/// </summary>
		WILL_BE_REMOVED /*            */ = 0x00000015,
		/// <summary>
		/// Devinst is disabled
		/// </summary>
		DISABLED /*                   */ = 0x00000016,
		/// <summary>
		/// Devloader not ready
		/// </summary>
		DEVLOADER_NOT_READY /*        */ = 0x00000017,
		/// <summary>
		/// Device doesn't exist
		/// </summary>
		DEVICE_NOT_THERE /*           */ = 0x00000018,
		/// <summary>
		/// 
		/// </summary>
		MOVED /*                      */ = 0x00000019,
		/// <summary>
		/// 
		/// </summary>
		TOO_EARLY /*                  */ = 0x0000001A,
		/// <summary>
		/// No valid log config
		/// </summary>
		NO_VALID_LOG_CONF /*          */ = 0x0000001B,
		/// <summary>
		/// Install failed
		/// </summary>
		FAILED_INSTALL /*             */ = 0x0000001C,
		/// <summary>
		/// Device disabled
		/// </summary>
		HARDWARE_DISABLED /*          */ = 0x0000001D,
		/// <summary>
		/// Can't share IRQ
		/// </summary>
		CANT_SHARE_IRQ /*             */ = 0x0000001E,
		/// <summary>
		/// Driver failed add
		/// </summary>
		FAILED_ADD /*                 */ = 0x0000001F,
		/// <summary>
		/// Service's Start = 4
		/// </summary>
		DISABLED_SERVICE /*           */ = 0x00000020,
		/// <summary>
		/// Resource translation failed
		/// </summary>
		TRANSLATION_FAILED /*         */ = 0x00000021,
		/// <summary>
		/// No soft config
		/// </summary>
		NO_SOFTCONFIG /*              */ = 0x00000022,
		/// <summary>
		/// Device missing in BIOS table
		/// </summary>
		BIOS_TABLE /*                 */ = 0x00000023,
		/// <summary>
		/// IRQ translator failed
		/// </summary>
		IRQ_TRANSLATION_FAILED /*     */ = 0x00000024,
		/// <summary>
		/// DriverEntry() failed.
		/// </summary>
		FAILED_DRIVER_ENTRY /*        */ = 0x00000025,
		/// <summary>
		/// Driver should have unloaded.
		/// </summary>
		DRIVER_FAILED_PRIOR_UNLOAD /* */ = 0x00000026,
		/// <summary>
		/// Driver load unsuccessful.
		/// </summary>
		DRIVER_FAILED_LOAD /*         */ = 0x00000027,
		/// <summary>
		/// Error accessing driver's service key
		/// </summary>
		DRIVER_SERVICE_KEY_INVALID /* */ = 0x00000028,
		/// <summary>
		/// Loaded legacy service created no devices
		/// </summary>
		LEGACY_SERVICE_NO_DEVICES /*  */ = 0x00000029,
		/// <summary>
		/// Two devices were discovered with the same name
		/// </summary>
		DUPLICATE_DEVICE /*           */ = 0x0000002A,
		/// <summary>
		/// The drivers set the device state to failed
		/// </summary>
		FAILED_POST_START /*          */ = 0x0000002B,
		/// <summary>
		/// This device was failed post start via usermode
		/// </summary>
		HALTED /*                     */ = 0x0000002C,
		/// <summary>
		/// The devinst currently exists only in the registry
		/// </summary>
		PHANTOM /*                    */ = 0x0000002D,
		/// <summary>
		/// The system is shutting down
		/// </summary>
		SYSTEM_SHUTDOWN /*            */ = 0x0000002E,
		/// <summary>
		/// The device is offline awaiting removal
		/// </summary>
		HELD_FOR_EJECT /*             */ = 0x0000002F,
		/// <summary>
		/// One or more drivers is blocked from loading
		/// </summary>
		DRIVER_BLOCKED /*             */ = 0x00000030,
		/// <summary>
		/// System hive has grown too large
		/// </summary>
		REGISTRY_TOO_LARGE /*         */ = 0x00000031,
		/// <summary>
		/// Failed to apply one or more registry properties  
		/// </summary>
		SETPROPERTIES_FAILED /*       */ = 0x00000032,
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum DeviceRemovalPolicy : int
	{
		UNKNOWN /*           */ = 0,
		NO_REMOVAL /*        */ = 1,
		ORDERLY_REMOVAL /*   */ = 2,
		SURPRISE_REMOVAL /*  */ = 3
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum DeviceInstallState : int
	{
		INSTALLED /*       */ = 0,
		NEEDS_REINSTALL /* */ = 1,
		FAILED_INSTALL /*  */ = 2,
		FINISH_INSTALL /*  */ = 3,
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum DeviceInterfaceType : int
	{
		Undefined /*         */ = -1,
		Internal /*          */ = 0,
		ISA /*               */ = 1,
		EISA /*              */ = 2,
		MicroChannel /*      */ = 3,
		TurboChannel /*      */ = 4,
		PCI /*               */ = 5,
		VME /*               */ = 6,
		NU /*                */ = 7,
		PCMCIA /*            */ = 8,
		C /*                 */ = 9,
		MPI /*               */ = 10,
		MPSA /*              */ = 11,
		ProcessorInternal /* */ = 12,
		InternalPower /*     */ = 13,
		PNPISA /*            */ = 14,
		PNPBus /*            */ = 15,
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum FileDeviceType : int
	{
		NONE /*                */ = 0x00000000,
		BEEP /*                */ = 0x00000001,
		CD_ROM /*              */ = 0x00000002,
		CD_ROM_FILE_SYSTEM /*  */ = 0x00000003,
		CONTROLLER /*          */ = 0x00000004,
		DATALINK /*            */ = 0x00000005,
		DFS /*                 */ = 0x00000006,
		DISK /*                */ = 0x00000007,
		DISK_FILE_SYSTEM /*    */ = 0x00000008,
		FILE_SYSTEM /*         */ = 0x00000009,
		INPORT_PORT /*         */ = 0x0000000A,
		KEYBOARD /*            */ = 0x0000000B,
		MAILSLOT /*            */ = 0x0000000C,
		MIDI_IN /*             */ = 0x0000000D,
		MIDI_OUT /*            */ = 0x0000000E,
		MOUSE /*               */ = 0x0000000F,
		MULTI_UNC_PROVIDER /*  */ = 0x00000010,
		NAMED_PIPE /*          */ = 0x00000011,
		NETWORK /*             */ = 0x00000012,
		NETWORK_BROWSER /*     */ = 0x00000013,
		NETWORK_FILE_SYSTEM /* */ = 0x00000014,
		NULL /*                */ = 0x00000015,
		PARALLEL_PORT /*       */ = 0x00000016,
		PHYSICAL_NETCARD /*    */ = 0x00000017,
		PRINTER /*             */ = 0x00000018,
		SCANNER /*             */ = 0x00000019,
		SERIAL_MOUSE_PORT /*   */ = 0x0000001A,
		SERIAL_PORT /*         */ = 0x0000001B,
		SCREEN /*              */ = 0x0000001C,
		SOUND /*               */ = 0x0000001D,
		STREAMS /*             */ = 0x0000001E,
		TAPE /*                */ = 0x0000001F,
		TAPE_FILE_SYSTEM /*    */ = 0x00000020,
		TRANSPORT /*           */ = 0x00000021,
		UNKNOWN /*             */ = 0x00000022,
		VIDEO /*               */ = 0x00000023,
		VIRTUAL_DISK /*        */ = 0x00000024,
		WAVE_IN /*             */ = 0x00000025,
		WAVE_OUT /*            */ = 0x00000026,
		PORT_8042 /*           */ = 0x00000027,
		NETWORK_REDIRECTOR /*  */ = 0x00000028,
		BATTERY /*             */ = 0x00000029,
		BUS_EXTENDER /*        */ = 0x0000002A,
		MODEM /*               */ = 0x0000002B,
		VDM /*                 */ = 0x0000002C,
		MASS_STORAGE /*        */ = 0x0000002D,
		SMB /*                 */ = 0x0000002E,
		KS /*                  */ = 0x0000002F,
		CHANGER /*             */ = 0x00000030,
		SMARTCARD /*           */ = 0x00000031,
		ACPI /*                */ = 0x00000032,
		DVD /*                 */ = 0x00000033,
		FULLSCREEN_VIDEO /*    */ = 0x00000034,
		DFS_FILE_SYSTEM /*     */ = 0x00000035,
		DFS_VOLUME /*          */ = 0x00000036,
		SERENUM /*             */ = 0x00000037,
		TERMSRV /*             */ = 0x00000038,
		KSEC /*                */ = 0x00000039,
		FIPS /*                */ = 0x0000003A,
		INFINIBAND /*          */ = 0x0000003B,
		VMBUS /*               */ = 0x0000003E,
		CRYPT_PROVIDER /*      */ = 0x0000003F,
		WPD /*                 */ = 0x00000040,
		BLUETOOTH /*           */ = 0x00000041
	}

	#endregion

	#region Properties

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum DevicePropertyID : int
	{
		Description /*                 */ = 0x00000000,
		HardwareIds /*                 */ = 0x00000001,
		CompatibleIds /*               */ = 0x00000002,
		Service /*                     */ = 0x00000004,
		Class /*                       */ = 0x00000007,
		ClassGuid /*                   */ = 0x00000008,
		Driver /*                      */ = 0x00000009,
		ConfigFlags /*                 */ = 0x0000000A,
		Manufacturer /*                */ = 0x0000000B,
		FriendlyName /*                */ = 0x0000000C,
		LocationInformation /*         */ = 0x0000000D,
		PhysicalDeviceObjectName /*    */ = 0x0000000E,
		Capabilities /*                */ = 0x0000000F,
		UINumber /*                    */ = 0x00000010,
		UpperFilters /*                */ = 0x00000011,
		LowerFilters /*                */ = 0x00000012,
		BusTypeGuid /*                 */ = 0x00000013,
		LegacyBusType /*               */ = 0x00000014,
		BusNumber /*                   */ = 0x00000015,
		EnumeratorName /*              */ = 0x00000016,
		Security /*                    */ = 0x00000017,
		SecuritySDS /*                 */ = 0x00000018,
		DeviceType /*                  */ = 0x00000019,
		Exclusive /*                   */ = 0x0000001A,
		Characteristics /*             */ = 0x0000001B,
		Address /*                     */ = 0x0000001C,
		UINumberDescFormat /*          */ = 0x0000001D,
		DevicePowerData /*             */ = 0x0000001E,
		RemovalPolicy /*               */ = 0x0000001F,
		RemovalPolicyHWDefault /*      */ = 0x00000020,
		RemovalPolicyOverride /*       */ = 0x00000021,
		InstallState /*                */ = 0x00000022,
		LocationPaths /*               */ = 0x00000023,
	}

	#endregion

	#region Resources

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	internal enum DeviceResourceType : int
	{
		All /*    */ = 0,
		Memory /* */ = 1,
		Port /*   */ = 2,
		DMA /*    */ = 3,
		IRQ /*    */ = 4
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum DeviceMemoryFlags : int
	{
		IsWritable /*      */ = 0x0001,
		Is32Bit /*         */ = 0x0002,
		IsPrefetchable /*  */ = 0x0004,
		IsReadable /*      */ = 0x0008,
		IsWriteCombined /* */ = 0x0010,
		IsCacheable /*     */ = 0x0020,
		IsDecodeWindow /*  */ = 0x0040,
		IsBar /*           */ = 0x0080
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum DevicePortFlags : int
	{
		IsIoPort /*         */ = 0x0001,
		IsDecode10Bit /*    */ = 0x0004,
		IsDecode12Bit /*    */ = 0x0008,
		IsDecode16Bit /*    */ = 0x0010,
		IsDecodePositive /* */ = 0x0020,
		IsDecodePassive /*  */ = 0x0040,
		IsDecodeWindow /*   */ = 0x0080,
		IsBar /*            */ = 0x0100
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum DeviceDmaFlags : int
	{
		Is16Bit /*     */ = 0x0001,
		Is32Bit /*     */ = 0x0002,
		IsBusMaster /* */ = 0x0004,
		IsTypeF /*     */ = 0x0018,
		IsTypeA /*     */ = 0x0008,
		IsTypeB /*     */ = 0x0010
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum DeviceIrqFlags : int
	{
		IsShared /*        */ = 0x0001,
		IsEdgeSensitive /* */ = 0x0002,
	}

	#endregion

	#region Install

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Usage", "CA2217:DoNotMarkEnumsWithFlags")]
	[Flags]
	public enum DeviceInstallFlags : int
	{
		/// <summary>
		/// Support Other... button
		/// </summary>
		SHOWOEM /*                */ = 0x00000001,
		/// <summary>
		/// Show compatibility list
		/// </summary>
		SHOWCOMPAT /*             */ = 0x00000002,
		/// <summary>
		/// Show class list
		/// </summary>
		SHOWCLASS /*              */ = 0x00000004,
		/// <summary>
		/// Don't create a new copy queue use caller-supplied FileQueue
		/// </summary>
		NOVCP /*                  */ = 0x00000008,
		/// <summary>
		/// Searched for compatible devices
		/// </summary>
		DIDCOMPAT /*              */ = 0x00000010,
		/// <summary>
		/// Searched for class devices
		/// </summary>
		DIDCLASS /*               */ = 0x00000020,
		/// <summary>
		/// No UI for resources if possible
		/// </summary>
		AUTOASSIGNRES /*          */ = 0x00000040,
		/// <summary>
		/// Restart required to take effect
		/// </summary>
		NEEDRESTART /*            */ = 0x00000080,
		/// <summary>
		/// Reboot required to take effect
		/// </summary>
		NEEDREBOOT /*             */ = 0x00000100,
		/// <summary>
		/// No Browse... in InsertDisk
		/// </summary>
		NOBROWSE /*               */ = 0x00000200,
		/// <summary>
		/// Set if multiple manufacturers in class driver list
		/// </summary>
		MULTMFGS /*               */ = 0x00000400,
		/// <summary>
		/// Set if device disabled
		/// </summary>
		DISABLED /*               */ = 0x00000800,
		/// <summary>
		/// 
		/// </summary>
		GENERALPAGE_ADDED /*      */ = 0x00001000,
		/// <summary>
		/// 
		/// </summary>
		RESOURCEPAGE_ADDED /*     */ = 0x00002000,
		/// <summary>
		/// Setting properties for this Device (or class) caused a change so the Dev Mgr UI probably needs to be updated.
		/// </summary>
		PROPERTIES_CHANGE /*      */ = 0x00004000,
		/// <summary>
		/// Sorting from the INF file should be used.
		/// </summary>
		INF_IS_SORTED /*          */ = 0x00008000,
		/// <summary>
		/// Only the the INF specified by SP_DEVINSTALL_PARAMS.DriverPath should be searched.
		/// </summary>
		ENUMSINGLEINF /*          */ = 0x00010000,
		/// <summary>
		/// Prevent ConfigMgr from removing/re-enumerating devices during device registration, installation, and deletion.
		/// </summary>
		DONOTCALLCONFIGMG /*      */ = 0x00020000,
		/// <summary>
		/// Install a device disabled
		/// </summary>
		INSTALLDISABLED /*        */ = 0x00040000,
		/// <summary>
		/// Cause SetupDiBuildDriverInfoList to build a device's compatible driver list from its existing class driver list, instead of the normal INF search.
		/// </summary>
		COMPAT_FROM_CLASS /*      */ = 0x00080000,
		/// <summary>
		/// Class Install params should be used.
		/// </summary>
		CLASSINSTALLPARAMS /*     */ = 0x00100000,
		/// <summary>
		/// Caller of DiCallClassInstaller does NOT want the internal default action performed if the Class installer returns ERROR_DI_DO_DEFAULT.
		/// </summary>
		NODI_DEFAULTACTION /*     */ = 0x00200000,
		/// <summary>
		/// Don't confuse the user with questions or excess info
		/// </summary>
		QUIETINSTALL /*           */ = 0x00800000,
		/// <summary>
		/// No file Copy necessary
		/// </summary>
		NOFILECOPY /*             */ = 0x01000000,
		/// <summary>
		/// Force files to be copied from install path
		/// </summary>
		FORCECOPY /*              */ = 0x02000000,
		/// <summary>
		/// Prop provider added Driver page.
		/// </summary>
		DRIVERPAGE_ADDED /*       */ = 0x04000000,
		/// <summary>
		/// Use Class Installer Provided strings in the Select Device Dlg
		/// </summary>
		USECI_SELECTSTRINGS /*    */ = 0x08000000,
		/// <summary>
		/// Override INF flags
		/// </summary>
		OVERRIDE_INFFLAGS /*      */ = 0x10000000,
		/// <summary>
		/// No Enable/Disable in General Props
		/// </summary>
		PROPS_NOCHANGEUSAGE /*    */ = 0x20000000,
		/// <summary>
		/// No small icons in select device dialogs
		/// </summary>
		NOSELECTICONS /*          */ = 0x40000000,
		/// <summary>
		/// Don't write HW & Compat IDs on install
		/// </summary>
		NOWRITE_IDS /*            */ = (int)(0x80000000 - 0x100000000)
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Usage", "CA2217:DoNotMarkEnumsWithFlags")]
	[Flags]
	public enum DeviceInstallFlagsEx : int
	{
		/// <summary>
		/// Failed to Load/Call class installer
		/// </summary>
		CI_FAILED /*                */ = 0x00000004,
		/// <summary>
		/// Class/co-installer wants to get a DIF_FINISH_INSTALL action in client context.
		/// </summary>
		FINISHINSTALL_ACTION /*     */ = 0x00000008,
		/// <summary>
		/// Did the Class Info List
		/// </summary>
		DIDINFOLIST /*              */ = 0x00000010,
		/// <summary>
		/// Did the Compat Info List
		/// </summary>
		DIDCOMPATINFO /*            */ = 0x00000020,
		/// <summary>
		/// 
		/// </summary>
		FILTERCLASSES /*            */ = 0x00000040,
		/// <summary>
		/// 
		/// </summary>
		SETFAILEDINSTALL /*         */ = 0x00000080,
		/// <summary>
		/// 
		/// </summary>
		DEVICECHANGE /*             */ = 0x00000100,
		/// <summary>
		/// 
		/// </summary>
		ALWAYSWRITEIDS /*           */ = 0x00000200,
		/// <summary>
		/// One or more device property sheets have had changes made to them, and need to have a DIF_PROPERTYCHANGE occur.
		/// </summary>
		PROPCHANGE_PENDING /*       */ = 0x00000400,
		/// <summary>
		/// 
		/// </summary>
		ALLOWEXCLUDEDDRVS /*        */ = 0x00000800,
		/// <summary>
		/// 
		/// </summary>
		NOUIONQUERYREMOVE /*        */ = 0x00001000,
		/// <summary>
		/// Use the device's class when building compat drv list (Ignored if DI_COMPAT_FROM_CLASS flag is specified.)
		/// </summary>
		USECLASSFORCOMPAT /*        */ = 0x00002000,
		/// <summary>
		/// Don't run AddReg and DelReg for device's software (driver) key.
		/// </summary>
		NO_DRVREG_MODIFY /*         */ = 0x00008000,
		/// <summary>
		/// Installation is occurring during initial system setup.
		/// </summary>
		IN_SYSTEM_SETUP /*          */ = 0x00010000,
		/// <summary>
		/// Driver came from Windows Update
		/// </summary>
		INET_DRIVER /*              */ = 0x00020000,
		/// <summary>
		/// Cause SetupDiBuildDriverInfoList to append a new driver list to an existing list.
		/// </summary>
		APPENDDRIVERLIST /*         */ = 0x00040000,
		/// <summary>
		/// 
		/// </summary>
		PREINSTALLBACKUP /*         */ = 0x00080000,
		/// <summary>
		/// 
		/// </summary>
		BACKUPONREPLACE /*          */ = 0x00100000,
		/// <summary>
		/// Build driver list from INF(s) retrieved from URL specified in SP_DEVINSTALL_PARAMS.DriverPath (empty string means Windows Update website)
		/// </summary>
		DRIVERLIST_FROM_URL /*      */ = 0x00200000,
		/// <summary>
		/// Don't include old Internet drivers when building a driver list. Ignored on Windows Vista and later.
		/// </summary>
		EXCLUDE_OLD_INET_DRIVERS /* */ = 0x00800000,
		/// <summary>
		/// Class installer added their own power page
		/// </summary>
		POWERPAGE_ADDED /*          */ = 0x01000000,
		/// <summary>
		/// Only include similar drivers in class list
		/// </summary>
		FILTERSIMILARDRIVERS /*     */ = 0x02000000,
		/// <summary>
		/// Only add the installed driver to the class or compat driver list. Used in calls to SetupDiBuildDriverInfoList
		/// </summary>
		INSTALLEDDRIVER /*          */ = 0x04000000,
		/// <summary>
		/// Don't remove identical driver nodes from the class list
		/// </summary>
		NO_CLASSLIST_NODE_MERGE /*  */ = 0x08000000,
		/// <summary>
		/// Build driver list based on alternate platform information specified in associated file queue
		/// </summary>
		ALTPLATFORM_DRVSEARCH /*    */ = 0x10000000,
		/// <summary>
		/// Only restart the device drivers are being installed on as opposed to restarting all devices using those drivers.
		/// </summary>
		RESTART_DEVICE_ONLY /*      */ = 0x20000000,
		/// <summary>
		/// Tell SetupDiBuildDriverInfoList to do a recursive search
		/// </summary>
		RECURSIVESEARCH /*          */ = 0x40000000,
		/// <summary>
		/// Tell SetupDiBuildDriverInfoList to do a "published INF" search
		/// </summary>
		SEARCH_PUBLISHED_INFS /*    */ = (int)(0x80000000 - 0x100000000),
	}

	public enum DeviceDriverType : int
	{
		Unknown,
		Class,
		Compatible
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum DeviceDriverInstallFunction : int
	{
		Unknown /*                       */ = 0x00000000,
		SelectDevice /*                  */ = 0x00000001,
		InstallDevice /*                 */ = 0x00000002,
		AssignResources /*               */ = 0x00000003,
		Properties /*                    */ = 0x00000004,
		Remove /*                        */ = 0x00000005,
		FirstTimeSetup /*                */ = 0x00000006,
		FoundDevice /*                   */ = 0x00000007,
		SelectClassDrivers /*            */ = 0x00000008,
		ValidateClassDrivers /*          */ = 0x00000009,
		InstallClassDrivers /*           */ = 0x0000000A,
		CalcDiskSpace /*                 */ = 0x0000000B,
		DestroyPrivateData /*            */ = 0x0000000C,
		ValidateDriver /*                */ = 0x0000000D,
		MoveDevice /*                    */ = 0x0000000E,
		Detect /*                        */ = 0x0000000F,
		InstallWizard /*                 */ = 0x00000010,
		DestroyWizardData /*             */ = 0x00000011,
		PropertyChange /*                */ = 0x00000012,
		EnableClass /*                   */ = 0x00000013,
		DetectVerify /*                  */ = 0x00000014,
		InstallDeviceFiles /*            */ = 0x00000015,
		Unremove /*                      */ = 0x00000016,
		SelectBestCompatibleDriver /*    */ = 0x00000017,
		AllowInstall /*                  */ = 0x00000018,
		RegisterDevice /*                */ = 0x00000019,
		NewDeviceWizardPreselect /*      */ = 0x0000001A,
		NewDeviceWizardSelect /*         */ = 0x0000001B,
		NewDeviceWizardPreanalyze /*     */ = 0x0000001C,
		NewDeviceWizardPostanalyze /*    */ = 0x0000001D,
		NewDeviceWizardFinishInstall /*  */ = 0x0000001E,
		Unused /*                        */ = 0x0000001F,
		InstallInterfaces /*             */ = 0x00000020,
		DetectCancel /*                  */ = 0x00000021,
		RegisterCoInstallers /*          */ = 0x00000022,
		AddPropertyPageAdvanced /*       */ = 0x00000023,
		AddPropertyPageBasic /*          */ = 0x00000024,
		Reserved /*                      */ = 0x00000025,
		Troubleshooter /*                */ = 0x00000026,
		PowerMessageWake /*              */ = 0x00000027
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum DeviceDriverFileQueueFlags : int
	{
		FilePresence /*  */ = 0x00000001,
		FileValidity /*  */ = 0x00000002,
		UseCallback /*   */ = 0x00000004,
		UseCallbackEx /* */ = 0x00000008,
		InformUser /*    */ = 0x00000010,
		PruneCopy /*     */ = 0x00000020
	}

	#endregion

	#endregion

	#region Types

	#region Info

	[StructLayout(LayoutKind.Sequential, Pack = 4)]
	internal struct DeviceInstanceInfo
	{
		public int Size;
		public Guid ClassGuid;
		public int InstanceID;
		public IntPtr Reserved;
	};

	#endregion

	#region Resoruces

	[StructLayout(LayoutKind.Sequential, Pack = 4)]
	internal struct DeviceMemoryRangeDescriptor
	{
		public long Align;
		public int Size;
		public long Min;
		public long Max;
		[MarshalAs(UnmanagedType.U4)]
		public DeviceMemoryFlags Flags;
		public int Reserved;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 4)]
	internal struct DeviceMemoryDescriptor
	{
		public int Count;
		public int Size;
		public long Min;
		public long Max;
		[MarshalAs(UnmanagedType.U4)]
		public DeviceMemoryFlags Flags;
		public int Reserved;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 4)]
	internal struct DevicePortRangeDescriptor
	{
		public long Align;
		public int Count;
		public long Min;
		public long Max;
		[MarshalAs(UnmanagedType.U4)]
		public DevicePortFlags Flags;
		public long Alias;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 4)]
	internal struct DevicePortDescriptor
	{
		public int Count;
		public int Size;
		public long Min;
		public long Max;
		[MarshalAs(UnmanagedType.U4)]
		public DevicePortFlags Flags;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 4)]
	internal struct DeviceDmaRangeDescriptor
	{
		public int Min;
		public int Max;
		[MarshalAs(UnmanagedType.U4)]
		public DeviceDmaFlags Flags;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 4)]
	internal struct DeviceDmaDescriptor
	{
		public int Count;
		public int Size;
		[MarshalAs(UnmanagedType.U4)]
		public DeviceDmaFlags Flags;
		public int Channel;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 4)]
	internal struct DeviceIrqRangeDescriptor
	{
		public int Min;
		public int Max;
		[MarshalAs(UnmanagedType.U4)]
		public DeviceIrqFlags Flags;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 4)]
	internal struct DeviceIrqDescriptor
	{
		public int Count;
		public int Size;
		[MarshalAs(UnmanagedType.U4)]
		public DeviceIrqFlags Flags;
		public int Number;
		public IntPtr CpuAffinity;
	}

	#endregion

	#region Install

	internal delegate int DeviceDriverFileCallback(
		[In] IntPtr context,
		[In] int notification,
		[In] IntPtr param1,
		[In] IntPtr param2);

	[StructLayout(LayoutKind.Sequential, Pack = 4, CharSet = CharSet.Unicode)]
	internal struct DeviceInstallParameters
	{
		public int Size;
		[MarshalAs(UnmanagedType.U4)]
		public DeviceInstallFlags Flags;
		[MarshalAs(UnmanagedType.U4)]
		public DeviceInstallFlagsEx FlagsEx;
		public IntPtr hWindow;
		[MarshalAs(UnmanagedType.FunctionPtr)]
		public DeviceDriverFileCallback InstallHandler;
		public IntPtr InstallContext;
		public IntPtr hFileQueue;
		public IntPtr Reserved1;
		public IntPtr Reserved2;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
		public string DriverPath;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 4, CharSet = CharSet.Unicode)]
	internal struct DeviceDriverInfo
	{
		public int Size;
		[MarshalAs(UnmanagedType.U4)]
		public DeviceDriverType DriverType;
		public IntPtr Reserved;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
		public string Description;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
		public string Manufacturer;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
		public string Provider;
		public long ReleaseDate;
		public ulong Version;
	}

	#endregion

	#endregion

	#endregion

	#region Task Scheduler

	#region Constants

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum TaskFlags : int
	{
		Interactive /*              */ = 0x0001,
		DeleteWhenDone /*           */ = 0x0002,
		Disabled /*                 */ = 0x0004,
		StartOnlyIfIdle /*          */ = 0x0010,
		KillOnIdleEnd /*            */ = 0x0020,
		DontStartIfOnBatteries /*   */ = 0x0040,
		KillIfGoingOnBatteries /*   */ = 0x0080,
		RunOnlyIfDocked /*          */ = 0x0100,
		Hidden /*                   */ = 0x0200,
		RunIfConnectedToInternet /* */ = 0x0400,
		RestartOnIdleResume /*      */ = 0x0800,
		SystemRequired /*           */ = 0x1000,
		RunOnlyIfLoggedOn /*        */ = 0x2000
	}

	internal enum TaskTriggerType
	{
		TimeTriggerOnce,
		TimeTriggerDaily,
		TimeTriggerWeekly,
		TimeTriggerMonthlyByDate,
		TimeTriggerMonthlyByDayOfWeek,
		EventTriggerSystemIdle,
		EventTriggerSystemStart,
		EventTriggerUserLogon
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum TaskDaysOfTheWeek : int
	{
		Sunday /*    */ = 1,
		Monday /*    */ = 2,
		Tuesday /*   */ = 4,
		Wednesday /* */ = 8,
		Thursday /*  */ = 16,
		Saturday /*  */ = 64,
		Friday /*    */ = 32,
	}

	public enum TaskWeekOfMonth : int
	{
		None,
		FirstWeek,
		SecondWeek,
		ThirdWeek,
		FourthWeek,
		LastWeek
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum TaskMonthsOfTheYear : int
	{
		January /*  */ = 0x0001,
		February /* */ = 0x0002,
		March /*    */ = 0x0004,
		April /*    */ = 0x0008,
		May /*      */ = 0x0010,
		June /*     */ = 0x0020,
		July /*     */ = 0x0040,
		August /*   */ = 0x0080,
		September /**/ = 0x0100,
		October /*  */ = 0x0200,
		November /* */ = 0x0400,
		December /* */ = 0x0800,
	}

	#endregion

	#region Types

	[StructLayout(LayoutKind.Sequential)]
	internal struct SystemTime
	{
		public ushort Year;
		public ushort Month;
		public ushort DayOfWeek;
		public ushort Day;
		public ushort Hour;
		public ushort Minute;
		public ushort Second;
		public ushort Milliseconds;
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct TriggerDailyDescriptor
	{
		public ushort Interval;
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct TriggerMonthlyByDateDescriptor
	{
		public uint Days;
		[MarshalAs(UnmanagedType.U2)]
		public TaskMonthsOfTheYear Months;
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct TriggerMonthlyByDayOfWeekDescriptor
	{
		[MarshalAs(UnmanagedType.U2)]
		public TaskWeekOfMonth Week;
		[MarshalAs(UnmanagedType.U2)]
		public TaskDaysOfTheWeek DaysOfTheWeek;
		public ushort Months;
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct TriggerWeeklyDescriptor
	{
		public ushort Interval;
		[MarshalAs(UnmanagedType.U2)]
		public TaskDaysOfTheWeek DaysOfTheWeek;
	}

	[StructLayout(LayoutKind.Explicit)]
	internal struct TriggerTypeData
	{
		// Fields
		[FieldOffset(0)]
		public TriggerDailyDescriptor Daily;
		[FieldOffset(0)]
		public TriggerMonthlyByDateDescriptor MonthlyByDate;
		[FieldOffset(0)]
		public TriggerMonthlyByDayOfWeekDescriptor MonthlyByDayOfWeek;
		[FieldOffset(0)]
		public TriggerWeeklyDescriptor Weekly;
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct TaskTriggerDescriptor
	{
		public ushort TriggerSize;
		public ushort Reserved1;
		public ushort BeginYear;
		public ushort BeginMonth;
		public ushort BeginDay;
		public ushort EndYear;
		public ushort EndMonth;
		public ushort EndDay;
		public ushort StartHour;
		public ushort StartMinute;
		public uint MinutesDuration;
		public uint MinutesInterval;
		public uint Flags;
		public TaskTriggerType Type;
		public TriggerTypeData Data;
		public ushort Reserved2;
		public ushort RandomMinutesInterval;
	}

	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("148BD528-A2AB-11CE-B11F-00AA00530503")]
	internal interface IEnumWorkItems
	{
		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult Next(
			[In] int requestCount,
			[Out] out IntPtr lpNames,
			[Out] out int fetched);

		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult Skip(
			[In] int Count);

		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult Reset();

		[PreserveSig]
		[return: MarshalAs(UnmanagedType.Error)]
		ComResult Clone(
			[Out][MarshalAs(UnmanagedType.Interface)] out IEnumWorkItems enumWorkItems);
	}

	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("4086658a-cbbb-11cf-b604-00c04fd8d565")]
	internal interface IProvideTaskPage
	{
		void GetPage(
			[In] int pageType,
			[In][MarshalAs(UnmanagedType.Bool)] bool persistChanges,
			[Out] out IntPtr lphPage);
	}

	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("148BD52B-A2AB-11CE-B11F-00AA00530503")]
	internal interface ITaskTrigger
	{
		void SetTrigger(
			[In][MarshalAs(UnmanagedType.Struct)] ref TaskTriggerDescriptor trigger);

		void GetTrigger(
			[Out][MarshalAs(UnmanagedType.Struct)] out TaskTriggerDescriptor trigger);

		void GetTriggerString(
			out IntPtr lpDescription);
	}

	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("148BD524-A2AB-11CE-B11F-00AA00530503")]
	internal interface ITask
	{
		void CreateTrigger(
			[Out] out ushort index,
			[Out][MarshalAs(UnmanagedType.Interface)] out ITaskTrigger trigger);

		void DeleteTrigger(
			[In] ushort index);

		void GetTriggerCount(
			out ushort count);

		void GetTrigger(
			[In] ushort index,
			[Out][MarshalAs(UnmanagedType.Interface)] out ITaskTrigger trigger);

		void GetTriggerString(
			[In] ushort index,
			out IntPtr description);

		void GetRunTimes(
			[In][MarshalAs(UnmanagedType.Struct)] ref SystemTime timeBegin,
			[In][MarshalAs(UnmanagedType.Struct)] ref SystemTime timeEnd,
			[In][Out] ref ushort count,
			[In][Out][MarshalAs(UnmanagedType.LPArray)] SystemTime[] times);

		void GetNextRunTime(
			[Out][MarshalAs(UnmanagedType.Struct)] out SystemTime time);

		void SetIdleWait(
			[In] ushort IdleMinutes,
			[In] ushort deadlineMinutes);

		void GetIdleWait(
			[Out] out ushort idleMinutes,
			[Out] out ushort deadlineMinutes);

		void Run();

		void Terminate();

		void EditWorkItem(
			[In] IntPtr hWindowParent,
			[In] int reserved);

		void GetMostRecentRunTime(
			[Out][MarshalAs(UnmanagedType.Struct)] out SystemTime time);

		void GetStatus(
			[Out][MarshalAs(UnmanagedType.Error)] out ComResult status);

		void GetExitCode(
			[Out] out int exitCode);

		void SetComment(
			[In][MarshalAs(UnmanagedType.LPWStr)] string comment);

		void GetComment(
			[Out] out IntPtr lpComment);

		void SetCreator(
			[In][MarshalAs(UnmanagedType.LPWStr)] string creator);

		void GetCreator(
			[Out] out IntPtr lpCreator);

		void SetWorkItemData(
			[In] ushort dataLength,
			[In][MarshalAs(UnmanagedType.LPArray, ArraySubType = UnmanagedType.U1, SizeParamIndex = 0)] byte[] data);

		void GetWorkItemData(
			[Out] out ushort dataLength,
			[Out] out IntPtr lpData);

		void SetErrorRetryCount(
			[In] ushort retryCount);

		void GetErrorRetryCount(
			[Out] out ushort retryCount);

		void SetErrorRetryInterval(
			[In] ushort retryInterval);

		void GetErrorRetryInterval(
			[Out] out ushort retryInterval);

		void SetFlags(
			[In][MarshalAs(UnmanagedType.U4)] TaskFlags flags);

		void GetFlags(
			[Out][MarshalAs(UnmanagedType.U4)] out TaskFlags flags);

		void SetAccountInformation(
			[In][MarshalAs(UnmanagedType.LPWStr)] string accountName,
			[In][MarshalAs(UnmanagedType.LPWStr)] string password);

		void GetAccountInformation(
			[Out] out IntPtr lpAccountName);

		void SetApplicationName(
			[In][MarshalAs(UnmanagedType.LPWStr)] string applicationName);

		void GetApplicationName(
			[Out] out IntPtr lpApplicationName);

		void SetParameters(
			[In][MarshalAs(UnmanagedType.LPWStr)] string parameters);

		void GetParameters(
			[Out] out IntPtr lpParameters);

		void SetWorkingDirectory(
			[In][MarshalAs(UnmanagedType.LPWStr)] string workingDirectory);

		void GetWorkingDirectory(
			[Out] out IntPtr lpWorkingDirectory);

		void SetPriority(
			[In] int priority);

		void GetPriority(
			[Out] out int priority);

		void SetTaskFlags(
			[In] uint flags);

		void GetTaskFlags(
			[Out] out uint flags);

		void SetMaxRunTime(
			[In] uint time);

		void GetMaxRunTime(
			[Out] out uint time);
	}

	[InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
	[Guid("148BD527-A2AB-11CE-B11F-00AA00530503")]
	internal interface ITaskScheduler
	{
		void SetTargetComputer(
			[In][MarshalAs(UnmanagedType.LPWStr)] string computer);

		void GetTargetComputer(
			out IntPtr lpComputer);

		void Enum(
			[MarshalAs(UnmanagedType.Interface)] out IEnumWorkItems enumWorkItems);

		void Activate(
			[In][MarshalAs(UnmanagedType.LPWStr)] string name,
			[In] ref Guid riid,
			[Out][MarshalAs(UnmanagedType.IUnknown)] out object obj);

		void Delete(
			[In][MarshalAs(UnmanagedType.LPWStr)] string name);

		void NewWorkItem(
			[In][MarshalAs(UnmanagedType.LPWStr)] string name,
			[In] ref Guid clsid,
			[In] ref Guid riid,
			[MarshalAs(UnmanagedType.IUnknown)] out object obj);

		void AddWorkItem(
			[In][MarshalAs(UnmanagedType.LPWStr)] string name,
			[In][MarshalAs(UnmanagedType.Interface)] ITask workItem);

		void IsOfType(
			[In][MarshalAs(UnmanagedType.LPWStr)] string name,
			[In] ref Guid riid);
	}

	[ComImport]
	[Guid("148BD520-A2AB-11CE-B11F-00AA00530503")]
	internal class CoTask
	{
	}

	[ComImport]
	[Guid("148BD52A-A2AB-11CE-B11F-00AA00530503")]
	internal class CoTaskScheduler
	{
	}

	internal static class CoTaskMemory
	{
		[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.UnmanagedCode)]
		public static string PtrToStringAndFree(IntPtr address)
		{
			string value = Marshal.PtrToStringUni(address);

			Marshal.FreeCoTaskMem(address);

			return value;
		}
	}

	#endregion

	#endregion
}
/******************************************************************************/
/*                              END OF FILE                                   */
/******************************************************************************/
