﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Diagnostics Library                      */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Security.Permissions;

namespace Nabu.Diagnostics
{
	public sealed class TaskScheduler : DisposableBase
	{
		#region TaskScheduler

		#region Fields

		private static Guid ITaskGuid = Marshal.GenerateGuidForType(typeof(ITask));
		private static Guid CoTaskGuid = Marshal.GenerateGuidForType(typeof(CoTask));

		private CoTaskScheduler _taskSchedulerService;
		private ITaskScheduler _taskScheduler;

		#endregion

		#region Methods

		public TaskScheduler()
		{
			this._taskSchedulerService = new CoTaskScheduler();
			this._taskScheduler = this._taskSchedulerService as ITaskScheduler;
		}

		[CLSCompliant(false)]
		public ScheduledTask CreateTask(string name)
		{
			base.CheckDisposed();

			object scheduledTask;

			this._taskScheduler.NewWorkItem(name, ref CoTaskGuid, ref ITaskGuid, out scheduledTask);

			return new ScheduledTask((ITask)scheduledTask, name);
		}

		[CLSCompliant(false)]
		public ScheduledTask OpenTask(string name)
		{
			base.CheckDisposed();

			object scheduledTask;

			this._taskScheduler.Activate(name, ref ITaskGuid, out scheduledTask);

			return new ScheduledTask((ITask)scheduledTask, name);
		}

		public void DeleteTask(string name)
		{
			base.CheckDisposed();

			this._taskScheduler.Delete(name);
		}

		#endregion

		#region Properties

		public string TargetComputer
		{
			get
			{
				base.CheckDisposed();

				IntPtr address;

				this._taskScheduler.GetTargetComputer(out address);

				return CoTaskMemory.PtrToStringAndFree(address);
			}

			set
			{
				base.CheckDisposed();

				this._taskScheduler.SetTargetComputer(value);
			}
		}

		public ICollection<string> TaskNames
		{
			[SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.UnmanagedCode)]
			get
			{
				base.CheckDisposed();

				ICollection<string> taskNames = new List<string>();
				IEnumWorkItems enumItems;
				IntPtr lpNames;
				int fetched;
				ComResult result;

				this._taskScheduler.Enum(out enumItems);

				do
				{
					result = enumItems.Next(1000, out lpNames, out fetched);

					if (NativeResult.ComSucceeded(result))
					{
						for (int index = 0; index < fetched; index++)
						{
							IntPtr lpName = Marshal.ReadIntPtr(lpNames, index * IntPtr.Size);

							taskNames.Add(CoTaskMemory.PtrToStringAndFree(lpName));
						}
					}

					Marshal.FreeCoTaskMem(lpNames);
				} while (result == ComResult.S_OK);

				Marshal.ReleaseComObject(enumItems);

				return taskNames;
			}
		}

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			Marshal.ReleaseComObject(this._taskSchedulerService);
			this._taskSchedulerService = null;

			base.Dispose(disposing);
		}

		#endregion
	}
}
/******************************************************************************/
/*                              END OF FILE                                   */
/******************************************************************************/
