﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Drawing Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;

namespace Nabu.Drawing
{
#if !COMPACT_FRAMEWORK
	[System.Diagnostics.DebuggerDisplay("H: {H}; S: {S}; L: {L};")]
#endif
	public struct ColorHSL
	{
		#region ColorHSL

		#region Constants

		public const float HMin = 0.000F;
		public const float HMax = 1.000F;
		public const float SMin = 0.000F;
		public const float SMax = 1.000F;
		public const float LMin = 0.000F;
		public const float LMax = 1.000F;

		private const string ComponentExceptionMessage = "Color component must belong to range 0.000 - 1.000";

		#endregion

		#region Fields

		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public float H;
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public float S;
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public float L;

		#endregion

		#region Methods

		public ColorHSL(float h, float s, float l)
		{
			if ((h < HMin) || (h > HMax))
			{
				throw new ArgumentOutOfRangeException("h", ComponentExceptionMessage);
			}

			if ((s < SMin) || (s > SMax))
			{
				throw new ArgumentOutOfRangeException("s", ComponentExceptionMessage);
			}

			if ((l < LMin) || (l > LMax))
			{
				throw new ArgumentOutOfRangeException("l", ComponentExceptionMessage);
			}

			this.H = h;
			this.S = s;
			this.L = l;
		}

		#endregion

		#endregion

		#region Object

		public override string ToString()
		{
			return
				string.Format(
					CultureInfo.InvariantCulture,
					"H: {0:0.0}; S: {1:0.0}; L: {2:0.0};",
					this.H,
					this.S,
					this.L);
		}

		public override bool Equals(object obj)
		{
			ColorHSL color = (ColorHSL)obj;

			return (this.H == color.H) && (this.S == color.S) && (this.L == color.L);
		}

		public override int GetHashCode()
		{
			return (int)(16777216 * this.H + 65536 * this.S + 256 * this.L);
		}

		public static bool operator ==(ColorHSL left, ColorHSL right)
		{
			return (left.H == right.H) && (left.S == right.S) && (left.L == right.L);
		}

		public static bool operator !=(ColorHSL left, ColorHSL right)
		{
			return (left.H != right.H) || (left.S != right.S) || (left.L != right.L);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
