﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Drawing Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;

namespace Nabu.Drawing
{
#if !COMPACT_FRAMEWORK
	[System.Diagnostics.DebuggerDisplay("H: {H}; S: {S}; V: {V};")]
#endif
	public struct ColorHSV
	{
		#region ColorHSV

		#region Constants

		public const float HMin = 0.000F;
		public const float HMax = 1.000F;
		public const float SMin = 0.000F;
		public const float SMax = 1.000F;
		public const float VMin = 0.000F;
		public const float VMax = 1.000F;

		private const string ComponentExceptionMessage = "Color component must belong to range 0.000 - 1.000";

		#endregion

		#region Fields

		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public float H;
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public float S;
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public float V;

		#endregion

		#region Methods

		public ColorHSV(float h, float s, float v)
		{
			if ((h < HMin) || (h > HMax))
			{
				throw new ArgumentOutOfRangeException("h", ComponentExceptionMessage);
			}

			if ((s < SMin) || (s > SMax))
			{
				throw new ArgumentOutOfRangeException("s", ComponentExceptionMessage);
			}

			if ((v < VMin) || (v > VMax))
			{
				throw new ArgumentOutOfRangeException("v", ComponentExceptionMessage);
			}

			this.H = h;
			this.S = s;
			this.V = v;
		}

		#endregion

		#endregion

		#region Object

		public override string ToString()
		{
			return
				string.Format(
					CultureInfo.InvariantCulture,
					"H: {0:0.0}; S: {1:0.0}; V: {2:0.0};",
					this.H,
					this.S,
					this.V);
		}

		public override bool Equals(object obj)
		{
			ColorHSV color = (ColorHSV)obj;

			return (this.H == color.H) && (this.S == color.S) && (this.V == color.V);
		}

		public override int GetHashCode()
		{
			return (int)(16777216 * this.H + 65536 * this.S + 256 * this.V);
		}

		public static bool operator ==(ColorHSV left, ColorHSV right)
		{
			return (left.H == right.H) && (left.S == right.S) && (left.V == right.V);
		}

		public static bool operator !=(ColorHSV left, ColorHSV right)
		{
			return (left.H != right.H) || (left.S != right.S) || (left.V != right.V);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
