﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Drawing Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Drawing;
using System.Globalization;

namespace Nabu.Drawing
{
#if !COMPACT_FRAMEWORK
	[System.Diagnostics.DebuggerDisplay("R: {R}; G: {G}; B: {B}")]
#endif
	public struct ColorRGB
	{
		#region ColorRGB

		#region Constants

		public const float RMin = 0.000F;
		public const float RMax = 1.000F;
		public const float GMin = 0.000F;
		public const float GMax = 1.000F;
		public const float BMin = 0.000F;
		public const float BMax = 1.000F;

		private const string ComponentExceptionMessage = "Color component must belong to range 0.000 - 1.000";

		#endregion

		#region Fields

		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public float R;
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public float G;
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public float B;

		#endregion

		#region Methods

		public ColorRGB(float r, float g, float b)
		{
			if ((r < RMin) || (r > RMax))
			{
				throw new ArgumentOutOfRangeException("r", ComponentExceptionMessage);
			}

			if ((g < GMin) || (g > GMax))
			{
				throw new ArgumentOutOfRangeException("g", ComponentExceptionMessage);
			}

			if ((b < BMin) || (b > BMax))
			{
				throw new ArgumentOutOfRangeException("b", ComponentExceptionMessage);
			}

			this.R = r;
			this.G = g;
			this.B = b;
		}

		public Color ToColor()
		{
			return Color.FromArgb(
				unchecked((int)0xFF000000) |
				((int)Math.Floor(255.0F * R + 0.5F) << 16) |
				((int)Math.Floor(255.0F * G + 0.5F) << 8) |
				((int)Math.Floor(255.0F * B + 0.5F)));
		}

		public static ColorRGB FromColor(Color color)
		{
			return new ColorRGB(color.R / 255.0F, color.G / 255.0F, color.B / 255.0F);
		}

		#endregion

		#endregion

		#region Object

		public override string ToString()
		{
			return
				string.Format(
					CultureInfo.InvariantCulture,
					"R: {0:0.0}; G: {1:0.0}; B: {2:0.0};",
					this.R,
					this.G,
					this.B);
		}

		public override bool Equals(object obj)
		{
			ColorRGB color = (ColorRGB)obj;

			return (this.R == color.R) && (this.G == color.G) && (this.B == color.B);
		}

		public override int GetHashCode()
		{
			return (int)(16777216 * this.R + 65536 * this.G + 256 * this.B);
		}

		public static bool operator ==(ColorRGB left, Color right)
		{
			return (left.R == right.R) && (left.G == right.G) && (left.B == right.B);
		}

		public static bool operator !=(ColorRGB left, Color right)
		{
			return (left.R != right.R) || (left.G != right.G) || (left.B != right.B);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
