﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Drawing Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;

namespace Nabu.Drawing
{
#if !COMPACT_FRAMEWORK
	[System.Diagnostics.DebuggerDisplay("Y: {Y}; U: {U}; V: {V}")]
#endif
	public struct ColorYUV
	{
		#region ColorYUV

		#region Constants

		public const float YMin = 0.000F;
		public const float YMax = 1.000F;
		public const float UMin = -0.436F;
		public const float UMax = +0.436F;
		public const float VMin = -0.615F;
		public const float VMax = +0.615F;

		private const string ComponentYExceptionMessage = "Color component must belong to range +0.000 - +1.000";
		private const string ComponentUExceptionMessage = "Color component must belong to range -0.436 - +0.436";
		private const string ComponentVExceptionMessage = "Color component must belong to range -0.615 - +0.615";

		#endregion

		#region Fields

		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public float Y;
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public float U;
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public float V;

		#endregion

		#region Methods

		public ColorYUV(float y, float u, float v)
		{
			if ((y < YMin) || (y > YMax))
			{
				throw new ArgumentOutOfRangeException("y", ComponentYExceptionMessage);
			}

			if ((u < UMin) || (u > UMax))
			{
				throw new ArgumentOutOfRangeException("u", ComponentUExceptionMessage);
			}

			if ((v < VMin) || (v > VMax))
			{
				throw new ArgumentOutOfRangeException("v", ComponentVExceptionMessage);
			}

			this.Y = y;
			this.U = u;
			this.V = v;
		}

		#endregion

		#endregion

		#region Object

		public override string ToString()
		{
			return
				string.Format(
					CultureInfo.InvariantCulture,
					"Y: {0:0.0}; U: {1:0.0}; V: {2:0.0};",
					this.Y,
					this.U,
					this.V);
		}

		public override bool Equals(object obj)
		{
			ColorYUV color = (ColorYUV)obj;

			return (this.Y == color.Y) && (this.U == color.U) && (this.V == color.V);
		}

		public override int GetHashCode()
		{
			return (int)(16777216 * this.Y + 65536 * this.U + 256 * this.V);
		}

		public static bool operator ==(ColorYUV left, ColorYUV right)
		{
			return (left.Y == right.Y) && (left.U == right.U) && (left.V == right.V);
		}

		public static bool operator !=(ColorYUV left, ColorYUV right)
		{
			return (left.Y != right.Y) || (left.U != right.U) || (left.V != right.V);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
