﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Globalization;

namespace Nabu.Forms.Html
{
	[HtmlBehaviorName("accesskeys")]
	public sealed class HtmlAccessKeysBehavior : HtmlBehaviorBase
	{
		#region HtmlAccessKeysBehavior

		#region Fields

		private string[] _keyShortNames = new string[8];
		private string[] _keyLongNames = new string[8];

		#endregion

		#region Methods

		#region Utility

		private static string GetKeyName(int keyCode)
		{
			string keyName = null;

			switch (keyCode)
			{
				case 8: keyName = "Backspace"; break;
				case 9: keyName = "Tab"; break;
				case 13: keyName = "Enter"; break;
				case 27: keyName = "Escape"; break;
				case 32: keyName = "Space"; break;
				case 33: keyName = "PageUp"; break;
				case 34: keyName = "PageDown"; break;
				case 35: keyName = "End"; break;
				case 36: keyName = "Home"; break;
				case 37: keyName = "Left"; break;
				case 38: keyName = "Up"; break;
				case 39: keyName = "Right"; break;
				case 40: keyName = "Down"; break;
				case 45: keyName = "Insert"; break;
				case 46: keyName = "Delete"; break;
				case 96: keyName = "NumPad0"; break;
				case 97: keyName = "NumPad1"; break;
				case 98: keyName = "NumPad2"; break;
				case 99: keyName = "NumPad3"; break;
				case 100: keyName = "NumPad4"; break;
				case 101: keyName = "NumPad5"; break;
				case 102: keyName = "NumPad6"; break;
				case 103: keyName = "NumPad7"; break;
				case 104: keyName = "NumPad8"; break;
				case 105: keyName = "NumPad9"; break;
				case 106: keyName = "NumPadMul"; break;
				case 107: keyName = "NumPadAdd"; break;
				case 109: keyName = "NumPadSub"; break;
				case 110: keyName = "NumPadDecimal"; break;
				case 111: keyName = "NumPadDiv"; break;
				case 112: keyName = "F1"; break;
				case 113: keyName = "F2"; break;
				case 114: keyName = "F3"; break;
				case 115: keyName = "F4"; break;
				case 116: keyName = "F5"; break;
				case 117: keyName = "F6"; break;
				case 118: keyName = "F7"; break;
				case 119: keyName = "F8"; break;
				case 120: keyName = "F9"; break;
				case 121: keyName = "F10"; break;
				case 122: keyName = "F11"; break;
				case 123: keyName = "F12"; break;
				case 124: keyName = "F13"; break;
				case 125: keyName = "F14"; break;
				case 126: keyName = "F15"; break;
				case 127: keyName = "F16"; break;
				case 128: keyName = "F17"; break;
				case 129: keyName = "F18"; break;
				case 130: keyName = "F19"; break;
				case 131: keyName = "F20"; break;
				case 132: keyName = "F21"; break;
				case 133: keyName = "F22"; break;
				case 134: keyName = "F23"; break;
				case 135: keyName = "F24"; break;
				case 144: keyName = "NumLock"; break;
				case 186: keyName = "Semicolon"; break;
				case 187: keyName = "Equal"; break;
				case 188: keyName = "Comma"; break;
				case 189: keyName = "Minus"; break;
				case 190: keyName = "Dot"; break;
				case 191: keyName = "Slash"; break;
				case 192: keyName = "~"; break;
				case 219: keyName = "["; break;
				case 220: keyName = "BackSlash"; break;
				case 221: keyName = "]"; break;
				case 222: keyName = "Apostrophe"; break;
				default:
					if (
						((keyCode >= 48) && (keyCode <= 57)) ||
						((keyCode >= 65) && (keyCode <= 90)))
					{
						keyName = new string((char)(keyCode), 1);
					}
					break;
			}

			return keyName;
		}

		private static void ProcessShortcut(HtmlTag tag, string shortcut)
		{
			foreach (HtmlTag target in tag.Select(
				string.Format(
					CultureInfo.InvariantCulture,
					"[accesskey='{0}'],[accesskey-alt='{0}'],[accesskey-0=='{0}'],[accesskey-1=='{0}'],[accesskey-2=='{0}'],[accesskey-3=='{0}']",
					shortcut)))
			{
				if (target.IsEnabled)
				{
					using (HtmlButton button = new HtmlButton(target))
					{
						button.PerformClick();
					}
				}
			}
		}

		#endregion

		#region Constructors

		public HtmlAccessKeysBehavior()
			: base(HtmlEventGroup.InitializationAndKeyboard)
		{
		}

		#endregion

		#endregion

		#endregion

		#region HtmlEventSinkBase

		protected override void OnKeyboard(HtmlTag sender, HtmlKeyboardEventArgs e)
		{
			if (e.Command == (HtmlKeyboardEventCommand.IsSinking | HtmlKeyboardEventCommand.Up))
			{
				string keyName = GetKeyName(e.KeyCode);

				if (keyName != null)
				{
					string keyShortName = keyName;
					string keyLongName = keyName;
					bool isLongNameAvailable = false;

					if ((e.KeyboardState & HtmlEventKeyboardState.ShiftPressed) != 0)
					{
						keyShortName = "~" + keyShortName;
						keyLongName = "Shift+" + keyLongName;
						isLongNameAvailable = true;
					}

					if ((e.KeyboardState & HtmlEventKeyboardState.ControlPressed) != 0)
					{
						keyShortName = "^" + keyShortName;
						keyLongName = "Ctrl+" + keyLongName;
						isLongNameAvailable = true;
					}

					if ((e.KeyboardState & HtmlEventKeyboardState.AltPressed) != 0)
					{
						keyShortName = "!" + keyShortName;
						keyLongName = "Alt+" + keyLongName;
						isLongNameAvailable = true;
					}

					using (HtmlTag root = sender.Root)
					{
						string keySequenceShortName = keyShortName;
						string keySequenceLongName = keyLongName;

						ProcessShortcut(root, keySequenceShortName);

						if (isLongNameAvailable)
						{
							ProcessShortcut(root, keySequenceLongName);
						}

						for (int index = 0; index < this._keyShortNames.Length; index++)
						{
							if (this._keyShortNames[index] != null)
							{
								keySequenceShortName = this._keyShortNames[index] + "," + keySequenceShortName;
								ProcessShortcut(root, keySequenceShortName);
							}
						}

						if (isLongNameAvailable)
						{
							for (int index = 0; index < this._keyLongNames.Length; index++)
							{
								if (this._keyLongNames[index] != null)
								{
									keySequenceLongName = this._keyLongNames[index] + "," + keySequenceLongName;
									ProcessShortcut(root, keySequenceLongName);
								}
							}
						}
					}

					for (int index = this._keyShortNames.Length - 1; index > 0; --index)
					{
						this._keyShortNames[index] = this._keyShortNames[index - 1];
					}

					for (int index = this._keyLongNames.Length - 1; index > 0; --index)
					{
						this._keyLongNames[index] = this._keyLongNames[index - 1];
					}

					this._keyShortNames[0] = keyShortName;
					this._keyLongNames[0] = keyLongName;
				}
			}

			base.OnKeyboard(sender, e);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
