﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace Nabu.Forms.Html
{
	[HtmlBehaviorName("hyperlink")]
	public sealed class HtmlHyperlinkBehavior : HtmlBehaviorBase
	{
		#region HtmlHyperlinkBehavior

		private static void NotifyHyperlink(
			HtmlTag htmlTag,
			HtmlEventKeyboardState keyboardState)
		{
			IHtmlHost htmlHost =
				((GCHandle)NativeMethods.Window_GetLong(
					htmlTag.GetWindow(false),
					WindowLong.UserData)).Target as IHtmlHost;

			if (htmlHost != null)
			{
				htmlHost.NotificationHandler.OnHyperLink(
					htmlTag,
					htmlTag.Attributes["href"],
					htmlTag.Attributes["target"],
					(((keyboardState & HtmlEventKeyboardState.AltPressed) != 0) ? Keys.Alt : 0) |
					(((keyboardState & HtmlEventKeyboardState.ControlPressed) != 0) ? Keys.Control : 0) |
					(((keyboardState & HtmlEventKeyboardState.ShiftPressed) != 0) ? Keys.Shift : 0));
			}
		}

		public HtmlHyperlinkBehavior() :
			base(HtmlEventGroup.InitializationAndAllInput)
		{
		}

		#endregion

		#region HtmlBehaviorBase

		protected override void OnMouse(HtmlTag sender, HtmlMouseEventArgs e)
		{
			switch (e.Command)
			{
				case HtmlMouseEventCommand.Down:
					if (e.MouseState == HtmlEventMouseState.Left)
					{
						sender.SetState(HtmlTagState.Current, true, true);
						sender.SetState(HtmlTagState.Pressed, true, true);
						e.IsHandled = true;
					}

					break;
				case HtmlMouseEventCommand.Up:
					if (sender.GetState(HtmlTagState.Current))
					{
						sender.SetState(HtmlTagState.Current, false, true);
						sender.SetState(HtmlTagState.Pressed, false, true);
						NotifyHyperlink(sender, e.KeyboardState);

						e.IsHandled = true;
					}

					break;
				case HtmlMouseEventCommand.Enter:
					if (sender.GetState(HtmlTagState.Current))
					{
						if (e.MouseState == HtmlEventMouseState.Left)
						{
							sender.SetState(HtmlTagState.Pressed, true, true);
						}
					}
					break;
				case HtmlMouseEventCommand.Leave:
					if (sender.GetState(HtmlTagState.Current))
					{
						sender.SetState(HtmlTagState.Pressed, false, true);
					}
					break;
			}

			base.OnMouse(sender, e);
		}

		protected override void OnKeyboard(HtmlTag sender, HtmlKeyboardEventArgs e)
		{
			if ((e.KeyCode == ' ') || (e.KeyCode == '\n'))
			{
				using (HtmlTag target = e.Target)
				{
					NotifyHyperlink(target, e.KeyboardState);
				}
			}

			base.OnKeyboard(sender, e);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
