﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;

namespace Nabu.Forms.Html
{
	[HtmlBehaviorName("mouse-zoom")]
	public sealed class HtmlMouseZoomBehavior : HtmlBehaviorBase
	{
		#region HtmlMouseZoomBehavior

		public HtmlMouseZoomBehavior()
			: base(HtmlEventGroup.InitializationAndMouse)
		{
		}

		#endregion

		#region HtmlBehaviorBase

		protected override void OnMouse(HtmlTag sender, HtmlMouseEventArgs e)
		{
			switch (e.Command)
			{
				case (HtmlMouseEventCommand.IsSinking | HtmlMouseEventCommand.Wheel):
					if ((e.KeyboardState & HtmlEventKeyboardState.ControlPressed) != 0)
					{
						int oldZoom = HtmlAttribute.GetInt32(sender, "zoom", 4);
						int newZoom;

						if (e.MouseState == HtmlEventMouseState.Up)
						{
							newZoom = Math.Min(oldZoom + 1, 7);
						}
						else
						{
							newZoom = Math.Max(oldZoom - 1, 1);
						}

						if (newZoom != oldZoom)
						{
							HtmlAttribute.SetInt32(sender, "zoom", newZoom);
							sender.Update(true);
							e.IsHandled = true;
						}
					}
					break;
			}

			base.OnMouse(sender, e);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
