﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;

namespace Nabu.Forms.Html
{
	[HtmlBehaviorName("splitter")]
	public sealed class HtmlSplitterBehavior : HtmlBehaviorBase
	{
		#region HtmlSplitterBehavior

		#region Fields

		private bool _isCaptured;
		private bool _isHorizontal;
		private HtmlTag _parent;
		private HtmlTag _prevSibling;
		private HtmlTag _nextSibling;
		private int _startPrevSize;
		private int _startPosition;

		#endregion

		#region Methods

		public HtmlSplitterBehavior() :
			base(HtmlEventGroup.InitializationAndMouse)
		{
		}

		#endregion

		#endregion

		#region HtmlBehaviorBase

		protected override void OnMouse(HtmlTag sender, HtmlMouseEventArgs e)
		{
			if ((e.MouseState & HtmlEventMouseState.Left) != 0)
			{
				switch (e.Command)
				{
					case HtmlMouseEventCommand.Down:
						{
							using (HtmlTag splitter = e.Target)
							{
								HtmlTag parent = splitter.Parent;
								HtmlTag leftSibling = splitter.PrevSibling;
								HtmlTag rightSibling = splitter.NextSibling;

								if ((parent != null) && (leftSibling != null) && (rightSibling != null))
								{
									this._parent = parent;
									this._prevSibling = leftSibling;
									this._nextSibling = rightSibling;
									this._isHorizontal = this._parent.GetStyle("flow") == "horizontal";

									if (this._isHorizontal)
									{
										this._startPosition = e.PositionInDocument.X;
										this._startPrevSize = this._prevSibling.GetLocation(HtmlTagArea.RootRelative).Width;
									}
									else
									{
										this._startPosition = e.PositionInDocument.Y;
										this._startPrevSize = this._prevSibling.GetLocation(HtmlTagArea.RootRelative).Height;
									}

									splitter.SetCapture();
									this._isCaptured = true;
									e.IsHandled = true;
								}
							}
						}
						break;
					case HtmlMouseEventCommand.Up:
						if (this._isCaptured)
						{
							this._parent = null;
							this._prevSibling = null;
							this._nextSibling = null;

							HtmlTag.ReleaseCapture();
							this._isCaptured = false;
							e.IsHandled = true;
						}
						break;
					case HtmlMouseEventCommand.Move:
						if (this._isCaptured)
						{
							using (HtmlTag splitter = e.Target)
							{
								this._parent.Update(true);

								if (this._isHorizontal)
								{
									int prevSize = this._startPrevSize + (e.PositionInDocument.X - this._startPosition);

									this._prevSibling.SetStyle(
										"width",
										prevSize + "px");
									this._nextSibling.SetStyle(
										"width",
										"100%%");
								}
								else
								{
									int prevSize = this._startPrevSize + (e.PositionInDocument.Y - this._startPosition);

									this._prevSibling.SetStyle(
										"height",
										prevSize + "px");
									this._nextSibling.SetStyle(
										"height",
										"100%%");
								}
							}

							e.IsHandled = true;
						}
						break;
				}
			}

			base.OnMouse(sender, e);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
