﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;

namespace Nabu.Forms.Html
{
	public static class HtmlBehaviorManager
	{
		#region Fields

		private static Dictionary<string, Type> _behaviorTypes = new Dictionary<string, Type>(16);
		private static Dictionary<IHtmlHost, IList<HtmlBehaviorBase>> _behaviorInstances = new Dictionary<IHtmlHost, IList<HtmlBehaviorBase>>();

		#endregion

		#region Methods

		static HtmlBehaviorManager()
		{
			RegisterBehavior(typeof(HtmlAccessKeysBehavior));
			RegisterBehavior(typeof(HtmlCodeEditorBehavior));
			RegisterBehavior(typeof(HtmlCollapsibleItemBehavior));
			RegisterBehavior(typeof(HtmlCollapsibleListBehavior));
#if !COMPACT_FRAMEWORK
			RegisterBehavior(typeof(HtmlDockingWindowsBehavior));
#endif
			RegisterBehavior(typeof(HtmlGripperBehavior));
			RegisterBehavior(typeof(HtmlHyperlinkBehavior));
#if !COMPACT_FRAMEWORK
			RegisterBehavior(typeof(HtmlListViewBehavior));
#endif
			RegisterBehavior(typeof(HtmlMouseZoomBehavior));
#if !COMPACT_FRAMEWORK
			RegisterBehavior(typeof(HtmlShellIconBehavior));
#endif
			RegisterBehavior(typeof(HtmlSplitterBehavior));
			RegisterBehavior(typeof(HtmlTabBehavior));
		}

		public static void RegisterBehavior(Type behaviorType, string behaviorName)
		{
			_behaviorTypes.Add(behaviorName, behaviorType);
		}

		public static void UnregisterBehavior(string behaviorName)
		{
			_behaviorTypes.Remove(behaviorName);
		}

		public static void RegisterBehavior(Type behaviorType)
		{
			if (!typeof(HtmlBehaviorBase).IsAssignableFrom(behaviorType))
			{
				throw
					new ArgumentException(
						"Behavior implementation should be inherited from HtmlBehaviorBase.",
						"behaviorType");
			}

			HtmlBehaviorNameAttribute[] nameAttributes = (HtmlBehaviorNameAttribute[])behaviorType.GetCustomAttributes(typeof(HtmlBehaviorNameAttribute), false);

			if (nameAttributes.Length == 0)
			{
				throw new
					ArgumentException(
						"Behavior name missing.",
						"behaviorType");
			}

			RegisterBehavior(behaviorType, nameAttributes[0].Name);
		}

		public static void UnregisterBehavior(Type behaviorType)
		{
			if (!typeof(HtmlBehaviorBase).IsAssignableFrom(behaviorType))
			{
				throw
					new ArgumentException(
						"Behavior implementation should be inherited from HtmlBehaviorBase.",
						"behaviorType");
			}

			HtmlBehaviorNameAttribute[] nameAttributes = (HtmlBehaviorNameAttribute[])behaviorType.GetCustomAttributes(typeof(HtmlBehaviorNameAttribute), false);

			if (nameAttributes.Length == 0)
			{
				throw new
					ArgumentException(
						"Behavior name missing.",
						"behaviorType");
			}

			UnregisterBehavior(nameAttributes[0].Name);
		}

		public static HtmlBehaviorBase GetBehavior(string behaviorName, IHtmlHost htmlHost)
		{
			Type behaviorType;

			if (_behaviorTypes.TryGetValue(behaviorName, out behaviorType))
			{
				HtmlBehaviorBase behavior = (HtmlBehaviorBase)Activator.CreateInstance(behaviorType);
				IList<HtmlBehaviorBase> instances;

				if (htmlHost == null)
				{
					throw new ArgumentNullException("htmlHost");
				}

				if (!_behaviorInstances.TryGetValue(htmlHost, out instances))
				{
					instances = new List<HtmlBehaviorBase>(4);

					_behaviorInstances[htmlHost] = instances;
					htmlHost.DocumentReseted += new EventHandler(HtmlWindow_DocumentReseted);
				}

				instances.Add(behavior);

				return behavior;
			}
			else
			{
				return null;
			}
		}

		private static void HtmlWindow_DocumentReseted(object sender, EventArgs e)
		{
			_behaviorInstances.Remove((IHtmlHost)sender);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
