﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;

using Nabu.Collections;

namespace Nabu.Forms.Html
{
	[HtmlBehaviorName("list-view")]
	public sealed class HtmlListViewBehavior : HtmlTemplateBehaviorBase
	{
		#region HtmlListViewBehavior

		#region Fields

		private ICouplingList _couplingList;
		private HtmlTag _containerTag;
		private object _binder;

		#endregion

		#region Methods

		private static Type GetBindingListItemType(object dataSource)
		{
			Type dataSourceType = dataSource.GetType();

			foreach (Type interfaceType in dataSourceType.GetInterfaces())
			{
				if (interfaceType.GetGenericTypeDefinition() == typeof(IBindingListEx<>))
				{
					return interfaceType.GetGenericArguments()[0];
				}
			}

			return null;
		}

		public HtmlListViewBehavior()
			: base(
					HtmlEventGroup.InitializationAndBehavior |
					HtmlEventGroup.InitializationAndMethod |
					HtmlEventGroup.InitializationAndAllInput)
		{
		}

		public void SetBindingList(object bindingList)
		{
			Type itemType = GetBindingListItemType(bindingList);

			if (itemType == null)
			{
				throw new ArgumentNullException("bindingList");
			}

			Type binderType = typeof(HtmlListViewBehaviorBinder<>).MakeGenericType(itemType);

			this._binder = Activator.CreateInstance(binderType);

			binderType.GetMethod("Initialize").Invoke(
				this._binder,
				new object[]
				{
					base.Templates,
					bindingList,
					this._containerTag
				});
		}

		public void SetCouplingList(ICouplingList couplingList)
		{
		}

		#endregion

		#endregion

		#region HtmlBehaviorBase

		protected override void OnInitialization(HtmlTag sender, HtmlInitializationEventArgs e)
		{
			this._containerTag = sender.SelectFirst("tbody");

			if (this._containerTag == null)
			{
				this._containerTag = sender;
			}

			base.OnInitialization(sender, e);
		}

		protected override void OnKeyboard(HtmlTag sender, HtmlKeyboardEventArgs e)
		{
			base.OnKeyboard(sender, e);
		}

		protected override void OnMouse(HtmlTag sender, HtmlMouseEventArgs e)
		{
			switch (e.Command)
			{
				case HtmlMouseEventCommand.Down:
					break;
			}

			base.OnMouse(sender, e);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
