﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
namespace Nabu.Forms.Html
{
	[HtmlBehaviorName("collapsible-item")]
	public sealed class HtmlCollapsibleItemBehavior : HtmlBehaviorBase
	{
		#region HtmlCollapsibleBehavior

		#region Methods

		public HtmlCollapsibleItemBehavior()
			: base(HtmlEventGroup.InitializationAndAllInput)
		{
		}

		private static HtmlTag GetGlyph(HtmlTag sender)
		{
			return sender.SelectFirst("#expand_collapse");
		}

		private static void SwitchItemState(HtmlTag sender, HtmlTag target)
		{
			using (HtmlTag sinkGlyph = GetGlyph(sender))
			{
				if (sinkGlyph == target)
				{
					bool isExpanded = sender.GetState(HtmlTagState.Expanded);

					if (isExpanded)
					{
						sender.SetState(HtmlTagState.Collapsed, true, true);
					}
					else
					{
						sender.SetState(HtmlTagState.Expanded, true, true);
					}
				}
			}
		}

		#endregion

		#endregion

		#region HtmlBehaviorBase

		protected override void OnInitialization(HtmlTag sender, HtmlInitializationEventArgs e)
		{
			switch (e.Command)
			{
				case HtmlInitializationEventCommand.AttachBehavior:
					{
						if (
							((sender.Attributes["expanded"] != "false") && (sender.Attributes["expanded"] != null)) ||
							(sender.Attributes["collapsed"] == "false"))
						{
							sender.SetState(HtmlTagState.Expanded, true, true);
						}
						else
						{
							sender.SetState(HtmlTagState.Collapsed, true, true);
						}
					}
					break;
			}

			base.OnInitialization(sender, e);
		}

		protected override void OnMouse(HtmlTag sender, HtmlMouseEventArgs e)
		{
			switch (e.Command)
			{
				case (HtmlMouseEventCommand.IsSinking | HtmlMouseEventCommand.Up):
					if ((e.MouseState & HtmlEventMouseState.Left) != 0)
					{
						using (HtmlTag target = e.Target)
						{
							SwitchItemState(sender, target);
						}
					}
					break;
			}

			base.OnMouse(sender, e);
		}

		protected override void OnKeyboard(HtmlTag sender, HtmlKeyboardEventArgs e)
		{
			switch (e.Command)
			{
				case (HtmlKeyboardEventCommand.IsSinking | HtmlKeyboardEventCommand.Up):
					using (HtmlTag target = e.Target)
					{
						SwitchItemState(sender, target);
					}
					break;
			}

			base.OnKeyboard(sender, e);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
