﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
namespace Nabu.Forms.Html
{
	[HtmlBehaviorName("collapsible-list")]
	public sealed class HtmlCollapsibleListBehavior : HtmlBehaviorBase
	{
		#region HtmlCollapsibleBehavior

		#region Methods

		public HtmlCollapsibleListBehavior()
			: base(HtmlEventGroup.InitializationAndAllInput)
		{
		}

		private static void SetItemState(HtmlTag listItem, bool isExpanded)
		{
			if (isExpanded)
			{
				listItem.SetState(HtmlTagState.Expanded, true, true);
				listItem.SetState(HtmlTagState.Current, true, true);
			}
			else
			{
				listItem.SetState(HtmlTagState.Collapsed, true, true);
				listItem.SetState(HtmlTagState.Current, false, true);
			}
		}

		private static void SetCurrentItem(HtmlTag list, HtmlTag listItem)
		{
			list.Update(true);

			foreach (HtmlTag child in list.Children)
			{
				SetItemState(child, child == listItem);
			}
		}

		private static HtmlTag GetItem(HtmlTag list, HtmlTag listItem)
		{
			if (list == listItem)
			{
				return null;
			}

			if (listItem.Parent == list)
			{
				return listItem;
			}
			else
			{
				return GetItem(list, listItem.Parent);
			}
		}

		#endregion

		#endregion

		#region HtmlBehaviorBase

		protected override void OnInitialization(HtmlTag sender, HtmlInitializationEventArgs e)
		{
			switch (e.Command)
			{
				case HtmlInitializationEventCommand.AttachBehavior:
					{
						foreach (HtmlTag child in sender.Children)
						{
							if (
							((sender.Attributes["expanded"] != "false") && (sender.Attributes["expanded"] != null)) ||
							(sender.Attributes["collapsed"] == "false"))
							{
								SetItemState(child, true);
							}
							else
							{
								SetItemState(child, false);
							}
						}
					}
					break;
			}

			base.OnInitialization(sender, e);
		}

		protected override void OnMouse(HtmlTag sender, HtmlMouseEventArgs e)
		{
			switch (e.Command)
			{
				case (HtmlMouseEventCommand.IsSinking | HtmlMouseEventCommand.Up):
					if ((e.MouseState & HtmlEventMouseState.Left) != 0)
					{
						using (HtmlTag listItem = GetItem(sender, e.Target))
						{
							if (listItem != null)
							{
								SetCurrentItem(sender, listItem);
							}
						}
					}
					break;
			}

			base.OnMouse(sender, e);
		}

		protected override void OnKeyboard(HtmlTag sender, HtmlKeyboardEventArgs e)
		{
			switch (e.Command)
			{
				case (HtmlKeyboardEventCommand.IsSinking | HtmlKeyboardEventCommand.Up):
					using (HtmlTag listItem = GetItem(sender, e.Target))
					{
						if (listItem != null)
						{
							SetCurrentItem(sender, listItem);
						}
					}
					break;
			}

			base.OnKeyboard(sender, e);
		}

		protected override void OnBehavior(HtmlTag sender, HtmlBehaviorEventArgs e)
		{
			switch (e.Command)
			{
				case HtmlBehaviorEventCommand.CustomActivateChild:
					using (HtmlTag listItem = e.Source)
					{
						if (listItem != null)
						{
							SetCurrentItem(sender, listItem);
						}
					}
					break;
			}

			base.OnBehavior(sender, e);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
