﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace Nabu.Forms.Html
{
	public sealed class HtmlAttributeCollection : IDictionary<string, string>
	{
		#region HtmlAttributeCollection

		#region Types

		public sealed class HtmlAttributeCollectionEnumerator :
			DisposableBase,
			IEnumerator<KeyValuePair<string, string>>,
			System.Collections.IEnumerator
		{
			#region HtmlTagCollectionEnumerator

			#region Fields

			private IList<KeyValuePair<string, string>> _collection;
			private int _index;

			#endregion

			#region Methods

			public HtmlAttributeCollectionEnumerator(HtmlAttributeCollection collection)
			{
				this._collection = new List<KeyValuePair<string, string>>();

				int attributeCount;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetAttributeCount(
						collection._tag.Handle,
						out attributeCount));

				for (int index = 0; index < attributeCount; index++)
				{
					IntPtr namePtr;

					HtmlDomException.ThrowOnError(
						HtmlDynamicInterop.Dom.GetAttributeNameByIndex(
							collection._tag.Handle,
							index,
							out namePtr,
							IntPtr.Zero));

					IntPtr valuePtr;

					HtmlDomException.ThrowOnError(
						HtmlDynamicInterop.Dom.GetAttributeValueByIndex(
							collection._tag.Handle,
							index,
							IntPtr.Zero,
							out valuePtr));

#if COMPACT_FRAMEWORK
					this._collection.Add(
						new KeyValuePair<string, string>(
							UnsafeNativeMethods.PtrToStringAnsi(namePtr),
							Marshal.PtrToStringUni(valuePtr)));
#else
					this._collection.Add(
						new KeyValuePair<string, string>(
							Marshal.PtrToStringAnsi(namePtr),
							Marshal.PtrToStringUni(valuePtr)));
#endif
				}

				this._index = -1;
			}

			#endregion

			#endregion

			#region DisposableBase

			protected override void Dispose(bool disposing)
			{
				base.Dispose(disposing);
			}

			#endregion

			#region IEnumerator<HtmlTag>

			KeyValuePair<string, string> IEnumerator<KeyValuePair<string, string>>.Current
			{
				get
				{
					base.CheckDisposed();

					return this._collection[this._index];
				}
			}

			#endregion

			#region IEnumerator

			object System.Collections.IEnumerator.Current
			{
				get
				{
					base.CheckDisposed();

					return this._collection[this._index];
				}
			}

			bool System.Collections.IEnumerator.MoveNext()
			{
				base.CheckDisposed();

				this._index++;

				return this._index < this._collection.Count;
			}

			void System.Collections.IEnumerator.Reset()
			{
				base.CheckDisposed();

				this._index = -1;
			}

			#endregion
		}

		#endregion

		#region Fields

		private HtmlTag _tag;

		#endregion

		#region Methods

		public HtmlAttributeCollection(HtmlTag tag)
		{
			this._tag = tag;
		}

		#endregion

		#endregion

		#region IDictionary<string, string>

		#region Methods

		public void Add(string key, string value)
		{
			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.SetAttributeValueByName(
					this._tag.Handle,
					key,
					value));
		}

		public bool Remove(string key)
		{
			IntPtr valuePtr;

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.GetAttributeValueByName(
					this._tag.Handle,
					key,
					out valuePtr));

			if (valuePtr != IntPtr.Zero)
			{
				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.SetAttributeValueByName(
						this._tag.Handle,
						key,
						null));

				return true;
			}
			else
			{
				return false;
			}
		}

		public bool ContainsKey(string key)
		{
			IntPtr valuePtr;

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.GetAttributeValueByName(
					this._tag.Handle,
					key,
					out valuePtr));

			return valuePtr != IntPtr.Zero;
		}

		public bool TryGetValue(string key, out string value)
		{
			IntPtr valuePtr;

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.GetAttributeValueByName(
					this._tag.Handle,
					key,
					out valuePtr));

			value = valuePtr == IntPtr.Zero ? null : Marshal.PtrToStringUni(valuePtr);

			return value != null;
		}

		#endregion

		#region Properties

		public ICollection<string> Keys
		{
			get
			{
				int attributeCount;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetAttributeCount(
						this._tag.Handle,
						out attributeCount));

				List<string> keys = new List<string>(attributeCount);

				for (int index = 0; index < attributeCount; index++)
				{
					IntPtr namePtr;

					HtmlDomException.ThrowOnError(
						HtmlDynamicInterop.Dom.GetAttributeNameByIndex(
							this._tag.Handle,
							index,
							out namePtr,
							IntPtr.Zero));
#if COMPACT_FRAMEWORK
					keys.Add(UnsafeNativeMethods.PtrToStringAnsi(namePtr));
#else
					keys.Add(Marshal.PtrToStringAnsi(namePtr));
#endif
				}

				return keys;
			}
		}

		public ICollection<string> Values
		{
			get
			{
				int attributeCount;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetAttributeCount(
						this._tag.Handle,
						out attributeCount));

				List<string> values = new List<string>(attributeCount);

				for (int index = 0; index < attributeCount; index++)
				{
					IntPtr valuePtr;

					HtmlDomException.ThrowOnError(
						HtmlDynamicInterop.Dom.GetAttributeValueByIndex(
							this._tag.Handle,
							index,
							IntPtr.Zero,
							out valuePtr));

					values.Add(Marshal.PtrToStringUni(valuePtr));
				}

				return values;
			}
		}

		public string this[string key]
		{
			get
			{
				IntPtr valuePtr;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetAttributeValueByName(
						this._tag.Handle,
						key,
						out valuePtr));

				return valuePtr == IntPtr.Zero ? null : Marshal.PtrToStringUni(valuePtr);
			}

			set
			{
				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.SetAttributeValueByName(
						this._tag.Handle,
						key,
						value));
			}
		}

		#endregion

		#endregion

		#region ICollection<KeyValuePair<string, string>>

		#region Methods

		public void Clear()
		{
			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.ClearAttributes(
					this._tag.Handle));
		}

		public void Add(KeyValuePair<string, string> item)
		{
			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.SetAttributeValueByName(
					this._tag.Handle,
					item.Key,
					item.Value));
		}

		public bool Remove(KeyValuePair<string, string> item)
		{
			IntPtr valuePtr;

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.GetAttributeValueByName(
					this._tag.Handle,
					item.Key,
					out valuePtr));

			if (Marshal.PtrToStringUni(valuePtr) == item.Value)
			{
				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.SetAttributeValueByName(
						this._tag.Handle,
						item.Key,
						null));

				return true;
			}
			else
			{
				return false;
			}
		}

		public bool Contains(KeyValuePair<string, string> item)
		{
			string value;

			if (TryGetValue(item.Key, out value))
			{
				return value == item.Value;
			}
			else
			{
				return false;
			}
		}

		public void CopyTo(KeyValuePair<string, string>[] array, int arrayIndex)
		{
			int attributeCount;

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.GetAttributeCount(
					this._tag.Handle,
					out attributeCount));

			for (int index = 0; index < attributeCount; index++)
			{
				IntPtr name;
				IntPtr value;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetAttributeByIndex(
						this._tag.Handle,
						index,
						out name,
						out value));

				array[arrayIndex + index] = new KeyValuePair<string, string>(
#if COMPACT_FRAMEWORK
					UnsafeNativeMethods.PtrToStringAnsi(name),
#else
Marshal.PtrToStringAnsi(name),
#endif
 Marshal.PtrToStringUni(value));
			}
		}

		#endregion

		#region Properties

		public int Count
		{
			get
			{
				int attributeCount;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetAttributeCount(
						this._tag.Handle,
						out attributeCount));

				return attributeCount;
			}
		}

		public bool IsReadOnly
		{
			[DebuggerStepThrough]
			get { return false; }
		}

		#endregion

		#endregion

		#region IEnumerable<KeyValuePair<string, string>>

		public IEnumerator<KeyValuePair<string, string>> GetEnumerator()
		{
			return new HtmlAttributeCollectionEnumerator(this);
		}

		#endregion

		#region IEnumerable

		System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
		{
			return new HtmlAttributeCollectionEnumerator(this);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
