﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;

namespace Nabu.Forms.Html
{
	public sealed class HtmlScrollBar : IDisposable
	{
		#region HtmlScrollBar

		#region Fields

		private HtmlTag _tag;
		private int _value;
		private int _minValue;
		private int _maxValue;
		private int _pageValue;
		private int _stepValue;

		#endregion

		#region Methods

		private void GetParameters()
		{
			HtmlScrollBarValueMethodEventParameters parameters = new HtmlScrollBarValueMethodEventParameters();

			parameters.MethodID = HtmlTagMethodID.GetScrollValue;

			this._tag.CallMethod(parameters);

			this._value = parameters.Value;
			this._minValue = parameters.MinValue;
			this._maxValue = parameters.MaxValue;
			this._pageValue = parameters.PageValue;
			this._stepValue = parameters.StepValue;
		}

		private void SetParameters()
		{
			HtmlScrollBarValueMethodEventParameters parameters = new HtmlScrollBarValueMethodEventParameters();

			parameters.MethodID = HtmlTagMethodID.SetScrollValue;
			parameters.Value = this._value;
			parameters.MinValue = this._minValue;
			parameters.MaxValue = this._maxValue;
			parameters.PageValue = this._pageValue;
			parameters.StepValue = this._stepValue;

			this._tag.CallMethod(parameters);
		}

		public HtmlScrollBar(HtmlTag tag)
		{
			this._tag = tag;
			GetParameters();
		}

		#endregion

		#region Properties

		public int Value
		{
			get
			{
				return this._value;
			}

			set
			{
				if (this._value != value)
				{
					this._value = value;
					SetParameters();
				}
			}
		}

		public int MinValue
		{
			get
			{
				return this._minValue;
			}

			set
			{
				if (this._minValue != value)
				{
					this._minValue = value;
					SetParameters();
				}
			}
		}

		public int MaxValue
		{
			get
			{
				return this._maxValue;
			}

			set
			{
				if (this._maxValue != value)
				{
					this._maxValue = value;
					SetParameters();
				}
			}
		}

		public int PageValue
		{
			get
			{
				return this._pageValue;
			}

			set
			{
				if (this._pageValue != value)
				{
					this._pageValue = value;
					SetParameters();
				}
			}
		}

		public int StepValue
		{
			get
			{
				return this._stepValue;
			}

			set
			{
				if (this._stepValue != value)
				{
					this._stepValue = value;
					SetParameters();
				}
			}
		}

		#endregion

		#endregion

		#region IDisposable

		public void Dispose()
		{
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
