﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Globalization;
#if !COMPACT_FRAMEWORK
using System.Runtime.Serialization;
#endif

namespace Nabu.Forms.Html
{
#if !COMPACT_FRAMEWORK
	[Serializable]
#endif
	public sealed class HtmlDomException : Exception
	{
		#region Utility

		private static string MessageFromCode(int errorCode)
		{
			switch ((HtmlResult)errorCode)
			{
				case HtmlResult.DOM_Success:
					return "No error";
				case HtmlResult.DOM_Failure_InvalidHWND:
					return "Invalid hWnd.";
				case HtmlResult.DOM_Failure_InvalidHandle:
					return "Invalid element handle.";
				case HtmlResult.DOM_Failure_PassiveHandle:
					return "Passive element handle. Call UseElement";
				case HtmlResult.DOM_Failure_InvalidParameter:
					return "Invalid parameter.";
				case HtmlResult.DOM_Failure_OperationFailed:
					return "Operation failed.";
				case HtmlResult.DOM_Success_NotHandled:
					return "Operation not handled.";
				case HtmlResult.DOM_Failure_Unsupported:
					return "Operation is unsupported.";
				default:
					return
						string.Format(
							CultureInfo.InvariantCulture,
							"Unknown error code {0}.",
							errorCode);
			}
		}

		public static void ThrowOnError(int errorCode)
		{
			switch ((HtmlResult)errorCode)
			{
				case HtmlResult.DOM_Success:
					break;
				default:
					throw new HtmlDomException(errorCode);
			}
		}

		#endregion

		#region Constructors

		internal HtmlDomException(int errorCode)
			: base(MessageFromCode(errorCode))
		{
		}

		public HtmlDomException()
		{
		}

		public HtmlDomException(string message)
			: base(message)
		{
		}

		public HtmlDomException(string message, Exception innerException) :
			base(message, innerException)
		{
		}

#if !COMPACT_FRAMEWORK
		private HtmlDomException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}
#endif

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
