﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;

namespace Nabu.Forms.Html
{
	public sealed class HtmlTagCollection : IList<HtmlTag>
	{
		#region HtmlTagCollection

		#region Types

		public sealed class HtmlTagCollectionEnumerator :
			DisposableBase,
			IEnumerator<HtmlTag>,
			System.Collections.IEnumerator
		{
			#region HtmlTagCollectionEnumerator

			#region Fields

			private HtmlTagCollection _collection;
			private int _index;

			#endregion

			#region Methods

			public HtmlTagCollectionEnumerator(HtmlTagCollection collection)
			{
				this._collection = collection;
				this._index = -1;
			}

			#endregion

			#endregion

			#region DisposableBase

			protected override void Dispose(bool disposing)
			{
				base.Dispose(disposing);
			}

			#endregion

			#region IEnumerator<HtmlTag>

			HtmlTag IEnumerator<HtmlTag>.Current
			{
				get
				{
					base.CheckDisposed();

					return this._collection[this._index];
				}
			}

			#endregion

			#region IEnumerator

			object System.Collections.IEnumerator.Current
			{
				get
				{
					base.CheckDisposed();

					return this._collection[this._index];
				}
			}

			bool System.Collections.IEnumerator.MoveNext()
			{
				base.CheckDisposed();

				this._index++;

				return this._index < this._collection.Count;
			}

			void System.Collections.IEnumerator.Reset()
			{
				base.CheckDisposed();

				this._index = -1;
			}

			#endregion
		}

		#endregion

		#region Fields

		private HtmlTag _parent;

		#endregion

		#region Methods

		public HtmlTagCollection(HtmlTag parent)
		{
			this._parent = parent;
		}

		public void Sort(
			int firstIndex,
			int lastIndex,
			DOM_TagSortHandler sortHandler,
			IntPtr param)
		{
			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.SortChildTags(
					this._parent.Handle,
					firstIndex,
					lastIndex,
					sortHandler,
					param));

			GC.KeepAlive(sortHandler);
		}

		public void Sort(
			int firstIndex,
			int lastIndex,
			DOM_TagSortHandler sortHandler)
		{
			Sort(firstIndex, lastIndex, sortHandler, IntPtr.Zero);
		}

		public void Sort(DOM_TagSortHandler sortHandler)
		{
			Sort(0, this.Count - 1, sortHandler);
		}

		#endregion

		#endregion

		#region IList<HtmlTag>

		public int IndexOf(HtmlTag item)
		{
			int childrenCount;

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.GetChildrenCount(
					this._parent.Handle,
					out childrenCount));

			for (int index = 0; index < childrenCount; index++)
			{
				IntPtr hTagChild;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetChildTag(
						this._parent.Handle,
						index,
						out hTagChild));

				if (item.Handle == hTagChild)
				{
					return index;
				}
			}

			return -1;
		}

		public void Insert(int index, HtmlTag item)
		{
			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.InsertTag(
					item.Handle,
					this._parent.Handle,
					index));
		}

		public void RemoveAt(int index)
		{
			IntPtr hTagChild;

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.GetChildTag(
					this._parent.Handle,
					index,
					out hTagChild));

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.DetachTag(
					hTagChild));
		}

		public HtmlTag this[int index]
		{
			get
			{
				IntPtr hTagChild;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetChildTag(
						this._parent.Handle,
						index,
						out hTagChild));

				if (hTagChild == IntPtr.Zero)
				{
					return null;
				}
				else
				{
					return HtmlTag.FromHandle(hTagChild);
				}
			}

			set
			{
				HtmlDomException.ThrowOnError((int)HtmlResult.DOM_Failure_Unsupported);
			}
		}

		#endregion

		#region ICollection<HtmlTag>

		#region Methods

		public void Add(HtmlTag item)
		{
			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.InsertTag(
					item.Handle,
					this._parent.Handle,
					int.MaxValue));
		}

		public void Clear()
		{
			int childrenCount;

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.GetChildrenCount(
					this._parent.Handle,
					out childrenCount));

			for (int index = childrenCount - 1; index >= 0; --index)
			{
				IntPtr hTagChild;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetChildTag(
						this._parent.Handle,
						index,
						out hTagChild));

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.DetachTag(
						hTagChild));
			}
		}

		public bool Contains(HtmlTag item)
		{
			return IndexOf(item) != -1;
		}

		public void CopyTo(HtmlTag[] array, int arrayIndex)
		{
			int childrenCount;

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.GetChildrenCount(
					this._parent.Handle,
					out childrenCount));

			for (int index = 0; index < childrenCount; index++)
			{
				IntPtr hTagChild;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetChildTag(
						this._parent.Handle,
						index,
						out hTagChild));

				array[arrayIndex + index] = HtmlTag.FromHandle(hTagChild);
			}
		}

		#endregion

		#region Properties

		public int Count
		{
			get
			{
				int childrenCount;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetChildrenCount(
						this._parent.Handle,
						out childrenCount));

				return childrenCount;
			}
		}

		public bool IsReadOnly
		{
			get
			{
				return false;
			}
		}

		public bool Remove(HtmlTag item)
		{
			if (item.Parent == this._parent)
			{
				RemoveAt(item.Index);
			}
			else
			{
				return false;
			}

			return true;
		}

		#endregion

		#endregion

		#region IEnumerable<HtmlTag>

		public IEnumerator<HtmlTag> GetEnumerator()
		{
			return new HtmlTagCollectionEnumerator(this);
		}

		#endregion

		#region IEnumerable

		System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
		{
			return new HtmlTagCollectionEnumerator(this);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
