﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.ComponentModel;
using System.Runtime.InteropServices;

namespace Nabu.Forms.Html
{
	public sealed partial class HtmlTag
	{
		#region Methods

		/// <summary>
		/// Declares new HTML tag type.
		/// </summary>
		/// <param name="name">Name.</param>
		/// <param name="model">Layout model.</param>
		public static void DeclareType(string name, TagLayoutModel model)
		{
			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.DeclareElementType(
					name,
					model));
		}

		/// <summary>
		/// Replaces tag HTML code with new one.
		/// </summary>
		/// <param name="html">New HTML code.</param>
		public void ReplaceHtml(string html)
		{
			base.CheckDisposed();

			byte[] htmlData = HtmlProtocolBase.BinaryDataFromString(html);

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.SetTagHtml(
					this._hTag,
					htmlData,
					htmlData.Length,
					SetTagHtmlBehavior.ReplaceInnerContent));
		}

		/// <summary>
		/// Inserts HtmlTag HTML code at the end of existing one.
		/// </summary>
		/// <param name="html">HTML code.</param>
		public void AppendHtml(string html)
		{
			base.CheckDisposed();

			byte[] htmlData = HtmlProtocolBase.BinaryDataFromString(html);

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.SetTagHtml(
					this._hTag,
					htmlData,
					htmlData.Length,
					SetTagHtmlBehavior.InsertAfterInnerEnd));
		}

		/// <summary>
		/// Inserts HtmlTag HTML code at the beginning of existing one.
		/// </summary>
		/// <param name="html">HTML code.</param>
		public void PrependHtml(string html)
		{
			base.CheckDisposed();

			byte[] htmlData = HtmlProtocolBase.BinaryDataFromString(html);

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Dom.SetTagHtml(
					this._hTag,
					htmlData,
					htmlData.Length,
					SetTagHtmlBehavior.InsertBeforeInnerStart));
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets or sets HtmlTag inner text.
		/// </summary>
		public string InnerText
		{
			get
			{
				base.CheckDisposed();

				IntPtr textPtr;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetTagInnerText16(
						this._hTag,
						out textPtr));

				return textPtr == IntPtr.Zero ? null : Marshal.PtrToStringUni(textPtr);
			}

			set
			{
				base.CheckDisposed();

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.SetTagInnerText16(
						this._hTag,
						value,
						value.Length));
			}
		}

		/// <summary>
		/// Gets or sets HtmlTag inner HTML.
		/// </summary>
		public string InnerHtml
		{
			get
			{
				base.CheckDisposed();

				IntPtr htmlPtr;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetTagHtml(
						this._hTag,
						out htmlPtr,
						false));

				int htmlLength = UnsafeNativeMethods.String_LengthA(htmlPtr);
				byte[] htmlData = new byte[htmlLength];

				Marshal.Copy(htmlPtr, htmlData, 0, htmlLength);

				return HtmlProtocolBase.StringFromBinaryData(htmlData);
			}

			set
			{
				base.CheckDisposed();

				byte[] htmlData = HtmlProtocolBase.BinaryDataFromString(value);

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.SetTagHtml(
						this._hTag,
						htmlData,
						htmlData.Length,
						SetTagHtmlBehavior.ReplaceInnerContent));
			}
		}

		/// <summary>
		/// Gets or sets outer HtmlTag HTML.
		/// </summary>
		public string OuterHtml
		{
			get
			{
				base.CheckDisposed();

				IntPtr htmlPtr;

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.GetTagHtml(
						this._hTag,
						out htmlPtr,
						true));

				int htmlLength = UnsafeNativeMethods.String_LengthA(htmlPtr);
				byte[] htmlData = new byte[htmlLength];

				Marshal.Copy(htmlPtr, htmlData, 0, htmlLength);

				return HtmlProtocolBase.StringFromBinaryData(htmlData);
			}

			set
			{
				base.CheckDisposed();

				byte[] htmlData = HtmlProtocolBase.BinaryDataFromString(value);

				HtmlDomException.ThrowOnError(
					HtmlDynamicInterop.Dom.SetTagHtml(
						this._hTag,
						htmlData,
						htmlData.Length,
						SetTagHtmlBehavior.ReplaceOuterContent));
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
