﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Globalization;
#if !COMPACT_FRAMEWORK
using System.Runtime.Serialization;
#endif

namespace Nabu.Forms.Html
{
#if !COMPACT_FRAMEWORK
	[Serializable]
#endif
	public sealed class HtmlValueException : Exception
	{
		#region Utility

		private static string MessageFromCode(int errorCode)
		{
			switch ((HtmlResult)errorCode)
			{
				case HtmlResult.Value_Success:
					return "No error.";
				case HtmlResult.Value_Bad_Parameter:
					return "Bad parameter.";
				case HtmlResult.Value_Incompatible_Type:
					return "Incompatible type.";
				default:
					return
						string.Format(
							CultureInfo.InvariantCulture,
							"Unknown error code {0}.",
							errorCode);
			}
		}

		public static void ThrowOnError(int errorCode)
		{
			switch ((HtmlResult)errorCode)
			{
				case HtmlResult.Value_Success:
					break;
				default:
					throw new HtmlValueException(errorCode);
			}
		}

		#endregion

		#region Constructors

		internal HtmlValueException(int errorCode)
			: base(MessageFromCode(errorCode))
		{
		}

		public HtmlValueException()
		{
		}

		public HtmlValueException(string message)
			: base(message)
		{
		}

		public HtmlValueException(string message, Exception innerException) :
			base(message, innerException)
		{
		}

#if !COMPACT_FRAMEWORK
		private HtmlValueException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}
#endif

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
