﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Diagnostics;
using System.Drawing;

namespace Nabu.Forms.Html
{
	public sealed class HtmlMouseEventArgs : HtmlEventArgsBase
	{
		#region Constants

		private const int MouseEventCommandDraging = 0x00000100;

		#endregion

		#region Fields

		private HtmlMouseEventParameters _parameters;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		internal HtmlMouseEventArgs(HtmlMouseEventParameters parameters)
			: base(parameters.HTarget)
		{
			this._parameters = parameters;
		}

		[DebuggerStepThrough]
		internal HtmlMouseEventArgs(HtmlMouseEventParameters parameters, HtmlMouseEventCommand command)
			: base(parameters.HTarget)
		{
			this._parameters = parameters;
			this._parameters.Command = command;
		}

		#endregion

		#region Properties

		internal HtmlMouseEventParameters Parameters
		{
			[DebuggerStepThrough]
			get { return this._parameters; }
		}

		public HtmlMouseEventCommand Command
		{
			[DebuggerStepThrough]
			get { return (HtmlMouseEventCommand)((int)this._parameters.Command & ~MouseEventCommandDraging); }
		}

		public bool IsDragging
		{
			[DebuggerStepThrough]
			get { return ((int)this._parameters.Command & MouseEventCommandDraging) != 0; }
		}

		public Point PositionInTag
		{
			[DebuggerStepThrough]
			get { return this._parameters.PositionInTag; }
		}

		public Point PositionInDocument
		{
			[DebuggerStepThrough]
			get { return this._parameters.PositionInDocument; }
		}

		public HtmlEventMouseState MouseState
		{
			[DebuggerStepThrough]
			get { return this._parameters.MouseState; }
		}

		public HtmlEventKeyboardState KeyboardState
		{
			[DebuggerStepThrough]
			get { return this._parameters.KeyboardState; }
		}

		public HtmlCursorType CursorType
		{
			[DebuggerStepThrough]
			get { return this._parameters.CursorType; }
			[DebuggerStepThrough]
			set { this._parameters.CursorType = value; }
		}

		public bool IsOverIcon
		{
			[DebuggerStepThrough]
			get { return this._parameters.IsOverIcon; }
		}

		public HtmlTag DragTarget
		{
			[DebuggerStepThrough]
			get { return HtmlTag.FromHandle(this._parameters.HDragTarget); }
			[DebuggerStepThrough]
			set { this._parameters.HDragTarget = value.Handle; }
		}

		public HtmlEventDragMode DragMode
		{
			[DebuggerStepThrough]
			get { return this._parameters.DragMode; }
			[DebuggerStepThrough]
			set { this._parameters.DragMode = value; }
		}

		#endregion
	};
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
