﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;

namespace Nabu.Forms.Html
{
	internal sealed class HtmlTagEventSink : HtmlEventSinkBase
	{
		#region HtmlTagEventSink

		#region Methods

		public HtmlTagEventSink()
			: base(HtmlEventGroup.All)
		{
		}

		#endregion

		#region Events

		public event EventHandler<HtmlInitializationEventArgs> Initialization;
		public event EventHandler<HtmlMouseEventArgs> Mouse;
		public event EventHandler<HtmlKeyboardEventArgs> Keyboard;
		public event EventHandler<HtmlFocusEventArgs> Focus;
		public event EventHandler<HtmlScrollEventArgs> Scroll;
		public event EventHandler<HtmlTimerEventArgs> Timer;
		public event EventHandler<EventArgs> Size;
		public event EventHandler<HtmlDrawEventArgs> Draw;
		public event EventHandler<HtmlDataEventArgs> Data;
		public event EventHandler<HtmlBehaviorEventArgs> Behavior;

		#endregion

		#endregion

		#region HtmlEventSinkBase

		protected override void OnInitialization(HtmlTag sender, HtmlInitializationEventArgs e)
		{
			if (this.Initialization != null)
			{
				this.Initialization(sender, e);
			}

			base.OnInitialization(sender, e);
		}

		protected override void OnMouse(HtmlTag sender, HtmlMouseEventArgs e)
		{
			if (this.Mouse != null)
			{
				this.Mouse(sender, e);
			}

			base.OnMouse(sender, e);
		}

		protected override void OnKeyboard(HtmlTag sender, HtmlKeyboardEventArgs e)
		{
			if (this.Keyboard != null)
			{
				this.Keyboard(sender, e);
			}

			base.OnKeyboard(sender, e);
		}

		protected override void OnFocus(HtmlTag sender, HtmlFocusEventArgs e)
		{
			if (this.Focus != null)
			{
				this.Focus(sender, e);
			}

			base.OnFocus(sender, e);
		}

		protected override void OnScroll(HtmlTag sender, HtmlScrollEventArgs e)
		{
			if (this.Scroll != null)
			{
				this.Scroll(sender, e);
			}

			base.OnScroll(sender, e);
		}

		protected override void OnTimer(HtmlTag sender, HtmlTimerEventArgs e)
		{
			if (this.Timer != null)
			{
				this.Timer(sender, e);
			}

			base.OnTimer(sender, e);
		}

		protected override void OnSize(HtmlTag sender, EventArgs e)
		{
			if (this.Size != null)
			{
				this.Size(sender, e);
			}

			base.OnSize(sender, e);
		}

		protected override void OnDraw(HtmlTag sender, HtmlDrawEventArgs e)
		{
			if (this.Draw != null)
			{
				this.Draw(sender, e);
			}

			base.OnDraw(sender, e);
		}

		protected override void OnData(HtmlTag sender, HtmlDataEventArgs e)
		{
			if (this.Data != null)
			{
				this.Data(sender, e);
			}

			base.OnData(sender, e);
		}

		protected override void OnBehavior(HtmlTag sender, HtmlBehaviorEventArgs e)
		{
			if (this.Behavior != null)
			{
				this.Behavior(sender, e);
			}

			base.OnBehavior(sender, e);
		}

		protected override void OnMethod(HtmlTag sender, HtmlMethodEventArgs e)
		{
			// Process methods in behaviors, not event handlers.
			base.OnMethod(sender, e);
		}

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				this.Initialization = null;
				this.Mouse = null;
				this.Keyboard = null;
				this.Focus = null;
				this.Scroll = null;
				this.Timer = null;
				this.Size = null;
				this.Draw = null;
				this.Data = null;
				this.Behavior = null;
			}

			base.Dispose(disposing);
		}

		#endregion
	};
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
