﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Text;

namespace Nabu.Forms.Html
{
	public abstract class HtmlProtocolBase
	{
		#region Methods

		/// <summary>
		/// Returns binary form of Image object.
		/// </summary>
		/// <param name="value">Image object.</param>
		/// <returns>Binary form of Image object</returns>
		public static byte[] BinaryDataFromImage(Image value)
		{
			using (MemoryStream stream = new MemoryStream(value.Width * value.Height))
			{
#if COMPACT_FRAMEWORK
				value.Save(stream, ImageFormat.Png);
#else
				if (value.RawFormat.Equals(ImageFormat.MemoryBmp) || value.RawFormat.Equals(ImageFormat.Bmp))
				{
					value.Save(stream, ImageFormat.Png);
				}
				else
				{
					value.Save(stream, value.RawFormat);
				}
#endif
				byte[] binaryData = new byte[stream.Length];
				Array.Copy(stream.GetBuffer(), binaryData, binaryData.Length);

				return binaryData;
			}
		}

		/// <summary>
		/// Returns binary form of string object.
		/// </summary>
		/// <param name="value">String object.</param>
		/// <returns>Binary form of string object</returns>
		public static byte[] BinaryDataFromString(string value)
		{
			byte[] preamble = Encoding.UTF8.GetPreamble();
			byte[] binaryData = new byte[preamble.Length + Encoding.UTF8.GetByteCount(value)];

			Array.Copy(preamble, binaryData, preamble.Length);
			Encoding.UTF8.GetBytes(value, 0, value.Length, binaryData, preamble.Length);

			return binaryData;
		}

		/// <summary>
		/// Returns string object recreated from binary data.
		/// </summary>
		/// <param name="value">Binary data.</param>
		/// <returns>String object recreated from binary data</returns>
		public static string StringFromBinaryData(byte[] value)
		{
			return Encoding.UTF8.GetString(value, 0, value.Length);
		}

		public abstract byte[] LoadResource(
			Uri resourceUri,
			HtmlResourceType resourceUriType);

		#endregion

		#region Properties

		[SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays")]
		public abstract string[] Names
		{
			get;
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
