﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
using System.Text;

namespace Nabu.Forms.Html
{
	public sealed class HtmlProtocolConst : HtmlProtocolBase
	{
		#region HtmlProtocolConst

		#region Constants

		private readonly string[] _names = new string[] { "const" };

		#region Basic Layout

		private readonly byte[] _cssBasicLayout =
			Encoding.ASCII.GetBytes(
@".h-strip
{
	flow: horizontal;
}
.v-strip
{
	flow: vertical;
}
.h-flow
{
	flow: horizontal;
}
.v-flow
{
	flow: vertical;
}

.h-spring
{
	width: 100%%;
}
.v-spring
{
	height: 100%%;
}

.h-container
{
	flow: horizontal;
	width: 100%%;
	height: 100%%;
}
.v-container
{
	flow: vertical;
	width: 100%%;
	height: 100%%;
}

.left-align
{
	text-align: left;
}
.center-align
{
	text-align: center;
}
.right-align
{
	text-align: right;
}
"
			);

		#endregion

		#region Native Controls

		private readonly byte[] _cssNativeControls =
			Encoding.ASCII.GetBytes(
@"html[native-controls='true']
{
	behavior: ~ tab;
	background-color: ThreeDFace;
}
html[native-controls='true'] body
{
	margin:  0.0em;
	padding: 0.0em;
	font-family: 'Sans Serif';
}
/* MENU */
html[native-controls='true'] ul.menu-bar
{
	behavior: menu-bar;
	padding: 0.4em 0.2em 0.0em 0.2em;
	margin: 0;
	flow: horizontal;
	background-color: ThreeDFace Window Window ThreeDFace;
	color: menutext;
}
html[native-controls='true'] ul.menu-bar > li
{
	width: max-intrinsic;
	min-height: 24px;
	padding: 0.1em 0.6em 0.1em 0.6em;
	margin: 0.0em 0.2em 0.0em 0.0em;
	border: 1px solid transparent;
	list-style: none;
}
html[native-controls='true'] ul.menu-bar > li:current
{
	background-color: Window;
	border: 1px solid ThreeDShadow;
	border-bottom:1px solid transparent;
}
html[native-controls='true'] ul.menu-bar menu
{
	behavior: menu;
	flow: vertical;
	width: max-intrinsic;
	min-width: 10em;
	display: none;
	background-color: Window;
	border: 1px solid ThreeDShadow;
	color: windowtext;
	margin: 0;
	padding: 0;
}
html[native-controls='true'] ul.menu-bar > li > menu:popup
{
	border-top:none;
	background-image:url(const:h-line.png);
	background-repeat:no-repeat;
	background-image-transformation:colorize(ThreeDShadow);
	assigned!: self::background-position-left = self.parent().box-border-width() - 2;
}
html[native-controls='true'] ul.menu-bar menu > li
{
	width: *;
	min-height: 32px;
	padding: 0.4em 0.2em -0.4em 40px;
	margin: 0;
}
html[native-controls='true'] ul.menu-bar menu > li:current
{
	background-color: Highlight;
	color: HighlightText;
}
/* LABEL */
html[native-controls='true'] div.label
{
	margin: 0.2em 0.2em 0.2em 0.2em;
}
/* GROUPBOX */
html[native-controls='true'] div.groupbox
{
	margin:  1.0em 0.4em 0.4em 0.4em;
	padding: 0.6em 0.4em 0.4em 0.4em;
	background-image: url(theme:groupbox-normal);
	background-repeat: stretch;
	overflow: visible;
}
html[native-controls='true'] div.groupbox:disabled
{
	background-image: url(theme:groupbox-disabled);
	background-repeat: stretch;
}
html[native-controls='true'] div.groupbox > div.title
{
	position: absolute;
	left: 1.0em;
	top: -0.6em;
	width: max-intrinsic;
	background-color: ThreeDFace;
}
html[native-controls='true'] div.groupbox:disabled > div.title
{
	color: GrayText;
}
html[native-controls='true'] div.tab-panel > div.groupbox >  div.title
{
	background-color: ThreeDHighlight;
}
/* TAB */
html[native-controls='true'] div.tab-header-strip
{
	padding: 0.4em 0.2em 0.0em 0.2em;
	flow: horizontal;
}
html[native-controls='true'] div.tab-header
{
	width: max-intrinsic;
	padding: 4px 0.6em 2px 0.6em;
	background-image: url(theme:tab-item-normal);
	background-repeat: stretch;
	background-position: 0.0em 0.0em 0.0em 0.0em;
}
html[native-controls='true'] div.tab-header:hover
{
	background-image:  url(theme:tab-item-hover);
	background-repeat: stretch;
}
html[native-controls='true'] div.tab-header[selected]
{
	position: relative;
	padding: 2px 0.6em 4px 0.6em;
	background-image: url(theme:tab-item-selected);
	background-repeat: stretch;
	background-position: -2px 0.0em -2px 0.0em;
}
html[native-controls='true'] div.tab-header[selected]:hover
{
	background-image:  url(theme:tab-item-selected);
	background-repeat: stretch;
}
html[native-controls='true'] div.tab-header-strip:focus div.tab-header[selected]
{
	background-image: url(theme:tab-item-selected);
	background-repeat: stretch;
	outline: 1px dotted invert -3px;
}
html[native-controls='true'] div.tab-panel-strip
{
}
html[native-controls='true'] div.tab-panel
{
	visibility: collapse;
}
html[native-controls='true'] div.tab-panel:expanded
{
	height: 100%%;
	padding: 0.4em;
	visibility: visible;
	background-image: url(theme:tab-panel);
	background-repeat: stretch;
}
/* STATUSBAR */
html[native-controls='true'] div.status-bar
{
	height: 1.4em;
	padding: 0.1em 0.4em 0.1em 0.4em;
	background-color: ThreeDFace;
	border-top: 2px groove ThreeDShadow;
}
/* COMBOBOX, LISTBOX */
html[native-controls='true'] select
{
	margin: 0.0em 0.2em 0.0em 0.2em;
}
/* BUTTON */
html[native-controls='true'] widget[type='button']
{
	width: max-intrinsic;
	margin: 0.0em 0.2em 0.0em 0.2em;
	padding: 0.4em 1.2em 0.4em 1.2em;
}
/* CHECKBOX */
html[native-controls='true'] widget[type='checkbox']
{
	margin: 0.0em 0.2em 0.0em 0.2em;
}
/* RADIOBUTTON */
html[native-controls='true'] widget[type='radio']
{
	margin: 0.0em 0.2em 0.0em 0.2em;
}
/* TEXTBOX */
html[native-controls='true'] widget[type='text']
{
	margin: 0.0em 0.2em 0.0em 0.2em;
}
html[native-controls='true'] widget[type='number']
{
	margin: 0.0em 0.2em 0.0em 0.2em;
}
html[native-controls='true'] plaintext
{
	margin: 0.2em 0.2em 0.2em 0.2em;
}
/* SPLITTER */
html[native-controls='true'] hr.h-splitter
{
	behavior: splitter;
	width: 3px;
	height: 100%%;
	cursor: w-resize;
}
html[native-controls='true'] hr.v-splitter
{
	behavior: splitter;
	width: 100%%;
	height: 4px;
	cursor: n-resize;
}
"
			);

		#endregion

		#region Docking Windows

		private readonly byte[] _cssDockingWindows =
			Encoding.UTF8.GetBytes(
@"html
{
	behavior: ~ docking-windows;
}
/* CONTENT */
div.dock-content
{
	width: 100%%;
	height: 100%%;
}
div.dock-content-title
{
}
/* CONTAINER */
div.dock-container
{
	width: *;
	height: *;
	flow: vertical;
}
/* TAB */
div.dock-container > div.tab-header-strip
{
}
div.dock-container > div.tab-header-strip > div.tab-header
{
	margin-left: -1px;
	padding: 0px 2px 2px 2px;
	background-image: none;
	background-color: ThreeDLightShadow ThreeDLightShadow Window Window;
	border: 1px solid ThreeDShadow;
	overflow: hidden;
	text-overflow: ellipsis;
	white-space: nowrap;
}
div.dock-container > div.tab-header-strip > div.tab-header:hover
{
	background-image: none;
	background-color: Highlight Highlight ThreeDLightShadow ThreeDLightShadow;
}
div.dock-container > div.tab-header-strip > div.tab-header[selected]
{
	position: static;
	padding: 0px 2px 6px 2px;
	background-image: none;
	background-color: Window;
	border-top: none;
}
div.dock-container > div.tab-header-strip > div.tab-header[selected]:hover
{
	background-image: none;
	background-color: ThreeDLightShadow ThreeDLightShadow Window Window;
}
div.dock-container > div.tab-header-strip:focus > div.tab-header[selected]
{
	background-image: none;
	background-color: Window;
	outline: 1px dotted invert -4px;
}
div.dock-container > div.tab-panel-strip
{
	width: 100%%;
	height: 100%%;
}
/* DRAG MARKERS */
div.dock-drag
{
	padding: 3px;
}
div.dock-drag > div
{
	width:  100%%;
	height: 100%%;
	border-style:    solid;
	border-width:    3px;
	border-color:    #CFCFCF;
	outline-style:   dotted;
	outline-width:   3px;
	outline-color:   #3F3FFF;
	outline-offset: -3px;
}
div.dock-edge
{
}
div.dock-center
{
	position: absolute;
}
table.dock-edge
{
	position: absolute;
	top:    0;
	left:   0;
	bottom: 0;
	right:  0;
}
table.dock-edge > tr:nth-child(1)
{
	height: 32px;
}
table.dock-edge > tr:nth-child(3)
{
	height: 32px;
}
table.dock-edge > tr:nth-child(5)
{
	height: 32px;
}
table.dock-edge > tr > td
{
	overflow: hidden;
}
table.dock-edge > tr > td:nth-child(1)
{
	width: 32px;
}
table.dock-edge > tr > td:nth-child(3)
{
	width: 32px;
}
table.dock-edge > tr > td:nth-child(5)
{
	width: 32px;
}
table.dock-edge > tr:nth-child(1) > td:nth-child(3)
{
	background-image: url(const:docking-target-edge.png);
}
table.dock-edge > tr:nth-child(3) > td:nth-child(1)
{
	background-image: url(const:docking-target-edge.png);
}
table.dock-edge > tr:nth-child(3) > td:nth-child(5)
{
	background-image: url(const:docking-target-edge.png);
}
table.dock-edge > tr:nth-child(5) > td:nth-child(3)
{
	background-image: url(const:docking-target-edge.png);
}
table.dock-center
{
	width:  100%;
	height: 100%;
	background-image: url(const:docking-target-center.png);
	background-repeat: no-repeat;
	background-position: center center;
}
table.dock-center > tr:nth-child(2)
{
	height: 32px;
}
table.dock-center > tr:nth-child(3)
{
	height: 32px;
}
table.dock-center > tr:nth-child(4)
{
	height: 32px;
}
table.dock-center > tr > td:nth-child(2)
{
	width: 32px;
}
table.dock-center > tr > td:nth-child(3)
{
	width: 32px;
}
table.dock-center > tr > td:nth-child(4)
{
	width: 32px;
}
table.dock-center > tr > td
{
	overflow: hidden;
}
div.dock-target
{
	position: absolute;
	width:  22px;
	height: 22px;
	background-repeat: no-repeat;
}
div.dock-target:current
{
	outline: 5px glow blue;
}
div.dock-target#center-top
{
	top:    5px;
	left:   *;
	right:  *;
	background-image: url(const:docking-target-top.png);
}
div.dock-target#center-left
{
	top:    *;
	left:   5px;
	bottom: *;
	background-image: url(const:docking-target-left.png);
}
div.dock-target#center-bottom
{
	left:   *;
	bottom: 5px;
	right:  *;
	background-image: url(const:docking-target-bottom.png);
}
div.dock-target#center-right
{
	top:    *;
	bottom: *;
	right:  5px;
	background-image: url(const:docking-target-right.png);
}
div.dock-target#center-merge
{
	top:    *;
	left:   *;
	bottom: *;
	right:  *;
	width:  24px;
	height: 24px;
	background-image: url(const:docking-target-merge.png);
}
div.dock-target#edge-top
{
	top:    5px;
	left:   *;
	right:  *;
	background-image: url(const:docking-target-top.png);
}
div.dock-target#edge-left
{
	top:    *;
	left:   5px;
	bottom: *;
	background-image: url(const:docking-target-left.png);
}
div.dock-target#edge-bottom
{
	left:   *;
	bottom: 5px;
	right:  *;
	background-image: url(const:docking-target-bottom.png);
}
div.dock-target#edge-right
{
	top:    *;
	bottom: *;
	right:  5px;
	background-image: url(const:docking-target-right.png);
}
");

		#endregion

		#endregion

		#region Methods

		public HtmlProtocolConst()
		{
		}

		#endregion

		#endregion

		#region IHtmlProtocol

		#region Methods

		public override byte[] LoadResource(
			Uri resourceUri,
			HtmlResourceType resourceUriType)
		{
			switch (resourceUri.LocalPath)
			{
				case "basic-layout.css":
					return this._cssBasicLayout;
				case "native-controls.css":
					return this._cssNativeControls;
				case "docking-windows.css":
					return this._cssDockingWindows;
				case "h-line.png":
					return Properties.Resources.HLine;
				case "v-line.png":
					return Properties.Resources.VLine;
				case "docking-target-center.png":
					return Properties.Resources.DockingTargetCenterBitmap;
				case "docking-target-edge.png":
					return Properties.Resources.DockingTargetEdgeBitmap;
				case "docking-target-bottom.png":
					return Properties.Resources.DockingTargetBottomBitmap;
				case "docking-target-left.png":
					return Properties.Resources.DockingTargetLeftBitmap;
				case "docking-target-merge.png":
					return Properties.Resources.DockingTargetMergeBitmap;
				case "docking-target-right.png":
					return Properties.Resources.DockingTargetRightBitmap;
				case "docking-target-top.png":
					return Properties.Resources.DockingTargetTopBitmap;
				default:
					return null;
			}
		}

		#endregion

		#region Properties

		public override string[] Names
		{
			[DebuggerStepThrough]
			get { return this._names; }
		}

		#endregion

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
