﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net;
#if !COMPACT_FRAMEWORK
using System.Net.Cache;
#endif

namespace Nabu.Forms.Html
{
	public sealed class HtmlProtocolHttp : HtmlProtocolBase
	{
		#region HtmlProtocolHttp

		#region Constants

		private readonly string[] _names = new string[] { "http", "https" };

		#endregion

		#region Fields

		private string _userAgent;

		#endregion

		#region Methods

		public HtmlProtocolHttp()
		{
			this._userAgent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 2.0.50727)";
		}

		public HtmlProtocolHttp(string userAgent)
		{
			this._userAgent = userAgent;
		}

		#endregion

		#endregion

		#region IHtmlProtocol

		#region Methods

		public override byte[] LoadResource(
			Uri resourceUri,
			HtmlResourceType resourceUriType)
		{
			HttpWebRequest webRequest = (HttpWebRequest)HttpWebRequest.Create(resourceUri);

			webRequest.AllowAutoRedirect = true;

#if !COMPACT_FRAMEWORK
			webRequest.AutomaticDecompression = DecompressionMethods.Deflate | DecompressionMethods.GZip;
			webRequest.CachePolicy = new RequestCachePolicy(RequestCacheLevel.Revalidate);
#endif

			webRequest.MaximumAutomaticRedirections = 4;
			webRequest.Timeout = 10 * 1000;
			webRequest.UserAgent = this._userAgent;

			using (HttpWebResponse webResponse = (HttpWebResponse)webRequest.GetResponse())
			{
				if (webResponse.ContentLength > 0)
				{
					byte[] resourceData = new byte[webResponse.ContentLength];

					using (Stream responseStream = webResponse.GetResponseStream())
					{
						for (
							int reponseStreamOffset = 0;
							reponseStreamOffset < resourceData.Length; )
						{
							reponseStreamOffset += responseStream.Read(resourceData, reponseStreamOffset, resourceData.Length - reponseStreamOffset);
						}
					}

					return resourceData;
				}
				else
				{
					List<byte> responseBuffer = new List<byte>(64 * 1024);

					using (Stream responseStream = webResponse.GetResponseStream())
					{
						for (
							int value = responseStream.ReadByte();
							value != -1;
							value = responseStream.ReadByte())
						{
							responseBuffer.Add((byte)value);
						}
					}

					return responseBuffer.ToArray();
				}
			}
		}

		#endregion

		#region Properties

		public override string[] Names
		{
			[DebuggerStepThrough]
			get { return this._names; }
		}

		#endregion

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
