﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Resources;
using System.Text.RegularExpressions;

namespace Nabu.Forms.Html
{
	public sealed class HtmlProtocolResX : HtmlProtocolBase
	{
		#region HtmlProtocolResX

		#region Constants

		private readonly string[] _names = new string[] { "resx", "nres" };

		#endregion

		#region Fields

		private Regex _regex = new Regex(@"/([A-Za-z0-9_\\.]+)/([A-Za-z0-9_\\.]+)(:[a-z][a-z]-[A-Z][A-Z]|)", RegexOptions.Compiled);

		#endregion

		#endregion

		#region IHtmlProtocol

		#region Methods

		public override byte[] LoadResource(
			Uri resourceUri,
			HtmlResourceType resourceUriType)
		{
			Match matchResource = _regex.Match(resourceUri.LocalPath);

			if (matchResource.Success)
			{
				string resourceFileName = resourceUri.Host;
				string resourceBaseName = matchResource.Groups[1].Value;
				string resourceObjectName = matchResource.Groups[2].Value;
				string resourceLocaleName = matchResource.Groups[3].Value;
				ResourceManager resourceManager = null;

				if (resourceFileName == ".")
				{
					resourceFileName = SpecialPath.ApplicationFile;
				}

				try
				{
#if COMPACT_FRAMEWORK
					resourceManager = new ResourceManager(
						resourceBaseName,
						Assembly.Load(resourceFileName));
#else
					try
					{
						resourceManager =
							new ResourceManager(
								resourceBaseName,
								Assembly.ReflectionOnlyLoadFrom(resourceFileName));
					}
					catch (FileLoadException)
					{
						resourceManager =
							new ResourceManager(
								resourceBaseName,
								Assembly.Load(resourceFileName));
					}
#endif
					object resourceObject = resourceManager.GetObject(resourceObjectName, new CultureInfo(resourceLocaleName));
					string resourceAsString = resourceObject as string;
					Image resourceAsImage = resourceObject as Image;

					switch (resourceUriType)
					{
						case HtmlResourceType.Html:
						case HtmlResourceType.StyleSheet:
							return HtmlProtocolBase.BinaryDataFromString(resourceAsString);
						case HtmlResourceType.Image:
						case HtmlResourceType.Cursor:
							return HtmlProtocolBase.BinaryDataFromImage(resourceAsImage);
						case HtmlResourceType.Unknown:
							if (resourceAsString != null)
							{
								return HtmlProtocolBase.BinaryDataFromString(resourceAsString);
							}
							else if (resourceAsImage != null)
							{
								return HtmlProtocolBase.BinaryDataFromImage(resourceAsImage);
							}
							break;
					}
				}
				finally
				{
					if (resourceManager != null)
					{
						resourceManager.ReleaseAllResources();
					}
				}
			}

			throw new MissingManifestResourceException();
		}

		#endregion

		#region Properties

		public override string[] Names
		{
			[DebuggerStepThrough]
			get { return this._names; }
		}

		#endregion

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
