using System;
using System.Drawing;
using System.Runtime.InteropServices;

namespace Nabu.Forms.Html
{
	internal sealed class JsonValueDynamicInterop : DynamicInterop
	{
		#region Types
#if !COMPACT_FRAMEWORK
		private delegate int ClearHandler([In][Out] ref JsonValueData value);

		private delegate int GetBinaryDataHandler([In] ref JsonValueData value, [Out] out IntPtr typedValueData, [Out] out int typedValueSize);

		private delegate int GetElementHandler([In] ref JsonValueData value, [In] int index, [Out] out JsonValueData elementValue);

		private delegate int GetElementCountHandler([In] ref JsonValueData value, [Out] out int elementCount);

		private delegate int GetElementKeyHandler([In] ref JsonValueData value, [In] int index, [Out] out JsonValueData elementKey);

		private delegate int GetFloatDataHandler([In] ref JsonValueData value, [Out] out double typedValue);

		private delegate int GetIntDataHandler([In] ref JsonValueData value, [Out] out int typedValue);

		private delegate int GetLongDataHandler([In] ref JsonValueData value, [Out] out long typedValue);

		private delegate int GetStringDataHandler([In] ref JsonValueData value, [Out] out IntPtr typedValueData, [Out] out int typedValueSize);

		private delegate int GetTypeHandler([In] ref JsonValueData value, [Out] out JsonValueType valueType, [Out] out JsonValueUnit valueUnit);

		private delegate int InitHandler([Out] out JsonValueData value);

		private delegate int SetBinaryDataHandler([In][Out] ref JsonValueData value, [In] byte[] typedValueData, [In] int typedValueSize, [In][MarshalAs(UnmanagedType.U4)] JsonValueType typedValueType, [In][MarshalAs(UnmanagedType.U4)] JsonValueUnit typedValueUnit);

		private delegate int SetElementHandler([In][Out] ref JsonValueData value, [In] int index, [In] ref JsonValueData elementValue);

		private delegate int SetElementValueHandler([In][Out] ref JsonValueData value, [In] ref JsonValueData elementKey, [In] ref JsonValueData elementValue);

		private delegate int SetFloatDataHandler([In][Out] ref JsonValueData value, [In] double typedValueData, [In][MarshalAs(UnmanagedType.U4)] JsonValueType typedValueType, [In][MarshalAs(UnmanagedType.U4)] JsonValueUnit typedValueUnit);

		private delegate int SetIntDataHandler([In][Out] ref JsonValueData value, [In] int typedValueData, [In][MarshalAs(UnmanagedType.U4)] JsonValueType typedValueType, [In][MarshalAs(UnmanagedType.U4)] JsonValueUnit typedValueUnit);

		private delegate int SetLongDataHandler([In][Out] ref JsonValueData value, [In] long typedValueData, [In][MarshalAs(UnmanagedType.U4)] JsonValueType typedValueType, [In][MarshalAs(UnmanagedType.U4)] JsonValueUnit typedValueUnit);

		private delegate int SetStringDataHandler([In][Out] ref JsonValueData value, [In][MarshalAs(UnmanagedType.LPWStr)] string typedValueData, [In] int typedValueSize, [In][MarshalAs(UnmanagedType.U4)] JsonValueUnit typedValueUnit);

#endif
		#endregion

		#region Fields

#if COMPACT_FRAMEWORK
		private IntPtr _clear;
		private IntPtr _getBinaryData;
		private IntPtr _getElement;
		private IntPtr _getElementCount;
		private IntPtr _getElementKey;
		private IntPtr _getFloatData;
		private IntPtr _getIntData;
		private IntPtr _getLongData;
		private IntPtr _getStringData;
		private IntPtr _getType;
		private IntPtr _init;
		private IntPtr _setBinaryData;
		private IntPtr _setElement;
		private IntPtr _setElementValue;
		private IntPtr _setFloatData;
		private IntPtr _setIntData;
		private IntPtr _setLongData;
		private IntPtr _setStringData;
#else
		private ClearHandler _clear;
		private GetBinaryDataHandler _getBinaryData;
		private GetElementHandler _getElement;
		private GetElementCountHandler _getElementCount;
		private GetElementKeyHandler _getElementKey;
		private GetFloatDataHandler _getFloatData;
		private GetIntDataHandler _getIntData;
		private GetLongDataHandler _getLongData;
		private GetStringDataHandler _getStringData;
		private GetTypeHandler _getType;
		private InitHandler _init;
		private SetBinaryDataHandler _setBinaryData;
		private SetElementHandler _setElement;
		private SetElementValueHandler _setElementValue;
		private SetFloatDataHandler _setFloatData;
		private SetIntDataHandler _setIntData;
		private SetLongDataHandler _setLongData;
		private SetStringDataHandler _setStringData;
#endif

		#endregion

		#region Methods

		public JsonValueDynamicInterop(IntPtr hModule, string namePrefix)
			: base(hModule)
		{
#if COMPACT_FRAMEWORK
			this._clear = GetProcAddress(namePrefix + "Clear");
			this._getBinaryData = GetProcAddress(namePrefix + "BinaryData");
			this._getElement = GetProcAddress(namePrefix + "NthElementValue");
			this._getElementCount = GetProcAddress(namePrefix + "ElementsCount");
			this._getElementKey = GetProcAddress(namePrefix + "NthElementKey");
			this._getFloatData = GetProcAddress(namePrefix + "FloatData");
			this._getIntData = GetProcAddress(namePrefix + "IntData");
			this._getLongData = GetProcAddress(namePrefix + "Int64Data");
			this._getStringData = GetProcAddress(namePrefix + "StringData");
			this._getType = GetProcAddress(namePrefix + "Type");
			this._init = GetProcAddress(namePrefix + "Init");
			this._setBinaryData = GetProcAddress(namePrefix + "BinaryDataSet");
			this._setElement = GetProcAddress(namePrefix + "NthElementValueSet");
			this._setElementValue = GetProcAddress(namePrefix + "SetValueToKey");
			this._setFloatData = GetProcAddress(namePrefix + "FloatDataSet");
			this._setIntData = GetProcAddress(namePrefix + "IntDataSet");
			this._setLongData = GetProcAddress(namePrefix + "Int64DataSet");
			this._setStringData = GetProcAddress(namePrefix + "StringDataSet");
#else
			this._clear = GetProcAddress<ClearHandler>(namePrefix + "Clear");
			this._getBinaryData = GetProcAddress<GetBinaryDataHandler>(namePrefix + "BinaryData");
			this._getElement = GetProcAddress<GetElementHandler>(namePrefix + "NthElementValue");
			this._getElementCount = GetProcAddress<GetElementCountHandler>(namePrefix + "ElementsCount");
			this._getElementKey = GetProcAddress<GetElementKeyHandler>(namePrefix + "NthElementKey");
			this._getFloatData = GetProcAddress<GetFloatDataHandler>(namePrefix + "FloatData");
			this._getIntData = GetProcAddress<GetIntDataHandler>(namePrefix + "IntData");
			this._getLongData = GetProcAddress<GetLongDataHandler>(namePrefix + "Int64Data");
			this._getStringData = GetProcAddress<GetStringDataHandler>(namePrefix + "StringData");
			this._getType = GetProcAddress<GetTypeHandler>(namePrefix + "Type");
			this._init = GetProcAddress<InitHandler>(namePrefix + "Init");
			this._setBinaryData = GetProcAddress<SetBinaryDataHandler>(namePrefix + "BinaryDataSet");
			this._setElement = GetProcAddress<SetElementHandler>(namePrefix + "NthElementValueSet");
			this._setElementValue = GetProcAddress<SetElementValueHandler>(namePrefix + "SetValueToKey");
			this._setFloatData = GetProcAddress<SetFloatDataHandler>(namePrefix + "FloatDataSet");
			this._setIntData = GetProcAddress<SetIntDataHandler>(namePrefix + "IntDataSet");
			this._setLongData = GetProcAddress<SetLongDataHandler>(namePrefix + "Int64DataSet");
			this._setStringData = GetProcAddress<SetStringDataHandler>(namePrefix + "StringDataSet");
#endif
		}

		public int Clear(ref JsonValueData value)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._clear,
						valuePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
			}
#else
			return this._clear(ref value);
#endif
		}

		public int GetBinaryData(ref JsonValueData value, out IntPtr typedValueData, out int typedValueSize)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr typedValueDataPtr = IntPtr.Zero;
			IntPtr typedValueSizePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				typedValueDataPtr = Marshal.AllocHGlobal(IntPtr.Size);
				typedValueSizePtr = Marshal.AllocHGlobal(4);

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getBinaryData,
						valuePtr,
						typedValueDataPtr,
						typedValueSizePtr);

				typedValueData = Marshal.ReadIntPtr(typedValueDataPtr);
				typedValueSize = Marshal.ReadInt32(typedValueSizePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(typedValueDataPtr);
				Marshal.FreeHGlobal(typedValueSizePtr);
			}
#else
			return this._getBinaryData(ref value, out typedValueData, out typedValueSize);
#endif
		}

		public int GetElement(ref JsonValueData value, int index, out JsonValueData elementValue)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr elementValuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				elementValuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getElement,
						valuePtr,
						index,
						elementValuePtr);

				elementValue = (JsonValueData)Marshal.PtrToStructure(elementValuePtr, typeof(JsonValueData));

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(elementValuePtr);
			}
#else
			return this._getElement(ref value, index, out elementValue);
#endif
		}

		public int GetElementCount(ref JsonValueData value, out int elementCount)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr elementCountPtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				elementCountPtr = Marshal.AllocHGlobal(4);

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getElementCount,
						valuePtr,
						elementCountPtr);

				elementCount = Marshal.ReadInt32(elementCountPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(elementCountPtr);
			}
#else
			return this._getElementCount(ref value, out elementCount);
#endif
		}

		public int GetElementKey(ref JsonValueData value, int index, out JsonValueData elementKey)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr elementKeyPtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				elementKeyPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getElementKey,
						valuePtr,
						index,
						elementKeyPtr);

				elementKey = (JsonValueData)Marshal.PtrToStructure(elementKeyPtr, typeof(JsonValueData));

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(elementKeyPtr);
			}
#else
			return this._getElementKey(ref value, index, out elementKey);
#endif
		}

		public int GetFloatData(ref JsonValueData value, out double typedValue)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr typedValuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				typedValuePtr = Marshal.AllocHGlobal(8);

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getFloatData,
						valuePtr,
						typedValuePtr);

				// TODO
				typedValue = 0;

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(typedValuePtr);
			}
#else
			return this._getFloatData(ref value, out typedValue);
#endif
		}

		public int GetIntData(ref JsonValueData value, out int typedValue)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr typedValuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				typedValuePtr = Marshal.AllocHGlobal(4);

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getIntData,
						valuePtr,
						typedValuePtr);

				typedValue = Marshal.ReadInt32(typedValuePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(typedValuePtr);
			}
#else
			return this._getIntData(ref value, out typedValue);
#endif
		}

		public int GetLongData(ref JsonValueData value, out long typedValue)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr typedValuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				typedValuePtr = Marshal.AllocHGlobal(8);

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getLongData,
						valuePtr,
						typedValuePtr);

				typedValue = Marshal.ReadInt64(typedValuePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(typedValuePtr);
			}
#else
			return this._getLongData(ref value, out typedValue);
#endif
		}

		public int GetStringData(ref JsonValueData value, out IntPtr typedValueData, out int typedValueSize)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr typedValueDataPtr = IntPtr.Zero;
			IntPtr typedValueSizePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				typedValueDataPtr = Marshal.AllocHGlobal(IntPtr.Size);
				typedValueSizePtr = Marshal.AllocHGlobal(4);

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getStringData,
						valuePtr,
						typedValueDataPtr,
						typedValueSizePtr);

				typedValueData = Marshal.ReadIntPtr(typedValueDataPtr);
				typedValueSize = Marshal.ReadInt32(typedValueSizePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(typedValueDataPtr);
				Marshal.FreeHGlobal(typedValueSizePtr);
			}
#else
			return this._getStringData(ref value, out typedValueData, out typedValueSize);
#endif
		}

		public int GetType(ref JsonValueData value, out JsonValueType valueType, out JsonValueUnit valueUnit)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr valueTypePtr = IntPtr.Zero;
			IntPtr valueUnitPtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				valueTypePtr = Marshal.AllocHGlobal(4);
				valueUnitPtr = Marshal.AllocHGlobal(4);

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getType,
						valuePtr,
						valueTypePtr,
						valueUnitPtr);

				valueType = (JsonValueType)Marshal.ReadInt32(valueTypePtr);
				valueUnit = (JsonValueUnit)Marshal.ReadInt32(valueUnitPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(valueTypePtr);
				Marshal.FreeHGlobal(valueUnitPtr);
			}
#else
			return this._getType(ref value, out valueType, out valueUnit);
#endif
		}

		public int Init(out JsonValueData value)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._init,
						valuePtr);

				value = (JsonValueData)Marshal.PtrToStructure(valuePtr, typeof(JsonValueData));

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
			}
#else
			return this._init(out value);
#endif
		}

		public int SetBinaryData(ref JsonValueData value, byte[] typedValueData, int typedValueSize, JsonValueType typedValueType, JsonValueUnit typedValueUnit)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr typedValueDataPtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				typedValueDataPtr = Marshal.AllocHGlobal(typedValueSize);

				Marshal.StructureToPtr(value, valuePtr, false);
				Marshal.Copy(typedValueData, 0, typedValueDataPtr, typedValueSize);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setBinaryData,
						valuePtr,
						typedValueData,
						typedValueSize,
						(int)typedValueType,
						(int)typedValueUnit);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(typedValueDataPtr);
			}
#else
			return this._setBinaryData(ref value, typedValueData, typedValueSize, typedValueType, typedValueUnit);
#endif
		}

		public int SetElement(ref JsonValueData value, int index, ref JsonValueData elementValue)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr elementValuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				elementValuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));

				Marshal.StructureToPtr(value, valuePtr, false);
				Marshal.StructureToPtr(elementValue, elementValuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setElement,
						valuePtr,
						index,
						elementValuePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(elementValuePtr);
			}
#else
			return this._setElement(ref value, index, ref elementValue);
#endif
		}

		public int SetElementValue(ref JsonValueData value, ref JsonValueData elementKey, ref JsonValueData elementValue)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr elementKeyPtr = IntPtr.Zero;
			IntPtr elementValuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				elementKeyPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				elementValuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));

				Marshal.StructureToPtr(value, valuePtr, false);
				Marshal.StructureToPtr(elementKey, elementKeyPtr, false);
				Marshal.StructureToPtr(elementValue, elementValuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setElementValue,
						valuePtr,
						elementKeyPtr,
						elementValuePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(elementKeyPtr);
				Marshal.FreeHGlobal(elementValuePtr);
			}
#else
			return this._setElementValue(ref value, ref elementKey, ref elementValue);
#endif
		}

		public int SetFloatData(ref JsonValueData value, double typedValueData, JsonValueType typedValueType, JsonValueUnit typedValueUnit)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setFloatData,
						valuePtr,
						typedValueData,
						(int)typedValueType,
						(int)typedValueUnit);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
			}
#else
			return this._setFloatData(ref value, typedValueData, typedValueType, typedValueUnit);
#endif
		}

		public int SetIntData(ref JsonValueData value, int typedValueData, JsonValueType typedValueType, JsonValueUnit typedValueUnit)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setIntData,
						valuePtr,
						typedValueData,
						(int)typedValueType,
						(int)typedValueUnit);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
			}
#else
			return this._setIntData(ref value, typedValueData, typedValueType, typedValueUnit);
#endif
		}

		public int SetLongData(ref JsonValueData value, long typedValueData, JsonValueType typedValueType, JsonValueUnit typedValueUnit)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setLongData,
						valuePtr,
						typedValueData,
						(int)typedValueType,
						(int)typedValueUnit);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
			}
#else
			return this._setLongData(ref value, typedValueData, typedValueType, typedValueUnit);
#endif
		}

		public int SetStringData(ref JsonValueData value, string typedValueData, int typedValueSize, JsonValueUnit typedValueUnit)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;
			IntPtr typedValueDataPtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));
				typedValueDataPtr = MarshalCF.StringToHGlobalUni(typedValueData);

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setStringData,
						valuePtr,
						typedValueDataPtr,
						typedValueSize,
						(int)typedValueUnit);

				typedValueData = Marshal.PtrToStringUni(typedValueDataPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
				Marshal.FreeHGlobal(typedValueDataPtr);
			}
#else
			return this._setStringData(ref value, typedValueData, typedValueSize, typedValueUnit);
#endif
		}

		#endregion
	}

	internal sealed class DomDynamicInterop : DynamicInterop
	{
		#region Types
#if !COMPACT_FRAMEWORK
		private delegate int AttachEventHandlerHandler([In] IntPtr hTag, [In] DOM_EventHandler callback, [In] int handlerID, [In][MarshalAs(UnmanagedType.U4)] HtmlEventGroup eventGroup);

		private delegate int CallBehaviorMethodHandler([In] IntPtr hTag, [In] IntPtr parameters);

		private delegate int ClearAttributesHandler([In] IntPtr hTag);

		private delegate int CloneTagHandler([In] IntPtr hTagSource, [Out] out IntPtr hTagDestination);

		private delegate int CreateTagHandler([In][MarshalAs(UnmanagedType.LPStr)] string type, [In][MarshalAs(UnmanagedType.LPWStr)] string text, [Out] out IntPtr hTag);

		private delegate int DeclareElementTypeHandler([In][MarshalAs(UnmanagedType.LPStr)] string name, [In] TagLayoutModel model);

		private delegate int DeleteTagHandler([In] IntPtr hTag);

		private delegate int DetachEventHandlerHandler([In] IntPtr hTag, [In] DOM_EventHandler callback, [In] int handlerID);

		private delegate int DetachTagHandler([In] IntPtr hTag);

		private delegate int EnumerateStylesHandler([In] IntPtr hTag, [In] DOM_StyleEnumHandler callback, [In] IntPtr param);

		private delegate int EnumerateSymbolsHandler([In] IntPtr hTag, [In] DOM_SymbolEnumHandler callback, [In] IntPtr param, [In][MarshalAs(UnmanagedType.Bool)] bool direction);

		private delegate int GetAttributeByIndexHandler([In] IntPtr hTag, [In] int index, [Out] out IntPtr namePtr, [Out] out IntPtr valuePtr);

		private delegate int GetAttributeCountHandler([In] IntPtr hTag, [Out] out int attributeCount);

		private delegate int GetAttributeNameByIndexHandler([In] IntPtr hTag, [In] int index, [Out] out IntPtr namePtr, [In] IntPtr zeroPtr);

		private delegate int GetAttributeValueByIndexHandler([In] IntPtr hTag, [In] int index, [In] IntPtr zeroPtr, [Out] out IntPtr valuePtr);

		private delegate int GetAttributeValueByNameHandler([In] IntPtr hTag, [In][MarshalAs(UnmanagedType.LPStr)] string name, [Out] out IntPtr valuePtr);

		private delegate int GetChildrenCountHandler([In] IntPtr hTagParent, [Out] out int childrenCount);

		private delegate int GetChildTagHandler([In] IntPtr hTagParent, [In] int index, [Out] out IntPtr hTagChild);

		private delegate int GetControlTypeHandler([In] IntPtr hTag, [Out][MarshalAs(UnmanagedType.U4)] out HtmlTagControlType controlType);

		private delegate int GetControlValueHandler([In] IntPtr hTag, [Out] out JsonValueData value);

		private delegate int GetParentTagHandler([In] IntPtr hTagChild, [Out] out IntPtr hTagParent);

		private delegate int GetScrollInfoHandler([In] IntPtr hTag, [Out] out Point position, [Out] out Rectangle view, [Out] out Size size);

		private delegate int GetStyleAttributeHandler([In] IntPtr hTag, [In][MarshalAs(UnmanagedType.LPStr)] string name, [Out] out IntPtr valuePtr);

		private delegate int GetSymbolRectHandler([In] IntPtr hTag, [In] int position, [Out] out Rectangle rect);

		private delegate int GetTagExpandoHandler([In] IntPtr hTag, [Out] out IntPtr lpExpando);

		private delegate int GetTagHtmlHandler([In] IntPtr hTag, [Out] out IntPtr html, [In][MarshalAs(UnmanagedType.Bool)] bool outer);

		private delegate int GetTagIndexHandler([In] IntPtr hTag, [Out] out int index);

		private delegate int GetTagInnerText16Handler([In] IntPtr hTag, [Out] out IntPtr textPtr);

		private delegate int GetTagIntrinsicHeightHandler([In] IntPtr hTag, [In] int width, [Out] out int height);

		private delegate int GetTagIntrinsicWidthHandler([In] IntPtr hTag, [Out] out int minWidth, [Out] out int maxWidth);

		private delegate int GetTagLocationHandler([In] IntPtr hTag, [Out] out Rectangle location, [In][MarshalAs(UnmanagedType.U4)] HtmlTagArea areas);

		private delegate int GetTagStateHandler([In] IntPtr hTag, [Out] out int stateBits);

		private delegate int GetTagTypeHandler([In] IntPtr hTag, [Out] out IntPtr typePtr);

		private delegate int GetTagWindowViewHandler([In] IntPtr hTag, [Out] out IntPtr hWindow, [In][MarshalAs(UnmanagedType.Bool)] bool rootWindow);

		private delegate int GetTagWindowlessViewHandler([In] IntPtr hTag, [Out] out IntPtr hWindow);

		private delegate int HidePopupHandler([In] IntPtr hTag);

		private delegate int InsertTagHandler([In] IntPtr hTag, [In] IntPtr hTagParent, [In] int index);

		private delegate int IsEnabledHandler([In] IntPtr hTag, [Out][MarshalAs(UnmanagedType.Bool)] out bool isEnabled);

		private delegate int IsVisibleHandler([In] IntPtr hTag, [Out][MarshalAs(UnmanagedType.Bool)] out bool isVisible);

		private delegate int MoveTagHandler([In] IntPtr hTag, [In] int left, [In] int top, [In] int width, [In] int height);

		private delegate int PostEventHandler([In] IntPtr hTag, [In] int code, [In] IntPtr hTagSource, [In] IntPtr parameter);

		private delegate int ScrollToViewHandler([In] IntPtr hTag, [In][MarshalAs(UnmanagedType.U4)] HtmlScrollBehavior flags);

		private delegate int SelectParentTagHandler([In] IntPtr hTag, [In][MarshalAs(UnmanagedType.LPWStr)] string cssSelector, [In] int depth, [Out] out IntPtr hTagParent);

		private delegate int SelectTagsHandler([In] IntPtr hTag, [In][MarshalAs(UnmanagedType.LPWStr)] string cssSelectors, [In] DOM_TagEnumHandler callback, [In] IntPtr param);

		private delegate int SendEventHandler([In] IntPtr hTag, [In] int code, [In] IntPtr hTagSource, [In] IntPtr parameter, [Out][MarshalAs(UnmanagedType.Bool)] out bool isHandled);

		private delegate int SetAttributeValueByNameHandler([In] IntPtr hTag, [In][MarshalAs(UnmanagedType.LPStr)] string name, [In][MarshalAs(UnmanagedType.LPWStr)] string value);

		private delegate int SetCaptureHandler([In] IntPtr hTag);

		private delegate int SetControlValueHandler([In] IntPtr hTag, [In] ref JsonValueData value);

		private delegate int SetScrollInfoHandler([In] IntPtr hTag, [In] Point position, [In][MarshalAs(UnmanagedType.Bool)] bool smooth);

		private delegate int SetStyleAttributeHandler([In] IntPtr hTag, [In][MarshalAs(UnmanagedType.LPStr)] string name, [In][MarshalAs(UnmanagedType.LPWStr)] string value);

		private delegate int SetTagExpandoHandler([In] IntPtr hTag, [In] IntPtr lpExpando);

		private delegate int SetTagHtmlHandler([In] IntPtr hTag, [In][MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] byte[] html, [In] int htmlLength, [In][MarshalAs(UnmanagedType.U4)] SetTagHtmlBehavior setMethod);

		private delegate int SetTagInnerText16Handler([In] IntPtr hTag, [In][MarshalAs(UnmanagedType.LPWStr)] string text, [In] int textLength);

		private delegate int SetTagStateHandler([In] IntPtr hTag, [In] int bitsToSet, [In] int bitsToClear, [In][MarshalAs(UnmanagedType.Bool)] bool redraw);

		private delegate int ShowPopupHandler([In] IntPtr hTagPopup, [In] IntPtr hTagAnchor, [In][MarshalAs(UnmanagedType.U4)] HtmlPopupAlignment placement);

		private delegate int ShowPopupAtHandler([In] IntPtr hTagPopup, [In] Point position, [In][MarshalAs(UnmanagedType.Bool)] bool isAnimated);

		private delegate int SortChildTagsHandler([In] IntPtr hTag, [In] int firstChildIndex, [In] int lastChildIndex, [In] DOM_TagSortHandler sortHandler, [In] IntPtr param);

		private delegate int SwapTagsHandler([In] IntPtr hTag1, [In] IntPtr hTag2);

		private delegate int TrackPopupAtHandler([In] IntPtr hTagPopup, [In] Point position, [In][MarshalAs(UnmanagedType.Bool)] bool isAnimated, [Out] out IntPtr hTagItem);

		private delegate int UnuseTagHandler([In] IntPtr hTag);

		private delegate int UpdateTagHandler([In] IntPtr hTag, [In][MarshalAs(UnmanagedType.Bool)] bool redraw);

		private delegate int UseTagHandler([In] IntPtr hTag);

#endif
		#endregion

		#region Fields

#if COMPACT_FRAMEWORK
		private IntPtr _attachEventHandler;
		private IntPtr _callBehaviorMethod;
		private IntPtr _clearAttributes;
		private IntPtr _cloneTag;
		private IntPtr _createTag;
		private IntPtr _declareElementType;
		private IntPtr _deleteTag;
		private IntPtr _detachEventHandler;
		private IntPtr _detachTag;
		private IntPtr _enumerateStyles;
		private IntPtr _enumerateSymbols;
		private IntPtr _getAttributeByIndex;
		private IntPtr _getAttributeCount;
		private IntPtr _getAttributeNameByIndex;
		private IntPtr _getAttributeValueByIndex;
		private IntPtr _getAttributeValueByName;
		private IntPtr _getChildrenCount;
		private IntPtr _getChildTag;
		private IntPtr _getControlType;
		private IntPtr _getControlValue;
		private IntPtr _getParentTag;
		private IntPtr _getScrollInfo;
		private IntPtr _getStyleAttribute;
		private IntPtr _getSymbolRect;
		private IntPtr _getTagExpando;
		private IntPtr _getTagHtml;
		private IntPtr _getTagIndex;
		private IntPtr _getTagInnerText16;
		private IntPtr _getTagIntrinsicHeight;
		private IntPtr _getTagIntrinsicWidth;
		private IntPtr _getTagLocation;
		private IntPtr _getTagState;
		private IntPtr _getTagType;
		private IntPtr _getTagWindowView;
		private IntPtr _getTagWindowlessView;
		private IntPtr _hidePopup;
		private IntPtr _insertTag;
		private IntPtr _isEnabled;
		private IntPtr _isVisible;
		private IntPtr _moveTag;
		private IntPtr _postEvent;
		private IntPtr _scrollToView;
		private IntPtr _selectParentTag;
		private IntPtr _selectTags;
		private IntPtr _sendEvent;
		private IntPtr _setAttributeValueByName;
		private IntPtr _setCapture;
		private IntPtr _setControlValue;
		private IntPtr _setScrollInfo;
		private IntPtr _setStyleAttribute;
		private IntPtr _setTagExpando;
		private IntPtr _setTagHtml;
		private IntPtr _setTagInnerText16;
		private IntPtr _setTagState;
		private IntPtr _showPopup;
		private IntPtr _showPopupAt;
		private IntPtr _sortChildTags;
		private IntPtr _swapTags;
		private IntPtr _trackPopupAt;
		private IntPtr _unuseTag;
		private IntPtr _updateTag;
		private IntPtr _useTag;
#else
		private AttachEventHandlerHandler _attachEventHandler;
		private CallBehaviorMethodHandler _callBehaviorMethod;
		private ClearAttributesHandler _clearAttributes;
		private CloneTagHandler _cloneTag;
		private CreateTagHandler _createTag;
		private DeclareElementTypeHandler _declareElementType;
		private DeleteTagHandler _deleteTag;
		private DetachEventHandlerHandler _detachEventHandler;
		private DetachTagHandler _detachTag;
		private EnumerateStylesHandler _enumerateStyles;
		private EnumerateSymbolsHandler _enumerateSymbols;
		private GetAttributeByIndexHandler _getAttributeByIndex;
		private GetAttributeCountHandler _getAttributeCount;
		private GetAttributeNameByIndexHandler _getAttributeNameByIndex;
		private GetAttributeValueByIndexHandler _getAttributeValueByIndex;
		private GetAttributeValueByNameHandler _getAttributeValueByName;
		private GetChildrenCountHandler _getChildrenCount;
		private GetChildTagHandler _getChildTag;
		private GetControlTypeHandler _getControlType;
		private GetControlValueHandler _getControlValue;
		private GetParentTagHandler _getParentTag;
		private GetScrollInfoHandler _getScrollInfo;
		private GetStyleAttributeHandler _getStyleAttribute;
		private GetSymbolRectHandler _getSymbolRect;
		private GetTagExpandoHandler _getTagExpando;
		private GetTagHtmlHandler _getTagHtml;
		private GetTagIndexHandler _getTagIndex;
		private GetTagInnerText16Handler _getTagInnerText16;
		private GetTagIntrinsicHeightHandler _getTagIntrinsicHeight;
		private GetTagIntrinsicWidthHandler _getTagIntrinsicWidth;
		private GetTagLocationHandler _getTagLocation;
		private GetTagStateHandler _getTagState;
		private GetTagTypeHandler _getTagType;
		private GetTagWindowViewHandler _getTagWindowView;
		private GetTagWindowlessViewHandler _getTagWindowlessView;
		private HidePopupHandler _hidePopup;
		private InsertTagHandler _insertTag;
		private IsEnabledHandler _isEnabled;
		private IsVisibleHandler _isVisible;
		private MoveTagHandler _moveTag;
		private PostEventHandler _postEvent;
		private ScrollToViewHandler _scrollToView;
		private SelectParentTagHandler _selectParentTag;
		private SelectTagsHandler _selectTags;
		private SendEventHandler _sendEvent;
		private SetAttributeValueByNameHandler _setAttributeValueByName;
		private SetCaptureHandler _setCapture;
		private SetControlValueHandler _setControlValue;
		private SetScrollInfoHandler _setScrollInfo;
		private SetStyleAttributeHandler _setStyleAttribute;
		private SetTagExpandoHandler _setTagExpando;
		private SetTagHtmlHandler _setTagHtml;
		private SetTagInnerText16Handler _setTagInnerText16;
		private SetTagStateHandler _setTagState;
		private ShowPopupHandler _showPopup;
		private ShowPopupAtHandler _showPopupAt;
		private SortChildTagsHandler _sortChildTags;
		private SwapTagsHandler _swapTags;
		private TrackPopupAtHandler _trackPopupAt;
		private UnuseTagHandler _unuseTag;
		private UpdateTagHandler _updateTag;
		private UseTagHandler _useTag;
#endif

		#endregion

		#region Methods

		public DomDynamicInterop(IntPtr hModule, string namePrefix)
			: base(hModule)
		{
#if COMPACT_FRAMEWORK
			this._attachEventHandler = GetProcAddress(namePrefix + "AttachEventHandlerEx");
			this._callBehaviorMethod = GetProcAddress(namePrefix + "CallBehaviorMethod");
			this._clearAttributes = GetProcAddress(namePrefix + "ClearAttributes");
			this._cloneTag = GetProcAddress(namePrefix + "CloneElement");
			this._createTag = GetProcAddress(namePrefix + "CreateElement");
			this._declareElementType = GetProcAddress(namePrefix + "DeclareElementType");
			this._deleteTag = GetProcAddress(namePrefix + "DeleteElement");
			this._detachEventHandler = GetProcAddress(namePrefix + "DetachEventHandler");
			this._detachTag = GetProcAddress(namePrefix + "DetachElement");
			this._enumerateStyles = GetProcAddress(namePrefix + "EnumElementStyles");
			this._enumerateSymbols = GetProcAddress(namePrefix + "Enumerate");
			this._getAttributeByIndex = GetProcAddress(namePrefix + "GetNthAttribute");
			this._getAttributeCount = GetProcAddress(namePrefix + "GetAttributeCount");
			this._getAttributeNameByIndex = GetProcAddress(namePrefix + "GetNthAttribute");
			this._getAttributeValueByIndex = GetProcAddress(namePrefix + "GetNthAttribute");
			this._getAttributeValueByName = GetProcAddress(namePrefix + "GetAttributeByName");
			this._getChildrenCount = GetProcAddress(namePrefix + "GetChildrenCount");
			this._getChildTag = GetProcAddress(namePrefix + "GetNthChild");
			this._getControlType = GetProcAddress(namePrefix + "ControlGetType");
			this._getControlValue = GetProcAddress(namePrefix + "ControlGetValue");
			this._getParentTag = GetProcAddress(namePrefix + "GetParentElement");
			this._getScrollInfo = GetProcAddress(namePrefix + "GetScrollInfo");
			this._getStyleAttribute = GetProcAddress(namePrefix + "GetStyleAttribute");
			this._getSymbolRect = GetProcAddress(namePrefix + "GetCharacterRect");
			this._getTagExpando = GetProcAddress(namePrefix + "ElementGetExpando");
			this._getTagHtml = GetProcAddress(namePrefix + "GetElementHtml");
			this._getTagIndex = GetProcAddress(namePrefix + "GetElementIndex");
			this._getTagInnerText16 = GetProcAddress(namePrefix + "GetElementInnerText16");
			this._getTagIntrinsicHeight = GetProcAddress(namePrefix + "GetElementIntrinsicHeight");
			this._getTagIntrinsicWidth = GetProcAddress(namePrefix + "GetElementIntrinsicWidths");
			this._getTagLocation = GetProcAddress(namePrefix + "GetElementLocation");
			this._getTagState = GetProcAddress(namePrefix + "GetElementState");
			this._getTagType = GetProcAddress(namePrefix + "GetElementType");
			this._getTagWindowView = GetProcAddress("HTMLayoutGetElementHwnd");
			this._getTagWindowlessView = GetProcAddress("HTMLiteGetElementHTMLITE");
			this._hidePopup = GetProcAddress(namePrefix + "HidePopup");
			this._insertTag = GetProcAddress(namePrefix + "InsertElement");
			this._isEnabled = GetProcAddress(namePrefix + "IsElementEnabled");
			this._isVisible = GetProcAddress(namePrefix + "IsElementVisible");
			this._moveTag = GetProcAddress(namePrefix + "MoveElementEx");
			this._postEvent = GetProcAddress(namePrefix + "PostEvent");
			this._scrollToView = GetProcAddress(namePrefix + "ScrollToView");
			this._selectParentTag = GetProcAddress(namePrefix + "SelectParentW");
			this._selectTags = GetProcAddress(namePrefix + "SelectElementsW");
			this._sendEvent = GetProcAddress(namePrefix + "SendEvent");
			this._setAttributeValueByName = GetProcAddress(namePrefix + "SetAttributeByName");
			this._setCapture = GetProcAddress(namePrefix + "SetCapture");
			this._setControlValue = GetProcAddress(namePrefix + "ControlSetValue");
			this._setScrollInfo = GetProcAddress(namePrefix + "SetScrollPos");
			this._setStyleAttribute = GetProcAddress(namePrefix + "SetStyleAttribute");
			this._setTagExpando = GetProcAddress(namePrefix + "ElementSetExpando");
			this._setTagHtml = GetProcAddress(namePrefix + "SetElementHtml");
			this._setTagInnerText16 = GetProcAddress(namePrefix + "SetElementInnerText16");
			this._setTagState = GetProcAddress(namePrefix + "SetElementState");
			this._showPopup = GetProcAddress(namePrefix + "ShowPopup");
			this._showPopupAt = GetProcAddress(namePrefix + "ShowPopupAt");
			this._sortChildTags = GetProcAddress(namePrefix + "SortElements");
			this._swapTags = GetProcAddress(namePrefix + "SwapElements");
			this._trackPopupAt = GetProcAddress(namePrefix + "TrackPopupAt");
			this._unuseTag = GetProcAddress(namePrefix + "_UnuseElement");
			this._updateTag = GetProcAddress(namePrefix + "UpdateElement");
			this._useTag = GetProcAddress(namePrefix + "_UseElement");
#else
			this._attachEventHandler = GetProcAddress<AttachEventHandlerHandler>(namePrefix + "AttachEventHandlerEx");
			this._callBehaviorMethod = GetProcAddress<CallBehaviorMethodHandler>(namePrefix + "CallBehaviorMethod");
			this._clearAttributes = GetProcAddress<ClearAttributesHandler>(namePrefix + "ClearAttributes");
			this._cloneTag = GetProcAddress<CloneTagHandler>(namePrefix + "CloneElement");
			this._createTag = GetProcAddress<CreateTagHandler>(namePrefix + "CreateElement");
			this._declareElementType = GetProcAddress<DeclareElementTypeHandler>(namePrefix + "DeclareElementType");
			this._deleteTag = GetProcAddress<DeleteTagHandler>(namePrefix + "DeleteElement");
			this._detachEventHandler = GetProcAddress<DetachEventHandlerHandler>(namePrefix + "DetachEventHandler");
			this._detachTag = GetProcAddress<DetachTagHandler>(namePrefix + "DetachElement");
			this._enumerateStyles = GetProcAddress<EnumerateStylesHandler>(namePrefix + "EnumElementStyles");
			this._enumerateSymbols = GetProcAddress<EnumerateSymbolsHandler>(namePrefix + "Enumerate");
			this._getAttributeByIndex = GetProcAddress<GetAttributeByIndexHandler>(namePrefix + "GetNthAttribute");
			this._getAttributeCount = GetProcAddress<GetAttributeCountHandler>(namePrefix + "GetAttributeCount");
			this._getAttributeNameByIndex = GetProcAddress<GetAttributeNameByIndexHandler>(namePrefix + "GetNthAttribute");
			this._getAttributeValueByIndex = GetProcAddress<GetAttributeValueByIndexHandler>(namePrefix + "GetNthAttribute");
			this._getAttributeValueByName = GetProcAddress<GetAttributeValueByNameHandler>(namePrefix + "GetAttributeByName");
			this._getChildrenCount = GetProcAddress<GetChildrenCountHandler>(namePrefix + "GetChildrenCount");
			this._getChildTag = GetProcAddress<GetChildTagHandler>(namePrefix + "GetNthChild");
			this._getControlType = GetProcAddress<GetControlTypeHandler>(namePrefix + "ControlGetType");
			this._getControlValue = GetProcAddress<GetControlValueHandler>(namePrefix + "ControlGetValue");
			this._getParentTag = GetProcAddress<GetParentTagHandler>(namePrefix + "GetParentElement");
			this._getScrollInfo = GetProcAddress<GetScrollInfoHandler>(namePrefix + "GetScrollInfo");
			this._getStyleAttribute = GetProcAddress<GetStyleAttributeHandler>(namePrefix + "GetStyleAttribute");
			this._getSymbolRect = GetProcAddress<GetSymbolRectHandler>(namePrefix + "GetCharacterRect");
			this._getTagExpando = GetProcAddress<GetTagExpandoHandler>(namePrefix + "ElementGetExpando");
			this._getTagHtml = GetProcAddress<GetTagHtmlHandler>(namePrefix + "GetElementHtml");
			this._getTagIndex = GetProcAddress<GetTagIndexHandler>(namePrefix + "GetElementIndex");
			this._getTagInnerText16 = GetProcAddress<GetTagInnerText16Handler>(namePrefix + "GetElementInnerText16");
			this._getTagIntrinsicHeight = GetProcAddress<GetTagIntrinsicHeightHandler>(namePrefix + "GetElementIntrinsicHeight");
			this._getTagIntrinsicWidth = GetProcAddress<GetTagIntrinsicWidthHandler>(namePrefix + "GetElementIntrinsicWidths");
			this._getTagLocation = GetProcAddress<GetTagLocationHandler>(namePrefix + "GetElementLocation");
			this._getTagState = GetProcAddress<GetTagStateHandler>(namePrefix + "GetElementState");
			this._getTagType = GetProcAddress<GetTagTypeHandler>(namePrefix + "GetElementType");
			this._getTagWindowView = GetProcAddress<GetTagWindowViewHandler>("HTMLayoutGetElementHwnd");
			this._getTagWindowlessView = GetProcAddress<GetTagWindowlessViewHandler>("HTMLiteGetElementHTMLITE");
			this._hidePopup = GetProcAddress<HidePopupHandler>(namePrefix + "HidePopup");
			this._insertTag = GetProcAddress<InsertTagHandler>(namePrefix + "InsertElement");
			this._isEnabled = GetProcAddress<IsEnabledHandler>(namePrefix + "IsElementEnabled");
			this._isVisible = GetProcAddress<IsVisibleHandler>(namePrefix + "IsElementVisible");
			this._moveTag = GetProcAddress<MoveTagHandler>(namePrefix + "MoveElementEx");
			this._postEvent = GetProcAddress<PostEventHandler>(namePrefix + "PostEvent");
			this._scrollToView = GetProcAddress<ScrollToViewHandler>(namePrefix + "ScrollToView");
			this._selectParentTag = GetProcAddress<SelectParentTagHandler>(namePrefix + "SelectParentW");
			this._selectTags = GetProcAddress<SelectTagsHandler>(namePrefix + "SelectElementsW");
			this._sendEvent = GetProcAddress<SendEventHandler>(namePrefix + "SendEvent");
			this._setAttributeValueByName = GetProcAddress<SetAttributeValueByNameHandler>(namePrefix + "SetAttributeByName");
			this._setCapture = GetProcAddress<SetCaptureHandler>(namePrefix + "SetCapture");
			this._setControlValue = GetProcAddress<SetControlValueHandler>(namePrefix + "ControlSetValue");
			this._setScrollInfo = GetProcAddress<SetScrollInfoHandler>(namePrefix + "SetScrollPos");
			this._setStyleAttribute = GetProcAddress<SetStyleAttributeHandler>(namePrefix + "SetStyleAttribute");
			this._setTagExpando = GetProcAddress<SetTagExpandoHandler>(namePrefix + "ElementSetExpando");
			this._setTagHtml = GetProcAddress<SetTagHtmlHandler>(namePrefix + "SetElementHtml");
			this._setTagInnerText16 = GetProcAddress<SetTagInnerText16Handler>(namePrefix + "SetElementInnerText16");
			this._setTagState = GetProcAddress<SetTagStateHandler>(namePrefix + "SetElementState");
			this._showPopup = GetProcAddress<ShowPopupHandler>(namePrefix + "ShowPopup");
			this._showPopupAt = GetProcAddress<ShowPopupAtHandler>(namePrefix + "ShowPopupAt");
			this._sortChildTags = GetProcAddress<SortChildTagsHandler>(namePrefix + "SortElements");
			this._swapTags = GetProcAddress<SwapTagsHandler>(namePrefix + "SwapElements");
			this._trackPopupAt = GetProcAddress<TrackPopupAtHandler>(namePrefix + "TrackPopupAt");
			this._unuseTag = GetProcAddress<UnuseTagHandler>(namePrefix + "_UnuseElement");
			this._updateTag = GetProcAddress<UpdateTagHandler>(namePrefix + "UpdateElement");
			this._useTag = GetProcAddress<UseTagHandler>(namePrefix + "_UseElement");
#endif
		}

		public int AttachEventHandler(IntPtr hTag, DOM_EventHandler callback, int handlerID, HtmlEventGroup eventGroup)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._attachEventHandler,
					hTag,
					Marshal.GetFunctionPointerForDelegate(callback),
					handlerID,
					(int)eventGroup);

			return result.ToInt32();
#else
			return this._attachEventHandler(hTag, callback, handlerID, eventGroup);
#endif
		}

		public int CallBehaviorMethod(IntPtr hTag, IntPtr parameters)
		{
#if COMPACT_FRAMEWORK

			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._callBehaviorMethod,
					hTag,
					parameters);

			return result.ToInt32();
#else
			return this._callBehaviorMethod(hTag, parameters);
#endif
		}

		public int ClearAttributes(IntPtr hTag)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._clearAttributes,
					hTag);

			return result.ToInt32();
#else
			return this._clearAttributes(hTag);
#endif
		}

		public int CloneTag(IntPtr hTagSource, out IntPtr hTagDestination)
		{
#if COMPACT_FRAMEWORK
			IntPtr hTagDestinationPtr = IntPtr.Zero;

			try
			{
				hTagDestinationPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._cloneTag,
						hTagSource,
						hTagDestinationPtr);

				hTagDestination = Marshal.ReadIntPtr(hTagDestinationPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(hTagDestinationPtr);
			}
#else
			return this._cloneTag(hTagSource, out hTagDestination);
#endif
		}

		public int CreateTag(string type, string text, out IntPtr hTag)
		{
#if COMPACT_FRAMEWORK
			IntPtr typePtr = IntPtr.Zero;
			IntPtr textPtr = IntPtr.Zero;
			IntPtr hTagPtr = IntPtr.Zero;

			try
			{
				typePtr = MarshalCF.StringToHGlobalUni(type);
				textPtr = MarshalCF.StringToHGlobalUni(text);
				hTagPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._createTag,
						typePtr,
						textPtr,
						hTagPtr);

				type = Marshal.PtrToStringUni(typePtr);
				text = Marshal.PtrToStringUni(textPtr);
				hTag = Marshal.ReadIntPtr(hTagPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(typePtr);
				Marshal.FreeHGlobal(textPtr);
				Marshal.FreeHGlobal(hTagPtr);
			}
#else
			return this._createTag(type, text, out hTag);
#endif
		}

		public int DeclareElementType(string name, TagLayoutModel model)
		{
#if COMPACT_FRAMEWORK
			IntPtr namePtr = IntPtr.Zero;

			try
			{
				namePtr = MarshalCF.StringToHGlobalUni(name);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._declareElementType,
						namePtr,
						(int)model);

				name = Marshal.PtrToStringUni(namePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(namePtr);
			}
#else
			return this._declareElementType(name, model);
#endif
		}

		public int DeleteTag(IntPtr hTag)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._deleteTag,
					hTag);

			return result.ToInt32();
#else
			return this._deleteTag(hTag);
#endif
		}

		public int DetachEventHandler(IntPtr hTag, DOM_EventHandler callback, int handlerID)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._detachEventHandler,
					hTag,
					Marshal.GetFunctionPointerForDelegate(callback),
					handlerID);

			return result.ToInt32();
#else
			return this._detachEventHandler(hTag, callback, handlerID);
#endif
		}

		public int DetachTag(IntPtr hTag)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._detachTag,
					hTag);

			return result.ToInt32();
#else
			return this._detachTag(hTag);
#endif
		}

		public int EnumerateStyles(IntPtr hTag, DOM_StyleEnumHandler callback, IntPtr param)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._enumerateStyles,
					hTag,
					Marshal.GetFunctionPointerForDelegate(callback),
					param);

			return result.ToInt32();
#else
			return this._enumerateStyles(hTag, callback, param);
#endif
		}

		public int EnumerateSymbols(IntPtr hTag, DOM_SymbolEnumHandler callback, IntPtr param, bool direction)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._enumerateSymbols,
					hTag,
					Marshal.GetFunctionPointerForDelegate(callback),
					param,
					direction);

			return result.ToInt32();
#else
			return this._enumerateSymbols(hTag, callback, param, direction);
#endif
		}

		public int GetAttributeByIndex(IntPtr hTag, int index, out IntPtr namePtr, out IntPtr valuePtr)
		{
#if COMPACT_FRAMEWORK
			IntPtr namePtrPtr = IntPtr.Zero;
			IntPtr valuePtrPtr = IntPtr.Zero;

			try
			{
				namePtrPtr = Marshal.AllocHGlobal(IntPtr.Size);
				valuePtrPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getAttributeByIndex,
						hTag,
						index,
						namePtrPtr,
						valuePtrPtr);

				namePtr = Marshal.ReadIntPtr(namePtrPtr);
				valuePtr = Marshal.ReadIntPtr(valuePtrPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(namePtrPtr);
				Marshal.FreeHGlobal(valuePtrPtr);
			}
#else
			return this._getAttributeByIndex(hTag, index, out namePtr, out valuePtr);
#endif
		}

		public int GetAttributeCount(IntPtr hTag, out int attributeCount)
		{
#if COMPACT_FRAMEWORK
			IntPtr attributeCountPtr = IntPtr.Zero;

			try
			{
				attributeCountPtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getAttributeCount,
						hTag,
						attributeCountPtr);

				attributeCount = Marshal.ReadInt32(attributeCountPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(attributeCountPtr);
			}
#else
			return this._getAttributeCount(hTag, out attributeCount);
#endif
		}

		public int GetAttributeNameByIndex(IntPtr hTag, int index, out IntPtr namePtr, IntPtr zeroPtr)
		{
#if COMPACT_FRAMEWORK
			IntPtr namePtrPtr = IntPtr.Zero;

			try
			{
				namePtrPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getAttributeNameByIndex,
						hTag,
						index,
						namePtrPtr,
						zeroPtr);

				namePtr = Marshal.ReadIntPtr(namePtrPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(namePtrPtr);
			}
#else
			return this._getAttributeNameByIndex(hTag, index, out namePtr, zeroPtr);
#endif
		}

		public int GetAttributeValueByIndex(IntPtr hTag, int index, IntPtr zeroPtr, out IntPtr valuePtr)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtrPtr = IntPtr.Zero;

			try
			{
				valuePtrPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getAttributeValueByIndex,
						hTag,
						index,
						zeroPtr,
						valuePtrPtr);

				valuePtr = Marshal.ReadIntPtr(valuePtrPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtrPtr);
			}
#else
			return this._getAttributeValueByIndex(hTag, index, zeroPtr, out valuePtr);
#endif
		}

		public int GetAttributeValueByName(IntPtr hTag, string name, out IntPtr valuePtr)
		{
#if COMPACT_FRAMEWORK
			IntPtr namePtr = IntPtr.Zero;
			IntPtr valuePtrPtr = IntPtr.Zero;

			try
			{
				namePtr = MarshalCF.StringToHGlobalUni(name);
				valuePtrPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getAttributeValueByName,
						hTag,
						namePtr,
						valuePtrPtr);

				name = Marshal.PtrToStringUni(namePtr);
				valuePtr = Marshal.ReadIntPtr(valuePtrPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(namePtr);
				Marshal.FreeHGlobal(valuePtrPtr);
			}
#else
			return this._getAttributeValueByName(hTag, name, out valuePtr);
#endif
		}

		public int GetChildrenCount(IntPtr hTagParent, out int childrenCount)
		{
#if COMPACT_FRAMEWORK
			IntPtr childrenCountPtr = IntPtr.Zero;

			try
			{
				childrenCountPtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getChildrenCount,
						hTagParent,
						childrenCountPtr);

				childrenCount = Marshal.ReadInt32(childrenCountPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(childrenCountPtr);
			}
#else
			return this._getChildrenCount(hTagParent, out childrenCount);
#endif
		}

		public int GetChildTag(IntPtr hTagParent, int index, out IntPtr hTagChild)
		{
#if COMPACT_FRAMEWORK
			IntPtr hTagChildPtr = IntPtr.Zero;

			try
			{
				hTagChildPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getChildTag,
						hTagParent,
						index,
						hTagChildPtr);

				hTagChild = Marshal.ReadIntPtr(hTagChildPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(hTagChildPtr);
			}
#else
			return this._getChildTag(hTagParent, index, out hTagChild);
#endif
		}

		public int GetControlType(IntPtr hTag, out HtmlTagControlType controlType)
		{
#if COMPACT_FRAMEWORK
			IntPtr controlTypePtr = IntPtr.Zero;

			try
			{
				controlTypePtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getControlType,
						hTag,
						controlTypePtr);

				controlType = (HtmlTagControlType)Marshal.ReadInt32(controlTypePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(controlTypePtr);
			}
#else
			return this._getControlType(hTag, out controlType);
#endif
		}

		public int GetControlValue(IntPtr hTag, out JsonValueData value)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getControlValue,
						hTag,
						valuePtr);

				value = (JsonValueData)Marshal.PtrToStructure(valuePtr, typeof(JsonValueData));

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
			}
#else
			return this._getControlValue(hTag, out value);
#endif
		}

		public int GetParentTag(IntPtr hTagChild, out IntPtr hTagParent)
		{
#if COMPACT_FRAMEWORK
			IntPtr hTagParentPtr = IntPtr.Zero;

			try
			{
				hTagParentPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getParentTag,
						hTagChild,
						hTagParentPtr);

				hTagParent = Marshal.ReadIntPtr(hTagParentPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(hTagParentPtr);
			}
#else
			return this._getParentTag(hTagChild, out hTagParent);
#endif
		}

		public int GetScrollInfo(IntPtr hTag, out Point position, out Rectangle view, out Size size)
		{
#if COMPACT_FRAMEWORK
			IntPtr positionPtr = IntPtr.Zero;
			IntPtr viewPtr = IntPtr.Zero;
			IntPtr sizePtr = IntPtr.Zero;

			try
			{
				positionPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(Point)));
				viewPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(Rectangle)));
				sizePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(Size)));

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getScrollInfo,
						hTag,
						positionPtr,
						viewPtr,
						sizePtr);

				position = (Point)Marshal.PtrToStructure(positionPtr, typeof(Point));
				view = (Rectangle)Marshal.PtrToStructure(viewPtr, typeof(Rectangle));
				size = (Size)Marshal.PtrToStructure(sizePtr, typeof(Size));

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(positionPtr);
				Marshal.FreeHGlobal(viewPtr);
				Marshal.FreeHGlobal(sizePtr);
			}
#else
			return this._getScrollInfo(hTag, out position, out view, out size);
#endif
		}

		public int GetStyleAttribute(IntPtr hTag, string name, out IntPtr valuePtr)
		{
#if COMPACT_FRAMEWORK
			IntPtr namePtr = IntPtr.Zero;
			IntPtr valuePtrPtr = IntPtr.Zero;

			try
			{
				namePtr = MarshalCF.StringToHGlobalUni(name);
				valuePtrPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getStyleAttribute,
						hTag,
						namePtr,
						valuePtrPtr);

				name = Marshal.PtrToStringUni(namePtr);
				valuePtr = Marshal.ReadIntPtr(valuePtrPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(namePtr);
				Marshal.FreeHGlobal(valuePtrPtr);
			}
#else
			return this._getStyleAttribute(hTag, name, out valuePtr);
#endif
		}

		public int GetSymbolRect(IntPtr hTag, int position, out Rectangle rect)
		{
#if COMPACT_FRAMEWORK
			IntPtr rectPtr = IntPtr.Zero;

			try
			{
				rectPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(Rectangle)));

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getSymbolRect,
						hTag,
						position,
						rectPtr);

				rect = (Rectangle)Marshal.PtrToStructure(rectPtr, typeof(Rectangle));

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(rectPtr);
			}
#else
			return this._getSymbolRect(hTag, position, out rect);
#endif
		}

		public int GetTagExpando(IntPtr hTag, out IntPtr lpExpando)
		{
#if COMPACT_FRAMEWORK
			IntPtr lpExpandoPtr = IntPtr.Zero;

			try
			{
				lpExpandoPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getTagExpando,
						hTag,
						lpExpandoPtr);

				lpExpando = Marshal.ReadIntPtr(lpExpandoPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(lpExpandoPtr);
			}
#else
			return this._getTagExpando(hTag, out lpExpando);
#endif
		}

		public int GetTagHtml(IntPtr hTag, out IntPtr html, bool outer)
		{
#if COMPACT_FRAMEWORK
			IntPtr htmlPtr = IntPtr.Zero;

			try
			{
				htmlPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getTagHtml,
						hTag,
						htmlPtr,
						outer);

				html = Marshal.ReadIntPtr(htmlPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(htmlPtr);
			}
#else
			return this._getTagHtml(hTag, out html, outer);
#endif
		}

		public int GetTagWindowView(IntPtr hTag, out IntPtr hWindow, bool rootWindow)
		{
#if COMPACT_FRAMEWORK
			IntPtr hWindowPtr = IntPtr.Zero;

			try
			{
				hWindowPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getTagWindowView,
						hTag,
						hWindowPtr,
						rootWindow);

				hWindow = Marshal.ReadIntPtr(hWindowPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(hWindowPtr);
			}
#else
			return this._getTagWindowView(hTag, out hWindow, rootWindow);
#endif
		}

		public int GetTagWindowlessView(IntPtr hTag, out IntPtr hWindow)
		{
#if COMPACT_FRAMEWORK
			IntPtr hWindowPtr = IntPtr.Zero;

			try
			{
				hWindowPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getTagWindowlessView,
						hTag,
						hWindowPtr);

				hWindow = Marshal.ReadIntPtr(hWindowPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(hWindowPtr);
			}
#else
			return this._getTagWindowlessView(hTag, out hWindow);
#endif
		}

		public int GetTagIndex(IntPtr hTag, out int index)
		{
#if COMPACT_FRAMEWORK
			IntPtr indexPtr = IntPtr.Zero;

			try
			{
				indexPtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getTagIndex,
						hTag,
						indexPtr);

				index = Marshal.ReadInt32(indexPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(indexPtr);
			}
#else
			return this._getTagIndex(hTag, out index);
#endif
		}

		public int GetTagInnerText16(IntPtr hTag, out IntPtr textPtr)
		{
#if COMPACT_FRAMEWORK
			IntPtr textPtrPtr = IntPtr.Zero;

			try
			{
				textPtrPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getTagInnerText16,
						hTag,
						textPtrPtr);

				textPtr = Marshal.ReadIntPtr(textPtrPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(textPtrPtr);
			}
#else
			return this._getTagInnerText16(hTag, out textPtr);
#endif
		}

		public int GetTagIntrinsicHeight(IntPtr hTag, int width, out int height)
		{
#if COMPACT_FRAMEWORK
			IntPtr heightPtr = IntPtr.Zero;

			try
			{
				heightPtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getTagIntrinsicHeight,
						hTag,
						width,
						heightPtr);

				height = Marshal.ReadInt32(heightPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(heightPtr);
			}
#else
			return this._getTagIntrinsicHeight(hTag, width, out height);
#endif
		}

		public int GetTagIntrinsicWidth(IntPtr hTag, out int minWidth, out int maxWidth)
		{
#if COMPACT_FRAMEWORK
			IntPtr minWidthPtr = IntPtr.Zero;
			IntPtr maxWidthPtr = IntPtr.Zero;

			try
			{
				minWidthPtr = Marshal.AllocHGlobal(4);
				maxWidthPtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getTagIntrinsicWidth,
						hTag,
						minWidthPtr,
						maxWidthPtr);

				minWidth = Marshal.ReadInt32(minWidthPtr);
				maxWidth = Marshal.ReadInt32(maxWidthPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(minWidthPtr);
				Marshal.FreeHGlobal(maxWidthPtr);
			}
#else
			return this._getTagIntrinsicWidth(hTag, out minWidth, out maxWidth);
#endif
		}

		public int GetTagLocation(IntPtr hTag, out Rectangle location, HtmlTagArea areas)
		{
#if COMPACT_FRAMEWORK
			IntPtr locationPtr = IntPtr.Zero;

			try
			{
				locationPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(Rectangle)));

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getTagLocation,
						hTag,
						locationPtr,
						(int)areas);

				location = (Rectangle)Marshal.PtrToStructure(locationPtr, typeof(Rectangle));

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(locationPtr);
			}
#else
			return this._getTagLocation(hTag, out location, areas);
#endif
		}

		public int GetTagState(IntPtr hTag, out int stateBits)
		{
#if COMPACT_FRAMEWORK
			IntPtr stateBitsPtr = IntPtr.Zero;

			try
			{
				stateBitsPtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getTagState,
						hTag,
						stateBitsPtr);

				stateBits = Marshal.ReadInt32(stateBitsPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(stateBitsPtr);
			}
#else
			return this._getTagState(hTag, out stateBits);
#endif
		}

		public int GetTagType(IntPtr hTag, out IntPtr typePtr)
		{
#if COMPACT_FRAMEWORK
			IntPtr typePtrPtr = IntPtr.Zero;

			try
			{
				typePtrPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getTagType,
						hTag,
						typePtrPtr);

				typePtr = Marshal.ReadIntPtr(typePtrPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(typePtrPtr);
			}
#else
			return this._getTagType(hTag, out typePtr);
#endif
		}

		public int HidePopup(IntPtr hTag)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._hidePopup,
					hTag);

			return result.ToInt32();
#else
			return this._hidePopup(hTag);
#endif
		}

		public int InsertTag(IntPtr hTag, IntPtr hTagParent, int index)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._insertTag,
					hTag,
					hTagParent,
					index);

			return result.ToInt32();
#else
			return this._insertTag(hTag, hTagParent, index);
#endif
		}

		public int IsEnabled(IntPtr hTag, out bool isEnabled)
		{
#if COMPACT_FRAMEWORK
			IntPtr isEnabledPtr = IntPtr.Zero;

			try
			{
				isEnabledPtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._isEnabled,
						hTag,
						isEnabledPtr);

				isEnabled = Marshal.ReadInt32(isEnabledPtr) != 0;

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(isEnabledPtr);
			}
#else
			return this._isEnabled(hTag, out isEnabled);
#endif
		}

		public int IsVisible(IntPtr hTag, out bool isVisible)
		{
#if COMPACT_FRAMEWORK
			IntPtr isVisiblePtr = IntPtr.Zero;

			try
			{
				isVisiblePtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._isVisible,
						hTag,
						isVisiblePtr);

				isVisible = Marshal.ReadInt32(isVisiblePtr) != 0;

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(isVisiblePtr);
			}
#else
			return this._isVisible(hTag, out isVisible);
#endif
		}

		public int MoveTag(IntPtr hTag, int left, int top, int width, int height)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._moveTag,
					hTag,
					left,
					top,
					width,
					height);

			return result.ToInt32();
#else
			return this._moveTag(hTag, left, top, width, height);
#endif
		}

		public int PostEvent(IntPtr hTag, int code, IntPtr hTagSource, IntPtr parameter)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._postEvent,
					hTag,
					code,
					hTagSource,
					parameter);

			return result.ToInt32();
#else
			return this._postEvent(hTag, code, hTagSource, parameter);
#endif
		}

		public int ScrollToView(IntPtr hTag, HtmlScrollBehavior flags)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._scrollToView,
					hTag,
					(int)flags);

			return result.ToInt32();
#else
			return this._scrollToView(hTag, flags);
#endif
		}

		public int SelectParentTag(IntPtr hTag, string cssSelector, int depth, out IntPtr hTagParent)
		{
#if COMPACT_FRAMEWORK
			IntPtr cssSelectorPtr = IntPtr.Zero;
			IntPtr hTagParentPtr = IntPtr.Zero;

			try
			{
				cssSelectorPtr = MarshalCF.StringToHGlobalUni(cssSelector);
				hTagParentPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._selectParentTag,
						hTag,
						cssSelectorPtr,
						depth,
						hTagParentPtr);

				cssSelector = Marshal.PtrToStringUni(cssSelectorPtr);
				hTagParent = Marshal.ReadIntPtr(hTagParentPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(cssSelectorPtr);
				Marshal.FreeHGlobal(hTagParentPtr);
			}
#else
			return this._selectParentTag(hTag, cssSelector, depth, out hTagParent);
#endif
		}

		public int SelectTags(IntPtr hTag, string cssSelectors, DOM_TagEnumHandler callback, IntPtr param)
		{
#if COMPACT_FRAMEWORK
			IntPtr cssSelectorsPtr = IntPtr.Zero;

			try
			{
				cssSelectorsPtr = MarshalCF.StringToHGlobalUni(cssSelectors);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._selectTags,
						hTag,
						cssSelectorsPtr,
						Marshal.GetFunctionPointerForDelegate(callback),
						param);

				cssSelectors = Marshal.PtrToStringUni(cssSelectorsPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(cssSelectorsPtr);
			}
#else
			return this._selectTags(hTag, cssSelectors, callback, param);
#endif
		}

		public int SendEvent(IntPtr hTag, int code, IntPtr hTagSource, IntPtr parameter, out bool isHandled)
		{
#if COMPACT_FRAMEWORK
			IntPtr isHandledPtr = IntPtr.Zero;

			try
			{
				isHandledPtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._sendEvent,
						hTag,
						code,
						hTagSource,
						parameter,
						isHandledPtr);

				isHandled = Marshal.ReadInt32(isHandledPtr) != 0;

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(isHandledPtr);
			}
#else
			return this._sendEvent(hTag, code, hTagSource, parameter, out isHandled);
#endif
		}

		public int SetAttributeValueByName(IntPtr hTag, string name, string value)
		{
#if COMPACT_FRAMEWORK
			IntPtr namePtr = IntPtr.Zero;
			IntPtr valuePtr = IntPtr.Zero;

			try
			{
				namePtr = MarshalCF.StringToHGlobalUni(name);
				valuePtr = MarshalCF.StringToHGlobalUni(value);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setAttributeValueByName,
						hTag,
						namePtr,
						valuePtr);

				name = Marshal.PtrToStringUni(namePtr);
				value = Marshal.PtrToStringUni(valuePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(namePtr);
				Marshal.FreeHGlobal(valuePtr);
			}
#else
			return this._setAttributeValueByName(hTag, name, value);
#endif
		}

		public int SetCapture(IntPtr hTag)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._setCapture,
					hTag);

			return result.ToInt32();
#else
			return this._setCapture(hTag);
#endif
		}

		public int SetControlValue(IntPtr hTag, ref JsonValueData value)
		{
#if COMPACT_FRAMEWORK
			IntPtr valuePtr = IntPtr.Zero;

			try
			{
				valuePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(JsonValueData)));

				Marshal.StructureToPtr(value, valuePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setControlValue,
						hTag,
						valuePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(valuePtr);
			}
#else
			return this._setControlValue(hTag, ref value);
#endif
		}

		public int SetScrollInfo(IntPtr hTag, Point position, bool smooth)
		{
#if COMPACT_FRAMEWORK
			IntPtr positionPtr = IntPtr.Zero;

			try
			{
				positionPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(Point)));

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setScrollInfo,
						hTag,
						position,
						smooth);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(positionPtr);
			}
#else
			return this._setScrollInfo(hTag, position, smooth);
#endif
		}

		public int SetStyleAttribute(IntPtr hTag, string name, string value)
		{
#if COMPACT_FRAMEWORK
			IntPtr namePtr = IntPtr.Zero;
			IntPtr valuePtr = IntPtr.Zero;

			try
			{
				namePtr = MarshalCF.StringToHGlobalUni(name);
				valuePtr = MarshalCF.StringToHGlobalUni(value);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setStyleAttribute,
						hTag,
						namePtr,
						valuePtr);

				name = Marshal.PtrToStringUni(namePtr);
				value = Marshal.PtrToStringUni(valuePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(namePtr);
				Marshal.FreeHGlobal(valuePtr);
			}
#else
			return this._setStyleAttribute(hTag, name, value);
#endif
		}

		public int SetStyleAttribute(IntPtr hTag)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._setStyleAttribute,
					hTag,
					IntPtr.Zero,
					IntPtr.Zero);

			return result.ToInt32();
#else
			return this._setStyleAttribute(hTag, null, null);
#endif
		}

		public int SetTagExpando(IntPtr hTag, IntPtr lpExpando)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._setTagExpando,
					hTag,
					lpExpando);

			return result.ToInt32();
#else
			return this._setTagExpando(hTag, lpExpando);
#endif
		}

		public int SetTagHtml(IntPtr hTag, byte[] html, int htmlSize, SetTagHtmlBehavior setMethod)
		{
#if COMPACT_FRAMEWORK
			IntPtr htmlPtr = IntPtr.Zero;

			try
			{
				htmlPtr = Marshal.AllocHGlobal(htmlSize);

				Marshal.Copy(html, 0, htmlPtr, htmlSize);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setTagHtml,
						hTag,
						html,
						htmlSize,
						(int)setMethod);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(htmlPtr);
			}
#else
			return this._setTagHtml(hTag, html, htmlSize, setMethod);
#endif
		}

		public int SetTagInnerText16(IntPtr hTag, string text, int textLength)
		{
#if COMPACT_FRAMEWORK
			IntPtr textPtr = IntPtr.Zero;

			try
			{
				textPtr = MarshalCF.StringToHGlobalUni(text);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setTagInnerText16,
						hTag,
						textPtr,
						textLength);

				text = Marshal.PtrToStringUni(textPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(textPtr);
			}
#else
			return this._setTagInnerText16(hTag, text, textLength);
#endif
		}

		public int SetTagState(IntPtr hTag, int bitsToSet, int bitsToClear, bool redraw)
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._setTagState,
					hTag,
					bitsToSet,
					bitsToClear,
					redraw);

			return result.ToInt32();
#else
			return this._setTagState(hTag, bitsToSet, bitsToClear, redraw);
#endif
		}

		public int ShowPopup(IntPtr hTagPopup, IntPtr hTagAnchor, HtmlPopupAlignment placement)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._showPopup,
						hTagPopup,
						hTagAnchor,
						(int)placement);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._showPopup(hTagPopup, hTagAnchor, placement);
#endif
		}

		public int ShowPopupAt(IntPtr hTagPopup, Point position, bool isAnimated)
		{
#if COMPACT_FRAMEWORK
			IntPtr positionPtr = IntPtr.Zero;

			try
			{
				positionPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(Point)));

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._showPopupAt,
						hTagPopup,
						position,
						isAnimated);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(positionPtr);
			}
#else
			return this._showPopupAt(hTagPopup, position, isAnimated);
#endif
		}

		public int SortChildTags(IntPtr hTag, int firstChildIndex, int lastChildIndex, DOM_TagSortHandler sortHandler, IntPtr param)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._sortChildTags,
						hTag,
						firstChildIndex,
						lastChildIndex,
						Marshal.GetFunctionPointerForDelegate(sortHandler),
						param);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._sortChildTags(hTag, firstChildIndex, lastChildIndex, sortHandler, param);
#endif
		}

		public int SwapTags(IntPtr hTag1, IntPtr hTag2)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._swapTags,
						hTag1,
						hTag2);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._swapTags(hTag1, hTag2);
#endif
		}

		public int TrackPopupAt(IntPtr hTagPopup, Point position, bool isAnimated, out IntPtr hTagItem)
		{
#if COMPACT_FRAMEWORK
			IntPtr positionPtr = IntPtr.Zero;
			IntPtr hTagItemPtr = IntPtr.Zero;

			try
			{
				positionPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(Point)));
				hTagItemPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._trackPopupAt,
						hTagPopup,
						position,
						isAnimated,
						hTagItemPtr);

				hTagItem = Marshal.ReadIntPtr(hTagItemPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(positionPtr);
				Marshal.FreeHGlobal(hTagItemPtr);
			}
#else
			return this._trackPopupAt(hTagPopup, position, isAnimated, out hTagItem);
#endif
		}

		public int UnuseTag(IntPtr hTag)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._unuseTag,
						hTag);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._unuseTag(hTag);
#endif
		}

		public int UpdateTag(IntPtr hTag, bool redraw)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._updateTag,
						hTag,
						redraw);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._updateTag(hTag, redraw);
#endif
		}

		public int UseTag(IntPtr hTag)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._useTag,
						hTag);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._useTag(hTag);
#endif
		}

		#endregion
	}

	internal sealed class WindowDynamicInterop : DynamicInterop
	{
		#region Types
#if !COMPACT_FRAMEWORK
		[return: MarshalAs(UnmanagedType.Bool)]
		private delegate bool AddMasterCSSHandler([In][MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 1)] byte[] css, [In] int cssSize);

		[return: MarshalAs(UnmanagedType.Bool)]
		private delegate bool CommitUpdateHandler([In] IntPtr hWindow);

		[return: MarshalAs(UnmanagedType.Bool)]
		private delegate bool ForceUpdateHandler([In] IntPtr hWindow);

		private delegate int GetFocusTagHandler([In] IntPtr hWindow, [Out] out IntPtr hTagFocus);

		private delegate int GetRootTagHandler([In] IntPtr hWindow, [Out] out IntPtr hTagRoot);

		private delegate IntPtr GetSelectedHTMLHandler([In] IntPtr hWindow, [Out] out int size);

		private delegate int GetTagFromPointHandler([In] IntPtr hWindow, [In] Point point, [Out] out IntPtr hTag);

		private delegate string GetUnicodeClassNameHandler();

		[return: MarshalAs(UnmanagedType.Bool)]
		private delegate bool IsSelectionExistHandler([In] IntPtr hWindow);

		[return: MarshalAs(UnmanagedType.Bool)]
		private delegate bool LoadHtmlHandler([In] IntPtr hWindow, [In][MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] byte[] html, [In] int htmlSize);

		[return: MarshalAs(UnmanagedType.Bool)]
		private delegate bool LoadHtmlExHandler([In] IntPtr hWindow, [In][MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] byte[] html, [In] int htmlSize, [In][MarshalAs(UnmanagedType.LPWStr)] string baseUrl);

		[return: MarshalAs(UnmanagedType.Bool)]
		private delegate bool ResourceReadyHandler([In] IntPtr hWindow, [In][MarshalAs(UnmanagedType.LPWStr)] string uri, [In][MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 3)] byte[] data, [In] int dataSize);

		private delegate void SetCallbackHandler([In] IntPtr hWindow, [In] HtmlWindowNotificationMessageHandler notificationCallback, [In] IntPtr vParam);

		private delegate void SetModeHandler([In] IntPtr hWindow, [In][MarshalAs(UnmanagedType.U4)] HtmlLayoutMode layoutMode);

		[return: MarshalAs(UnmanagedType.Bool)]
		private delegate bool SetOptionHandler([In] IntPtr hWindow, [In] HtmlEngineOption option, [In] int value);

		[return: MarshalAs(UnmanagedType.Bool)]
		private delegate bool TranslateMessageHandler([In][Out] ref MSG message);

		private delegate int TraverseMouseEventHandler([In] int mustBe1, [In] ref HtmlMouseEventParameters parameters, [Out][MarshalAs(UnmanagedType.Bool)] out bool isHandled);

		private delegate int TraverseKeyboardEventHandler([In] int mustBe2, [In] ref HtmlKeyboardEventParameters parameters, [Out][MarshalAs(UnmanagedType.Bool)] out bool isHandled);

		private delegate IntPtr WindowProcNoDefaultHandler([In] IntPtr hWindow, [In] int message, [In] IntPtr wParam, [In] IntPtr lParam, [Out][MarshalAs(UnmanagedType.Bool)] out bool isHandled);

#endif
		#endregion

		#region Fields

#if COMPACT_FRAMEWORK
		private IntPtr _addMasterCSS;
		private IntPtr _commitUpdate;
		private IntPtr _forceUpdate;
		private IntPtr _getFocusTag;
		private IntPtr _getRootTag;
		private IntPtr _getSelectedHTML;
		private IntPtr _getTagFromPoint;
		private IntPtr _getUnicodeClassName;
		private IntPtr _isSelectionExist;
		private IntPtr _loadHtml;
		private IntPtr _loadHtmlEx;
		private IntPtr _resourceReady;
		private IntPtr _setCallback;
		private IntPtr _setMode;
		private IntPtr _setOption;
		private IntPtr _translateMessage;
		private IntPtr _traverseEvent;
		private IntPtr _windowProcNoDefault;
#else
		private AddMasterCSSHandler _addMasterCSS;
		private CommitUpdateHandler _commitUpdate;
		private ForceUpdateHandler _forceUpdate;
		private GetFocusTagHandler _getFocusTag;
		private GetRootTagHandler _getRootTag;
		private GetSelectedHTMLHandler _getSelectedHTML;
		private GetTagFromPointHandler _getTagFromPoint;
		private GetUnicodeClassNameHandler _getUnicodeClassName;
		private IsSelectionExistHandler _isSelectionExist;
		private LoadHtmlHandler _loadHtml;
		private LoadHtmlExHandler _loadHtmlEx;
		private ResourceReadyHandler _resourceReady;
		private SetCallbackHandler _setCallback;
		private SetModeHandler _setMode;
		private SetOptionHandler _setOption;
		private TranslateMessageHandler _translateMessage;
		private TraverseMouseEventHandler _traverseMouseEvent;
		private TraverseKeyboardEventHandler _traverseKeyboardEvent;
		private WindowProcNoDefaultHandler _windowProcNoDefault;
#endif

		#endregion

		#region Methods

		public WindowDynamicInterop(IntPtr hModule, string namePrefix)
			: base(hModule)
		{
#if COMPACT_FRAMEWORK
			this._addMasterCSS = GetProcAddress(namePrefix + "AppendMasterCSS");
			this._commitUpdate = GetProcAddress(namePrefix + "CommitUpdates");
			this._forceUpdate = GetProcAddress(namePrefix + "UpdateWindow");
			this._getFocusTag = GetProcAddress(namePrefix + "GetFocusElement");
			this._getRootTag = GetProcAddress(namePrefix + "GetRootElement");
			this._getSelectedHTML = GetProcAddress(namePrefix + "GetSelectedHTML");
			this._getTagFromPoint = GetProcAddress(namePrefix + "FindElement");
			this._getUnicodeClassName = GetProcAddress(namePrefix + "ClassNameW");
			this._isSelectionExist = GetProcAddress(namePrefix + "SelectionExist");
			this._loadHtml = GetProcAddress(namePrefix + "LoadHtml");
			this._loadHtmlEx = GetProcAddress(namePrefix + "LoadHtmlEx");
			this._resourceReady = GetProcAddress(namePrefix + "DataReady");
			this._setCallback = GetProcAddress(namePrefix + "SetCallback");
			this._setMode = GetProcAddress(namePrefix + "SetMode");
			this._setOption = GetProcAddress(namePrefix + "SetOption");
			this._translateMessage = GetProcAddress(namePrefix + "TranslateMessage");
			this._traverseEvent = GetProcAddress(namePrefix + "TraverseUIEvent");
			this._windowProcNoDefault = GetProcAddress(namePrefix + "ProcND");
#else
			this._addMasterCSS = GetProcAddress<AddMasterCSSHandler>(namePrefix + "AppendMasterCSS");
			this._commitUpdate = GetProcAddress<CommitUpdateHandler>(namePrefix + "CommitUpdates");
			this._forceUpdate = GetProcAddress<ForceUpdateHandler>(namePrefix + "UpdateWindow");
			this._getFocusTag = GetProcAddress<GetFocusTagHandler>(namePrefix + "GetFocusElement");
			this._getRootTag = GetProcAddress<GetRootTagHandler>(namePrefix + "GetRootElement");
			this._getSelectedHTML = GetProcAddress<GetSelectedHTMLHandler>(namePrefix + "GetSelectedHTML");
			this._getTagFromPoint = GetProcAddress<GetTagFromPointHandler>(namePrefix + "FindElement");
			this._getUnicodeClassName = GetProcAddress<GetUnicodeClassNameHandler>(namePrefix + "ClassNameW");
			this._isSelectionExist = GetProcAddress<IsSelectionExistHandler>(namePrefix + "SelectionExist");
			this._loadHtml = GetProcAddress<LoadHtmlHandler>(namePrefix + "LoadHtml");
			this._loadHtmlEx = GetProcAddress<LoadHtmlExHandler>(namePrefix + "LoadHtmlEx");
			this._resourceReady = GetProcAddress<ResourceReadyHandler>(namePrefix + "DataReady");
			this._setCallback = GetProcAddress<SetCallbackHandler>(namePrefix + "SetCallback");
			this._setMode = GetProcAddress<SetModeHandler>(namePrefix + "SetMode");
			this._setOption = GetProcAddress<SetOptionHandler>(namePrefix + "SetOption");
			this._translateMessage = GetProcAddress<TranslateMessageHandler>(namePrefix + "TranslateMessage");
			this._traverseMouseEvent = GetProcAddress<TraverseMouseEventHandler>(namePrefix + "TraverseUIEvent");
			this._traverseKeyboardEvent = GetProcAddress<TraverseKeyboardEventHandler>(namePrefix + "TraverseUIEvent");
			this._windowProcNoDefault = GetProcAddress<WindowProcNoDefaultHandler>(namePrefix + "ProcND");
#endif
		}

		[return: MarshalAs(UnmanagedType.Bool)]
		public bool AddMasterCSS(byte[] css, int cssSize)
		{
#if COMPACT_FRAMEWORK
			IntPtr cssPtr = IntPtr.Zero;

			try
			{
				cssPtr = Marshal.AllocHGlobal(cssSize);

				Marshal.Copy(css, 0, cssPtr, cssSize);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._addMasterCSS,
						css,
						cssSize);

				return result != IntPtr.Zero;
			}
			finally
			{
				Marshal.FreeHGlobal(cssPtr);
			}
#else
			return this._addMasterCSS(css, cssSize);
#endif
		}

		[return: MarshalAs(UnmanagedType.Bool)]
		public bool CommitUpdate(IntPtr hWindow)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._commitUpdate,
						hWindow);

				return result != IntPtr.Zero;
			}
			finally
			{
			}
#else
			return this._commitUpdate(hWindow);
#endif
		}

		[return: MarshalAs(UnmanagedType.Bool)]
		public bool ForceUpdate(IntPtr hWindow)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._forceUpdate,
						hWindow);

				return result != IntPtr.Zero;
			}
			finally
			{
			}
#else
			return this._forceUpdate(hWindow);
#endif
		}

		public int GetFocusTag(IntPtr hWindow, out IntPtr hTagFocus)
		{
#if COMPACT_FRAMEWORK
			IntPtr hTagFocusPtr = IntPtr.Zero;

			try
			{
				hTagFocusPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getFocusTag,
						hWindow,
						hTagFocusPtr);

				hTagFocus = Marshal.ReadIntPtr(hTagFocusPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(hTagFocusPtr);
			}
#else
			return this._getFocusTag(hWindow, out hTagFocus);
#endif
		}

		public int GetRootTag(IntPtr hWindow, out IntPtr hTagRoot)
		{
#if COMPACT_FRAMEWORK
			IntPtr hTagRootPtr = IntPtr.Zero;

			try
			{
				hTagRootPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getRootTag,
						hWindow,
						hTagRootPtr);

				hTagRoot = Marshal.ReadIntPtr(hTagRootPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(hTagRootPtr);
			}
#else
			return this._getRootTag(hWindow, out hTagRoot);
#endif
		}

		public IntPtr GetSelectedHTML(IntPtr hWindow, out int size)
		{
#if COMPACT_FRAMEWORK
			IntPtr sizePtr = IntPtr.Zero;

			try
			{
				sizePtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getSelectedHTML,
						hWindow,
						sizePtr);

				size = Marshal.ReadInt32(sizePtr);

				return result;
			}
			finally
			{
				Marshal.FreeHGlobal(sizePtr);
			}
#else
			return this._getSelectedHTML(hWindow, out size);
#endif
		}

		public int GetTagFromPoint(IntPtr hWindow, Point point, out IntPtr hTag)
		{
#if COMPACT_FRAMEWORK
			IntPtr pointPtr = IntPtr.Zero;
			IntPtr hTagPtr = IntPtr.Zero;

			try
			{
				pointPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(Point)));
				hTagPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getTagFromPoint,
						hWindow,
						point,
						hTagPtr);

				hTag = Marshal.ReadIntPtr(hTagPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(pointPtr);
				Marshal.FreeHGlobal(hTagPtr);
			}
#else
			return this._getTagFromPoint(hWindow, point, out hTag);
#endif
		}

		public string GetUnicodeClassName()
		{
#if COMPACT_FRAMEWORK
			IntPtr result =
				MarshalCF.CallUnmanagedStdCallFunction(
					this._getUnicodeClassName);


			return Marshal.PtrToStringUni(result);
#else
			return this._getUnicodeClassName();
#endif
		}

		[return: MarshalAs(UnmanagedType.Bool)]
		public bool IsSelectionExist(IntPtr hWindow)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._isSelectionExist,
						hWindow);

				return result != IntPtr.Zero;
			}
			finally
			{
			}
#else
			return this._isSelectionExist(hWindow);
#endif
		}

		[return: MarshalAs(UnmanagedType.Bool)]
		public bool LoadHtml(IntPtr hWindow, byte[] html, int htmlSize)
		{
#if COMPACT_FRAMEWORK
			IntPtr htmlPtr = IntPtr.Zero;

			try
			{
				htmlPtr = Marshal.AllocHGlobal(htmlSize);

				Marshal.Copy(html, 0, htmlPtr, htmlSize);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._loadHtml,
						hWindow,
						html,
						htmlSize);

				return result != IntPtr.Zero;
			}
			finally
			{
				Marshal.FreeHGlobal(htmlPtr);
			}
#else
			return this._loadHtml(hWindow, html, htmlSize);
#endif
		}

		[return: MarshalAs(UnmanagedType.Bool)]
		public bool LoadHtmlEx(IntPtr hWindow, byte[] html, int htmlSize, string baseUrl)
		{
#if COMPACT_FRAMEWORK
			IntPtr htmlPtr = IntPtr.Zero;
			IntPtr baseUrlPtr = IntPtr.Zero;

			try
			{
				htmlPtr = Marshal.AllocHGlobal(htmlSize);
				baseUrlPtr = MarshalCF.StringToHGlobalUni(baseUrl);

				Marshal.Copy(html, 0, htmlPtr, htmlSize);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._loadHtmlEx,
						hWindow,
						html,
						htmlSize,
						baseUrlPtr);

				baseUrl = Marshal.PtrToStringUni(baseUrlPtr);

				return result != IntPtr.Zero;
			}
			finally
			{
				Marshal.FreeHGlobal(htmlPtr);
				Marshal.FreeHGlobal(baseUrlPtr);
			}
#else
			return this._loadHtmlEx(hWindow, html, htmlSize, baseUrl);
#endif
		}

		[return: MarshalAs(UnmanagedType.Bool)]
		public bool ResourceReady(IntPtr hWindow, string uri, byte[] data, int dataSize)
		{
#if COMPACT_FRAMEWORK
			IntPtr uriPtr = IntPtr.Zero;
			IntPtr dataPtr = IntPtr.Zero;

			try
			{
				uriPtr = MarshalCF.StringToHGlobalUni(uri);
				dataPtr = Marshal.AllocHGlobal(dataSize);

				Marshal.Copy(data, 0, dataPtr, dataSize);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._resourceReady,
						hWindow,
						uriPtr,
						data,
						dataSize);

				uri = Marshal.PtrToStringUni(uriPtr);

				return result != IntPtr.Zero;

			}
			finally
			{
				Marshal.FreeHGlobal(uriPtr);
				Marshal.FreeHGlobal(dataPtr);
			}
#else
			return this._resourceReady(hWindow, uri, data, dataSize);
#endif
		}

		public void SetCallback(IntPtr hWindow)
		{
#if COMPACT_FRAMEWORK

			try
			{

				MarshalCF.CallUnmanagedStdCallFunction(
					this._setCallback,
					hWindow,
					IntPtr.Zero,
					IntPtr.Zero);

			}
			finally
			{
			}
#else
			this._setCallback(hWindow, null, IntPtr.Zero);
#endif
		}

		public void SetCallback(IntPtr hWindow, HtmlWindowNotificationMessageHandler notificationCallback, IntPtr vParam)
		{
#if COMPACT_FRAMEWORK

			try
			{

				MarshalCF.CallUnmanagedStdCallFunction(
					this._setCallback,
					hWindow,
					Marshal.GetFunctionPointerForDelegate(notificationCallback),
					vParam);

			}
			finally
			{
			}
#else
			this._setCallback(hWindow, notificationCallback, vParam);
#endif
		}

		public void SetMode(IntPtr hWindow, HtmlLayoutMode layoutMode)
		{
#if COMPACT_FRAMEWORK

			try
			{

				MarshalCF.CallUnmanagedStdCallFunction(
					this._setMode,
					hWindow,
					(int)layoutMode);

			}
			finally
			{
			}
#else
			this._setMode(hWindow, layoutMode);
#endif
		}

		[return: MarshalAs(UnmanagedType.Bool)]
		public bool SetOption(IntPtr hWindow, HtmlEngineOption option, int value)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setOption,
						hWindow,
						(int)option,
						value);

				return result != IntPtr.Zero;
			}
			finally
			{
			}
#else
			return this._setOption(hWindow, option, value);
#endif
		}

		[return: MarshalAs(UnmanagedType.Bool)]
		public bool TranslateMessage(ref MSG message)
		{
#if COMPACT_FRAMEWORK
			IntPtr messagePtr = IntPtr.Zero;

			try
			{
				messagePtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(MSG)));

				Marshal.StructureToPtr(message, messagePtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._translateMessage,
						messagePtr);

				return result != IntPtr.Zero;
			}
			finally
			{
				Marshal.FreeHGlobal(messagePtr);
			}
#else
			return this._translateMessage(ref message);
#endif
		}

		public int TraverseEvent(ref HtmlMouseEventParameters parameters, out bool isHandled)
		{
#if COMPACT_FRAMEWORK
			IntPtr parametersPtr = IntPtr.Zero;
			IntPtr isHandledPtr = IntPtr.Zero;

			try
			{
				parametersPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(HtmlMouseEventParameters)));
				isHandledPtr = Marshal.AllocHGlobal(4);

				Marshal.StructureToPtr(parameters, parametersPtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._traverseEvent,
						1,
						parametersPtr,
						isHandledPtr);

				isHandled = Marshal.ReadInt32(isHandledPtr) != 0;

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(parametersPtr);
				Marshal.FreeHGlobal(isHandledPtr);
			}
#else
			return this._traverseMouseEvent(1, ref parameters, out isHandled);
#endif
		}

		public int TraverseEvent(ref HtmlKeyboardEventParameters parameters, out bool isHandled)
		{
#if COMPACT_FRAMEWORK
			IntPtr parametersPtr = IntPtr.Zero;
			IntPtr isHandledPtr = IntPtr.Zero;

			try
			{
				parametersPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(HtmlKeyboardEventParameters)));
				isHandledPtr = Marshal.AllocHGlobal(4);

				Marshal.StructureToPtr(parameters, parametersPtr, false);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._traverseEvent,
						2,
						parametersPtr,
						isHandledPtr);

				isHandled = Marshal.ReadInt32(isHandledPtr) != 0;

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(parametersPtr);
				Marshal.FreeHGlobal(isHandledPtr);
			}
#else
			return this._traverseKeyboardEvent(2, ref parameters, out isHandled);
#endif
		}

		public IntPtr WindowProcNoDefault(IntPtr hWindow, int message, IntPtr wParam, IntPtr lParam, out bool isHandled)
		{
#if COMPACT_FRAMEWORK
			IntPtr isHandledPtr = IntPtr.Zero;

			try
			{
				isHandledPtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._windowProcNoDefault,
						hWindow,
						message,
						wParam,
						lParam,
						isHandledPtr);

				isHandled = Marshal.ReadInt32(isHandledPtr) != 0;

				return result;
			}
			finally
			{
				Marshal.FreeHGlobal(isHandledPtr);
			}
#else
			return this._windowProcNoDefault(hWindow, message, wParam, lParam, out isHandled);
#endif
		}

		#endregion
	}

	internal sealed class WindowlessDynamicInterop : DynamicInterop
	{
		#region Types
#if !COMPACT_FRAMEWORK
		private delegate IntPtr CreateInstanceHandler();

		private delegate int DestroyInstanceHandler([In] IntPtr hWindowsless);

		private delegate int GetRootTagHandler([In] IntPtr hWindowsless, [Out] out IntPtr hTag);

		private delegate int GetFocusTagHandler([In] IntPtr hWindowsless, [Out] out IntPtr hTag, [Out] out bool isViewActive);

		private delegate int LoadHtmlHandler([In] IntPtr hWindowsless, [In][MarshalAs(UnmanagedType.LPWStr)] string baseUrl, [In][MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] byte[] html, [In] int htmlSize);

		private delegate int MeasureHandler([In] IntPtr hWindowsless, [In] int width, [In] int height);

		private delegate int RenderOnBitmapHandler([In] IntPtr hWindowsless, [In] IntPtr hBitmap, [In] int left, [In] int top, [In] int width, [In] int height);

		private delegate int RenderOnDCHandler([In] IntPtr hWindowsless, [In] IntPtr hDC, [In] int left, [In] int top, [In] int width, [In] int height);

		[return: MarshalAs(UnmanagedType.Bool)]
		private delegate bool UpdateViewHandler([In] IntPtr hWindowsless);

#endif
		#endregion

		#region Fields

#if COMPACT_FRAMEWORK
		private IntPtr _createInstance;
		private IntPtr _destroyInstance;
		private IntPtr _getRootTag;
		private IntPtr _getFocusTag;
		private IntPtr _loadHtml;
		private IntPtr _measure;
		private IntPtr _renderOnBitmap;
		private IntPtr _renderOnDC;
		private IntPtr _updateView;
#else
		private CreateInstanceHandler _createInstance;
		private DestroyInstanceHandler _destroyInstance;
		private GetRootTagHandler _getRootTag;
		private GetFocusTagHandler _getFocusTag;
		private LoadHtmlHandler _loadHtml;
		private MeasureHandler _measure;
		private RenderOnBitmapHandler _renderOnBitmap;
		private RenderOnDCHandler _renderOnDC;
		private UpdateViewHandler _updateView;
#endif

		#endregion

		#region Methods

		public WindowlessDynamicInterop(IntPtr hModule, string namePrefix)
			: base(hModule)
		{
#if COMPACT_FRAMEWORK
			this._createInstance = GetProcAddress(namePrefix + "CreateInstance");
			this._destroyInstance = GetProcAddress(namePrefix + "DestroyInstance");
			this._getRootTag = GetProcAddress(namePrefix + "GetRootElement");
			this._getFocusTag = GetProcAddress(namePrefix + "GetFocusElement");
			this._loadHtml = GetProcAddress(namePrefix + "LoadHtmlFromMemory");
			this._measure = GetProcAddress(namePrefix + "Measure");
			this._renderOnBitmap = GetProcAddress(namePrefix + "RenderOnBitmap");
			this._renderOnDC = GetProcAddress(namePrefix + "Render");
			this._updateView = GetProcAddress(namePrefix + "UpdateView");
#else
			this._createInstance = GetProcAddress<CreateInstanceHandler>(namePrefix + "CreateInstance");
			this._destroyInstance = GetProcAddress<DestroyInstanceHandler>(namePrefix + "DestroyInstance");
			this._getRootTag = GetProcAddress<GetRootTagHandler>(namePrefix + "GetRootElement");
			this._getFocusTag = GetProcAddress<GetFocusTagHandler>(namePrefix + "GetFocusElement");
			this._loadHtml = GetProcAddress<LoadHtmlHandler>(namePrefix + "LoadHtmlFromMemory");
			this._measure = GetProcAddress<MeasureHandler>(namePrefix + "Measure");
			this._renderOnBitmap = GetProcAddress<RenderOnBitmapHandler>(namePrefix + "RenderOnBitmap");
			this._renderOnDC = GetProcAddress<RenderOnDCHandler>(namePrefix + "Render");
			this._updateView = GetProcAddress<UpdateViewHandler>(namePrefix + "UpdateView");
#endif
		}

		public IntPtr CreateInstance()
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._createInstance);

				return result;
			}
			finally
			{
			}
#else
			return this._createInstance();
#endif
		}

		public int DestroyInstance(IntPtr hWindowsless)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._destroyInstance,
						hWindowsless);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._destroyInstance(hWindowsless);
#endif
		}

		public int GetRootTag(IntPtr hWindowsless, out IntPtr hTag)
		{
#if COMPACT_FRAMEWORK
			IntPtr hTagPtr = IntPtr.Zero;

			try
			{
				hTagPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getRootTag,
						hWindowsless,
						hTagPtr);

				hTag = Marshal.ReadIntPtr(hTagPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(hTagPtr);
			}
#else
			return this._getRootTag(hWindowsless, out hTag);
#endif
		}

		public int GetFocusTag(IntPtr hWindowsless, out IntPtr hTag, out bool isViewActive)
		{
#if COMPACT_FRAMEWORK
			IntPtr hTagPtr = IntPtr.Zero;
			IntPtr isViewActivePtr = IntPtr.Zero;

			try
			{
				hTagPtr = Marshal.AllocHGlobal(IntPtr.Size);
				isViewActivePtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getRootTag,
						hWindowsless,
						hTagPtr,
						isViewActivePtr);

				hTag = Marshal.ReadIntPtr(hTagPtr);
				isViewActive = Marshal.ReadInt32(isViewActivePtr) != 0;

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(hTagPtr);
			}
#else
			return this._getFocusTag(hWindowsless, out hTag, out isViewActive);
#endif
		}

		public int LoadHtml(IntPtr hWindowsless, string baseUrl, byte[] html, int htmlSize)
		{
#if COMPACT_FRAMEWORK
			IntPtr baseUrlPtr = IntPtr.Zero;
			IntPtr htmlPtr = IntPtr.Zero;

			try
			{
				baseUrlPtr = MarshalCF.StringToHGlobalUni(baseUrl);
				htmlPtr = Marshal.AllocHGlobal(htmlSize);

				Marshal.Copy(html, 0, htmlPtr, htmlSize);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._loadHtml,
						hWindowsless,
						baseUrlPtr,
						html,
						htmlSize);

				baseUrl = Marshal.PtrToStringUni(baseUrlPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(baseUrlPtr);
				Marshal.FreeHGlobal(htmlPtr);
			}
#else
			return this._loadHtml(hWindowsless, baseUrl, html, htmlSize);
#endif
		}

		public int Measure(IntPtr hWindowsless, int width, int height)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._measure,
						hWindowsless,
						width,
						height);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._measure(hWindowsless, width, height);
#endif
		}

		public int RenderOnBitmap(IntPtr hWindowsless, IntPtr hBitmap, int left, int top, int width, int height)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._renderOnBitmap,
						hWindowsless,
						hBitmap,
						left,
						top,
						width,
						height);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._renderOnBitmap(hWindowsless, hBitmap, left, top, width, height);
#endif
		}

		public int RenderOnDC(IntPtr hWindowsless, IntPtr hDC, int left, int top, int width, int height)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._renderOnDC,
						hWindowsless,
						hDC,
						left,
						top,
						width,
						height);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._renderOnDC(hWindowsless, hDC, left, top, width, height);
#endif
		}

		[return: MarshalAs(UnmanagedType.Bool)]
		public bool UpdateView(IntPtr hWindowsless)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._updateView,
						hWindowsless);

				return result != IntPtr.Zero;
			}
			finally
			{
			}
#else
			return this._updateView(hWindowsless);
#endif
		}

		#endregion
	}

	internal sealed class PrintDynamicInterop : DynamicInterop
	{
		#region Types
#if !COMPACT_FRAMEWORK
		private delegate IntPtr CreateInstanceHandler();

		private delegate int DestroyInstanceHandler([In] IntPtr hPrint);

		private delegate int GetRootTagHandler([In] IntPtr hPrint, [Out] out IntPtr hTag);

		private delegate int LoadHtmlHandler([In] IntPtr hPrint, [In][MarshalAs(UnmanagedType.LPStr)] string baseUri, [In][MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 3)] byte[] data, [In] int dataSize);

		private delegate int MeasureHandler([In] IntPtr hPrint, [In] IntPtr hDC, [In] int widthInPixels, [In] int widthInUnits, [In] int heightInUnits, [Out] out int pageNumber);

		private delegate int RenderOnDCHandler([In] IntPtr hPrint, [In] IntPtr hDC, [In] int left, [In] int top, [In] int pageIndex);

		private delegate int ResourceReadyHandler([In] IntPtr hPrint, [In][MarshalAs(UnmanagedType.LPStr)] string resourceUrl, [In][MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 3)] byte[] data, [In] int dataSize);

		private delegate int SetCallbackHandler([In] IntPtr hPrint, [In] HtmlWindowNotificationMessageHandler notificationCallback, [In] IntPtr vParam);

		private delegate int SetMediaTypeHandler([In] IntPtr hPrint, [In][MarshalAs(UnmanagedType.LPStr)] string mediatype);

		private delegate int SetResourceLoadCallbackHandler([In] IntPtr hPrint, [In] HtmlPrintLoadDataHandler callback);

#endif
		#endregion

		#region Fields

#if COMPACT_FRAMEWORK
		private IntPtr _createInstance;
		private IntPtr _destroyInstance;
		private IntPtr _getRootTag;
		private IntPtr _loadHtml;
		private IntPtr _measure;
		private IntPtr _renderOnDC;
		private IntPtr _resourceReady;
		private IntPtr _setCallback;
		private IntPtr _setMediaType;
		private IntPtr _setResourceLoadCallback;
#else
		private CreateInstanceHandler _createInstance;
		private DestroyInstanceHandler _destroyInstance;
		private GetRootTagHandler _getRootTag;
		private LoadHtmlHandler _loadHtml;
		private MeasureHandler _measure;
		private RenderOnDCHandler _renderOnDC;
		private ResourceReadyHandler _resourceReady;
		private SetCallbackHandler _setCallback;
		private SetMediaTypeHandler _setMediaType;
		private SetResourceLoadCallbackHandler _setResourceLoadCallback;
#endif

		#endregion

		#region Methods

		public PrintDynamicInterop(IntPtr hModule, string namePrefix)
			: base(hModule)
		{
#if COMPACT_FRAMEWORK
			this._createInstance = GetProcAddress(namePrefix + "CreateInstance");
			this._destroyInstance = GetProcAddress(namePrefix + "DestroyInstance");
			this._getRootTag = GetProcAddress(namePrefix + "GetRootElement");
			this._loadHtml = GetProcAddress(namePrefix + "LoadHtmlFromMemory");
			this._measure = GetProcAddress(namePrefix + "Measure");
			this._renderOnDC = GetProcAddress(namePrefix + "Render");
			this._resourceReady = GetProcAddress(namePrefix + "SetDataReady");
			this._setCallback = GetProcAddress(namePrefix + "SetCallback");
			this._setMediaType = GetProcAddress(namePrefix + "SetMediaType");
			this._setResourceLoadCallback = GetProcAddress(namePrefix + "SetLoadDataCallback");
#else
			this._createInstance = GetProcAddress<CreateInstanceHandler>(namePrefix + "CreateInstance");
			this._destroyInstance = GetProcAddress<DestroyInstanceHandler>(namePrefix + "DestroyInstance");
			this._getRootTag = GetProcAddress<GetRootTagHandler>(namePrefix + "GetRootElement");
			this._loadHtml = GetProcAddress<LoadHtmlHandler>(namePrefix + "LoadHtmlFromMemory");
			this._measure = GetProcAddress<MeasureHandler>(namePrefix + "Measure");
			this._renderOnDC = GetProcAddress<RenderOnDCHandler>(namePrefix + "Render");
			this._resourceReady = GetProcAddress<ResourceReadyHandler>(namePrefix + "SetDataReady");
			this._setCallback = GetProcAddress<SetCallbackHandler>(namePrefix + "SetCallback");
			this._setMediaType = GetProcAddress<SetMediaTypeHandler>(namePrefix + "SetMediaType");
			this._setResourceLoadCallback = GetProcAddress<SetResourceLoadCallbackHandler>(namePrefix + "SetLoadDataCallback");
#endif
		}

		public IntPtr CreateInstance()
		{
#if COMPACT_FRAMEWORK

			try
			{
				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._createInstance);

				return result;
			}
			finally
			{
			}
#else
			return this._createInstance();
#endif
		}

		public int DestroyInstance(IntPtr hPrint)
		{
#if COMPACT_FRAMEWORK
			try
			{
				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._destroyInstance,
						hPrint);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._destroyInstance(hPrint);
#endif
		}

		public int GetRootTag(IntPtr hPrint, out IntPtr hTag)
		{
#if COMPACT_FRAMEWORK
			IntPtr hTagPtr = IntPtr.Zero;

			try
			{
				hTagPtr = Marshal.AllocHGlobal(IntPtr.Size);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._getRootTag,
						hPrint,
						hTagPtr);

				hTag = Marshal.ReadIntPtr(hTagPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(hTagPtr);
			}
#else
			return this._getRootTag(hPrint, out hTag);
#endif
		}

		public int LoadHtml(IntPtr hPrint, string baseUri, byte[] data, int dataSize)
		{
#if COMPACT_FRAMEWORK
			IntPtr baseUriPtr = IntPtr.Zero;
			IntPtr dataPtr = IntPtr.Zero;

			try
			{
				baseUriPtr = MarshalCF.StringToHGlobalUni(baseUri);
				dataPtr = Marshal.AllocHGlobal(dataSize);

				Marshal.Copy(data, 0, dataPtr, dataSize);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._loadHtml,
						hPrint,
						baseUriPtr,
						data,
						dataSize);

				baseUri = Marshal.PtrToStringUni(baseUriPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(baseUriPtr);
				Marshal.FreeHGlobal(dataPtr);
			}
#else
			return this._loadHtml(hPrint, baseUri, data, dataSize);
#endif
		}

		public int Measure(IntPtr hPrint, IntPtr hDC, int widthInPixels, int widthInUnits, int heightInUnits, out int pageNumber)
		{
#if COMPACT_FRAMEWORK
			IntPtr pageNumberPtr = IntPtr.Zero;

			try
			{
				pageNumberPtr = Marshal.AllocHGlobal(4);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._measure,
						hPrint,
						hDC,
						widthInPixels,
						widthInUnits,
						heightInUnits,
						pageNumberPtr);

				pageNumber = Marshal.ReadInt32(pageNumberPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(pageNumberPtr);
			}
#else
			return this._measure(hPrint, hDC, widthInPixels, widthInUnits, heightInUnits, out pageNumber);
#endif
		}

		public int RenderOnDC(IntPtr hPrint, IntPtr hDC, int left, int top, int pageIndex)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._renderOnDC,
						hPrint,
						hDC,
						left,
						top,
						pageIndex);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._renderOnDC(hPrint, hDC, left, top, pageIndex);
#endif
		}

		public int ResourceReady(IntPtr hPrint, string resourceUrl, byte[] data, int dataSize)
		{
#if COMPACT_FRAMEWORK
			IntPtr resourceUrlPtr = IntPtr.Zero;
			IntPtr dataPtr = IntPtr.Zero;

			try
			{
				resourceUrlPtr = MarshalCF.StringToHGlobalUni(resourceUrl);
				dataPtr = Marshal.AllocHGlobal(dataSize);

				Marshal.Copy(data, 0, dataPtr, dataSize);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._resourceReady,
						hPrint,
						resourceUrlPtr,
						data,
						dataSize);

				resourceUrl = Marshal.PtrToStringUni(resourceUrlPtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(resourceUrlPtr);
				Marshal.FreeHGlobal(dataPtr);
			}
#else
			return this._resourceReady(hPrint, resourceUrl, data, dataSize);
#endif
		}

		public int SetCallback(IntPtr hPrint, HtmlWindowNotificationMessageHandler notificationCallback, IntPtr vParam)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setCallback,
						hPrint,
						Marshal.GetFunctionPointerForDelegate(notificationCallback),
						vParam);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._setCallback(hPrint, notificationCallback, vParam);
#endif
		}

		public int SetCallback(IntPtr hWindow)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setCallback,
						hWindow,
						IntPtr.Zero,
						IntPtr.Zero);

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._setCallback(hWindow, null, IntPtr.Zero);
#endif
		}

		public int SetMediaType(IntPtr hPrint, string mediatype)
		{
#if COMPACT_FRAMEWORK
			IntPtr mediatypePtr = IntPtr.Zero;

			try
			{
				mediatypePtr = MarshalCF.StringToHGlobalUni(mediatype);

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setMediaType,
						hPrint,
						mediatypePtr);

				mediatype = Marshal.PtrToStringUni(mediatypePtr);

				return result.ToInt32();
			}
			finally
			{
				Marshal.FreeHGlobal(mediatypePtr);
			}
#else
			return this._setMediaType(hPrint, mediatype);
#endif
		}

		public int SetResourceLoadCallback(IntPtr hPrint, HtmlPrintLoadDataHandler callback)
		{
#if COMPACT_FRAMEWORK

			try
			{

				IntPtr result =
					MarshalCF.CallUnmanagedStdCallFunction(
						this._setResourceLoadCallback,
						hPrint,
						Marshal.GetFunctionPointerForDelegate(callback));

				return result.ToInt32();
			}
			finally
			{
			}
#else
			return this._setResourceLoadCallback(hPrint, callback);
#endif
		}

		#endregion
	}

	internal static class HtmlDynamicInterop
	{
		#region Types

		private struct LibraryDescriptor
		{
			public string ModuleName;
			public string JsonPrefix;
			public string DomPrefix;
			public string WindowPrefix;
			public string WindowslessPrefix;
			public string PrintPrefix;
		}

		#endregion

		#region Constants

		private static LibraryDescriptor[] LibraryDescriptors = new LibraryDescriptor[]
		{
			new LibraryDescriptor()
			{
				ModuleName = "HTMLayout",
				JsonPrefix = "Value",
				DomPrefix = "HTMLayout",
				WindowPrefix = "HTMLayout",
				WindowslessPrefix = "HTMLite",
				PrintPrefix = "HTMPrint"
			}
		};

		#endregion

		#region Fields

		private static JsonValueDynamicInterop _json;
		private static DomDynamicInterop _dom;
		private static WindowDynamicInterop _window;
		private static WindowlessDynamicInterop _windowless;
		private static PrintDynamicInterop _print;

		#endregion

		#region Methods

		static HtmlDynamicInterop()
		{
			foreach (LibraryDescriptor libraryDescriptor in LibraryDescriptors)
			{
				string[] libraryNames = new string[]
				{
					string.Format(
						"{0}{1}.dll",
						libraryDescriptor.ModuleName,
						8 * IntPtr.Size),
					string.Format(
						"{0}.dll",
						libraryDescriptor.ModuleName),
				};

				foreach (string libraryName in libraryNames)
				{
					IntPtr hModule = NativeMethods.Module_Load(libraryName, IntPtr.Zero, 0);

					if (hModule != IntPtr.Zero)
					{
						_json = new JsonValueDynamicInterop(hModule, libraryDescriptor.JsonPrefix);
						_dom = new DomDynamicInterop(hModule, libraryDescriptor.DomPrefix);
						_window = new WindowDynamicInterop(hModule, libraryDescriptor.WindowPrefix);
						_windowless = new WindowlessDynamicInterop(hModule, libraryDescriptor.WindowslessPrefix);
						_print = new PrintDynamicInterop(hModule, libraryDescriptor.PrintPrefix);

						return;
					}
				}
			}
		}

		#endregion

		#region Properties

		public static JsonValueDynamicInterop Json
		{
			get { return _json; }
		}

		public static DomDynamicInterop Dom
		{
			get { return _dom; }
		}

		public static WindowDynamicInterop Window
		{
			get { return _window; }
		}

		public static WindowlessDynamicInterop Windowless
		{
			get { return _windowless; }
		}

		public static PrintDynamicInterop Print
		{
			get { return _print; }
		}

		#endregion
	}
}
