﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.InteropServices;

namespace Nabu.Forms.Html
{
	public enum HtmlResult : int
	{
		// HLDOM_OK_NOT_HANDLED
		DOM_Success_NotHandled = -1,
		// HLDOM_OK
		DOM_Success = 0,
		// HLDOM_INVALID_HWND
		DOM_Failure_InvalidHWND = 1,
		// HLDOM_INVALID_HANDLE
		DOM_Failure_InvalidHandle = 2,
		// HLDOM_PASSIVE_HANDLE
		DOM_Failure_PassiveHandle = 3,
		// HLDOM_INVALID_PARAMETER
		DOM_Failure_InvalidParameter = 4,
		// HLDOM_OPERATION_FAILED
		DOM_Failure_OperationFailed = 5,
		DOM_Failure_Unsupported = 256,

		// HPR_OK
		Print_Success = 0,
		// HPR_INVALID_HANDLE
		Print_Failure_InvalidHandle = 1,
		// HPR_INVALID_FORMAT
		Print_Failure_InvalidFormat = 2,
		// HPR_FILE_NOT_FOUND
		Print_Failure_FileNotFound = 3,
		// HPR_INVALID_PARAMETER
		Print_Failure_InvalidParameter = 4,
		// HPR_INVALID_STATE
		Print_Failure_InvalidState = 5,

		Value_Success = 0,
		// HV_BAD_PARAMETER
		Value_Bad_Parameter = 1,
		// HV_INCOMPATIBLE_TYPE
		Value_Incompatible_Type = 2
	}

	// HTMLayoutResourceType
	public enum HtmlResourceType : int
	{
		Unknown = -1,
		// HLRT_DATA_HTML
		Html = 0,
		// HLRT_DATA_IMAGE
		Image = 1,
		// HLRT_DATA_STYLE
		StyleSheet = 2,
		// HLRT_DATA_CURSOR
		Cursor = 3,
		// HLRT_DATA_SCRIPT
		Script = 4
	};

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum HtmlPopupAlignment : int
	{
		Bottom = 2,
		Left = 4,
		Right = 6,
		Top = 8
	};

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	public enum HtmlEngineOption : int
	{
		// HTMLAYOUT_SMOOTH_SCROLL
		// 0, 1
		ScrollSmoothing = 1,
		// HTMLAYOUT_CONNECTION_TIMEOUT
		HttpTimeout = 2,
		// HTMLAYOUT_HTTPS_ERROR
		// 0, 1, 2
		HttpsCertificateHandling = 3,
		// HTMLAYOUT_FONT_SMOOTHING
		// 0, 1, 2, 3
		FontSmoothing = 4
	};

	// HTMLAYOUT_DATA_WRITER
	internal delegate void Resource_DataWriter(
		[In][MarshalAs(UnmanagedType.LPWStr)] string uri,
		[In][MarshalAs(UnmanagedType.U4)] HtmlResourceType resourceType,
		[In][MarshalAs(UnmanagedType.LPArray)] byte[] dataBuffer,
		[In] int dataLength);

	// HTMLAYOUT_DATA_LOADER
	[return: MarshalAs(UnmanagedType.Bool)]
	internal delegate bool Resource_DataLoader(
		[In][MarshalAs(UnmanagedType.LPWStr)] string uri,
		[In][MarshalAs(UnmanagedType.U4)] HtmlResourceType resourceType,
		[In] Resource_DataWriter writer);

	// DEBUG_OUTPUT_PROC
	internal delegate void Debug_OutputHandler(
		[In] IntPtr param,
		[In] int symbol);

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum ShellFileInfoFlags
	{
		SmallIcon /*           */ = 0x000000001,
		OpenIcon /*            */ = 0x000000002,
		ShellIconSize /*       */ = 0x000000004,
		Pidl /*                */ = 0x000000008,
		UseFileAttributes /*   */ = 0x000000010,
		AddOverlays /*         */ = 0x000000020,
		OverlayIndex /*        */ = 0x000000040,
		Icon /*                */ = 0x000000100,
		DisplayName /*         */ = 0x000000200,
		Typename /*            */ = 0x000000400,
		Attributes /*          */ = 0x000000800,
		Iconlocation /*        */ = 0x000001000,
		ExeType /*             */ = 0x000002000,
		SysIconIndex /*        */ = 0x000004000,
		LinkOverlay /*         */ = 0x000008000,
		Selected /*            */ = 0x000010000,
		AttrSpecified /*       */ = 0x000020000,
		LargeIcon /*           */ = 0x000000000
	};

	[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
	internal struct ShellFileInfo
	{
		public IntPtr hIcon;
		public int IconIndex;
		public int Attributes;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
		public string DisplayName;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
		public string TypeName;
	};
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
