﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Drawing;
using System.Runtime.InteropServices;

namespace Nabu.Forms.Html
{
	#region Tags

	#region Destroy

	internal delegate void DOM_ExpandoRelease(
		[In] IntPtr lpExpando,
		[In] IntPtr hTag);

	#endregion

	#region Hierarchy

	// ELEMENT_MODEL
	public enum TagLayoutModel : int
	{
		// DATA_ELEMENT
		Data,
		// INLINE_TEXT_ELEMENT
		InlineText,
		// INLINE_BLOCK_ELEMENT
		InlineBlock,
		// BLOCK_TEXT_ELEMENT
		BlockText,
		// BLOCK_BLOCK_ELEMENT
		BlockBlock
	}

	// ELEMENT_COMPARATOR
	public delegate int DOM_TagSortHandler(
		[In] IntPtr hTag1,
		[In] IntPtr hTag2,
		[In] IntPtr param);

	#endregion

	#region Actions

	public enum SetTagHtmlBehavior : int
	{
		ReplaceInnerContent,
		InsertBeforeInnerStart,
		InsertAfterInnerEnd,
		ReplaceOuterContent,
		InsertBeforeOuterStart,
		InsertAfterOuterEnd
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum HtmlScrollBehavior : int
	{
		ToTop = 0x01,
		Smooth = 0x10
	}

	#endregion

	#region Position

	[Flags]
	public enum HtmlTagArea : int
	{
		// ROOT_RELATIVE
		RootRelative /*      */ = 0x01,
		// SELF_RELATIVE
		SelfRelative /*      */ = 0x02,
		// CONTAINER_RELATIVE
		ContainerRelative /* */ = 0x03,
		// VIEW_RELATIVE
		ViewRelative /*      */ = 0x04,
		// PADDING_BOX
		PaddingBox /*        */ = 0x10,
		// BORDER_BOX
		BorderBox /*         */ = 0x20,
		// MARGIN_BOX
		MarginBox /*         */ = 0x30,
		// BACK_IMAGE_AREA
		BackImage /*         */ = 0x40,
		// FORE_IMAGE_AREA
		ForeImage /*         */ = 0x50,
		// SCROLLABLE_AREA
		ScrollArea /*        */ = 0x60,
	};

	#endregion

	#region State

	// ELEMENT_STATE_BITS
	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum HtmlTagState : int
	{
		/// <summary>
		/// Selector :link.
		/// Element having href attribute.
		/// </summary>
		Link /*        */ = 0x00000001,
		/// <summary>
		/// Selector :hover.
		/// Element under the cursor, mouse hover.
		/// </summary>
		Hover /*       */ = 0x00000002,
		/// <summary>
		/// Selector :active.
		/// Activated element.
		/// </summary>
		Active /*      */ = 0x00000004,
		/// <summary>
		/// Selector :focus.
		/// Element in focus.
		/// </summary>
		Focus /*       */ = 0x00000008,
		/// <summary>
		/// Selector :visited.
		/// AUX flag - not used internally now.
		/// </summary>
		Visited /*     */ = 0x00000010,
		/// <summary>
		/// Selector :current.
		/// Current item in collection.
		/// </select>
		Current /*     */ = 0x00000020,
		/// <summary>
		/// Selector :checked.
		/// Checked element.
		/// </summary>
		Checked /*     */ = 0x00000040,
		/// <summary>
		/// Selector :disabled.
		/// Disabled element.
		/// </summary>
		Disabled /*    */ = 0x00000080,
		/// <summary>
		/// Selector :read-only.
		/// Read-only element is read-only.
		/// </summary>
		Readonly /*    */ = 0x00000100,
		/// <summary>
		/// Selector :expanded.
		/// Expanded element. Mutually exclusive with Collapsed.
		/// </select>
		Expanded /*    */ = 0x00000200,
		/// <summary>
		/// Selector :collapsed
		/// Collapsed element. Mutually exclusive with Expanded.
		/// </summary>
		Collapsed /*   */ = 0x00000400,
		/// <summary>
		/// Selector :incomplete.
		/// Incomplete element.
		/// </summary>
		Incomplete /*  */ = 0x00000800,
		/// <summary>
		/// Selector :animating
		/// Animating element.
		/// </summary>
		Animating /*   */ = 0x00001000,
		/// <summary>
		/// Selector :focusable.
		/// Focusable element.
		/// </summary>
		Focusable /*   */ = 0x00002000,
		/// <summary>
		/// Selector :N/A.
		/// First element in multiple selection.
		/// </select>
		Anchor /*      */ = 0x00004000,
		/// <summary>
		/// Selector :synthetic.
		/// Synthesized element.
		/// </summary>
		Synthetic /*   */ = 0x00008000,
		/// <summary>
		/// Selector :owns-popup.
		/// Owner of visible popup.
		/// </summary>
		OwnsPopup /*   */ = 0x00010000,
		/// <summary>
		/// Selector :tab-focus.
		/// Element got focus by tab traversal.
		/// </summary>
		TabFocus /*    */ = 0x00020000,
		/// <summary>
		/// Selector :empty.
		/// Element is empty.
		/// </summary>
		Empty /*       */ = 0x00040000,
		/// <summary>
		/// Selector :busy.
		/// Busy element.
		/// </summary>
		Busy /*        */ = 0x00080000,
		/// <summary>
		/// Drag over the block that can accept it (so is current drop target). Flag is set for the drop target block. At any given moment of time it can be only one such block.
		/// </summary>
		DragOver /*    */ = 0x00100000,
		/// <summary>
		/// Active drop target. Multiple elements can have this flag when D&D is active.
		/// </summary>
		DropTarget /*  */ = 0x00200000,
		/// <summary>
		/// Dragging/moving - the flag is set for the moving element (copy of the drag-source).
		/// </summary>
		Moving /*      */ = 0x00400000,
		/// <summary>
		/// Dragging/copying - the flag is set for the copying element (copy of the drag-source).
		/// </summary>
		Copying /*     */ = 0x00800000,
		/// <summary>
		/// Is set in element that is being dragged.
		/// </summary>
		DragSource /*  */ = 0x00C00000,
		/// <summary>
		/// Has single child.
		/// </summary>
		HasChild /*    */ = 0x01000000,
		/// <summary>
		/// Has more than one child.
		/// </summary>
		HasChildren /* */ = 0x02000000,
		/// <summary>
		/// Close to active but has wider life span - e.g. in MOUSE_UP it is still on, so behavior can check it in MOUSE_UP to discover CLICK condition.
		/// </summary>
		Pressed /*     */ = 0x04000000,
		/// <summary>
		/// Selector :rtl.
		/// The element or one of its nearest container has @dir and that dir has "rtl" value.
		/// </summary>
		IsRTL /*       */ = 0x20000000,
		/// <summary>
		/// Selector :ltr.
		/// The element or one of its nearest container has @dir and that dir has "ltr" value.
		/// </summary>
		IsLTR /*       */ = 0x10000000,
		/// <summary>
		/// This element is in popup state and presented to the user - out of flow now.
		/// </summary>
		Popup /*       */ = 0x08000000,
	};

	#endregion

	#region Styles

	public enum HtmlTagStyleSource : int
	{
		CssRule,
		InlineRule,
		ExplicitSet
	}

	// HTMLayoutStyleRuleCallback
	[return: MarshalAs(UnmanagedType.Bool)]
	internal delegate bool DOM_StyleEnumHandler(
		[In] ref HtmlTagStyle tagStyle,
		[In] IntPtr param);

#if COMPACT_FRAMEWORK
	// HTMLayoutCSSRuleDef
	[SuppressMessage("Microsoft.Performance", "CA1815:OverrideEqualsAndOperatorEqualsOnValueTypes")]
	[SuppressMessage("Microsoft.Design", "CA1049:TypesThatOwnNativeResourcesShouldBeDisposable")]
	[StructLayout(LayoutKind.Sequential)]
	public struct HtmlTagStyle
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlTagStyleSource Source;
		public IntPtr lpFileName;
		public int LineNumber;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string Selector;
	};
#else
	// HTMLayoutCSSRuleDef
	[SuppressMessage("Microsoft.Performance", "CA1815:OverrideEqualsAndOperatorEqualsOnValueTypes")]
	[StructLayout(LayoutKind.Sequential)]
	public struct HtmlTagStyle
	{
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		[MarshalAs(UnmanagedType.U4)]
		public HtmlTagStyleSource Source;
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		[MarshalAs(UnmanagedType.LPStr)]
		public string FileName;
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		public int LineNumber;
		[SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
		[MarshalAs(UnmanagedType.LPWStr)]
		public string Selector;
	};
#endif

	#endregion

	#region Symbols

	// HTMLayoutEnumerationCallback
	[return: MarshalAs(UnmanagedType.Bool)]
	public delegate bool DOM_SymbolEnumHandler(
		[In] IntPtr context,
		[In] IntPtr hTag,
		[In] int position,
		[In] int positionType,
		[In] char code);

	#endregion

	#region Select/Visit

	[return: MarshalAs(UnmanagedType.Bool)]
	internal delegate bool DOM_TagEnumHandler(
		[In] IntPtr hTag,
		[In] IntPtr param);

	#endregion

	#endregion

	#region Events

	#region Common

#if COMPACT_FRAMEWORK
	// ElementEventProc
	internal delegate int DOM_EventHandler(
		[In] IntPtr forInternalUse,
		[In] IntPtr hTag,
		[In] HtmlEventGroup eventGroup,
		[In] IntPtr parameters);
#else
	// ElementEventProc
	[return: MarshalAs(UnmanagedType.Bool)]
	internal delegate bool DOM_EventHandler(
		[In] IntPtr forInternalUse,
		[In] IntPtr hTag,
		[In] HtmlEventGroup eventGroup,
		[In] IntPtr parameters);
#endif

	// EVENT_GROUPS
	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Usage", "CA2217:DoNotMarkEnumsWithFlags")]
	[Flags]
	public enum HtmlEventGroup : int
	{
		// HANDLE_INITIALIZATION
		Initialization /*            */ = 0x00000000,
		// HANDLE_MOUSE
		InitializationAndMouse /*    */ = 0x00000001,
		// HANDLE_KEY
		InitializationAndKeyboard /* */ = 0x00000002,
		// HANDLE_FOCUS
		InitializationAndFocus /*    */ = 0x00000004,
		// HANDLE_SCROLL
		InitializationAndScroll /*   */ = 0x00000008,
		// HANDLE_TIMER
		InitializationAndTimer /*    */ = 0x00000010,
		// HANDLE_SIZE
		InitializationAndSize /*     */ = 0x00000020,
		// HANDLE_DRAW
		InitializationAndDraw /*     */ = 0x00000040,
		// HANDLE_DATA_ARRIVED
		InitializationAndData /*     */ = 0x00000080,
		// HANDLE_BEHAVIOR_EVENT
		InitializationAndBehavior /* */ = 0x00000100,
		// HANDLE_METHOD_CALL
		InitializationAndMethod /*   */ = 0x00000200,

		// HANDLE_ALL
		All = 0x000003FF,

		// DISABLE_INITIALIZATION
		NoInitialization = unchecked((int)0x80000000),

		Mouse /*    */ = InitializationAndMouse | NoInitialization,
		Keyboard /* */ = InitializationAndKeyboard | NoInitialization,
		Focus /*    */ = InitializationAndFocus | NoInitialization,
		Scroll /*   */ = InitializationAndScroll | NoInitialization,
		Timer /*    */ = InitializationAndTimer | NoInitialization,
		Size /*     */ = InitializationAndSize | NoInitialization,
		Draw /*     */ = InitializationAndDraw | NoInitialization,
		Data /*     */ = InitializationAndData | NoInitialization,
		Behavior /* */ = InitializationAndBehavior | NoInitialization,
		Method /*   */ = InitializationAndMethod | NoInitialization,

		InitializationAndAllInput /*      */ = InitializationAndMouse | InitializationAndKeyboard | InitializationAndFocus,
		InitializationAndAllBehavior /*   */ = InitializationAndScroll | InitializationAndTimer,
		InitializationAndAllAppearance /* */ = InitializationAndSize | InitializationAndDraw,
		InitializationAndAll /*           */ = InitializationAndAllInput | InitializationAndAllBehavior | InitializationAndAllAppearance | InitializationAndData | InitializationAndBehavior | InitializationAndMethod
	};

	#endregion

	#region Initialization

	// INITIALIZATION_EVENTS
	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum HtmlInitializationEventCommand : int
	{
		// BUBBLING
		IsBubbling = 0,
		// SINKING
		IsSinking = 0x08000,
		// HANDLED
		IsHandled = 0x10000,
		// BEHAVIOR_DETACH
		DetachBehavior /* */ = 0x00000000,
		// BEHAVIOR_ATTACH
		AttachBehavior /* */ = 0x00000001
	};

	// INITIALIZATION_PARAMS
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlInitializationEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlInitializationEventCommand Command;
	};

	#endregion

	#region Mouse

	// MOUSE_EVENTS
	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum HtmlMouseEventCommand : int
	{
		// BUBBLING
		IsBubbling = 0,
		// SINKING
		IsSinking = 0x08000,
		// HANDLED
		IsHandled = 0x10000,
		// MOUSE_ENTER
		Enter /*       */ = 0x00000000,
		// MOUSE_LEAVE
		Leave /*       */ = 0x00000001,
		// MOUSE_MOVE
		Move /*        */ = 0x00000002,
		// MOUSE_UP
		Up /*          */ = 0x00000003,
		// MOUSE_DOWN
		Down /*        */ = 0x00000004,
		// MOUSE_DCLICK
		DoubleClick /* */ = 0x00000005,
		// MOUSE_WHEEL
		Wheel /*       */ = 0x00000006,
		// MOUSE_TICK
		Tick /*        */ = 0x00000007,
		// MOUSE_IDLE
		Idle /*        */ = 0x00000008,
		// DROP
		Drop /*        */ = 0x00000009,
		// DRAG_ENTER
		DragEnter /*   */ = 0x0000000A,
		// DRAG_LEAVE
		DragLeave /*   */ = 0x0000000B,

		Click /*       */ = 0x000000FF,

		// DRAGGING
		IsDragging /*  */ = 0x00000100
	};

	// MOUSE_BUTTONS
	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	public enum HtmlEventMouseState : int
	{
		// MAIN_MOUSE_BUTTON
		Left = 1,
		// PROP_MOUSE_BUTTON
		Right = 2,
		// MIDDLE_MOUSE_BUTTON
		Middle = 4,

		Up = 1,
		Down = -1
	};

	// DRAGGING_TYPE
	public enum HtmlEventDragMode : int
	{
		// NO_DRAGGING
		None,
		// DRAGGING_MOVE
		Move,
		// DRAGGING_COPY
		Copy
	};

	// CURSOR_TYPE
	public enum HtmlCursorType : int
	{
		Arrow,
		IBeam,
		Wait,
		Cross,
		UpArrow,
		SizeNWSE,
		SizeNESW,
		SizeWE,
		SizeNS,
		SizeAll,
		No,
		AppStarting,
		Help,
		Hand,
		DragMove,
		DragCopy
	};

	// MOUSE_PARAMS
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlMouseEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlMouseEventCommand Command;
		public IntPtr HTarget;
		public Point PositionInTag;
		public Point PositionInDocument;
		[MarshalAs(UnmanagedType.U4)]
		public HtmlEventMouseState MouseState;
		[MarshalAs(UnmanagedType.U4)]
		public HtmlEventKeyboardState KeyboardState;
		[MarshalAs(UnmanagedType.U4)]
		public HtmlCursorType CursorType;
		public bool IsOverIcon;
		public IntPtr HDragTarget;
		[MarshalAs(UnmanagedType.U4)]
		public HtmlEventDragMode DragMode;
	};

	#endregion

	#region Keyboard

	// KEY_EVENTS
	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum HtmlKeyboardEventCommand : int
	{
		// BUBBLING
		IsBubbling = 0,
		// SINKING
		IsSinking = 0x08000,
		// HANDLED
		IsHandled = 0x10000,

		// KEY_DOWN
		Down /* */ = 0x00000000,
		// KEY_UP
		Up /*   */ = 0x00000001,
		// KEY_CHAR
		Char /* */ = 0x00000002
	};

	// KEYBOARD_STATES
	[Flags]
	public enum HtmlEventKeyboardState : int
	{
		ControlPressed = 1,
		ShiftPressed = 2,
		AltPressed = 4
	};

	// KEY_PARAMS
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlKeyboardEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlKeyboardEventCommand Command;
		public IntPtr HTarget;
		public int KeyCode;
		[MarshalAs(UnmanagedType.U4)]
		public HtmlEventKeyboardState KeyboardState;
	};

	#endregion

	#region Focus

	// FOCUS_EVENTS
	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum HtmlFocusEventCommand : int
	{
		// BUBBLING
		IsBubbling = 0,
		// SINKING
		IsSinking = 0x08000,
		// HANDLED
		IsHandled = 0x10000,

		// FOCUS_LOST
		Lost /* */ = 0x00000000,
		// FOCUS_GOT
		Got /*  */ = 0x00000001
	};

	// FOCUS_PARAMS
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlFocusEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlFocusEventCommand Command;
		public IntPtr HTarget;
		[MarshalAs(UnmanagedType.Bool)]
		public bool Mouse;
		[MarshalAs(UnmanagedType.Bool)]
		public bool Cancel;
	};

	#endregion

	#region Scroll

	// SCROLL_EVENTS
	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum HtmlScrollEventCommand : int
	{
		// BUBBLING
		IsBubbling = 0,
		// SINKING
		IsSinking = 0x08000,
		// HANDLED
		IsHandled = 0x10000,

		// SCROLL_HOME
		Home /*      */ = 0x00000000,
		// SCROLL_END
		End /*       */ = 0x00000001,
		// SCROLL_STEP_PLUS
		StepPlus /*  */ = 0x00000002,
		// SCROLL_STEP_MINUS
		StepMinus /* */ = 0x00000003,
		// SCROLL_PAGE_PLUS
		PagePlus /*  */ = 0x00000004,
		// SCROLL_PAGE_MINUS
		PageMinus /* */ = 0x00000005,
		// SCROLL_POS
		Position /*  */ = 0x00000006,
		// SCROLL_SLIDER_RELEASED
		Release /*   */ = 0x00000007
	};

	// SCROLL_PARAMS
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlScrollEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlScrollEventCommand Command;
		public IntPtr HTarget;
		public int Position;
		[MarshalAs(UnmanagedType.Bool)]
		public bool IsVertical;
	}

	#endregion

	#region Timer

	// TIMER_PARAMS
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlTimerEventParameters
	{
		public IntPtr ID;
	}

	#endregion

	#region Draw

	// DRAW_EVENTS
	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum HtmlDrawEventCommand : int
	{
		// BUBBLING
		IsBubbling = 0,
		// SINKING
		IsSinking = 0x08000,
		// HANDLED
		IsHandled = 0x10000,

		// DRAW_BACKGROUND
		Background /* */ = 0x00000000,
		// DRAW_CONTENT
		Content /*    */ = 0x00000001,
		// DRAW_FOREGROUND
		Foreground /* */ = 0x00000002
	};

	// DRAW_PARAMS
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlDrawEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlDrawEventCommand Command;
		public IntPtr HDC;
		public Rectangle Area;
		private int Reserved;
	};

	#endregion

	#region Data

	// DATA_ARRIVED_PARAMS
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlDataEventParameters
	{
		public IntPtr HTag;
		public IntPtr DataPointer;
		public int DataSize;
		public int DataType;
		public int Status;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string Uri;
	};

	#endregion

	#region Behavior

	// BEHAVIOR_EVENTS
	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Design", "CA1027:MarkEnumsWithFlags")]
	public enum HtmlBehaviorEventCommand : int
	{
		// BUBBLING
		IsBubbling = 0,
		// SINKING
		IsSinking = 0x08000,
		// HANDLED
		IsHandled = 0x10000,

		// BUTTON_CLICK
		ButtonClick /*             */ = 0x00000000,
		// BUTTON_PRESS
		ButtonPress /*             */ = 0x00000001,
		// BUTTON_STATE_CHANGED
		ButtonStateChanged /*      */ = 0x00000002,
		// EDIT_VALUE_CHANGING
		EditValueChanging /*       */ = 0x00000003,
		// EDIT_VALUE_CHANGED
		EditValueChanged /*        */ = 0x00000004,
		// SELECT_SELECTION_CHANGED
		SelectSelectionChanged /*  */ = 0x00000005,
		// SELECT_STATE_CHANGED
		SelectStateChanged /*      */ = 0x00000006,
		// POPUP_REQUEST
		PopupRequest /*            */ = 0x00000007,
		// POPUP_READY
		PopupReady /*              */ = 0x00000008,
		// POPUP_DISMISSED
		PopupDismissed /*          */ = 0x00000009,
		// MENU_ITEM_ACTIVE
		MenuItemActivate /*        */ = 0x0000000A,
		// MENU_ITEM_CLICK
		MenuItemClick /*           */ = 0x0000000B,
		// CONTEXT_MENU_SETUP
		ContextMenuValidate /*     */ = 0x0000000F,
		// CONTEXT_MENU_REQUEST
		ContextMenuRequest /*      */ = 0x00000010,
		// VISUAL_STATUS_CHANGED
		VisualStatusChanged /*     */ = 0x00000011,

		CustomActivateChild /*     */ = 0x00000100,

		CustomInitialize /*        */ = 0x00000110,
		CustomMeasure /*           */ = 0x00000120,
		CustomBindData /*          */ = 0x00000130,
		CustomExpandCollapse /*    */ = 0x00000140
	};

	// EVENT_REASON
	public enum HtmlEventReason : int
	{
		// BY_MOUSE_CLICK
		MouseClick,
		// BY_KEY_CLICK
		KeyClick,
		// SYNTHESIZED
		Synthesized,
		// BY_INS_CHAR
		InsertChar,
		// BY_INS_CHARS
		InsertChars,
		// BY_DEL_CHAR
		DeleteChar,
		// BY_DEL_CHARS
		DeleteChars,
	};

	// BEHAVIOR_EVENT_PARAMS
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlBehaviorEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlBehaviorEventCommand Command;
		public IntPtr HTarget;
		public IntPtr HSource;
		[MarshalAs(UnmanagedType.U4)]
		public HtmlEventReason Reason;
		public JsonValueData Value;
	};

	#endregion

	#endregion

	#region Behaviors

	// OutputStreamProc
	internal delegate int HtmlSelectionWriter(
		[In] IntPtr context,
		[In] int symbol);

	// BEHAVIOR_METHOD_IDENTIFIERS
	public enum HtmlTagMethodID : int
	{
		// DO_CLICK
		Click,
		// GET_TEXT_VALUE
		GetText,
		// SET_TEXT_VALUE
		SetText,
		// TEXT_EDIT_GET_SELECTION
		GetSelection,
		// TEXT_EDIT_SET_SELECTION
		SetSelection,
		// TEXT_EDIT_REPLACE_SELECTION
		ReplaceSelection,
		// SCROLL_BAR_GET_VALUE
		GetScrollValue,
		// SCROLL_BAR_SET_VALUE
		SetScrollValue,
		// TEXT_EDIT_GET_CARET_POSITION
		GetCaretPosition,
		// TEXT_EDIT_GET_SELECTION_TEXT
		GetSelectedText,
		// TEXT_EDIT_GET_SELECTION_HTML
		GetSelectedHTML,
		// XCALL
		XCall = 0xFF,
		// Nabu Specific
		ManagedCall = 0x100
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlMethodEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlTagMethodID MethodID;
	}

	// TEXT_VALUE_PARAMS
	// TEXT_EDIT_REPLACE_SELECTION_PARAMS
	[SuppressMessage("Microsoft.Design", "CA1049:TypesThatOwnNativeResourcesShouldBeDisposable")]
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlTextBoxValueMethodEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlTagMethodID MethodID;
		public IntPtr lpText;
		public int Length;
	};

	// TEXT_EDIT_SELECTION_PARAMS
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlTextBoxSelectionMethodEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlTagMethodID MethodID;
		public int SelectionStart;
		public int SelectionEnd;
	};

	// TEXT_CARET_POSITION_PARAMS
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlTextBoxCaretPositionMethodEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlTagMethodID MethodID;
		public Rectangle Position;
	};

	// SCROLLBAR_VALUE_PARAMS
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlScrollBarValueMethodEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlTagMethodID MethodID;
		public int Value;
		public int MinValue;
		public int MaxValue;
		public int PageValue;
		public int StepValue;
	};

	// TEXT_SELECTION_PARAMS
	[SuppressMessage("Microsoft.Design", "CA1049:TypesThatOwnNativeResourcesShouldBeDisposable")]
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlTextSelectionMethodEventParameters
	{
		[MarshalAs(UnmanagedType.U4)]
		public HtmlTagMethodID MethodID;
#if COMPACT_FRAMEWORK
		public IntPtr lpWriter;
#else
		[MarshalAs(UnmanagedType.FunctionPtr)]
		public HtmlSelectionWriter Writer;
#endif
	};

	// XCALL_PARAMS
	[SuppressMessage("Microsoft.Design", "CA1049:TypesThatOwnNativeResourcesShouldBeDisposable")]
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlXCallMethodEventParameters
	{
		public HtmlTagMethodID MethodID;
		public IntPtr lpMethodName;
		public int ArgumentCount;
		public IntPtr lpArgumentValues;
		public JsonValueData ReturnValue;
	};

	[SuppressMessage("Microsoft.Design", "CA1049:TypesThatOwnNativeResourcesShouldBeDisposable")]
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlManagedCallMethodEventParameters
	{
		public HtmlTagMethodID MethodID;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string MethodName;
		public int ArgumentCount;
		public IntPtr lpArgumentValues;
		public IntPtr ReturnValue;
	};

	#endregion

	#region Controls

	// CTL_TYPE
	public enum HtmlTagControlType : int
	{
		None,
		Unknown,
		TextBox_SingleLine,
		TextBox_Numeric,
		Button_Push,
		Button_Check,
		Button_Radio,
		Select_Single,
		Select_Multiple,
		Select_DropDown,
		TextBox_MultiLine,
		HtmlBox,
		TextBox_Password,
		Progress,
		Slider,
		TextBox_Decimal,
		TextBox_Currency,
		Scrollbar,
		TextBox_Hyperlink,
		Menu_MenuBar,
		Menu_Menu,
		Menu_MenuButton,
		DateTime_Calendar,
		DateTime_Date,
		DateTime_Time,
		Frame_Frame,
		Frame_FrameSet,
		Graphics,
		Sprite,
		List,
		RichText,
		Tooltip,
		Hidden,
		TextBox_Url,
		Toolbar,
		Form
	};

	#endregion
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
