﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.InteropServices;

namespace Nabu.Forms.Html
{
	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	public enum HtmlLayoutMode : int
	{
		// HLM_LAYOUT_ONLY
		LayoutOnly,
		// HLM_SHOW_SELECTION
		ShowSelection,
	};

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	public enum HtmlScrollMode : int
	{
		Default,
		Smooth
	};

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	public enum HtmlFontMode : int
	{
		SystemDefault,
		None,
		AntiAlias,
		ClearType
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	internal enum HtmlNotificationKind : int
	{
		// HLN_CREATE_CONTROL
		ControlCreating /*     */ = 0x0B00,
		// HLN_LOAD_DATA
		ResourceLoading /*     */ = 0x0B01,
		// HLN_CONTROL_CREATED
		ControlCreated /*      */ = 0x0B02,
		// HLN_DATA_LOADED
		ResourceLoaded /*      */ = 0x0B03,
		// HLN_DOCUMENT_COMPLETE
		DocumentLoaded /*      */ = 0x0B04,
		// HLN_UPDATE_UI
		UpdateUI /*            */ = 0x0B05,
		// HLN_DESTROY_CONTROL
		ControlDestroyed /*    */ = 0x0B06,
		// HLN_ATTACH_BEHAVIOR
		BehaviorAttaching /*   */ = 0x0B07,
		// HLN_BEHAVIOR_CHANGED
		BehaviorChanged /*     */ = 0x0B08,
		// HLN_DIALOG_CREATED
		DialogCreated /*       */ = 0x0B09,
		// HLN_DIALOG_CLOSE_RQ
		DialogDestroying /*    */ = 0x0B0A,
		// HLN_DOCUMENT_LOADED
		DocumentParsed /*      */ = 0x0B0B,
	}

	// HTMLAYOUT_NOTIFY
	internal delegate int HtmlWindowNotificationMessageHandler(
		[In] int message,
		[In] IntPtr wParam,
		[In] IntPtr lParam,
		[In] IntPtr vParam);

	// NMHL_CREATE_CONTROL
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlNotificationControlCreating
	{
		public NMHeader Header;
		public IntPtr hTag;
		public IntPtr hWindowHtml;
		public IntPtr hWindowChild;
		private uint Reserved1;
		private uint Reserved2;
	}

	// NMHL_LOAD_DATA
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlNotificationResourceLoading
	{
		public NMHeader Header;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string Uri;
		public IntPtr Data;
		public int DataSize;
		public HtmlResourceType DataType;
		public IntPtr hTagPrincipal;
		public IntPtr hTagInititator;
	}

	// NMHL_CREATE_CONTROL
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlNotificationControlCreated
	{
		public NMHeader Header;
		public IntPtr hTag;
		public IntPtr hWindowHtml;
		public IntPtr hWindowChild;
		private uint Reserved1;
		private uint Reserved2;
	}

	// NMHL_DATA_LOADED
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlNotificationResourceLoaded
	{
		public NMHeader Header;
		[MarshalAs(UnmanagedType.LPWStr)]
		public string Uri;
		public IntPtr Data;
		public int DataSize;
		public HtmlResourceType DataType;
		public int Status;
	}

	// NMHL_DESTROY_CONTROL
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlNotificationControlDestroying
	{
		public NMHeader Header;
		public IntPtr hTag;
		public IntPtr hWindowChild;
		private uint Reserved;
	}

#if COMPACT_FRAMEWORK
	// NMHL_ATTACH_BEHAVIOR
	[SuppressMessage("Microsoft.Design", "CA1049:TypesThatOwnNativeResourcesShouldBeDisposable")]
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlNotificationBehaviorAttaching
	{
		public NMHeader Header;
		public IntPtr hTag;
		public IntPtr lpBehaviorName;
		public IntPtr lpEventProc;
		public IntPtr Tag;
		[MarshalAs(UnmanagedType.U4)]
		public HtmlEventGroup EventGroup;
	}
#else
	// NMHL_ATTACH_BEHAVIOR
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlNotificationBehaviorAttaching
	{
		public NMHeader Header;
		public IntPtr hTag;
		[MarshalAs(UnmanagedType.LPStr)]
		public string BehaviorName;
		public DOM_EventHandler EventProc;
		public IntPtr Tag;
		[MarshalAs(UnmanagedType.U4)]
		public HtmlEventGroup EventGroup;
	}
#endif

#if COMPACT_FRAMEWORK
	// NMHL_BEHAVIOR_CHANGED
	[SuppressMessage("Microsoft.Design", "CA1049:TypesThatOwnNativeResourcesShouldBeDisposable")]
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlNotificationBehaviorChanged
	{
		public NMHeader Header;
		public IntPtr hTag;
		public IntPtr lpOldBehaviorNames;
		public IntPtr lpNewBehaviorNames;
	}
#else
	// NMHL_BEHAVIOR_CHANGED
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlNotificationBehaviorChanged
	{
		public NMHeader Header;
		public IntPtr hTag;
		[MarshalAs(UnmanagedType.LPStr)]
		public string OldBehaviorNames;
		[MarshalAs(UnmanagedType.LPStr)]
		public string NewBehaviorNames;
	}
#endif

	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlNotificationDialogCreated
	{
		public NMHeader Header;
	}

	// NMHL_DIALOG_CLOSE_RQ
	[StructLayout(LayoutKind.Sequential)]
	internal struct HtmlNotificationDialogDestroying
	{
		public NMHeader Header;
		[MarshalAs(UnmanagedType.Bool)]
		public bool Cancel;
	}

#if COMPACT_FRAMEWORK
	// HTMLAYOUT_CALLBACK_RES
	internal delegate bool HtmlWindowResourceEnumHandler(
		[In][MarshalAs(UnmanagedType.LPWStr)] string uri,
		[In] IntPtr lpType,
		[In][MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 3)] byte[] imageData,
		[In] int imageDataSize);
#else
	// HTMLAYOUT_CALLBACK_RES
	internal delegate bool HtmlWindowResourceEnumHandler(
		[In][MarshalAs(UnmanagedType.LPWStr)] string uri,
		[In][MarshalAs(UnmanagedType.LPStr)] string type,
		[In][MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 3)] byte[] imageData,
		[In] int imageDataSize);
#endif
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
