﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.Drawing;
using System.Windows.Forms;

namespace Nabu.Forms.Html
{
	public class HtmlPrinterNotificationHandler : IHtmlHostNotificationHandler
	{
		#region HtmlPrinterNotificationHandler

		#region Methods

		[DebuggerStepThrough]
		public HtmlPrinterNotificationHandler()
		{
		}

		#endregion

		#endregion

		#region IHtmlPrinterNotificationHandler

		public virtual bool OnControlCreating(
			IntPtr hTag,
			IntPtr hWindowHtml,
			ref IntPtr hWindowChild)
		{
			return false;
		}

		[SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
		public virtual bool OnResourceLoading(
			Uri resourceUrl,
			HtmlResourceType resourceUriType,
			out byte[] resourceData)
		{
			resourceUrl = HtmlProtocolManager.GetUrlRedirection(resourceUrl);
			resourceData = null;

			HtmlProtocolBase htmlProtocol =
				HtmlProtocolManager.GetProtocol(resourceUrl.Scheme);

			if (htmlProtocol != null)
			{
				try
				{
					resourceData =
						htmlProtocol.LoadResource(
							resourceUrl,
							resourceUriType);
				}
				catch (Exception exception)
				{
					resourceData =
						HtmlProtocolManager.OnObjectLoadFailure(
							resourceUrl,
							resourceUriType,
							exception);
				}
			}
			else
			{
				// http://code.google.com/p/nabu-library/issues/detail?id=9
				if (resourceUrl.Scheme != "data")
					resourceData =
						HtmlProtocolManager.OnProtocolLoadFailure(
							resourceUrl.Scheme,
							resourceUriType);
			}

			return resourceData.Length > 0;
		}

		public virtual void OnControlCreated(
			IntPtr hTag,
			IntPtr hWndHtml,
			IntPtr hWndChild)
		{
		}

		public virtual void OnResourceLoaded(
			Uri resourceUrl,
			HtmlResourceType resourceUriType,
			int status)
		{
		}

		public virtual void OnDocumentLoaded()
		{
		}

		public virtual void OnUpdateUI()
		{
		}

		public virtual bool OnControlDestroyed(
			IntPtr hTag,
			ref IntPtr hWndChild)
		{
			return true;
		}

		public virtual void OnBehaviorChanged(
			string[] oldNames,
			string[] newNames)
		{
		}

		public virtual void OnDialogCreated(
			IntPtr hWindowHtml)
		{
		}

		public virtual bool OnDialogDestroying(
			IntPtr hWindowHtml,
			ref bool cancel)
		{
			return false;
		}

		public virtual void OnDocumentParsed()
		{
		}

		public virtual bool OnHyperLink(
			HtmlTag htmlTag,
			string uri,
			string target,
			Keys keys)
		{
			return false;
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
