﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Globalization;
#if !COMPACT_FRAMEWORK
using System.Runtime.Serialization;
#endif

namespace Nabu.Forms.Html
{
#if !COMPACT_FRAMEWORK
	[Serializable]
#endif
	public sealed class HtmlPrintException : Exception
	{
		#region Utility

		private static string MessageFromCode(int errorCode)
		{
			switch ((HtmlResult)errorCode)
			{
				case HtmlResult.Print_Success:
					return "No error.";
				case HtmlResult.Print_Failure_InvalidHandle:
					return "Invalid handle.";
				case HtmlResult.Print_Failure_InvalidFormat:
					return "Invalid format.";
				case HtmlResult.Print_Failure_FileNotFound:
					return "File not found.";
				case HtmlResult.Print_Failure_InvalidParameter:
					return "Invalid parameter.";
				case HtmlResult.Print_Failure_InvalidState:
					return "Invalid state.";
				default:
					return
						string.Format(
							CultureInfo.InvariantCulture,
							"Unknown error code {0}.",
							errorCode);
			}
		}

		public static void ThrowOnError(int errorCode)
		{
			switch ((HtmlResult)errorCode)
			{
				case HtmlResult.Print_Success:
					break;
				default:
					throw new HtmlPrintException(errorCode);
			}
		}

		#endregion

		#region Constructors

		internal HtmlPrintException(int errorCode)
			: base(MessageFromCode(errorCode))
		{
		}

		public HtmlPrintException()
		{
		}

		public HtmlPrintException(string message)
			: base(message)
		{
		}

		public HtmlPrintException(string message, Exception innerException) :
			base(message, innerException)
		{
		}

#if !COMPACT_FRAMEWORK
		private HtmlPrintException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}
#endif

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
