﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Drawing;
using System.Runtime.InteropServices;

namespace Nabu.Forms.Html
{
	public static class HtmlWindowEngine
	{
		#region Traverse Event

		public static bool TraverseEvent(
			HtmlMouseEventCommand command,
			HtmlTag target,
			Point positionInTag,
			Point positionInDocument,
			HtmlEventMouseState mouseState,
			HtmlEventKeyboardState keyboardState,
			HtmlCursorType cursorType,
			bool isOverIcon,
			HtmlTag dragTarget,
			HtmlEventDragMode dragMode)
		{
			HtmlMouseEventParameters parameters = new HtmlMouseEventParameters();

			parameters.Command = command;
			parameters.HTarget = (target == null) ? IntPtr.Zero : target.Handle;
			parameters.PositionInTag = positionInTag;
			parameters.PositionInDocument = positionInDocument;
			parameters.MouseState = mouseState;
			parameters.KeyboardState = keyboardState;
			parameters.CursorType = cursorType;
			parameters.IsOverIcon = isOverIcon;
			parameters.HDragTarget = (dragTarget == null) ? IntPtr.Zero : dragTarget.Handle;
			parameters.DragMode = dragMode;

			bool isHandled;

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Window.TraverseEvent(
					ref parameters,
					out isHandled));

			return isHandled;
		}

		public static bool TraverseEvent(
			HtmlKeyboardEventCommand command,
			HtmlTag target,
			int keyCode,
			HtmlEventKeyboardState keyboardState)
		{
			HtmlKeyboardEventParameters parameters = new HtmlKeyboardEventParameters();

			parameters.Command = command;
			parameters.HTarget = (target == null) ? IntPtr.Zero : target.Handle;
			parameters.KeyCode = keyCode;
			parameters.KeyboardState = keyboardState;

			bool isHandled;

			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Window.TraverseEvent(
					ref parameters,
					out isHandled));

			return isHandled;
		}

		#endregion

		#region Get Tag

		internal static HtmlTag GetTagFromPoint(IntPtr hWindow, Point point)
		{
			IntPtr hTag;

			if (HtmlDynamicInterop.Window.GetTagFromPoint(
					hWindow,
					point,
					out hTag) == (int)HtmlResult.DOM_Success)
			{
				return HtmlTag.FromHandle(hTag);
			}
			else
			{
				return null;
			}
		}

		internal static HtmlTag GetRootTag(IntPtr hWindow)
		{
			IntPtr hTag;

			if (HtmlDynamicInterop.Window.GetRootTag(
				hWindow,
				out hTag) == (int)HtmlResult.Print_Success)
			{
				if (hTag != IntPtr.Zero)
				{
					return HtmlTag.FromHandle(hTag);
				}
			}

			return null;
		}

		internal static HtmlTag GetFocusTag(IntPtr hWindow)
		{
			IntPtr hTag;

			if (HtmlDynamicInterop.Window.GetFocusTag(
					hWindow,
					out hTag) == (int)HtmlResult.DOM_Success)
			{
				return HtmlTag.FromHandle(hTag);
			}

			return null;
		}

		internal static string GetSelectedHtml(IntPtr hWindow)
		{
			if (HtmlDynamicInterop.Window.IsSelectionExist(hWindow))
			{
				int htmlLength;
				IntPtr htmlPtr = HtmlDynamicInterop.Window.GetSelectedHTML(hWindow, out htmlLength);
				byte[] htmlData = new byte[htmlLength];

				Marshal.Copy(htmlPtr, htmlData, 0, htmlLength);

				return HtmlProtocolBase.StringFromBinaryData(htmlData);
			}
			else
			{
				return string.Empty;
			}
		}

		#endregion

		#region Set Option

		internal static void SetLayoutMode(IntPtr hWindow, HtmlLayoutMode layoutMode)
		{
			HtmlDynamicInterop.Window.SetMode(
				hWindow,
				layoutMode);
		}

		internal static void SetScrollMode(IntPtr hWindow, HtmlScrollMode scrollMode)
		{
			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Window.SetOption(
					hWindow,
					HtmlEngineOption.ScrollSmoothing,
					(scrollMode == HtmlScrollMode.Smooth) ? 1 : 0) ? 0 : 1);
		}

		internal static void SetFontSmoothingMode(IntPtr hWindow, HtmlFontMode mode)
		{
			HtmlDomException.ThrowOnError(
				HtmlDynamicInterop.Window.SetOption(
					hWindow,
					HtmlEngineOption.FontSmoothing,
					(int)mode) ? 0 : 1);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
