﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace Nabu.Forms.Html
{
	internal static class HtmlWindowNotificationDispatcher
	{
		#region Fields

		private static HtmlWindowNotificationMessageHandler _notificationCallback = new HtmlWindowNotificationMessageHandler(OnNotify);

		#endregion

		#region Methods

		private static int OnNotify(int message, IntPtr wParam, IntPtr lParam, IntPtr vParam)
		{
			Control control = (Control)((GCHandle)vParam).Target;
			IHtmlHost htmlHost = control as IHtmlHost;

			if (htmlHost != null)
			{
				IHtmlHostNotificationHandler notificationHandler = htmlHost.NotificationHandler;

				if (notificationHandler != null)
				{
					NMHeader notificationHeader = (NMHeader)Marshal.PtrToStructure(lParam, typeof(NMHeader));

					switch ((HtmlNotificationKind)notificationHeader.Code)
					{
						case HtmlNotificationKind.ControlCreating:
							{
								HtmlNotificationControlCreating notificationData =
									(HtmlNotificationControlCreating)Marshal.PtrToStructure(
										lParam,
										typeof(HtmlNotificationControlCreating));

								if (notificationHandler.OnControlCreating(
									notificationData.hTag,
									notificationData.hWindowHtml,
									ref notificationData.hWindowChild))
								{
									Marshal.StructureToPtr(notificationData, lParam, false);

									// HWND_DISCARD_CREATION
									return 1;
								}
							}
							break;
						case HtmlNotificationKind.ResourceLoading:
							{
								HtmlNotificationResourceLoading notificationData =
									(HtmlNotificationResourceLoading)Marshal.PtrToStructure(
										lParam,
										typeof(HtmlNotificationResourceLoading));

								byte[] resourceData = null;

								if (notificationHandler.OnResourceLoading(
									new Uri(notificationData.Uri),
									notificationData.DataType,
									out resourceData))
								{
									HtmlDynamicInterop.Window.ResourceReady(
										htmlHost.HtmlHandle,
										notificationData.Uri,
										resourceData,
										resourceData.Length);

									// LOAD_DISCARD
									return 1;
								}
							}
							break;
						case HtmlNotificationKind.ControlCreated:
							{
								HtmlNotificationControlCreating notificationData =
									(HtmlNotificationControlCreating)Marshal.PtrToStructure(
										lParam,
										typeof(HtmlNotificationControlCreating));

								notificationHandler.OnControlCreated(
									notificationData.hTag,
									notificationData.hWindowHtml,
									notificationData.hWindowChild);
							}
							break;
						case HtmlNotificationKind.ResourceLoaded:
							{
								HtmlNotificationResourceLoaded notificationData =
									(HtmlNotificationResourceLoaded)Marshal.PtrToStructure(
										lParam,
										typeof(HtmlNotificationResourceLoaded));

								notificationHandler.OnResourceLoaded(
									new Uri(notificationData.Uri),
									notificationData.DataType,
									notificationData.Status);
							}
							break;
						case HtmlNotificationKind.DocumentLoaded:
							notificationHandler.OnDocumentLoaded();
							break;
						case HtmlNotificationKind.UpdateUI:
							notificationHandler.OnUpdateUI();
							break;
						case HtmlNotificationKind.ControlDestroyed:
							{
								HtmlNotificationControlDestroying notificationData =
									(HtmlNotificationControlDestroying)Marshal.PtrToStructure(
										lParam,
										typeof(HtmlNotificationControlDestroying));

								if (notificationHandler.OnControlDestroyed(
									notificationData.hTag,
									ref notificationData.hWindowChild))
								{
									Marshal.StructureToPtr(notificationData, lParam, false);

									return 1;
								}
							}
							break;
						case HtmlNotificationKind.BehaviorAttaching:
							{
								HtmlNotificationBehaviorAttaching notificationData =
									(HtmlNotificationBehaviorAttaching)Marshal.PtrToStructure(
										lParam,
										typeof(HtmlNotificationBehaviorAttaching));

#if COMPACT_FRAMEWORK
								HtmlBehaviorBase htmlBehavior =
									HtmlBehaviorManager.GetBehavior(
										MarshalCF.PtrToStringAnsi(notificationData.lpBehaviorName),
										htmlHost);
#else
								HtmlBehaviorBase htmlBehavior =
									HtmlBehaviorManager.GetBehavior(
										notificationData.BehaviorName,
										htmlHost);
#endif

								if (htmlBehavior != null)
								{
									htmlBehavior.Attach(ref notificationData);
									Marshal.StructureToPtr(notificationData, lParam, false);

									return 1;
								}
							}
							break;
						case HtmlNotificationKind.BehaviorChanged:
							{
								HtmlNotificationBehaviorChanged notificationData =
									(HtmlNotificationBehaviorChanged)Marshal.PtrToStructure(
										lParam,
										typeof(HtmlNotificationBehaviorChanged));

#if COMPACT_FRAMEWORK
								notificationHandler.OnBehaviorChanged(
									MarshalCF.PtrToStringAnsi(notificationData.lpOldBehaviorNames).Split(' '),
									MarshalCF.PtrToStringAnsi(notificationData.lpNewBehaviorNames).Split(' '));
#else
								notificationHandler.OnBehaviorChanged(
									notificationData.OldBehaviorNames.Split(' '),
									notificationData.NewBehaviorNames.Split(' '));
#endif
							}
							break;
						case HtmlNotificationKind.DialogCreated:
							{
								HtmlNotificationDialogCreated notificationData =
									(HtmlNotificationDialogCreated)Marshal.PtrToStructure(
										lParam,
										typeof(HtmlNotificationDialogCreated));

								notificationHandler.OnDialogCreated(
									notificationData.Header.HWnd);
							}
							break;
						case HtmlNotificationKind.DialogDestroying:
							{
								HtmlNotificationDialogDestroying notificationData =
									(HtmlNotificationDialogDestroying)Marshal.PtrToStructure(
										lParam,
										typeof(HtmlNotificationDialogDestroying));

								if (notificationHandler.OnDialogDestroying(
									notificationData.Header.HWnd,
									ref notificationData.Cancel))
								{
									Marshal.StructureToPtr(notificationData, lParam, false);

									return 1;
								}
							}
							break;
						case HtmlNotificationKind.DocumentParsed:
							{
								notificationHandler.OnDocumentParsed();
							}
							break;
					}
				}
			}

			return 0;
		}

#if !COMPACT_FRAMEWORK
		public static bool WindowProc(ref Message message)
		{
			bool isHandled;
			IntPtr result =
				HtmlDynamicInterop.Window.WindowProcNoDefault(
					message.HWnd,
					message.Msg,
					message.WParam,
					message.LParam,
					out isHandled);

			if (isHandled)
			{
				message.Result = result;
			}

			return isHandled;
		}
#endif

		#endregion

		#region Properties

		public static HtmlWindowNotificationMessageHandler NotificationCallback
		{
			[DebuggerStepThrough]
			get { return _notificationCallback; }
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
