﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms.Html Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace Nabu.Forms.Html
{
	public class HtmlWindowlessNotificationHandler : IHtmlHostNotificationHandler
	{
		#region HtmlWindowNotificationHandler

		public HtmlWindowlessNotificationHandler()
		{
		}

		#endregion

		#region IHtmlHostNotificationHandler

		public bool OnControlCreating(
			IntPtr hTag,
			IntPtr hWindowHtml,
			ref IntPtr hWndChild)
		{
			return false;
		}

		[SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
		public bool OnResourceLoading(
			Uri resourceUrl,
			HtmlResourceType resourceUriType,
			out byte[] resourceData)
		{
			resourceUrl = HtmlProtocolManager.GetUrlRedirection(resourceUrl);
			resourceData = null;

			HtmlProtocolBase htmlProtocol =
				HtmlProtocolManager.GetProtocol(resourceUrl.Scheme);

			if (htmlProtocol != null)
			{
				try
				{
					resourceData =
						htmlProtocol.LoadResource(
							resourceUrl,
							resourceUriType);
				}
				catch (Exception exception)
				{
					resourceData =
						HtmlProtocolManager.OnObjectLoadFailure(
							resourceUrl,
							resourceUriType,
							exception);
				}
			}
			else
			{
				// http://code.google.com/p/nabu-library/issues/detail?id=9
				if (resourceUrl.Scheme != "data")
					resourceData =
						HtmlProtocolManager.OnProtocolLoadFailure(
							resourceUrl.Scheme,
							resourceUriType);
			}

			return resourceData.Length > 0;
		}

		public void OnControlCreated(
			IntPtr hTag,
			IntPtr hWindowHtml,
			IntPtr hWindowChild)
		{
		}

		public void OnResourceLoaded(
			Uri resourceUrl,
			HtmlResourceType resourceUriType,
			int status)
		{
		}

		public void OnDocumentLoaded()
		{
		}

		public void OnUpdateUI()
		{
		}

		public bool OnControlDestroyed(
			IntPtr hTag,
			ref IntPtr hWindowChild)
		{
			return false;
		}

		public void OnBehaviorChanged(
			string[] oldNames,
			string[] newNames)
		{
		}

		public void OnDialogCreated(
			IntPtr hWindowHtml)
		{
		}

		public bool OnDialogDestroying(
			IntPtr hWindowHtml,
			ref bool cancel)
		{
			return false;
		}

		public void OnDocumentParsed()
		{
		}

		public bool OnHyperLink(
			HtmlTag htmlTag,
			string uri,
			string target,
			Keys keys)
		{
			if (!string.IsNullOrEmpty(uri))
			{
				if (uri.StartsWith("#", StringComparison.OrdinalIgnoreCase))
				{
					using (HtmlTag root = htmlTag.Root)
					{
						using (HtmlTag anchor =
							root.SelectFirst(
								string.Format(
									CultureInfo.InvariantCulture,
									"[id='{0}'],[name='{0}']",
									uri.Substring(1))))
						{
							if (anchor != null)
							{
								anchor.ScrollToView(HtmlScrollBehavior.ToTop);
							}
						}
					}
				}
			}

			return true;
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
