﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms Library                            */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace Nabu.Forms
{
	public partial class ImageMap : PictureBox
	{
		#region PictureMapBox

		#region Fields

		private PointF _selectionMin = new PointF();
		private PointF _selectionMax = new PointF();
		private SizeF _selectionRange = new SizeF();
		private PointF _selectionPosition = new PointF();
		private Image _selectionImage;
		private Point _selectionImagePosition = new Point();
		private bool _xSelectable = true;
		private bool _ySelectable = true;

		#endregion

		#region Methods

		private void ValidateSelectionPosition(PointF selectionPosition)
		{
			selectionPosition.X = Math.Max(selectionPosition.X, this._selectionMin.X);
			selectionPosition.X = Math.Min(selectionPosition.X, this._selectionMax.X);

			selectionPosition.Y = Math.Max(selectionPosition.Y, this._selectionMin.Y);
			selectionPosition.Y = Math.Min(selectionPosition.Y, this._selectionMax.Y);

			if (!this._xSelectable)
			{
				selectionPosition.X = this._selectionMin.X + (this._selectionRange.Width / 2.0F);
			}

			if (!this._ySelectable)
			{
				selectionPosition.Y = this._selectionMin.Y + (this._selectionRange.Height / 2.0F);
			}

			if (this._selectionPosition != selectionPosition)
			{
				this._selectionPosition = selectionPosition;

				if (this._selectionImage != null)
				{
					base.Invalidate(
						new Rectangle(this._selectionImagePosition, this._selectionImage.Size),
						false);
					UpdateSelectionImagePosition();
					base.Invalidate(
						new Rectangle(this._selectionImagePosition, this._selectionImage.Size),
						false);
					base.Update();
				}

				if (this.SelectionPositionChanged != null)
				{
					this.SelectionPositionChanged(this, EventArgs.Empty);
				}
			}
		}

		private void UpdateSelectionImagePosition()
		{
			if (
				(this._selectionImage != null) &&
				(this._selectionRange.Width > 0) &&
				(this._selectionRange.Height > 0))
			{
				this._selectionImagePosition = new Point(
					(int)((this._selectionPosition.X - this._selectionMin.X) * base.ClientSize.Width / this._selectionRange.Width) - this._selectionImage.Width / 2,
					(int)((this._selectionPosition.Y - this._selectionMin.Y) * base.ClientSize.Height / this._selectionRange.Height) - this._selectionImage.Height / 2);
			}
		}

		public ImageMap()
		{
		}

		#endregion

		#region Properties

		[Localizable(true)]
		[Bindable(true)]
		[Category("Appearance")]
		public Image SelectionImage
		{
			get
			{
				return this._selectionImage;
			}

			set
			{
				this._selectionImage = value;
				UpdateSelectionImagePosition();
			}
		}

		[Category("Behavior")]
		public PointF SelectionMin
		{
			get
			{
				return this._selectionMin;
			}

			set
			{
				this._selectionMin = value;
				this._selectionRange.Width = this._selectionMax.X - this._selectionMin.X;
				this._selectionRange.Height = this._selectionMax.Y - this._selectionMin.Y;
				ValidateSelectionPosition(this._selectionPosition);
			}
		}

		[Category("Behavior")]
		public PointF SelectionMax
		{
			get
			{
				return this._selectionMax;
			}

			set
			{
				this._selectionMax = value;
				this._selectionRange.Width = this._selectionMax.X - this._selectionMin.X;
				this._selectionRange.Height = this._selectionMax.Y - this._selectionMin.Y;
				ValidateSelectionPosition(this._selectionPosition);
			}
		}

		[Category("Behavior")]
		public PointF SelectionPosition
		{
			get
			{
				return this._selectionPosition;
			}

			set
			{
				ValidateSelectionPosition(value);
			}
		}

		[Category("Behavior")]
		public bool XSelectable
		{
			get
			{
				return this._xSelectable;
			}

			set
			{
				this._xSelectable = value;
				ValidateSelectionPosition(this._selectionPosition);
			}
		}

		[Category("Behavior")]
		public bool YSelectable
		{
			get
			{
				return this._ySelectable;
			}

			set
			{
				this._ySelectable = value; ValidateSelectionPosition(this._selectionPosition);
			}
		}

		#endregion

		#region Events

		[Category("Property Changed")]
		public event EventHandler SelectionPositionChanged;

		#endregion

		#endregion

		#region Control

		protected override void OnPaint(PaintEventArgs pe)
		{
			base.OnPaint(pe);

			if (this._selectionImage != null)
			{
				pe.Graphics.DrawImage(this._selectionImage, this._selectionImagePosition);
			}
		}

		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.Capture = true;
			this.SelectionPosition = new PointF(
				(e.X * this._selectionRange.Width / base.ClientSize.Width) + this._selectionMin.X,
				(e.Y * this._selectionRange.Height / base.ClientSize.Height) + this._selectionMin.Y);
			base.OnMouseDown(e);
		}

		protected override void OnMouseUp(MouseEventArgs e)
		{
			base.Capture = false;
			base.OnMouseUp(e);
		}

		protected override void OnMouseMove(MouseEventArgs e)
		{
			if (base.Capture)
			{
				this.SelectionPosition = new PointF(
					(e.X * this._selectionRange.Width / base.ClientSize.Width) + this._selectionMin.X,
					(e.Y * this._selectionRange.Height / base.ClientSize.Height) + this._selectionMin.Y);
			}

			base.OnMouseMove(e);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
