﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms Library                            */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace Nabu.Forms
{
	public sealed class ImageView : ScrollableView
	{
		#region PictureView

		#region Fields

		private Image _image;
		private Point _contentPosition;
		private Point _mousePosition;

		#endregion

		#region Methods

		#region Event Raisers

		private void OnImageChanged(EventArgs e)
		{
			if (this.ImageChanged != null)
			{
				this.ImageChanged(this, e);
			}
		}

		#endregion

		#region Constructors

		public ImageView()
		{
			//base.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			base.SetStyle(ControlStyles.ContainerControl, false);
			base.SetStyle(ControlStyles.FixedHeight, false);
			base.SetStyle(ControlStyles.FixedWidth, false);
			base.SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
			base.SetStyle(ControlStyles.ResizeRedraw, true);
			base.SetStyle(ControlStyles.Selectable, false);
			base.SetStyle(ControlStyles.UserMouse, true);
			base.SetStyle(ControlStyles.UserPaint, true);
			base.SetStyle(ControlStyles.Selectable, true);
		}

		#endregion

		#endregion

		#region Properties

		[Browsable(true)]
		[Category("Appearance")]
		public Image Image
		{
			get
			{
				return this._image;
			}

			set
			{
				this._image = value;

				if (this._image != null)
				{
					base.ContentSize = this._image.Size;
				}
				else
				{
					base.ContentSize = Size.Empty;
				}

				base.ContentZoom = 1.0F;
				OnImageChanged(EventArgs.Empty);
			}
		}

		#endregion

		#region Events

		public event EventHandler ImageChanged;

		#endregion

		#endregion

		#region ScrollableView

		protected override void OnContentPositionChanged(EventArgs e)
		{
			base.Refresh();

			base.OnContentPositionChanged(e);
		}

		protected override void OnContentZoomChanged(EventArgs e)
		{
			base.Refresh();

			base.OnContentZoomChanged(e);
		}

		protected override void OnScroll(ScrollEventArgs e)
		{
			base.Refresh();

			base.OnScroll(e);
		}

		#endregion

		#region Control

		protected override void OnMouseDown(MouseEventArgs e)
		{
			switch (e.Button)
			{
				case MouseButtons.Left:
					this._contentPosition = base.ContentPosition;
					this._mousePosition = e.Location;
					base.Cursor = Cursors.SizeAll;
					base.Capture = true;
					break;
			}

			base.OnMouseDown(e);
		}

		protected override void OnMouseUp(MouseEventArgs e)
		{
			switch (e.Button)
			{
				case MouseButtons.Left:
					if (base.Capture)
					{
						base.Cursor = base.DefaultCursor;
						base.Capture = false;
					}
					break;
			}

			base.OnMouseUp(e);
		}

		protected override void OnMouseMove(MouseEventArgs e)
		{
			if (base.Capture)
			{
				base.ContentPosition =
					new Point(
						this._contentPosition.X + (int)((this._mousePosition.X - e.X) / this.ContentZoom),
						this._contentPosition.Y + (int)((this._mousePosition.Y - e.Y) / this.ContentZoom));
			}

			base.OnMouseMove(e);
		}

		protected override void OnMouseWheel(MouseEventArgs e)
		{
			if ((Control.ModifierKeys & Keys.Control) == Keys.Control)
			{
				if (e.Delta > 0)
				{
					base.SetContentZoom(e.Location, base.ContentZoom * 1.4142135623730950488016887242097F);
				}
				else
				{
					base.SetContentZoom(e.Location, base.ContentZoom * 0.70710678118654752440084436210485F);
				}
			}

			// base.OnMouseWheel(e);
		}

		protected override void OnPaint(PaintEventArgs pe)
		{
			if (this._image != null)
			{
				Rectangle destinationRectangle =
					new Rectangle(
						Point.Empty,
						base.ClientSize);

				pe.Graphics.DrawImage(
					this._image,
					destinationRectangle,
					base.ViewPort,
					GraphicsUnit.Pixel);
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
