﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms Library                            */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Design;
using System.Drawing.Printing;
using System.Runtime.InteropServices;
using System.Windows.Forms;

using Nabu.Forms.TextObjectModel;

namespace Nabu.Forms
{
	public sealed class RichTextView : RichTextBox
	{
		#region Methods

		private static int HundredthInchToTwips(int inch)
		{
			return (int)(14.4 * inch);
		}

		public RichTextView()
		{
		}

		public RichTextView(IContainer container)
		{
			container.Add(this);
		}

		public void PrintStart()
		{
		}

		public void PrintEnd()
		{
			NativeMethods.Window_SendMessage(Handle, WindowMessage.EM_FORMATRANGE, IntPtr.Zero, IntPtr.Zero);
		}

		public int PrintPage(PrintPageEventArgs printPageEventArgs, bool measureOnly, int startSymbol, int endSymbol)
		{
			FormatRange formatRange;
			IntPtr hDC = IntPtr.Zero;
			IntPtr lParam = IntPtr.Zero;
			int result = 0;

			try
			{
				hDC = printPageEventArgs.Graphics.GetHdc();

				formatRange.CharRange.Start = startSymbol;
				formatRange.CharRange.End = endSymbol;
				formatRange.HDC = hDC;
				formatRange.HDCTarget = hDC;
				formatRange.RectMargin = Rectangle.FromLTRB(
					HundredthInchToTwips(printPageEventArgs.MarginBounds.Left),
					HundredthInchToTwips(printPageEventArgs.MarginBounds.Top),
					HundredthInchToTwips(printPageEventArgs.MarginBounds.Right),
					HundredthInchToTwips(printPageEventArgs.MarginBounds.Bottom));

				formatRange.RectPage = Rectangle.FromLTRB(
					HundredthInchToTwips(printPageEventArgs.PageBounds.Left),
					HundredthInchToTwips(printPageEventArgs.PageBounds.Top),
					HundredthInchToTwips(printPageEventArgs.PageBounds.Right),
					HundredthInchToTwips(printPageEventArgs.PageBounds.Bottom));

				using (NativeMemory buffer = new NativeMemory(Marshal.SizeOf(formatRange), true))
				{
					buffer.FromStructure(formatRange, false);

					result =
						NativeMethods.Window_SendMessage(
							base.Handle,
							WindowMessage.EM_FORMATRANGE,
							(IntPtr)(measureOnly ? 0 : 1),
							buffer.Pointer);
				}
			}
			finally
			{
				printPageEventArgs.Graphics.ReleaseHdc(hDC);
			}

			return result;
		}

		#endregion

		#region Properties

		[Browsable(true)]
		[Category("Appearance")]
		[Editor("System.ComponentModel.Design.MultilineStringEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor))]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		public string RichText
		{
			[DebuggerStepThrough]
			get { return base.Rtf; }
			[DebuggerStepThrough]
			set { base.Rtf = value; }
		}

		[Browsable(true)]
		[Category("Appearance")]
		[Editor("System.ComponentModel.Design.MultilineStringEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor))]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public override string Text
		{
			[DebuggerStepThrough]
			get { return base.Text; }
			[DebuggerStepThrough]
			set { base.Text = value; }
		}

		[Browsable(false)]
		public IRichEditOle RichEditOle
		{
			get
			{
				IntPtr unknown = IntPtr.Zero;

				try
				{
					if (NativeMethods.Window_SendMessage(
						base.Handle,
						WindowMessage.EM_GETOLEINTERFACE,
						IntPtr.Zero,
						out unknown) == 0)
					{
						return null;
					}

					return Marshal.GetObjectForIUnknown(unknown) as IRichEditOle;
				}
				finally
				{
					Marshal.Release(unknown);
				}
			}
		}

		[Browsable(false)]
		public IRichTextDocument TextDocument
		{
			get
			{
				IntPtr unknown = IntPtr.Zero;

				try
				{
					if (NativeMethods.Window_SendMessage(base.Handle, WindowMessage.EM_GETOLEINTERFACE, IntPtr.Zero, out unknown) == 0)
					{
						return null;
					}

					return Marshal.GetObjectForIUnknown(unknown) as IRichTextDocument;
				}
				finally
				{
					Marshal.Release(unknown);
				}
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
