﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms Library                            */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Text;

namespace Nabu.Forms
{
	internal static class NativeMethods
	{
		#region Libraries
#if COMPACT_FRAMEWORK
		private const string Kernel32 = "coredll.dll";
		private const string User32 = "coredll.dll";
		private const string Gdi32 = "coredll.dll";
#else
		private const string Kernel32 = "kernel32.dll";
		private const string User32 = "user32.dll";
		private const string Gdi32 = "gdi32.dll";
#endif
		#endregion

		#region Modules

		[DllImport(Kernel32, EntryPoint = "LoadLibraryExW")]
		public static extern IntPtr Module_Load(
			[In] [MarshalAs(UnmanagedType.LPWStr)] string filename,
			[In] IntPtr reserved,
			[In] int flags);

		//	[DllImport(Kernel32, EntryPoint = "FreeLibrary")]
		//	[return: MarshalAs(UnmanagedType.Bool)]
		//	public static extern bool Module_Unload(
		//		[In] IntPtr hModule);

		[DllImport(Kernel32, EntryPoint = "GetModuleHandleW")]
		public static extern IntPtr Module_GetHandle(
			[In] [MarshalAs(UnmanagedType.LPWStr)] string filename);

		#endregion

		#region Threads

		[DllImport(Kernel32, EntryPoint = "GetCurrentThreadId")]
		public static extern int Thread_GetCurrentId();

		#endregion

		#region Resources

		[DllImport(Kernel32, EntryPoint = "FindResourceExW")]
		public static extern IntPtr Resource_Find(
			[In] IntPtr hModule,
			[In][MarshalAs(UnmanagedType.SysInt)] IntPtr resourceType,
			[In][MarshalAs(UnmanagedType.LPWStr)] string resourceName,
			[In] short resourceLanguage);

		[DllImport(Kernel32, EntryPoint = "FindResourceExW")]
		public static extern IntPtr Resource_Find(
			[In] IntPtr hModule,
			[In][MarshalAs(UnmanagedType.SysInt)] IntPtr resourceType,
			[In] int resourceID,
			[In] short resourceLanguage);

		[DllImport(Kernel32, EntryPoint = "FindResourceExW")]
		public static extern IntPtr Resource_Find(
			[In] IntPtr hModule,
			[In][MarshalAs(UnmanagedType.LPWStr)] string resourceType,
			[In][MarshalAs(UnmanagedType.LPWStr)] string resourceName,
			[In] short resourceLanguage);

		[DllImport(Kernel32, EntryPoint = "FindResourceExW")]
		public static extern IntPtr Resource_Find(
			[In] IntPtr hModule,
			[In][MarshalAs(UnmanagedType.LPWStr)] string resourceType,
			[In] int resourceID,
			[In] short resourceLanguage);

		[DllImport(Kernel32, EntryPoint = "LoadResource")]
		public static extern IntPtr Resource_Load(
			[In] IntPtr hModule,
			[In] IntPtr hResourceInfo);

		[DllImport(Kernel32, EntryPoint = "LockResource")]
		public static extern IntPtr Resource_Lock(
			[In] IntPtr hResource);

		[DllImport(Kernel32, EntryPoint = "SizeofResource")]
		public static extern int Resource_Size(
			[In] IntPtr hModule,
			[In] IntPtr hResource);

		[DllImport(User32, EntryPoint = "LoadString")]
		public static extern int String_Load(
			[In] IntPtr hModule,
			[In] int id,
			[Out][MarshalAs(UnmanagedType.LPWStr)] StringBuilder buffer,
			[In] int bufferSize);

		[DllImport(User32, EntryPoint = "LoadImage")]
		public static extern IntPtr Image_Load(
			[In] IntPtr hModule,
			[In] IntPtr imageID,
			[In][MarshalAs(UnmanagedType.U4)] ImageType imageType,
			[In] int imageWidth,
			[In] int imageHeight,
			[In][MarshalAs(UnmanagedType.U4)] ImageLoadFlags flags);

		[DllImport(User32, EntryPoint = "LoadImageW")]
		public static extern IntPtr Image_Load(
			[In] IntPtr hModule,
			[In][MarshalAs(UnmanagedType.LPWStr)] string imageName,
			[In][MarshalAs(UnmanagedType.U4)] ImageType imageType,
			[In] int imageWidth,
			[In] int imageHeight,
			[In][MarshalAs(UnmanagedType.U4)] ImageLoadFlags flags);

		[DllImport(Gdi32, EntryPoint = "DeleteObject")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool Bitmap_Delete(
			[In] IntPtr hBitmap);

		[DllImport(User32, EntryPoint = "DestroyIcon")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool Icon_Delete(
			[In] IntPtr hIcon);

		//	[DllImport(User32, EntryPoint = "DestroyCursor")]
		//	[return: MarshalAs(UnmanagedType.Bool)]
		//	public static extern bool Cursor_Delete(
		//		[In] IntPtr hCursor);

		#endregion

		#region Windows

		#region General

		#region Create/Destroy

		[DllImport(User32, EntryPoint = "CreateWindowExW")]
		public static extern IntPtr Window_Create(
			[In][MarshalAs(UnmanagedType.U4)] WindowStyleExtended styleExtended,
			[In][MarshalAs(UnmanagedType.LPWStr)] string className,
			[In][MarshalAs(UnmanagedType.LPWStr)] string windowName,
			[In][MarshalAs(UnmanagedType.U4)] WindowStyle style,
			[In] int left,
			[In] int top,
			[In] int width,
			[In] int height,
			[In] IntPtr hWndParent,
			[In] IntPtr hMenu,
			[In] IntPtr hInstance,
			[In] IntPtr lpParam);

		[DllImport(User32, EntryPoint = "DestroyWindow")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool Window_Destroy(IntPtr hWnd);

		#endregion

		#region Coordinates

		[DllImport(User32, EntryPoint = "ClientToScreen")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool Window_ClientToScreen(
			[In] IntPtr hWnd,
			[In][Out] ref Point point);

		[DllImport(User32, EntryPoint = "ScreenToClient")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool Window_ScreenToClient(
			[In] IntPtr hWnd,
			[In][Out] ref Point point);

		#endregion

		#region Messages

		[DllImport(User32, EntryPoint = "GetMessage")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool Window_GetMessage(
			[Out] out MSG message,
			[In] IntPtr hWindow,
			[In] int messageMin,
			[In] int messageMax);

		[DllImport(User32, EntryPoint = "DispatchMessage")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool Window_DispatchMessage(
			[In] ref MSG message);

		[DllImport(User32, EntryPoint = "TranslateMessage")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool Window_TranslateMessage(
			[In][Out] ref MSG message);

		[DllImport(User32, EntryPoint = "PostQuitMessage")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool Window_PostQuitMessage(
			[In] int exitCode);

#pragma warning disable 3006

		[DllImport(User32, EntryPoint = "SendMessage")]
		public static extern int Window_SendMessage(
			[In] IntPtr hWnd,
			[In] WindowMessage message,
			[In] IntPtr wParam,
			[In] IntPtr lParam);

		[DllImport(User32, EntryPoint = "SendMessage")]
		public static extern int Window_SendMessage(
			[In] IntPtr hWnd,
			[In] WindowMessage message,
			[In] IntPtr wParam,
			[Out] out IntPtr lParam);

#pragma warning restore 3006

		#endregion

		#region Window_Long*

		[DllImport(User32, EntryPoint = "GetWindowLong")]
		private static extern int Window_GetLong32(
			[In] IntPtr hWnd,
			[In][MarshalAs(UnmanagedType.U4)] WindowLong index);

		[DllImport(User32, EntryPoint = "SetWindowLong")]
		private static extern int Window_SetLong32(
			[In] IntPtr hWnd,
			[In][MarshalAs(UnmanagedType.U4)] WindowLong index,
			[In] int value);

		[DllImport(User32, EntryPoint = "GetWindowLongPtrW")]
		private static extern IntPtr Window_GetLong64(
			[In] IntPtr hWnd,
			[In][MarshalAs(UnmanagedType.U4)] WindowLong index);

		[DllImport(User32, EntryPoint = "SetWindowLongPtrW")]
		private static extern IntPtr Window_SetLong64(
			[In] IntPtr hWnd,
			[In][MarshalAs(UnmanagedType.U4)] WindowLong index,
			[In] IntPtr value);

		public static IntPtr Window_GetLong(
			IntPtr hWnd,
			WindowLong index)
		{
			// Vista WoW64 does not implement GetWindowLong
			if (IntPtr.Size == 4)
			{
				return (IntPtr)Window_GetLong32(hWnd, index);
			}
			else
			{
				return Window_GetLong64(hWnd, index);
			}
		}

		public static IntPtr Window_SetLong(
			IntPtr hWnd,
			WindowLong index,
			IntPtr value)
		{
			// Vista WoW64 does not implement SetWindowLong
			if (IntPtr.Size == 4)
			{
				return (IntPtr)Window_SetLong32(hWnd, index, value.ToInt32());
			}
			else
			{
				return Window_SetLong64(hWnd, index, value);
			}
		}

		#endregion

		#region Window_Position*

		[DllImport(User32, EntryPoint = "SetWindowPos")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool Window_SetPosition(
			[In] IntPtr hWnd,
			[In] IntPtr hWndInsertAfter,
			[In] int left,
			[In] int top,
			[In] int width,
			[In] int height,
			[In][MarshalAs(UnmanagedType.U4)] WindowPositionFlags Flags);

		#endregion

		#endregion

		#region ScrollBar

		[DllImport(User32, EntryPoint = "ShowScrollBar")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool ScrollBar_Show(
			[In] IntPtr hWnd,
			[In][MarshalAs(UnmanagedType.U4)] ScrollBarID bar,
			[In][MarshalAs(UnmanagedType.Bool)] bool show);

		[DllImport(User32, EntryPoint = "GetScrollInfo")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool Scroll_GetInfo(
			[In] IntPtr hWnd,
			[In][MarshalAs(UnmanagedType.U4)] ScrollBarID bar,
			[In][Out] ref ScrollInfo scrollInfo);

		[DllImport(User32, EntryPoint = "SetScrollInfo")]
		public static extern int Scroll_SetInfo(
			[In] IntPtr hWnd,
			[In][MarshalAs(UnmanagedType.U4)] ScrollBarID bar,
			[In] ref ScrollInfo scrollInfo,
			[In][MarshalAs(UnmanagedType.Bool)] bool redraw);

		#endregion

		#endregion

		#region Window Hooks

		[DllImport(User32, EntryPoint = "SetWindowsHookEx")]
		public static extern IntPtr WindowsHook_Add(
			[In][MarshalAs(UnmanagedType.U4)] WindowHookID hookID,
			[In][MarshalAs(UnmanagedType.FunctionPtr)] WindowsHookHandler handler,
			[In] IntPtr hModule,
			[In] int threadID);

		[DllImport(User32, EntryPoint = "UnhookWindowsHookEx")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool WindowsHook_Remove(
			[In] IntPtr hHook);

		#endregion

		#region Drawing

		[DllImport(User32, EntryPoint = "DrawFrameControl")]
		[return: MarshalAs(UnmanagedType.Bool)]
		public static extern bool FrameControl_Draw(
			[In] IntPtr hDC,
			[In][Out] ref Rectangle rect,
			[In][MarshalAs(UnmanagedType.U4)] FrameControlType type,
			[In][MarshalAs(UnmanagedType.U4)] FrameControlState state);

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
