﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms Library                            */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Drawing;
using System.Runtime.InteropServices;

namespace Nabu.Forms
{
	#region Resource

	internal enum ResourceType : int
	{
		Unknown /*       */ = 0,
		Cursor /*        */ = 1,
		Bitmap /*        */ = 2,
		Icon /*          */ = 3,
		Menu /*          */ = 4,
		Dialog /*        */ = 5,
		String /*        */ = 6,
		FontDir /*       */ = 7,
		Font /*          */ = 8,
		Accelerator /*   */ = 9,
		RCData /*        */ = 10,
		MessageTable /*  */ = 11,
		CursorGroup /*   */ = 12,
		IconGroup /*     */ = 14,
		Version /*       */ = 16,
		DialogInclude /* */ = 17,
		PlugPlay /*      */ = 19,
		Vxd /*           */ = 20,
		AniCursor /*     */ = 21,
		AniIcon /*       */ = 22,
		Html /*          */ = 23,
		Manifest /*      */ = 24,
		File /*          */ = 2110
	}

	internal enum ImageType : int
	{
		Bitmap /* */ = 0,
		Icon /*   */ = 1,
		Cursor /* */ = 2
	}

	[Flags]
	internal enum ImageLoadFlags : int
	{
		DefaultColor /*     */ = 0x0000,
		Monochrome /*       */ = 0x0001,
		Color /*            */ = 0x0002,
		CopyReturnOrg /*    */ = 0x0004,
		CopyDeleteOrg /*    */ = 0x0008,
		LoadFromFile /*     */ = 0x0010,
		LoadTransparent /*  */ = 0x0020,
		DefaultSize /*      */ = 0x0040,
		VgaColor /*         */ = 0x0080,
		LoadMap3DColors /*  */ = 0x1000,
		CreateDibSection /* */ = 0x2000,
		CopyFromResource /* */ = 0x4000,
		Shared /*           */ = 0x8000
	}

	#endregion

	#region Windows

	#region Styles

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[SuppressMessage("Microsoft.Usage", "CA2217:DoNotMarkEnumsWithFlags")]
	[Flags]
	public enum WindowStyle : int
	{
		Overlapped /*       */ = 0x00000000,
		PopUp /*            */ = unchecked((int)0x80000000),
		Child /*            */ = 0x40000000,
		Minimize /*         */ = 0x20000000,
		Visible /*          */ = 0x10000000,
		Disabled /*         */ = 0x08000000,
		ClipSiblings /*     */ = 0x04000000,
		ClipChildren /*     */ = 0x02000000,
		Maximize /*         */ = 0x01000000,
		Caption /*          */ = 0x00C00000,
		Border /*           */ = 0x00800000,
		DialogFrame /*      */ = 0x00400000,
		VScroll /*          */ = 0x00200000,
		HScroll /*          */ = 0x00100000,
		SysMenu /*          */ = 0x00080000,
		ThickFrame /*       */ = 0x00040000,
		Group /*            */ = 0x00020000,
		TabStop /*          */ = 0x00010000,
		MinimizeBox /*      */ = 0x00020000,
		MaximizeBox /*      */ = 0x00010000,
		Tiled /*            */ = Overlapped,
		Iconic /*           */ = Minimize,
		SizeBox /*          */ = ThickFrame,
		TiledWindow /*      */ = OverlappedWindow,
		OverlappedWindow /* */= Overlapped | Caption | SysMenu | ThickFrame | MinimizeBox | MaximizeBox,
		PopUpWindow /*      */ = PopUp | Border | SysMenu,
		ChildWindow /*      */ = Child
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum WindowStyleExtended : int
	{
		DialogModalFrame /* */ = 0x00000001,
		NoParentNotify /*   */ = 0x00000004,
		Topmost /*          */ = 0x00000008,
		AcceptFiles /*      */ = 0x00000010,
		Transparent /*      */ = 0x00000020,
		MdiChild /*         */ = 0x00000040,
		ToolWindow /*       */ = 0x00000080,
		WindowEdge /*       */ = 0x00000100,
		ClientEdge /*       */ = 0x00000200,
		ContextHelp /*      */ = 0x00000400,
		Right /*            */ = 0x00001000,
		Left /*             */ = 0x00000000,
		RtlReading /*       */ = 0x00002000,
		LtrReading /*       */ = 0x00000000,
		LeftScrollbar /*    */ = 0x00004000,
		RightScrollbar /*   */ = 0x00000000,
		ControlParent /*    */ = 0x00010000,
		StaticEdge /*       */ = 0x00020000,
		AppWindow /*        */ = 0x00040000,
		OverlappedWindow /* */ = WindowEdge | ClientEdge,
		PaletteWindow /*    */ = WindowEdge | ToolWindow | Topmost,
		Layered /*          */ = 0x00080000,
		NoInheritLayout /*  */ = 0x00100000,
		LayoutRtl /*        */ = 0x00400000,
		Composited /*       */ = 0x02000000,
		NoActivate /*       */ = 0x08000000,
	}

	[Flags]
	public enum WindowClassStyle : int
	{
		VReDraw /*         */ = 0x00000001,
		HReDraw /*         */ = 0x00000002,
		DoubleClicks /*    */ = 0x00000008,
		OwnDC /*           */ = 0x00000020,
		ClassDC /*         */ = 0x00000040,
		ParentDC /*        */ = 0x00000080,
		NoClose /*         */ = 0x00000200,
		SaveBits /*        */ = 0x00000800,
		ByteAlignClient /* */ = 0x00001000,
		ByteAlignWindow /* */ = 0x00002000,
		GlobalClass /*     */ = 0x00004000,
		Ime /*             */ = 0x00010000,
		DropShadow /*      */ = 0x00020000
	}

	#endregion

	#region Messages

	public enum WindowMessage : int
	{
		#region General

		WM_NULL /*                      */ = 0x0000,
		WM_CREATE /*                    */ = 0x0001,
		WM_DESTROY /*                   */ = 0x0002,
		WM_MOVE /*                      */ = 0x0003,
		WM_SIZE /*                      */ = 0x0005,
		WM_ACTIVATE /*                  */ = 0x0006,
		WM_SETFOCUS /*                  */ = 0x0007,
		WM_KILLFOCUS /*                 */ = 0x0008,
		WM_ENABLE /*                    */ = 0x000A,
		WM_SETREDRAW /*                 */ = 0x000B,
		WM_SETTEXT /*                   */ = 0x000C,
		WM_GETTEXT /*                   */ = 0x000D,
		WM_GETTEXTLENGTH /*             */ = 0x000E,
		WM_PAINT /*                     */ = 0x000F,
		WM_CLOSE /*                     */ = 0x0010,
		WM_QUERYENDSESSION /*           */ = 0x0011,
		WM_QUERYOPEN /*                 */ = 0x0013,
		WM_ENDSESSION /*                */ = 0x0016,
		WM_QUIT /*                      */ = 0x0012,
		WM_ERASEBKGND /*                */ = 0x0014,
		WM_SYSCOLORCHANGE /*            */ = 0x0015,
		WM_SHOWWINDOW /*                */ = 0x0018,
		WM_WININICHANGE /*              */ = 0x001A,
		WM_SETTINGCHANGE /*             */ = WM_WININICHANGE,
		WM_DEVMODECHANGE /*             */ = 0x001B,
		WM_ACTIVATEAPP /*               */ = 0x001C,
		WM_FONTCHANGE /*                */ = 0x001D,
		WM_TIMECHANGE /*                */ = 0x001E,
		WM_CANCELMODE /*                */ = 0x001F,
		WM_SETCURSOR /*                 */ = 0x0020,
		WM_MOUSEACTIVATE /*             */ = 0x0021,
		WM_CHILDACTIVATE /*             */ = 0x0022,
		WM_QUEUESYNC /*                 */ = 0x0023,
		WM_GETMINMAXINFO /*             */ = 0x0024,
		WM_PAINTICON /*                 */ = 0x0026,
		WM_ICONERASEBKGND /*            */ = 0x0027,
		WM_NEXTDLGCTL /*                */ = 0x0028,
		WM_SPOOLERSTATUS /*             */ = 0x002A,
		WM_DRAWITEM /*                  */ = 0x002B,
		WM_MEASUREITEM /*               */ = 0x002C,
		WM_DELETEITEM /*                */ = 0x002D,
		WM_VKEYTOITEM /*                */ = 0x002E,
		WM_CHARTOITEM /*                */ = 0x002F,
		WM_SETFONT /*                   */ = 0x0030,
		WM_GETFONT /*                   */ = 0x0031,
		WM_SETHOTKEY /*                 */ = 0x0032,
		WM_GETHOTKEY /*                 */ = 0x0033,
		WM_QUERYDRAGICON /*             */ = 0x0037,
		WM_COMPAREITEM /*               */ = 0x0039,
		WM_GETOBJECT /*                 */ = 0x003D,
		WM_COMPACTING /*                */ = 0x0041,
		WM_COMMNOTIFY /*                */ = 0x0044,
		WM_WINDOWPOSCHANGING /*         */ = 0x0046,
		WM_WINDOWPOSCHANGED /*          */ = 0x0047,
		WM_POWER /*                     */ = 0x0048,
		WM_COPYDATA /*                  */ = 0x004A,
		WM_CANCELJOURNAL /*             */ = 0x004B,
		WM_NOTIFY /*                    */ = 0x004E,
		WM_INPUTLANGCHANGEREQUEST /*    */ = 0x0050,
		WM_INPUTLANGCHANGE /*           */ = 0x0051,
		WM_TCARD /*                     */ = 0x0052,
		WM_HELP /*                      */ = 0x0053,
		WM_USERCHANGED /*               */ = 0x0054,
		WM_NOTIFYFORMAT /*              */ = 0x0055,
		WM_CONTEXTMENU /*               */ = 0x007B,
		WM_STYLECHANGING /*             */ = 0x007C,
		WM_STYLECHANGED /*              */ = 0x007D,
		WM_DISPLAYCHANGE /*             */ = 0x007E,
		WM_GETICON /*                   */ = 0x007F,
		WM_SETICON /*                   */ = 0x0080,
		WM_NCCREATE /*                  */ = 0x0081,
		WM_NCDESTROY /*                 */ = 0x0082,
		WM_NCCALCSIZE /*                */ = 0x0083,
		WM_NCHITTEST /*                 */ = 0x0084,
		WM_NCPAINT /*                   */ = 0x0085,
		WM_NCACTIVATE /*                */ = 0x0086,
		WM_GETDLGCODE /*                */ = 0x0087,
		WM_SYNCPAINT /*                 */ = 0x0088,
		WM_NCMOUSEMOVE /*               */ = 0x00A0,
		WM_NCLBUTTONDOWN /*             */ = 0x00A1,
		WM_NCLBUTTONUP /*               */ = 0x00A2,
		WM_NCLBUTTONDBLCLK /*           */ = 0x00A3,
		WM_NCRBUTTONDOWN /*             */ = 0x00A4,
		WM_NCRBUTTONUP /*               */ = 0x00A5,
		WM_NCRBUTTONDBLCLK /*           */ = 0x00A6,
		WM_NCMBUTTONDOWN /*             */ = 0x00A7,
		WM_NCMBUTTONUP /*               */ = 0x00A8,
		WM_NCMBUTTONDBLCLK /*           */ = 0x00A9,
		WM_NCXBUTTONDOWN /*             */ = 0x00AB,
		WM_NCXBUTTONUP /*               */ = 0x00AC,
		WM_NCXBUTTONDBLCLK /*           */ = 0x00AD,
		WM_INPUT /*                     */ = 0x00FF,
		WM_KEYFIRST /*                  */ = 0x0100,
		WM_KEYDOWN /*                   */ = 0x0100,
		WM_KEYUP /*                     */ = 0x0101,
		WM_CHAR /*                      */ = 0x0102,
		WM_DEADCHAR /*                  */ = 0x0103,
		WM_SYSKEYDOWN /*                */ = 0x0104,
		WM_SYSKEYUP /*                  */ = 0x0105,
		WM_SYSCHAR /*                   */ = 0x0106,
		WM_SYSDEADCHAR /*               */ = 0x0107,
		WM_UNICHAR /*                   */ = 0x0109,
		WM_KEYLAST /*                   */ = 0x0109,
		WM_IME_STARTCOMPOSITION /*      */ = 0x010D,
		WM_IME_ENDCOMPOSITION /*        */ = 0x010E,
		WM_IME_COMPOSITION /*           */ = 0x010F,
		WM_IME_KEYLAST /*               */ = 0x010F,
		WM_INITDIALOG /*                */ = 0x0110,
		WM_COMMAND /*                   */ = 0x0111,
		WM_SYSCOMMAND /*                */ = 0x0112,
		WM_TIMER /*                     */ = 0x0113,
		WM_HSCROLL /*                   */ = 0x0114,
		WM_VSCROLL /*                   */ = 0x0115,
		WM_INITMENU /*                  */ = 0x0116,
		WM_INITMENUPOPUP /*             */ = 0x0117,
		WM_MENUSELECT /*                */ = 0x011F,
		WM_MENUCHAR /*                  */ = 0x0120,
		WM_ENTERIDLE /*                 */ = 0x0121,
		WM_MENURBUTTONUP /*             */ = 0x0122,
		WM_MENUDRAG /*                  */ = 0x0123,
		WM_MENUGETOBJECT /*             */ = 0x0124,
		WM_UNINITMENUPOPUP /*           */ = 0x0125,
		WM_MENUCOMMAND /*               */ = 0x0126,
		WM_CHANGEUISTATE /*             */ = 0x0127,
		WM_UPDATEUISTATE /*             */ = 0x0128,
		WM_QUERYUISTATE /*              */ = 0x0129,
		WM_CTLCOLORMSGBOX /*            */ = 0x0132,
		WM_CTLCOLOREDIT /*              */ = 0x0133,
		WM_CTLCOLORLISTBOX /*           */ = 0x0134,
		WM_CTLCOLORBTN /*               */ = 0x0135,
		WM_CTLCOLORDLG /*               */ = 0x0136,
		WM_CTLCOLORSCROLLBAR /*         */ = 0x0137,
		WM_CTLCOLORSTATIC /*            */ = 0x0138,
		MN_GETHMENU /*                  */ = 0x01E1,
		WM_MOUSEFIRST /*                */ = 0x0200,
		WM_MOUSEMOVE /*                 */ = 0x0200,
		WM_LBUTTONDOWN /*               */ = 0x0201,
		WM_LBUTTONUP /*                 */ = 0x0202,
		WM_LBUTTONDBLCLK /*             */ = 0x0203,
		WM_RBUTTONDOWN /*               */ = 0x0204,
		WM_RBUTTONUP /*                 */ = 0x0205,
		WM_RBUTTONDBLCLK /*             */ = 0x0206,
		WM_MBUTTONDOWN /*               */ = 0x0207,
		WM_MBUTTONUP /*                 */ = 0x0208,
		WM_MBUTTONDBLCLK /*             */ = 0x0209,
		WM_MOUSEWHEEL /*                */ = 0x020A,
		WM_XBUTTONDOWN /*               */ = 0x020B,
		WM_XBUTTONUP /*                 */ = 0x020C,
		WM_XBUTTONDBLCLK /*             */ = 0x020D,
		WM_MOUSELAST /*                 */ = 0x020D,
		WM_PARENTNOTIFY /*              */ = 0x0210,
		WM_ENTERMENULOOP /*             */ = 0x0211,
		WM_EXITMENULOOP /*              */ = 0x0212,
		WM_NEXTMENU /*                  */ = 0x0213,
		WM_SIZING /*                    */ = 0x0214,
		WM_CAPTURECHANGED /*            */ = 0x0215,
		WM_MOVING /*                    */ = 0x0216,
		WM_POWERBROADCAST /*            */ = 0x0218,
		WM_DEVICECHANGE /*              */ = 0x0219,
		WM_MDICREATE /*                 */ = 0x0220,
		WM_MDIDESTROY /*                */ = 0x0221,
		WM_MDIACTIVATE /*               */ = 0x0222,
		WM_MDIRESTORE /*                */ = 0x0223,
		WM_MDINEXT /*                   */ = 0x0224,
		WM_MDIMAXIMIZE /*               */ = 0x0225,
		WM_MDITILE /*                   */ = 0x0226,
		WM_MDICASCADE /*                */ = 0x0227,
		WM_MDIICONARRANGE /*            */ = 0x0228,
		WM_MDIGETACTIVE /*              */ = 0x0229,
		WM_MDISETMENU /*                */ = 0x0230,
		WM_ENTERSIZEMOVE /*             */ = 0x0231,
		WM_EXITSIZEMOVE /*              */ = 0x0232,
		WM_DROPFILES /*                 */ = 0x0233,
		WM_MDIREFRESHMENU /*            */ = 0x0234,
		WM_IME_SETCONTEXT /*            */ = 0x0281,
		WM_IME_NOTIFY /*                */ = 0x0282,
		WM_IME_CONTROL /*               */ = 0x0283,
		WM_IME_COMPOSITIONFULL /*       */ = 0x0284,
		WM_IME_SELECT /*                */ = 0x0285,
		WM_IME_CHAR /*                  */ = 0x0286,
		WM_IME_REQUEST /*               */ = 0x0288,
		WM_IME_KEYDOWN /*               */ = 0x0290,
		WM_IME_KEYUP /*                 */ = 0x0291,
		WM_MOUSEHOVER /*                */ = 0x02A1,
		WM_MOUSELEAVE /*                */ = 0x02A3,
		WM_NCMOUSEHOVER /*              */ = 0x02A0,
		WM_NCMOUSELEAVE /*              */ = 0x02A2,
		WM_WTSSESSION_CHANGE /*         */ = 0x02B1,
		WM_TABLET_FIRST /*              */ = 0x02c0,
		WM_TABLET_LAST /*               */ = 0x02df,
		WM_CUT /*                       */ = 0x0300,
		WM_COPY /*                      */ = 0x0301,
		WM_PASTE /*                     */ = 0x0302,
		WM_CLEAR /*                     */ = 0x0303,
		WM_UNDO /*                      */ = 0x0304,
		WM_RENDERFORMAT /*              */ = 0x0305,
		WM_RENDERALLFORMATS /*          */ = 0x0306,
		WM_DESTROYCLIPBOARD /*          */ = 0x0307,
		WM_DRAWCLIPBOARD /*             */ = 0x0308,
		WM_PAINTCLIPBOARD /*            */ = 0x0309,
		WM_VSCROLLCLIPBOARD /*          */ = 0x030A,
		WM_SIZECLIPBOARD /*             */ = 0x030B,
		WM_ASKCBFORMATNAME /*           */ = 0x030C,
		WM_CHANGECBCHAIN /*             */ = 0x030D,
		WM_HSCROLLCLIPBOARD /*          */ = 0x030E,
		WM_QUERYNEWPALETTE /*           */ = 0x030F,
		WM_PALETTEISCHANGING /*         */ = 0x0310,
		WM_PALETTECHANGED /*            */ = 0x0311,
		WM_HOTKEY /*                    */ = 0x0312,
		WM_PRINT /*                     */ = 0x0317,
		WM_PRINTCLIENT /*               */ = 0x0318,
		WM_APPCOMMAND /*                */ = 0x0319,
		WM_THEMECHANGED /*              */ = 0x031A,
		WM_HANDHELDFIRST /*             */ = 0x0358,
		WM_HANDHELDLAST /*              */ = 0x035F,
		WM_AFXFIRST /*                  */ = 0x0360,
		WM_AFXLAST /*                   */ = 0x037F,
		WM_PENWINFIRST /*               */ = 0x0380,
		WM_PENWINLAST /*                */ = 0x038F,
		WM_USER /*                      */ = 0x0400,
		WM_APP /*                       */ = 0x8000,

		#endregion

		#region Edit, RichEdit

		EM_GETSEL /*                    */ = 0x00B0,
		EM_SETSEL /*                    */ = 0x00B1,
		EM_GETRECT /*                   */ = 0x00B2,
		EM_SETRECT /*                   */ = 0x00B3,
		EM_SETRECTNP /*                 */ = 0x00B4,
		EM_SCROLL /*                    */ = 0x00B5,
		EM_LINESCROLL /*                */ = 0x00B6,
		EM_SCROLLCARET /*               */ = 0x00B7,
		EM_GETMODIFY /*                 */ = 0x00B8,
		EM_SETMODIFY /*                 */ = 0x00B9,
		EM_GETLINECOUNT /*              */ = 0x00BA,
		EM_LINEINDEX /*                 */ = 0x00BB,
		EM_SETHANDLE /*                 */ = 0x00BC,
		EM_GETHANDLE /*                 */ = 0x00BD,
		EM_GETTHUMB /*                  */ = 0x00BE,
		EM_LINELENGTH /*                */ = 0x00C1,
		EM_REPLACESEL /*                */ = 0x00C2,
		EM_GETLINE /*                   */ = 0x00C4,
		EM_LIMITTEXT /*                 */ = 0x00C5,
		EM_CANUNDO /*                   */ = 0x00C6,
		EM_UNDO /*                      */ = 0x00C7,
		EM_FMTLINES /*                  */ = 0x00C8,
		EM_LINEFROMCHAR /*              */ = 0x00C9,
		EM_SETTABSTOPS /*               */ = 0x00CB,
		EM_SETPASSWORDCHAR /*           */ = 0x00CC,
		EM_EMPTYUNDOBUFFER /*           */ = 0x00CD,
		EM_GETFIRSTVISIBLELINE /*       */ = 0x00CE,
		EM_SETREADONLY /*               */ = 0x00CF,
		EM_SETWORDBREAKPROC /*          */ = 0x00D0,
		EM_GETWORDBREAKPROC /*          */ = 0x00D1,
		EM_GETPASSWORDCHAR /*           */ = 0x00D2,
		EM_SETMARGINS /*                */ = 0x00D3,
		EM_GETMARGINS /*                */ = 0x00D4,
		EM_SETLIMITTEXT /*              */ = EM_LIMITTEXT,
		EM_GETLIMITTEXT /*              */ = 0x00D5,
		EM_POSFROMCHAR /*               */ = 0x00D6,
		EM_CHARFROMPOS /*               */ = 0x00D7,
		EM_SETIMESTATUS /*              */ = 0x00D8,
		EM_GETIMESTATUS /*              */ = 0x00D9,

		//EM_GETLIMITTEXT                             = WM_USER +  37,
		//EM_POSFROMCHAR                              = WM_USER +  38,
		//EM_CHARFROMPOS                              = WM_USER +  39,
		//EM_SCROLLCARET                              = WM_USER +  49,
		EM_CANPASTE /*                  */ = WM_USER + 50,
		EM_DISPLAYBAND /*               */ = WM_USER + 51,
		EM_EXGETSEL /*                  */ = WM_USER + 52,
		EM_EXLIMITTEXT /*               */ = WM_USER + 53,
		EM_EXLINEFROMCHAR /*            */ = WM_USER + 54,
		EM_EXSETSEL /*                  */ = WM_USER + 55,
		EM_FINDTEXT /*                  */ = WM_USER + 56,
		EM_FORMATRANGE /*               */ = WM_USER + 57,
		EM_GETCHARFORMAT /*             */ = WM_USER + 58,
		EM_GETEVENTMASK /*              */ = WM_USER + 59,
		EM_GETOLEINTERFACE /*           */ = WM_USER + 60,
		EM_GETPARAFORMAT /*             */ = WM_USER + 61,
		EM_GETSELTEXT /*                */ = WM_USER + 62,
		EM_HIDESELECTION /*             */ = WM_USER + 63,
		EM_PASTESPECIAL /*              */ = WM_USER + 64,
		EM_REQUESTRESIZE /*             */ = WM_USER + 65,
		EM_SELECTIONTYPE /*             */ = WM_USER + 66,
		EM_SETBKGNDCOLOR /*             */ = WM_USER + 67,
		EM_SETCHARFORMAT /*             */ = WM_USER + 68,
		EM_SETEVENTMASK /*              */ = WM_USER + 69,
		EM_SETOLECALLBACK /*            */ = WM_USER + 70,
		EM_SETPARAFORMAT /*             */ = WM_USER + 71,
		EM_SETTARGETDEVICE /*           */ = WM_USER + 72,
		EM_STREAMIN /*                  */ = WM_USER + 73,
		EM_STREAMOUT /*                 */ = WM_USER + 74,
		EM_GETTEXTRANGE /*              */ = WM_USER + 75,
		EM_FINDWORDBREAK /*             */ = WM_USER + 76,
		EM_SETOPTIONS /*                */ = WM_USER + 77,
		EM_GETOPTIONS /*                */ = WM_USER + 78,
		EM_FINDTEXTEX /*                */ = WM_USER + 79,
		EM_GETWORDBREAKPROCEX /*        */ = WM_USER + 80,
		EM_SETWORDBREAKPROCEX /*        */ = WM_USER + 81,
		EM_SETUNDOLIMIT /*              */ = WM_USER + 82,
		EM_REDO /*                      */ = WM_USER + 84,
		EM_CANREDO /*                   */ = WM_USER + 85,
		EM_GETUNDONAME /*               */ = WM_USER + 86,
		EM_GETREDONAME /*               */ = WM_USER + 87,
		EM_STOPGROUPTYPING /*           */ = WM_USER + 88,
		EM_SETTEXTMODE /*               */ = WM_USER + 89,
		EM_GETTEXTMODE /*               */ = WM_USER + 90,
		EM_AUTOURLDETECT /*             */ = WM_USER + 91,
		EM_GETAUTOURLDETECT /*          */ = WM_USER + 92,
		EM_SETPALETTE /*                */ = WM_USER + 93,
		EM_GETTEXTEX /*                 */ = WM_USER + 94,
		EM_GETTEXTLENGTHEX /*           */ = WM_USER + 95,
		EM_SHOWSCROLLBAR /*             */ = WM_USER + 96,
		EM_SETTEXTEX /*                 */ = WM_USER + 97,
		EM_SETPUNCTUATION /*            */ = WM_USER + 100,
		EM_GETPUNCTUATION /*            */ = WM_USER + 101,
		EM_SETWORDWRAPMODE /*           */ = WM_USER + 102,
		EM_GETWORDWRAPMODE /*           */ = WM_USER + 103,
		EM_SETIMECOLOR /*               */ = WM_USER + 104,
		EM_GETIMECOLOR /*               */ = WM_USER + 105,
		EM_SETIMEOPTIONS /*             */ = WM_USER + 106,
		EM_GETIMEOPTIONS /*             */ = WM_USER + 107,
		EM_CONVPOSITION /*              */ = WM_USER + 108,
		EM_SETLANGOPTIONS /*            */ = WM_USER + 120,
		EM_GETLANGOPTIONS /*            */ = WM_USER + 121,
		EM_GETIMECOMPMODE /*            */ = WM_USER + 122,
		EM_FINDTEXTW /*                 */ = WM_USER + 123,
		EM_FINDTEXTEXW /*               */ = WM_USER + 124,
		EM_RECONVERSION /*              */ = WM_USER + 125,
		EM_SETIMEMODEBIAS /*            */ = WM_USER + 126,
		EM_GETIMEMODEBIAS /*            */ = WM_USER + 127,
		EM_SETBIDIOPTIONS /*            */ = WM_USER + 200,
		EM_GETBIDIOPTIONS /*            */ = WM_USER + 201,
		EM_SETTYPOGRAPHYOPTIONS /*      */ = WM_USER + 202,
		EM_GETTYPOGRAPHYOPTIONS /*      */ = WM_USER + 203,
		EM_SETEDITSTYLE /*              */ = WM_USER + 204,
		EM_GETEDITSTYLE /*              */ = WM_USER + 205,
		EM_OUTLINE /*                   */ = WM_USER + 220,
		EM_GETSCROLLPOS /*              */ = WM_USER + 221,
		EM_SETSCROLLPOS /*              */ = WM_USER + 222,
		EM_SETFONTSIZE /*               */ = WM_USER + 223,
		EM_GETZOOM /*                   */ = WM_USER + 224,
		EM_SETZOOM /*                   */ = WM_USER + 225,
		EM_GETVIEWKIND /*               */ = WM_USER + 226,
		EM_SETVIEWKIND /*               */ = WM_USER + 227,
		EM_GETPAGE /*                   */ = WM_USER + 228,
		EM_SETPAGE /*                   */ = WM_USER + 229,
		EM_GETHYPHENATEINFO /*          */ = WM_USER + 230,
		EM_SETHYPHENATEINFO /*          */ = WM_USER + 231,
		EM_GETPAGEROTATE /*             */ = WM_USER + 235,
		EM_SETPAGEROTATE /*             */ = WM_USER + 236,
		EM_GETCTFMODEBIAS /*            */ = WM_USER + 237,
		EM_SETCTFMODEBIAS /*            */ = WM_USER + 238,
		EM_GETCTFOPENSTATUS /*          */ = WM_USER + 240,
		EM_SETCTFOPENSTATUS /*          */ = WM_USER + 241,
		EM_GETIMECOMPTEXT /*            */ = WM_USER + 242,
		EM_ISIME /*                     */ = WM_USER + 243,
		EM_GETIMEPROPERTY /*            */ = WM_USER + 244,

		#endregion

		#region ListView

		LVM_FIRST /*                    */ = 0x1000,
		LVN_FIRST /*                    */ = -100,

		LVM_GETBKCOLOR /*               */ = LVM_FIRST + 0,
		LVM_SETBKCOLOR /*               */ = LVM_FIRST + 1,
		LVM_GETIMAGELIST /*             */ = LVM_FIRST + 2,
		LVM_SETIMAGELIST /*             */ = LVM_FIRST + 3,
		LVM_GETITEMCOUNT /*             */ = LVM_FIRST + 4,
		LVM_GETITEMA /*                 */ = LVM_FIRST + 5,
		LVM_SETITEMA /*                 */ = LVM_FIRST + 6,
		LVM_INSERTITEMA /*              */ = LVM_FIRST + 7,
		LVM_DELETEITEM /*               */ = LVM_FIRST + 8,
		LVM_DELETEALLITEMS /*           */ = LVM_FIRST + 9,
		LVM_GETCALLBACKMASK /*          */ = LVM_FIRST + 10,
		LVM_SETCALLBACKMASK /*          */ = LVM_FIRST + 11,
		LVM_GETNEXTITEM /*              */ = LVM_FIRST + 12,
		LVM_FINDITEMA /*                */ = LVM_FIRST + 13,
		LVM_GETITEMRECT /*              */ = LVM_FIRST + 14,
		LVM_SETITEMPOSITION /*          */ = LVM_FIRST + 15,
		LVM_GETITEMPOSITION /*          */ = LVM_FIRST + 16,
		LVM_GETSTRINGWIDTHA /*          */ = LVM_FIRST + 17,
		LVM_HITTEST /*                  */ = LVM_FIRST + 18,
		LVM_ENSUREVISIBLE /*            */ = LVM_FIRST + 19,
		LVM_SCROLL /*                   */ = LVM_FIRST + 20,
		LVM_REDRAWITEMS /*              */ = LVM_FIRST + 21,
		LVM_ARRANGE /*                  */ = LVM_FIRST + 22,
		LVM_EDITLABELA /*               */ = LVM_FIRST + 23,
		LVM_GETEDITCONTROL /*           */ = LVM_FIRST + 24,
		LVM_GETCOLUMNA /*               */ = LVM_FIRST + 25,
		LVM_SETCOLUMNA /*               */ = LVM_FIRST + 26,
		LVM_INSERTCOLUMNA /*            */ = LVM_FIRST + 27,
		LVM_DELETECOLUMN /*             */ = LVM_FIRST + 28,
		LVM_GETCOLUMNWIDTH /*           */ = LVM_FIRST + 29,
		LVM_SETCOLUMNWIDTH /*           */ = LVM_FIRST + 30,
		LVM_GETHEADER /*                */ = LVM_FIRST + 31,
		LVM_CREATEDRAGIMAGE /*          */ = LVM_FIRST + 33,
		LVM_GETVIEWRECT /*              */ = LVM_FIRST + 34,
		LVM_GETTEXTCOLOR /*             */ = LVM_FIRST + 35,
		LVM_SETTEXTCOLOR /*             */ = LVM_FIRST + 36,
		LVM_GETTEXTBKCOLOR /*           */ = LVM_FIRST + 37,
		LVM_SETTEXTBKCOLOR /*           */ = LVM_FIRST + 38,
		LVM_GETTOPINDEX /*              */ = LVM_FIRST + 39,
		LVM_GETCOUNTPERPAGE /*          */ = LVM_FIRST + 40,
		LVM_GETORIGIN /*                */ = LVM_FIRST + 41,
		LVM_UPDATE /*                   */ = LVM_FIRST + 42,
		LVM_SETITEMSTATE /*             */ = LVM_FIRST + 43,
		LVM_GETITEMSTATE /*             */ = LVM_FIRST + 44,
		LVM_GETITEMTEXTA /*             */ = LVM_FIRST + 45,
		LVM_SETITEMTEXTA /*             */ = LVM_FIRST + 46,
		LVM_SETITEMCOUNT /*             */ = LVM_FIRST + 47,
		LVM_SORTITEMS /*                */ = LVM_FIRST + 48,
		LVM_SETITEMPOSITION32 /*        */ = LVM_FIRST + 49,
		LVM_GETSELECTEDCOUNT /*         */ = LVM_FIRST + 50,
		LVM_GETITEMSPACING /*           */ = LVM_FIRST + 51,
		LVM_GETISEARCHSTRINGA /*        */ = LVM_FIRST + 52,
		LVM_SETICONSPACING /*           */ = LVM_FIRST + 53,
		LVM_SETEXTENDEDLISTVIEWSTYLE /* */ = LVM_FIRST + 54,
		LVM_GETEXTENDEDLISTVIEWSTYLE /* */ = LVM_FIRST + 55,
		LVM_GETSUBITEMRECT /*           */ = LVM_FIRST + 56,
		LVM_SUBITEMHITTEST /*           */ = LVM_FIRST + 57,
		LVM_SETCOLUMNORDERARRAY /*      */ = LVM_FIRST + 58,
		LVM_GETCOLUMNORDERARRAY /*      */ = LVM_FIRST + 59,
		LVM_SETHOTITEM /*               */ = LVM_FIRST + 60,
		LVM_GETHOTITEM /*               */ = LVM_FIRST + 61,
		LVM_SETHOTCURSOR /*             */ = LVM_FIRST + 62,
		LVM_GETHOTCURSOR /*             */ = LVM_FIRST + 63,
		LVM_APPROXIMATEVIEWRECT /*      */ = LVM_FIRST + 64,
		LVM_SETWORKAREAS /*             */ = LVM_FIRST + 65,
		LVM_SETBKIMAGEA /*              */ = LVM_FIRST + 68,
		LVM_GETWORKAREAS /*             */ = LVM_FIRST + 70,
		LVM_GETNUMBEROFWORKAREAS /*     */ = LVM_FIRST + 73,
		LVM_GETSELECTIONMARK /*         */ = LVM_FIRST + 66,
		LVM_SETSELECTIONMARK /*         */ = LVM_FIRST + 67,
		LVM_GETBKIMAGEA /*              */ = LVM_FIRST + 69,
		LVM_SETHOVERTIME /*             */ = LVM_FIRST + 71,
		LVM_GETHOVERTIME /*             */ = LVM_FIRST + 72,
		LVM_SETTOOLTIPS /*              */ = LVM_FIRST + 74,
		LVM_GETITEMW /*                 */ = LVM_FIRST + 75,
		LVM_SETITEMW /*                 */ = LVM_FIRST + 76,
		LVM_INSERTITEMW /*              */ = LVM_FIRST + 77,
		LVM_GETTOOLTIPS /*              */ = LVM_FIRST + 78,
		LVM_SORTITEMSEX /*              */ = LVM_FIRST + 81,
		LVM_FINDITEMW /*                */ = LVM_FIRST + 83,
		LVM_GETSTRINGWIDTHW /*          */ = LVM_FIRST + 87,
		LVM_GETCOLUMNW /*               */ = LVM_FIRST + 95,
		LVM_SETCOLUMNW /*               */ = LVM_FIRST + 96,
		LVM_INSERTCOLUMNW /*            */ = LVM_FIRST + 97,
		LVM_GETITEMTEXTW /*             */ = LVM_FIRST + 115,
		LVM_SETITEMTEXTW /*             */ = LVM_FIRST + 116,
		LVM_GETISEARCHSTRINGW /*        */ = LVM_FIRST + 117,
		LVM_EDITLABELW /*               */ = LVM_FIRST + 118,
		LVM_SETBKIMAGEW /*              */ = LVM_FIRST + 138,
		LVM_GETBKIMAGEW /*              */ = LVM_FIRST + 139,
		LVM_SETSELECTEDCOLUMN /*        */ = LVM_FIRST + 140,
		LVM_SETTILEWIDTH /*             */ = LVM_FIRST + 141,
		LVM_SETVIEW /*                  */ = LVM_FIRST + 142,
		LVM_GETVIEW /*                  */ = LVM_FIRST + 143,
		LVM_INSERTGROUP /*              */ = LVM_FIRST + 145,
		LVM_SETGROUPINFO /*             */ = LVM_FIRST + 147,
		LVM_GETGROUPINFO /*             */ = LVM_FIRST + 149,
		LVM_REMOVEGROUP /*              */ = LVM_FIRST + 150,
		LVM_MOVEGROUP /*                */ = LVM_FIRST + 151,
		LVM_MOVEITEMTOGROUP /*          */ = LVM_FIRST + 154,
		LVM_SETGROUPMETRICS /*          */ = LVM_FIRST + 155,
		LVM_GETGROUPMETRICS /*          */ = LVM_FIRST + 156,
		LVM_ENABLEGROUPVIEW /*          */ = LVM_FIRST + 157,
		LVM_SORTGROUPS /*               */ = LVM_FIRST + 158,
		LVM_INSERTGROUPSORTED /*        */ = LVM_FIRST + 159,
		LVM_REMOVEALLGROUPS /*          */ = LVM_FIRST + 160,
		LVM_HASGROUP /*                 */ = LVM_FIRST + 161,
		LVM_SETTILEVIEWINFO /*          */ = LVM_FIRST + 162,
		LVM_GETTILEVIEWINFO /*          */ = LVM_FIRST + 163,
		LVM_SETTILEINFO /*              */ = LVM_FIRST + 164,
		LVM_GETTILEINFO /*              */ = LVM_FIRST + 165,
		LVM_SETINSERTMARK /*            */ = LVM_FIRST + 166,
		LVM_GETINSERTMARK /*            */ = LVM_FIRST + 167,
		LVM_INSERTMARKHITTEST /*        */ = LVM_FIRST + 168,
		LVM_GETINSERTMARKRECT /*        */ = LVM_FIRST + 169,
		LVM_SETINSERTMARKCOLOR /*       */ = LVM_FIRST + 170,
		LVM_GETINSERTMARKCOLOR /*       */ = LVM_FIRST + 171,
		LVM_SETINFOTIP /*               */ = LVM_FIRST + 173,
		LVM_GETSELECTEDCOLUMN /*        */ = LVM_FIRST + 174,
		LVM_ISGROUPVIEWENABLED /*       */ = LVM_FIRST + 175,
		LVM_GETOUTLINECOLOR /*          */ = LVM_FIRST + 176,
		LVM_SETOUTLINECOLOR /*          */ = LVM_FIRST + 177,
		LVM_CANCELEDITLABEL /*          */ = LVM_FIRST + 179,
		LVM_MAPINDEXTOID /*             */ = LVM_FIRST + 180,
		LVM_MAPIDTOINDEX /*             */ = LVM_FIRST + 181,

		LVN_ITEMCHANGING /*             */ = LVN_FIRST - 0,
		LVN_ITEMCHANGED /*              */ = LVN_FIRST - 1,
		LVN_INSERTITEM /*               */ = LVN_FIRST - 2,
		LVN_DELETEITEM /*               */ = LVN_FIRST - 3,
		LVN_DELETEALLITEMS /*           */ = LVN_FIRST - 4,
		LVN_BEGINLABELEDITA /*          */ = LVN_FIRST - 5,
		LVN_ENDLABELEDITA /*            */ = LVN_FIRST - 6,
		LVN_COLUMNCLICK /*              */ = LVN_FIRST - 8,
		LVN_BEGINDRAG /*                */ = LVN_FIRST - 9,
		LVN_BEGINRDRAG /*               */ = LVN_FIRST - 11,
		LVN_ODCACHEHINT /*              */ = LVN_FIRST - 13,
		LVN_ITEMACTIVATE /*             */ = LVN_FIRST - 14,
		LVN_ODSTATECHANGED /*           */ = LVN_FIRST - 15,
		LVN_HOTTRACK /*                 */ = LVN_FIRST - 21,
		LVN_GETDISPINFOA /*             */ = LVN_FIRST - 50,
		LVN_SETDISPINFOA /*             */ = LVN_FIRST - 51,
		LVN_ODFINDITEMA /*              */ = LVN_FIRST - 52,
		LVN_KEYDOWN /*                  */ = LVN_FIRST - 55,
		LVN_MARQUEEBEGIN /*             */ = LVN_FIRST - 56,
		LVN_GETINFOTIPA /*              */ = LVN_FIRST - 57,
		LVN_GETINFOTIPW /*              */ = LVN_FIRST - 58,
		LVN_BEGINLABELEDITW /*          */ = LVN_FIRST - 75,
		LVN_ENDLABELEDITW /*            */ = LVN_FIRST - 76,
		LVN_GETDISPINFOW /*             */ = LVN_FIRST - 77,
		LVN_SETDISPINFOW /*             */ = LVN_FIRST - 78,
		LVN_ODFINDITEMW /*              */ = LVN_FIRST - 79,
		LVN_BEGINSCROLL /*              */ = LVN_FIRST - 80,
		LVN_ENDSCROLL /*                */ = LVN_FIRST - 81,

		#endregion

		#region ToolTip

		TTM_ACTIVATE /*                 */ = WM_USER + 1,
		TTM_SETDELAYTIME /*             */ = WM_USER + 3,
		TTM_ADDTOOLA /*                 */ = WM_USER + 4,
		TTM_DELTOOLA /*                 */ = WM_USER + 5,
		TTM_NEWTOOLRECTA /*             */ = WM_USER + 6,
		TTM_RELAYEVENT /*               */ = WM_USER + 7,
		TTM_GETTOOLINFOA /*             */ = WM_USER + 8,
		TTM_SETTOOLINFOA /*             */ = WM_USER + 9,
		TTM_HITTESTA /*                 */ = WM_USER + 10,
		TTM_GETTEXTA /*                 */ = WM_USER + 11,
		TTM_UPDATETIPTEXTA /*           */ = WM_USER + 12,
		TTM_GETTOOLCOUNT /*             */ = WM_USER + 13,
		TTM_ENUMTOOLSA /*               */ = WM_USER + 14,
		TTM_GETCURRENTTOOLA /*          */ = WM_USER + 15,
		TTM_WINDOWFROMPOINT /*          */ = WM_USER + 16,
		TTM_TRACKACTIVATE /*            */ = WM_USER + 17,
		TTM_TRACKPOSITION /*            */ = WM_USER + 18,
		TTM_SETTIPBKCOLOR /*            */ = WM_USER + 19,
		TTM_SETTIPTEXTCOLOR /*          */ = WM_USER + 20,
		TTM_GETDELAYTIME /*             */ = WM_USER + 21,
		TTM_GETTIPBKCOLOR /*            */ = WM_USER + 22,
		TTM_GETTIPTEXTCOLOR /*          */ = WM_USER + 23,
		TTM_SETMAXTIPWIDTH /*           */ = WM_USER + 24,
		TTM_GETMAXTIPWIDTH /*           */ = WM_USER + 25,
		TTM_SETMARGIN /*                */ = WM_USER + 26,
		TTM_GETMARGIN /*                */ = WM_USER + 27,
		TTM_POP /*                      */ = WM_USER + 28,
		TTM_UPDATE /*                   */ = WM_USER + 29,
		TTM_GETBUBBLESIZE /*            */ = WM_USER + 30,
		TTM_ADJUSTRECT /*               */ = WM_USER + 31,
		TTM_SETTITLEA /*                */ = WM_USER + 32,
		TTM_SETTITLEW /*                */ = WM_USER + 33,
		TTM_POPUP /*                    */ = WM_USER + 34,
		TTM_GETTITLE /*                 */ = WM_USER + 35,
		TTM_ADDTOOLW /*                 */ = WM_USER + 50,
		TTM_DELTOOLW /*                 */ = WM_USER + 51,
		TTM_NEWTOOLRECTW /*             */ = WM_USER + 52,
		TTM_GETTOOLINFOW /*             */ = WM_USER + 53,
		TTM_SETTOOLINFOW /*             */ = WM_USER + 54,
		TTM_HITTESTW /*                 */ = WM_USER + 55,
		TTM_GETTEXTW /*                 */ = WM_USER + 56,
		TTM_UPDATETIPTEXTW /*           */ = WM_USER + 57,
		TTM_ENUMTOOLSW /*               */ = WM_USER + 58,
		TTM_GETCURRENTTOOLW /*          */ = WM_USER + 59,

		#endregion
	};

	[StructLayout(LayoutKind.Sequential)]
	public struct MSG
	{
		public IntPtr HWnd;
		[MarshalAs(UnmanagedType.U4)]
		public WindowMessage Message;
		public IntPtr WParam;
		public IntPtr LParam;
		public int Time;
		public Point Point;
	}

	#endregion

	#region Message Parameters

	public enum HitTestCode : int
	{
		Error /*       */ = -2,
		Transparent /* */ = -1,
		NoWhere /*     */ = 0,
		Client /*      */ = 1,
		Caption /*     */ = 2,
		SysMenu /*     */ = 3,
		GrowBox /*     */ = 4,
		Size /*        */ = GrowBox,
		Menu /*        */ = 5,
		HScroll /*     */ = 6,
		VScroll /*     */ = 7,
		MinButton /*   */ = 8,
		MaxButton /*   */ = 9,
		Left /*        */ = 10,
		Right /*       */ = 11,
		Top /*         */ = 12,
		TopLeft /*     */ = 13,
		TopRight /*    */ = 14,
		Bottom /*      */ = 15,
		BottomLeft /*  */ = 16,
		BottomRight /* */ = 17,
		Border /*      */ = 18,
		Reduce /*      */ = MinButton,
		Zoom /*        */ = MaxButton,
		SizeFirst /*   */ = Left,
		SizeLast /*    */ = BottomRight
	}

	[Flags]
	internal enum WindowPositionFlags : int
	{
		NoSize /*         */ = 0x0001,
		NoMove /*         */ = 0x0002,
		NoZOrder /*       */ = 0x0004,
		NoReDraw /*       */ = 0x0008,
		NoActivate /*     */ = 0x0010,
		FrameChanged /*   */ = 0x0020,
		ShowWindow /*     */ = 0x0040,
		HideWindow /*     */ = 0x0080,
		NoCopyBits /*     */ = 0x0100,
		NoOwnerZOrder /*  */ = 0x0200,
		NoSendChanging /* */ = 0x0400,
		DrawFrame /*      */ = FrameChanged,
		NoReposition /*   */ = NoOwnerZOrder,
		DeferErase /*     */ = 0x2000,
		AsyncWindowPos /* */ = 0x4000
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct WindowPosition
	{
		public IntPtr HWnd;
		public IntPtr HWndAfter;
		public int Left;
		public int Top;
		public int Width;
		public int Height;
		[MarshalAs(UnmanagedType.U4)]
		public WindowPositionFlags Flags;
	}

	internal enum WindowLong : int
	{
		WindowProc /* */ = -4,
		HInstance /*  */ = -6,
		HWndParent /* */ = -8,
		Style /*      */ = -16,
		ExStyle /*    */ = -20,
		UserData /*   */ = -21,
		ID /*         */ = -12,
	}

	internal enum VirtualKey : int
	{
		VK_LBUTTON /*             */ = 0x01,
		VK_RBUTTON /*             */ = 0x02,
		VK_CANCEL /*              */ = 0x03,
		VK_MBUTTON /*             */ = 0x04,
		VK_XBUTTON1 /*            */ = 0x05,
		VK_XBUTTON2 /*            */ = 0x06,
		VK_BACK /*                */ = 0x08,
		VK_TAB /*                 */ = 0x09,
		VK_CLEAR /*               */ = 0x0C,
		VK_RETURN /*              */ = 0x0D,
		VK_SHIFT /*               */ = 0x10,
		VK_CONTROL /*             */ = 0x11,
		VK_MENU /*                */ = 0x12,
		VK_PAUSE /*               */ = 0x13,
		VK_CAPITAL /*             */ = 0x14,
		VK_KANA /*                */ = 0x15,
		VK_HANGEUL /*             */ = 0x15,
		VK_HANGUL /*              */ = 0x15,
		VK_JUNJA /*               */ = 0x17,
		VK_FINAL /*               */ = 0x18,
		VK_HANJA /*               */ = 0x19,
		VK_KANJI /*               */ = 0x19,
		VK_ESCAPE /*              */ = 0x1B,
		VK_CONVERT /*             */ = 0x1C,
		VK_NONCONVERT /*          */ = 0x1D,
		VK_ACCEPT /*              */ = 0x1E,
		VK_MODECHANGE /*          */ = 0x1F,
		VK_SPACE /*               */ = 0x20,
		VK_PRIOR /*               */ = 0x21,
		VK_NEXT /*                */ = 0x22,
		VK_END /*                 */ = 0x23,
		VK_HOME /*                */ = 0x24,
		VK_LEFT /*                */ = 0x25,
		VK_UP /*                  */ = 0x26,
		VK_RIGHT /*               */ = 0x27,
		VK_DOWN /*                */ = 0x28,
		VK_SELECT /*              */ = 0x29,
		VK_PRINT /*               */ = 0x2A,
		VK_EXECUTE /*             */ = 0x2B,
		VK_SNAPSHOT /*            */ = 0x2C,
		VK_INSERT /*              */ = 0x2D,
		VK_DELETE /*              */ = 0x2E,
		VK_HELP /*                */ = 0x2F,
		VK_0 /*                   */ = 0x30,
		VK_1 /*                   */ = 0x31,
		VK_2 /*                   */ = 0x32,
		VK_3 /*                   */ = 0x33,
		VK_4 /*                   */ = 0x34,
		VK_5 /*                   */ = 0x35,
		VK_6 /*                   */ = 0x36,
		VK_7 /*                   */ = 0x37,
		VK_8 /*                   */ = 0x38,
		VK_9 /*                   */ = 0x39,
		VK_A /*                   */ = 0x41,
		VK_B /*                   */ = 0x42,
		VK_C /*                   */ = 0x43,
		VK_D /*                   */ = 0x44,
		VK_E /*                   */ = 0x45,
		VK_F /*                   */ = 0x46,
		VK_G /*                   */ = 0x47,
		VK_H /*                   */ = 0x48,
		VK_I /*                   */ = 0x49,
		VK_J /*                   */ = 0x4A,
		VK_K /*                   */ = 0x4B,
		VK_L /*                   */ = 0x4C,
		VK_M /*                   */ = 0x4D,
		VK_N /*                   */ = 0x4E,
		VK_O /*                   */ = 0x4F,
		VK_P /*                   */ = 0x50,
		VK_Q /*                   */ = 0x51,
		VK_R /*                   */ = 0x52,
		VK_S /*                   */ = 0x53,
		VK_T /*                   */ = 0x54,
		VK_U /*                   */ = 0x55,
		VK_V /*                   */ = 0x56,
		VK_W /*                   */ = 0x57,
		VK_X /*                   */ = 0x58,
		VK_Y /*                   */ = 0x59,
		VK_Z /*                   */ = 0x5A,
		VK_LWIN /*                */ = 0x5B,
		VK_RWIN /*                */ = 0x5C,
		VK_APPS /*                */ = 0x5D,
		VK_SLEEP /*               */ = 0x5F,
		VK_NUMPAD0 /*             */ = 0x60,
		VK_NUMPAD1 /*             */ = 0x61,
		VK_NUMPAD2 /*             */ = 0x62,
		VK_NUMPAD3 /*             */ = 0x63,
		VK_NUMPAD4 /*             */ = 0x64,
		VK_NUMPAD5 /*             */ = 0x65,
		VK_NUMPAD6 /*             */ = 0x66,
		VK_NUMPAD7 /*             */ = 0x67,
		VK_NUMPAD8 /*             */ = 0x68,
		VK_NUMPAD9 /*             */ = 0x69,
		VK_MULTIPLY /*            */ = 0x6A,
		VK_ADD /*                 */ = 0x6B,
		VK_SEPARATOR /*           */ = 0x6C,
		VK_SUBTRACT /*            */ = 0x6D,
		VK_DECIMAL /*             */ = 0x6E,
		VK_DIVIDE /*              */ = 0x6F,
		VK_F1 /*                  */ = 0x70,
		VK_F2 /*                  */ = 0x71,
		VK_F3 /*                  */ = 0x72,
		VK_F4 /*                  */ = 0x73,
		VK_F5 /*                  */ = 0x74,
		VK_F6 /*                  */ = 0x75,
		VK_F7 /*                  */ = 0x76,
		VK_F8 /*                  */ = 0x77,
		VK_F9 /*                  */ = 0x78,
		VK_F10 /*                 */ = 0x79,
		VK_F11 /*                 */ = 0x7A,
		VK_F12 /*                 */ = 0x7B,
		VK_F13 /*                 */ = 0x7C,
		VK_F14 /*                 */ = 0x7D,
		VK_F15 /*                 */ = 0x7E,
		VK_F16 /*                 */ = 0x7F,
		VK_F17 /*                 */ = 0x80,
		VK_F18 /*                 */ = 0x81,
		VK_F19 /*                 */ = 0x82,
		VK_F20 /*                 */ = 0x83,
		VK_F21 /*                 */ = 0x84,
		VK_F22 /*                 */ = 0x85,
		VK_F23 /*                 */ = 0x86,
		VK_F24 /*                 */ = 0x87,
		VK_NUMLOCK /*             */ = 0x90,
		VK_SCROLL /*              */ = 0x91,
		VK_OEM_NEC_EQUAL /*       */ = 0x92,
		VK_OEM_FJ_JISHO /*        */ = 0x92,
		VK_OEM_FJ_MASSHOU /*      */ = 0x93,
		VK_OEM_FJ_TOUROKU /*      */ = 0x94,
		VK_OEM_FJ_LOYA /*         */ = 0x95,
		VK_OEM_FJ_ROYA /*         */ = 0x96,
		VK_LSHIFT /*              */ = 0xA0,
		VK_RSHIFT /*              */ = 0xA1,
		VK_LCONTROL /*            */ = 0xA2,
		VK_RCONTROL /*            */ = 0xA3,
		VK_LMENU /*               */ = 0xA4,
		VK_RMENU /*               */ = 0xA5,
		VK_BROWSER_BACK /*        */ = 0xA6,
		VK_BROWSER_FORWARD /*     */ = 0xA7,
		VK_BROWSER_REFRESH /*     */ = 0xA8,
		VK_BROWSER_STOP /*        */ = 0xA9,
		VK_BROWSER_SEARCH /*      */ = 0xAA,
		VK_BROWSER_FAVORITES /*   */ = 0xAB,
		VK_BROWSER_HOME /*        */ = 0xAC,
		VK_VOLUME_MUTE /*         */ = 0xAD,
		VK_VOLUME_DOWN /*         */ = 0xAE,
		VK_VOLUME_UP /*           */ = 0xAF,
		VK_MEDIA_NEXT_TRACK /*    */ = 0xB0,
		VK_MEDIA_PREV_TRACK /*    */ = 0xB1,
		VK_MEDIA_STOP /*          */ = 0xB2,
		VK_MEDIA_PLAY_PAUSE /*    */ = 0xB3,
		VK_LAUNCH_MAIL /*         */ = 0xB4,
		VK_LAUNCH_MEDIA_SELECT /* */ = 0xB5,
		VK_LAUNCH_APP1 /*         */ = 0xB6,
		VK_LAUNCH_APP2 /*         */ = 0xB7,
		VK_OEM_1 /*               */ = 0xBA,
		VK_OEM_PLUS /*            */ = 0xBB,
		VK_OEM_COMMA /*           */ = 0xBC,
		VK_OEM_MINUS /*           */ = 0xBD,
		VK_OEM_PERIOD /*          */ = 0xBE,
		VK_OEM_2 /*               */ = 0xBF,
		VK_OEM_3 /*               */ = 0xC0,
		VK_OEM_4 /*               */ = 0xDB,
		VK_OEM_5 /*               */ = 0xDC,
		VK_OEM_6 /*               */ = 0xDD,
		VK_OEM_7 /*               */ = 0xDE,
		VK_OEM_8 /*               */ = 0xDF,
		VK_OEM_AX /*              */ = 0xE1,
		VK_OEM_102 /*             */ = 0xE2,
		VK_ICO_HELP /*            */ = 0xE3,
		VK_ICO_00 /*              */ = 0xE4,
		VK_PROCESSKEY /*          */ = 0xE5,
		VK_ICO_CLEAR /*           */ = 0xE6,
		VK_PACKET /*              */ = 0xE7,
		VK_OEM_RESET /*           */ = 0xE9,
		VK_OEM_JUMP /*            */ = 0xEA,
		VK_OEM_PA1 /*             */ = 0xEB,
		VK_OEM_PA2 /*             */ = 0xEC,
		VK_OEM_PA3 /*             */ = 0xED,
		VK_OEM_WSCTRL /*          */ = 0xEE,
		VK_OEM_CUSEL /*           */ = 0xEF,
		VK_OEM_ATTN /*            */ = 0xF0,
		VK_OEM_FINISH /*          */ = 0xF1,
		VK_OEM_COPY /*            */ = 0xF2,
		VK_OEM_AUTO /*            */ = 0xF3,
		VK_OEM_ENLW /*            */ = 0xF4,
		VK_OEM_BACKTAB /*         */ = 0xF5,
		VK_ATTN /*                */ = 0xF6,
		VK_CRSEL /*               */ = 0xF7,
		VK_EXSEL /*               */ = 0xF8,
		VK_EREOF /*               */ = 0xF9,
		VK_PLAY /*                */ = 0xFA,
		VK_ZOOM /*                */ = 0xFB,
		VK_NONAME /*              */ = 0xFC,
		VK_PA1 /*                 */ = 0xFD,
		VK_OEM_CLEAR /*           */ = 0xFE
	}

	#endregion

	#region Hooks

	public enum WindowHookID : int
	{
		WH_MSGFILTER /*          */ = -1,
		WH_JOURNALRECORD /*      */ = 0,
		WH_JOURNALPLAYBACK /*    */ = 1,
		WH_KEYBOARD /*           */ = 2,
		WH_GETMESSAGE /*         */ = 3,
		WH_CALLWNDPROC /*        */ = 4,
		WH_CBT /*                */ = 5,
		WH_SYSMSGFILTER /*       */ = 6,
		WH_MOUSE /*              */ = 7,
		WH_HARDWARE /*           */ = 8,
		WH_DEBUG /*              */ = 9,
		WH_SHELL /*             */ = 10,
		WH_FOREGROUNDIDLE /*    */ = 11,
		WH_CALLWNDPROCRET /*    */ = 12,
		WH_KEYBOARD_LL /*       */ = 13,
		WH_MOUSE_LL /*          */ = 14
	}

	public delegate int WindowsHookHandler(
		[In] int code,
		[In] IntPtr wParam,
		[In] IntPtr lParam);

	#endregion

	#endregion

	#region Controls

	#region Styles

	[Flags]
	internal enum CommonControlStyle
	{
		Top /*           */ = 0x00000001,
		NoMoveY /*       */ = 0x00000002,
		Bottom /*        */ = 0x00000003,
		NoResize /*      */ = 0x00000004,
		NoParentAlign /* */ = 0x00000008,
		Adjustable /*    */ = 0x00000020,
		NoDivider /*     */ = 0x00000040,
		Vert /*          */ = 0x00000080,
		Left /*          */ = Vert | Top,
		Right /*         */ = Vert | Bottom,
		NoMoveX /*       */ = Vert | NoMoveY
	}

	#endregion

	#region Notifications

	internal static class CommonControlNotifications
	{
		public const int NM_OUTOFMEMORY = -1;
		public const int NM_CLICK = -2;
		public const int NM_DBLCLK = -3;
		public const int NM_RETURN = -4;
		public const int NM_RCLICK = -5;
		public const int NM_RDBLCLK = -6;
		public const int NM_SETFOCUS = -7;
		public const int NM_KILLFOCUS = -8;
		public const int NM_CUSTOMDRAW = -12;
		public const int NM_HOVER = -13;
		public const int NM_NCHITTEST = -14;
		public const int NM_KEYDOWN = -15;
		public const int NM_RELEASEDCAPTURE = -16;
		public const int NM_SETCURSOR = -17;
		public const int NM_CHAR = -18;
		public const int NM_TOOLTIPSCREATED = -19;
		public const int NM_LDOWN = -20;
		public const int NM_RDOWN = -21;
		public const int NM_THEMECHANGED = -22;
	};

	#endregion

	#region Notification Parameters

	[Flags]
	internal enum CustomDrawStage : int
	{
		PrePaint /*         */ = 0x00000001,
		PostPaint /*        */ = 0x00000002,
		PreErase /*         */ = 0x00000003,
		PostErase /*        */ = 0x00000004,
		Item /*             */ = 0x00010000,
		ItemPrePaint /*     */ = Item | PrePaint,
		ItemPostPaint /*    */ = Item | PostPaint,
		ItemPreErase /*     */ = Item | PreErase,
		ItemPostErase /*    */ = Item | PostErase,
		SubItem /*          */ = 0x00020000,
		SubItemPrePaint /*  */ = SubItem | PrePaint,
		SubItemPostPaint /* */ = SubItem | PostPaint,
		SubItemPreErase  /* */ = SubItem | PreErase,
		SubItemPostErase /* */ = SubItem | PostErase,
	}

	[Flags]
	internal enum CustomDrawItemState : int
	{
		Selected /*         */ = 0x0001,
		Grayed /*           */ = 0x0002,
		Disabled /*         */ = 0x0004,
		Checked /*          */ = 0x0008,
		Focus /*            */ = 0x0010,
		Default /*          */ = 0x0020,
		Hot /*              */ = 0x0040,
		Marked /*           */ = 0x0080,
		Indeterminate /*    */ = 0x0100,
		ShowKeyboardCues /* */ = 0x0200
	}

	[Flags]
	internal enum CustomDrawReturn : int
	{
		DoDefault /*         */ = 0x00000000,
		NewFont /*           */ = 0x00000002,
		SkipDefault /*       */ = 0x00000004,
		NotifyPostPaint /*   */ = 0x00000010,
		NotifyItemDraw /*    */ = 0x00000020,
		NotifySubItemDraw /* */ = 0x00000020,
		NotifyPostErase /*   */ = 0x00000040
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct NMHeader
	{
		public IntPtr HWnd;
		public IntPtr ID;
		public int Code;
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct NotificationMouse
	{
		public NMHeader Header;
		public IntPtr Reserved1;
		public IntPtr Reserved2;
		public Point Point;
		public int HitInfo;
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct NotificationKey
	{
		public NMHeader Header;
		public int VKey;
		public int Flags;
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct NotificationChar
	{
		public NMHeader Header;
		public int Symbol;
		public int Reserved1;
		public int Reserved2;
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct NotificationTooltipsCreated
	{
		public NMHeader Header;
		public IntPtr HWndTooltips;
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct NotificationCustomDrawInfo
	{
		public NMHeader Header;
		[MarshalAs(UnmanagedType.U4)]
		public CustomDrawStage DrawStage;
		public IntPtr HDC;
		public Rectangle Rect;
		public int Item;
		[MarshalAs(UnmanagedType.U4)]
		public CustomDrawItemState ItemState;
		public int LParam;
	}

	#endregion

	#region Scrollbar

	[Flags]
	internal enum ScrollBarID : int
	{
		Horizontal,
		Vertical,
		Control,
		Both
	}

	[Flags]
	internal enum ScrollBarArrow : int
	{
		Enable /*       */ = 0,
		DisableLeft /*  */ = 1,
		DisableUp /*    */ = 1,
		DisableRight /* */ = 2,
		DisableDown /*  */ = 2,
		DisableBoth /*  */ = 3
	}

	internal enum ScrollBarObject : int
	{
		HScroll = unchecked((int)0xFFFFFFFA),
		VScroll = unchecked((int)0xFFFFFFFB),
		Client = unchecked((int)0xFFFFFFFC)
	}

	[Flags]
	internal enum ScrollBarInfoFlag
	{
		Range /*           */ = 1,
		Page /*            */ = 2,
		Position /*        */ = 4,
		DisableNoScroll /* */ = 8,
		TrackPosition /*   */ = 16,
		All /*             */ = (Range | Page | Position | DisableNoScroll | TrackPosition)
	}

	internal enum ScrollBarNotificationCode : int
	{
		LineUp /*        */ = 0,
		LineLeft /*      */ = 0,
		LineDown /*      */ = 1,
		LineRight /*     */ = 1,
		PageUp /*        */ = 2,
		PageLeft /*      */ = 2,
		PageDown /*      */ = 3,
		PageRight /*     */ = 3,
		ThumbPosition /* */ = 4,
		ThumbTrack /*    */ = 5,
		Top /*           */ = 6,
		Left /*          */ = 6,
		Bottom /*        */ = 7,
		Right /*         */ = 7,
		End /*           */ = 8
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct ScrollBarInfo
	{
		public int Size;
		public Rectangle ScrollBar;
		public int ThumbSize;
		public int ThumbTop;
		public int ThumbBottom;
		public int Reserved;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 6)]
		public int[] State;
	}

	[StructLayout(LayoutKind.Sequential)]
	internal struct ScrollInfo
	{
		public int Size;
		[MarshalAs(UnmanagedType.U4)]
		public ScrollBarInfoFlag Mask;
		public int Min;
		public int Max;
		public int Page;
		public int Position;
		public int TrackPosition;
	}

	[Flags]
	internal enum ScrollWindowFlags : int
	{
		ScrollChildren /* */ = 1,
		Invalidate /*     */ = 2,
		Erase /*          */ = 4,
		Smooth /*         */ = 16
	}

	#endregion

	#region Tooltip

	[Flags]
	internal enum ToolTipFlags : int
	{
		IDIsHWnd /*    */ = 0x0001,
		CenterTip /*   */ = 0x0002,
		RTLReading /*  */ = 0x0004,
		Subclass /*    */ = 0x0010,
		Track /*       */ = 0x0020,
		Absolute /*    */ = 0x0080,
		Transparent /* */ = 0x0100,
		ParseLinks /*  */ = 0x1000,
		SetItem /*     */ = 0x8000
	}

	[Flags]
	internal enum ToolTipDelayType : int
	{
		Automatic,
		ReShow,
		AutoPop,
		Initial
	}

	[Flags]
	internal enum ToolTipTitleIcon : int
	{
		None,
		Info,
		Warning,
		Error
	}

	[Flags]
	internal enum ToolTipStyle : int
	{
		AlwaysTip /* */ = 0x01,
		NoPrefix /*  */ = 0x02,
		NoAnimate /* */ = 0x10,
		NoFade /*    */ = 0x20,
		Balloon /*   */ = 0x40,
		Close /*     */ = 0x80
	}

	#endregion

	#endregion

	#region GDI

	internal enum DeviceCapIndex : int
	{
		DriverVersion /*   */ = 0,
		Technology /*      */ = 2,
		HorzSize /*        */ = 4,
		VertSize /*        */ = 6,
		HorzRes /*         */ = 8,
		VertRes /*         */ = 10,
		BitsPixel /*       */ = 12,
		Planes /*          */ = 14,
		NumBrushes /*      */ = 16,
		NumPens /*         */ = 18,
		NumMarkers /*      */ = 20,
		NumFonts /*        */ = 22,
		NumColors /*       */ = 24,
		PDeviceSize /*     */ = 26,
		CurveCaps /*       */ = 28,
		LineCaps /*        */ = 30,
		PolygonalCaps /*   */ = 32,
		TextCaps /*        */ = 34,
		ClipCaps /*        */ = 36,
		RasterCaps /*      */ = 38,
		AspectX /*         */ = 40,
		AspectY /*         */ = 42,
		AspectXY /*        */ = 44,
		LogPixelsX /*      */ = 88,
		LogPixelsY /*      */ = 90,
		SizePalette /*     */ = 104,
		NumReserved /*     */ = 106,
		ColorRes /*        */ = 108,
		PhysicalWidth /*   */ = 110,
		PhysicalHeight /*  */ = 111,
		PhysicalOffsetX /* */ = 112,
		PhysicalOffsetY /* */ = 113,
		ScalingFactorX /*  */ = 114,
		ScalingFactorY /*  */ = 115,
		VRefresh /*        */ = 116,
		DesktopVertRes /*  */ = 117,
		DesktopHorzRes /*  */ = 118,
		BltAlignment /*    */ = 119,
		ShadeBlendCaps /*  */ = 120,
		ColorMgmtCaps /*   */ = 121
	}

	internal enum FrameControlType : int
	{
		Caption /*   */ = 1,
		Menu /*      */ = 2,
		Scroll /*    */ = 3,
		Button /*    */ = 4,
		PopUpMenu /* */ = 5
	}

	[Flags]
	internal enum FrameControlState : int
	{
		CaptionClose /*        */ = 0x0000,
		CaptionMin /*          */ = 0x0001,
		CaptionMax /*          */ = 0x0002,
		CaptionRestore /*      */ = 0x0003,
		CaptionHelp /*         */ = 0x0004,
		MenuArrow /*           */ = 0x0000,
		MenuCheck /*           */ = 0x0001,
		MenuBullet /*          */ = 0x0002,
		MenuArrowRight /*      */ = 0x0004,
		ScrollUp /*            */ = 0x0000,
		ScrollDown /*          */ = 0x0001,
		ScrollLeft /*          */ = 0x0002,
		ScrollRight /*         */ = 0x0003,
		ScrollComboBox /*      */ = 0x0005,
		ScrollSizeGrip /*      */ = 0x0008,
		ScrollSizeGripRight /* */ = 0x0010,
		ButtonCheck /*         */ = 0x0000,
		ButtonRadioImage /*    */ = 0x0001,
		ButtonRadioMask /*     */ = 0x0002,
		ButtonRadio /*         */ = 0x0004,
		Button3State /*        */ = 0x0008,
		ButtonPush /*          */ = 0x0010,
		Inactive /*            */ = 0x0100,
		Pushed /*              */ = 0x0200,
		Checked /*             */ = 0x0400,
		Transparent /*         */ = 0x0800,
		Hot /*                 */ = 0x1000,
		AdjustRect /*          */ = 0x2000,
		Flat /*                */ = 0x4000,
		Mono /*                */ = 0x8000
	}

	internal struct PaintStruct
	{
		public IntPtr HDC;
		public bool Erase;
		public Rectangle RectPaint;
		public bool Reserved1;
		public bool Reserved2;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 32)]
		public byte[] Reserved3;
	}
#if COMPACT_FRAMEWORK
	[StructLayout(LayoutKind.Sequential)]
#else
	[StructLayout(LayoutKind.Sequential, Pack = 4)]
#endif
	internal struct LogFontA
	{
		public int Height;
		public int Width;
		public int Escapement;
		public int Orientation;
		public int Weight;
		public byte Italic;
		public byte Underline;
		public byte StrikeOut;
		public byte CharSet;
		public byte OutPrecision;
		public byte ClipPrecision;
		public byte Quality;
		public byte PitchAndFamily;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 0x20)]
		public char[] FaceName;
	}
#if COMPACT_FRAMEWORK
	[StructLayout(LayoutKind.Sequential)]
#else
	[StructLayout(LayoutKind.Sequential, Pack = 4)]
#endif
	internal struct LogFontW
	{
		public int Height;
		public int Width;
		public int Escapement;
		public int Orientation;
		public int Weight;
		public byte Italic;
		public byte Underline;
		public byte StrikeOut;
		public byte CharSet;
		public byte OutPrecision;
		public byte ClipPrecision;
		public byte Quality;
		public byte PitchAndFamily;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 0x20)]
		public short[] FaceName;
	}

	#endregion
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
