﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms Library                            */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Drawing;

namespace Nabu.Forms.TextObjectModel
{
	public static class ColorRTF
	{
		#region Fields

		private const int AutoColor = -9999997;

		#endregion

		#region Methods

		public static int FromColor(Color color)
		{
			if (color == Color.Transparent)
			{
				return AutoColor;
			}
			else
			{
				return (color.B << 16) | (color.G << 8) | (color.R);
			}
		}

		public static Color ToColor(int color)
		{
			if (color == AutoColor)
			{
				return Color.Transparent;
			}
			else
			{
				return Color.FromArgb((color) & 0xFF, (color >> 8) & 0xFF, (color >> 16) & 0xFF);
			}
		}

		#endregion
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum DocumentOpenSaveFlags : int
	{
		FormatRTF /*            */ = 1,
		FormatText /*           */ = 2,
		FormatHTML /*           */ = 3,
		FormatWordDocument /*   */ = 4,
		ModeCreateNew /*        */ = 0x10,
		ModeCreateAlways /*     */ = 0x20,
		ModeOpenExisting /*     */ = 0x30,
		ModeOpenAlways /*       */ = 0x40,
		ModeTruncateExisting /* */ = 0x50,
		ModeReadOnly /*         */ = 0x100,
		ModeShareDenyRead /*    */ = 0x200,
		ModeShareDenyWrite /*   */ = 0x400,
		ModePasteFile /*        */ = 0x1000
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	public enum ResetFlags : int
	{
		Default = -9999996,
		Undefined = -9999999
	}

	public enum AnimationType : int
	{
		None,
		LasVegasLights,
		BlinkingBackground,
		SparkleText,
		MarchingBlackAnts,
		MarchingRedAnts,
		Shimmer,
		WipeDown,
		WipeRight
	}

	public enum FormatCommand : int
	{
		Disable /*   */ = 0,
		Enable /*    */ = -1,
		Toggle /*    */ = -9999998,
		Undefined /* */ = -9999999,
	}

	public enum UnderlineType : int
	{
		None /*      */ = 0,
		Single /*    */ = 1,
		Words /*     */ = 2,
		Double /*    */ = 3,
		Dotted /*    */ = 4,
		True /*      */ = -1,
		Toggle /*    */ = -9999998,
		Undefined /* */ = -9999999,
	}

	public enum AlignmentFlags : int
	{
		Left /*     */ = 0,
		Center /*   */ = 1,
		Right /*    */ = 2,
		Justify /*  */ = 3,
		Decimal /*  */ = 3,
		Bar /*      */ = 4
	}

	[Flags]
	public enum ListTypeFlags : int
	{
		None /*             */ = 0,
		Bullet /*           */ = 1,
		NumberAsArabic /*   */ = 2,
		NumberAsLCLetter /* */ = 3,
		NumberAsUCLetter /* */ = 4,
		NumberAsLCRoman /*  */ = 5,
		NumberAsUCRoman /*  */ = 6,
		NumberAsSequence /* */ = 7,
		Parentheses /*      */ = 0x10000,
		Period /*           */ = 0x20000,
		Plain /*            */ = 0x30000
	}

	public enum LineSpacingFlags : int
	{
		Single /*    */ = 0,
		OnePtFive /* */ = 1,
		Double /*    */ = 2,
		AtLeast /*   */ = 3,
		Exactly /*   */ = 4,
		Multiple /*  */ = 5
	}

	public enum TabLeaderType : int
	{
		Spaces,
		Dots,
		Dashes,
		Lines
	}

	public enum StoryType : int
	{
		Unknown,
		MainText,
		Footnotes,
		Endnotes,
		Comments,
		TextFrame,
		EvenPagesHeader,
		PrimaryHeader,
		EvenPagesFooter,
		PrimaryFooter,
		FirstPageHeader,
		FirstPageFooter
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	[Flags]
	public enum PositionType : int
	{
		Start = 0x20,
		End = 0
	}

	public enum UnitType : int
	{
		Undefined,
		Character,
		Word,
		Sentence,
		Paragraph,
		Line,
		Story,
		Screen,
		Section,
		Column,
		Row,
		Window,
		Cell,
		CharFormat,
		ParaFormat,
		Table,
		Object
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	public enum FindDirectionFlags : int
	{
		Forward = 0x3FFFFFFF
	}

	[Flags]
	public enum FingMatchFlags : int
	{
		MatchWord = 2,
		MatchCase = 4,
		MatchPattern = 8
	}

	public enum CaseType : int
	{
		Lower,
		Upper,
		Title,
		Reserved,
		Sentence,
		Toggle
	}

	[Flags]
	public enum SelectionFlags : int
	{
		StartActive /* */ = 0x01,
		AtEOL /*       */ = 0x02,
		Overtype /*    */ = 0x04,
		Active /*      */ = 0x08,
		Replace /*     */ = 0x10
	}

	public enum SelectionType : int
	{
		None,
		IP,
		Normal,
		Frame,
		Column,
		Row,
		Block,
		InlineShape,
		Shape
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
