﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms Library                            */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace Nabu.Forms
{
	public static class CrossThreadControlAccessor
	{
		#region Control

		#region Properties

		#region ReadOnly

		#region AccessibilityObject

		public static AccessibleObject GetAccessibilityObject(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (AccessibleObject)control.Invoke(ControlDelegate.GetAccessibilityObject(control));
			}
			else
			{
				return ControlDelegate.GetAccessibilityObject(control).Invoke();
			}
		}

		#endregion

		#region Bottom

		public static int GetBottom(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (int)control.Invoke(ControlDelegate.GetBottom(control));
			}
			else
			{
				return ControlDelegate.GetBottom(control).Invoke();
			}
		}

		#endregion

		#region CanFocus

		public static bool GetCanFocus(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetCanFocus(control));
			}
			else
			{
				return ControlDelegate.GetCanFocus(control).Invoke();
			}
		}

		#endregion

		#region CanSelect

		public static bool GetCanSelect(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetCanSelect(control));
			}
			else
			{
				return ControlDelegate.GetCanSelect(control).Invoke();
			}
		}

		#endregion

		#region ClientRectangle

		public static Rectangle GetClientRectangle(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Rectangle)control.Invoke(ControlDelegate.GetClientRectangle(control));
			}
			else
			{
				return ControlDelegate.GetClientRectangle(control).Invoke();
			}
		}

		#endregion

		#region CompanyName

		public static string GetCompanyName(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (string)control.Invoke(ControlDelegate.GetCompanyName(control));
			}
			else
			{
				return ControlDelegate.GetCompanyName(control).Invoke();
			}
		}

		#endregion

		#region Container

		public static IContainer GetContainer(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (IContainer)control.Invoke(ControlDelegate.GetContainer(control));
			}
			else
			{
				return ControlDelegate.GetContainer(control).Invoke();
			}
		}

		#endregion

		#region ContainsFocus

		public static bool GetContainsFocus(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetContainsFocus(control));
			}
			else
			{
				return ControlDelegate.GetContainsFocus(control).Invoke();
			}
		}

		#endregion

		#region Controls

		public static Control.ControlCollection GetControls(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Control.ControlCollection)control.Invoke(ControlDelegate.GetControls(control));
			}
			else
			{
				return ControlDelegate.GetControls(control).Invoke();
			}
		}

		#endregion

		#region Created

		public static bool GetCreated(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetCreated(control));
			}
			else
			{
				return ControlDelegate.GetCreated(control).Invoke();
			}
		}

		#endregion

		#region DataBindings

		public static ControlBindingsCollection GetDataBindings(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (ControlBindingsCollection)control.Invoke(ControlDelegate.GetDataBindings(control));
			}
			else
			{
				return ControlDelegate.GetDataBindings(control).Invoke();
			}
		}

		#endregion

		#region DisplayRectangle

		public static Rectangle GetDisplayRectangle(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Rectangle)control.Invoke(ControlDelegate.GetDisplayRectangle(control));
			}
			else
			{
				return ControlDelegate.GetDisplayRectangle(control).Invoke();
			}
		}

		#endregion

		#region Disposing

		public static bool GetDisposing(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetDisposing(control));
			}
			else
			{
				return ControlDelegate.GetDisposing(control).Invoke();
			}
		}

		#endregion

		#region Focused

		public static bool GetFocused(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetFocused(control));
			}
			else
			{
				return ControlDelegate.GetFocused(control).Invoke();
			}
		}

		#endregion

		#region Handle

		public static IntPtr GetHandle(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (IntPtr)control.Invoke(ControlDelegate.GetHandle(control));
			}
			else
			{
				return ControlDelegate.GetHandle(control).Invoke();
			}
		}

		#endregion

		#region HasChildren

		public static bool GetHasChildren(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetHasChildren(control));
			}
			else
			{
				return ControlDelegate.GetHasChildren(control).Invoke();
			}
		}

		#endregion

		#region InvokeRequired

		public static bool GetInvokeRequired(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetInvokeRequired(control));
			}
			else
			{
				return ControlDelegate.GetInvokeRequired(control).Invoke();
			}
		}

		#endregion

		#region IsDisposed

		public static bool GetIsDisposed(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetIsDisposed(control));
			}
			else
			{
				return ControlDelegate.GetIsDisposed(control).Invoke();
			}
		}

		#endregion

		#region IsHandleCreated

		public static bool GetIsHandleCreated(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetIsHandleCreated(control));
			}
			else
			{
				return ControlDelegate.GetIsHandleCreated(control).Invoke();
			}
		}

		#endregion

		#region IsMirrored

		public static bool GetIsMirrored(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetIsMirrored(control));
			}
			else
			{
				return ControlDelegate.GetIsMirrored(control).Invoke();
			}
		}

		#endregion

		#region PreferredSize

		public static Size GetPreferredSize(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Size)control.Invoke(ControlDelegate.GetPreferredSize(control));
			}
			else
			{
				return ControlDelegate.GetPreferredSize(control).Invoke();
			}
		}

		#endregion

		#region ProductName

		public static string GetProductName(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (string)control.Invoke(ControlDelegate.GetProductName(control));
			}
			else
			{
				return ControlDelegate.GetProductName(control).Invoke();
			}
		}

		#endregion

		#region ProductVersion

		public static string GetProductVersion(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (string)control.Invoke(ControlDelegate.GetProductVersion(control));
			}
			else
			{
				return ControlDelegate.GetProductVersion(control).Invoke();
			}
		}

		#endregion

		#region RecreatingHandle

		public static bool GetRecreatingHandle(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetRecreatingHandle(control));
			}
			else
			{
				return ControlDelegate.GetRecreatingHandle(control).Invoke();
			}
		}

		#endregion

		#region Right

		public static int GetRight(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (int)control.Invoke(ControlDelegate.GetRight(control));
			}
			else
			{
				return ControlDelegate.GetRight(control).Invoke();
			}
		}

		#endregion

		#region TopLevelControl

		public static Control GetTopLevelControl(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Control)control.Invoke(ControlDelegate.GetTopLevelControl(control));
			}
			else
			{
				return ControlDelegate.GetTopLevelControl(control).Invoke();
			}
		}

		#endregion

		#endregion

		#region ReadWrite

		#region AccessibleDefaultActionDescription

		public static string GetAccessibleDefaultActionDescription(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (string)control.Invoke(ControlDelegate.GetAccessibleDefaultActionDescription(control));
			}
			else
			{
				return ControlDelegate.GetAccessibleDefaultActionDescription(control).Invoke();
			}
		}

		public static void SetAccessibleDefaultActionDescription(Control control, string value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetAccessibleDefaultActionDescription(control), value);
			}
			else
			{
				ControlDelegate.SetAccessibleDefaultActionDescription(control).Invoke(value);
			}
		}

		#endregion

		#region AccessibleDescription

		public static string GetAccessibleDescription(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (string)control.Invoke(ControlDelegate.GetAccessibleDescription(control));
			}
			else
			{
				return ControlDelegate.GetAccessibleDescription(control).Invoke();
			}
		}

		public static void SetAccessibleDescription(Control control, string value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetAccessibleDescription(control), value);
			}
			else
			{
				ControlDelegate.SetAccessibleDescription(control).Invoke(value);
			}
		}

		#endregion

		#region AccessibleName

		public static string GetAccessibleName(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (string)control.Invoke(ControlDelegate.GetAccessibleName(control));
			}
			else
			{
				return ControlDelegate.GetAccessibleName(control).Invoke();
			}
		}

		public static void SetAccessibleName(Control control, string value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetAccessibleName(control), value);
			}
			else
			{
				ControlDelegate.SetAccessibleName(control).Invoke(value);
			}
		}

		#endregion

		#region AccessibleRole

		public static AccessibleRole GetAccessibleRole(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (AccessibleRole)control.Invoke(ControlDelegate.GetAccessibleRole(control));
			}
			else
			{
				return ControlDelegate.GetAccessibleRole(control).Invoke();
			}
		}

		public static void SetAccessibleRole(Control control, AccessibleRole value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetAccessibleRole(control), value);
			}
			else
			{
				ControlDelegate.SetAccessibleRole(control).Invoke(value);
			}
		}

		#endregion

		#region AllowDrop

		public static bool GetAllowDrop(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetAllowDrop(control));
			}
			else
			{
				return ControlDelegate.GetAllowDrop(control).Invoke();
			}
		}

		public static void SetAllowDrop(Control control, bool value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetAllowDrop(control), value);
			}
			else
			{
				ControlDelegate.SetAllowDrop(control).Invoke(value);
			}
		}

		#endregion

		#region Anchor

		public static AnchorStyles GetAnchor(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (AnchorStyles)control.Invoke(ControlDelegate.GetAnchor(control));
			}
			else
			{
				return ControlDelegate.GetAnchor(control).Invoke();
			}
		}

		public static void SetAnchor(Control control, AnchorStyles value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetAnchor(control), value);
			}
			else
			{
				ControlDelegate.SetAnchor(control).Invoke(value);
			}
		}

		#endregion

		#region AutoScrollOffset

		public static Point GetAutoScrollOffset(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Point)control.Invoke(ControlDelegate.GetAutoScrollOffset(control));
			}
			else
			{
				return ControlDelegate.GetAutoScrollOffset(control).Invoke();
			}
		}

		public static void SetAutoScrollOffset(Control control, Point value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetAutoScrollOffset(control), value);
			}
			else
			{
				ControlDelegate.SetAutoScrollOffset(control).Invoke(value);
			}
		}

		#endregion

		#region AutoSize

		public static bool GetAutoSize(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetAutoSize(control));
			}
			else
			{
				return ControlDelegate.GetAutoSize(control).Invoke();
			}
		}

		public static void SetAutoSize(Control control, bool value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetAutoSize(control), value);
			}
			else
			{
				ControlDelegate.SetAutoSize(control).Invoke(value);
			}
		}

		#endregion

		#region BackColor

		public static Color GetBackColor(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Color)control.Invoke(ControlDelegate.GetBackColor(control));
			}
			else
			{
				return ControlDelegate.GetBackColor(control).Invoke();
			}
		}

		public static void SetBackColor(Control control, Color value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetBackColor(control), value);
			}
			else
			{
				ControlDelegate.SetBackColor(control).Invoke(value);
			}
		}

		#endregion

		#region BackgroundImage

		public static Image GetBackgroundImage(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Image)control.Invoke(ControlDelegate.GetBackgroundImage(control));
			}
			else
			{
				return ControlDelegate.GetBackgroundImage(control).Invoke();
			}
		}

		public static void SetBackgroundImage(Control control, Image value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetBackgroundImage(control), value);
			}
			else
			{
				ControlDelegate.SetBackgroundImage(control).Invoke(value);
			}
		}

		#endregion

		#region BackgroundImageLayout

		public static ImageLayout GetBackgroundImageLayout(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (ImageLayout)control.Invoke(ControlDelegate.GetBackgroundImageLayout(control));
			}
			else
			{
				return ControlDelegate.GetBackgroundImageLayout(control).Invoke();
			}
		}

		public static void SetBackgroundImageLayout(Control control, ImageLayout value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetBackgroundImageLayout(control), value);
			}
			else
			{
				ControlDelegate.SetBackgroundImageLayout(control).Invoke(value);
			}
		}

		#endregion

		#region BindingContext

		public static BindingContext GetBindingContext(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (BindingContext)control.Invoke(ControlDelegate.GetBindingContext(control));
			}
			else
			{
				return ControlDelegate.GetBindingContext(control).Invoke();
			}
		}

		public static void SetBindingContext(Control control, BindingContext value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetBindingContext(control), value);
			}
			else
			{
				ControlDelegate.SetBindingContext(control).Invoke(value);
			}
		}

		#endregion

		#region Bounds

		public static Rectangle GetBounds(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Rectangle)control.Invoke(ControlDelegate.GetBounds(control));
			}
			else
			{
				return ControlDelegate.GetBounds(control).Invoke();
			}
		}

		public static void SetBounds(Control control, Rectangle value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetBounds(control), value);
			}
			else
			{
				ControlDelegate.SetBounds(control).Invoke(value);
			}
		}

		#endregion

		#region Capture

		public static bool GetCapture(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetCapture(control));
			}
			else
			{
				return ControlDelegate.GetCapture(control).Invoke();
			}
		}

		public static void SetCapture(Control control, bool value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetCapture(control), value);
			}
			else
			{
				ControlDelegate.SetCapture(control).Invoke(value);
			}
		}

		#endregion

		#region CausesValidation

		public static bool GetCausesValidation(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetCausesValidation(control));
			}
			else
			{
				return ControlDelegate.GetCausesValidation(control).Invoke();
			}
		}

		public static void SetCausesValidation(Control control, bool value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetCausesValidation(control), value);
			}
			else
			{
				ControlDelegate.SetCausesValidation(control).Invoke(value);
			}
		}

		#endregion

		#region ClientSize

		public static Size GetClientSize(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Size)control.Invoke(ControlDelegate.GetClientSize(control));
			}
			else
			{
				return ControlDelegate.GetClientSize(control).Invoke();
			}
		}

		public static void SetClientSize(Control control, Size value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetClientSize(control), value);
			}
			else
			{
				ControlDelegate.SetClientSize(control).Invoke(value);
			}
		}

		#endregion

		#region ContextMenu

		public static ContextMenu GetContextMenu(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (ContextMenu)control.Invoke(ControlDelegate.GetContextMenu(control));
			}
			else
			{
				return ControlDelegate.GetContextMenu(control).Invoke();
			}
		}

		public static void SetContextMenu(Control control, ContextMenu value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetContextMenu(control), value);
			}
			else
			{
				ControlDelegate.SetContextMenu(control).Invoke(value);
			}
		}

		#endregion

		#region ContextMenuStrip

		public static ContextMenuStrip GetContextMenuStrip(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (ContextMenuStrip)control.Invoke(ControlDelegate.GetContextMenuStrip(control));
			}
			else
			{
				return ControlDelegate.GetContextMenuStrip(control).Invoke();
			}
		}

		public static void SetContextMenuStrip(Control control, ContextMenuStrip value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetContextMenuStrip(control), value);
			}
			else
			{
				ControlDelegate.SetContextMenuStrip(control).Invoke(value);
			}
		}

		#endregion

		#region Cursor

		public static Cursor GetCursor(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Cursor)control.Invoke(ControlDelegate.GetCursor(control));
			}
			else
			{
				return ControlDelegate.GetCursor(control).Invoke();
			}
		}

		public static void SetCursor(Control control, Cursor value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetCursor(control), value);
			}
			else
			{
				ControlDelegate.SetCursor(control).Invoke(value);
			}
		}

		#endregion

		#region Dock

		public static DockStyle GetDock(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (DockStyle)control.Invoke(ControlDelegate.GetDock(control));
			}
			else
			{
				return ControlDelegate.GetDock(control).Invoke();
			}
		}

		public static void SetDock(Control control, DockStyle value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetDock(control), value);
			}
			else
			{
				ControlDelegate.SetDock(control).Invoke(value);
			}
		}

		#endregion

		#region Enabled

		public static bool GetEnabled(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetEnabled(control));
			}
			else
			{
				return ControlDelegate.GetEnabled(control).Invoke();
			}
		}

		public static void SetEnabled(Control control, bool value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetEnabled(control), value);
			}
			else
			{
				ControlDelegate.SetEnabled(control).Invoke(value);
			}
		}

		#endregion

		#region Font

		public static Font GetFont(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Font)control.Invoke(ControlDelegate.GetFont(control));
			}
			else
			{
				return ControlDelegate.GetFont(control).Invoke();
			}
		}

		public static void SetFont(Control control, Font value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetFont(control), value);
			}
			else
			{
				ControlDelegate.SetFont(control).Invoke(value);
			}
		}

		#endregion

		#region ForeColor

		public static Color GetForeColor(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Color)control.Invoke(ControlDelegate.GetForeColor(control));
			}
			else
			{
				return ControlDelegate.GetForeColor(control).Invoke();
			}
		}

		public static void SetForeColor(Control control, Color value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetForeColor(control), value);
			}
			else
			{
				ControlDelegate.SetForeColor(control).Invoke(value);
			}
		}

		#endregion

		#region Height

		public static int GetHeight(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (int)control.Invoke(ControlDelegate.GetHeight(control));
			}
			else
			{
				return ControlDelegate.GetHeight(control).Invoke();
			}
		}

		public static void SetHeight(Control control, int value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetHeight(control), value);
			}
			else
			{
				ControlDelegate.SetHeight(control).Invoke(value);
			}
		}

		#endregion

		#region ImeMode

		public static ImeMode GetImeMode(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (ImeMode)control.Invoke(ControlDelegate.GetImeMode(control));
			}
			else
			{
				return ControlDelegate.GetImeMode(control).Invoke();
			}
		}

		public static void SetImeMode(Control control, ImeMode value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetImeMode(control), value);
			}
			else
			{
				ControlDelegate.SetImeMode(control).Invoke(value);
			}
		}

		#endregion

		#region IsAccessible

		public static bool GetIsAccessible(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetIsAccessible(control));
			}
			else
			{
				return ControlDelegate.GetIsAccessible(control).Invoke();
			}
		}

		public static void SetIsAccessible(Control control, bool value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetIsAccessible(control), value);
			}
			else
			{
				ControlDelegate.SetIsAccessible(control).Invoke(value);
			}
		}

		#endregion

		#region Left

		public static int GetLeft(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (int)control.Invoke(ControlDelegate.GetLeft(control));
			}
			else
			{
				return ControlDelegate.GetLeft(control).Invoke();
			}
		}

		public static void SetLeft(Control control, int value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetLeft(control), value);
			}
			else
			{
				ControlDelegate.SetLeft(control).Invoke(value);
			}
		}

		#endregion

		#region Location

		public static Point GetLocation(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Point)control.Invoke(ControlDelegate.GetLocation(control));
			}
			else
			{
				return ControlDelegate.GetLocation(control).Invoke();
			}
		}

		public static void SetLocation(Control control, Point value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetLocation(control), value);
			}
			else
			{
				ControlDelegate.SetLocation(control).Invoke(value);
			}
		}

		#endregion

		#region Margin

		public static Padding GetMargin(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Padding)control.Invoke(ControlDelegate.GetMargin(control));
			}
			else
			{
				return ControlDelegate.GetMargin(control).Invoke();
			}
		}

		public static void SetMargin(Control control, Padding value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetMargin(control), value);
			}
			else
			{
				ControlDelegate.SetMargin(control).Invoke(value);
			}
		}

		#endregion

		#region MaximumSize

		public static Size GetMaximumSize(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Size)control.Invoke(ControlDelegate.GetMaximumSize(control));
			}
			else
			{
				return ControlDelegate.GetMaximumSize(control).Invoke();
			}
		}

		public static void SetMaximumSize(Control control, Size value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetMaximumSize(control), value);
			}
			else
			{
				ControlDelegate.SetMaximumSize(control).Invoke(value);
			}
		}

		#endregion

		#region MinimumSize

		public static Size GetMinimumSize(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Size)control.Invoke(ControlDelegate.GetMinimumSize(control));
			}
			else
			{
				return ControlDelegate.GetMinimumSize(control).Invoke();
			}
		}

		public static void SetMinimumSize(Control control, Size value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetMinimumSize(control), value);
			}
			else
			{
				ControlDelegate.SetMinimumSize(control).Invoke(value);
			}
		}

		#endregion

		#region Name

		public static string GetName(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (string)control.Invoke(ControlDelegate.GetName(control));
			}
			else
			{
				return ControlDelegate.GetName(control).Invoke();
			}
		}

		public static void SetName(Control control, string value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetName(control), value);
			}
			else
			{
				ControlDelegate.SetName(control).Invoke(value);
			}
		}

		#endregion

		#region Padding

		public static Padding GetPadding(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Padding)control.Invoke(ControlDelegate.GetPadding(control));
			}
			else
			{
				return ControlDelegate.GetPadding(control).Invoke();
			}
		}

		public static void SetPadding(Control control, Padding value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetPadding(control), value);
			}
			else
			{
				ControlDelegate.SetPadding(control).Invoke(value);
			}
		}

		#endregion

		#region Parent

		public static Control GetParent(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Control)control.Invoke(ControlDelegate.GetParent(control));
			}
			else
			{
				return ControlDelegate.GetParent(control).Invoke();
			}
		}

		public static void SetParent(Control control, Control value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetParent(control), value);
			}
			else
			{
				ControlDelegate.SetParent(control).Invoke(value);
			}
		}

		#endregion

		#region Region

		public static Region GetRegion(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Region)control.Invoke(ControlDelegate.GetRegion(control));
			}
			else
			{
				return ControlDelegate.GetRegion(control).Invoke();
			}
		}

		public static void SetRegion(Control control, Region value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetRegion(control), value);
			}
			else
			{
				ControlDelegate.SetRegion(control).Invoke(value);
			}
		}

		#endregion

		#region RightToLeft

		public static RightToLeft GetRightToLeft(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (RightToLeft)control.Invoke(ControlDelegate.GetRightToLeft(control));
			}
			else
			{
				return ControlDelegate.GetRightToLeft(control).Invoke();
			}
		}

		public static void SetRightToLeft(Control control, RightToLeft value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetRightToLeft(control), value);
			}
			else
			{
				ControlDelegate.SetRightToLeft(control).Invoke(value);
			}
		}

		#endregion

		#region Site

		public static ISite GetSite(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (ISite)control.Invoke(ControlDelegate.GetSite(control));
			}
			else
			{
				return ControlDelegate.GetSite(control).Invoke();
			}
		}

		public static void SetSite(Control control, ISite value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetSite(control), value);
			}
			else
			{
				ControlDelegate.SetSite(control).Invoke(value);
			}
		}

		#endregion

		#region Size

		public static Size GetSize(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Size)control.Invoke(ControlDelegate.GetSize(control));
			}
			else
			{
				return ControlDelegate.GetSize(control).Invoke();
			}
		}

		public static void SetSize(Control control, Size value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetSize(control), value);
			}
			else
			{
				ControlDelegate.SetSize(control).Invoke(value);
			}
		}

		#endregion

		#region TabIndex

		public static int GetTabIndex(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (int)control.Invoke(ControlDelegate.GetTabIndex(control));
			}
			else
			{
				return ControlDelegate.GetTabIndex(control).Invoke();
			}
		}

		public static void SetTabIndex(Control control, int value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetTabIndex(control), value);
			}
			else
			{
				ControlDelegate.SetTabIndex(control).Invoke(value);
			}
		}

		#endregion

		#region TabStop

		public static bool GetTabStop(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetTabStop(control));
			}
			else
			{
				return ControlDelegate.GetTabStop(control).Invoke();
			}
		}

		public static void SetTabStop(Control control, bool value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetTabStop(control), value);
			}
			else
			{
				ControlDelegate.SetTabStop(control).Invoke(value);
			}
		}

		#endregion

		#region Tag

		public static object GetTag(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (object)control.Invoke(ControlDelegate.GetTag(control));
			}
			else
			{
				return ControlDelegate.GetTag(control).Invoke();
			}
		}

		public static void SetTag(Control control, object value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetTag(control), value);
			}
			else
			{
				ControlDelegate.SetTag(control).Invoke(value);
			}
		}

		#endregion

		#region Text

		public static string GetText(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (string)control.Invoke(ControlDelegate.GetText(control));
			}
			else
			{
				return ControlDelegate.GetText(control).Invoke();
			}
		}

		public static void SetText(Control control, string value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetText(control), value);
			}
			else
			{
				ControlDelegate.SetText(control).Invoke(value);
			}
		}

		#endregion

		#region Top

		public static int GetTop(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (int)control.Invoke(ControlDelegate.GetTop(control));
			}
			else
			{
				return ControlDelegate.GetTop(control).Invoke();
			}
		}

		public static void SetTop(Control control, int value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetTop(control), value);
			}
			else
			{
				ControlDelegate.SetTop(control).Invoke(value);
			}
		}

		#endregion

		#region UseWaitCursor

		public static bool GetUseWaitCursor(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetUseWaitCursor(control));
			}
			else
			{
				return ControlDelegate.GetUseWaitCursor(control).Invoke();
			}
		}

		public static void SetUseWaitCursor(Control control, bool value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetUseWaitCursor(control), value);
			}
			else
			{
				ControlDelegate.SetUseWaitCursor(control).Invoke(value);
			}
		}

		#endregion

		#region Visible

		public static bool GetVisible(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetVisible(control));
			}
			else
			{
				return ControlDelegate.GetVisible(control).Invoke();
			}
		}

		public static void SetVisible(Control control, bool value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetVisible(control), value);
			}
			else
			{
				ControlDelegate.SetVisible(control).Invoke(value);
			}
		}

		#endregion

		#region Width

		public static int GetWidth(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (int)control.Invoke(ControlDelegate.GetWidth(control));
			}
			else
			{
				return ControlDelegate.GetWidth(control).Invoke();
			}
		}

		public static void SetWidth(Control control, int value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetWidth(control), value);
			}
			else
			{
				ControlDelegate.SetWidth(control).Invoke(value);
			}
		}

		#endregion

		#region WindowTarget

		public static IWindowTarget GetWindowTarget(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (IWindowTarget)control.Invoke(ControlDelegate.GetWindowTarget(control));
			}
			else
			{
				return ControlDelegate.GetWindowTarget(control).Invoke();
			}
		}

		public static void SetWindowTarget(Control control, IWindowTarget value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetWindowTarget(control), value);
			}
			else
			{
				ControlDelegate.SetWindowTarget(control).Invoke(value);
			}
		}

		#endregion

		#endregion

		#endregion

		#endregion

		#region TextBoxBase

		#region Methods

		#region AppendText

		public static void AppendText(TextBoxBase control, string text)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.AppendText(control), text);
			}
			else
			{
				ControlDelegate.AppendText(control).Invoke(text);
			}
		}

		#endregion

		#region Clear

		public static void Clear(TextBoxBase control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.Clear(control));
			}
			else
			{
				ControlDelegate.Clear(control).Invoke();
			}
		}

		#endregion

		#region Cut

		public static void Cut(TextBoxBase control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.Cut(control));
			}
			else
			{
				ControlDelegate.Cut(control).Invoke();
			}
		}

		#endregion

		#region Copy

		public static void Copy(TextBoxBase control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.Copy(control));
			}
			else
			{
				ControlDelegate.Copy(control).Invoke();
			}
		}

		#endregion

		#region Paste

		public static void Paste(TextBoxBase control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.Paste(control));
			}
			else
			{
				ControlDelegate.Paste(control).Invoke();
			}
		}

		#endregion

		#endregion

		#region Properties

		#region TextLength

		public static int GetTextLength(TextBoxBase control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (int)control.Invoke(ControlDelegate.GetTextLength(control));
			}
			else
			{
				return ControlDelegate.GetTextLength(control).Invoke();
			}
		}

		public static void SetTextLength(TextBoxBase control, int value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetTextLength(control), value);
			}
			else
			{
				ControlDelegate.SetTextLength(control).Invoke(value);
			}
		}

		#endregion

		#endregion

		#endregion

		#region ListControl

		#region Properties

		#region SelectedIndex

		public static int GetSelectedIndex(ListControl control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (int)control.Invoke(ControlDelegate.GetSelectedIndex(control));
			}
			else
			{
				return ControlDelegate.GetSelectedIndex(control).Invoke();
			}
		}

		public static void SetSelectedIndex(ListControl control, int value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetSelectedIndex(control), value);
			}
			else
			{
				ControlDelegate.SetSelectedIndex(control).Invoke(value);
			}
		}

		#endregion

		#region SelectedItem

		public static object GetSelectedItem(ListControl control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (object)control.Invoke(ControlDelegate.GetSelectedItem(control));
			}
			else
			{
				return ControlDelegate.GetSelectedItem(control).Invoke();
			}
		}

		public static void SetSelectedItem(ListControl control, object value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetSelectedItem(control), value);
			}
			else
			{
				ControlDelegate.SetSelectedItem(control).Invoke(value);
			}
		}

		#endregion

		#endregion

		#endregion

		#region CheckBox

		#region Properties

		#region Checked

		public static bool GetChecked(CheckBox control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (bool)control.Invoke(ControlDelegate.GetChecked(control));
			}
			else
			{
				return ControlDelegate.GetChecked(control).Invoke();
			}
		}

		public static void SetChecked(CheckBox control, bool value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetChecked(control), value);
			}
			else
			{
				ControlDelegate.SetChecked(control).Invoke(value);
			}
		}

		#endregion

		#region CheckState

		public static CheckState GetCheckState(CheckBox control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (CheckState)control.Invoke(ControlDelegate.GetCheckState(control));
			}
			else
			{
				return ControlDelegate.GetCheckState(control).Invoke();
			}
		}

		public static void SetCheckState(CheckBox control, CheckState value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetCheckState(control), value);
			}
			else
			{
				ControlDelegate.SetCheckState(control).Invoke(value);
			}
		}

		#endregion

		#endregion

		#endregion

		#region Form

		#region Methods

		public static void Close(Form control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.Close(control));
			}
			else
			{
				ControlDelegate.Close(control).Invoke();
			}
		}

		#endregion

		#region Properties

		public static DialogResult GetDialogResult(Form control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (DialogResult)control.Invoke(ControlDelegate.GetDialogResult(control));
			}
			else
			{
				return ControlDelegate.GetDialogResult(control).Invoke();
			}
		}

		public static void SetDialogResult(Form control, DialogResult value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetDialogResult(control), value);
			}
			else
			{
				ControlDelegate.SetDialogResult(control).Invoke(value);
			}
		}

		#endregion

		#endregion

		#region ProgressBar

		#region Properties

		#region Value

		public static int GetValue(ProgressBar control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (int)control.Invoke(ControlDelegate.GetValue(control));
			}
			else
			{
				return ControlDelegate.GetValue(control).Invoke();
			}
		}

		public static void SetValue(ProgressBar control, int value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetValue(control), value);
			}
			else
			{
				ControlDelegate.SetValue(control).Invoke(value);
			}
		}

		#endregion

		#endregion

		#endregion

		#region ProgressDisk

		#region Properties

		#region CurrentPiece

		public static int GetCurrentPiece(ProgressDisk control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (int)control.Invoke(ControlDelegate.GetCurrentPiece(control));
			}
			else
			{
				return ControlDelegate.GetCurrentPiece(control).Invoke();
			}
		}

		public static void SetCurrentPiece(ProgressDisk control, int value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetCurrentPiece(control), value);
			}
			else
			{
				ControlDelegate.SetCurrentPiece(control).Invoke(value);
			}
		}

		#endregion

		#region ColorFrom

		public static Color GetColorFrom(ProgressDisk control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Color)control.Invoke(ControlDelegate.GetColorFrom(control));
			}
			else
			{
				return ControlDelegate.GetColorFrom(control).Invoke();
			}
		}

		public static void SetColorFrom(ProgressDisk control, Color value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetColorFrom(control), value);
			}
			else
			{
				ControlDelegate.SetColorFrom(control).Invoke(value);
			}
		}

		#endregion

		#region ColorTo

		public static Color GetColorTo(ProgressDisk control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (Color)control.Invoke(ControlDelegate.GetColorTo(control));
			}
			else
			{
				return ControlDelegate.GetColorTo(control).Invoke();
			}
		}

		public static void SetColorTo(ProgressDisk control, Color value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetColorTo(control), value);
			}
			else
			{
				ControlDelegate.SetColorTo(control).Invoke(value);
			}
		}

		#endregion

		#endregion

		#endregion

		#region HtmlView/HtmlForm

		#region Properties

		#region Html

		public static string GetHtml(Control control)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				return (string)control.Invoke(ControlDelegate.GetHtml(control));
			}
			else
			{
				return ControlDelegate.GetHtml(control).Invoke();
			}
		}

		public static void SetHtml(Control control, string value)
		{
			if (control == null)
			{
				throw new ArgumentNullException("control");
			}

			if (control.InvokeRequired)
			{
				control.Invoke(ControlDelegate.SetHtml(control), value);
			}
			else
			{
				ControlDelegate.SetHtml(control).Invoke(value);
			}
		}

		#endregion

		#endregion

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
