﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.IO Library                               */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
using System.IO;

namespace Nabu.IO
{
	public sealed class FileFindInfo
	{
		#region Fields

		private string _name;
		private string _alternateName;
		private FileAttributes _attributes;
		private DateTime _creationTime;
		private DateTime _lastAccessTime;
		private DateTime _lastWriteTime;
		private long _size;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		internal FileFindInfo(FileFindData data)
		{
			this._name =
				new string(
					data.Name,
					0,
					Array.IndexOf<char>(data.Name, '\0'));

			if (data.AlternateName[0] == '\0')
			{
				this._alternateName = this._name;
			}
			else
			{
				this._alternateName =
					new string(
						data.AlternateName,
						0,
						Array.IndexOf<char>(data.AlternateName, '\0'));
			}

			this._attributes = data.Attributes;
			this._creationTime =
				DateTime.FromFileTime(
					((long)data.CreationTimeHigh << 32) + ((long)data.CreationTimeLow));
			this._lastAccessTime =
				DateTime.FromFileTime(
					((long)data.LastAccessTimeHigh << 32) + ((long)data.LastAccessTimeLow));
			this._lastWriteTime =
				DateTime.FromFileTime(
					((long)data.LastWriteTimeHigh) + ((long)data.LastWriteTimeLow));
			this._size = (((long)data.SizeHigh) << 32) + ((long)data.SizeLow);
		}

		#endregion

		#region Properties

		public string Name
		{
			[DebuggerStepThrough]
			get { return this._name; }
		}

		public string AlternateName
		{
			[DebuggerStepThrough]
			get { return this._alternateName; }
		}

		public FileAttributes Attributes
		{
			[DebuggerStepThrough]
			get { return this._attributes; }
		}

		public DateTime CreationTime
		{
			[DebuggerStepThrough]
			get { return this._creationTime; }
		}

		public DateTime LastAccessTime
		{
			[DebuggerStepThrough]
			get { return this._lastAccessTime; }
		}

		public DateTime LastWriteTime
		{
			[DebuggerStepThrough]
			get { return this._lastWriteTime; }
		}

		public long Size
		{
			[DebuggerStepThrough]
			get { return this._size; }
		}

		public bool IsReadOnly
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.ReadOnly) == FileAttributes.ReadOnly; }
		}

		public bool IsHidden
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.Hidden) == FileAttributes.Hidden; }
		}

		public bool IsSystem
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.System) == FileAttributes.System; }
		}

		public bool IsDirectory
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.Directory) == FileAttributes.Directory; }
		}

		public bool IsArchive
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.Archive) == FileAttributes.Archive; }
		}

		public bool IsDevice
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.Device) == FileAttributes.Device; }
		}

		public bool IsNormal
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.Normal) == FileAttributes.Normal; }
		}

		public bool IsTemporary
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.Temporary) == FileAttributes.Temporary; }
		}

		public bool IsSparse
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.SparseFile) == FileAttributes.SparseFile; }
		}

		public bool IsReparsePoint
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.ReparsePoint) == FileAttributes.ReparsePoint; }
		}

		public bool IsCompressed
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.Compressed) == FileAttributes.Compressed; }
		}

		public bool IsOffline
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.Offline) == FileAttributes.Offline; }
		}

		public bool IsNotContentIndexed
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.NotContentIndexed) == FileAttributes.NotContentIndexed; }
		}

		public bool IsEncrypted
		{
			[DebuggerStepThrough]
			get { return (this._attributes & FileAttributes.Encrypted) == FileAttributes.Encrypted; }
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
