﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.IO Library                               */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.IO;

namespace Nabu.IO
{
	public sealed class MSCabArchiveReader
	{
		#region Constants

		private const string SignatureExceptionMessage = "CAB signature expected.";
		private const string SizeExceptionMessage = "Invalid CAB size.";

		#endregion

		#region Methods

		public MSCabArchiveReader()
		{
		}

		public void Read(Stream stream)
		{
			using (StreamHelper streamHelper = new StreamHelper(stream))
			{
				int signature = streamHelper.ReadI4();

				if (signature != 0x4643534d)
				{
					throw new InvalidStreamException("CAB signature expected.");
				}

				streamHelper.ReadU4();

				int cabinetSize = streamHelper.ReadI4();

				if (cabinetSize != stream.Length)
				{
					throw new InvalidStreamException("Invalid CAB size.");
				}

				streamHelper.ReadU4();

				int firstFileOffset = streamHelper.ReadI4();

				streamHelper.ReadU4();

				/* byte cabinetVersionMinor = */
				streamHelper.ReadU1();
				/* byte cabinetVersionMajor = */
				streamHelper.ReadU1();

				ushort folderCount = streamHelper.ReadU2();
				ushort fileCount = streamHelper.ReadU2();
				ushort cabinetFlags = streamHelper.ReadU2();
				/* ushort cabinetSetID = */
				streamHelper.ReadU2();
				/* ushort cabinetSetIndex = */
				streamHelper.ReadU2();
				ushort cabinetExtraDataSize = 0;
				byte folderExtraDataSize = 0;
				byte dataExtraDataSize = 0;

				// Extra Data Available
				if ((cabinetFlags & 4) != 0)
				{
					cabinetExtraDataSize = streamHelper.ReadU2();
					folderExtraDataSize = streamHelper.ReadU1();
					dataExtraDataSize = streamHelper.ReadU1();
				}

				stream.Seek(cabinetExtraDataSize, SeekOrigin.Current);

				// Previous Volume
				if ((cabinetFlags & 1) != 0)
				{
					// Cabinet Name
					for (
						byte value = streamHelper.ReadU1();
						value != 0;
						value = streamHelper.ReadU1())
					{
					}

					// Disk Name
					for (
						byte value = streamHelper.ReadU1();
						value != 0;
						value = streamHelper.ReadU1())
					{
					}
				}

				// Next Volume
				if ((cabinetFlags & 2) != 0)
				{
					// Cabinet Name
					for (
						byte value = streamHelper.ReadU1();
						value != 0;
						value = streamHelper.ReadU1())
					{
					}

					// Disk Name
					for (
						byte value = streamHelper.ReadU1();
						value != 0;
						value = streamHelper.ReadU1())
					{
					}
				}

				List<MSCabFolder> folders = new List<MSCabFolder>();

				for (ushort index = 0; index < folderCount; index++)
				{
					folders.Add(new MSCabFolder(streamHelper));
					stream.Seek(folderExtraDataSize, SeekOrigin.Current);
				}

				stream.Seek(firstFileOffset, SeekOrigin.Begin);

				List<MSCabFile> files = new List<MSCabFile>();

				for (ushort index = 0; index < fileCount; index++)
				{
					files.Add(new MSCabFile(streamHelper));
				}

				if (this.File != null)
				{
					foreach (MSCabFile file in files)
					{
						this.File(
							this,
							new MSCabFileEventArgs(
								stream,
								folders[file.FolderIndex],
								file,
								dataExtraDataSize));
					}
				}
			}
		}

		#endregion

		#region Events

		public event EventHandler<MSCabFileEventArgs> File;

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
