﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.IO Library                               */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;

namespace Nabu.IO
{
	internal sealed class MSCabFile
	{
		#region Fields

		private ushort _folderIndex;
		private int _decompressedOffset;
		private int _decompressedSize;
		private string _filePath;
		private DateTime _modificationTime;
		private FileAttributes _attributes;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public MSCabFile(StreamHelper streamHelper)
		{
			this._decompressedSize = streamHelper.ReadI4();
			this._decompressedOffset = streamHelper.ReadI4();
			this._folderIndex = streamHelper.ReadU2();

			ushort date = streamHelper.ReadU2();
			ushort time = streamHelper.ReadU2();

			this._modificationTime =
				new DateTime(
					1980 + ((date >> 9) & 0x7F),
					(date >> 5) & 0x0F,
					(date >> 0) & 0x1F,
					(time >> 11) & 0x1F,
					(time >> 5) & 0x3F,
					2 * ((time >> 0) & 0x1F));

			this._attributes = (FileAttributes)streamHelper.ReadU2();

			List<byte> filePath = new List<byte>();

			for (
					byte value = streamHelper.ReadU1();
					value != 0;
					value = streamHelper.ReadU1())
			{
				filePath.Add(value);
			}

			// Is Unicode?
			if ((this._attributes & FileAttributes.Normal) != 0)
			{
				byte[] filePathData = filePath.ToArray();

				this._filePath = Encoding.UTF8.GetString(filePathData, 0, filePathData.Length);
			}
			else
			{
				byte[] filePathData = filePath.ToArray();

				this._filePath = Encoding.Default.GetString(filePathData, 0, filePathData.Length);
			}
		}

		#endregion

		#region Properties

		public ushort FolderIndex
		{
			[DebuggerStepThrough]
			get { return this._folderIndex; }
		}

		public int DecompressedOffset
		{
			[DebuggerStepThrough]
			get { return this._decompressedOffset; }
		}

		public int DecompressedSize
		{
			[DebuggerStepThrough]
			get { return this._decompressedSize; }
		}

		public string FilePath
		{
			[DebuggerStepThrough]
			get { return this._filePath; }
		}

		public DateTime ModificationTime
		{
			[DebuggerStepThrough]
			get { return this._modificationTime; }
		}

		public FileAttributes Attributes
		{
			[DebuggerStepThrough]
			get { return this._attributes; }
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
