﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.IO Library                               */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.IO;

namespace Nabu.IO
{
	public sealed class MSCabFileEventArgs : EventArgs
	{
		#region Fields

		private readonly Stream _stream;
		private readonly int _dataOffset;
		private readonly MSCabCompressionType _compressionType;
		private readonly int _decompressedOffset;
		private readonly int _decompressedSize;
		private readonly string _filePath;
		private readonly DateTime _modificationTime;
		private readonly FileAttributes _attributes;
		private readonly byte _dataExtraDataSize;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		internal MSCabFileEventArgs(
			Stream stream,
			MSCabFolder folder,
			MSCabFile file,
			byte dataExtraDataSize)
		{
			this._stream = stream;
			this._dataOffset = folder.DataOffset;
			this._compressionType = folder.CompressionType;
			this._decompressedOffset = file.DecompressedOffset;
			this._decompressedSize = file.DecompressedSize;
			this._filePath = file.FilePath;
			this._modificationTime = file.ModificationTime;
			this._attributes = file.Attributes;
			this._dataExtraDataSize = dataExtraDataSize;
		}

		[SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
		public Stream GetDecompressedStream()
		{
			return
				new MSCabFileStream(
					this._stream,
					this._dataOffset,
					this._compressionType,
					this._decompressedOffset,
					this._decompressedSize,
					this._dataExtraDataSize);
		}

		#endregion

		#region Properties

		#region Compressor

		public int DecompressedSize
		{
			[DebuggerStepThrough]
			get { return this._decompressedSize; }
		}

		#endregion

		#region File

		public string FilePath
		{
			[DebuggerStepThrough]
			get { return this._filePath; }
		}

		public DateTime ModificationTime
		{
			[DebuggerStepThrough]
			get { return this._modificationTime; }
		}

		public FileAttributes Attributes
		{
			[DebuggerStepThrough]
			get { return this._attributes; }
		}

		#endregion

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
