﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.IO Library                               */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
using System.IO;

namespace Nabu.IO
{
	[CLSCompliant(false)]
	public sealed class Crc32Stream : Stream
	{
		#region Crc32Stream

		#region Fields

		private Stream _stream;
		private Crc32 _crc32Read;
		private Crc32 _crc32Write;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public Crc32Stream(Stream stream)
		{
			this._stream = stream;
			this._crc32Read = new Crc32();
			this._crc32Write = new Crc32();
		}

		#endregion

		#region Properties

		public uint Crc32Read
		{
			[DebuggerStepThrough]
			get { return this._crc32Read.Value; }
		}

		public uint Crc32Write
		{
			[DebuggerStepThrough]
			get { return this._crc32Write.Value; }
		}

		#endregion

		#endregion

		#region Stream

		#region Methods

		public override int Read(byte[] buffer, int offset, int count)
		{
			int result = this._stream.Read(buffer, offset, count);

			this._crc32Read.ComputeHash2(buffer, offset, result);

			return result;
		}

		public override void Write(byte[] buffer, int offset, int count)
		{
			this._stream.Write(buffer, offset, count);
			this._crc32Write.ComputeHash2(buffer, offset, count);
		}

		public override long Seek(long offset, SeekOrigin origin)
		{
			return this._stream.Seek(offset, origin);
		}

		public override void SetLength(long value)
		{
			this._stream.SetLength(value);
		}

		public override void Flush()
		{
			this._stream.Flush();
		}

		#endregion

		#region Properties

		public override bool CanRead
		{
			[DebuggerStepThrough]
			get { return this._stream.CanRead; }
		}

		public override bool CanWrite
		{
			[DebuggerStepThrough]
			get { return this._stream.CanWrite; }
		}

		public override bool CanSeek
		{
			[DebuggerStepThrough]
			get { return this._stream.CanSeek; }
		}

		public override long Length
		{
			[DebuggerStepThrough]
			get { return this._stream.Length; }
		}

		public override long Position
		{
			[DebuggerStepThrough]
			get { return this._stream.Position; }
			[DebuggerStepThrough]
			set { this._stream.Position = value; }
		}

		#endregion

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
