﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.IO Library                               */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.IO;

namespace Nabu.IO
{
	[CLSCompliant(false)]
	public sealed class StreamHelper : DisposableBase
	{
		#region StreamHelper

		#region Fields

		private Stream _stream;
		private byte[] _buffer;

		#endregion

		#region Methods

		#region Constructor

		public StreamHelper(Stream stream)
		{
			this._stream = stream;
			this._buffer = new byte[8];
		}

		#endregion

		#region Read

		public sbyte ReadI1()
		{
			base.CheckDisposed();

			this._stream.Read(this._buffer, 0, 1);

			return (sbyte)this._buffer[0];
		}

		public byte ReadU1()
		{
			base.CheckDisposed();

			this._stream.Read(this._buffer, 0, 1);

			return (byte)this._buffer[0];
		}

		public short ReadI2()
		{
			base.CheckDisposed();

			this._stream.Read(this._buffer, 0, 2);

			return (short)(
				((ushort)this._buffer[1] << 8) |
				((ushort)this._buffer[0]));
		}

		public ushort ReadU2()
		{
			base.CheckDisposed();

			this._stream.Read(this._buffer, 0, 2);

			return (ushort)(
				((ushort)this._buffer[1] << 8) |
				((ushort)this._buffer[0]));
		}

		public int ReadI4()
		{
			base.CheckDisposed();

			this._stream.Read(this._buffer, 0, 4);

			return (int)(
				((uint)this._buffer[3] << 24) |
				((uint)this._buffer[2] << 16) |
				((uint)this._buffer[1] << 8) |
				((uint)this._buffer[0]));
		}

		public uint ReadU4()
		{
			base.CheckDisposed();

			this._stream.Read(this._buffer, 0, 4);

			return (uint)(
				((uint)this._buffer[3] << 24) |
				((uint)this._buffer[2] << 16) |
				((uint)this._buffer[1] << 8) |
				((uint)this._buffer[0]));
		}

		public long ReadI8()
		{
			base.CheckDisposed();

			this._stream.Read(this._buffer, 0, 8);

			return (long)(
				((ulong)this._buffer[7] << 56) |
				((ulong)this._buffer[6] << 48) |
				((ulong)this._buffer[5] << 40) |
				((ulong)this._buffer[4] << 32) |
				((ulong)this._buffer[3] << 24) |
				((ulong)this._buffer[2] << 16) |
				((ulong)this._buffer[1] << 8) |
				((ulong)this._buffer[0]));
		}

		public ulong ReadU8()
		{
			base.CheckDisposed();

			this._stream.Read(this._buffer, 0, 8);

			return (ulong)(
				((ulong)this._buffer[7] << 56) |
				((ulong)this._buffer[6] << 48) |
				((ulong)this._buffer[5] << 40) |
				((ulong)this._buffer[4] << 32) |
				((ulong)this._buffer[3] << 24) |
				((ulong)this._buffer[2] << 16) |
				((ulong)this._buffer[1] << 8) |
				((ulong)this._buffer[0]));
		}

		public byte[] ReadArray(int length)
		{
			base.CheckDisposed();

			byte[] buffer = new byte[length];

			for (int index = 0; index < length; )
			{
				index += this._stream.Read(buffer, index, length - index);
			}

			return buffer;
		}

		#endregion

		#region Write

		public void WriteI1(sbyte value)
		{
			base.CheckDisposed();

			this._buffer[0] = (byte)value;
			this._stream.Write(this._buffer, 0, 1);
		}

		public void WriteU1(byte value)
		{
			base.CheckDisposed();

			this._buffer[0] = (byte)value;
			this._stream.Write(this._buffer, 0, 1);
		}

		public void WriteI2(short value)
		{
			base.CheckDisposed();

			this._buffer[0] = (byte)((value >> 0) & 0xFF);
			this._buffer[1] = (byte)((value >> 8) & 0xFF);
			this._stream.Write(this._buffer, 0, 2);
		}

		public void WriteU2(ushort value)
		{
			base.CheckDisposed();

			this._buffer[0] = (byte)((value >> 0) & 0xFF);
			this._buffer[1] = (byte)((value >> 8) & 0xFF);
			this._stream.Write(this._buffer, 0, 2);
		}

		public void WriteI4(int value)
		{
			base.CheckDisposed();

			this._buffer[0] = (byte)((value >> 0) & 0xFF);
			this._buffer[1] = (byte)((value >> 8) & 0xFF);
			this._buffer[2] = (byte)((value >> 16) & 0xFF);
			this._buffer[3] = (byte)((value >> 24) & 0xFF);
			this._stream.Write(this._buffer, 0, 4);
		}

		public void WriteU4(uint value)
		{
			base.CheckDisposed();

			this._buffer[0] = (byte)((value >> 0) & 0xFF);
			this._buffer[1] = (byte)((value >> 8) & 0xFF);
			this._buffer[2] = (byte)((value >> 16) & 0xFF);
			this._buffer[3] = (byte)((value >> 24) & 0xFF);
			this._stream.Write(this._buffer, 0, 4);
		}

		public void WriteI8(long value)
		{
			base.CheckDisposed();

			this._buffer[0] = (byte)((value >> 0) & 0xFF);
			this._buffer[1] = (byte)((value >> 8) & 0xFF);
			this._buffer[2] = (byte)((value >> 16) & 0xFF);
			this._buffer[3] = (byte)((value >> 24) & 0xFF);
			this._buffer[4] = (byte)((value >> 32) & 0xFF);
			this._buffer[5] = (byte)((value >> 40) & 0xFF);
			this._buffer[6] = (byte)((value >> 48) & 0xFF);
			this._buffer[7] = (byte)((value >> 56) & 0xFF);
			this._stream.Write(this._buffer, 0, 8);
		}

		public void WriteU8(ulong value)
		{
			base.CheckDisposed();

			this._buffer[0] = (byte)((value >> 0) & 0xFF);
			this._buffer[1] = (byte)((value >> 8) & 0xFF);
			this._buffer[2] = (byte)((value >> 16) & 0xFF);
			this._buffer[3] = (byte)((value >> 24) & 0xFF);
			this._buffer[4] = (byte)((value >> 32) & 0xFF);
			this._buffer[5] = (byte)((value >> 40) & 0xFF);
			this._buffer[6] = (byte)((value >> 48) & 0xFF);
			this._buffer[7] = (byte)((value >> 56) & 0xFF);
			this._stream.Write(this._buffer, 0, 8);
		}

		public void WriteArray(byte[] value)
		{
			base.CheckDisposed();

			if (value == null)
			{
				throw new ArgumentNullException("value");
			}

			this._stream.Write(value, 0, value.Length);
		}

		#endregion

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				this._stream = null;
			}

			base.Dispose(disposing);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
