﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.IO Library                               */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.IO;

namespace Nabu.IO
{
	[CLSCompliant(false)]
	public sealed class ZipArchiveReader
	{
		#region Constants

		private const string HeaderExceptionMessage = "Unknown header type.";

		#endregion

		#region Methods

		public ZipArchiveReader()
		{
		}

		public void Read(Stream stream)
		{
			using (StreamHelper streamHelper = new StreamHelper(stream))
			{
				for (; stream.Position < stream.Length; )
				{
					uint signature = streamHelper.ReadU4();

					switch (signature)
					{
						case ZipBlockSignature.LocalFileHeader:
							{
								ZipArchiveReaderFileLocalHeaderEventArgs block = new ZipArchiveReaderFileLocalHeaderEventArgs(stream);

								long nextPosition = stream.Position + block.CompressedSize;

								if (this.FileLocalHeader != null)
								{
									this.FileLocalHeader(this, block);
								}

								stream.Position = nextPosition;
							}
							break;
						case ZipBlockSignature.CentralFileHeader:
							{
								ZipArchiveReaderFileCentralHeaderEventArgs block = new ZipArchiveReaderFileCentralHeaderEventArgs(stream);

								if (this.FileCentralHeader != null)
								{
									this.FileCentralHeader(this, block);
								}
							}
							break;
						case ZipBlockSignature.CentralEnd:
							{
								ZipArchiveFooterEventArgs block = new ZipArchiveFooterEventArgs(stream);

								if (this.ArchiveFooter != null)
								{
									this.ArchiveFooter(this, block);
								}
							}
							break;
						default:
							throw new InvalidStreamException(HeaderExceptionMessage);
					}
				}
			}
		}

		#endregion

		#region Events

		public event EventHandler<ZipArchiveReaderFileLocalHeaderEventArgs> FileLocalHeader;
		public event EventHandler<ZipArchiveReaderFileCentralHeaderEventArgs> FileCentralHeader;
		public event EventHandler<ZipArchiveFooterEventArgs> ArchiveFooter;

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
