﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Logging Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;

namespace Nabu.Logging.LogWriters
{
	public sealed class MicrosoftSqlServerDatabaseLogWriter :
		DatabaseLogWriterBase, IDisposable
	{
		#region MicrosoftSqlServerDatabaseLogWriter

		#region Constants

		private const string TemplateConnectionStringSQL = "Data Source=%server%;Initial Catalog=%database%;User ID=%username%;Password=%password%";
		private const string TemplateConnectionStringNT = "Data Source=%server%;Initial Catalog=%database%;Integrated Security=SSPI;";

		private static string[] TemplateCommandListDefault =
		{
			"%sp_log_entry%",

			"%sp_log_data%",

			"%sp_log_stack%",

			"CREATE TABLE [%nabu_log_entry%] ( " +
			"[%entry_entry_id%] UNIQUEIDENTIFIER NOT NULL PRIMARY KEY, " +
			"[%entry_level%] VARCHAR(15) NOT NULL, " +
			"[%entry_source%] VARCHAR(15) NOT NULL, " +
			"[%entry_datetime%] DATETIME NOT NULL, " +
			"[%entry_thread_id%] INTEGER NOT NULL)",

			"CREATE TABLE [%nabu_log_data%] ( " +
			"[%data_data_id%] UNIQUEIDENTIFIER NOT NULL PRIMARY KEY, " +
			"[%data_entry_id%] UNIQUEIDENTIFIER NOT NULL FOREIGN KEY REFERENCES [%nabu_log_entry%]([%entry_entry_id%]), " +
			"[%data_datum%] TEXT NOT NULL)",

			"CREATE TABLE [%nabu_log_stack%] ( " +
			"[%stack_stack_id%] UNIQUEIDENTIFIER NOT NULL PRIMARY KEY, " +
			"[%stack_entry_id%] UNIQUEIDENTIFIER NOT NULL FOREIGN KEY REFERENCES [%nabu_log_entry%]([%entry_entry_id%]), " +
			"[%stack_assembly_name%] NVARCHAR(255) NOT NULL, " +
			"[%stack_method_name%] NVARCHAR(255) NOT NULL, " +
			"[%stack_file_name%] NVARCHAR(255) NOT NULL, " +
			"[%stack_line_number%] INTEGER NOT NULL, " +
			"[%stack_column_number%] INTEGER NOT NULL)",

			"CREATE PROC [%sp_log_entry%] " +
			"@entry_entry_id UNIQUEIDENTIFIER, " +
			"@entry_level VARCHAR(15), " +
			"@entry_source VARCHAR(15), " +
			"@entry_date_time DATETIME, " +
			"@entry_thread_id INT " +
			"AS " +
			"INSERT INTO [%nabu_log_entry%] " +
			"([%entry_entry_id%], [%entry_level%], [%entry_source%], [%entry_datetime%], [%entry_thread_id%]) " +
			"VALUES (@entry_entry_id, @entry_level, @entry_source, @entry_date_time, @entry_thread_id)",

			"CREATE PROC [%sp_log_data%] " +
			"@data_data_id UNIQUEIDENTIFIER, " +
			"@data_entry_id UNIQUEIDENTIFIER, " +
			"@data_datum NTEXT " +
			"AS " +
			"INSERT INTO [%nabu_log_data%] " +
			"([%data_data_id%], [%data_entry_id%], [%data_datum%]) " +
			"VALUES (@data_data_id, @data_entry_id, @data_datum)",

			"CREATE PROC [%sp_log_stack%] " +
			"@stack_stack_id UNIQUEIDENTIFIER, " +
			"@stack_entry_id UNIQUEIDENTIFIER, " +
			"@stack_assembly_name VARCHAR(255), " +
			"@stack_method_name VARCHAR(255), " +
			"@stack_file_name VARCHAR(255), " +
			"@stack_line_number INTEGER, " +
			"@stack_column_number INTEGER " +
			"AS " +
			"INSERT INTO [%nabu_log_stack%] " +
			"([%stack_stack_id%], [%stack_entry_id%], [%stack_assembly_name%], [%stack_method_name%], [%stack_file_name%], [%stack_line_number%], [%stack_column_number%]) " +
			"VALUES (@stack_stack_id, @stack_entry_id, @stack_assembly_name, @stack_method_name, @stack_file_name, @stack_line_number, @stack_column_number)"
		};

		#endregion

		#region Fields

		private SqlConnection _connection;
		private SqlCommand _commandEntry;
		private SqlCommand _commandData;
		private SqlCommand _commandStack;
		private string _connectionString;

		#endregion

		#region Methods

		public MicrosoftSqlServerDatabaseLogWriter()
			: base(TemplateCommandListDefault)
		{
			this._connectionString = string.Empty;
		}

		public MicrosoftSqlServerDatabaseLogWriter(
			string connectionString)
			: base(TemplateCommandListDefault)
		{
			if (connectionString == null)
			{
				throw new ArgumentNullException("connectionString");
			}

			this._connectionString = connectionString;
		}

		public MicrosoftSqlServerDatabaseLogWriter(
			string server,
			string database,
			string username,
			string password)
			: base(TemplateCommandListDefault)
		{
			if (server == null)
			{
				throw new ArgumentNullException("server");
			}

			if (database == null)
			{
				throw new ArgumentNullException("database");
			}

			if (username == null)
			{
				throw new ArgumentNullException("username");
			}

			if (password == null)
			{
				throw new ArgumentNullException("password");
			}

			if ((username == null) || (password == null))
			{
				this._connectionString = TemplateConnectionStringNT
					.Replace("%server%", server)
					.Replace("%database%", database);
			}
			else
			{
				this._connectionString = TemplateConnectionStringSQL
					.Replace("%server%", server)
					.Replace("%database%", database)
					.Replace("%username%", username)
					.Replace("%password%", password);
			}
		}

		public MicrosoftSqlServerDatabaseLogWriter(
			string connectionString,
			string[] templateObjectList)
			: base(TemplateCommandListDefault, templateObjectList)
		{
			if (connectionString == null)
			{
				throw new ArgumentNullException("connectionString");
			}

			this._connectionString = connectionString;
		}

		public MicrosoftSqlServerDatabaseLogWriter(
			string server,
			string database,
			string username,
			string password,
			string[] templateObjectList)
			: base(TemplateCommandListDefault, templateObjectList)
		{
			if (server == null)
			{
				throw new ArgumentNullException("server");
			}

			if (database == null)
			{
				throw new ArgumentNullException("database");
			}

			if (username == null)
			{
				throw new ArgumentNullException("username");
			}

			if (password == null)
			{
				throw new ArgumentNullException("password");
			}

			if ((username == null) || (password == null))
			{
				this._connectionString = TemplateConnectionStringNT
					.Replace("%server%", server)
					.Replace("%database%", database);
			}
			else
			{
				this._connectionString = TemplateConnectionStringSQL
					.Replace("%server%", server)
					.Replace("%database%", database)
					.Replace("%username%", username)
					.Replace("%password%", password);
			}
		}

		public MicrosoftSqlServerDatabaseLogWriter(
			string connectionString,
			string[] templateObjectList,
			string templateLine,
			string templateIndent)
			: base(templateLine, templateIndent, TemplateCommandListDefault, templateObjectList)
		{
			if (connectionString == null)
			{
				throw new ArgumentNullException("connectionString");
			}

			this._connectionString = connectionString;
		}

		public MicrosoftSqlServerDatabaseLogWriter(
			string server,
			string database,
			string username,
			string password,
			string[] templateObjectList,
			string templateLine,
			string templateIndent)
			: base(templateLine, templateIndent, TemplateCommandListDefault, templateObjectList)
		{
			if (server == null)
			{
				throw new ArgumentNullException("server");
			}

			if (database == null)
			{
				throw new ArgumentNullException("database");
			}

			if (username == null)
			{
				throw new ArgumentNullException("username");
			}

			if (password == null)
			{
				throw new ArgumentNullException("password");
			}

			if ((username == null) || (password == null))
			{
				this._connectionString = TemplateConnectionStringNT
					.Replace("%server%", server)
					.Replace("%database%", database);
			}
			else
			{
				this._connectionString = TemplateConnectionStringSQL
					.Replace("%server%", server)
					.Replace("%database%", database)
					.Replace("%username%", username)
					.Replace("%password%", password);
			}
		}

		~MicrosoftSqlServerDatabaseLogWriter()
		{
		}

		private IList<string> GetObjectList(string collectionName, string propertyName)
		{
			DataTable schema = this._connection.GetSchema(collectionName);

			int indexColumnProperty = 0;

			foreach (DataColumn column in schema.Columns)
			{
				if (column.ColumnName == propertyName)
				{
					indexColumnProperty = column.Ordinal;
				}
			}

			IList<string> objects = new List<string>(schema.Rows.Count);

			foreach (DataRow row in schema.Rows)
			{
				objects.Add(row[indexColumnProperty] as string);
			}

			return objects;
		}

		private void CreateObject(IList<string> objectList, string objectName, string objectCreateCommand)
		{
			if (!objectList.Contains(objectName))
			{
				using (SqlCommand command = new SqlCommand(objectCreateCommand, this._connection))
				{
					command.ExecuteNonQuery();
				}
			}
		}

		#endregion

		#region Properties

		public string ConnectionString
		{
			get { return this._connectionString; }
			set { this._connectionString = value; }
		}

		#endregion

		#endregion

		#region ILogWriter

		public override void Initialize()
		{
			try
			{
				base.UpdateCommandTemplates();

				this._connection = new SqlConnection(this._connectionString);
				this._connection.Open();

				IList<string> tableList = GetObjectList("Tables", "TABLE_NAME");

				CreateObject(tableList, this.ObjectTemplates[ObjectIndex.TableEntry], this.CommandTemplates[CommandIndex.CreateTableEntry]);
				CreateObject(tableList, this.ObjectTemplates[ObjectIndex.TableData], this.CommandTemplates[CommandIndex.CreateTableData]);
				CreateObject(tableList, this.ObjectTemplates[ObjectIndex.TableStack], this.CommandTemplates[CommandIndex.CreateTableStack]);

				IList<string> storedProcedureList = GetObjectList("Procedures", "ROUTINE_NAME");

				CreateObject(storedProcedureList, this.ObjectTemplates[ObjectIndex.StoredProcedureEntry], this.CommandTemplates[CommandIndex.CreateStoredProcInsertIntoEntry]);
				CreateObject(storedProcedureList, this.ObjectTemplates[ObjectIndex.StoredProcedureData], this.CommandTemplates[CommandIndex.CreateStoredProcInsertIntoData]);
				CreateObject(storedProcedureList, this.ObjectTemplates[ObjectIndex.StoredProcedureStack], this.CommandTemplates[CommandIndex.CreateStoredProcInsertIntoStack]);

				this._commandEntry = this._connection.CreateCommand();
				this._commandEntry.CommandText = this.CommandTemplates[CommandIndex.InsertIntoEntry];
				this._commandEntry.CommandType = CommandType.StoredProcedure;
				this._commandEntry.Parameters.Add("@entry_entry_id", SqlDbType.UniqueIdentifier);
				this._commandEntry.Parameters.Add("@entry_level", SqlDbType.VarChar);
				this._commandEntry.Parameters.Add("@entry_source", SqlDbType.VarChar);
				this._commandEntry.Parameters.Add("@entry_date_time", SqlDbType.DateTime);
				this._commandEntry.Parameters.Add("@entry_thread_id", SqlDbType.Int);
				this._commandEntry.Prepare();

				this._commandData = this._connection.CreateCommand();
				this._commandData.CommandText = this.CommandTemplates[CommandIndex.InsertIntoData];
				this._commandData.CommandType = CommandType.StoredProcedure;
				this._commandData.Parameters.Add("@data_data_id", SqlDbType.UniqueIdentifier);
				this._commandData.Parameters.Add("@data_entry_id", SqlDbType.UniqueIdentifier);
				this._commandData.Parameters.Add("@data_datum", SqlDbType.NText);
				this._commandData.Prepare();

				this._commandStack = this._connection.CreateCommand();
				this._commandStack.CommandText = this.CommandTemplates[CommandIndex.InsertIntoStack];
				this._commandStack.CommandType = CommandType.StoredProcedure;
				this._commandStack.Parameters.Add("@stack_stack_id", SqlDbType.UniqueIdentifier);
				this._commandStack.Parameters.Add("@stack_entry_id", SqlDbType.UniqueIdentifier);
				this._commandStack.Parameters.Add("@stack_assembly_name", SqlDbType.NVarChar);
				this._commandStack.Parameters.Add("@stack_method_name", SqlDbType.NVarChar);
				this._commandStack.Parameters.Add("@stack_file_name", SqlDbType.NVarChar);
				this._commandStack.Parameters.Add("@stack_line_number", SqlDbType.Int);
				this._commandStack.Parameters.Add("@stack_column_number", SqlDbType.Int);
				this._commandStack.Prepare();
			}
			catch (Exception exception)
			{
#if TRACE
				LogWriterInternal.LogException(exception);
#endif
				throw;
			}
		}

		public override void Terminate()
		{
			try
			{
				if (this._commandEntry != null)
				{
					this._commandEntry.Dispose();
					this._commandEntry = null;
				}

				if (this._commandData != null)
				{
					this._commandData.Dispose();
					this._commandData = null;
				}

				if (this._commandStack != null)
				{
					this._commandStack.Dispose();
					this._commandStack = null;
				}

				if (this._connection != null)
				{
					this._connection.Dispose();
					this._connection = null;
				}
			}
			catch (Exception exception)
			{
#if TRACE
				LogWriterInternal.LogException(exception);
#endif
				throw;
			}
		}

		public override void Log(LogLevel level, LogSource source, DateTime dateTime, int threadID, object[] data, IList<string[]> stackFrameList)
		{
			if (level == null)
			{
				throw new ArgumentNullException("level");
			}

			if (source == null)
			{
				throw new ArgumentNullException("source");
			}

			if (data == null)
			{
				throw new ArgumentNullException("data");
			}

			if (stackFrameList == null)
			{
				throw new ArgumentNullException("stackFrameList");
			}

			try
			{
				Guid entryID = Guid.NewGuid();

				using (SqlTransaction transaction = this._connection.BeginTransaction(IsolationLevel.ReadUncommitted))
				{
					this._commandEntry.Parameters["@entry_entry_id"].Value = entryID;
					this._commandEntry.Parameters["@entry_level"].Value = level.ToString();
					this._commandEntry.Parameters["@entry_source"].Value = source.ToString();
					this._commandEntry.Parameters["@entry_date_time"].Value = dateTime;
					this._commandEntry.Parameters["@entry_thread_id"].Value = threadID;
					this._commandEntry.Transaction = transaction;
					this._commandEntry.ExecuteNonQuery();

					foreach (object datum in data)
					{
						this._commandData.Parameters["@data_data_id"].Value = Guid.NewGuid();
						this._commandData.Parameters["@data_entry_id"].Value = entryID;
						this._commandData.Parameters["@data_datum"].Value = GetDatumString(datum);
						this._commandData.Transaction = transaction;
						this._commandData.ExecuteNonQuery();
					}

					foreach (string[] stackFrame in stackFrameList)
					{
						this._commandStack.Parameters["@stack_stack_id"].Value = Guid.NewGuid();
						this._commandStack.Parameters["@stack_entry_id"].Value = entryID;
						this._commandStack.Parameters["@stack_assembly_name"].Value = stackFrame[LogStackFrameElement.AssemblyName];
						this._commandStack.Parameters["@stack_method_name"].Value = stackFrame[LogStackFrameElement.MethodName];
						this._commandStack.Parameters["@stack_file_name"].Value = stackFrame[LogStackFrameElement.FileName];
						this._commandStack.Parameters["@stack_line_number"].Value = stackFrame[LogStackFrameElement.LineNumber];
						this._commandStack.Parameters["@stack_column_number"].Value = stackFrame[LogStackFrameElement.ColumnNumber];
						this._commandStack.Transaction = transaction;
						this._commandStack.ExecuteNonQuery();
					}

					transaction.Commit();
				}
			}
			catch (Exception exception)
			{
#if TRACE
				LogWriterInternal.LogException(exception);
#endif
				throw;
			}
		}

		#endregion

		#region IDisposable

		public void Dispose()
		{
			Terminate();
			GC.SuppressFinalize(this);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
