﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Logging Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;

namespace Nabu.Logging.LogWriters
{
	public class ConsoleLogWriter : TextLogWriterBase
	{
		#region ConsoleLogWriter

		#region Constants

		protected new const string TemplateEntryDefault =
			"\b0\f7Level: \fB%level%\f7; Source: \fA%source%\f7; Date/Time: \fE%date_time%\f7; ThreadID: \fC%thread_id%\f7;\n" +
			"Data:\n" +
			"%data%" +
			"Stack Trace:\n" +
			"%stack%";
		protected new const string TemplateDatumDefault =
			"\fE%datum%\f7\n";
		protected new const string TemplateStackDefault =
			"\t\fF%method_name%\f7\n";

		#endregion

		#region Methods

		private static ConsoleColor ColorFromCode(char code)
		{
			switch (code)
			{
				case '0':
					return ConsoleColor.Black;
				case '1':
					return ConsoleColor.DarkBlue;
				case '2':
					return ConsoleColor.DarkGreen;
				case '3':
					return ConsoleColor.DarkCyan;
				case '4':
					return ConsoleColor.DarkRed;
				case '5':
					return ConsoleColor.DarkMagenta;
				case '6':
					return ConsoleColor.DarkYellow;
				case '7':
					return ConsoleColor.Gray;
				case '8':
					return ConsoleColor.DarkGray;
				case '9':
					return ConsoleColor.Blue;
				case 'a':
				case 'A':
					return ConsoleColor.Green;
				case 'b':
				case 'B':
					return ConsoleColor.Cyan;
				case 'c':
				case 'C':
					return ConsoleColor.Red;
				case 'd':
				case 'D':
					return ConsoleColor.Magenta;
				case 'e':
				case 'E':
					return ConsoleColor.Yellow;
				case 'f':
				case 'F':
					return ConsoleColor.White;
				default:
					return ConsoleColor.Black;
			}
		}

		public ConsoleLogWriter()
			: base(TemplateLineDefault, TemplateIndentDefault, TemplateEntryDefault, TemplateDatumDefault, TemplateStackDefault)
		{
		}

		public ConsoleLogWriter(
			string templateEntry,
			string templateData,
			string templateStack)
			: base(TemplateLineDefault, TemplateIndentDefault, templateEntry, templateData, templateStack)
		{
		}

		#endregion

		#endregion

		#region ILogWriter

		public override void Initialize()
		{
			try
			{
				Console.SetWindowSize(128, 32);
				Console.SetBufferSize(128, 32767);
			}
			catch (Exception exception)
			{
#if TRACE
				LogWriterInternal.LogException(exception);
#endif
				throw;
			}
		}

		public override void Terminate()
		{
			try
			{
			}
			catch (Exception exception)
			{
#if TRACE
				LogWriterInternal.LogException(exception);
#endif
				throw;
			}
		}

		public override void Log(LogLevel level, LogSource source, DateTime dateTime, int threadID, object[] data, IList<string[]> stackFrameList)
		{
			if (level == null)
			{
				throw new ArgumentNullException("level");
			}

			if (source == null)
			{
				throw new ArgumentNullException("source");
			}

			if (data == null)
			{
				throw new ArgumentNullException("data");
			}

			if (stackFrameList == null)
			{
				throw new ArgumentNullException("stackFrameList");
			}

			try
			{
				string logString = base.GetLogString(level, source, dateTime, threadID, data, stackFrameList);

				char[] markerList = new char[] { '\b', '\f' };
				int indexFrom = logString.IndexOfAny(markerList);

				while (indexFrom >= 0)
				{
					int indexToReal = logString.IndexOfAny(markerList, indexFrom + 1);
					int indexTo = (indexToReal > 0) ? indexToReal : logString.Length;

					if (indexTo - indexFrom >= 2)
					{
						switch (logString[indexFrom])
						{
							case '\b': Console.BackgroundColor = ColorFromCode(logString[indexFrom + 1]); break;
							case '\f': Console.ForegroundColor = ColorFromCode(logString[indexFrom + 1]); break;
						}

						Console.Write(logString.Substring(indexFrom + 2, indexTo - indexFrom - 2));
					}
					else
					{
						Console.Write(logString.Substring(indexFrom, indexTo - indexFrom));
					}

					indexFrom = indexToReal;
				}

				Console.WriteLine();
			}
			catch (Exception exception)
			{
#if TRACE
				LogWriterInternal.LogException(exception);
#endif
				throw;
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
