﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Logging Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Diagnostics;


namespace Nabu.Logging.LogWriters
{
	public class EventLogLogWriter : TextLogWriterBase
	{
		#region EventLogLogWriter

		#region Constants

		private const string TemplateLogName = "Application";

		#endregion

		#region Fields

		string _logName;

		#endregion

		#region Methods

		public EventLogLogWriter()
		{
			this._logName = TemplateLogName;
		}

		public EventLogLogWriter(
			string logName)
		{
			if (logName == null)
			{
				throw new ArgumentNullException("logName");
			}

			this._logName = logName;
		}

		#endregion

		#region Properties

		public string LogName
		{
			get { return this._logName; }
		}

		#endregion

		#endregion

		#region ILogWriter

		public override void Initialize()
		{
			try
			{
			}
			catch (Exception exception)
			{
#if TRACE
				LogWriterInternal.LogException(exception);
#endif
				throw;
			}
		}

		public override void Terminate()
		{
			try
			{
			}
			catch (Exception exception)
			{
#if TRACE
				LogWriterInternal.LogException(exception);
#endif
				throw;
			}
		}

		public override void Log(LogLevel level, LogSource source, DateTime dateTime, int threadID, object[] data, IList<string[]> stackFrameList)
		{
			if (level == null)
			{
				throw new ArgumentNullException("level");
			}

			if (source == null)
			{
				throw new ArgumentNullException("source");
			}

			if (data == null)
			{
				throw new ArgumentNullException("data");
			}

			if (stackFrameList == null)
			{
				throw new ArgumentNullException("stackFrameList");
			}

			try
			{
				string sourceName = source.ToString();

				if (!EventLog.SourceExists(sourceName))
				{
					EventLog.CreateEventSource(sourceName, this._logName);
				}

				string levelName = level.ToString();
				EventLogEntryType entryType = EventLogEntryType.Information;

				if (levelName.IndexOf("warning", StringComparison.InvariantCultureIgnoreCase) != -1)
				{
					entryType = EventLogEntryType.Warning;
				}
				else if (levelName.IndexOf("error", StringComparison.InvariantCultureIgnoreCase) != -1)
				{
					entryType = EventLogEntryType.Error;
				}

				int levelHash = levelName.GetHashCode();
				int sourceHash = sourceName.GetHashCode();

				EventLog.WriteEntry(
					sourceName,
					base.GetLogString(level, source, dateTime, threadID, data, stackFrameList),
					entryType,
					(levelHash ^ (sourceHash >> 3)) & 0xFFFF);
			}
			catch (Exception exception)
			{
#if TRACE
				LogWriterInternal.LogException(exception);
#endif
				throw;
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
