﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Logging Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Globalization;
using System.Text.RegularExpressions;

namespace Nabu.Logging.LogWriters
{
	public static class LogFilePathExpander
	{
		#region Fields

		private static Regex _regexDateTimeLocal = new Regex("%ltime-\\[(.+)\\]%");
		private static Regex _regexDateTimeUTC = new Regex("%utime-\\[(.+)\\]%");

		#endregion

		#region Methods

		private static string DateTimeLocalMatchEvaluator(Match match)
		{
			return DateTime.Now.ToString(
				match.Value.Substring(8, match.Value.Length - 10),
				CultureInfo.CurrentCulture.DateTimeFormat);
		}

		private static string DateTimeUtcMatchEvaluator(Match match)
		{
			return DateTime.UtcNow.ToString(
				match.Value.Substring(8, match.Value.Length - 10),
				CultureInfo.CurrentCulture.DateTimeFormat);
		}

		public static string Expand(string path)
		{
			path = Environment.ExpandEnvironmentVariables(path);
			path = path.Replace("%[cucm]%", SpecialPath.CurrentUserCurrentMachineSettingsPath);
			path = path.Replace("%[cuam]%", SpecialPath.CurrentUserAllMachinesSettingsPath);
			path = path.Replace("%[aucm]%", SpecialPath.AllUsersCurrentMachineSettingsPath);
			path = _regexDateTimeLocal.Replace(path, DateTimeLocalMatchEvaluator);
			path = _regexDateTimeUTC.Replace(path, DateTimeUtcMatchEvaluator);

			return path;
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
