﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Logging Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;

namespace Nabu.Logging
{
	/// <summary>
	/// Writer of logs.
	/// </summary>
	public interface ILogWriter
	{
		/// <summary>
		/// Initialize log writing. Must not be called directly. Is called only once.
		/// </summary>
		void Initialize();
		/// <summary>
		/// Terminate log writing. Must not be called directly. Is called only once.
		/// </summary>
		void Terminate();
		/// <summary>
		/// Write log entry. Must not be called directly. Is called after Initialize() and before Terminate().
		/// </summary>
		/// <param name="level">Level of the log entry.</param>
		/// <param name="source">Source of the log entry.</param>
		/// <param name="dateTime">Date and time of entry creation.</param>
		/// <param name="threadID">ThreadID of thread created entry.</param>
		/// <param name="data">Objects associated with entry.</param>
		/// <param name="stackFrameList">Stack trace at the moment of entry creation.</param>
		void Log(LogLevel level, LogSource source, DateTime dateTime, int threadID, object[] data, IList<string[]> stackFrameList);
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
