﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Logging Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Reflection;

namespace Nabu.Logging
{
	/// <summary>
	/// Attribute which assigns <see cref="ILogFormatter"/> implementation to some type.
	/// </summary>
	[AttributeUsage(AttributeTargets.Class | AttributeTargets.Struct, AllowMultiple = false, Inherited = true)]
	public sealed class LogFormatterAttribute : Attribute
	{
		#region Fields

		private Type _typeLogFormatter;

		#endregion

		#region Methods

#if DEBUG
		public bool TypeFilterMatch(Type m, object filterCriteria)
		{
			return m == filterCriteria as Type;
		}

		/// <summary>
		/// Constructs LogFormatterAttribute.
		/// </summary>
		/// <param name="typeLogFormatter">Type of <see cref="ILogFormatter"/> implementation.</param>
		public LogFormatterAttribute(Type typeLogFormatter)
		{
			if (typeLogFormatter.FindInterfaces(new TypeFilter(TypeFilterMatch), typeof(ILogFormatter)).Length == 0)
			{
				throw new ArgumentException("typeLogFormatter must implement ILogFormatter");
			}

			this._typeLogFormatter = typeLogFormatter;
		}
#else
		/// <summary>
		/// Constructs LogFormatterAttribute.
		/// </summary>
		/// <param name="typeLogFormatter">Type of <see cref="ILogFormatter"/> implementation.</param>
		public LogFormatterAttribute(Type typeLogFormatter)
		{
			this._typeLogFormatter = typeLogFormatter;
		}
#endif
		#endregion

		#region Properties

		/// <summary>
		/// Type of <see cref="ILogFormatter"/> implementation.
		/// </summary>
		public Type TypeLogFormatter
		{
			get { return this._typeLogFormatter; }
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
