﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Logging Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Collections.Generic;
using System.Text;

namespace Nabu.Logging
{
	/// <summary>
	/// Indent manager for <see cref="ILogFormatter"/> implementations.
	/// </summary>
	public sealed class LogFormatterIndent
	{
		#region Fields

		private static Dictionary<string, Dictionary<int, string>> _indentMap = new Dictionary<string, Dictionary<int, string>>();

		#endregion

		#region Methods

		private LogFormatterIndent()
		{
		}

		/// <summary>
		/// Get indent for given indent character sequence and number of repetitions.
		/// </summary>
		/// <param name="indentString">Indent character sequence.</param>
		/// <param name="indentValue">Number of repetitions.</param>
		/// <returns></returns>
		public static string GetIndent(string indentString, int indentValue)
		{
			Dictionary<int, string> indentValueMap;

			if (!_indentMap.TryGetValue(indentString, out indentValueMap))
			{
				_indentMap[indentString] = indentValueMap = new Dictionary<int, string>();
			}

			string indent;

			if (!indentValueMap.TryGetValue(indentValue, out indent))
			{
				StringBuilder indentBuilder = new StringBuilder();

				for (int index = 0; index < indentValue; index++)
				{
					indentBuilder.Append(indentString);
				}

				indentValueMap[indentValue] = indent = indentBuilder.ToString();
			}

			return indent;
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
