﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Logging Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/

namespace Nabu.Logging
{
	/// <summary>
	/// Log level.
	/// </summary>
	public class LogLevel
	{
		#region Fields

		public static readonly LogLevel Unknown = new LogLevel("Unknown");
		public static readonly LogLevel Sink = new LogLevel("Sink");
		public static readonly LogLevel Debug = new LogLevel("Debug");
		public static readonly LogLevel Info = new LogLevel("Info");
		public static readonly LogLevel Warning = new LogLevel("Warning");
		public static readonly LogLevel Error = new LogLevel("Error");

		public static readonly LogLevel[] SinkList = new LogLevel[] { Sink };
		public static readonly LogLevel[] DefaultList = new LogLevel[]
		{
			Unknown,
			Sink,
			Debug,
			Info,
			Warning,
			Error
		};

		private string _name;

		#endregion

		#region Methods

		public LogLevel(string name)
		{
			this._name = name;
		}

		public override bool Equals(object obj)
		{
			LogLevel logLevel = obj as LogLevel;

			if ((object)logLevel == null)
			{
				return false;
			}

			return this._name == logLevel._name;
		}

		public override int GetHashCode()
		{
			return this._name.GetHashCode();
		}

		public override string ToString()
		{
			return this._name;
		}

		public static bool operator ==(LogLevel left, LogLevel right)
		{
			if ((object)left == null)
			{
				if ((object)right == null)
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				if ((object)right == null)
				{
					return false;
				}
				else
				{
					return left._name == right._name;
				}
			}
		}

		public static bool operator !=(LogLevel left, LogLevel right)
		{
			if ((object)left == null)
			{
				if ((object)right == null)
				{
					return false;
				}
				else
				{
					return true;
				}
			}
			else
			{
				if ((object)right == null)
				{
					return true;
				}
				else
				{
					return left._name != right._name;
				}
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
