﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Logging Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/

namespace Nabu.Logging
{
	/// <summary>
	/// Log source.
	/// </summary>
	public class LogSource
	{
		#region Fields

		public static readonly LogSource Unknown = new LogSource("Unknown");

		public static readonly LogSource Sink = new LogSource("Sink");
		public static readonly LogSource BuisenessLogic = new LogSource("BL");

		public static readonly LogSource Database = new LogSource("DB");
		public static readonly LogSource MicrosoftAccessDB = new LogSource("DB.MySQL");
		public static readonly LogSource MicrosoftSQLServerDB = new LogSource("DB.MSSQL");
		public static readonly LogSource OracleDB = new LogSource("DB.Oracle");

		public static readonly LogSource UserInterface = new LogSource("UI");

		public static readonly LogSource FileIO = new LogSource("IO.File");
		public static readonly LogSource BinaryFileIO = new LogSource("IO.File.Binary");
		public static readonly LogSource TextFileIO = new LogSource("IO.File.Text");
		public static readonly LogSource XmlFileIO = new LogSource("IO.File.Xml");

		public static readonly LogSource SocketIO = new LogSource("IO.Socket");
		public static readonly LogSource TcpSocketIO = new LogSource("IO.Socket.Tcp");
		public static readonly LogSource UdpSocketIO = new LogSource("IO.Socket.Udp");

		public static readonly LogSource[] SinkList = new LogSource[] { Sink };
		public static readonly LogSource[] DefaultList = new LogSource[]
		{
			Unknown,
			Sink,
			BuisenessLogic,
			Database,
			UserInterface,
			FileIO,
			SocketIO
		};

		private string _name;

		#endregion

		#region Methods

		public LogSource(string name)
		{
			this._name = name;
		}

		public override bool Equals(object obj)
		{
			LogSource logSource = obj as LogSource;

			if ((object)logSource == null)
			{
				return false;
			}

			return this._name == logSource._name;
		}

		public override int GetHashCode()
		{
			return this._name.GetHashCode();
		}

		public override string ToString()
		{
			return this._name;
		}

		public static bool operator ==(LogSource left, LogSource right)
		{
			if ((object)left == null)
			{
				if ((object)right == null)
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				if ((object)right == null)
				{
					return false;
				}
				else
				{
					return left._name == right._name;
				}
			}
		}

		public static bool operator !=(LogSource left, LogSource right)
		{
			if ((object)left == null)
			{
				if ((object)right == null)
				{
					return false;
				}
				else
				{
					return true;
				}
			}
			else
			{
				if ((object)right == null)
				{
					return true;
				}
				else
				{
					return left._name != right._name;
				}
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
