﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Logging Library                          */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Configuration;
using System.Xml;
using System.Xml.Serialization;

namespace Nabu.Logging
{
	/// <summary>
	/// Provides configuration load from App.config file.
	/// </summary>
	public class LogXmlSection : IConfigurationSectionHandler
	{
		#region LogAppConfig

		#region Types

		public class LogParameterData
		{
			#region Fields

			private string _type = string.Empty;
			private string _name = string.Empty;
			private string _value = string.Empty;

			#endregion

			#region Properties

			[XmlAttribute("log_type")]
			public string Type
			{
				get { return this._type; }
				set { this._type = value; }
			}

			[XmlAttribute("log_name")]
			public string Name
			{
				get { return this._name; }
				set { this._name = value; }
			}

			[XmlAttribute("log_value")]
			public string Value
			{
				get { return this._value; }
				set { this._value = value; }
			}

			#endregion
		}

		public class LogWriterData
		{
			#region Fields

			private string _type;
			private LogParameterData[] _logParameters;

			#endregion

			#region Properties

			[XmlAttribute("log_type")]
			public string Type
			{
				get { return this._type; }
				set { this._type = value; }
			}

			[XmlArray("log_parameter_list")]
			[XmlArrayItem("log_parameter")]
			public LogParameterData[] Parameters
			{
				get { return this._logParameters; }
				set { this._logParameters = value; }
			}

			#endregion
		}

		public class LogGroupData
		{
			#region Fields

			private LogLevel[] _logLevels = new LogLevel[0];
			private LogSource[] _logSources = new LogSource[0];
			private LogWriterData[] _logWriters = new LogWriterData[0];

			#endregion

			#region Properties

			[XmlArray("log_level_list")]
			[XmlArrayItem("log_level")]
			public LogLevel[] Levels
			{
				get { return this._logLevels; }
				set { this._logLevels = value; }
			}

			[XmlArray("log_source_list")]
			[XmlArrayItem("log_source")]
			public LogSource[] Sources
			{
				get { return this._logSources; }
				set { this._logSources = value; }
			}

			[XmlArray("log_writer_list")]
			[XmlArrayItem("log_writer")]
			public LogWriterData[] Writers
			{
				get { return this._logWriters; }
				set { this._logWriters = value; }
			}

			#endregion
		}

		[XmlRoot("log_nabu")]
		public class LogSectionData
		{
			#region Fields

			private LogGroupData[] _logGroups = new LogGroupData[0];

			#endregion

			#region Properties

			[XmlArray("log_group_list")]
			[XmlArrayItem("log_group")]
			public LogGroupData[] Groups
			{
				get { return this._logGroups; }
				set { this._logGroups = value; }
			}

			#endregion
		}

		#endregion

		#region Methods

		public static LogSectionData ReadSection(XmlReader xmlReader)
		{
			try
			{
				return (LogSectionData)(new XmlSerializer(typeof(LogSectionData))).Deserialize(xmlReader);
			}
			catch (Exception exception)
			{
#if TRACE
				LogWriterInternal.LogException(exception);
#endif
				throw;
			}
		}

		#endregion

		#endregion

		#region IConfigurationSectionHandler

		public object Create(object parent, object configContext, XmlNode section)
		{
			return ReadSection(new XmlNodeReader(section));
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
