﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Math Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;

namespace Nabu.Math
{
	public sealed class NumerationConverter
	{
		#region Fields

		private char[] _alphabet;
		private Dictionary<char, byte> _map;

		#endregion

		#region Methods

		#region Static

		private static void Divide(
			byte[] outputValue,
			out byte remainder,
			byte[] inputValue,
			int scale,
			int divider)
		{
			remainder = 0;

			for (int index = 0; index < inputValue.Length; index++)
			{
				int accumulator = scale * remainder + inputValue[index];

				outputValue[index] = (byte)(accumulator / divider);
				remainder = (byte)(accumulator % divider);
			}
		}

		private static void Convert(
			out byte[] outputValue,
			int outputScale,
			byte[] inputValue,
			int inputScale)
		{
			outputValue = new byte[(int)System.Math.Ceiling(inputValue.Length * (System.Math.Log(inputScale) / System.Math.Log(outputScale)))];

			int outputIndex = outputValue.Length;
			double power = System.Math.Pow(inputScale, inputValue.Length);

			byte[][] data = new byte[2][];

			data[0] = new byte[inputValue.Length];
			data[1] = new byte[inputValue.Length];

			Array.Copy(inputValue, data[0], inputValue.Length);

			for (int index = 0; power > 1.0; index++)
			{
				byte[] input = data[(index + 0) % 2];
				byte[] output = data[(index + 1) % 2];
				byte remainder;

				Divide(
					output,
					out remainder,
					input,
					inputScale,
					outputScale);

				outputIndex--;
				outputValue[outputIndex] = remainder;

				power /= outputScale;
			}
		}

		public static byte[] Convert(
			int outputScale,
			byte[] inputValue,
			int inputScale)
		{
			byte[] outputValue;

			Convert(out outputValue, outputScale, inputValue, inputScale);

			if (outputValue[0] != 0)
			{
				return outputValue;
			}
			else
			{
				for (int index = 1; index < outputValue.Length; index++)
				{
					if (outputValue[index] != 0)
					{
						byte[] result = new byte[outputValue.Length - index];

						Array.Copy(outputValue, index, result, 0, result.Length);

						return result;
					}
				}

				return new byte[1];
			}
		}

		#endregion

		#region Instance

		public NumerationConverter(char[] alphabet)
		{
			if (alphabet == null)
			{
				throw new ArgumentNullException("alphabet");
			}

			if ((alphabet.Length < 2) || (alphabet.Length > 256))
			{
				throw new ArgumentOutOfRangeException("alphabet");
			}

			this._map = new Dictionary<char, byte>(alphabet.Length);

			for (int index = 0; index < alphabet.Length; index++)
			{
				this._map.Add(alphabet[index], (byte)index);
			}

			this._alphabet = alphabet;
		}

		public string FromBinary(byte[] input)
		{
			byte[] outputValue;

			Convert(
				out outputValue,
				(byte)this._alphabet.Length,
				input,
				256);

			int startIndex = 0;

			for (; (startIndex < outputValue.Length - 1) && (outputValue[startIndex] == 0); startIndex++)
			{
			}

			StringBuilder result = new StringBuilder(outputValue.Length - startIndex);

			for (int index = startIndex; index < outputValue.Length; index++)
			{
				result.Append(this._alphabet[outputValue[index]]);
			}

			return result.ToString();
		}

		public byte[] FromString(string input)
		{
			byte[] inputValue = new byte[input.Length];

			for (int index = 0; index < input.Length; index++)
			{
				inputValue[index] = this._map[input[index]];
			}

			return
				Convert(
					256,
					inputValue,
					(byte)this._alphabet.Length);
		}

		#endregion

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
