﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Messaging Library                        */
/*       {_.-``-'         {_/   Copyright © 2005-2008, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;

namespace Nabu.Messaging
{
	public sealed class OutlookAppointment : IComObjectWrapper
	{
		#region OutlookAppointment

		#region Fields

		private IOutlookAppointment _appointment;

		#endregion

		#region Methods

		public OutlookAppointment()
		{
		}

		internal OutlookAppointment(IOutlookAppointment appointment)
		{
			this._appointment = appointment;
		}

		public void Save()
		{
			this._appointment.Save();
		}

		public void Send()
		{
			this._appointment.Send();
		}

		public void Delete()
		{
			this._appointment.Delete();
		}

		public void Cancel()
		{
			this._appointment.Cancel();
		}

		public OutlookAppointment Copy()
		{
			IOutlookAppointment appointment;

			this._appointment.Copy(out appointment);

			return new OutlookAppointment(appointment);
		}

		public void Display()
		{
			this._appointment.Display();
		}

		#endregion

		#region Properties

		#region RecurrencePattern

		public OutlookRecurrencePattern RecurrencePattern
		{
			get
			{
				IOutlookRecurrencePattern value;

				this._appointment.GetRecurrencePattern(out value);

				return new OutlookRecurrencePattern(value);
			}

			set
			{
				if (value == null)
				{
					this._appointment.ClearRecurrencePattern();
				}
			}
		}

		#endregion

		#region IsRecurring

		public bool IsRecurring
		{
			get
			{
				bool value;

				this._appointment.get_IsRecurring(out value);

				return value;
			}
		}

		#endregion

		#region Subject

		public string Subject
		{
			get
			{
				string value;

				this._appointment.get_Subject(out value);

				return value;
			}

			set
			{
				this._appointment.put_Subject(value);
			}
		}

		#endregion

		#region Location

		public string Location
		{
			get
			{
				string value;

				this._appointment.get_Location(out value);

				return value;
			}

			set
			{
				this._appointment.put_Location(value);
			}
		}

		#endregion

		#region Categories

		public string Categories
		{
			get
			{
				string value;

				this._appointment.get_Categories(out value);

				return value;
			}

			set
			{
				this._appointment.put_Categories(value);
			}
		}

		#endregion

		#region Start

		public DateTime Start
		{
			get
			{
				double oleTime;

				this._appointment.get_Start(out oleTime);

				SystemTime systemTime;

				NativeMethods.VariantTimeToSystemTime(oleTime, out systemTime);

				return
					new DateTime(
						systemTime.Year,
						systemTime.Month,
						systemTime.Day,
						systemTime.Hour,
						systemTime.Minute,
						systemTime.Second,
						systemTime.Millisecond);
			}

			set
			{
				SystemTime systemTime = new SystemTime();

				systemTime.Year = (short)value.Year;
				systemTime.Month = (short)value.Month;
				systemTime.Day = (short)value.Day;
				systemTime.Hour = (short)value.Hour;
				systemTime.Minute = (short)value.Minute;
				systemTime.Second = (short)value.Second;
				systemTime.Millisecond = (short)value.Millisecond;

				double oleTime;

				NativeMethods.SystemTimeToVariantTime(ref systemTime, out oleTime);

				this._appointment.put_Start(oleTime);
			}
		}

		#endregion

		#region Duration

		public int Duration
		{
			get
			{
				int value;

				this._appointment.get_Duration(out value);

				return value;
			}

			set
			{
				this._appointment.put_Duration(value);
			}
		}

		#endregion

		#region End

		public DateTime End
		{
			get
			{
				double oleTime;

				this._appointment.get_End(out oleTime);

				SystemTime systemTime;

				NativeMethods.VariantTimeToSystemTime(oleTime, out systemTime);

				return
					new DateTime(
						systemTime.Year,
						systemTime.Month,
						systemTime.Day,
						systemTime.Hour,
						systemTime.Minute,
						systemTime.Second,
						systemTime.Millisecond);
			}

			set
			{
				SystemTime systemTime = new SystemTime();

				systemTime.Year = (short)value.Year;
				systemTime.Month = (short)value.Month;
				systemTime.Day = (short)value.Day;
				systemTime.Hour = (short)value.Hour;
				systemTime.Minute = (short)value.Minute;
				systemTime.Second = (short)value.Second;
				systemTime.Millisecond = (short)value.Millisecond;

				double oleTime;

				NativeMethods.SystemTimeToVariantTime(ref systemTime, out oleTime);

				this._appointment.put_End(oleTime);
			}
		}

		#endregion

		#region IsAllDayEvent

		public bool IsAllDayEvent
		{
			get
			{
				bool value;

				this._appointment.get_AllDayEvent(out value);

				return value;
			}

			set
			{
				this._appointment.put_AllDayEvent(value);
			}
		}

		#endregion

		#region BusyStatus

		public OutlookBusyStatus BusyStatus
		{
			get
			{
				OutlookBusyStatus value;

				this._appointment.get_BusyStatus(out value);

				return value;
			}

			set
			{
				this._appointment.put_BusyStatus(value);
			}
		}

		#endregion

		#region Sensitivity

		public OutlookSensitivity Sensitivity
		{
			get
			{
				OutlookSensitivity value;

				this._appointment.get_Sensitivity(out value);

				return value;
			}

			set
			{
				this._appointment.put_Sensitivity(value);
			}
		}

		#endregion

		#region Body

		public string Body
		{
			get
			{
				string value;

				this._appointment.get_Body(out value);

				return value;
			}

			set
			{
				this._appointment.put_Body(value);
			}
		}

		#endregion

		#region Recipients

		public OutlookRecipientCollection Recipients
		{
			get
			{
				IOutlookRecipients recipients;

				this._appointment.get_Recipients(out recipients);

				return new OutlookRecipientCollection(recipients);
			}
		}

		#endregion

		#region MeetingStatus

		public OutlookMeetingStatus MeetingStatus
		{
			get
			{
				OutlookMeetingStatus value;

				this._appointment.get_MeetingStatus(out value);

				return value;
			}
		}

		#endregion

		#region IsReminderSet

		public bool IsReminderSet
		{
			get
			{
				bool value;

				this._appointment.get_ReminderSet(out value);

				return value;
			}

			set
			{
				this._appointment.put_ReminderSet(value);
			}
		}

		#endregion

		#region ReminderSoundFile

		public string ReminderSoundFile
		{
			get
			{
				string value;

				this._appointment.get_ReminderSoundFile(out value);

				return value;
			}

			set
			{
				this._appointment.put_ReminderSoundFile(value);
			}
		}

		#endregion

		#region ReminderMinutesBeforeStart

		public int ReminderMinutesBeforeStart
		{
			get
			{
				int value;

				this._appointment.get_ReminderMinutesBeforeStart(out value);

				return value;
			}

			set
			{
				this._appointment.put_ReminderMinutesBeforeStart(value);
			}
		}

		#endregion

		#region ReminderOptions

		public OutlookReminderOptions ReminderOptions
		{
			get
			{
				OutlookReminderOptions value;

				this._appointment.get_ReminderOptions(out value);

				return value;
			}

			set
			{
				this._appointment.put_ReminderOptions(value);
			}
		}

		#endregion

		#endregion

		#endregion

		#region IComObjectWrapper

		public object ComObject
		{
			get { return this._appointment; }
			set { this._appointment = (IOutlookAppointment)value; }
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
