﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Messaging Library                        */
/*       {_.-``-'         {_/   Copyright © 2005-2008, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;

namespace Nabu.Messaging
{
	public sealed class OutlookItemCollection<T> : IEnumerable<T> where T : new()
	{
		#region OutlookItemCollection

		#region Fields

		private IOutlookItemCollection _collection;

		#endregion

		#region Methods

		private T CreateItem(object item)
		{
			T value = new T();

			((IComObjectWrapper)value).ComObject = item;

			return value;
		}

		internal OutlookItemCollection(IOutlookItemCollection collection)
		{
			this._collection = collection;
		}

		public T Add()
		{
			object item;

			this._collection.Add(out item);

			return CreateItem(item);
		}

		public void RemoveAt(int index)
		{
			this._collection.Remove(index);
		}

		public T Find(string restriction)
		{
			object item;

			this._collection.Find(restriction, out item);

			return (item == null) ? default(T) : CreateItem(item);
		}

		public T FindNext()
		{
			object item;

			this._collection.FindNext(out item);

			return (item == null) ? default(T) : CreateItem(item);
		}

		public OutlookItemCollection<T> Restrict(string restriction)
		{
			IOutlookItemCollection collection;

			this._collection.Restrict(restriction, out collection);

			return new OutlookItemCollection<T>(collection);
		}

		public void FindNext(string property, bool isDescending)
		{
			this._collection.Sort(property, isDescending);
		}

		#endregion

		#region Properties

		public T this[int index]
		{
			get
			{
				object item;

				this._collection.Item(index, out item);

				return CreateItem(item);
			}
		}

		public int Count
		{
			get
			{
				int value;

				this._collection.get_Count(out value);

				return value;
			}
		}

		public bool AreRecurrencesIncluded
		{
			get
			{
				bool value;

				this._collection.get_IncludeRecurrences(out value);

				return value;
			}

			set
			{
				this._collection.put_IncludeRecurrences(value);
			}
		}

		#endregion

		#endregion

		#region IEnumerable

		System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
		{
			return ((IEnumerable<T>)this).GetEnumerator();
		}

		#endregion

		#region IEnumerable<OutlookRecipient>

		public IEnumerator<T> GetEnumerator()
		{
			IEnumVariant enumVariant;

			this._collection.get__NewEnum(out enumVariant);

			object value;
			int fetched;

			while (
				(enumVariant.Next(1, out value, out fetched) == ComResult.S_OK) &&
				(fetched == 1))
			{
				yield return CreateItem(value);
			}

			yield break;
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
