﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Messaging Library                        */
/*       {_.-``-'         {_/   Copyright © 2005-2008, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;

namespace Nabu.Messaging
{
	public class OutlookRecurrencePattern
	{
		#region Fields

		private IOutlookRecurrencePattern _pattern;

		#endregion

		#region Methods

		internal OutlookRecurrencePattern(IOutlookRecurrencePattern pattern)
		{
			this._pattern = pattern;
		}

		public OutlookAppointment GetOccurrence(DateTime value)
		{
			SystemTime systemTime = new SystemTime();

			systemTime.Year = (short)value.Year;
			systemTime.Month = (short)value.Month;
			systemTime.Day = (short)value.Day;
			systemTime.Hour = (short)value.Hour;
			systemTime.Minute = (short)value.Minute;
			systemTime.Second = (short)value.Second;
			systemTime.Millisecond = (short)value.Millisecond;

			double oleTime;

			NativeMethods.SystemTimeToVariantTime(ref systemTime, out oleTime);

			IOutlookAppointment appointment;

			this._pattern.GetOccurrence(oleTime, out appointment);

			return new OutlookAppointment(appointment);
		}

		#endregion

		#region Properties

		#region RecurrenceType

		public OutlookRecurrenceType RecurrenceType
		{
			get
			{
				OutlookRecurrenceType value;

				this._pattern.get_RecurrenceType(out value);

				return value;
			}

			set
			{
				this._pattern.put_RecurrenceType(value);
			}
		}

		#endregion

		#region PatternStartDate

		public DateTime PatternStartDate
		{
			get
			{
				double oleTime;

				this._pattern.get_PatternStartDate(out oleTime);

				SystemTime systemTime;

				NativeMethods.VariantTimeToSystemTime(oleTime, out systemTime);

				return
					new DateTime(
						systemTime.Year,
						systemTime.Month,
						systemTime.Day);
			}

			set
			{
				SystemTime systemTime = new SystemTime();

				systemTime.Year = (short)value.Year;
				systemTime.Month = (short)value.Month;
				systemTime.Day = (short)value.Day;

				double oleTime;

				NativeMethods.SystemTimeToVariantTime(ref systemTime, out oleTime);

				this._pattern.put_PatternStartDate(oleTime);
			}
		}

		#endregion

		#region StartTime

		public TimeSpan StartTime
		{
			get
			{
				double oleTime;

				this._pattern.get_StartTime(out oleTime);

				SystemTime systemTime;

				NativeMethods.VariantTimeToSystemTime(oleTime, out systemTime);

				return
					new TimeSpan(
						0,
						systemTime.Hour,
						systemTime.Minute,
						systemTime.Second,
						systemTime.Millisecond);
			}

			set
			{
				SystemTime systemTime = new SystemTime();

				systemTime.Day = (short)value.Days;
				systemTime.Hour = (short)value.Hours;
				systemTime.Minute = (short)value.Minutes;
				systemTime.Second = (short)value.Seconds;
				systemTime.Millisecond = (short)value.Milliseconds;

				double oleTime;

				NativeMethods.SystemTimeToVariantTime(ref systemTime, out oleTime);

				this._pattern.put_StartTime(oleTime);
			}
		}

		#endregion

		#region EndTime

		public TimeSpan EndTime
		{
			get
			{
				double oleTime;

				this._pattern.get_EndTime(out oleTime);

				SystemTime systemTime;

				NativeMethods.VariantTimeToSystemTime(oleTime, out systemTime);

				return
					new TimeSpan(
						0,
						systemTime.Hour,
						systemTime.Minute,
						systemTime.Second,
						systemTime.Millisecond);
			}

			set
			{
				SystemTime systemTime = new SystemTime();

				systemTime.Hour = (short)value.Hours;
				systemTime.Minute = (short)value.Minutes;
				systemTime.Second = (short)value.Seconds;
				systemTime.Millisecond = (short)value.Milliseconds;

				double oleTime;

				NativeMethods.SystemTimeToVariantTime(ref systemTime, out oleTime);

				this._pattern.put_EndTime(oleTime);
			}
		}

		#endregion

		#region PatternEndDate

		public DateTime PatternEndDate
		{
			get
			{
				double oleTime;

				this._pattern.get_PatternEndDate(out oleTime);

				SystemTime systemTime;

				NativeMethods.VariantTimeToSystemTime(oleTime, out systemTime);

				return
					new DateTime(
						systemTime.Year,
						systemTime.Month,
						systemTime.Day);
			}

			set
			{
				SystemTime systemTime = new SystemTime();

				systemTime.Year = (short)value.Year;
				systemTime.Month = (short)value.Month;
				systemTime.Day = (short)value.Day;

				double oleTime;

				NativeMethods.SystemTimeToVariantTime(ref systemTime, out oleTime);

				this._pattern.put_PatternEndDate(oleTime);
			}
		}

		#endregion

		#region NoEndDate

		public bool NoEndDate
		{
			get
			{
				bool value;

				this._pattern.get_NoEndDate(out value);

				return value;
			}

			set
			{
				this._pattern.put_NoEndDate(value);
			}
		}

		#endregion

		#region Occurrences

		public int Occurrences
		{
			get
			{
				int value;

				this._pattern.get_Occurrences(out value);

				return value;
			}

			set
			{
				this._pattern.put_Occurrences(value);
			}
		}

		#endregion

		#region Interval

		public int Interval
		{
			get
			{
				int value;

				this._pattern.get_Interval(out value);

				return value;
			}

			set
			{
				this._pattern.put_Interval(value);
			}
		}

		#endregion

		#region DayOfWeekMask

		public OutlookDaysOfWeek DayOfWeekMask
		{
			get
			{
				OutlookDaysOfWeek value;

				this._pattern.get_DayOfWeekMask(out value);

				return value;
			}

			set
			{
				this._pattern.put_DayOfWeekMask(value);
			}
		}

		#endregion

		#region DayOfMonth

		public int DayOfMonth
		{
			get
			{
				int value;

				this._pattern.get_DayOfMonth(out value);

				return value;
			}

			set
			{
				this._pattern.put_DayOfMonth(value);
			}
		}

		#endregion

		#region Instance

		public int Instance
		{
			get
			{
				int value;

				this._pattern.get_Instance(out value);

				return value;
			}

			set
			{
				this._pattern.put_Instance(value);
			}
		}

		#endregion

		#region Duration

		public int Duration
		{
			get
			{
				int value;

				this._pattern.get_Duration(out value);

				return value;
			}

			set
			{
				this._pattern.put_Duration(value);
			}
		}

		#endregion

		#region MonthOfYear

		public int MonthOfYear
		{
			get
			{
				int value;

				this._pattern.get_MonthOfYear(out value);

				return value;
			}

			set
			{
				this._pattern.put_MonthOfYear(value);
			}
		}

		#endregion

		#region Exceptions

		public OutlookExceptionCollection Exceptions
		{
			get
			{
				IOutlookExceptions exceptions;

				this._pattern.get_Exceptions(out exceptions);

				return new OutlookExceptionCollection(exceptions);
			}
		}

		#endregion

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
