﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Mobile Library                           */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;

namespace Nabu.Mobile
{
	public sealed class MobileDevice
	{
		#region MobileDevice

		#region Fields

		private IRapiDevice _rapiDevice;
		private Guid _deviceID;
		private string _deviceName;
		private string _devicePlatform;
		private byte[] _deviceAddress;
		private byte[] _hostAddress;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		internal MobileDevice(IRapiDevice rapiDevice)
		{
			this._rapiDevice = rapiDevice;

			DeviceInfo deviceInfo;

			this._rapiDevice.GetDeviceInfo(out deviceInfo);
			this._deviceID = deviceInfo.ID;
			this._deviceName = deviceInfo.Name;
			this._devicePlatform = deviceInfo.Platform;

			ConnectionInfo connectionInfo;

			this._rapiDevice.GetConnectionInfo(out connectionInfo);

			this._deviceAddress = connectionInfo.DeviceAddress.AddressData;
			this._hostAddress = connectionInfo.HostAddress.AddressData;
		}

		[DebuggerStepThrough]
		public MobileSession CreateSession()
		{
			IRapiSession rapiSession;

			this._rapiDevice.CreateSession(out rapiSession);

			return new MobileSession(rapiSession);
		}

		#endregion

		#region Properties

		public Guid DeviceID
		{
			[DebuggerStepThrough]
			get { return this._deviceID; }
		}

		public string DeviceName
		{
			[DebuggerStepThrough]
			get { return this._deviceName; }
		}

		public string DevicePlatform
		{
			[DebuggerStepThrough]
			get { return this._devicePlatform; }
		}

		[SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays")]
		public byte[] DeviceAddress
		{
			[DebuggerStepThrough]
			get { return this._deviceAddress; }
		}

		[SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays")]
		public byte[] HostAddress
		{
			[DebuggerStepThrough]
			get { return this._hostAddress; }
		}

		#endregion

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
