﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Mobile Library                           */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Runtime.Serialization;
using System.Text;

namespace Nabu.Mobile
{
	[Serializable]
	public sealed class MobileException : Exception
	{
		#region Utility

		private static string GetMessage(
			MobileSession session,
			string operation,
			string sourcePath,
			string destinationPath)
		{
			StringBuilder messageBuilder = new StringBuilder();

			messageBuilder.AppendFormat(
				"Operation: {0}",
				operation);
			messageBuilder.AppendLine();

			if (session.LastHostError != ComResult.S_OK)
			{
				messageBuilder.AppendFormat(
					"Host: {0}, {1}",
					session.LastHostError,
					NativeResult.GetMessage((int)session.LastHostError));
				messageBuilder.AppendLine();
			}

			if (session.LastDeviceError != ComResult.S_OK)
			{
				messageBuilder.AppendFormat(
					"Device: {0}, {1}",
					session.LastDeviceError,
					NativeResult.GetMessage((int)session.LastDeviceError));
				messageBuilder.AppendLine();
			}

			if (destinationPath == null)
			{
				if (sourcePath != null)
				{
					messageBuilder.AppendFormat(
						"Path: {0}",
						sourcePath);
					messageBuilder.AppendLine();
				}
			}
			else
			{
				messageBuilder.AppendFormat(
					"Source Path: {0}",
					sourcePath);
				messageBuilder.AppendLine();
				messageBuilder.AppendFormat(
					"Destination Path: {0}",
					sourcePath);
				messageBuilder.AppendLine();
			}

			return messageBuilder.ToString();
		}

		#endregion

		#region Constructors

		internal MobileException(
			MobileSession session,
			string operation,
			string sourcePath,
			string destinationPath)
			: base(GetMessage(session, operation, sourcePath, destinationPath))
		{
			base.HResult = (int)session.LastDeviceError;
		}

		internal MobileException(
			MobileSession session,
			string operation,
			string path)
			: base(GetMessage(session, operation, path, null))
		{
			base.HResult = (int)session.LastDeviceError;
		}

		internal MobileException(
			MobileSession session,
			string operation)
			: base(GetMessage(session, operation, null, null))
		{
			base.HResult = (int)session.LastDeviceError;
		}

		public MobileException()
		{
		}

		public MobileException(string message)
			: base(message)
		{
		}

		public MobileException(
			string message,
			Exception innerException)
			: base(message, innerException)
		{
		}

		private MobileException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
