﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Mobile Library                           */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.IO;

namespace Nabu.Mobile
{
	public static class MobileFile
	{
		[SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")]
		public static void WriteAllBytes(
			MobileSession session,
			string path,
			byte[] data,
			int step,
			EventHandler<ProgressEventArgs<int, string>> progress)
		{
			using (Stream stream = session.CreateFileStream(path, FileMode.Create, FileAccess.Write, FileShare.Read))
			{
				for (int index = 0; index < data.Length; index += step)
				{
					if (progress != null)
					{
						progress(
							stream,
							new ProgressEventArgs<int, string>(data.Length, index, path));
					}

					stream.Write(data, index, Math.Min(data.Length - index, step));
				}

				if (progress != null)
				{
					progress(
						stream,
						new ProgressEventArgs<int, string>(data.Length, data.Length, path));
				}
			}
		}

		public static void WriteAllBytes(
			MobileSession session,
			string path,
			byte[] data)
		{
			WriteAllBytes(session, path, data, 65536, null);
		}

		[SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")]
		public static byte[] ReadAllBytes(
			MobileSession session,
			string path,
			int step,
			EventHandler<ProgressEventArgs<int, string>> progress)
		{
			using (Stream stream = session.CreateFileStream(path, FileMode.Open, FileAccess.Read, FileShare.Read))
			{
				byte[] data = new byte[stream.Length];
				int bytesRead = 0;

				for (int index = 0; index < data.Length; index += bytesRead)
				{
					if (progress != null)
					{
						progress(
							stream,
							new ProgressEventArgs<int, string>(data.Length, index, path));
					}

					bytesRead = stream.Read(data, index, Math.Min(data.Length - index, step));
				}

				if (progress != null)
				{
					progress(
						stream,
						new ProgressEventArgs<int, string>(data.Length, data.Length, path));
				}

				return data;
			}
		}

		public static byte[] ReadAllBytes(
			MobileSession session,
			string path)
		{
			return ReadAllBytes(session, path, 65536, null);
		}
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
