﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Mobile Library                           */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;

namespace Nabu.Mobile
{
	public sealed class MobileFileEnumerator :
		DisposableBase,
		IEnumerable<MobileFileFindInfo>
	{
		#region MobileFileEnumerator

		#region Fields

		private MobileSession _session;
		private string _mask;

		#endregion

		#region Methods

		public MobileFileEnumerator(
			MobileSession session,
			string mask)
		{
			this._session = session;
			this._session.RegisterSessionObject(this);
			this._mask = mask;
		}

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			this._session.UnregisterSessionObject(this);
			base.Dispose(disposing);
		}

		#endregion

		#region IEnumerable<FileFindInfo>

		public IEnumerator<MobileFileFindInfo> GetEnumerator()
		{
			base.CheckDisposed();

			FileFindData filefindData;

			IntPtr hSearch =
				this._session.RapiSession.CeFindFirstFile(
				this._mask,
				out filefindData);

			if (
				(hSearch != IntPtr.Zero) &&
				(hSearch != new IntPtr(-1)))
			{
				yield return new MobileFileFindInfo(filefindData);

				while (this._session.RapiSession.CeFindNextFile(hSearch, out filefindData))
				{
					yield return new MobileFileFindInfo(filefindData);
				}

				this._session.RapiSession.CeFindClose(hSearch);
			}

			yield break;
		}

		#endregion

		#region IEnumerable

		System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
		{
			return ((IEnumerable<MobileFileFindInfo>)this).GetEnumerator();
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
