﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Mobile Library                           */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
using System.IO;

namespace Nabu.Mobile
{
	public sealed class MobileFileFindInfo
	{
		#region Fields

		private string _name;
		private FileAttributes _attributes;
		private DateTime _creationTime;
		private DateTime _lastAccessTime;
		private DateTime _lastWriteTime;
		private long _size;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		internal MobileFileFindInfo(FileFindData data)
		{
			this._name = data.Name;
			this._attributes = data.Attributes;
			this._creationTime =
				DateTime.FromFileTime(
					((long)data.CreationTimeHigh << 32) + ((long)data.CreationTimeLow));
			this._lastAccessTime =
				DateTime.FromFileTime(
					((long)data.LastAccessTimeHigh << 32) + ((long)data.LastAccessTimeLow));
			this._lastWriteTime =
				DateTime.FromFileTime(
					((long)data.LastWriteTimeHigh) + ((long)data.LastWriteTimeLow));
			this._size = (((long)data.SizeHigh) << 32) + ((long)data.SizeLow);
		}

		#endregion

		#region Properties

		public string Name
		{
			[DebuggerStepThrough]
			get { return this._name; }
		}

		public FileAttributes Attributes
		{
			[DebuggerStepThrough]
			get { return this._attributes; }
		}

		public DateTime CreationTime
		{
			[DebuggerStepThrough]
			get { return this._creationTime; }
		}

		public DateTime LastAccessTime
		{
			[DebuggerStepThrough]
			get { return this._lastAccessTime; }
		}

		public DateTime LastWriteTime
		{
			[DebuggerStepThrough]
			get { return this._lastWriteTime; }
		}

		public long Size
		{
			[DebuggerStepThrough]
			get { return this._size; }
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
