﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Mobile Library                           */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace Nabu.Mobile
{
	public sealed class MobileHost : DisposableBase
	{
		#region MobileHost

		#region Types

		private class ActiveSyncSink : IRapiSink
		{
			#region ActiveSyncSink

			#region Fields

			private MobileHost _host;

			#endregion

			#region Methods

			public ActiveSyncSink(MobileHost host)
			{
				this._host = host;
			}

			#endregion

			#endregion

			#region IRapiSink

			public void OnDeviceConnected(IRapiDevice rapiDevice)
			{
				MobileDevice device = new MobileDevice(rapiDevice);

				this._host._devices.Add(device);

				if (this._host.DeviceConnected != null)
				{
					this._host.DeviceConnected(this._host, new EventArgs<MobileDevice>(device));
				}
			}

			public void OnDeviceDisconnected(IRapiDevice rapiDevice)
			{
				DeviceInfo deviceInfo;

				rapiDevice.GetDeviceInfo(out deviceInfo);

				for (int index = 0; index < this._host._devices.Count; index++)
				{
					MobileDevice device = this._host._devices[index];

					if (device.DeviceID == deviceInfo.ID)
					{
						if (this._host.DeviceDisconnected != null)
						{
							this._host.DeviceDisconnected(this._host, new EventArgs<MobileDevice>(device));
						}

						this._host._devices.RemoveAt(index);
					}
				}
			}

			#endregion
		}

		#endregion

		#region Fields

		private IRapiDesktop _rapiDesktop;
		private List<MobileDevice> _devices;
		private ActiveSyncSink _sink;
		private int _sinkCookie;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public MobileHost()
		{
			this._rapiDesktop = (new RapiDesktopClass()) as IRapiDesktop;
			this._devices = new List<MobileDevice>();

			IRapiEnumDevices rapiEnumDevices;

			this._rapiDesktop.EnumDevices(out rapiEnumDevices);

			IRapiDevice rapiDevice;

			while (rapiEnumDevices.Next(out rapiDevice) == ComResult.S_OK)
			{
				this._devices.Add(new MobileDevice(rapiDevice));
			}

			Marshal.ReleaseComObject(rapiEnumDevices);
			rapiEnumDevices = null;

			this._sink = new ActiveSyncSink(this);
			this._rapiDesktop.Advise(this._sink, out this._sinkCookie);
		}

		#endregion

		#region Properties

		public IEnumerable<MobileDevice> Devices
		{
			[DebuggerStepThrough]
			get { return this._devices; }
		}

		#endregion

		#region Events

		public event EventHandler<EventArgs<MobileDevice>> DeviceConnected;
		public event EventHandler<EventArgs<MobileDevice>> DeviceDisconnected;

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			if (this._rapiDesktop != null)
			{
				if (this._sinkCookie != 0)
				{
					this._rapiDesktop.UnAdvise(this._sinkCookie);
				}
			}

			base.Dispose(disposing);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
