﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Mobile Library                           */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using System.Runtime.InteropServices;

namespace Nabu.Mobile
{
#pragma warning disable 649

	internal enum ConnectionType : int
	{
		Usb,
		InfraRed,
		Serial,
		Network
	}

	internal enum DeviceOpCode : int
	{
		NonBlocking,
		Blocking
	}

	[Flags]
	internal enum FindAllFilesFlags : int
	{
		QueryAttributes /*       */ = 0x00000001,
		QueryCreationTime /*     */ = 0x00000002,
		QueryLastAccessTime /*   */ = 0x00000004,
		QueryLastWriteTime /*    */ = 0x00000008,
		QuerySize /*             */ = 0x00000030,
		QueryObjectID /*         */ = 0x00000040,
		QueryName /*             */ = 0x00000080,
		FilterChildren /*        */ = 0x00010000,
		FilterNoHidden /*        */ = 0x00020000,
		FilterFoldersOnly /*     */ = 0x00040000,
		FilterNoRomModules /*    */ = 0x00080000,
	}

	internal enum GetWindowCommand : int
	{
		First,
		Last,
		Next,
		Prev,
		Owner,
		Child
	}

	public enum MobileDeviceCaps : int
	{
		DriverVersion /*         */ = 0,
		Technology /*            */ = 2,
		HorizontalSizeMM /*      */ = 4,
		VerticalSizeMM /*        */ = 6,
		HorizontalSizePX /*      */ = 8,
		VerticalSizePX /*        */ = 10,
		BitPerPixel /*           */ = 12,
		PlanesPerPixel /*        */ = 14,
		BrushNumber /*           */ = 16,
		PenNumber /*             */ = 18,
		MarkerNumber /*          */ = 20,
		FontNumber /*            */ = 22,
		ColorNumber /*           */ = 24,
		DeviceDescriptorSize /*  */ = 26,
		CurveCaps /*             */ = 28,
		LineCaps /*              */ = 30,
		PolygonalCaps /*         */ = 32,
		TextCaps /*              */ = 34,
		ClipCaps /*              */ = 36,
		RasterCaps /*            */ = 38,
		AspectX /*               */ = 40,
		AspectY /*               */ = 42,
		AspectXY /*              */ = 44,
		PixelPerInchX /*         */ = 88,
		PixelPerInchY /*         */ = 90,
		PaletteSize /*           */ = 104,
		PaletteFree /*           */ = 106,
		ColorResolution /*       */ = 108,
		HorizontalSizeDU /*      */ = 110,
		VerticalSizeDU /*        */ = 111,
		HorizontalOffsetX /*     */ = 112,
		HorizontalOffsetY /*     */ = 113,
		ShadeBlendCaps /*        */ = 120
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	public enum MobileSpecialFolder : int
	{
		Temp /*       */ = -1,
		Programs /*   */ = 2,
		Controls /*   */ = 3,
		Printers /*   */ = 4,
		Personal /*   */ = 5,
		Favorites /*  */ = 6,
		Startup /*    */ = 7,
		Recent /*     */ = 8,
		SendTo /*     */ = 9,
		BitBucket /*  */ = 10,
		StartMenu /*  */ = 11,
		MyMusic /*    */ = 14,
		Desktop /*    */ = 16,
		Drives /*     */ = 17,
		Network /*    */ = 18,
		Nethood /*    */ = 19,
		Fonts /*      */ = 20,
		Templates /*  */ = 21,
		AppData /*    */ = 26
	}

	public enum MobileSystemMetrics : int
	{
		ScreenWidth /*          */ = 0,
		ScreenHeight /*         */ = 1,
		VScrollWidth /*         */ = 2,
		HScrollHeight /*        */ = 3,
		CaptionHeight /*        */ = 4,
		BorderWidth /*          */ = 5,
		BorderHeight /*         */ = 6,
		DialogFrameWidth /*     */ = 7,
		DialogFrameHeight /*    */ = 8,

		IconWidth /*            */ = 11,
		IconHeight /*           */ = 12,

		MenuHeight /*           */ = 15,
		FullScreenWidth /*      */ = 16,
		FullScreenHeight /*     */ = 17,

		MousePresent /*         */ = 19,
		VScrollHeight /*        */ = 20,
		HScrollWidth /*         */ = 21,
		Debug /*                */ = 22,

		DoubleClickWidth /*     */ = 36,
		DoubleClickHeight /*    */ = 37,
		IconSpacingWidth /*     */ = 38,
		IconSpacingHeight /*    */ = 39,

		EdgeWidth /*            */ = 45,
		EdgeHeight /*           */ = 46,

		SmallIconWidth /*       */ = 49,
		SmallIconHeight /*      */ = 50,

		VirtualScreenX /*       */ = 76,
		VirtualScreenY /*       */ = 77,
		VirtualScreenWidth /*   */ = 78,
		VirtualScreenHeight /*  */ = 79,

		MonitorNumber /*        */ = 80,
		SameDisplayformat /*    */ = 81
	}

	public enum MobileProcessorArchitecture : int
	{
		Intel,
		MIPS,
		Alpha,
		PowerPC,
		SHX,
		ARM,
		IA64,
		Alpha64,
		Unknown
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	public enum MobileProcessorType : int
	{
		Intel386 = 386,
		Intel486 = 486,
		IntelPentiumI = 586,
		IntelPentiumII = 686,
		MipsR4000 = 4000,
		MipsR5000 = 5000,
		Alpha21064 = 21064,
		PowerPC403 = 403,
		PowerPC601 = 601,
		PowerPC603 = 603,
		PowerPC604 = 604,
		PowerPC620 = 620,
		HitachiSH3 = 10003,
		HitachiSH3E = 10004,
		HitachiSH4 = 10005,
		Motorola821 = 821,
		SHxSH3 = 103,
		SHxSH4 = 104,
		SHxSH3DSP = 105,
		StrongARM = 2577,
		ARM720 = 1824,
		ARM820 = 2080,
		ARM920 = 2336,
		ARM7TDMI = 70001
	}

	public enum MobileRegistryRights : int
	{
		Undefined /*          */ = 0x00000000,
		QueryValue /*         */ = 0x00000001,
		SetValue /*           */ = 0x00000002,
		CreateSubKey /*       */ = 0x00000004,
		EnumerateSubKeys /*   */ = 0x00000008,
		KeyNotify /*          */ = 0x00000010,
		KeyCreateLink /*      */ = 0x00000020,
		Read /*               */ = 0x00020019,
		Write /*              */ = 0x00020006,
		All /*                */ = 0x001F003F
	}

	[SuppressMessage("Microsoft.Design", "CA1008:EnumsShouldHaveZeroValue")]
	public enum MobileRegistryKeys : int
	{
		HKEY_CLASSES_ROOT = (int)(0x80000000 - 0x100000000),
		HKEY_CURRENT_USER = (int)(0x80000001 - 0x100000000),
		HKEY_LOCAL_MACHINE = (int)(0x80000002 - 0x100000000),
		HKEY_USERS = (int)(0x80000003 - 0x100000000)
	}

	[StructLayout(LayoutKind.Sequential, Pack = 1)]
	internal struct ConnectionInfo
	{
		public SockAddrStorage DeviceAddress;
		public SockAddrStorage HostAddress;
		public ConnectionType ConnectionType;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 1)]
	internal struct DeviceInfo
	{
		public Guid ID;
		public int VersionMajor;
		public int VersionMinor;
		[MarshalAs(UnmanagedType.BStr)]
		public string Name;
		[MarshalAs(UnmanagedType.BStr)]
		public string Platform;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 1, CharSet = CharSet.Unicode)]
	internal struct FileFindData
	{
		[MarshalAs(UnmanagedType.U4)]
		public FileAttributes Attributes;
		public uint CreationTimeLow;
		public uint CreationTimeHigh;
		public uint LastAccessTimeLow;
		public uint LastAccessTimeHigh;
		public uint LastWriteTimeLow;
		public uint LastWriteTimeHigh;
		public uint SizeHigh;
		public uint SizeLow;
		public int ObjectID;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
		public string Name;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 1)]
	internal struct MemoryStatus
	{
		public int Size;
		public int Load;
		public int TotalPhys;
		public int AvailPhys;
		public int TotalPageFile;
		public int AvailPageFile;
		public int TotalVirtual;
		public int AvailVirtual;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 1, CharSet = CharSet.Unicode)]
	internal struct OSVersionInfo
	{
		public int Size;
		public int MajorVersion;
		public int MinorVersion;
		public int BuildNumber;
		public int PlatformID;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
		public string Version;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 1)]
	internal struct ProcessInformation
	{
		public IntPtr hProcess;
		public IntPtr hThread;
		public int ProcessID;
		public int ThreadID;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 1)]
	internal struct SockAddrStorage
	{
		public short AddressFamily;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 126)]
		public byte[] AddressData;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 1)]
	internal struct SortSpecification
	{
		public int PropertyID;
		public int Flags;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 1)]
	internal struct StoreInformation
	{
		public int StoreSize;
		public int FreeSize;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 1)]
	internal struct SystemInfo
	{
		[MarshalAs(UnmanagedType.U4)]
		public MobileProcessorArchitecture ProcessorArchitecture;
		public short Reserved;
		public int PageSize;
		public int MinimumApplicationAddress;
		public int MaximumApplicationAddress;
		public int ActiveProcessorMask;
		public int ProcessorNumber;
		[MarshalAs(UnmanagedType.U4)]
		public MobileProcessorType ProcessorType;
		public int AllocationGranularity;
		public short ProcessorLevel;
		public short ProcessorRevision;
	}

	[StructLayout(LayoutKind.Sequential, Pack = 1)]
	internal struct SystemPowerStatusEx
	{
		/*
		public byte ACLineStatus;
		public byte BatteryFlag;
		public byte BatteryLifePercent;
		public byte Reserved1;
		public int BatteryLifeTime;
		public int BatteryFullLifeTime;
		public byte Reserved2;
		public byte BackupBatteryFlag;
		public byte BackupBatteryLifePercent;
		public byte Reserved3;
		public int BackupBatteryLifeTime;
		public int BackupBatteryFullLifeTime;
		*/
	}

#pragma warning restore 649
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
