﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Net Library                              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.IO;

using Nabu.Collections;

namespace Nabu.Net
{
	[SuppressMessage("Microsoft.Design", "CA1001:TypesThatOwnDisposableFieldsShouldBeDisposable")]
	public abstract class NetworkSessionBase<TSocket>
	{
		#region Types

		private class ReceiveBufferStream : Stream
		{
			#region ReceiveStream

			#region Fields

			private NetworkSessionBase<TSocket> _session;

			#endregion

			#region Methods

			[DebuggerStepThrough]
			public ReceiveBufferStream(NetworkSessionBase<TSocket> session)
			{
				this._session = session;
			}

			#endregion

			#endregion

			#region Stream

			#region Methods

			[DebuggerStepThrough]
			public override int Read(byte[] buffer, int offset, int count)
			{
				if (this._session.ReceiveOffset + count > this._session.ReceiveBuffer.Count)
				{
					count = this._session.ReceiveBuffer.Count - this._session.ReceiveOffset;
				}

				this._session.ReceiveBuffer.CopyTo(
					this._session.ReceiveOffset,
					buffer,
					offset,
					count);

				this._session.ReceiveOffset += count;

				return count;
			}

			[DebuggerStepThrough]
			public override int ReadByte()
			{
				byte value = this._session.ReceiveBuffer[this._session.ReceiveOffset];

				this._session.ReceiveOffset++;

				return value;
			}

			[DebuggerStepThrough]
			public override void Write(byte[] buffer, int offset, int count)
			{
				throw new NotImplementedException();
			}

			[DebuggerStepThrough]
			public override long Seek(long offset, SeekOrigin origin)
			{
				throw new NotImplementedException();
			}

			[DebuggerStepThrough]
			public override void Flush()
			{
			}

			[DebuggerStepThrough]
			public override void SetLength(long value)
			{
				throw new NotImplementedException();
			}

			#endregion

			#region Properties

			public override bool CanRead
			{
				[DebuggerStepThrough]
				get { return true; }
			}

			public override bool CanWrite
			{
				[DebuggerStepThrough]
				get { return false; }
			}

			public override bool CanSeek
			{
				[DebuggerStepThrough]
				get { return false; }
			}

			public override long Length
			{
				[DebuggerStepThrough]
				get { return this._session.ReceiveSize; }
			}

			public override long Position
			{
				[DebuggerStepThrough]
				get { throw new NotImplementedException(); }
				[DebuggerStepThrough]
				set { throw new NotImplementedException(); }
			}

			#endregion

			#endregion
		}

		private class SendBufferStream : Stream
		{
			#region SendStream

			#region Fields

			private NetworkSessionBase<TSocket> _session;

			#endregion

			#region Methods

			[DebuggerStepThrough]
			public SendBufferStream(NetworkSessionBase<TSocket> session)
			{
				this._session = session;
			}

			#endregion

			#endregion

			#region Stream

			#region Methods

			[DebuggerStepThrough]
			public override int Read(byte[] buffer, int offset, int count)
			{
				throw new NotImplementedException();
			}

			[DebuggerStepThrough]
			public override void Write(byte[] buffer, int offset, int count)
			{
				this._session.SendBuffer.AddRange(buffer, offset, count);
			}

			[DebuggerStepThrough]
			public override long Seek(long offset, SeekOrigin origin)
			{
				throw new NotImplementedException();
			}

			[DebuggerStepThrough]
			public override void Flush()
			{
			}

			[DebuggerStepThrough]
			public override void SetLength(long value)
			{
				throw new NotImplementedException();
			}

			#endregion

			#region Properties

			public override bool CanRead
			{
				[DebuggerStepThrough]
				get { return false; }
			}

			public override bool CanWrite
			{
				[DebuggerStepThrough]
				get { return true; }
			}

			public override bool CanSeek
			{
				[DebuggerStepThrough]
				get { return false; }
			}

			public override long Length
			{
				[DebuggerStepThrough]
				get { return this._session.SendSize; }
			}

			public override long Position
			{
				[DebuggerStepThrough]
				get { throw new NotImplementedException(); }
				[DebuggerStepThrough]
				set { throw new NotImplementedException(); }
			}

			#endregion

			#endregion
		}

		#endregion

		#region Fields

		private TSocket _socket;
		private ListEx<byte> _receiveBuffer;
		private ListEx<byte> _sendBuffer;
		private int _receiveOffset;
		private int _sendOffset;
		private Stream _receiveStream;
		private Stream _sendStream;
		private bool _isReceiving;
		private bool _isSending;
		private bool _isClosing;

		#endregion

		#region  Methods

		[DebuggerStepThrough]
		protected NetworkSessionBase()
		{
			this._receiveBuffer = new ListEx<byte>(256 * 1024);
			this._sendBuffer = new ListEx<byte>(256 * 1024);
			this._receiveStream = new ReceiveBufferStream(this);
			this._sendStream = new SendBufferStream(this);
		}

		[DebuggerStepThrough]
		internal void RaiseException(Exception exception)
		{
			if (this.Exception != null)
			{
				this.Exception(
					this,
					new EventArgs<Exception>(exception));
			}
		}

		[DebuggerStepThrough]
		internal void ShrinkReceiveBuffer()
		{
			this._receiveBuffer.RemoveRange(0, this._receiveOffset);
			this._receiveOffset = 0;
		}

		[DebuggerStepThrough]
		internal void ShrinkSendBuffer()
		{
			this._sendBuffer.RemoveRange(0, this._sendOffset);
			this._sendOffset = 0;
		}

		#endregion

		#region Properties

		public TSocket Socket
		{
			[DebuggerStepThrough]
			get { return this._socket; }
			[DebuggerStepThrough]
			set { this._socket = value; }
		}

		internal ListEx<byte> SendBuffer
		{
			[DebuggerStepThrough]
			get { return this._sendBuffer; }
		}

		internal ListEx<byte> ReceiveBuffer
		{
			[DebuggerStepThrough]
			get { return this._receiveBuffer; }
		}

		internal int ReceiveOffset
		{
			[DebuggerStepThrough]
			get { return this._receiveOffset; }
			[DebuggerStepThrough]
			set { this._receiveOffset = value; }
		}

		internal int SendOffset
		{
			[DebuggerStepThrough]
			get { return this._sendOffset; }
			[DebuggerStepThrough]
			set { this._sendOffset = value; }
		}

		internal int ReceiveSize
		{
			[DebuggerStepThrough]
			get { return this._receiveBuffer.Count - this._receiveOffset; }
		}

		internal int SendSize
		{
			[DebuggerStepThrough]
			get { return this._sendBuffer.Count - this._sendOffset; }
		}

		public Stream ReceiveStream
		{
			[DebuggerStepThrough]
			get { return this._receiveStream; }
		}

		public Stream SendStream
		{
			[DebuggerStepThrough]
			get { return this._sendStream; }
		}

		public bool IsReceiving
		{
			[DebuggerStepThrough]
			get { return this._isReceiving; }
			[DebuggerStepThrough]
			set { this._isReceiving = value; }
		}

		public bool IsSending
		{
			[DebuggerStepThrough]
			get { return this._isSending; }
			[DebuggerStepThrough]
			set { this._isSending = value; }
		}

		public bool IsClosing
		{
			[DebuggerStepThrough]
			get { return this._isClosing; }
			[DebuggerStepThrough]
			set { this._isClosing = value; }
		}

		#endregion

		#region Events

		public event EventHandler<EventArgs<Exception>> Exception;

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
