﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Net Library                              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;

namespace Nabu.Net
{
	[SuppressMessage("Microsoft.Design", "CA1005:AvoidExcessiveParametersOnGenericTypes")]
	public abstract class TcpClientBase<TSocket, TSession, TCallback> :
		TcpNodeBase<TSocket, TSession, TCallback>
		where TSocket : ITcpClientSocket, new()
		where TSession : NetworkSessionBase<TSocket>
		where TCallback : ITcpNodeCallback<TSession>
	{
		protected TcpClientBase(TCallback callback)
			: base(callback)
		{
		}

		protected void SslAuthenticate(
			TSession session,
			string hostname)
		{
			X509CertificateCollection certificates;
			bool mutualAuthenticationRequired;
			SslProtocols enabledSslProtocols;
			bool checkCertificateRevocation;

			base.Callback.OnSslAuthenticate(session, out certificates, out enabledSslProtocols, out mutualAuthenticationRequired, out checkCertificateRevocation);

			session.Socket.AuthenticateAsClient(
				hostname,
				certificates,
				enabledSslProtocols,
				checkCertificateRevocation);
		}

		[SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
		public void OpenSession(
			string hostname,
			int port,
			TcpConnectionWrapper wrapper,
			object customData)
		{
			base.CheckDisposed();

			TSession session = CreateSession(customData);

			try
			{
				session.Socket = new TSocket();
				session.Socket.Connect(
					new TcpSecurityCallback(session, base.Callback),
					hostname,
					port);

				switch (wrapper)
				{
					case TcpConnectionWrapper.SSL:
						SslAuthenticate(session, hostname);
						break;
				}

				base.Callback.OnOpenSession(session);

				RunSession(session);
			}
			catch (Exception exception)
			{
				OnException(session, exception);
			}
		}
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
