﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Net Library                              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Net;
using System.Net.Sockets;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;

namespace Nabu.Net
{
	public sealed class TcpClientSocket :
		TcpNodeSocketBase,
		ITcpClientSocket
	{
		#region TcpClientSocket

		#region Fields

		private ITcpSecurityCallback _securityCallback;

		#endregion

		#region Methods

		public TcpClientSocket()
		{
		}

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			base.Dispose(disposing);
		}

		#endregion

		#region IClientSocket

		public void Connect(
			ITcpSecurityCallback securityCallback,
			string hostname,
			int port)
		{
			base.CheckDisposed();

			this._securityCallback = securityCallback;
			this.Socket.Connect(
				new IPEndPoint(
					SocketUtil.IPAddressFromHostName(hostname),
					port));
			this.RawStream = new NetworkStream(this.Socket, false);
			this.CurrentStream = this.RawStream;
		}

		public void AuthenticateAsClient(
			string targetHost,
			X509CertificateCollection clientCertificates,
			SslProtocols enabledSslProtocols,
			bool checkCertificateRevocation)
		{
			base.CheckDisposed();

			this.SslStream =
				new TcpNodeStream(
					this._securityCallback,
					this.RawStream);
			this.SslStream.AuthenticateAsClient(
				targetHost,
				clientCertificates,
				enabledSslProtocols,
				checkCertificateRevocation);
			this.CurrentStream = this.SslStream;
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
