﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Net Library                              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Net.Sockets;
using System.Threading;

namespace Nabu.Net
{
	[SuppressMessage("Microsoft.Design", "CA1005:AvoidExcessiveParametersOnGenericTypes")]
	public abstract class TcpServerBase<TSocketListen, TSocketIO, TSession, TCallback> :
		TcpNodeBase<TSocketIO, TSession, TCallback>
		where TSocketListen : ITcpServerSocketListen, new()
		where TSocketIO : ITcpServerSocketIO
		where TSession : NetworkSessionBase<TSocketIO>
		where TCallback : ITcpNodeCallback<TSession>
	{
		#region TcpServerBase

		#region Fields

		private Thread _thread;
		private TSocketListen _socket;

		#endregion

		#region Methods

		[SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
		private void ThreadProcListen()
		{
			base.CheckDisposed();

			try
			{
				for (; ; )
				{
					TSession session = CreateSession(null);
					TSocketIO socket = (TSocketIO)this._socket.Accept(new TcpSecurityCallback(session, base.Callback));

					if (socket == null)
					{
						break;
					}
					else
					{
						session.Socket = socket;

						try
						{
							base.Callback.OnOpenSession(session);
							RunSession(session);
						}
						catch (Exception exception)
						{
							OnException(session, exception);
						}
					}
				}
			}
			catch (SocketException)
			{
				// Expecting
				// A blocking operation was interrupted by a call to WSACancelBlockingCall
			}
		}

		protected TcpServerBase(TCallback callback)
			: base(callback)
		{
		}

		public void Start(string hostname, int port)
		{
			base.CheckDisposed();

			this._socket = new TSocketListen();
			this._socket.Listen(hostname, port);
			this._thread = new Thread(new ThreadStart(ThreadProcListen));
			this._thread.Start();
		}

		public void Stop()
		{
			base.CheckDisposed();

			if (this._thread != null)
			{
				this._socket.Dispose();
				this._socket = default(TSocketListen);
				this._thread.Join();
				this._thread = null;
				base.WaitForSessions();
			}
		}

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			Stop();
			base.Dispose(disposing);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
