﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Net Library                              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Net;
using System.Net.Sockets;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;

namespace Nabu.Net
{
	public sealed class TcpServerSocketListen :
		DisposableBase,
		ITcpServerSocketListen
	{
		#region TcpServerSocketListen

		#region Fields

		private Socket _socket;

		#endregion

		#region Methods

		public TcpServerSocketListen()
		{
			this._socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
		}

		#endregion

		#endregion

		#region IServerSocketListen

		public void Listen(
			string hostname,
			int port)
		{
			base.CheckDisposed();

			this._socket.Bind(
				new IPEndPoint(
					SocketUtil.IPAddressFromHostName(hostname),
					port));
			this._socket.Listen(16);
		}

		public ITcpServerSocketIO Accept(ITcpSecurityCallback securityCallback)
		{
			base.CheckDisposed();

			return
				new TcpServerSocketIO(
					securityCallback,
					this._socket.Accept());
		}

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			if (this._socket != null)
			{
				this._socket.Close();
				this._socket = null;
			}

			base.Dispose(disposing);
		}

		#endregion
	}

	public sealed class TcpServerSocketIO :
		TcpNodeSocketBase,
		ITcpServerSocketIO
	{
		#region TcpServerSocketIO

		#region Fields

		private ITcpSecurityCallback _securityCallback;

		#endregion

		#region Methods

		public TcpServerSocketIO(
			ITcpSecurityCallback securityCallback,
			Socket socket)
			: base(socket)
		{
			this.RawStream = new NetworkStream(this.Socket, false);
			this.CurrentStream = this.RawStream;
			this._securityCallback = securityCallback;
		}

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			base.Dispose(disposing);
		}

		#endregion

		#region IServerSocketIO

		public void AuthenticateAsServer(
			X509Certificate serverCertificate,
			bool clientCertificateRequired,
			SslProtocols enabledSslProtocols,
			bool checkCertificateRevocation)
		{
			base.CheckDisposed();

			this.SslStream =
				new TcpNodeStream(
					this._securityCallback,
					this.RawStream);
			this.SslStream.AuthenticateAsServer(
				serverCertificate,
				clientCertificateRequired,
				enabledSslProtocols,
				checkCertificateRevocation);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
