﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Net Library                              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Text;

namespace Nabu.Net
{
	public static class TextProtocol
	{
		public static int ParseInteger(string text, int defaultValue)
		{
			int result;

			if (!int.TryParse(text, out result))
			{
				result = defaultValue;
			}

			return result;
		}

		public static int ParseInteger(string text)
		{
			return ParseInteger(text, 0);
		}

		public static void Write<TSocket>(
			NetworkSessionBase<TSocket> session,
			string text)
		{
			byte[] data = Encoding.ASCII.GetBytes(text);

			session.SendStream.Write(
				data,
				0,
				data.Length);
		}

		public static string ReadLine<TSocket>(
			NetworkSessionBase<TSocket> session)
		{
			int index =
				session.ReceiveBuffer.IndexOf(
					10,
					session.ReceiveOffset,
					session.ReceiveSize);
			int countToRead;

			if (index == -1)
			{
				countToRead = session.ReceiveSize;
			}
			else
			{
				countToRead = index - session.ReceiveOffset;
			}

			int countToSkip = countToRead;

			while (
				(countToRead > 0) &&
				(session.ReceiveBuffer[session.ReceiveOffset + countToRead] < 32))
			{
				countToRead--;
			}

			while (
				(countToSkip < session.ReceiveSize) &&
				(session.ReceiveBuffer[session.ReceiveOffset + countToSkip] < 32))
			{
				countToSkip++;
			}

			string text =
				Encoding.ASCII.GetString(
					session.ReceiveBuffer.DangerousGetArray(),
					session.ReceiveOffset,
					countToRead + 1);

			session.ReceiveOffset += countToSkip;

			return text;
		}

		public static string ReadToEnd<TSocket>(
			NetworkSessionBase<TSocket> session)
		{
			string text =
				Encoding.ASCII.GetString(
					session.ReceiveBuffer.DangerousGetArray(),
					session.ReceiveOffset,
					session.ReceiveSize);

			session.ReceiveBuffer.Count = 0;

			return text;
		}
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
