﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Net Library                              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Net;
using System.Net.Sockets;

namespace Nabu.Net
{
	public sealed class UdpClientSocket :
		DisposableBase,
		IUdpClientSocket
	{
		#region UdpClientSocket

		#region Fields

		private Socket _socket;

		#endregion

		#region Methods

		public UdpClientSocket()
		{
			this._socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
		}

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				this._socket.Close();
				this._socket = null;
			}

			base.Dispose(disposing);
		}

		#endregion

		#region IUdpClientSocket

		public IAsyncResult BeginSendTo(
			IPEndPoint address,
			byte[] buffer,
			int offset,
			int count,
			AsyncCallback callback,
			object state)
		{
			base.CheckDisposed();

			return
				this._socket.BeginSendTo(
					buffer,
					offset,
					count,
					SocketFlags.None,
					address,
					callback,
					state);
		}

		public int EndSendTo(IAsyncResult asyncResult)
		{
			base.CheckDisposed();

			return this._socket.EndSendTo(asyncResult);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
