﻿/******************************************************************************/
using System;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;

namespace Nabu.Net
{
	[SuppressMessage("Microsoft.Design", "CA1005:AvoidExcessiveParametersOnGenericTypes")]
	public abstract class UdpNodeBase<TSocket, TSession, TCallback> : DisposableBase
		where TSocket : IDisposable, new()
		where TSession : NetworkSessionBase<TSocket>
		where TCallback : IUdpNodeCallback<TSession>
	{
		#region UdpClientBase

		#region Fields

		private TCallback _callback;

		#endregion

		#region Methods

		private void CloseSession(TSession session)
		{
			if (session != null)
			{
				base.CheckDisposed();

				session.Socket.Dispose();
				session.Socket = default(TSocket);
				session = null;

				this._callback.OnCloseSession(session);
			}
		}

		protected abstract TSession CreateSession(object sessionCustomData);

		protected void OnException(
			TSession session,
			Exception exception)
		{
			session.RaiseException(exception);
			CloseSession(session);
		}

		protected UdpNodeBase(TCallback callback)
		{
			if (callback == null)
			{
				throw new ArgumentNullException("callback");
			}

			this._callback = callback;
		}

		#endregion

		#region Properties

		protected TCallback Callback
		{
			[DebuggerStepThrough]
			get
			{
				base.CheckDisposed();

				return this._callback;
			}
		}

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			base.Dispose(disposing);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
