﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Net Library                              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Net;
using System.Net.Sockets;

namespace Nabu.Net
{
	public sealed class UdpServerSocket :
		DisposableBase,
		IUdpServerSocket
	{
		#region UdpServerSocket

		#region Fields

		private Socket _socket;

		#endregion

		#region Methods

		public UdpServerSocket()
		{
			this._socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
		}

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				this._socket.Close();
				this._socket = null;
			}

			base.Dispose(disposing);
		}

		#endregion

		#region IUdpServerSocket

		public void Bind(string hostname, int port)
		{
			base.CheckDisposed();

			this._socket.Bind(new IPEndPoint(SocketUtil.IPAddressFromHostName(hostname), port));
		}

		public IAsyncResult BeginReceiveFrom(
			byte[] buffer,
			int offset,
			int count,
			AsyncCallback callback,
			object state)
		{
			base.CheckDisposed();

			EndPoint address = new IPEndPoint(IPAddress.Any, 0);

			return
				this._socket.BeginReceiveFrom(
					buffer,
					offset,
					count,
					SocketFlags.None,
					ref address,
					callback,
					state);
		}

		public int EndReceiveFrom(out IPEndPoint address, IAsyncResult asyncResult)
		{
			base.CheckDisposed();

			EndPoint endPoint = new IPEndPoint(IPAddress.Any, 0);
			int result = this._socket.EndReceiveFrom(asyncResult, ref endPoint);

			address = (IPEndPoint)endPoint;

			return result;
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
