﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Net Library                              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Globalization;
using System.Net;
using System.Text;

namespace Nabu.Net
{
	public class SysLogClientBase<TSocket> :
		UdpClientBase<TSocket, SysLogClientSession<TSocket>, ISysLogClientCallback<TSocket>>
		where TSocket : IUdpClientSocket, new()
	{
		#region SysLogClientBase

		public SysLogClientBase(ISysLogClientCallback<TSocket> callback)
			: base(callback)
		{
		}

		#endregion

		#region UdpClientBase

		protected override SysLogClientSession<TSocket> CreateSession(object sessionCustomData)
		{
			return new SysLogClientSession<TSocket>();
		}

		protected override void OnSend(
			SysLogClientSession<TSocket> session,
			out IPEndPoint endPoint)
		{
			SysLogFacility facility;
			SysLogSeverity severity;
			DateTime timestamp;
			string hostname;
			string message;

			if (base.Callback.GetMessage(
				out endPoint,
				out facility,
				out severity,
				out timestamp,
				out hostname,
				out message))
			{
				string packet =
					string.Format(
						CultureInfo.InvariantCulture,
						"<{0}>{1:MMM dd HH:mm:ss} {2} {3}",
						8 * (int)facility + (int)severity,
						timestamp,
						hostname,
						message);

				byte[] data = Encoding.Default.GetBytes(packet);

				session.SendStream.Write(data, 0, data.Length);
			}
			else
			{
				session.IsClosing = true;
			}
		}

		#endregion
	}

	public sealed class SysLogClient : SysLogClientBase<UdpClientSocket>
	{
		public SysLogClient(ISysLogClientCallback<UdpClientSocket> callback)
			: base(callback)
		{
		}
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
