﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Net Library                              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Globalization;
using System.Net;
using System.Text.RegularExpressions;

namespace Nabu.Net
{
	public class SysLogServerBase<TSocket> :
		UdpServerBase<TSocket, SysLogServerSession<TSocket>, ISysLogServerCallback<TSocket>>
		where TSocket : IUdpServerSocket, new()
	{
		#region SysLogServerBase

		#region Fields

		private Regex _regEx = new Regex(@"\<(\d+)\>((?:Jan|Feb|Mar|AprMay|Jun|Jul|Aug|Sep|Oct|Nov|Dec) [ 0123456789]\d \d\d:\d\d:\d\d) ([\S]+) (.*)");

		#endregion

		#region Methods

		public SysLogServerBase(ISysLogServerCallback<TSocket> callback)
			: base(callback)
		{
		}

		#endregion

		#endregion

		#region UdpServerBase

		protected override SysLogServerSession<TSocket> CreateSession(object sessionCustomData)
		{
			return new SysLogServerSession<TSocket>();
		}

		protected override void OnReceive(
			SysLogServerSession<TSocket> session,
			IPEndPoint endPoint)
		{
			string message = TextProtocol.ReadToEnd<TSocket>(session);
			Match match = this._regEx.Match(message);

			if (match.Success)
			{
				int priority =
					TextProtocol.ParseInteger(
						match.Groups[1].Value);
				DateTime timestamp =
					DateTime.ParseExact(
						match.Groups[2].Value,
						"MMM dd HH:mm:ss",
						CultureInfo.InvariantCulture);

				if (!base.Callback.OnMessage(
					endPoint,
					(SysLogFacility)(priority / 8),
					(SysLogSeverity)(priority % 8),
					timestamp,
					match.Groups[3].Value,
					match.Groups[4].Value))
				{
					session.IsClosing = true;
				}
			}
		}

		#endregion
	}

	public sealed class SysLogServer : SysLogServerBase<UdpServerSocket>
	{
		public SysLogServer(ISysLogServerCallback<UdpServerSocket> callback)
			: base(callback)
		{
		}
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
