﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Net Library                              */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Runtime.InteropServices;

namespace Nabu.Net
{
	public static class WirelessManager
	{
		#region Methods

		public static bool GetDeviceState(WirelessDeviceType deviceType)
		{
			IntPtr rootPointer;

			NativeMethods.GetWirelessDeviceList(out rootPointer, 0);

			try
			{
				IntPtr detailPointer = rootPointer;

				while (detailPointer != IntPtr.Zero)
				{
					WirelessDeviceDetails detail =
						(WirelessDeviceDetails)Marshal.PtrToStructure(
							detailPointer,
							typeof(WirelessDeviceDetails));
					detailPointer = detail.Next;

					if (detail.Type == deviceType)
					{
						return detail.CurrentState != 0;
					}
				}

				throw new ArgumentOutOfRangeException("deviceType");
			}
			finally
			{
				NativeMethods.FreeWirelessDeviceList(rootPointer);
			}
		}

		public static bool SetDeviceState(WirelessDeviceType deviceType, bool state)
		{
			IntPtr rootPointer;

			NativeMethods.GetWirelessDeviceList(out rootPointer, 0);

			try
			{
				IntPtr detailPointer = rootPointer;

				while (detailPointer != IntPtr.Zero)
				{
					WirelessDeviceDetails detail =
						(WirelessDeviceDetails)Marshal.PtrToStructure(
							detailPointer,
							typeof(WirelessDeviceDetails));
					detailPointer = detail.Next;

					if (detail.Type == deviceType)
					{
						if ((detail.CurrentState != 0) ^ state)
						{
							NativeMethods.ChangeWirelessDeviceState(
								ref detail,
								state ? 1 : 0,
								1);

							return true;
						}
						else
						{
							return false;
						}
					}
				}

				throw new ArgumentOutOfRangeException("deviceType");
			}
			finally
			{
				NativeMethods.FreeWirelessDeviceList(rootPointer);
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
