﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Reflection Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.Reflection.Emit;

namespace Nabu.Reflection
{
	public sealed class ILInstruction
	{
		#region Fields

		private int _offset;
		private int _length;
		private OpCode _opCode;
		private object[] _operands;

		#endregion

		#region Methods

		public ILInstruction(
			int offset,
			int length,
			OpCode opCode,
			params object[] parameters)
		{
			this._offset = offset;
			this._length = length;
			this._opCode = opCode;
			this._operands = parameters;
		}

		public ILInstruction(
			int offset,
			int length,
			OpCode opCode)
		{
			this._offset = offset;
			this._length = length;
			this._opCode = opCode;
		}

		#endregion

		#region Properties

		public int Offset
		{
			[DebuggerStepThrough]
			get { return this._offset; }
		}

		public int Length
		{
			[DebuggerStepThrough]
			get { return this._length; }
		}

		public OpCode OpCode
		{
			[DebuggerStepThrough]
			get { return this._opCode; }
		}

		[SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays")]
		public object[] Operands
		{
			[DebuggerStepThrough]
			get { return this._operands; }
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
