﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Reflection Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Reflection;
using System.Text;

namespace Nabu.Reflection
{
	public static class MemberDeclaration
	{
		#region Methods

		public static void GetTypeName(
			StringBuilder result,
			Type type,
			ProgrammingLanguage language)
		{
			string typeName = type.ToString();

			if (typeName.EndsWith("&", StringComparison.OrdinalIgnoreCase))
			{
				typeName = typeName.Substring(0, typeName.Length - 1);
			}

			switch (language)
			{
				case ProgrammingLanguage.CSharp:
					typeName = typeName
						.Replace("System.Void", "void")
						.Replace("System.Char", "char")
						.Replace("System.SByte", "sbyte")
						.Replace("System.Byte", "byte")
						.Replace("System.Int16", "short")
						.Replace("System.UInt16", "ushort")
						.Replace("System.Int32", "int")
						.Replace("System.UInt32", "uint")
						.Replace("System.Int64", "long")
						.Replace("System.UInt64", "ulong")
						.Replace("System.Single", "float")
						.Replace("System.Double", "double")
						.Replace(",", ", ");
					break;
				case ProgrammingLanguage.VBNet:
					typeName = typeName
						//.Replace("System.Void", "?")
						.Replace("System.Char", "Char")
						.Replace("System.SByte", "SByte")
						.Replace("System.Byte", "Byte")
						.Replace("System.Int16", "Short")
						.Replace("System.UInt16", "UShort")
						.Replace("System.Int32", "Integer")
						.Replace("System.UInt32", "UInteger")
						.Replace("System.Int64", "Long")
						.Replace("System.UInt64", "ULong")
						.Replace("System.Single", "Single")
						.Replace("System.Double", "Double")
						.Replace(".", ".")
						.Replace(",", ", ");
					break;
				case ProgrammingLanguage.CPlusPlus:
					typeName = typeName
						.Replace("System.Void", "void")
						.Replace("System.Char", "char")
						.Replace("System.SByte", "sbyte")
						.Replace("System.Byte", "byte")
						.Replace("System.Int16", "short")
						.Replace("System.UInt16", "ushort")
						.Replace("System.Int32", "int")
						.Replace("System.UInt32", "uint")
						.Replace("System.Int64", "long")
						.Replace("System.UInt64", "ulong")
						.Replace("System.Single", "float")
						.Replace("System.Double", "double")
						.Replace(".", "::")
						.Replace(",", ", ");
					break;
			}

			if (type.IsGenericType)
			{
				int indexFrom = typeName.IndexOf('`');

				switch (language)
				{
					case ProgrammingLanguage.CSharp:
						result.Append(typeName.Substring(0, indexFrom));
						result.Append("<");
						result.Append(typeName.Substring(indexFrom + 3, typeName.Length - indexFrom - 4));
						result.Append(">");
						break;
					case ProgrammingLanguage.VBNet:
						result.Append(typeName.Substring(0, indexFrom));
						result.Append("( Of");
						result.Append(typeName.Substring(indexFrom + 3, typeName.Length - indexFrom - 4));
						result.Append(")");
						break;
					case ProgrammingLanguage.CPlusPlus:
						result.Append(typeName.Substring(0, indexFrom));
						result.Append("<");
						result.Append(typeName.Substring(indexFrom + 3, typeName.Length - indexFrom - 4));
						result.Append(">");
						break;
				}

			}
			else
			{
				result.Append(typeName);
			}
		}

		private static void GetMethodAttributes(
			StringBuilder result,
			MethodBase method,
			ProgrammingLanguage language)
		{
			switch (language)
			{
				case ProgrammingLanguage.CSharp:
					if (method.IsAssembly) result.Append("internal ");
					if (method.IsPrivate) result.Append("private ");
					if (method.IsFamily) result.Append("protected ");
					if (method.IsPublic) result.Append("public ");
					if (method.IsFinal) result.Append("sealed ");
					if (method.IsAbstract) result.Append("abstract ");
					if (method.IsStatic) result.Append("static ");
					break;
				case ProgrammingLanguage.VBNet:
					if (method.IsAssembly) result.Append("Friend ");
					if (method.IsPrivate) result.Append("Private ");
					if (method.IsFamily) result.Append("Protected ");
					if (method.IsPublic) result.Append("Public ");
					if (method.IsFinal) result.Append("NotInheritable ");
					if (method.IsAbstract) result.Append("MustOverride ");
					if (method.IsStatic) result.Append("Shared ");
					break;
				case ProgrammingLanguage.CPlusPlus:
					if (method.IsAssembly) result.Append("internal ");
					if (method.IsPrivate) result.Append("private ");
					if (method.IsFamily) result.Append("protected ");
					if (method.IsPublic) result.Append("public ");
					if (method.IsFinal) result.Append("sealed ");
					if (method.IsAbstract) result.Append("abstract ");
					if (method.IsStatic) result.Append("static ");
					break;
			}
		}

		private static void GetMethodName(
			StringBuilder result,
			MethodBase method,
			ProgrammingLanguage language)
		{
			result.Append(method.Name);

			if (method.IsGenericMethod)
			{
				Type[] typeList = method.GetGenericArguments();

				switch (language)
				{
					case ProgrammingLanguage.CSharp:
						result.Append("<");

						GetTypeName(result, typeList[0], language);

						for (int index = 1; index < typeList.Length; index++)
						{
							result.Append(", ");
							GetTypeName(result, typeList[0], language);
						}

						result.Append(">");
						break;
					case ProgrammingLanguage.VBNet:
						result.Append("( Of ");

						GetTypeName(result, typeList[0], language);

						for (int index = 1; index < typeList.Length; index++)
						{
							result.Append(", ");
							GetTypeName(result, typeList[0], language);
						}

						result.Append(")");
						break;
					case ProgrammingLanguage.CPlusPlus:
						result.Append("<");

						GetTypeName(result, typeList[0], language);

						for (int index = 1; index < typeList.Length; index++)
						{
							result.Append(", ");
							GetTypeName(result, typeList[0], language);
						}

						result.Append(">");
						break;
				}
			}
		}

		private static void GetMethodParameterAttribute(
			StringBuilder result,
			ParameterInfo parameterInfo,
			ProgrammingLanguage language)
		{
			switch (language)
			{
				case ProgrammingLanguage.CSharp:
					if (parameterInfo.ParameterType.IsByRef)
					{
						if (parameterInfo.IsOut)
						{
							result.Append("out ");
						}
						else
						{
							result.Append("ref ");
						}
					}
					break;
				case ProgrammingLanguage.VBNet:
					if (parameterInfo.ParameterType.IsByRef)
					{
						if (parameterInfo.IsOut)
						{
							//result.Append("? ");
						}
						else
						{
							result.Append("ByRef ");
						}
					}
					else
					{
						result.Append("ByVal ");
					}
					break;
				case ProgrammingLanguage.CPlusPlus:
					if (parameterInfo.ParameterType.IsByRef)
					{
						if (parameterInfo.IsOut)
						{
							result.Append("out ");
						}
						else
						{
							result.Append("ref ");
						}
					}
					break;
			}
		}

		private static void GetMethodParameterList(
			StringBuilder result,
			MethodBase method,
			ProgrammingLanguage language)
		{
			ParameterInfo[] parameters = method.GetParameters();

			switch (language)
			{
				case ProgrammingLanguage.CSharp:
					result.Append("(");

					if (parameters.Length > 0)
					{
						GetMethodParameterAttribute(result, parameters[0], language);
						GetTypeName(result, parameters[0].ParameterType, language);
						result.AppendFormat(" {0}", parameters[0].Name);

						for (int index = 1; index < parameters.Length; index++)
						{
							result.Append(", ");
							GetMethodParameterAttribute(result, parameters[index], language);
							GetTypeName(result, parameters[index].ParameterType, language);
							result.AppendFormat(" {0}", parameters[index].Name);
						}
					}

					result.Append(")");
					break;
				case ProgrammingLanguage.VBNet:
					result.Append("(");

					if (parameters.Length > 0)
					{
						result.AppendFormat("{0} As ", parameters[0].Name);
						GetTypeName(result, parameters[0].ParameterType, language);
						GetMethodParameterAttribute(result, parameters[0], language);

						for (int index = 1; index < parameters.Length; index++)
						{
							result.Append(", ");
							result.AppendFormat("{0} As ", parameters[index].Name);
							GetTypeName(result, parameters[index].ParameterType, language);
							GetMethodParameterAttribute(result, parameters[index], language);
						}
					}

					result.Append(")");
					break;
				case ProgrammingLanguage.CPlusPlus:
					result.Append("(");

					if (parameters.Length > 0)
					{
						GetMethodParameterAttribute(result, parameters[0], language);
						GetTypeName(result, parameters[0].ParameterType, language);
						result.AppendFormat(" {0}", parameters[0].Name);

						for (int index = 1; index < parameters.Length; index++)
						{
							result.Append(", ");
							GetMethodParameterAttribute(result, parameters[index], language);
							GetTypeName(result, parameters[index].ParameterType, language);
							result.AppendFormat(" {0}", parameters[index].Name);
						}
					}

					result.Append(")");
					break;
			}
		}

		public static string GetDeclaration(MemberInfo memberInfo, ProgrammingLanguage language)
		{
			StringBuilder result = new StringBuilder();

			switch (memberInfo.MemberType)
			{
				case MemberTypes.Constructor:
					{
						ConstructorInfo constructorInfo = memberInfo as ConstructorInfo;

						switch (language)
						{
							case ProgrammingLanguage.CSharp:
								GetMethodAttributes(result, constructorInfo, language);
								result.Append("void ");
								GetTypeName(result, constructorInfo.DeclaringType, language);
								result.Append(".");
								GetMethodName(result, constructorInfo, language);
								GetMethodParameterList(result, constructorInfo, language);
								break;
							case ProgrammingLanguage.VBNet:
								break;
							case ProgrammingLanguage.CPlusPlus:
								break;
						}
					}
					break;
				case MemberTypes.Event:
					switch (language)
					{
						case ProgrammingLanguage.CSharp:
							break;
						case ProgrammingLanguage.VBNet:
							break;
						case ProgrammingLanguage.CPlusPlus:
							break;
					}
					break;
				case MemberTypes.Field:
					{
						FieldInfo field = memberInfo as FieldInfo;

						switch (language)
						{
							case ProgrammingLanguage.CSharp:
								GetTypeName(result, field.FieldType, language);
								result.Append(" ");
								GetTypeName(result, field.DeclaringType, language);
								result.AppendFormat(".{0}", field.Name);
								break;
							case ProgrammingLanguage.VBNet:
								break;
							case ProgrammingLanguage.CPlusPlus:
								break;
						}
					}
					break;
				case MemberTypes.Method:
					{
						MethodInfo method = memberInfo as MethodInfo;

						switch (language)
						{
							case ProgrammingLanguage.CSharp:
								if (method.ReturnType == typeof(void))
								{
									GetMethodAttributes(result, method, language);
									result.Append("void ");
									GetTypeName(result, method.DeclaringType, language);
									result.Append(".");
									GetMethodName(result, method, language);
									GetMethodParameterList(result, method, language);
								}
								else
								{
									GetMethodAttributes(result, method, language);
									GetTypeName(result, method.ReturnType, language);
									result.Append(" ");
									GetTypeName(result, method.DeclaringType, language);
									result.Append(".");
									GetMethodName(result, method, language);
									GetMethodParameterList(result, method, language);
								}
								break;
							case ProgrammingLanguage.VBNet:
								break;
							case ProgrammingLanguage.CPlusPlus:
								break;
						}
					}
					break;
				case MemberTypes.Property:
					switch (language)
					{
						case ProgrammingLanguage.CSharp:
							break;
						case ProgrammingLanguage.VBNet:
							break;
						case ProgrammingLanguage.CPlusPlus:
							break;
					}
					break;
				case MemberTypes.TypeInfo:
					switch (language)
					{
						case ProgrammingLanguage.CSharp:
							break;
						case ProgrammingLanguage.VBNet:
							break;
						case ProgrammingLanguage.CPlusPlus:
							break;
					}
					break;
				case MemberTypes.Custom:
					switch (language)
					{
						case ProgrammingLanguage.CSharp:
							break;
						case ProgrammingLanguage.VBNet:
							break;
						case ProgrammingLanguage.CPlusPlus:
							break;
					}
					break;
				case MemberTypes.NestedType:
					switch (language)
					{
						case ProgrammingLanguage.CSharp:
							break;
						case ProgrammingLanguage.VBNet:
							break;
						case ProgrammingLanguage.CPlusPlus:
							break;
					}
					break;
			}

			return result.ToString();
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
