﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Reflection Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Diagnostics;
#if !COMPACT_FRAMEWORK
using System.Runtime.Serialization;
#endif

namespace Nabu.Reflection
{
#if !COMPACT_FRAMEWORK
	[Serializable]
#endif
	public sealed class ScriptEngineCompilerException : Exception
	{
		#region ScriptEngineCompilerException

		#region Fields

		private ScriptEngineCompilerMessage[] _messages;

		#endregion

		#region Methods

		internal ScriptEngineCompilerException(ScriptEngineCompilerMessage[] messages)
		{
			this._messages = messages;
		}

		public ScriptEngineCompilerException()
		{
		}

		public ScriptEngineCompilerException(string message)
			: base(message)
		{
		}

		public ScriptEngineCompilerException(
			string message,
			Exception innerException)
			: base(message, innerException)
		{
		}

		#if !COMPACT_FRAMEWORK
		private ScriptEngineCompilerException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			this._messages = (ScriptEngineCompilerMessage[])info.GetValue("_messages", typeof(ScriptEngineCompilerMessage[]));
		}
		#endif

		#endregion

		#region Properties

		public IEnumerable<ScriptEngineCompilerMessage> Messages
		{
			[DebuggerStepThrough]
			get { return this._messages; }
		}

		#endregion

		#endregion

#if !COMPACT_FRAMEWORK
		#region ISerializable

		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			if (info == null)
			{
				throw new ArgumentNullException("info");
			}

			info.AddValue("_messages", this._messages, typeof(ScriptEngineCompilerMessage[]));

			base.GetObjectData(info, context);
		}

		#endregion
#endif
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
