﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Reflection Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Globalization;

using Nabu.Text;

namespace Nabu.Reflection
{
	public abstract class ManagedScriptExecutionHostBase :
		DisposableBase,
		IScriptExecutionHost
	{
		#region ManagedScriptExecutionHostBase

		#region Fields

		private AppDomain _appDomain;

		#endregion

		#region Methods

		protected ManagedScriptExecutionHostBase()
		{
		}

		protected abstract AppDomain CreateDomain();

		#endregion

		#region Properties

		public AppDomain AppDomain
		{
			get
			{
				if (this._appDomain == null)
				{
					this._appDomain = CreateDomain();
				}

				return this._appDomain;
			}
		}

		#endregion

		#endregion

		#region IScriptExecutionHost

		public IScriptExecutionEngine CreateEngine(
			IEnumerable<ISourceCodeUnit> sourceCodeUnits)
		{
			string language = null;

			foreach (ISourceCodeUnit sourceCodeUnit in sourceCodeUnits)
			{
				language = sourceCodeUnit.Language;
				break;
			}

			switch (language.ToUpper(CultureInfo.InvariantCulture))
			{
				case "C#":
				case "C#v2.0":
				case "C#v3.0":
				case "C#v3.5":
					return
						new ManagedScriptExecutionEngine(
							this,
							sourceCodeUnits);
				default:
					throw new NotImplementedException(language);
			}
		}

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (
					(this._appDomain != null) &&
					(this._appDomain != AppDomain.CurrentDomain))
				{
					AppDomain.Unload(this._appDomain);
					this._appDomain = null;
				}
			}

			base.Dispose(disposing);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
