﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Reflection Library                       */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Net;
using System.Security;
using System.Security.Permissions;
using System.Security.Policy;

namespace Nabu.Reflection
{
	public sealed class ManagedScriptSecureExecutionHost : ManagedScriptExecutionHostBase
	{
		#region ManagedScriptSecureExecutionHost

		#region Fields

		private string _applicationBase;

		#endregion

		#region Methods

		public ManagedScriptSecureExecutionHost()
		{
			this._applicationBase = AppDomain.CurrentDomain.SetupInformation.ApplicationBase;
		}

		public ManagedScriptSecureExecutionHost(string applicationBase)
		{
			this._applicationBase = applicationBase;
		}

		#endregion

		#endregion

		#region ManagedScriptExecutionHostBase

		protected override AppDomain CreateDomain()
		{
			AppDomainSetup setup = new AppDomainSetup();

			setup.ApplicationBase = this._applicationBase;

			AppDomain appDomain =
				AppDomain.CreateDomain(
					this.GetType().Name,
					null,
					setup);

			PermissionSet permissionSet = new PermissionSet(PermissionState.None);

			permissionSet.AddPermission(new DnsPermission(PermissionState.None));
			permissionSet.AddPermission(new EnvironmentPermission(PermissionState.None));
			permissionSet.AddPermission(new FileDialogPermission(PermissionState.None));
			permissionSet.AddPermission(new FileIOPermission(PermissionState.None));
			permissionSet.AddPermission(new ReflectionPermission(ReflectionPermissionFlag.NoFlags));
			permissionSet.AddPermission(new RegistryPermission(PermissionState.None));
			permissionSet.AddPermission(new SecurityPermission(SecurityPermissionFlag.Execution));
			permissionSet.AddPermission(new SocketPermission(PermissionState.None));
			permissionSet.AddPermission(new UIPermission(UIPermissionWindow.NoWindows));
			permissionSet.AddPermission(new WebPermission(PermissionState.None));

			CodeGroup rootGroup =
				new UnionCodeGroup(
					new AllMembershipCondition(),
					new PolicyStatement(
						permissionSet,
						PolicyStatementAttribute.LevelFinal));

			PolicyLevel policyLevel = PolicyLevel.CreateAppDomainLevel();

			policyLevel.RootCodeGroup = rootGroup;

			appDomain.SetAppDomainPolicy(policyLevel);

			return appDomain;
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
