﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Text Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Diagnostics;
using System.Globalization;

namespace Nabu.Text
{
	[System.Diagnostics.DebuggerDisplay("Kind: {Kind}; Value: {Value};")]
	public sealed class CodeLexicalToken
	{
		#region CodeLexicalToken

		#region Fields

		private long _positionFrom;
		private long _positionTo;
		private string _kind;
		private string _value;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public CodeLexicalToken(
			long positionFrom,
			long positionTo,
			string kind)
		{
			this._positionFrom = positionFrom;
			this._positionTo = positionTo;
			this._kind = kind;
		}

		[DebuggerStepThrough]
		public CodeLexicalToken(
			long positionFrom,
			long positionTo,
			string kind,
			string value)
		{
			this._positionFrom = positionFrom;
			this._positionTo = positionTo;
			this._kind = kind;
			this._value = value;
		}

		#endregion

		#region Properties

		public long StartPosition
		{
			[DebuggerStepThrough]
			get { return this._positionFrom; }
			[DebuggerStepThrough]
			set { this._positionFrom = value; }
		}

		public long EndPosition
		{
			[DebuggerStepThrough]
			get { return this._positionTo; }
			[DebuggerStepThrough]
			set { this._positionTo = value; }
		}

		public string Kind
		{
			[DebuggerStepThrough]
			get { return this._kind; }
			[DebuggerStepThrough]
			set { this._kind = value; }
		}

		public string Value
		{
			[DebuggerStepThrough]
			get { return this._value; }
			[DebuggerStepThrough]
			set { this._value = value; }
		}

		#endregion

		#endregion

		#region Object

		[DebuggerStepThrough]
		public override string ToString()
		{
			return
				string.Format(
					CultureInfo.InvariantCulture,
					"Kind: {0}; Value: {1};",
					this._kind,
					this._value == null ? "null" : this._value);
		}

		#endregion
	};
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
