﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Text Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;

using Nabu.Collections;

namespace Nabu.Text
{
	[System.Diagnostics.DebuggerDisplay("Type: {Type}; Value: {Value};")]
	public sealed class CodeSyntacticToken
	{
		#region CodeSyntacticToken

		#region Fields

		private ListEx<CodeLexicalToken> _lexicalTokens;
		private int _version;
		private string _type;
		private string _value;
		private object _data;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public CodeSyntacticToken(
			string type)
		{
			this._lexicalTokens = new ListEx<CodeLexicalToken>(4);
			this._type = type;
		}

		[DebuggerStepThrough]
		public CodeSyntacticToken(
			int version,
			string type,
			string value)
		{
			this._lexicalTokens = new ListEx<CodeLexicalToken>(4);
			this._version = version;
			this._type = type;
			this._value = value;
		}

		[DebuggerStepThrough]
		public void AppendLexicalTokens(IEnumerable<CodeLexicalToken> collection)
		{
			this._lexicalTokens.AddRange(collection);
		}

		#endregion

		#region Properties

		public IEnumerable<CodeLexicalToken> LexicalTokens
		{
			[DebuggerStepThrough]
			get { return this._lexicalTokens; }
		}

		public long StartPosition
		{
			[DebuggerStepThrough]
			get { return this._lexicalTokens.First.StartPosition; }
		}

		public long EndPosition
		{
			[DebuggerStepThrough]
			get { return this._lexicalTokens.Last.EndPosition; }
		}

		public int Version
		{
			[DebuggerStepThrough]
			get { return this._version; }
		}

		public string Type
		{
			[DebuggerStepThrough]
			get { return this._type; }
			[DebuggerStepThrough]
			set { this._type = value; }
		}

		public string Value
		{
			[DebuggerStepThrough]
			get { return this._value; }
			[DebuggerStepThrough]
			set { this._value = value; }
		}

		public object Data
		{
			[DebuggerStepThrough]
			get { return this._data; }
			[DebuggerStepThrough]
			set { this._data = value; }
		}

		#endregion

		#endregion

		#region Object

		public override string ToString()
		{
			return
				string.Format(
					CultureInfo.InvariantCulture,
					"Type: {0}; Value: {1};",
					this._type,
					this._value);
		}

		#endregion
	};
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
