﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Text Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Runtime.Serialization;

namespace Nabu.Text
{
	[Serializable]
	public sealed class CodeSyntaxException : Exception
	{
		#region CodeSyntaxException

		#region Fields

		private CodeLexicalToken _unexpectedToken;
		private KeyValuePair<string, string>[] _expectedTokens;
		private CodeSyntacticToken[] _unclosedElements;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		internal CodeSyntaxException(
			CodeLexicalToken unexpectedToken,
			CodeSyntacticToken[] unclosedElements,
			params KeyValuePair<string, string>[] expectedTokens)
			: base(
				string.Format(
					CultureInfo.InvariantCulture,
					"{0} '{1}' unexpected.",
					unexpectedToken.Kind,
					unexpectedToken.Value))
		{
			this._unexpectedToken = unexpectedToken;
			this._expectedTokens = (expectedTokens == null) ? new KeyValuePair<string, string>[0] : expectedTokens;
			this._unclosedElements = (unclosedElements == null) ? new CodeSyntacticToken[0] : unclosedElements;
		}

		[DebuggerStepThrough]
		internal CodeSyntaxException(
			CodeLexicalToken unexpectedToken,
			params KeyValuePair<string, string>[] expectedTokens)
			: this(unexpectedToken, null, expectedTokens)
		{
		}

		[DebuggerStepThrough]
		public CodeSyntaxException()
		{
		}

		[DebuggerStepThrough]
		public CodeSyntaxException(string message)
			: base(message)
		{
		}

		[DebuggerStepThrough]
		public CodeSyntaxException(
			string message,
			Exception innerException)
			: base(message, innerException)
		{
		}

		[DebuggerStepThrough]
		private CodeSyntaxException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			this._unexpectedToken = (CodeLexicalToken)info.GetValue("_unexpectedToken", typeof(CodeLexicalToken));
			this._expectedTokens = (KeyValuePair<string, string>[])info.GetValue("_expectedTokens", typeof(KeyValuePair<string, string>[]));
			this._unclosedElements = (CodeSyntacticToken[])info.GetValue("_unclosedElements", typeof(CodeSyntacticToken[]));
		}

		#endregion

		#region Properties

		public CodeLexicalToken UnexpectedToken
		{
			[DebuggerStepThrough]
			get { return this._unexpectedToken; }
		}

		public KeyValuePair<string, string>[] ExpectedTokens
		{
			[DebuggerStepThrough]
			get { return this._expectedTokens; }
		}

		public CodeSyntacticToken[] UnclosedElements
		{
			[DebuggerStepThrough]
			get { return this._unclosedElements; }
		}

		#endregion

		#endregion

		#region ISerializable

		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			if (info == null)
			{
				throw new ArgumentNullException("info");
			}

			info.AddValue("_unexpectedToken", this._unexpectedToken, typeof(CodeLexicalToken));
			info.AddValue("_expectedTokens", this._expectedTokens, typeof(KeyValuePair<string, string>[]));
			info.AddValue("_unclosedElements", this._unclosedElements, typeof(CodeSyntacticToken[]));

			base.GetObjectData(info, context);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
