﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Text Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Collections.Generic;
using System.Globalization;
using System.IO;

namespace Nabu.Text
{
	public sealed class FileTemplateAccessor : ITemplateAccessor
	{
		#region FileTemplateAccessor

		#region Fields

		private string[] _basePaths;

		#endregion

		#region Methods

		public FileTemplateAccessor(params string[] basePaths)
		{
			this._basePaths = basePaths;
		}

		#endregion

		#endregion

		#region ITemplateAccessor

		public string[] GetContent(string templateName)
		{
			foreach (string basePath in this._basePaths)
			{
				string templatePath = basePath + templateName;

				if (File.Exists(templatePath))
				{
#if COMPACT_FRAMEWORK
					using(TextReader templateReader = File.OpenText(templatePath))
					{
						List<string> lines = new List<string>();

						for (
							string line = templateReader.ReadLine();
							line != null;
							line = templateReader.ReadLine())
						{
							lines.Add(line);
						}

						return lines.ToArray();
					}
#else
					return File.ReadAllLines(templatePath);
#endif
				}
			}

			throw new FileNotFoundException(
				string.Format(
					CultureInfo.InvariantCulture,
					"Template file '{0}' not found",
					templateName));
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
