﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Text Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
#if !COMPACT_FRAMEWORK
using System.Runtime.Serialization;
#endif

namespace Nabu.Text
{
#if !COMPACT_FRAMEWORK
	[Serializable]
#endif
	public sealed class TemplateParserException : Exception
	{
		#region TemplateParserException

		#region Fields

		private TemplateParserErrorType _errorType;
		private TextPosition _position;
		private string _symbols;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		internal TemplateParserException(
			TemplateParserErrorType errorType,
			TextPosition position,
			string symbols)
			: base(position.ToString())
		{
			this._errorType = errorType;
			this._position = position;
			this._symbols = symbols;
		}

		[DebuggerStepThrough]
		public TemplateParserException()
		{
		}

		[DebuggerStepThrough]
		public TemplateParserException(string message)
			: base(message)
		{
		}

		[DebuggerStepThrough]
		public TemplateParserException(
			string message,
			Exception innerException)
			: base(message, innerException)
		{
		}

#if !COMPACT_FRAMEWORK
		[DebuggerStepThrough]
		private TemplateParserException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			this._errorType = (TemplateParserErrorType)info.GetValue("_errorType", typeof(TemplateParserErrorType));
			this._position = (TextPosition)info.GetValue("_position", typeof(TextPosition));
			this._symbols = info.GetString("_symbols");
		}
#endif

		#endregion

		#region Properties

		public TemplateParserErrorType ErrorType
		{
			[DebuggerStepThrough]
			get { return this._errorType; }
		}

		public TextPosition Position
		{
			[DebuggerStepThrough]
			get { return this._position; }
		}

		public string Symbols
		{
			[DebuggerStepThrough]
			get { return this._symbols; }
		}

		#endregion

		#endregion

#if !COMPACT_FRAMEWORK
		#region ISerializable

		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			if (info == null)
			{
				throw new ArgumentNullException("info");
			}

			info.AddValue("_errorType", this._errorType, typeof(TemplateParserErrorType));
			info.AddValue("_position", this._position, typeof(TextPosition));
			info.AddValue("_symbols", this._symbols);

			base.GetObjectData(info, context);
		}

		#endregion
#endif
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
