﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Text Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Diagnostics;
using System.Globalization;

namespace Nabu.Text
{
#if !COMPACT_FRAMEWORK
	[System.Diagnostics.DebuggerDisplay("{File}({Line},{Column})")]
#endif
	public class TextPosition
	{
		#region TextPosition

		#region Fields

		private string _file;
		private int _line;
		private int _column;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public TextPosition(string file, int line, int column)
		{
			this._file = file;
			this._line = line;
			this._column = column;
		}

		#endregion

		#region Properties

		public string File
		{
			[DebuggerStepThrough]
			get { return this._file; }
		}

		public int Line
		{
			[DebuggerStepThrough]
			get { return this._line; }
		}

		public int Column
		{
			[DebuggerStepThrough]
			get { return this._column; }
		}

		#endregion

		#endregion

		#region Object

		public static bool operator ==(TextPosition position1, TextPosition position2)
		{
			return
				(position1.File == position2.File) &&
				(position1.Line == position2.Line) &&
				(position1.Column == position2.Column);
		}

		public static bool operator !=(TextPosition position1, TextPosition position2)
		{
			return
				(position1.File != position2.File) ||
				(position1.Line != position2.Line) ||
				(position1.Column != position2.Column);
		}

		public override bool Equals(object obj)
		{
			TextPosition position = (TextPosition)obj;

			return
				(this._file == position.File) &&
				(this._line == position.Line) &&
				(this._column == position.Column);
		}

		public override int GetHashCode()
		{
			return this._file.GetHashCode() ^ this._line ^ (this._column << 16);
		}

		public override string ToString()
		{
			return
				string.Format(
					CultureInfo.InvariantCulture,
					"{0}({1},{2})",
					this._file,
					this._line,
					this._column);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
