﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Threading Library                        */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Threading;

namespace Nabu.Threading
{
	public sealed class ReverseSemaphore : DisposableBase
	{
		#region ReverseSemaphore

		#region Variable

		private ManualResetEvent _event = new ManualResetEvent(true);
		private int _counter;

		#endregion

		#region Methods

		public ReverseSemaphore()
		{
		}

		public void Increment()
		{
			base.CheckDisposed();

			lock (this)
			{
				this._counter++;

				if (this._counter == 1)
				{
					this._event.Reset();
				}
			}
		}

		public void Decrement()
		{
			base.CheckDisposed();

			lock (this)
			{
				this._counter--;

				if (this._counter == 0)
				{
					this._event.Set();
				}
			}
		}

		public bool WaitOne()
		{
			base.CheckDisposed();

			return this._event.WaitOne();
		}

		public bool WaitOne(int timeout, bool exitContext)
		{
			base.CheckDisposed();

			return this._event.WaitOne(timeout, exitContext);
		}

#if !COMPACT_FRAMEWORK
		public bool WaitOne(TimeSpan timeout, bool exitContext)
		{
			base.CheckDisposed();

			return this._event.WaitOne(timeout, exitContext);
		}
#endif

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
			if (this._event != null)
			{
				this._event.Close();
				this._event = null;
			}

			base.Dispose(disposing);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
