﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

using Nabu.Collections;

using NUnit.Framework;

namespace Nabu.UnitTest.nUnit.Nabu.Collections
{
	[TestFixture]
	public sealed class BindingListExTest : BindingListExTestBase
	{
		[Test]
		public void Add()
		{
			BindingListEx<int> list = new BindingListEx<int>();
			BindingListExEventSink<int> sink = new BindingListExEventSink<int>(list);

			for (int index = 0; index < 100; index++)
			{
				list.Add(0);

				TestAdd<int>(sink, index);
				TestNoRemove<int>(sink);
				TestNoChange<int>(sink);
				TestNoMove<int>(sink);
				TestNoReset<int>(sink);
			}
		}

		[Test]
		public void Remove()
		{
			BindingListEx<int> list = new BindingListEx<int>();

			for (int index = 0; index < 100; index++)
			{
				list.Add(0);
			}

			BindingListExEventSink<int> sink = new BindingListExEventSink<int>(list);

			for (int index = 99; index >= 0; index--)
			{
				list.RemoveAt(index);

				TestRemove<int>(sink, index);
				TestNoAdd<int>(sink);
				TestNoChange<int>(sink);
				TestNoMove<int>(sink);
				TestNoReset<int>(sink);
			}
		}

		[Test]
		public void Change()
		{
			BindingListEx<int> list = new BindingListEx<int>();

			for (int index = 0; index < 100; index++)
			{
				list.Add(0);
			}

			BindingListExEventSink<int> sink = new BindingListExEventSink<int>(list);

			for (int index = 0; index < 100; index++)
			{
				list[index] = index;

				TestChange<int>(sink, index);
				TestNoAdd<int>(sink);
				TestNoRemove<int>(sink);
				TestNoMove<int>(sink);
				TestNoReset<int>(sink);
			}
		}

		[Test]
		public void Swap()
		{
			BindingListEx<int> list = new BindingListEx<int>();

			for (int index = 0; index < 100; index++)
			{
				list.Add(0);
			}

			BindingListExEventSink<int> sink = new BindingListExEventSink<int>(list);

			for (int index1 = 0; index1 < 50; index1++)
			{
				for (int index2 = 50; index2 < 100; index2++)
				{
					list.Swap(index1, index2);

					TestMove<int>(sink, index1, index2);
					TestNoAdd<int>(sink);
					TestNoRemove<int>(sink);
					TestNoChange<int>(sink);
					TestNoReset<int>(sink);
				}
			}
		}

		[Test]
		public void ChangeItem()
		{
			BindingListEx<BindingListExItem> list = new BindingListEx<BindingListExItem>();

			list.Add(new BindingListExItem());

			BindingListExEventSink<BindingListExItem> sink = new BindingListExEventSink<BindingListExItem>(list);

			list[0].ChangeProperty();

			Assert.IsNotNull(sink.ListChanged);
			Assert.IsNotNull(sink.BeforeChange);
			Assert.IsNotNull(sink.AfterChange);
			Assert.IsNull(sink.BeforeAdd);
			Assert.IsNull(sink.AfterAdd);
			Assert.IsNull(sink.BeforeRemove);
			Assert.IsNull(sink.AfterRemove);
			Assert.IsNull(sink.BeforeMove);
			Assert.IsNull(sink.AfterMove);
			Assert.IsNull(sink.BeforeReset);
			Assert.IsNull(sink.AfterReset);
		}

		[Test]
		public void Reset()
		{
			BindingListEx<int> list = new BindingListEx<int>();
			BindingListExEventSink<int> sink = new BindingListExEventSink<int>(list);

			list.FromArray(new int[] { 0, 1 }, true);

			TestReset<int>(sink);
			TestNoAdd<int>(sink);
			TestNoRemove<int>(sink);
			TestNoChange<int>(sink);
			TestNoMove<int>(sink);
		}

		[Test]
		public void Sort()
		{
			Random random = new Random();

			for (int step = 0; step < 100; step++)
			{
				BindingListEx<int> list = new BindingListEx<int>();

				for (int index = 0; index < 50 + random.Next(50); index++)
				{
					list.Add(random.Next(10));
				}

				list.Sort(Comparer<int>.Default);

				for (int index = 0; index < list.Count - 1; index++)
				{
					Assert.LessOrEqual(list[index], list[index + 1]);
				}
			}
		}
	}

	public class BindingListExItem :
		INotifyPropertyChanging,
		INotifyPropertyChanged
	{
		#region BindingListExItem

		public BindingListExItem()
		{
		}

		public void ChangeProperty()
		{
			if (PropertyChanging != null)
			{
				PropertyChanging(this, new PropertyChangingEventArgs("Test"));
			}

			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs("Test"));
			}
		}

		#endregion

		#region INotifyPropertyChanging

		public event PropertyChangingEventHandler PropertyChanging;

		#endregion

		#region INotifyPropertyChanged

		public event PropertyChangedEventHandler PropertyChanged;

		#endregion
	}
}
