﻿using Nabu.Collections;
using NUnit.Framework;

namespace Nabu.UnitTest.nUnit.Nabu.Collections
{
	[TestFixture]
	public sealed class TreeTest : TreeTestBase
	{
		#region Utility

		/*
                                 0
                              /  |  \
                          /      |      \
                      /          |          \
                  /              |              \
              /                  |                 \
          /                      |                      \
         1                       8                      15
      /     \                 /     \                 /    \
     /       \               /       \               /      \
    /         \             /         \             /        \
   2           5-C         9-CH       12          16          19
  / \         / \         / \         / \        /  \        / \
 /   \       /   \       /   \       /   \      /    \      /   \
3     4     6     7    10    11    13    14    17    18    20    21

Index             0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 21
All Preorder      0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 21
All Postorder     3  4  2  6  7  5  1 10 11  9 13 14 12  8 17 18 16 20 21 19 15  0
Visible Preorder  0  1  2  3  4  5  8 12 13 14 15 16 17 18 19 20 21
Visible Postorder 3  4  2  5  1 13 14 12  8 17 18 16 20 21 19 15  0
*/
		public ITree<int> BuildTree()
		{
			ITree<int> tree = new BindingTree<int>();

			ITreeNode<int> root = tree.Root;
			ITreeNode<int> node1 = root.AppendChild(1, false);
			ITreeNode<int> node2 = node1.AppendChild(2, false);
			ITreeNode<int> node3 = node2.AppendChild(3, false);
			ITreeNode<int> node4 = node2.AppendChild(4, false);
			ITreeNode<int> node5 = node1.AppendChild(5, true);
			ITreeNode<int> node6 = node5.AppendChild(6, false);
			ITreeNode<int> node7 = node5.AppendChild(7, false);
			ITreeNode<int> node8 = root.AppendChild(8, false);
			ITreeNode<int> node9 = node8.AppendChild(9, true);
			ITreeNode<int> node10 = node9.AppendChild(10, false);
			ITreeNode<int> node11 = node9.AppendChild(11, false);
			ITreeNode<int> node12 = node8.AppendChild(12, false);
			ITreeNode<int> node13 = node12.AppendChild(13, false);
			ITreeNode<int> node14 = node12.AppendChild(14, false);
			ITreeNode<int> node15 = root.AppendChild(15, false);
			ITreeNode<int> node16 = node15.AppendChild(16, false);
			ITreeNode<int> node17 = node16.AppendChild(17, false);
			ITreeNode<int> node18 = node16.AppendChild(18, false);
			ITreeNode<int> node19 = node15.AppendChild(19, false);
			ITreeNode<int> node20 = node19.AppendChild(20, false);
			ITreeNode<int> node21 = node19.AppendChild(21, false);

			node9.IsVisible = false;

			return tree;
		}

		#endregion

		#region Test

		[Test]
		public void AddRemove()
		{
			ITree<int> tree = new BindingTree<int>();
			ITreeNode<int> root = tree.Root;

			root.AppendChild(6, false);
			CheckNodeIntegrity(root);
			root.AppendChild(8, false);
			CheckNodeIntegrity(root);
			root.PrependChild(5, false);
			CheckNodeIntegrity(root);
			root.PrependChild(3, false);
			CheckNodeIntegrity(root);

			root.FirstChild.AppendSibling(4, false);
			CheckNodeIntegrity(root);
			root.LastChild.PrependSibling(7, false);
			CheckNodeIntegrity(root);

			root.FirstChild.PrependSibling(2, false);
			CheckNodeIntegrity(root);
			root.LastChild.AppendSibling(9, false);
			CheckNodeIntegrity(root);

			root.FirstChild.Remove();
			CheckNodeIntegrity(root);
			root.LastChild.Remove();
			CheckNodeIntegrity(root);
			root.FirstChild.NextSibling.Remove();
			CheckNodeIntegrity(root);
			root.LastChild.PrevSibling.Remove();
			CheckNodeIntegrity(root);
		}

		[Test]
		public void GetCount()
		{
			ITree<int> tree = BuildTree();

			Assert.AreEqual(22, tree.Root.Count);
			Assert.AreEqual(17, tree.Root.VisibleCount);
		}

		[Test]
		public void Preorder()
		{
			ITree<int> tree = BuildTree();

			Assert.AreEqual("1;2;3;4;5;6;7;8;9;10;11;12;", PreorderString(tree, 1, 12, false));
			Assert.AreEqual("2;3;4;5;6;7;8;9;10;11;12;", PreorderString(tree, 2, 11, false));
			Assert.AreEqual("3;4;5;6;7;8;9;10;11;12;", PreorderString(tree, 3, 10, false));
			Assert.AreEqual("4;5;6;7;8;9;10;11;12;", PreorderString(tree, 4, 9, false));
			Assert.AreEqual("5;6;7;8;9;10;11;12;", PreorderString(tree, 5, 8, false));

			Assert.AreEqual("1;2;3;4;5;6;7;8;9;10;", PreorderString(tree, 1, 10, false));
			Assert.AreEqual("2;3;4;5;6;7;8;9;10;", PreorderString(tree, 2, 9, false));
			Assert.AreEqual("3;4;5;6;7;8;9;10;", PreorderString(tree, 3, 8, false));
			Assert.AreEqual("4;5;6;7;8;9;10;", PreorderString(tree, 4, 7, false));
			Assert.AreEqual("5;6;7;8;9;10;", PreorderString(tree, 5, 6, false));

			Assert.AreEqual("1;2;3;4;5;6;7;8;9;10;11;12;", PreorderString(tree, 1, 12, false));
			Assert.AreEqual("1;2;3;4;5;6;7;8;9;10;11;", PreorderString(tree, 1, 11, false));
			Assert.AreEqual("1;2;3;4;5;6;7;8;9;10;", PreorderString(tree, 1, 10, false));
			Assert.AreEqual("1;2;3;4;5;6;7;8;9;", PreorderString(tree, 1, 9, false));
			Assert.AreEqual("1;2;3;4;5;6;7;8;", PreorderString(tree, 1, 8, false));

			Assert.AreEqual("12;13;14;15;16;17;18;", PreorderString(tree, 7, 7, true));
			Assert.AreEqual("13;14;15;16;17;18;", PreorderString(tree, 8, 6, true));
			Assert.AreEqual("14;15;16;17;18;", PreorderString(tree, 9, 5, true));
			Assert.AreEqual("15;16;17;18;", PreorderString(tree, 10, 4, true));
			Assert.AreEqual("16;17;18;", PreorderString(tree, 11, 3, true));

			Assert.AreEqual("12;13;14;15;16;", PreorderString(tree, 7, 5, true));
			Assert.AreEqual("13;14;15;16;", PreorderString(tree, 8, 4, true));
			Assert.AreEqual("14;15;16;", PreorderString(tree, 9, 3, true));
			Assert.AreEqual("15;16;", PreorderString(tree, 10, 2, true));
			Assert.AreEqual("16;", PreorderString(tree, 11, 1, true));

			Assert.AreEqual("12;13;14;15;16;", PreorderString(tree, 7, 5, true));
			Assert.AreEqual("12;13;14;15;", PreorderString(tree, 7, 4, true));
			Assert.AreEqual("12;13;14;", PreorderString(tree, 7, 3, true));
			Assert.AreEqual("12;13;", PreorderString(tree, 7, 2, true));
			Assert.AreEqual("12;", PreorderString(tree, 7, 1, true));
		}

		[Test]
		public void Postorder()
		{
			ITree<int> tree = BuildTree();

			Assert.AreEqual("4;2;6;7;5;1;10;11;9;13;14;12;", PostorderString(tree, 1, 12, false));
			Assert.AreEqual("2;6;7;5;1;10;11;9;13;14;12;", PostorderString(tree, 2, 11, false));
			Assert.AreEqual("6;7;5;1;10;11;9;13;14;12;", PostorderString(tree, 3, 10, false));
			Assert.AreEqual("7;5;1;10;11;9;13;14;12;", PostorderString(tree, 4, 9, false));
			Assert.AreEqual("5;1;10;11;9;13;14;12;", PostorderString(tree, 5, 8, false));

			Assert.AreEqual("4;2;6;7;5;1;10;11;9;13;", PostorderString(tree, 1, 10, false));
			Assert.AreEqual("2;6;7;5;1;10;11;9;13;", PostorderString(tree, 2, 9, false));
			Assert.AreEqual("6;7;5;1;10;11;9;13;", PostorderString(tree, 3, 8, false));
			Assert.AreEqual("7;5;1;10;11;9;13;", PostorderString(tree, 4, 7, false));
			Assert.AreEqual("5;1;10;11;9;13;", PostorderString(tree, 5, 6, false));

			Assert.AreEqual("4;2;6;7;5;1;10;11;9;13;14;12;", PostorderString(tree, 1, 12, false));
			Assert.AreEqual("4;2;6;7;5;1;10;11;9;13;14;", PostorderString(tree, 1, 11, false));
			Assert.AreEqual("4;2;6;7;5;1;10;11;9;13;", PostorderString(tree, 1, 10, false));
			Assert.AreEqual("4;2;6;7;5;1;10;11;9;", PostorderString(tree, 1, 9, false));
			Assert.AreEqual("4;2;6;7;5;1;10;11;", PostorderString(tree, 1, 8, false));

			Assert.AreEqual("13;14;12;8;17;18;16;", PostorderString(tree, 5, 7, true));
			Assert.AreEqual("14;12;8;17;18;16;", PostorderString(tree, 6, 6, true));
			Assert.AreEqual("12;8;17;18;16;", PostorderString(tree, 7, 5, true));
			Assert.AreEqual("8;17;18;16;", PostorderString(tree, 8, 4, true));
			Assert.AreEqual("17;18;16;", PostorderString(tree, 9, 3, true));

			Assert.AreEqual("13;14;12;8;17;", PostorderString(tree, 5, 5, true));
			Assert.AreEqual("14;12;8;17;", PostorderString(tree, 6, 4, true));
			Assert.AreEqual("12;8;17;", PostorderString(tree, 7, 3, true));
			Assert.AreEqual("8;17;", PostorderString(tree, 8, 2, true));
			Assert.AreEqual("17;", PostorderString(tree, 9, 1, true));

			Assert.AreEqual("13;14;12;8;17;", PostorderString(tree, 5, 5, true));
			Assert.AreEqual("13;14;12;8;", PostorderString(tree, 5, 4, true));
			Assert.AreEqual("13;14;12;", PostorderString(tree, 5, 3, true));
			Assert.AreEqual("13;14;", PostorderString(tree, 5, 2, true));
			Assert.AreEqual("13;", PostorderString(tree, 5, 1, true));
		}

		#endregion
	}
}
