﻿using System.Collections.Generic;
using System.Text;

using Nabu.Collections;

using NUnit.Framework;

namespace Nabu.UnitTest.nUnit.Nabu.Collections
{
	public abstract class TreeTestBase
	{
		protected void CheckNodeIntegrity<T>(ITreeNode<T> node)
		{
			if (node.PrevSibling != null)
			{
				Assert.AreEqual(node, node.PrevSibling.NextSibling);
			}
			else
			{
				if (node.Parent != null)
				{
					Assert.AreEqual(node, node.Parent.FirstChild);
				}
			}

			if (node.NextSibling != null)
			{
				Assert.AreEqual(node, node.NextSibling.PrevSibling);
			}
			else
			{
				if (node.Parent != null)
				{
					Assert.AreEqual(node, node.Parent.LastChild);
				}
			}

			if (node.FirstChild != null)
			{
				int totalCount = 1;
				int visibleCount = 1;

				foreach (ITreeNode<T> child in node.Children)
				{
					Assert.AreEqual(node, child.Parent);
					CheckNodeIntegrity(child);

					totalCount += child.Count;
					visibleCount += child.GetCount(true);
				}

				Assert.AreEqual(totalCount, node.Count);
				Assert.AreEqual(visibleCount, node.VisibleCount);
			}
		}

		protected string PreorderString(ITree<int> tree, int from, int count, bool visibleNodes)
		{
			StringBuilder stringBuilder = new StringBuilder();

			foreach (KeyValuePair<ITreeNode<int>, int> node in tree.Preorder(from, count, visibleNodes))
			{
				stringBuilder.AppendFormat("{0};", node.Key.Data);
			}

			return stringBuilder.ToString();
		}

		protected string PostorderString(ITree<int> tree, int from, int count, bool visibleNodes)
		{
			StringBuilder stringBuilder = new StringBuilder();

			foreach (KeyValuePair<ITreeNode<int>, int> node in tree.Postorder(from, count, visibleNodes))
			{
				stringBuilder.AppendFormat("{0};", node.Key.Data);
			}

			return stringBuilder.ToString();
		}
	}
}
