﻿using System;

using Nabu.Data;

using NUnit.Framework;

namespace Nabu.UnitTest.nUnit.Nabu.Data
{
	[TestFixture]
	public class DiffPatchTest
	{
		[Test]
		public void Hirschberg()
		{
			byte[] oldContent = new byte[1024];
			byte[] newContent = new byte[1024];
			byte[] testContent = new byte[1024];

			for (int seed = 0; seed < 1000; seed++)
			{
				Random random = new Random(seed);

				random.NextBytes(oldContent);
				random.NextBytes(newContent);

				int oldOffset = random.Next(oldContent.Length / 4);
				int oldCount = random.Next(oldContent.Length - oldOffset);
				int newOffset = random.Next(newContent.Length / 4);
				int newCount = random.Next(newContent.Length - newOffset);

				PatchOperationList<byte> poList = new PatchOperationList<byte>();
				DiffHirschbergBlob diff = new DiffHirschbergBlob(oldContent, newContent);

				diff.BuildPatch(
					poList,
					oldOffset,
					oldCount,
					newOffset,
					newCount);

				PatchBinary patch = new PatchBinary();

				patch.Apply(
					testContent,
					newOffset,
					oldContent,
					oldOffset,
					oldCount,
					poList);

				for (int index = newOffset; index < newOffset + newCount; index++)
				{
					Assert.AreEqual(newContent[index], testContent[index]);
				}
			}
		}

		[Test]
		public void Myers()
		{
			byte[] oldContent = new byte[1024];
			byte[] newContent = new byte[1024];
			byte[] testContent = new byte[1024];

			for (int seed = 0; seed < 1000; seed++)
			{
				Random random = new Random(seed);

				random.NextBytes(oldContent);
				random.NextBytes(newContent);

				int oldOffset = random.Next(oldContent.Length / 4);
				int oldCount = random.Next(oldContent.Length - oldOffset);
				int newOffset = random.Next(newContent.Length / 4);
				int newCount = random.Next(newContent.Length - newOffset);

				PatchOperationList<byte> poList = new PatchOperationList<byte>();
				DiffMyersBlob diff = new DiffMyersBlob(oldContent, newContent);

				diff.BuildPatch(
					poList,
					oldOffset,
					oldCount,
					newOffset,
					newCount);

				PatchBinary patch = new PatchBinary();

				patch.Apply(
					testContent,
					newOffset,
					oldContent,
					oldOffset,
					oldCount,
					poList);

				for (int index = newOffset; index < newOffset + newCount; index++)
				{
					Assert.AreEqual(newContent[index], testContent[index]);
				}
			}
		}
	}
}
