﻿using Nabu.Drawing;

using NUnit.Framework;

namespace Nabu.UnitTest.nUnit.Nabu.Drawing
{
	[TestFixture]
	public class ColorSpaceTransformTest
	{
		private const float StepNumber = 97.0F;
		private const float StepR = (ColorRGB.RMax - ColorRGB.RMin) / StepNumber;
		private const float StepG = (ColorRGB.RMax - ColorRGB.RMin) / StepNumber;
		private const float StepB = (ColorRGB.RMax - ColorRGB.RMin) / StepNumber;
		private const float DeltaR = (ColorRGB.RMax - ColorRGB.RMin) / (10.0F * StepNumber);
		private const float DeltaG = (ColorRGB.RMax - ColorRGB.RMin) / (10.0F * StepNumber);
		private const float DeltaB = (ColorRGB.RMax - ColorRGB.RMin) / (10.0F * StepNumber);

		[Test]
		public void RGB_HSL()
		{
			for (float r = ColorRGB.RMin; r < ColorRGB.RMax; r += StepR)
			{
				for (float g = ColorRGB.GMin; g < ColorRGB.GMax; g += StepG)
				{
					for (float b = ColorRGB.BMin; b < ColorRGB.BMax; b += StepB)
					{
						ColorHSL colorHSL = ColorSpaceConverter.HSLFromRGB(new ColorRGB(r, g, b));
						ColorRGB colorRGB = ColorSpaceConverter.RGBFromHSL(colorHSL);

						Assert.AreEqual(colorRGB.R, r, DeltaR);
						Assert.AreEqual(colorRGB.G, g, DeltaG);
						Assert.AreEqual(colorRGB.B, b, DeltaB);
					}
				}
			}
		}

		[Test]
		public void RGB_HSV()
		{
			for (float r = ColorRGB.RMin; r < ColorRGB.RMax; r += StepR)
			{
				for (float g = ColorRGB.GMin; g < ColorRGB.GMax; g += StepG)
				{
					for (float b = ColorRGB.BMin; b < ColorRGB.BMax; b += StepB)
					{
						ColorHSV colorHSV = ColorSpaceConverter.HSVFromRGB(new ColorRGB(r, g, b));
						ColorRGB colorRGB = ColorSpaceConverter.RGBFromHSV(colorHSV);

						Assert.AreEqual(colorRGB.R, r, DeltaR);
						Assert.AreEqual(colorRGB.G, g, DeltaG);
						Assert.AreEqual(colorRGB.B, b, DeltaB);
					}
				}
			}
		}

		[Test]
		public void RGB_YUV()
		{
			for (float r = ColorRGB.RMin; r < ColorRGB.RMax; r += StepR)
			{
				for (float g = ColorRGB.GMin; g < ColorRGB.GMax; g += StepG)
				{
					for (float b = ColorRGB.BMin; b < ColorRGB.BMax; b += StepB)
					{
						ColorYUV colorYUV = ColorSpaceConverter.YUVFromRGB(new ColorRGB(r, g, b));
						ColorRGB colorRGB = ColorSpaceConverter.RGBFromYUV(colorYUV);

						Assert.AreEqual(colorRGB.R, r, DeltaR);
						Assert.AreEqual(colorRGB.G, g, DeltaG);
						Assert.AreEqual(colorRGB.B, b, DeltaB);
					}
				}
			}
		}
	}
}
