﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu Library                                  */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;

namespace Nabu
{
	public abstract class CommandLineLocalization
	{
		#region Fields

		private static readonly Dictionary<int, CommandLineLocalization> _localizations = new Dictionary<int, CommandLineLocalization>();

		#endregion

		#region Methods

		[SuppressMessage("Microsoft.Performance", "CA1810")]
		static CommandLineLocalization()
		{
			_localizations.Add(1033, new CommandLineLocalization_enUS());
			_localizations.Add(1049, new CommandLineLocalization_ruRU());
			_localizations.Add(2057, new CommandLineLocalization_enGB());
		}

		public static CommandLineLocalization Create(int localeID)
		{
			CommandLineLocalization result;

			if (_localizations.TryGetValue(localeID, out result))
			{
				return result;
			}
			else
			{
				return _localizations[1033];
			}
		}

		public abstract string HeadingUsage();
		public abstract string HeadingParameters();
		public abstract string MessageNamedArgumentIsUnknown(string argumentName);
		public abstract string MessageNamedArgumentShortage(string argumentName, int count, int limit);
		public abstract string MessageNamedArgumentExcess(string argumentName, int count, int limit);
		public abstract string MessageUnnamedArgumentShortage(int count, int limit, string[] arguments);
		public abstract string MessageUnnamedArgumentExcess(int count, int limit);
		public abstract string MessageParameterShortage(string argumentName, string[] parameterNames);
		public abstract string MessageParameterTypeIsInvalid(string argumentName, string parameterName, Type parameterType);

		#endregion

		#region Properties

		public abstract IFormatProvider FormatProvider { get; }

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
