﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu Library                                  */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
using System.Globalization;
using System.Text;

using Nabu.Text;

namespace Nabu
{
	internal sealed class CommandLineLocalization_ruRU : CommandLineLocalization
	{
		#region CommandLineLocalization_ruRU

		#region Constants

		private static readonly string[] ArgumentInstanceForms = new string[]
		{
			"вхождение аргумента", "вхождения аргумента", "вхождению аргумента",  "вхождение аргумента", "вхождением аргумента",  "вхождении аргумента",
			"вхождения аргумента", "вхождений аргумента", "вхождениям аргумента", "вхождения аргумента", "вхождениями аргумента", "вхождениях аргумента"
		};

		private static readonly string[] TimeForms = new string[]
		{
			"раз",  "раза", "разу",  "раз",  "разом",  "разе",
			"разы", "раз",  "разам", "разы", "разами", "разах"
		};

		private static readonly string[] UnnamedArgumentForms = new string[]
		{
			"безымянный аргумент",  "безымянных аргумента",  "безымянному аргументу", "безымянный аргумент",  "безымянным аргументом",   "безымянном аргументе",
			"безымянные аргументы", "безымянных аргументов", "безымянным аргументам", "безымянные аргументы", "безымянными аргументами", "безымянных аргументах"
		};

		#endregion

		#region Fields

		private readonly NumeralFormatter _numeralFormatter = NumeralFormatter.Create(1049);
		private readonly CultureInfo _cultureInfo = CultureInfo.GetCultureInfo(1049);

		#endregion

		#region Methods

		private int WordFormIndexFromNumeral(
			long value,
			WordGender gender,
			WordForms form)
		{
			switch (_numeralFormatter.NounFormFromNumeral(
				value,
				NumeralType.Cardinal,
				gender,
				form))
			{
				case WordForms.RU_Singular_SubjectiveCase:
					return 0x0000;
				case WordForms.RU_Singular_GenitiveCase:
					return 0x0001;
				case WordForms.RU_Singular_DativeCase:
					return 0x0002;
				case WordForms.RU_Singular_AccusativeCase:
					return 0x0003;
				case WordForms.RU_Singular_InstrumentalCase:
					return 0x0004;
				case WordForms.RU_Singular_PrepositionalCase:
					return 0x0005;
				case WordForms.RU_Plural_SubjectiveCase:
					return 0x0006;
				case WordForms.RU_Plural_GenitiveCase:
					return 0x0007;
				case WordForms.RU_Plural_DativeCase:
					return 0x0008;
				case WordForms.RU_Plural_AccusativeCase:
					return 0x0009;
				case WordForms.RU_Plural_InstrumentalCase:
					return 0x000A;
				case WordForms.RU_Plural_PrepositionalCase:
					return 0x000B;
			}

			return 0;
		}

		private string NumeralNameFromValue(
			long value,
			WordGender gender,
			WordForms form)
		{
			return _numeralFormatter.NumeralNameFromValue(
				value,
				NumeralType.Cardinal,
				gender,
				form,
				false);
		}

		#endregion

		#endregion

		#region CommandLineLocalization

		#region Methods

		public override string HeadingUsage()
		{
			return "Использование:";
		}

		public override string HeadingParameters()
		{
			return "Параметры:";
		}

		public override string MessageNamedArgumentIsUnknown(string argumentName)
		{
			return string.Format(
				this._cultureInfo,
				"Аргумент '{0}' не определён.",
				argumentName);
		}

		public override string MessageNamedArgumentShortage(string argumentName, int count, int limit)
		{
			if (count == 0)
			{
				return string.Format(
					this._cultureInfo,
					"Аргумент '{0}' не указан. Ожидается не менее {1} {2}.",
					argumentName,
					NumeralNameFromValue(limit, WordGender.Masculine, WordForms.RU_GenitiveCase),
					ArgumentInstanceForms[WordFormIndexFromNumeral(limit, WordGender.Masculine, WordForms.RU_GenitiveCase)]);
			}
			else
			{
				return string.Format(
					this._cultureInfo,
					"Аргумент '{0}' был указан {1} {2}. Ожидается не менее {3} {4}.",
					argumentName,
					NumeralNameFromValue(count, WordGender.Masculine, WordForms.RU_SubjectiveCase),
					TimeForms[WordFormIndexFromNumeral(count, WordGender.Masculine, WordForms.RU_SubjectiveCase)],
					NumeralNameFromValue(limit, WordGender.Masculine, WordForms.RU_GenitiveCase),
					ArgumentInstanceForms[WordFormIndexFromNumeral(limit, WordGender.Masculine, WordForms.RU_GenitiveCase)]);
			}
		}

		public override string MessageNamedArgumentExcess(string argumentName, int count, int limit)
		{
			return string.Format(
				this._cultureInfo,
				"Аргумент '{0}' был указан {1} {2}. Ожидается не более {3} {4}.",
				argumentName,
				NumeralNameFromValue(count, WordGender.Masculine, WordForms.RU_SubjectiveCase),
				TimeForms[WordFormIndexFromNumeral(count, WordGender.Masculine, WordForms.RU_SubjectiveCase)],
				NumeralNameFromValue(limit, WordGender.Masculine, WordForms.RU_GenitiveCase),
				ArgumentInstanceForms[WordFormIndexFromNumeral(limit, WordGender.Masculine, WordForms.RU_GenitiveCase)]);
		}

		public override string MessageUnnamedArgumentShortage(int count, int limit, string[] arguments)
		{
			StringBuilder result = new StringBuilder();

			if (count == 0)
			{
				result.AppendFormat(
					this._cultureInfo,
					"Не указаны безымянные аргументы. Ожидаетеся не менее {0} {1} ({2}",
					NumeralNameFromValue(limit, WordGender.Masculine, WordForms.RU_GenitiveCase),
					ArgumentInstanceForms[WordFormIndexFromNumeral(limit, WordGender.Masculine, WordForms.RU_GenitiveCase)],
					arguments[0]);
			}
			else
			{
				result.AppendFormat(
					this._cultureInfo,
					(count == 1) ? "Указан {0} {1}. Ожидается не менее {2} {3} ({4}" : "Указано {0} {1}. Ожидается не менее {2} {3} ({4}",
					NumeralNameFromValue(count, WordGender.Masculine, WordForms.RU_SubjectiveCase),
					UnnamedArgumentForms[WordFormIndexFromNumeral(count, WordGender.Masculine, WordForms.RU_SubjectiveCase)],
					NumeralNameFromValue(limit, WordGender.Masculine, WordForms.RU_GenitiveCase),
					UnnamedArgumentForms[WordFormIndexFromNumeral(limit, WordGender.Masculine, WordForms.RU_GenitiveCase)],
					arguments[0]);
			}

			for (int index = 1; index < arguments.Length; index++)
			{
				result.AppendFormat(
					this._cultureInfo,
					", {0}",
					arguments[index]);
			}

			result.Append(").");

			return result.ToString();
		}

		public override string MessageUnnamedArgumentExcess(int count, int limit)
		{
			if (limit == 0)
			{
				return string.Format(
					this._cultureInfo,
					"Указано {0} {1}. Безымянные аргументы не ожидались.",
					NumeralNameFromValue(count, WordGender.Masculine, WordForms.RU_SubjectiveCase),
					UnnamedArgumentForms[WordFormIndexFromNumeral(count, WordGender.Masculine, WordForms.RU_SubjectiveCase)]);
			}
			else
			{
				return string.Format(
					this._cultureInfo,
					"Указано {0} {1}. Ожидается не более {2} {3}.",
					NumeralNameFromValue(count, WordGender.Masculine, WordForms.RU_SubjectiveCase),
					UnnamedArgumentForms[WordFormIndexFromNumeral(count, WordGender.Masculine, WordForms.RU_SubjectiveCase)],
					NumeralNameFromValue(limit, WordGender.Masculine, WordForms.RU_GenitiveCase),
					UnnamedArgumentForms[WordFormIndexFromNumeral(limit, WordGender.Masculine, WordForms.RU_GenitiveCase)]);
			}
		}

		public override string MessageParameterShortage(string argumentName, string[] parameterNames)
		{
			StringBuilder result = new StringBuilder();

			result.AppendFormat(
				this._cultureInfo,
				(parameterNames.Length == 1) ? "У аргумента '{0}' отсутсвует параметр '{1}'" : "У аргумента '{0}' отсутсвуют параметры '{1}'",
				argumentName,
				parameterNames[0]);

			for (int index = 1; index < parameterNames.Length; index++)
			{
				result.AppendFormat(
					this._cultureInfo,
					", '{0}'",
					parameterNames[index]);
			}

			result.Append(".");

			return result.ToString();
		}

		public override string MessageParameterTypeIsInvalid(string argumentName, string parameterName, Type parameterType)
		{
			if (parameterType == null)
			{
				throw new ArgumentNullException("parameterType");
			}

			string parameterTypeName;

			if (parameterType == typeof(sbyte))
			{
				parameterTypeName = string.Format(
					this._cultureInfo,
					"целым числом от {0} до {1}",
					sbyte.MinValue,
					sbyte.MaxValue);
			}
			else if (parameterType == typeof(byte))
			{
				parameterTypeName = string.Format(
					this._cultureInfo,
					"целым числом от {0} до {1}",
					byte.MinValue,
					byte.MaxValue);
			}
			else if (parameterType == typeof(short))
			{
				parameterTypeName = string.Format(
					this._cultureInfo,
					"целым числом от {0} до {1}",
					short.MinValue,
					short.MaxValue);
			}
			else if (parameterType == typeof(ushort))
			{
				parameterTypeName = string.Format(
					this._cultureInfo,
					"целым числом от {0} до {1}",
					ushort.MinValue,
					ushort.MaxValue);
			}
			else if (parameterType == typeof(int))
			{
				parameterTypeName = string.Format(
					this._cultureInfo,
					"целым числом от {0} до {1}",
					int.MinValue,
					int.MaxValue);
			}
			else if (parameterType == typeof(uint))
			{
				parameterTypeName = string.Format(
					this._cultureInfo,
					"целым числом от {0} до {1}",
					uint.MinValue,
					uint.MaxValue);
			}
			else if (parameterType == typeof(long))
			{
				parameterTypeName = string.Format(
					this._cultureInfo,
					"целым числом от {0} до {1}",
					long.MinValue,
					long.MaxValue);
			}
			else if (parameterType == typeof(ulong))
			{
				parameterTypeName = string.Format(
					this._cultureInfo,
					"целым числом от {0} до {1}",
					ulong.MinValue,
					ulong.MaxValue);
			}
			else
			{
				parameterTypeName = string.Format(
					this._cultureInfo,
					"приводимым к типу '{0}'",
					parameterType.FullName);
			}

			return string.Format(
				this._cultureInfo,
				"Параметр '{1}' аргумента '{0}' должен быть {2}",
				argumentName,
				parameterName,
				parameterTypeName);
		}

		#endregion

		#region Properties

		public override IFormatProvider FormatProvider
		{
			[DebuggerStepThrough]
			get { return this._cultureInfo; }
		}

		#endregion

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
