﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu Library                                  */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Collections.Generic;
using System.Diagnostics;

namespace Nabu
{
	public sealed class CommandLineNamedArgumentDescriptor
	{
		#region Fields

		private readonly string _name;
		private readonly string _description;
		private readonly int _minInstanceCount;
		private readonly int _maxInstanceCount;
		private readonly CommandLineParameterDescriptor[] _parameters;

		#endregion

		#region Methods

		public CommandLineNamedArgumentDescriptor(
			string name,
			string description)
		{
			this._name = name;
			this._description = description;
			this._maxInstanceCount = 1;
			this._parameters = CommandLine.NoParameters;
		}

		public CommandLineNamedArgumentDescriptor(
			string name,
			string description,
			bool isMandatory)
		{
			this._name = name;
			this._description = description;
			this._minInstanceCount = isMandatory ? 1 : 0;
			this._maxInstanceCount = 1;
			this._parameters = CommandLine.NoParameters;
		}

		public CommandLineNamedArgumentDescriptor(
			string name,
			string description,
			bool isMandatory,
			CommandLineParameterDescriptor[] parameters)
		{
			this._name = name;
			this._description = description;
			this._minInstanceCount = isMandatory ? 1 : 0;
			this._maxInstanceCount = 1;
			this._parameters = parameters;
		}

		public CommandLineNamedArgumentDescriptor(
			string name,
			string description,
			int minInstanceCount,
			int maxInstanceCount,
			CommandLineParameterDescriptor[] parameters)
		{
			this._name = name;
			this._description = description;
			this._minInstanceCount = minInstanceCount;
			this._maxInstanceCount = maxInstanceCount;
			this._parameters = parameters;
		}

		#endregion

		#region Properties

		public string Name
		{
			[DebuggerStepThrough]
			get { return this._name; }
		}

		public string Description
		{
			[DebuggerStepThrough]
			get { return this._description; }
		}

		public int MinInstanceCount
		{
			[DebuggerStepThrough]
			get { return this._minInstanceCount; }
		}

		public int MaxInstanceCount
		{
			[DebuggerStepThrough]
			get { return this._maxInstanceCount; }
		}

		public IList<CommandLineParameterDescriptor> Parameters
		{
			[DebuggerStepThrough]
			get { return this._parameters; }
		}

		#endregion
	};
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
