﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu Library                                  */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;

namespace Nabu
{
	public class CancelMutableEventArgs<T> : EventArgs
	{
		#region Fields

		private T _value;
		private bool _cancel;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public CancelMutableEventArgs()
		{
			this._value = default(T);
		}

		[DebuggerStepThrough]
		public CancelMutableEventArgs(T value)
		{
			this._value = value;
		}

		#endregion

		#region Properties

		public T Value
		{
			[DebuggerStepThrough]
			get { return this._value; }
			[DebuggerStepThrough]
			set { this._value = value; }
		}

		public bool Cancel
		{
			[DebuggerStepThrough]
			get { return this._cancel; }
			[DebuggerStepThrough]
			set { this._cancel = value; }
		}

		#endregion
	}

	public class CancelMutableEventArgs<T, U> : EventArgs
	{
		#region Fields

		private T _value1;
		private U _value2;
		private bool _cancel;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public CancelMutableEventArgs()
		{
			this._value1 = default(T);
			this._value2 = default(U);
		}

		[DebuggerStepThrough]
		public CancelMutableEventArgs(
			T value1,
			U value2)
		{
			this._value1 = value1;
			this._value2 = value2;
		}

		#endregion

		#region Properties

		public T Value1
		{
			[DebuggerStepThrough]
			get { return this._value1; }
			[DebuggerStepThrough]
			set { this._value1 = value; }
		}

		public U Value2
		{
			[DebuggerStepThrough]
			get { return this._value2; }
			[DebuggerStepThrough]
			set { this._value2 = value; }
		}

		public bool Cancel
		{
			[DebuggerStepThrough]
			get { return this._cancel; }
			[DebuggerStepThrough]
			set { this._cancel = value; }
		}

		#endregion
	}

	[SuppressMessage("Microsoft.Design", "CA1005:AvoidExcessiveParametersOnGenericTypes")]
	public class CancelMutableEventArgs<T, U, V> : EventArgs
	{
		#region Fields

		private T _value1;
		private U _value2;
		private V _value3;
		private bool _cancel;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		public CancelMutableEventArgs()
		{
			this._value1 = default(T);
			this._value2 = default(U);
			this._value3 = default(V);
		}

		[DebuggerStepThrough]
		public CancelMutableEventArgs(
			T value1,
			U value2,
			V value3)
		{
			this._value1 = value1;
			this._value2 = value2;
			this._value3 = value3;
		}

		#endregion

		#region Properties

		public T Value1
		{
			[DebuggerStepThrough]
			get { return this._value1; }
			[DebuggerStepThrough]
			set { this._value1 = value; }
		}

		public U Value2
		{
			[DebuggerStepThrough]
			get { return this._value2; }
			[DebuggerStepThrough]
			set { this._value2 = value; }
		}

		public V Value3
		{
			[DebuggerStepThrough]
			get { return this._value3; }
			[DebuggerStepThrough]
			set { this._value3 = value; }
		}

		public bool Cancel
		{
			[DebuggerStepThrough]
			get { return this._cancel; }
			[DebuggerStepThrough]
			set { this._cancel = value; }
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
