﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu Library                                  */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Diagnostics;

namespace Nabu
{
	#region Message

	public sealed class MessageEventArgs : EventArgs<string>
	{
		#region Methods

		[DebuggerStepThrough]
		public MessageEventArgs(string value)
			: base(value)
		{
		}

		#endregion

		#region Properties

		public string Message
		{
			[DebuggerStepThrough]
			get { return base.Value; }
		}

		#endregion
	}

	public sealed class CancelMessageEventArgs : CancelEventArgs<string>
	{
		#region Methods

		[DebuggerStepThrough]
		public CancelMessageEventArgs(string value)
			: base(value)
		{
		}

		#endregion

		#region Properties

		public string Message
		{
			[DebuggerStepThrough]
			get { return base.Value; }
		}

		#endregion
	}

	public sealed class HandleMessageEventArgs : HandleEventArgs<string>
	{
		#region Methods

		[DebuggerStepThrough]
		public HandleMessageEventArgs(string value)
			: base(value)
		{
		}

		#endregion

		#region Properties

		public string Message
		{
			[DebuggerStepThrough]
			get { return base.Value; }
		}

		#endregion
	}

	#endregion

	#region Range

	public sealed class RangeEventArgs<T> : EventArgs<T, T>
	{
		#region Methods

		[DebuggerStepThrough]
		public RangeEventArgs(T offset, T length)
			: base(offset, length)
		{
		}

		#endregion

		#region Properties

		public T Offset
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
		}

		public T Length
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
		}

		#endregion
	}

	public sealed class RangeEventArgs<T, U> : EventArgs<T, T, U>
	{
		#region Methods

		[DebuggerStepThrough]
		public RangeEventArgs(T offset, T length, U context)
			: base(offset, length, context)
		{
		}

		#endregion

		#region Properties

		public T Offset
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
		}

		public T Length
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
		}

		public U Context
		{
			[DebuggerStepThrough]
			get { return base.Value3; }
		}

		#endregion
	}

	public sealed class RangeMutableEventArgs<T> : MutableEventArgs<T, T>
	{
		#region Methods

		[DebuggerStepThrough]
		public RangeMutableEventArgs()
		{
		}

		[DebuggerStepThrough]
		public RangeMutableEventArgs(T offset, T length)
			: base(offset, length)
		{
		}

		#endregion

		#region Properties

		public T Offset
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
			[DebuggerStepThrough]
			set { base.Value1 = value; }
		}

		public T Length
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
			[DebuggerStepThrough]
			set { base.Value2 = value; }
		}

		#endregion
	}

	public sealed class RangeMutableEventArgs<T, U> : MutableEventArgs<T, T, U>
	{
		#region Methods

		[DebuggerStepThrough]
		public RangeMutableEventArgs()
			: base()
		{
		}

		[DebuggerStepThrough]
		public RangeMutableEventArgs(T offset, T length, U context)
			: base(offset, length, context)
		{
		}

		#endregion

		#region Properties

		public T Offset
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
			[DebuggerStepThrough]
			set { base.Value1 = value; }
		}

		public T Length
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
			[DebuggerStepThrough]
			set { base.Value2 = value; }
		}

		public U Context
		{
			[DebuggerStepThrough]
			get { return base.Value3; }
			[DebuggerStepThrough]
			set { base.Value3 = value; }
		}

		#endregion
	}

	public sealed class CancelRangeEventArgs<T> : CancelEventArgs<T, T>
	{
		#region Methods

		[DebuggerStepThrough]
		public CancelRangeEventArgs(T offset, T length)
			: base(offset, length)
		{
		}

		#endregion

		#region Properties

		public T Offset
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
		}

		public T Length
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
		}

		#endregion
	}

	public sealed class CancelRangeEventArgs<T, U> : CancelEventArgs<T, T, U>
	{
		#region Methods

		[DebuggerStepThrough]
		public CancelRangeEventArgs(T offset, T length, U context)
			: base(offset, length, context)
		{
		}

		#endregion

		#region Properties

		public T Offset
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
		}

		public T Length
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
		}

		public U Context
		{
			[DebuggerStepThrough]
			get { return base.Value3; }
		}

		#endregion
	}

	public sealed class CancelRangeMutableEventArgs<T> : CancelMutableEventArgs<T, T>
	{
		#region Methods

		[DebuggerStepThrough]
		public CancelRangeMutableEventArgs()
		{
		}

		[DebuggerStepThrough]
		public CancelRangeMutableEventArgs(T offset, T length)
			: base(offset, length)
		{
		}

		#endregion

		#region Properties

		public T Offset
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
			[DebuggerStepThrough]
			set { base.Value1 = value; }
		}

		public T Length
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
			[DebuggerStepThrough]
			set { base.Value2 = value; }
		}

		#endregion
	}

	public sealed class CancelRangeMutableEventArgs<T, U> : CancelMutableEventArgs<T, T, U>
	{
		#region Methods

		[DebuggerStepThrough]
		public CancelRangeMutableEventArgs()
			: base()
		{
		}

		[DebuggerStepThrough]
		public CancelRangeMutableEventArgs(T offset, T length, U context)
			: base(offset, length, context)
		{
		}

		#endregion

		#region Properties

		public T Offset
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
			[DebuggerStepThrough]
			set { base.Value1 = value; }
		}

		public T Length
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
			[DebuggerStepThrough]
			set { base.Value2 = value; }
		}

		public U Context
		{
			[DebuggerStepThrough]
			get { return base.Value3; }
			[DebuggerStepThrough]
			set { base.Value3 = value; }
		}

		#endregion
	}

	public sealed class HandleRangeEventArgs<T> : HandleEventArgs<T, T>
	{
		#region Methods

		[DebuggerStepThrough]
		public HandleRangeEventArgs(T offset, T length)
			: base(offset, length)
		{
		}

		#endregion

		#region Properties

		public T Offset
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
		}

		public T Length
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
		}

		#endregion
	}

	public sealed class HandleRangeEventArgs<T, U> : HandleEventArgs<T, T, U>
	{
		#region Methods

		[DebuggerStepThrough]
		public HandleRangeEventArgs(T offset, T length, U context)
			: base(offset, length, context)
		{
		}

		#endregion

		#region Properties

		public T Offset
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
		}

		public T Length
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
		}

		public U Context
		{
			[DebuggerStepThrough]
			get { return base.Value3; }
		}

		#endregion
	}

	public sealed class HandleRangeMutableEventArgs<T> : HandleMutableEventArgs<T, T>
	{
		#region Methods

		[DebuggerStepThrough]
		public HandleRangeMutableEventArgs()
		{
		}

		[DebuggerStepThrough]
		public HandleRangeMutableEventArgs(T offset, T length)
			: base(offset, length)
		{
		}

		#endregion

		#region Properties

		public T Offset
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
			[DebuggerStepThrough]
			set { base.Value1 = value; }
		}

		public T Length
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
			[DebuggerStepThrough]
			set { base.Value2 = value; }
		}

		#endregion
	}

	public sealed class HandleRangeMutableEventArgs<T, U> : HandleMutableEventArgs<T, T, U>
	{
		#region Methods

		[DebuggerStepThrough]
		public HandleRangeMutableEventArgs()
			: base()
		{
		}

		[DebuggerStepThrough]
		public HandleRangeMutableEventArgs(T offset, T length, U context)
			: base(offset, length, context)
		{
		}

		#endregion

		#region Properties

		public T Offset
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
			[DebuggerStepThrough]
			set { base.Value1 = value; }
		}

		public T Length
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
			[DebuggerStepThrough]
			set { base.Value2 = value; }
		}

		public U Context
		{
			[DebuggerStepThrough]
			get { return base.Value3; }
			[DebuggerStepThrough]
			set { base.Value3 = value; }
		}

		#endregion
	}

	#endregion

	#region Progress

	public sealed class ProgressEventArgs<T> : EventArgs<T, T>
	{
		#region Methods

		[DebuggerStepThrough]
		public ProgressEventArgs(T total, T complete)
			: base(total, complete)
		{
		}

		#endregion

		#region Properties

		public T Total
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
		}

		public T Complete
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
		}

		#endregion
	}

	public sealed class ProgressEventArgs<T, U> : EventArgs<T, T, U>
	{
		#region Methods

		[DebuggerStepThrough]
		public ProgressEventArgs(T total, T complete, U context)
			: base(total, complete, context)
		{
		}

		#endregion

		#region Properties

		public T Total
		{
			[DebuggerStepThrough]
			get { return base.Value1; }
		}

		public T Complete
		{
			[DebuggerStepThrough]
			get { return base.Value2; }
		}

		public U Context
		{
			[DebuggerStepThrough]
			get { return base.Value3; }
		}

		#endregion
	}

	#endregion
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
