﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu Library                                  */
/*       {_.-``-'         {_/   Copyright © 2005-2008, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Runtime.InteropServices;
using System.Text;

namespace Nabu
{
#if COMPACT_FRAMEWORK
	public static class MarshalCF
	{
		#region String

		unsafe public static string PtrToStringAnsi(IntPtr pointer)
		{
			byte* lpStart = (byte*)pointer;
			byte* lpEnd = lpStart;

			for (; *lpEnd != 0; ++lpEnd)
			{
			}

			int size = (int)(lpEnd - lpStart);

			if (size == 0)
			{
				return string.Empty;
			}
			else
			{
				Decoder decoder = Encoding.ASCII.GetDecoder();
				char[] buffer = new char[decoder.GetCharCount(lpStart, size, true)];

				fixed (char* lpBuffer = buffer)
				{
					decoder.GetChars(lpStart, size, lpBuffer, buffer.Length, true);
				}

				return new string(buffer, 0, buffer.Length);
			}
		}

		unsafe public static string PtrToStringAnsi(IntPtr pointer, int size)
		{
			byte* lpStart = (byte*)pointer;
			Decoder decoder = Encoding.ASCII.GetDecoder();
			char[] buffer = new char[decoder.GetCharCount(lpStart, size, true)];

			fixed (char* lpBuffer = buffer)
			{
				decoder.GetChars(lpStart, size, lpBuffer, buffer.Length, true);
			}

			return new string(buffer, 0, buffer.Length);
		}

		unsafe public static IntPtr StringToHGlobalAnsi(string value)
		{
			Encoder encoder = Encoding.ASCII.GetEncoder();

			fixed (char* lpValue = value)
			{
				int stringLength = encoder.GetByteCount(lpValue, value.Length, true);
				IntPtr lpString = Marshal.AllocHGlobal(stringLength + 1);

				encoder.GetBytes(lpValue, value.Length, (byte*)lpString, stringLength, true);

				*(((byte*)lpString) + stringLength) = 0;

				return lpString;
			}
		}

		unsafe public static IntPtr StringToHGlobalUni(string value)
		{
			Encoder encoder = Encoding.Unicode.GetEncoder();

			fixed (char* lpValue = value)
			{
				int stringLength = encoder.GetByteCount(lpValue, value.Length, true);
				IntPtr lpString = Marshal.AllocHGlobal(stringLength + 2);

				encoder.GetBytes(lpValue, value.Length, (byte*)lpString, stringLength, true);

				*(((byte*)lpString) + stringLength) = 0;

				return lpString;
			}
		}

		unsafe public static IntPtr StringToCoTaskMemAnsi(string value)
		{
			Encoder encoder = Encoding.ASCII.GetEncoder();

			fixed (char* lpValue = value)
			{
				int stringLength = encoder.GetByteCount(lpValue, value.Length, true);
				IntPtr lpString = Marshal.AllocCoTaskMem(stringLength + 1);

				encoder.GetBytes(lpValue, value.Length, (byte*)lpString, stringLength, true);

				*(((byte*)lpString) + stringLength) = 0;

				return lpString;
			}
		}

		unsafe public static IntPtr StringToCoTaskMemUni(string value)
		{
			Encoder encoder = Encoding.Unicode.GetEncoder();

			fixed (char* lpValue = value)
			{
				int stringLength = encoder.GetByteCount(lpValue, value.Length, true);
				IntPtr lpString = Marshal.AllocCoTaskMem(stringLength + 2);

				encoder.GetBytes(lpValue, value.Length, (byte*)lpString, stringLength, true);

				*(((byte*)lpString) + stringLength) = 0;

				return lpString;
			}
		}

		#endregion

		#region Delegate

		#region IntPtr

		public static IntPtr GetIntPtr(object value)
		{
			Type type = value.GetType();

			switch (type.FullName)
			{
				case "System.Boolean":
					return new IntPtr(((bool)value) ? 1 : 0);
				case "System.Char":
					return new IntPtr((int)(char)value);
				case "System.SByte":
					return new IntPtr((sbyte)value);
				case "System.Byte":
					return new IntPtr((int)(byte)value);
				case "System.Int16":
					return new IntPtr((short)value);
				case "System.UInt16":
					return new IntPtr((int)(ushort)value);
				case "System.Int32":
					return new IntPtr((int)value);
				case "System.UInt32":
					return new IntPtr((long)(uint)value);
				case "System.Int64":
					return new IntPtr((long)value);
				case "System.UInt64":
					return new IntPtr((long)(ulong)value);
				case "System.IntPtr":
					return (IntPtr)value;
				case "System.UIntPtr":
					return new IntPtr((long)((UIntPtr)value).ToUInt64());
			}

			throw new ArgumentException("value");
		}

		#endregion

		#region CallUnmanagedCDeclFunction

		public static IntPtr CallUnmanagedCDeclFunction(IntPtr lpFunction, params object[] arguments)
		{
			IntPtr[] args = new IntPtr[arguments.Length];

			for (int index = 0; index < args.Length; ++index)
			{
				args[index] = GetIntPtr(arguments[index]);
			}

			return
				NativeMethods.CallUnmanagedFunctionPointer(
					true,
					lpFunction,
					args.Length,
					args);
		}

		#endregion

		#region CallUnmanagedStdCallFunction

		public static IntPtr CallUnmanagedStdCallFunction(
			IntPtr lpFunction,
			params object[] arguments)
		{
			IntPtr[] args = new IntPtr[arguments.Length];

			for (int index = 0; index < args.Length; ++index)
			{
				args[index] = GetIntPtr(arguments[index]);
			}

			return
				NativeMethods.CallUnmanagedFunctionPointer(
					false,
					lpFunction,
					args.Length,
					args);
		}

		#endregion

		#endregion
	}
#endif
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
