﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu Library                                  */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Runtime.InteropServices;
#if !COMPACT_FRAMEWORK
using System.Runtime.CompilerServices;
using System.Security;
#endif

namespace Nabu
{
	public sealed class NativeMemory : DisposableBase
	{
		#region NativeMemory

		#region Types

		private enum NativeMemoryType
		{
			GlobalBuffer,
			GlobalStringAnsi,
			GlobalStringUnicode,
			GlobalSecureStringAnsi,
			GlobalSecureStringUnicode,
			ComBuffer,
			ComStringAnsi,
			ComStringUnicode,
			ComStringBSTR,
			ComSecureStringAnsi,
			ComSecureStringUnicode,
			ComSecureStringBSTR
		}

		#endregion

		#region Fields

		private IntPtr _pointer;
		private int _size;
		private NativeMemoryType _type;

		#endregion

		#region Methods

		#region Constructors

		private NativeMemory(NativeMemoryType type, int size)
		{
#if !COMPACT_FRAMEWORK
			RuntimeHelpers.PrepareConstrainedRegions();
#endif

			try
			{
			}
			finally
			{
				switch (type)
				{
					case NativeMemoryType.GlobalBuffer:
						this._pointer = Marshal.AllocHGlobal(size);
						break;
					case NativeMemoryType.ComBuffer:
						this._pointer = Marshal.AllocCoTaskMem(size);
						break;
					default:
						throw new ArgumentOutOfRangeException("type");
				}

				this._size = size;
				this._type = type;
#if !COMPACT_FRAMEWORK
				GC.AddMemoryPressure(this._size + 1);
#endif
			}
		}

		private NativeMemory(NativeMemoryType type, string value)
		{
#if !COMPACT_FRAMEWORK
			RuntimeHelpers.PrepareConstrainedRegions();
#endif

			try
			{
			}
			finally
			{
				switch (type)
				{
					case NativeMemoryType.GlobalStringAnsi:
#if COMPACT_FRAMEWORK
						this._pointer = MarshalCF.StringToHGlobalAnsi(value);
#else
						this._pointer = Marshal.StringToHGlobalAnsi(value);
#endif
						this._size = value.Length;
						break;
					case NativeMemoryType.GlobalStringUnicode:
#if COMPACT_FRAMEWORK
						this._pointer = MarshalCF.StringToHGlobalUni(value);
#else
						this._pointer = Marshal.StringToHGlobalUni(value);
#endif
						this._size = 2 * value.Length;
						break;
					case NativeMemoryType.ComStringAnsi:
#if COMPACT_FRAMEWORK
						this._pointer = MarshalCF.StringToHGlobalAnsi(value);
#else
						this._pointer = Marshal.StringToCoTaskMemAnsi(value);
#endif
						this._size = value.Length;
						break;
					case NativeMemoryType.ComStringUnicode:
#if COMPACT_FRAMEWORK
						this._pointer = MarshalCF.StringToCoTaskMemUni(value);
#else
						this._pointer = Marshal.StringToCoTaskMemUni(value);
#endif
						this._size = 2 * value.Length;
						break;
					case NativeMemoryType.ComStringBSTR:
						this._pointer = Marshal.StringToBSTR(value);
						this._size = 2 * value.Length;
						break;
					default:
						throw new ArgumentOutOfRangeException("type");
				}

				this._type = type;
#if !COMPACT_FRAMEWORK
				GC.AddMemoryPressure(this._size + 1);
#endif
			}
		}

#if !COMPACT_FRAMEWORK
		private NativeMemory(NativeMemoryType type, SecureString value)
		{
			RuntimeHelpers.PrepareConstrainedRegions();

			try
			{
			}
			finally
			{
				switch (type)
				{
					case NativeMemoryType.GlobalSecureStringAnsi:
						this._pointer = Marshal.SecureStringToGlobalAllocAnsi(value);
						this._size = value.Length;
						break;
					case NativeMemoryType.GlobalSecureStringUnicode:
						this._pointer = Marshal.SecureStringToGlobalAllocUnicode(value);
						this._size = 2 * value.Length;
						break;
					case NativeMemoryType.ComSecureStringAnsi:
						this._pointer = Marshal.SecureStringToCoTaskMemAnsi(value);
						this._size = value.Length;
						break;
					case NativeMemoryType.ComSecureStringUnicode:
						this._pointer = Marshal.SecureStringToCoTaskMemUnicode(value);
						this._size = 2 * value.Length;
						break;
					case NativeMemoryType.ComSecureStringBSTR:
						this._pointer = Marshal.SecureStringToBSTR(value);
						this._size = 2 * value.Length;
						break;
					default:
						throw new ArgumentOutOfRangeException("type");
				}

				this._type = type;
				GC.AddMemoryPressure(this._size + 1);
			}
		}
#endif

		public NativeMemory(int size, bool isCom)
			: this(isCom ? NativeMemoryType.ComBuffer : NativeMemoryType.GlobalBuffer, size)
		{
		}

		public NativeMemory(int size)
			: this(NativeMemoryType.GlobalBuffer, size)
		{
		}

		public NativeMemory(string value)
			: this(NativeMemoryType.GlobalStringUnicode, value)
		{
		}

#if !COMPACT_FRAMEWORK
		public NativeMemory(SecureString value)
			: this(NativeMemoryType.GlobalSecureStringUnicode, value)
		{
		}
#endif

		public static NativeMemory FromGlobalStringAnsi(string value)
		{
			return new NativeMemory(NativeMemoryType.GlobalStringAnsi, value);
		}

#if !COMPACT_FRAMEWORK
		public static NativeMemory FromGlobalStringAnsi(SecureString value)
		{
			return new NativeMemory(NativeMemoryType.GlobalSecureStringAnsi, value);
		}
#endif

		public static NativeMemory FromGlobalStringUnicode(string value)
		{
			return new NativeMemory(NativeMemoryType.GlobalStringUnicode, value);
		}

#if !COMPACT_FRAMEWORK
		public static NativeMemory FromGlobalStringUnicode(SecureString value)
		{
			return new NativeMemory(NativeMemoryType.GlobalSecureStringUnicode, value);
		}
#endif

		public static NativeMemory FromComStringAnsi(string value)
		{
			return new NativeMemory(NativeMemoryType.ComStringAnsi, value);
		}

#if !COMPACT_FRAMEWORK
		public static NativeMemory FromComStringAnsi(SecureString value)
		{
			return new NativeMemory(NativeMemoryType.ComSecureStringAnsi, value);
		}
#endif

		public static NativeMemory FromComStringUnicode(string value)
		{
			return new NativeMemory(NativeMemoryType.ComStringUnicode, value);
		}

#if !COMPACT_FRAMEWORK
		public static NativeMemory FromComStringUnicode(SecureString value)
		{
			return new NativeMemory(NativeMemoryType.ComSecureStringUnicode, value);
		}
#endif

		public static NativeMemory FromBSTR(string value)
		{
			return new NativeMemory(NativeMemoryType.ComStringBSTR, value);
		}

#if !COMPACT_FRAMEWORK
		public static NativeMemory FromBSTR(SecureString value)
		{
			return new NativeMemory(NativeMemoryType.ComSecureStringBSTR, value);
		}
#endif

		#endregion

		#region Actions

		public string ToAnsiString()
		{
#if COMPACT_FRAMEWORK
			return MarshalCF.PtrToStringAnsi(this._pointer, this._size);
#else
			return Marshal.PtrToStringAnsi(this._pointer, this._size);
#endif
		}

		public string ToUnicodeString()
		{
			return Marshal.PtrToStringUni(this._pointer, this._size);
		}

		public object ToStructure(Type type, int offset)
		{
			return
				Marshal.PtrToStructure(
					new IntPtr(this._pointer.ToInt64() + offset),
					type);
		}

		public T ToStructure<T>(int offset)
		{
			return
				(T)Marshal.PtrToStructure(
					new IntPtr(this._pointer.ToInt64() + offset),
					typeof(T));
		}

		public object ToStructure(Type type)
		{
			return Marshal.PtrToStructure(this._pointer, type);
		}

		public T ToStructure<T>()
		{
			return (T)Marshal.PtrToStructure(this._pointer, typeof(T));
		}

		public void FromStructure(object value, bool deleteOld, int offset)
		{
			Marshal.StructureToPtr(
				value,
				new IntPtr(this._pointer.ToInt64() + offset),
				deleteOld);
		}

		public void FromStructure(object value, bool deleteOld)
		{
			Marshal.StructureToPtr(value, this._pointer, deleteOld);
		}

		public byte ReadByte(int offset)
		{
			return Marshal.ReadByte(this._pointer, offset);
		}

		public short ReadInt16(int offset)
		{
			return Marshal.ReadInt16(this._pointer, offset);
		}

		public int ReadInt32(int offset)
		{
			return Marshal.ReadInt32(this._pointer, offset);
		}

		public long ReadInt64(int offset)
		{
			return Marshal.ReadInt64(this._pointer, offset);
		}

		public IntPtr ReadIntPtr(int offset)
		{
#if COMPACT_FRAMEWORK
			return Marshal.ReadIntPtr(new IntPtr(this._pointer.ToInt64() + offset));
#else
			return Marshal.ReadIntPtr(this._pointer, offset);
#endif
		}

		public void WriteByte(int offset, byte value)
		{
			Marshal.WriteByte(this._pointer, offset, value);
		}

		public void WriteInt16(int offset, short value)
		{
			Marshal.WriteInt16(this._pointer, offset, value);
		}

		public void WriteInt32(int offset, int value)
		{
			Marshal.WriteInt32(this._pointer, offset, value);
		}

		public void WriteInt64(int offset, long value)
		{
			Marshal.WriteInt64(this._pointer, offset, value);
		}

		public void WriteIntPtr(int offset, IntPtr value)
		{
#if COMPACT_FRAMEWORK
			Marshal.WriteIntPtr(new IntPtr(this._pointer.ToInt64() + offset), value);
#else
			Marshal.WriteIntPtr(this._pointer, offset, value);
#endif
		}

		public void Copy(byte[] array, int offset, int count)
		{
		}

		public void CopyFrom(byte[] source, int startIndex, int length)
		{
			Marshal.Copy(source, startIndex, this._pointer, length);
		}

		public void CopyFrom(char[] source, int startIndex, int length)
		{
			Marshal.Copy(source, startIndex, this._pointer, length);
		}

		public void CopyFrom(double[] source, int startIndex, int length)
		{
			Marshal.Copy(source, startIndex, this._pointer, length);
		}

		public void CopyFrom(float[] source, int startIndex, int length)
		{
			Marshal.Copy(source, startIndex, this._pointer, length);
		}

		public void CopyFrom(int[] source, int startIndex, int length)
		{
			Marshal.Copy(source, startIndex, this._pointer, length);
		}

		public void CopyFrom(short[] source, int startIndex, int length)
		{
			Marshal.Copy(source, startIndex, this._pointer, length);
		}

		public void CopyFrom(long[] source, int startIndex, int length)
		{
			Marshal.Copy(source, startIndex, this._pointer, length);
		}

#if !COMPACT_FRAMEWORK
		public void CopyFrom(IntPtr[] source, int startIndex, int length)
		{
			Marshal.Copy(source, startIndex, this._pointer, length);
		}
#endif

		public void CopyTo(byte[] destination, int startIndex, int length)
		{
			Marshal.Copy(this._pointer, destination, startIndex, length);
		}

		public void CopyTo(char[] destination, int startIndex, int length)
		{
			Marshal.Copy(this._pointer, destination, startIndex, length);
		}

		public void CopyTo(double[] destination, int startIndex, int length)
		{
			Marshal.Copy(this._pointer, destination, startIndex, length);
		}

		public void CopyTo(float[] destination, int startIndex, int length)
		{
			Marshal.Copy(this._pointer, destination, startIndex, length);
		}

		public void CopyTo(int[] destination, int startIndex, int length)
		{
			Marshal.Copy(this._pointer, destination, startIndex, length);
		}

		public void CopyTo(short[] destination, int startIndex, int length)
		{
			Marshal.Copy(this._pointer, destination, startIndex, length);
		}

		public void CopyTo(long[] destination, int startIndex, int length)
		{
			Marshal.Copy(this._pointer, destination, startIndex, length);
		}

#if !COMPACT_FRAMEWORK
		public void CopyTo(IntPtr[] destination, int startIndex, int length)
		{
			Marshal.Copy(this._pointer, destination, startIndex, length);
		}
#endif

		#endregion

		#endregion

		#region Properties

		public IntPtr Pointer
		{
			get { return this._pointer; }
		}

		public int Size
		{
			get { return this._size; }
		}

		#endregion

		#endregion

		#region DisposableBase

		protected override void Dispose(bool disposing)
		{
#if !COMPACT_FRAMEWORK
			RuntimeHelpers.PrepareConstrainedRegions();
#endif

			try
			{
			}
			finally
			{
				if (this._pointer != IntPtr.Zero)
				{
					switch (this._type)
					{
						case NativeMemoryType.GlobalBuffer:
							Marshal.FreeHGlobal(this._pointer);
							break;
						case NativeMemoryType.GlobalStringAnsi:
							Marshal.FreeHGlobal(this._pointer);
							break;
						case NativeMemoryType.GlobalStringUnicode:
							Marshal.FreeHGlobal(this._pointer);
							break;
#if !COMPACT_FRAMEWORK
						case NativeMemoryType.GlobalSecureStringAnsi:
							Marshal.ZeroFreeGlobalAllocAnsi(this._pointer);
							break;
						case NativeMemoryType.GlobalSecureStringUnicode:
							Marshal.ZeroFreeGlobalAllocUnicode(this._pointer);
							break;
#endif
						case NativeMemoryType.ComBuffer:
							Marshal.FreeCoTaskMem(this._pointer);
							break;
						case NativeMemoryType.ComStringAnsi:
							Marshal.FreeCoTaskMem(this._pointer);
							break;
						case NativeMemoryType.ComStringUnicode:
							Marshal.FreeCoTaskMem(this._pointer);
							break;
						case NativeMemoryType.ComStringBSTR:
							Marshal.FreeBSTR(this._pointer);
							break;
#if !COMPACT_FRAMEWORK
						case NativeMemoryType.ComSecureStringAnsi:
							Marshal.ZeroFreeCoTaskMemAnsi(this._pointer);
							break;
						case NativeMemoryType.ComSecureStringUnicode:
							Marshal.ZeroFreeCoTaskMemUnicode(this._pointer);
							break;
						case NativeMemoryType.ComSecureStringBSTR:
							Marshal.ZeroFreeBSTR(this._pointer);
							break;
#endif
					}

#if !COMPACT_FRAMEWORK
					GC.RemoveMemoryPressure(this._size + 1);
#endif
				}
			}

			base.Dispose(disposing);
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
