﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu Library                                  */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.InteropServices;
using System.Text;

namespace Nabu
{
	internal static class NativeMethods
	{
		#region Libraries

#if COMPACT_FRAMEWORK
		private const string Kernel32 = "coredll.dll";
		private const string Shell32 = "coredll.dll";
#else
		private const string Kernel32 = "kernel32.dll";
		private const string Shell32 = "shell32.dll";
		private const string Gdi32 = "gdi32.dll";
#endif

		#endregion

		#region NativeResult

		[SuppressMessage("Microsoft.Design", "CA1021:AvoidOutParameters", MessageId = "4#")]
		[DllImport(Kernel32, SetLastError = true, CharSet = CharSet.Unicode)]
		public static extern int FormatMessageW(
			[In] int flags,
			[In] IntPtr source,
			[In] int messageId,
			[In] int languageId,
			[Out][MarshalAs(UnmanagedType.LPWStr)] StringBuilder buffer,
			[In] int bufferSize,
			[In] IntPtr arguments);

		#endregion

		#region SpecialPath

		[SuppressMessage("Microsoft.Design", "CA1021:AvoidOutParameters", MessageId = "1#")]
		[DllImport(Kernel32, CharSet = CharSet.Unicode)]
		public static extern int GetModuleFileNameW(
			[In] IntPtr hModule,
			[In][Out][MarshalAs(UnmanagedType.LPWStr)] StringBuilder path,
			[In] int size);

		#endregion

		#region DynamicInterop

#if COMPACT_FRAMEWORK
		[DllImport("kernel32.dll", SetLastError = true)]
		public static extern IntPtr GetProcAddress(
			[In] IntPtr hModule,
			[In][MarshalAs(UnmanagedType.LPWStr)] string name);
#else
		[DllImport("kernel32.dll", ExactSpelling = true, SetLastError = true)]
		public static extern IntPtr GetProcAddress(
			[In] IntPtr hModule,
			[In][MarshalAs(UnmanagedType.LPStr)] string name);
#endif

		#endregion

		#region Bifrost

#if COMPACT_FRAMEWORK

		[DllImport("bifrost.dll")]
		public static extern IntPtr CallUnmanagedFunctionPointer(
			[In][MarshalAs(UnmanagedType.Bool)] bool isCDecl,
			[In] IntPtr lpFunction,
			[In] int argCount,
			[In][MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 3)] IntPtr[] args);

#endif

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
