﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu Library                                  */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.ComponentModel;
using System.Diagnostics;

namespace Nabu
{
	public abstract class DisposableBase : IDisposable
	{
		#region DisposableBase

		#region Fields

		private bool _isDisposed;

		#endregion

		#region Methods

		[DebuggerStepThrough]
		protected void CheckDisposed()
		{
			if (this._isDisposed)
			{
				throw new ObjectDisposedException(GetType().FullName);
			}
		}

		[DebuggerStepThrough]
		protected virtual void Dispose(bool disposing)
		{
			try
			{
				if (disposing)
				{
					if (this.Disposed != null)
					{
						this.Disposed(this, EventArgs.Empty);
					}
				}
			}
			finally
			{
				this._isDisposed = true;
			}
		}

		protected DisposableBase()
		{
		}

		~DisposableBase()
		{
			if (!this._isDisposed)
			{
				Dispose(false);
			}
		}

		#endregion

		#region Properties

#if !COMPACT_FRAMEWORK
		[Browsable(false)]
#endif
		public bool IsDisposed
		{
			[DebuggerStepThrough]
			get { return this._isDisposed; }
			[DebuggerStepThrough]
			protected set { this._isDisposed = value; }
		}

		#endregion

		#region Events

		public event EventHandler Disposed;

		#endregion

		#endregion

		#region IDisposable

		[DebuggerStepThrough]
		public void Dispose()
		{
			if (!this._isDisposed)
			{
				Dispose(true);
				GC.SuppressFinalize(this);
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
