﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu Library                                  */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;

namespace Nabu
{
	public sealed class NaturalStringComparer : IComparer<string>
	{
		public int Compare(string a, string b)
		{
			int startIndexA = 0;
			int startIndexB = 0;
			int endIndexA = 0;
			int endIndexB = 0;
			bool isDigitA = !char.IsDigit(a, 0);
			bool isDigitB = !char.IsDigit(b, 0);

			for (; ; )
			{
				if ((startIndexA == a.Length) && (startIndexB == b.Length))
				{
					return 0;
				}

				while ((endIndexA < a.Length) && (char.IsDigit(a, endIndexA) ^ isDigitA))
				{
					endIndexA++;
				}

				while ((endIndexB < b.Length) && (char.IsDigit(b, endIndexB) ^ isDigitB))
				{
					endIndexB++;
				}

				// One block consists from digits and another is not.
				if (isDigitA ^ isDigitB)
				{
					if (char.IsLetter(a, startIndexA))
					{
						// Letter > Digit.
						return 1;
					}
					else
					{
						// Punctuation < Digit.
						return -1;
					}
				}
				else
				{
					int lengthA = endIndexA - startIndexA;
					int lengthB = endIndexB - startIndexB;

					if (isDigitA)
					{
						// Both blocks consist from non-digits.
						int result =
							string.Compare(
								a,
								startIndexA,
								b,
								startIndexB,
								Math.Min(lengthA, lengthB),
								StringComparison.OrdinalIgnoreCase);

						if (result == 0)
						{
							if (lengthA != lengthB)
							{
								return lengthA - lengthB;
							}
						}
						else
						{
							return result;
						}
					}
					else
					{
						// Both blocks consist from digits.
						while ((startIndexA < endIndexA) && (a[startIndexA] == '0'))
						{
							startIndexA++;
							lengthA--;
						}

						while ((startIndexB < endIndexB) && (b[startIndexB] == '0'))
						{
							startIndexB++;
							lengthB--;
						}

						if (lengthA == lengthB)
						{
							int result =
								string.Compare(
									a,
									startIndexA,
									b,
									startIndexB,
									Math.Min(lengthA, lengthB),
									StringComparison.OrdinalIgnoreCase);

							if (result == 0)
							{
								if (lengthA != lengthB)
								{
									return lengthA - lengthB;
								}
							}
							else
							{
								return result;
							}
						}
						else
						{
							return lengthA - lengthB;
						}
					}
				}

				startIndexA = endIndexA;
				startIndexB = endIndexB;
				isDigitA = !isDigitA;
				isDigitB = !isDigitB;
			}
		}
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
