﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Text Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;

namespace Nabu.Text
{
	public abstract class NumeralFormatter
	{
		#region Fields

		private static readonly Dictionary<int, NumeralFormatter> _formatters = new Dictionary<int, NumeralFormatter>();

		#endregion

		#region Methods

		[SuppressMessage("Microsoft.Performance", "CA1810")]
		static NumeralFormatter()
		{
			_formatters.Add(1033, new NumeralFormatter_enUS());
			_formatters.Add(1049, new NumeralFormatter_ruRU());
			_formatters.Add(2057, new NumeralFormatter_enGB());
		}

		public static NumeralFormatter Create(int localeID)
		{
			NumeralFormatter result;

			if (_formatters.TryGetValue(localeID, out result))
			{
				return result;
			}
			else
			{
				return _formatters[1033];
			}
		}

		public abstract string NumeralNameFromValue(
			long numeralValue,
			NumeralType numeralType,
			WordGender numeralGender,
			WordForms numeralForm,
			bool suppressExtraNumbers);

		public abstract WordForms NounFormFromNumeral(
			long numeralValue,
			NumeralType numeralType,
			WordGender numeralGender,
			WordForms numeralForm);

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
