﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Text Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;

namespace Nabu.Text
{
	internal abstract class NumeralFormatter_en : NumeralFormatter
	{
		#region NumberFormatter_en

		#region Fields

		private readonly string[] _unitForms;
		private readonly CultureInfo _cultureInfo;
		private readonly IFormatProvider _numberProvider;

		#endregion

		#region Methods

		#region Constructors

		protected NumeralFormatter_en(int localeID, params string[] unitForms)
		{
			this._cultureInfo = new CultureInfo(localeID);
			this._numberProvider = this._cultureInfo.NumberFormat;
			this._unitForms = unitForms;
		}

		#endregion

		#region Cardinals

		private string CardinalNameFromValueInternal(
			long numeralValue)
		{
			switch (numeralValue)
			{
				case 0: return "zero";
				case 1: return "one";
				case 2: return "two";
				case 3: return "three";
				case 4: return "four";
				case 5: return "five";
				case 6: return "six";
				case 7: return "seven";
				case 8: return "eight";
				case 9: return "nine";
				case 10: return "ten";
				case 11: return "eleven";
				case 12: return "twelve";
				case 13: return "thirteen";
				case 14: return "fourteen";
				case 15: return "fifteen";
				case 16: return "sixteen";
				case 17: return "seventeen";
				case 18: return "eighteen";
				case 19: return "nineteen";
				case 20: return "twenty";
				case 30: return "thirty";
				case 40: return "fourty";
				case 50: return "fifty";
				case 60: return "sixty";
				case 70: return "seventy";
				case 80: return "eighty";
				case 90: return "ninety";
			}

			return string.Format(
				this._numberProvider,
				"[{0}]",
				numeralValue);
		}

		[SuppressMessage("Microsoft.Usage", "CA1801", MessageId = "suppressExtraNumbers")]
		private string CardinalNameFromValueInternal(
			long numeralValue,
			bool suppressExtraNumbers)
		{
			string result = string.Format(
				this._numberProvider,
				"[{0}]",
				numeralValue);
			long numeralValuePart0 = (numeralValue / 1) % 10;
			long numeralValuePart1 = (numeralValue / 10) % 10;
			long numeralValuePart2 = (numeralValue / 100) % 10;

			if (numeralValuePart2 == 0)
			{
				if (numeralValuePart1 < 2)
				{
					result = CardinalNameFromValueInternal(10 * numeralValuePart1 + numeralValuePart0);
				}
				else
				{
					if (numeralValuePart0 == 0)
					{
						result = CardinalNameFromValueInternal(10 * numeralValuePart1);
					}
					else
					{
						result =
							CardinalNameFromValueInternal(10 * numeralValuePart1) +
							" " +
							CardinalNameFromValueInternal(numeralValuePart0);
					}
				}
			}
			else
			{
				if (numeralValuePart1 == 0)
				{
					if (numeralValuePart0 == 0)
					{
						result =
							CardinalNameFromValueInternal(numeralValuePart2) +
							" hundred";
					}
					else
					{
						result =
							CardinalNameFromValueInternal(numeralValuePart2) +
							" hundred and " +
							CardinalNameFromValueInternal(numeralValuePart0);
					}
				}
				else if (numeralValuePart1 == 1)
				{
					result =
						CardinalNameFromValueInternal(numeralValuePart2) +
						" hundred and " +
						CardinalNameFromValueInternal(10 * numeralValuePart1 + numeralValuePart0);
				}
				else
				{
					if (numeralValuePart0 == 0)
					{
						result =
							CardinalNameFromValueInternal(numeralValuePart2) +
							" hundred and " +
							CardinalNameFromValueInternal(10 * numeralValuePart1);
					}
					else
					{
						result =
							CardinalNameFromValueInternal(numeralValuePart2) +
							" hundred and " +
							CardinalNameFromValueInternal(10 * numeralValuePart1) +
							" " +
							CardinalNameFromValueInternal(numeralValuePart0);
					}
				}
			}

			return result;
		}

		[SuppressMessage("Microsoft.Usage", "CA1801", MessageId = "numeralGender")]
		[SuppressMessage("Microsoft.Usage", "CA1801", MessageId = "numeralForm")]
		private string CardinalStringFromNumber(
			long numeralValue,
			WordGender numeralGender,
			WordForms numeralForm,
			bool suppressExtraNumbers)
		{
			long numeralValuePart0 = (numeralValue / 1) % 1000;
			long numeralValuePart3 = (numeralValue / 1000) % 1000;
			long numeralValuePart6 = (numeralValue / 1000000) % 1000;
			long numeralValuePart9 = (numeralValue / 1000000000) % 1000;

			string result = string.Empty;

			if (numeralValuePart9 > 0)
			{
				result +=
					((result.Length == 0) ? string.Empty : " ") +
					CardinalNameFromValueInternal(
						numeralValuePart9,
						suppressExtraNumbers) +
					this._unitForms[3];
				suppressExtraNumbers = false;
			}

			if (numeralValuePart6 > 0)
			{
				result +=
					((result.Length == 0) ? string.Empty : " ") +
					CardinalNameFromValueInternal(
						numeralValuePart6,
						suppressExtraNumbers) +
					this._unitForms[2];
				suppressExtraNumbers = false;
			}

			if (numeralValuePart3 > 0)
			{
				result +=
					((result.Length == 0) ? string.Empty : " ") +
					CardinalNameFromValueInternal(
						numeralValuePart3,
						suppressExtraNumbers) +
					this._unitForms[1];
				suppressExtraNumbers = false;
			}

			if ((numeralValuePart0 > 0) || (result.Length == 0))
			{
				result +=
					((result.Length == 0) ? string.Empty : " ") +
					CardinalNameFromValueInternal(
						numeralValuePart0,
						suppressExtraNumbers) +
					this._unitForms[0];
			}

			return result;
		}

		#endregion

		#region Ordinals

		private string OrdinalNameFromValueInternal(
			long numeralValue)
		{
			switch (numeralValue)
			{
				//case 0: return ""; No generic ordinal for zero.
				case 1: return "first";
				case 2: return "second";
				case 3: return "third";
				case 4: return "fourth";
				case 5: return "fifth";
				case 6: return "sixth";
				case 7: return "seventh";
				case 8: return "eighth";
				case 9: return "ninth";
				case 10: return "tenth";
				case 11: return "eleventh";
				case 12: return "twelfth";
				case 13: return "thirteenth";
				case 14: return "fourteenth";
				case 15: return "fifteenth";
				case 16: return "sixteenth";
				case 17: return "seventeenth";
				case 18: return "eighteenth";
				case 19: return "nineteenth";
				case 20: return "twentieth";
				case 30: return "thirtieth";
				case 40: return "fortieth";
				case 50: return "fiftieth";
				case 60: return "sixtieth";
				case 70: return "seventieth";
				case 80: return "eightieth";
				case 90: return "ninetieth";
			}

			return string.Format(
				this._numberProvider,
				"[{0}]",
				numeralValue);
		}

		[SuppressMessage("Microsoft.Usage", "CA1801", MessageId = "suppressExtraNumbers")]
		private string OrdinalNameFromValueInternal(
			long numeralValue,
			bool suppressExtraNumbers)
		{
			string result = string.Format(
				this._numberProvider,
				"[{0}]",
				numeralValue);
			long numeralValuePart0 = (numeralValue / 1) % 10;
			long numeralValuePart1 = (numeralValue / 10) % 10;
			long numeralValuePart2 = (numeralValue / 100) % 10;

			if (numeralValuePart2 == 0)
			{
				if (numeralValuePart1 < 2)
				{
					result = OrdinalNameFromValueInternal(10 * numeralValuePart1 + numeralValuePart0);
				}
				else
				{
					if (numeralValuePart0 == 0)
					{
						result = OrdinalNameFromValueInternal(10 * numeralValuePart1);
					}
					else
					{
						result =
							CardinalNameFromValueInternal(10 * numeralValuePart1) +
							" " +
							OrdinalNameFromValueInternal(numeralValuePart0);
					}
				}
			}
			else
			{
				if (numeralValuePart1 == 0)
				{
					if (numeralValuePart0 == 0)
					{
						result =
							OrdinalNameFromValueInternal(numeralValuePart2) +
							" hundred";
					}
					else
					{
						result =
							CardinalNameFromValueInternal(numeralValuePart2) +
							" hundred and " +
							OrdinalNameFromValueInternal(numeralValuePart0);
					}
				}
				else if (numeralValuePart1 == 1)
				{
					result =
						CardinalNameFromValueInternal(numeralValuePart2) +
						" hundred and " +
						OrdinalNameFromValueInternal(10 * numeralValuePart1 + numeralValuePart0);
				}
				else
				{
					if (numeralValuePart0 == 0)
					{
						result =
							CardinalNameFromValueInternal(numeralValuePart2) +
							" hundred and " +
							OrdinalNameFromValueInternal(10 * numeralValuePart1);
					}
					else
					{
						result =
							CardinalNameFromValueInternal(numeralValuePart2) +
							" hundred and " +
							CardinalNameFromValueInternal(10 * numeralValuePart1) +
							" " +
							OrdinalNameFromValueInternal(numeralValuePart0);
					}
				}
			}

			return result;
		}

		[SuppressMessage("Microsoft.Usage", "CA1801", MessageId = "numeralGender")]
		[SuppressMessage("Microsoft.Usage", "CA1801", MessageId = "numeralForm")]
		private string OrdinalNameFromValue(
			long numeralValue,
			WordGender numeralGender,
			WordForms numeralForm,
			bool suppressExtraNumbers)
		{
			long numeralValuePart0 = (numeralValue / 1) % 1000;
			long numeralValuePart3 = (numeralValue / 1000) % 1000;
			long numeralValuePart6 = (numeralValue / 1000000) % 1000;
			long numeralValuePart9 = (numeralValue / 1000000000) % 1000;

			string result = string.Empty;

			if (numeralValuePart9 > 0)
			{
				if (
					(numeralValuePart6 > 0) ||
					(numeralValuePart3 > 0) ||
					(numeralValuePart0 > 0))
				{
					result +=
						((result.Length == 0) ? string.Empty : " ") +
						CardinalNameFromValueInternal(
							numeralValuePart9,
							suppressExtraNumbers) +
						this._unitForms[3];
				}
				else
				{
					result +=
						((result.Length == 0) ? string.Empty : " ") +
						OrdinalNameFromValueInternal(
							numeralValuePart9,
							suppressExtraNumbers) +
						this._unitForms[3];
				}

				suppressExtraNumbers = false;
			}

			if (numeralValuePart6 > 0)
			{
				if (
					(numeralValuePart3 > 0) ||
					(numeralValuePart0 > 0))
				{
					result +=
						((result.Length == 0) ? string.Empty : " ") +
						CardinalNameFromValueInternal(
							numeralValuePart6,
							suppressExtraNumbers) +
						this._unitForms[2];
				}
				else
				{
					result +=
						((result.Length == 0) ? string.Empty : " ") +
						OrdinalNameFromValueInternal(
							numeralValuePart6,
							suppressExtraNumbers) +
						this._unitForms[2];
				}

				suppressExtraNumbers = false;
			}

			if (numeralValuePart3 > 0)
			{
				if (numeralValuePart0 > 0)
				{
					result +=
						((result.Length == 0) ? string.Empty : " ") +
						CardinalNameFromValueInternal(
							numeralValuePart3,
							suppressExtraNumbers) +
						this._unitForms[1];
				}
				else
				{
					result +=
						((result.Length == 0) ? string.Empty : " ") +
						OrdinalNameFromValueInternal(
							numeralValuePart3,
							suppressExtraNumbers) +
						this._unitForms[1];
				}

				suppressExtraNumbers = false;
			}

			if ((numeralValuePart0 > 0) || (result.Length == 0))
			{
				result +=
					((result.Length == 0) ? string.Empty : " ") +
					OrdinalNameFromValueInternal(
						numeralValuePart0,
						suppressExtraNumbers) +
					this._unitForms[0];
			}

			return result;
		}

		#endregion

		#endregion

		#endregion

		#region NumeralFormatter

		public override string NumeralNameFromValue(
			long numeralValue,
			NumeralType numeralType,
			WordGender numeralGender,
			WordForms numeralForm,
			bool suppressExtraNumbers)
		{
			switch (numeralType)
			{
				case NumeralType.Cardinal:
					return CardinalStringFromNumber(
						numeralValue,
						numeralGender,
						numeralForm,
						suppressExtraNumbers);
				case NumeralType.Ordinal:
					return OrdinalNameFromValue(
						numeralValue,
						numeralGender,
						numeralForm,
						suppressExtraNumbers);
			}

			return string.Format(
				this._numberProvider,
				"[{0}]",
				numeralValue);
		}

		public override WordForms NounFormFromNumeral(
			long numeralValue,
			NumeralType numeralType,
			WordGender numeralGender,
			WordForms numeralForm)
		{
			return (numeralValue == 1) ? WordForms.EN_Singular : WordForms.EN_Plural;
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
