﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Text Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;

namespace Nabu.Text
{
	internal sealed class NumeralFormatter_ruRU : NumeralFormatter
	{
		#region NumeralFormatter_ruRU

		#region Constants

		#region Unit Forms

		private static readonly string[] UnitFormsEmpty =
		{
			/*          Subjective     Genitive       Dative         Accusative     Instrumental   Prepositional */
			/* Singular 0 */ "",   /*  1 */ "",   /*  2 */ "",   /*  3 */ "",   /*  4 */ "",   /*  5 */ "",
			/* Plural   6 */ "",   /*  7 */ "",   /*  8 */ "",   /*  9 */ "",   /* 10 */ "",   /* 11 */ "",
		};

		private static readonly string[] UnitFormsThousand =
		{
			/*          Subjective          Genitive            Dative              Accusative           Instrumental          Prepositional */
			/* Singular 0 */ " тысяча", /*  1 */ " тысячи", /*  2 */ " тысяче",  /*  3 */ " тысячу", /*  4 */ " тысячей",  /*  5 */ " тысяче",
			/* Plural   6 */ " тысячи", /*  7 */ " тысяч",  /*  8 */ " тысячам", /*  9 */ " тысячи", /* 10 */ " тысячами", /* 11 */ " тысячах",
		};

		private static readonly string[] UnitFormsMillion =
		{
			/*          Subjective            Genitive               Dative                 Accusative            Instrumental            Prepositional */
			/* Singular 0 */ " миллион",  /*  1 */ " миллиона",  /*  2 */ " миллиону",  /*  3 */ " миллион",  /*  4 */ " миллионом",  /*  5 */ " миллионе",
			/* Plural   6 */ " миллионы", /*  7 */ " миллионов", /*  8 */ " миллионам", /*  9 */ " миллионы", /* 10 */ " миллионами", /* 11 */ " миллионах",
		};

		private static readonly string[] UnitFormsMilliard =
		{
			/*          Subjective             Genitive                Dative                  Accusative            Instrumental             Prepositional */
			/* Singular 0 */ " миллиард",  /*  1 */ " миллиарда",  /*  2 */ " миллиарду",  /*  3 */ " миллиард", /*  4 */ " миллиардом",  /*  5 */ " миллиарде",
			/* Plural   6 */ " миллиарды", /*  7 */ " миллиардов", /*  8 */ " миллиардам", /*  9 */ " миллиард", /* 10 */ " миллиардами", /* 11 */ " миллиардах",
		};

		#endregion

		#endregion

		#region Fields

		//private CultureInfo _cultureInfo = CultureInfo.GetCultureInfo(1049);
		private readonly IFormatProvider _numberProvider = CultureInfo.GetCultureInfo(1049).NumberFormat;

		#endregion

		#region Methods

		#region UnitForms

		private static int UnitFormIndex(WordForms wordForm)
		{
			switch (wordForm)
			{
				case WordForms.RU_Singular_SubjectiveCase:
					return 0;
				case WordForms.RU_Singular_GenitiveCase:
					return 1;
				case WordForms.RU_Singular_DativeCase:
					return 2;
				case WordForms.RU_Singular_AccusativeCase:
					return 3;
				case WordForms.RU_Singular_InstrumentalCase:
					return 4;
				case WordForms.RU_Singular_PrepositionalCase:
					return 5;
				case WordForms.RU_Plural_SubjectiveCase:
					return 6;
				case WordForms.RU_Plural_GenitiveCase:
					return 7;
				case WordForms.RU_Plural_DativeCase:
					return 8;
				case WordForms.RU_Plural_AccusativeCase:
					return 9;
				case WordForms.RU_Plural_InstrumentalCase:
					return 10;
				case WordForms.RU_Plural_PrepositionalCase:
					return 11;
				default:
					return -1;
			}
		}

		#endregion

		#region Cardinals

		private string CardinalNameFromValue(
			long numeralValue,
			WordGender numeralGender,
			WordForms numeralForm)
		{
			switch (numeralValue)
			{
				#region case   0:

				case 0:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "ноль";
						case WordForms.RU_GenitiveCase:
							return "нуля";
						case WordForms.RU_DativeCase:
							return "нулю";
						case WordForms.RU_AccusativeCase:
							return "ноль";
						case WordForms.RU_InstrumentalCase:
							return "нулём";
						case WordForms.RU_PrepositionalCase:
							return "нуле";
					}

					break;

				#endregion

				#region case   1:

				case 1:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "один";
								case WordGender.Neuter:
									return "одно";
								case WordGender.Feminine:
									return "одна";
							}

							break;

						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "одного";
								case WordGender.Neuter:
									return "одного";
								case WordGender.Feminine:
									return "одной";
							}

							break;

						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "одному";
								case WordGender.Neuter:
									return "одному";
								case WordGender.Feminine:
									return "одной";
							}

							break;

						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "один";
								case WordGender.Neuter:
									return "одно";
								case WordGender.Feminine:
									return "одну";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "одиного";
								case WordGender.Neuter:
									return "одно";
								case WordGender.Feminine:
									return "одну";
							}

							break;

						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "одним";
								case WordGender.Neuter:
									return "одним";
								case WordGender.Feminine:
									return "одной";
							}

							break;

						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "одном";
								case WordGender.Neuter:
									return "одном";
								case WordGender.Feminine:
									return "одной";
							}

							break;
					}

					break;

				#endregion

				#region case   2:

				case 2:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "два";
								case WordGender.Neuter:
									return "два";
								case WordGender.Feminine:
									return "две";
							}

							break;

						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							return "двух";

						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							return "двум";

						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двух";
								case WordGender.Neuter:
									return "два";
								case WordGender.Feminine:
									return "двух";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							return "двух";

						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							return "двумя";

						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							return "двух";
					}

					break;

				#endregion

				#region case   3:

				case 3:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							return "три";

						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							return "трёх";
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							return "трём";

						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёх";
								case WordGender.Neuter:
									return "три";
								case WordGender.Feminine:
									return "трёх";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							return "трёх";

						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							return "тремя";

						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							return "трёх";
					}

					break;

				#endregion

				#region case   4:

				case 4:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							return "четыре";

						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							return "четырёх";

						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							return "четырём";

						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёх";
								case WordGender.Neuter:
									return "четыре";
								case WordGender.Feminine:
									return "четырёх";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							return "четырёх";

						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							return "четырьмя";
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							return "четырёх";
					}

					break;

				#endregion

				#region case   5:

				case 5:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "пять";
						case WordForms.RU_GenitiveCase:
							return "пяти";
						case WordForms.RU_DativeCase:
							return "пяти";
						case WordForms.RU_AccusativeCase:
							return "пять";
						case WordForms.RU_InstrumentalCase:
							return "пятью";
						case WordForms.RU_PrepositionalCase:
							return "пяти";
					}

					break;

				#endregion

				#region case   6:

				case 6:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "шесть";
						case WordForms.RU_GenitiveCase:
							return "шести";
						case WordForms.RU_DativeCase:
							return "шести";
						case WordForms.RU_AccusativeCase:
							return "шесть";
						case WordForms.RU_InstrumentalCase:
							return "шестью";
						case WordForms.RU_PrepositionalCase:
							return "шести";
					}

					break;

				#endregion

				#region case   7:

				case 7:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "семь";
						case WordForms.RU_GenitiveCase:
							return "семи";
						case WordForms.RU_DativeCase:
							return "семи";
						case WordForms.RU_AccusativeCase:
							return "семь";
						case WordForms.RU_InstrumentalCase:
							return "семью";
						case WordForms.RU_PrepositionalCase:
							return "семи";
					}

					break;

				#endregion

				#region case   8:

				case 8:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "восемь";
						case WordForms.RU_GenitiveCase:
							return "восьми";
						case WordForms.RU_DativeCase:
							return "восьми";
						case WordForms.RU_AccusativeCase:
							return "восемь";
						case WordForms.RU_InstrumentalCase:
							return "восемью";
						case WordForms.RU_PrepositionalCase:
							return "восьми";
					}

					break;

				#endregion

				#region case   9:

				case 9:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "девять";
						case WordForms.RU_GenitiveCase:
							return "девяти";
						case WordForms.RU_DativeCase:
							return "девяти";
						case WordForms.RU_AccusativeCase:
							return "девять";
						case WordForms.RU_InstrumentalCase:
							return "девятью";
						case WordForms.RU_PrepositionalCase:
							return "девяти";
					}

					break;

				#endregion

				#region case  10:

				case 10:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "десять";
						case WordForms.RU_GenitiveCase:
							return "десяти";
						case WordForms.RU_DativeCase:
							return "десяти";
						case WordForms.RU_AccusativeCase:
							return "десять";
						case WordForms.RU_InstrumentalCase:
							return "десятью";
						case WordForms.RU_PrepositionalCase:
							return "десяти";
					}

					break;

				#endregion

				#region case  11:

				case 11:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "одиннадцать";
						case WordForms.RU_GenitiveCase:
							return "одиннадцати";
						case WordForms.RU_DativeCase:
							return "одиннадцати";
						case WordForms.RU_AccusativeCase:
							return "одиннадцать";
						case WordForms.RU_InstrumentalCase:
							return "одиннадцатью";
						case WordForms.RU_PrepositionalCase:
							return "одиннадцати";
					}

					break;

				#endregion

				#region case  12:

				case 12:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "двенадцать";
						case WordForms.RU_GenitiveCase:
							return "двенадцати";
						case WordForms.RU_DativeCase:
							return "двенадцати";
						case WordForms.RU_AccusativeCase:
							return "двенадцать";
						case WordForms.RU_InstrumentalCase:
							return "двенадцатью";
						case WordForms.RU_PrepositionalCase:
							return "двенадцати";
					}

					break;

				#endregion

				#region case  13:

				case 13:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "тринадцать";
						case WordForms.RU_GenitiveCase:
							return "тринадцати";
						case WordForms.RU_DativeCase:
							return "тринадцати";
						case WordForms.RU_AccusativeCase:
							return "тринадцать";
						case WordForms.RU_InstrumentalCase:
							return "тринадцатью";
						case WordForms.RU_PrepositionalCase:
							return "тринадцати";
					}

					break;

				#endregion

				#region case  14:

				case 14:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "четырнадцать";
						case WordForms.RU_GenitiveCase:
							return "четырнадцати";
						case WordForms.RU_DativeCase:
							return "четырнадцати";
						case WordForms.RU_AccusativeCase:
							return "четырнадцать";
						case WordForms.RU_InstrumentalCase:
							return "четырнадцатью";
						case WordForms.RU_PrepositionalCase:
							return "четырнадцати";
					}

					break;

				#endregion

				#region case  15:

				case 15:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "пятнадцать";
						case WordForms.RU_GenitiveCase:
							return "пятнадцати";
						case WordForms.RU_DativeCase:
							return "пятнадцати";
						case WordForms.RU_AccusativeCase:
							return "пятнадцать";
						case WordForms.RU_InstrumentalCase:
							return "пятнадцатью";
						case WordForms.RU_PrepositionalCase:
							return "пятнадцати";
					}

					break;

				#endregion

				#region case  16:

				case 16:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "шестнадцать";
						case WordForms.RU_GenitiveCase:
							return "шестнадцати";
						case WordForms.RU_DativeCase:
							return "шестнадцати";
						case WordForms.RU_AccusativeCase:
							return "шестнадцать";
						case WordForms.RU_InstrumentalCase:
							return "шестнадцатью";
						case WordForms.RU_PrepositionalCase:
							return "шестнадцати";
					}

					break;

				#endregion

				#region case  17:

				case 17:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "семнадцать";
						case WordForms.RU_GenitiveCase:
							return "семнадцати";
						case WordForms.RU_DativeCase:
							return "семнадцати";
						case WordForms.RU_AccusativeCase:
							return "семнадцать";
						case WordForms.RU_InstrumentalCase:
							return "семнадцатью";
						case WordForms.RU_PrepositionalCase:
							return "семнадцати";
					}

					break;

				#endregion

				#region case  18:

				case 18:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "восемнадцать";
						case WordForms.RU_GenitiveCase:
							return "восемнадцати";
						case WordForms.RU_DativeCase:
							return "восемнадцати";
						case WordForms.RU_AccusativeCase:
							return "восемнадцать";
						case WordForms.RU_InstrumentalCase:
							return "восемнадцатью";
						case WordForms.RU_PrepositionalCase:
							return "восемнадцати";
					}

					break;

				#endregion

				#region case  19:

				case 19:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "девятнадцать";
						case WordForms.RU_GenitiveCase:
							return "девятнадцати";
						case WordForms.RU_DativeCase:
							return "девятнадцати";
						case WordForms.RU_AccusativeCase:
							return "девятнадцать";
						case WordForms.RU_InstrumentalCase:
							return "девятнадцатью";
						case WordForms.RU_PrepositionalCase:
							return "девятнадцати";
					}

					break;

				#endregion

				#region case  20:

				case 20:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "двадцать";
						case WordForms.RU_GenitiveCase:
							return "двадцати";
						case WordForms.RU_DativeCase:
							return "двадцати";
						case WordForms.RU_AccusativeCase:
							return "двадцать";
						case WordForms.RU_InstrumentalCase:
							return "двадцатью";
						case WordForms.RU_PrepositionalCase:
							return "двадцати";
					}

					break;

				#endregion

				#region case  30:

				case 30:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "тридцать";
						case WordForms.RU_GenitiveCase:
							return "тридцати";
						case WordForms.RU_DativeCase:
							return "тридцати";
						case WordForms.RU_AccusativeCase:
							return "тридцать";
						case WordForms.RU_InstrumentalCase:
							return "тридцатью";
						case WordForms.RU_PrepositionalCase:
							return "тридцати";
					}

					break;

				#endregion

				#region case  40:

				case 40:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "сорок";
						case WordForms.RU_GenitiveCase:
							return "сорока";
						case WordForms.RU_DativeCase:
							return "сорока";
						case WordForms.RU_AccusativeCase:
							return "сорок";
						case WordForms.RU_InstrumentalCase:
							return "сорока";
						case WordForms.RU_PrepositionalCase:
							return "сорока";
					}

					break;

				#endregion

				#region case  50:

				case 50:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "пятьдесят";
						case WordForms.RU_GenitiveCase:
							return "пятидесяти";
						case WordForms.RU_DativeCase:
							return "пятидесяти";
						case WordForms.RU_AccusativeCase:
							return "пятьдесят";
						case WordForms.RU_InstrumentalCase:
							return "пятьюдесятью";
						case WordForms.RU_PrepositionalCase:
							return "пятидесяти";
					}

					break;

				#endregion

				#region case  60:

				case 60:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "шестьдесят";
						case WordForms.RU_GenitiveCase:
							return "шестидесяти";
						case WordForms.RU_DativeCase:
							return "шестидесяти";
						case WordForms.RU_AccusativeCase:
							return "шестьдесят";
						case WordForms.RU_InstrumentalCase:
							return "шестьюдесятью";
						case WordForms.RU_PrepositionalCase:
							return "шестидесяти";
					}

					break;

				#endregion

				#region case  70:

				case 70:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "семьдесят";
						case WordForms.RU_GenitiveCase:
							return "семидесяти";
						case WordForms.RU_DativeCase:
							return "семидесяти";
						case WordForms.RU_AccusativeCase:
							return "семьдесят";
						case WordForms.RU_InstrumentalCase:
							return "семьюдесятью";
						case WordForms.RU_PrepositionalCase:
							return "семидесяти";
					}

					break;

				#endregion

				#region case  80:

				case 80:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "восемьдесят";
						case WordForms.RU_GenitiveCase:
							return "восьмидесяти";
						case WordForms.RU_DativeCase:
							return "восьмидесяти";
						case WordForms.RU_AccusativeCase:
							return "восемьдесят";
						case WordForms.RU_InstrumentalCase:
							return "восьмьюдесятью";
						case WordForms.RU_PrepositionalCase:
							return "восьмидесяти";
					}

					break;

				#endregion

				#region case  90:

				case 90:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "девяносто";
						case WordForms.RU_GenitiveCase:
							return "девяноста";
						case WordForms.RU_DativeCase:
							return "девяноста";
						case WordForms.RU_AccusativeCase:
							return "девяносто";
						case WordForms.RU_InstrumentalCase:
							return "девяноста";
						case WordForms.RU_PrepositionalCase:
							return "девяноста";
					}

					break;

				#endregion

				#region case 100:

				case 100:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "сто";
						case WordForms.RU_GenitiveCase:
							return "ста";
						case WordForms.RU_DativeCase:
							return "ста";
						case WordForms.RU_AccusativeCase:
							return "сто";
						case WordForms.RU_InstrumentalCase:
							return "ста";
						case WordForms.RU_PrepositionalCase:
							return "ста";
					}

					break;

				#endregion

				#region case 200:

				case 200:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "двести";
						case WordForms.RU_GenitiveCase:
							return "двухсот";
						case WordForms.RU_DativeCase:
							return "двустам";
						case WordForms.RU_AccusativeCase:
							return "двести";
						case WordForms.RU_InstrumentalCase:
							return "двумястами";
						case WordForms.RU_PrepositionalCase:
							return "двустах";
					}

					break;

				#endregion

				#region case 300:

				case 300:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "триста";
						case WordForms.RU_GenitiveCase:
							return "трёхсот";
						case WordForms.RU_DativeCase:
							return "трёмстам";
						case WordForms.RU_AccusativeCase:
							return "триста";
						case WordForms.RU_InstrumentalCase:
							return "тремястами";
						case WordForms.RU_PrepositionalCase:
							return "трёмстах";
					}

					break;

				#endregion

				#region case 400:

				case 400:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "четыреста";
						case WordForms.RU_GenitiveCase:
							return "четырёхсот";
						case WordForms.RU_DativeCase:
							return "четырёхстам";
						case WordForms.RU_AccusativeCase:
							return "четыреста";
						case WordForms.RU_InstrumentalCase:
							return "четырьмястами";
						case WordForms.RU_PrepositionalCase:
							return "четырёхстах";
					}

					break;

				#endregion

				#region case 500:

				case 500:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "пятьсот";
						case WordForms.RU_GenitiveCase:
							return "пятисот";
						case WordForms.RU_DativeCase:
							return "пятистам";
						case WordForms.RU_AccusativeCase:
							return "пятьсот";
						case WordForms.RU_InstrumentalCase:
							return "пятьюстами";
						case WordForms.RU_PrepositionalCase:
							return "пятьюстах";
					}

					break;

				#endregion

				#region case 600:

				case 600:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "шестьсот";
						case WordForms.RU_GenitiveCase:
							return "шестисот";
						case WordForms.RU_DativeCase:
							return "шестистам";
						case WordForms.RU_AccusativeCase:
							return "шестьсот";
						case WordForms.RU_InstrumentalCase:
							return "шестьюстами";
						case WordForms.RU_PrepositionalCase:
							return "шестьюстах";
					}

					break;

				#endregion

				#region case 700:

				case 700:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "семьсот";
						case WordForms.RU_GenitiveCase:
							return "семисот";
						case WordForms.RU_DativeCase:
							return "семистам";
						case WordForms.RU_AccusativeCase:
							return "семьсот";
						case WordForms.RU_InstrumentalCase:
							return "семьюстами";
						case WordForms.RU_PrepositionalCase:
							return "семьюстах";
					}

					break;

				#endregion

				#region case 800:

				case 800:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "восемьсот";
						case WordForms.RU_GenitiveCase:
							return "восьмисот";
						case WordForms.RU_DativeCase:
							return "восьмистам";
						case WordForms.RU_AccusativeCase:
							return "восемьсот";
						case WordForms.RU_InstrumentalCase:
							return "восемьюстами";
						case WordForms.RU_PrepositionalCase:
							return "восемьюстах";
					}

					break;

				#endregion

				#region case 900:

				case 900:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							return "девятьсот";
						case WordForms.RU_GenitiveCase:
							return "девятисот";
						case WordForms.RU_DativeCase:
							return "девятистам";
						case WordForms.RU_AccusativeCase:
							return "девятьсот";
						case WordForms.RU_InstrumentalCase:
							return "девятьюстами";
						case WordForms.RU_PrepositionalCase:
							return "девятьюстах";
					}

					break;

				#endregion
			}

			return string.Format(
				this._numberProvider,
				"[{0}]",
				numeralValue);
		}

		private static WordForms CardinalNounFormFromValueInternal(
			long numeralValuePart0,
			long numeralValuePart1,
			WordForms numeralForm)
		{
			int numeralCategory = 5;

			if (numeralValuePart1 != 1)
			{
				switch (numeralValuePart0)
				{
					case 0:
						numeralCategory = 0;
						break;
					case 1:
						numeralCategory = 1;
						break;
					case 2:
					case 3:
					case 4:
						numeralCategory = 2;
						break;
				}
			}

			switch (numeralForm & WordForms.RU_Case_Mask)
			{
				case WordForms.RU_SubjectiveCase:
					switch (numeralCategory)
					{
						case 0: return WordForms.RU_Plural_GenitiveCase;
						case 1: return WordForms.RU_Singular_SubjectiveCase;
						case 2: return WordForms.RU_Singular_GenitiveCase;
						case 5: return WordForms.RU_Plural_GenitiveCase;
					}
					break;
				case WordForms.RU_GenitiveCase:
					switch (numeralCategory)
					{
						case 0: return WordForms.RU_Plural_GenitiveCase;
						case 1: return WordForms.RU_Singular_GenitiveCase;
						case 2: return WordForms.RU_Plural_GenitiveCase;
						case 5: return WordForms.RU_Plural_GenitiveCase;
					}
					break;
				case WordForms.RU_DativeCase:
					switch (numeralCategory)
					{
						case 0: return WordForms.RU_Plural_GenitiveCase;
						case 1: return WordForms.RU_Singular_DativeCase;
						case 2: return WordForms.RU_Plural_DativeCase;
						case 5: return WordForms.RU_Plural_DativeCase;
					}
					break;
				case WordForms.RU_AccusativeCase:
					switch (numeralCategory)
					{
						case 0: return WordForms.RU_Plural_GenitiveCase;
						case 1: return WordForms.RU_Singular_AccusativeCase;
						case 2: return WordForms.RU_Plural_AccusativeCase;
						case 5: return WordForms.RU_Plural_GenitiveCase;
					}
					break;
				case WordForms.RU_InstrumentalCase:
					switch (numeralCategory)
					{
						case 0: return WordForms.RU_Plural_GenitiveCase;
						case 1: return WordForms.RU_Singular_InstrumentalCase;
						case 2: return WordForms.RU_Plural_InstrumentalCase;
						case 5: return WordForms.RU_Plural_InstrumentalCase;
					}
					break;
				case WordForms.RU_PrepositionalCase:
					switch (numeralCategory)
					{
						case 0: return WordForms.RU_Plural_GenitiveCase;
						case 1: return WordForms.RU_Singular_PrepositionalCase;
						case 2: return WordForms.RU_Plural_PrepositionalCase;
						case 5: return WordForms.RU_Plural_PrepositionalCase;
					}
					break;
			}

			return WordForms.RU_Default;
		}

		[SuppressMessage("Microsoft.Performance", "CA1822")]
		private string CardinalNameFromValueInternal(
			long numeralValue,
			WordGender numeralGender,
			WordForms numeralForm,
			bool suppressExtraNumbers,
			string[] unitForms)
		{
			string result = string.Format(
				this._numberProvider,
				"[{0}]",
				numeralValue);
			long numeralValuePart0 = (numeralValue / 1) % 10;
			long numeralValuePart1 = (numeralValue / 10) % 10;
			long numeralValuePart2 = (numeralValue / 100) % 10;
			string unitForm = unitForms[UnitFormIndex(CardinalNounFormFromValueInternal(numeralValuePart0, numeralValuePart1, numeralForm))];

			if (numeralValuePart2 == 0)
			{
				if (numeralValuePart1 == 0)
				{
					if ((numeralValuePart0 == 1) && suppressExtraNumbers)
					{
						result = unitForm.Trim();
					}
					else
					{
						result =
							CardinalNameFromValue(numeralValuePart0, numeralGender, numeralForm) +
							unitForm;
					}
				}
				else if (numeralValuePart1 == 1)
				{
					result =
						CardinalNameFromValue(10 + numeralValuePart0, numeralGender, numeralForm) +
						unitForm;
				}
				else
				{
					if (numeralValuePart0 == 0)
					{
						result =
							CardinalNameFromValue(10 * numeralValuePart1, numeralGender, numeralForm) +
							unitForm;
					}
					else
					{
						result =
							CardinalNameFromValue(10 * numeralValuePart1, numeralGender, numeralForm) +
							" " +
							CardinalNameFromValue(numeralValuePart0, numeralGender, numeralForm) +
							unitForm;
					}
				}
			}
			else
			{
				if (numeralValuePart1 == 0)
				{
					if (numeralValuePart0 == 0)
					{
						result =
							CardinalNameFromValue(100 * numeralValuePart2, numeralGender, numeralForm) +
							unitForm;
					}
					else
					{
						result =
							CardinalNameFromValue(100 * numeralValuePart2, numeralGender, numeralForm) +
							" " +
							CardinalNameFromValue(numeralValuePart0, numeralGender, numeralForm) +
							unitForm;
					}
				}
				else if (numeralValuePart1 == 1)
				{
					result =
						CardinalNameFromValue(100 * numeralValuePart2, numeralGender, numeralForm) +
						" " +
						CardinalNameFromValue(10 + numeralValuePart0, numeralGender, numeralForm) +
						unitForm;
				}
				else
				{
					if (numeralValuePart0 == 0)
					{
						result =
							CardinalNameFromValue(100 * numeralValuePart2, numeralGender, numeralForm) +
							" " +
							CardinalNameFromValue(10 * numeralValuePart1, numeralGender, numeralForm) +
							unitForm;
					}
					else
					{
						result =
							CardinalNameFromValue(100 * numeralValuePart2, numeralGender, numeralForm) +
							" " +
							CardinalNameFromValue(10 * numeralValuePart1, numeralGender, numeralForm) +
							" " +
							CardinalNameFromValue(numeralValuePart0, numeralGender, numeralForm) +
							unitForm;
					}
				}
			}

			return result;
		}

		public string CardinalStringFromNumber(
			long numeralValue,
			WordGender numeralGender,
			WordForms numeralForm,
			bool suppressExtraNumbers)
		{
			long numeralValuePart0 = (numeralValue / 1) % 1000;
			long numeralValuePart3 = (numeralValue / 1000) % 1000;
			long numeralValuePart6 = (numeralValue / 1000000) % 1000;
			long numeralValuePart9 = (numeralValue / 1000000000) % 1000;
			string result = string.Empty;

			if (numeralValuePart9 != 0)
			{
				result +=
					((result.Length == 0) ? string.Empty : " ") +
					CardinalNameFromValueInternal(
						numeralValuePart9,
						WordGender.Masculine,
						numeralForm,
						suppressExtraNumbers,
						UnitFormsMilliard);
				suppressExtraNumbers = false;
			}

			if (numeralValuePart6 != 0)
			{
				result +=
					((result.Length == 0) ? string.Empty : " ") +
					CardinalNameFromValueInternal(
						numeralValuePart6,
						WordGender.Masculine,
						numeralForm,
						suppressExtraNumbers,
						UnitFormsMillion);
				suppressExtraNumbers = false;
			}

			if (numeralValuePart3 != 0)
			{
				result +=
					((result.Length == 0) ? string.Empty : " ") +
					CardinalNameFromValueInternal(
						numeralValuePart3,
						WordGender.Feminine,
						numeralForm,
						suppressExtraNumbers,
						UnitFormsThousand);
				suppressExtraNumbers = false;
			}

			if ((numeralValuePart0 != 0) || (result.Length == 0))
			{
				result +=
					((result.Length == 0) ? string.Empty : " ") +
					CardinalNameFromValueInternal(
						numeralValuePart0,
						numeralGender,
						numeralForm,
						suppressExtraNumbers,
						UnitFormsEmpty);
			}

			return result;
		}

		#endregion

		#region Ordinals

		private string OrdinalNameFromValue(
			long numeralValue,
			WordGender numeralGender,
			WordForms numeralForm)
		{
			switch (numeralValue)
			{
				#region case   0:

				case 0:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "нулевой";
								case WordGender.Neuter:
									return "нулевое";
								case WordGender.Feminine:
									return "нулевая";
							}

							break;

						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "нулевого";
								case WordGender.Neuter:
									return "нулевого";
								case WordGender.Feminine:
									return "нулевой";
							}

							break;

						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "нулевому";
								case WordGender.Neuter:
									return "нулевому";
								case WordGender.Feminine:
									return "нулевой";
							}

							break;

						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "нулевой";
								case WordGender.Neuter:
									return "нулевое";
								case WordGender.Feminine:
									return "нулевую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "нулевого";
								case WordGender.Neuter:
									return "нулевое";
								case WordGender.Feminine:
									return "нулевую";
							}

							break;

						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "нулевым";
								case WordGender.Neuter:
									return "нулевым";
								case WordGender.Feminine:
									return "нулевой";
							}

							break;

						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "нулевом";
								case WordGender.Neuter:
									return "нулевом";
								case WordGender.Feminine:
									return "нулевой";
							}

							break;
					}

					break;

				#endregion

				#region case   1:

				case 1:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "первый";
								case WordGender.Neuter:
									return "первое";
								case WordGender.Feminine:
									return "первая";
							}

							break;

						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "первого";
								case WordGender.Neuter:
									return "первого";
								case WordGender.Feminine:
									return "первой";
							}

							break;

						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "первому";
								case WordGender.Neuter:
									return "первому";
								case WordGender.Feminine:
									return "первой";
							}

							break;

						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "первый";
								case WordGender.Neuter:
									return "первое";
								case WordGender.Feminine:
									return "первую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "первого";
								case WordGender.Neuter:
									return "первое";
								case WordGender.Feminine:
									return "первую";
							}

							break;

						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "первым";
								case WordGender.Neuter:
									return "первым";
								case WordGender.Feminine:
									return "первой";
							}

							break;

						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "первом";
								case WordGender.Neuter:
									return "первом";
								case WordGender.Feminine:
									return "первой";
							}

							break;
					}

					break;

				#endregion

				#region case   2:

				case 2:
					switch (numeralForm & WordForms.RU_Case_Mask)
					{
						case WordForms.RU_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "второй";
								case WordGender.Neuter:
									return "второе";
								case WordGender.Feminine:
									return "вторая";
							}

							break;
						case WordForms.RU_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "второго";
								case WordGender.Neuter:
									return "второго";
								case WordGender.Feminine:
									return "второй";
							}

							break;
						case WordForms.RU_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "второму";
								case WordGender.Neuter:
									return "второму";
								case WordGender.Feminine:
									return "второй";
							}

							break;
						case WordForms.RU_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "второго";
								case WordGender.Neuter:
									return "второе";
								case WordGender.Feminine:
									return "вторую";
							}

							break;
						case WordForms.RU_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "вторым";
								case WordGender.Neuter:
									return "вторым";
								case WordGender.Feminine:
									return "второй";
							}

							break;
						case WordForms.RU_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "втором";
								case WordGender.Neuter:
									return "втором";
								case WordGender.Feminine:
									return "второй";
							}

							break;
					}

					break;

				#endregion

				#region case   3:

				case 3:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "третий";
								case WordGender.Neuter:
									return "третье";
								case WordGender.Feminine:
									return "третья";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "третьего";
								case WordGender.Neuter:
									return "третьего";
								case WordGender.Feminine:
									return "третьей";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "третьему";
								case WordGender.Neuter:
									return "третьему";
								case WordGender.Feminine:
									return "третьей";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "третье";
								case WordGender.Neuter:
									return "третье";
								case WordGender.Feminine:
									return "третью";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "третьего";
								case WordGender.Neuter:
									return "третье";
								case WordGender.Feminine:
									return "третью";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "третьим";
								case WordGender.Neuter:
									return "третьим";
								case WordGender.Feminine:
									return "третьей";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "третьем";
								case WordGender.Neuter:
									return "третьем";
								case WordGender.Feminine:
									return "третьей";
							}

							break;
					}

					break;

				#endregion

				#region case   4:

				case 4:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четвёртый";
								case WordGender.Neuter:
									return "четвёртое";
								case WordGender.Feminine:
									return "четвёртая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четвёртого";
								case WordGender.Neuter:
									return "четвёртого";
								case WordGender.Feminine:
									return "четвёртую";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четвёртому";
								case WordGender.Neuter:
									return "четвёртому";
								case WordGender.Feminine:
									return "четвёртой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четвёртый";
								case WordGender.Neuter:
									return "четвёртое";
								case WordGender.Feminine:
									return "четвёртую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четвёртого";
								case WordGender.Neuter:
									return "четвёртое";
								case WordGender.Feminine:
									return "четвёртую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четвёртым";
								case WordGender.Neuter:
									return "четвёртым";
								case WordGender.Feminine:
									return "четвёртой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четвёртом";
								case WordGender.Neuter:
									return "четвёртом";
								case WordGender.Feminine:
									return "четвёртой";
							}

							break;
					}

					break;

				#endregion

				#region case   5:

				case 5:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятый";
								case WordGender.Neuter:
									return "пятое";
								case WordGender.Feminine:
									return "пятая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятого";
								case WordGender.Neuter:
									return "пятого";
								case WordGender.Feminine:
									return "пятой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятому";
								case WordGender.Neuter:
									return "пятому";
								case WordGender.Feminine:
									return "пятой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятый";
								case WordGender.Neuter:
									return "пятое";
								case WordGender.Feminine:
									return "пятую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятого";
								case WordGender.Neuter:
									return "пятое";
								case WordGender.Feminine:
									return "пятую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятым";
								case WordGender.Neuter:
									return "пятым";
								case WordGender.Feminine:
									return "пятой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятом";
								case WordGender.Neuter:
									return "пятом";
								case WordGender.Feminine:
									return "пятой";
							}

							break;
					}

					break;

				#endregion

				#region case   6:

				case 6:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестой";
								case WordGender.Neuter:
									return "шестое";
								case WordGender.Feminine:
									return "шестая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестого";
								case WordGender.Neuter:
									return "шестого";
								case WordGender.Feminine:
									return "шестой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестому";
								case WordGender.Neuter:
									return "шестому";
								case WordGender.Feminine:
									return "шестой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестой";
								case WordGender.Neuter:
									return "шестое";
								case WordGender.Feminine:
									return "шестую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестого";
								case WordGender.Neuter:
									return "шестое";
								case WordGender.Feminine:
									return "шестую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестым";
								case WordGender.Neuter:
									return "шестым";
								case WordGender.Feminine:
									return "шестой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестом";
								case WordGender.Neuter:
									return "шестом";
								case WordGender.Feminine:
									return "шестой";
							}

							break;
					}

					break;

				#endregion

				#region case   7:

				case 7:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "седьмой";
								case WordGender.Neuter:
									return "седьмое";
								case WordGender.Feminine:
									return "седьмая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "седьмого";
								case WordGender.Neuter:
									return "седьмого";
								case WordGender.Feminine:
									return "седьмой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "седьмому";
								case WordGender.Neuter:
									return "седьмому";
								case WordGender.Feminine:
									return "седьмой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семьмой";
								case WordGender.Neuter:
									return "седьмое";
								case WordGender.Feminine:
									return "седьмую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "седьмого";
								case WordGender.Neuter:
									return "седьмое";
								case WordGender.Feminine:
									return "седьмую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "седьмым";
								case WordGender.Neuter:
									return "седьмым";
								case WordGender.Feminine:
									return "седьмой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "седьмом";
								case WordGender.Neuter:
									return "седьмом";
								case WordGender.Feminine:
									return "седьмой";
							}

							break;
					}

					break;

				#endregion

				#region case   8:

				case 8:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "весьмой";
								case WordGender.Neuter:
									return "восьмое";
								case WordGender.Feminine:
									return "восьмая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмого";
								case WordGender.Neuter:
									return "восьмого";
								case WordGender.Feminine:
									return "восьмой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмому";
								case WordGender.Neuter:
									return "восьмому";
								case WordGender.Feminine:
									return "восьмой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмой";
								case WordGender.Neuter:
									return "восьмое";
								case WordGender.Feminine:
									return "восьмую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмого";
								case WordGender.Neuter:
									return "восьмое";
								case WordGender.Feminine:
									return "восьмую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмым";
								case WordGender.Neuter:
									return "восьмым";
								case WordGender.Feminine:
									return "восьмой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмом";
								case WordGender.Neuter:
									return "восьмом";
								case WordGender.Feminine:
									return "восьмой";
							}

							break;
					}

					break;

				#endregion

				#region case   9:

				case 9:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятый";
								case WordGender.Neuter:
									return "девятое";
								case WordGender.Feminine:
									return "девятая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятого";
								case WordGender.Neuter:
									return "девятого";
								case WordGender.Feminine:
									return "девятой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятому";
								case WordGender.Neuter:
									return "девятому";
								case WordGender.Feminine:
									return "девятой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятый";
								case WordGender.Neuter:
									return "девятое";
								case WordGender.Feminine:
									return "девятую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятого";
								case WordGender.Neuter:
									return "девятое";
								case WordGender.Feminine:
									return "девятую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятым";
								case WordGender.Neuter:
									return "девятым";
								case WordGender.Feminine:
									return "девятой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятом";
								case WordGender.Neuter:
									return "девятом";
								case WordGender.Feminine:
									return "девятой";
							}

							break;
					}

					break;

				#endregion

				#region case  10:

				case 10:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятый";
								case WordGender.Neuter:
									return "десятое";
								case WordGender.Feminine:
									return "десятая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятого";
								case WordGender.Neuter:
									return "десятого";
								case WordGender.Feminine:
									return "десятой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятому";
								case WordGender.Neuter:
									return "десятому";
								case WordGender.Feminine:
									return "десятой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятый";
								case WordGender.Neuter:
									return "десятое";
								case WordGender.Feminine:
									return "десятую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятого";
								case WordGender.Neuter:
									return "десятое";
								case WordGender.Feminine:
									return "десятую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятым";
								case WordGender.Neuter:
									return "десятым";
								case WordGender.Feminine:
									return "десятой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятом";
								case WordGender.Neuter:
									return "десятом";
								case WordGender.Feminine:
									return "десятой";
							}

							break;
					}

					break;

				#endregion

				#region case  11:

				case 11:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "одиннадцатый";
								case WordGender.Neuter:
									return "одиннадцатое";
								case WordGender.Feminine:
									return "одиннадцатая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "одиннадцатого";
								case WordGender.Neuter:
									return "одиннадцатого";
								case WordGender.Feminine:
									return "одиннадцатой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "одиннадцатому";
								case WordGender.Neuter:
									return "одиннадцатому";
								case WordGender.Feminine:
									return "одиннадцатой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "одиннадцатый";
								case WordGender.Neuter:
									return "одиннадцатое";
								case WordGender.Feminine:
									return "одиннадцатую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "одиннадцатого";
								case WordGender.Neuter:
									return "одиннадцатое";
								case WordGender.Feminine:
									return "одиннадцатую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "одиннадцатым";
								case WordGender.Neuter:
									return "одиннадцатым";
								case WordGender.Feminine:
									return "одиннадцатой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "одиннадцатом";
								case WordGender.Neuter:
									return "одиннадцатом";
								case WordGender.Feminine:
									return "одиннадцатой";
							}

							break;
					}

					break;

				#endregion

				#region case  12:

				case 12:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двенадцатый";
								case WordGender.Neuter:
									return "двенадцатое";
								case WordGender.Feminine:
									return "двенадцатая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двенадцатого";
								case WordGender.Neuter:
									return "двенадцатого";
								case WordGender.Feminine:
									return "двенадцатой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двенадцатому";
								case WordGender.Neuter:
									return "двенадцатому";
								case WordGender.Feminine:
									return "двенадцатой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двенадцатый";
								case WordGender.Neuter:
									return "двенадцатое";
								case WordGender.Feminine:
									return "двенадцатую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двенадцатого";
								case WordGender.Neuter:
									return "двенадцатое";
								case WordGender.Feminine:
									return "двенадцатую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двенадцатым";
								case WordGender.Neuter:
									return "двенадцатым";
								case WordGender.Feminine:
									return "двенадцатой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двенадцатом";
								case WordGender.Neuter:
									return "двенадцатом";
								case WordGender.Feminine:
									return "двенадцатой";
							}

							break;
					}

					break;

				#endregion

				#region case  13:

				case 13:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тринадцатый";
								case WordGender.Neuter:
									return "тринадцатое";
								case WordGender.Feminine:
									return "тринадцатая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тринадцатого";
								case WordGender.Neuter:
									return "тринадцатого";
								case WordGender.Feminine:
									return "тринадцатой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тринадцатому";
								case WordGender.Neuter:
									return "тринадцатому";
								case WordGender.Feminine:
									return "тринадцатой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тринадцатый";
								case WordGender.Neuter:
									return "тринадцатое";
								case WordGender.Feminine:
									return "тринадцатую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тринадцатого";
								case WordGender.Neuter:
									return "тринадцатое";
								case WordGender.Feminine:
									return "тринадцатую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тринадцатым";
								case WordGender.Neuter:
									return "тринадцатым";
								case WordGender.Feminine:
									return "тринадцатой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тринадцатом";
								case WordGender.Neuter:
									return "тринадцатом";
								case WordGender.Feminine:
									return "тринадцатой";
							}

							break;
					}

					break;

				#endregion

				#region case  14:

				case 14:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырнадцатый";
								case WordGender.Neuter:
									return "четырнадцатое";
								case WordGender.Feminine:
									return "четырнадцатая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырнадцатого";
								case WordGender.Neuter:
									return "четырнадцатого";
								case WordGender.Feminine:
									return "четырнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырнадцатому";
								case WordGender.Neuter:
									return "четырнадцатому";
								case WordGender.Feminine:
									return "четырнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырнадцатый";
								case WordGender.Neuter:
									return "четырнадцатое";
								case WordGender.Feminine:
									return "четырнадцатую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырнадцатого";
								case WordGender.Neuter:
									return "четырнадцатое";
								case WordGender.Feminine:
									return "четырнадцатую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырнадцатым";
								case WordGender.Neuter:
									return "четырнадцатым";
								case WordGender.Feminine:
									return "четырнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырнадцатом";
								case WordGender.Neuter:
									return "четырнадцатом";
								case WordGender.Feminine:
									return "четырнадцатой";
							}

							break;
					}

					break;

				#endregion

				#region case  15:

				case 15:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятнадцатый";
								case WordGender.Neuter:
									return "пятнадцатое";
								case WordGender.Feminine:
									return "пятнадцатая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятнадцатого";
								case WordGender.Neuter:
									return "пятнадцатого";
								case WordGender.Feminine:
									return "пятнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятнадцатому";
								case WordGender.Neuter:
									return "пятнадцатому";
								case WordGender.Feminine:
									return "пятнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятнадцатый";
								case WordGender.Neuter:
									return "пятнадцатое";
								case WordGender.Feminine:
									return "пятнадцатую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятнадцатого";
								case WordGender.Neuter:
									return "пятнадцатое";
								case WordGender.Feminine:
									return "пятнадцатую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятнадцатым";
								case WordGender.Neuter:
									return "пятнадцатым";
								case WordGender.Feminine:
									return "пятнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятнадцатом";
								case WordGender.Neuter:
									return "пятнадцатом";
								case WordGender.Feminine:
									return "пятнадцатой";
							}

							break;
					}

					break;

				#endregion

				#region case  16:

				case 16:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестнадцатый";
								case WordGender.Neuter:
									return "шестнадцатое";
								case WordGender.Feminine:
									return "шестнадцатая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестнадцатого";
								case WordGender.Neuter:
									return "шестнадцатого";
								case WordGender.Feminine:
									return "шестнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестнадцатому";
								case WordGender.Neuter:
									return "шестнадцатому";
								case WordGender.Feminine:
									return "шестнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестнадцатый";
								case WordGender.Neuter:
									return "шестнадцатое";
								case WordGender.Feminine:
									return "шестнадцатую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестнадцатого";
								case WordGender.Neuter:
									return "шестнадцатое";
								case WordGender.Feminine:
									return "шестнадцатую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестнадцатым";
								case WordGender.Neuter:
									return "шестнадцатым";
								case WordGender.Feminine:
									return "шестнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестнадцатом";
								case WordGender.Neuter:
									return "шестнадцатом";
								case WordGender.Feminine:
									return "шестнадцатой";
							}

							break;
					}

					break;

				#endregion

				#region case  17:

				case 17:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семнадцатый";
								case WordGender.Neuter:
									return "семнадцатое";
								case WordGender.Feminine:
									return "семнадцатая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семнадцатого";
								case WordGender.Neuter:
									return "семнадцатого";
								case WordGender.Feminine:
									return "семнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семнадцатому";
								case WordGender.Neuter:
									return "семнадцатому";
								case WordGender.Feminine:
									return "семнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семнадцатый";
								case WordGender.Neuter:
									return "семнадцатое";
								case WordGender.Feminine:
									return "семнадцатую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семнадцатого";
								case WordGender.Neuter:
									return "семнадцатое";
								case WordGender.Feminine:
									return "семнадцатую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семнадцатым";
								case WordGender.Neuter:
									return "семнадцатым";
								case WordGender.Feminine:
									return "семнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семнадцатом";
								case WordGender.Neuter:
									return "семнадцатом";
								case WordGender.Feminine:
									return "семнадцатой";
							}

							break;
					}

					break;

				#endregion

				#region case  18:

				case 18:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восемнадцатый";
								case WordGender.Neuter:
									return "восемнадцатое";
								case WordGender.Feminine:
									return "восемнадцатая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восемнадцатого";
								case WordGender.Neuter:
									return "восемнадцатого";
								case WordGender.Feminine:
									return "восемнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восемнадцатому";
								case WordGender.Neuter:
									return "восемнадцатому";
								case WordGender.Feminine:
									return "восемнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восемнадцатый";
								case WordGender.Neuter:
									return "восемнадцатое";
								case WordGender.Feminine:
									return "восемнадцатую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восемнадцатого";
								case WordGender.Neuter:
									return "восемнадцатое";
								case WordGender.Feminine:
									return "восемнадцатую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восемнадцатым";
								case WordGender.Neuter:
									return "восемнадцатым";
								case WordGender.Feminine:
									return "восемнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восемнадцатом";
								case WordGender.Neuter:
									return "восемнадцатом";
								case WordGender.Feminine:
									return "восемнадцатой";
							}

							break;
					}

					break;

				#endregion

				#region case  19:

				case 19:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятнадцатый";
								case WordGender.Neuter:
									return "девятнадцатое";
								case WordGender.Feminine:
									return "девятнадцатая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятнадцатого";
								case WordGender.Neuter:
									return "девятнадцатого";
								case WordGender.Feminine:
									return "девятнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятнадцатому";
								case WordGender.Neuter:
									return "девятнадцатому";
								case WordGender.Feminine:
									return "девятнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятнадцатый";
								case WordGender.Neuter:
									return "девятнадцатое";
								case WordGender.Feminine:
									return "девятнадцатую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятнадцатого";
								case WordGender.Neuter:
									return "девятнадцатое";
								case WordGender.Feminine:
									return "девятнадцатую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятнадцатым";
								case WordGender.Neuter:
									return "девятнадцатым";
								case WordGender.Feminine:
									return "девятнадцатой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятнадцатом";
								case WordGender.Neuter:
									return "девятнадцатом";
								case WordGender.Feminine:
									return "девятнадцатой";
							}

							break;
					}

					break;

				#endregion

				#region case  20:

				case 20:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатый";
								case WordGender.Neuter:
									return "двадцатое";
								case WordGender.Feminine:
									return "двадцатая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатого";
								case WordGender.Neuter:
									return "двадцатого";
								case WordGender.Feminine:
									return "двадцатой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатому";
								case WordGender.Neuter:
									return "двадцатому";
								case WordGender.Feminine:
									return "двадцатой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатый";
								case WordGender.Neuter:
									return "двадцатое";
								case WordGender.Feminine:
									return "двадцатую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатого";
								case WordGender.Neuter:
									return "двадцатое";
								case WordGender.Feminine:
									return "двадцатую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатым";
								case WordGender.Neuter:
									return "двадцатым";
								case WordGender.Feminine:
									return "двадцатой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатом";
								case WordGender.Neuter:
									return "двадцатом";
								case WordGender.Feminine:
									return "двадцатой";
							}

							break;
					}

					break;

				#endregion

				#region case  30:

				case 30:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатый";
								case WordGender.Neuter:
									return "тридцатое";
								case WordGender.Feminine:
									return "тридцатая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатого";
								case WordGender.Neuter:
									return "тридцатого";
								case WordGender.Feminine:
									return "тридцатой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатому";
								case WordGender.Neuter:
									return "тридцатому";
								case WordGender.Feminine:
									return "тридцатой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатый";
								case WordGender.Neuter:
									return "тридцатое";
								case WordGender.Feminine:
									return "тридцатую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатого";
								case WordGender.Neuter:
									return "тридцатое";
								case WordGender.Feminine:
									return "тридцатую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатым";
								case WordGender.Neuter:
									return "тридцатым";
								case WordGender.Feminine:
									return "тридцатой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатом";
								case WordGender.Neuter:
									return "тридцатом";
								case WordGender.Feminine:
									return "тридцатой";
							}

							break;
					}

					break;

				#endregion

				#region case  40:

				case 40:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сороковой";
								case WordGender.Neuter:
									return "сороковое";
								case WordGender.Feminine:
									return "сороковая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сорокового";
								case WordGender.Neuter:
									return "сорокового";
								case WordGender.Feminine:
									return "сороковой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сороковому";
								case WordGender.Neuter:
									return "сороковому";
								case WordGender.Feminine:
									return "сороковой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сороковой";
								case WordGender.Neuter:
									return "сороковое";
								case WordGender.Feminine:
									return "сороковую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сорокового";
								case WordGender.Neuter:
									return "сороковое";
								case WordGender.Feminine:
									return "сороковую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сороковым";
								case WordGender.Neuter:
									return "сороковым";
								case WordGender.Feminine:
									return "сороковой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сороковом";
								case WordGender.Neuter:
									return "сороковом";
								case WordGender.Feminine:
									return "сороковой";
							}

							break;
					}

					break;

				#endregion

				#region case  50:

				case 50:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятый";
								case WordGender.Neuter:
									return "пятидесятое";
								case WordGender.Feminine:
									return "пятидесятая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятого";
								case WordGender.Neuter:
									return "пятидесятого";
								case WordGender.Feminine:
									return "пятидесятой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятому";
								case WordGender.Neuter:
									return "пятидесятому";
								case WordGender.Feminine:
									return "пятидесятой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятый";
								case WordGender.Neuter:
									return "пятидесятое";
								case WordGender.Feminine:
									return "пятидесятую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятого";
								case WordGender.Neuter:
									return "пятидесятое";
								case WordGender.Feminine:
									return "пятидесятую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятым";
								case WordGender.Neuter:
									return "пятидесятым";
								case WordGender.Feminine:
									return "пятидесятой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятом";
								case WordGender.Neuter:
									return "пятидесятом";
								case WordGender.Feminine:
									return "пятидесятой";
							}

							break;
					}

					break;

				#endregion

				#region case  60:

				case 60:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятый";
								case WordGender.Neuter:
									return "шестидесятое";
								case WordGender.Feminine:
									return "шестидесятая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятого";
								case WordGender.Neuter:
									return "шестидесятого";
								case WordGender.Feminine:
									return "шестидесятой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятму";
								case WordGender.Neuter:
									return "шестидесятому";
								case WordGender.Feminine:
									return "шестидесятой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятый";
								case WordGender.Neuter:
									return "шестидесятое";
								case WordGender.Feminine:
									return "шестидесятую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятого";
								case WordGender.Neuter:
									return "шестидесятое";
								case WordGender.Feminine:
									return "шестидесятую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятым";
								case WordGender.Neuter:
									return "шестидесятым";
								case WordGender.Feminine:
									return "шестидесятой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятом";
								case WordGender.Neuter:
									return "шестидесятом";
								case WordGender.Feminine:
									return "шестидесятой";
							}

							break;
					}

					break;

				#endregion

				#region case  70:

				case 70:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятый";
								case WordGender.Neuter:
									return "семидесятое";
								case WordGender.Feminine:
									return "семидесятая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятого";
								case WordGender.Neuter:
									return "семидесятого";
								case WordGender.Feminine:
									return "семидесятой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятому";
								case WordGender.Neuter:
									return "семидесятому";
								case WordGender.Feminine:
									return "семидесятой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятый";
								case WordGender.Neuter:
									return "семидесяттое";
								case WordGender.Feminine:
									return "семидесятую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятого";
								case WordGender.Neuter:
									return "семидесятое";
								case WordGender.Feminine:
									return "семидесятую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятым";
								case WordGender.Neuter:
									return "семидесятым";
								case WordGender.Feminine:
									return "семидесятой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятом";
								case WordGender.Neuter:
									return "семидесятом";
								case WordGender.Feminine:
									return "семидесятой";
							}

							break;
					}

					break;

				#endregion

				#region case  80:

				case 80:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятый";
								case WordGender.Neuter:
									return "восьмидесятое";
								case WordGender.Feminine:
									return "восьмидесятая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятого";
								case WordGender.Neuter:
									return "восьмидесятого";
								case WordGender.Feminine:
									return "восьмидесятой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятому";
								case WordGender.Neuter:
									return "восьмидесятому";
								case WordGender.Feminine:
									return "восьмидесятой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятый";
								case WordGender.Neuter:
									return "восьмидесятое";
								case WordGender.Feminine:
									return "восьмидесятую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятого";
								case WordGender.Neuter:
									return "восьмидесятое";
								case WordGender.Feminine:
									return "восьмидесятую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятым";
								case WordGender.Neuter:
									return "восьмидесятым";
								case WordGender.Feminine:
									return "восьмидесятой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятом";
								case WordGender.Neuter:
									return "восьмидесятом";
								case WordGender.Feminine:
									return "восьмидесятой";
							}

							break;
					}

					break;

				#endregion

				#region case  90:

				case 90:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностый";
								case WordGender.Neuter:
									return "девяностое";
								case WordGender.Feminine:
									return "девяностая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностого";
								case WordGender.Neuter:
									return "девяностого";
								case WordGender.Feminine:
									return "девяностой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностому";
								case WordGender.Neuter:
									return "девяностому";
								case WordGender.Feminine:
									return "девяностой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностый";
								case WordGender.Neuter:
									return "девяностое";
								case WordGender.Feminine:
									return "девяностую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностого";
								case WordGender.Neuter:
									return "девяностое";
								case WordGender.Feminine:
									return "девяностую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностым";
								case WordGender.Neuter:
									return "девяностым";
								case WordGender.Feminine:
									return "девяностой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностом";
								case WordGender.Neuter:
									return "девяностом";
								case WordGender.Feminine:
									return "девяностой";
							}

							break;
					}

					break;

				#endregion

				#region case 100:

				case 100:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сотый";
								case WordGender.Neuter:
									return "сотое";
								case WordGender.Feminine:
									return "сотая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сотого";
								case WordGender.Neuter:
									return "сотого";
								case WordGender.Feminine:
									return "сотой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сотому";
								case WordGender.Neuter:
									return "сотому";
								case WordGender.Feminine:
									return "сотой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сотый";
								case WordGender.Neuter:
									return "сотое";
								case WordGender.Feminine:
									return "сотую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сотого";
								case WordGender.Neuter:
									return "сотое";
								case WordGender.Feminine:
									return "сотую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сотым";
								case WordGender.Neuter:
									return "сотым";
								case WordGender.Feminine:
									return "сотой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сотом";
								case WordGender.Neuter:
									return "сотом";
								case WordGender.Feminine:
									return "сотой";
							}

							break;
					}

					break;

				#endregion

				#region case 200:

				case 200:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухсотый";
								case WordGender.Neuter:
									return "двухсотое";
								case WordGender.Feminine:
									return "двухсотая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухсотого";
								case WordGender.Neuter:
									return "двухсотого";
								case WordGender.Feminine:
									return "двухсотой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухсотому";
								case WordGender.Neuter:
									return "двухсотому";
								case WordGender.Feminine:
									return "двухсотой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухсотый";
								case WordGender.Neuter:
									return "двухсотое";
								case WordGender.Feminine:
									return "двухсотую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухсотого";
								case WordGender.Neuter:
									return "двухсотое";
								case WordGender.Feminine:
									return "двухсотую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухсотым";
								case WordGender.Neuter:
									return "двухсотым";
								case WordGender.Feminine:
									return "двухсотой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухсотом";
								case WordGender.Neuter:
									return "двухсотом";
								case WordGender.Feminine:
									return "двухсотой";
							}

							break;
					}

					break;

				#endregion

				#region case 300:

				case 300:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхсотый";
								case WordGender.Neuter:
									return "трёхсотое";
								case WordGender.Feminine:
									return "трёхсотая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхсотого";
								case WordGender.Neuter:
									return "трёхсотого";
								case WordGender.Feminine:
									return "трёхсотой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхсотому";
								case WordGender.Neuter:
									return "трёхсотому";
								case WordGender.Feminine:
									return "трёхсотой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхсотый";
								case WordGender.Neuter:
									return "трёхсотое";
								case WordGender.Feminine:
									return "трёхсотую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхсотого";
								case WordGender.Neuter:
									return "трёхсотое";
								case WordGender.Feminine:
									return "трёхсотую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхсотым";
								case WordGender.Neuter:
									return "трёхсотым";
								case WordGender.Feminine:
									return "трёхсотой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхсотом";
								case WordGender.Neuter:
									return "трёхсотом";
								case WordGender.Feminine:
									return "трёхсотой";
							}

							break;
					}

					break;

				#endregion

				#region case 400:

				case 400:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхсотый";
								case WordGender.Neuter:
									return "четырёхсотое";
								case WordGender.Feminine:
									return "четырёхсотая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхсотого";
								case WordGender.Neuter:
									return "четырёхсотого";
								case WordGender.Feminine:
									return "четырёхсотой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхсотому";
								case WordGender.Neuter:
									return "четырёхсотому";
								case WordGender.Feminine:
									return "четырёхсотой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхсотый";
								case WordGender.Neuter:
									return "четырёхсотое";
								case WordGender.Feminine:
									return "четырёхсотую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхсотого";
								case WordGender.Neuter:
									return "четырёхсотое";
								case WordGender.Feminine:
									return "четырёхсотую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхсотым";
								case WordGender.Neuter:
									return "четырёхсотым";
								case WordGender.Feminine:
									return "четырёхсотой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхсотом";
								case WordGender.Neuter:
									return "четырёхсотом";
								case WordGender.Feminine:
									return "четырёхсотой";
							}

							break;
					}

					break;

				#endregion

				#region case 500:

				case 500:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятисотый";
								case WordGender.Neuter:
									return "пятисотое";
								case WordGender.Feminine:
									return "пятисотая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятисотого";
								case WordGender.Neuter:
									return "пятисотого";
								case WordGender.Feminine:
									return "пятисотой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятисотому";
								case WordGender.Neuter:
									return "пятисотому";
								case WordGender.Feminine:
									return "пятисотой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятисотый";
								case WordGender.Neuter:
									return "пятисотое";
								case WordGender.Feminine:
									return "пятисотую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятисотого";
								case WordGender.Neuter:
									return "пятисотое";
								case WordGender.Feminine:
									return "пятисотую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятисотым";
								case WordGender.Neuter:
									return "пятисотым";
								case WordGender.Feminine:
									return "пятисотой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятисотом";
								case WordGender.Neuter:
									return "пятисотом";
								case WordGender.Feminine:
									return "пятисотой";
							}

							break;
					}

					break;

				#endregion

				#region case 600:

				case 600:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестисотый";
								case WordGender.Neuter:
									return "шестисотое";
								case WordGender.Feminine:
									return "шестисотая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестисотого";
								case WordGender.Neuter:
									return "шестисотого";
								case WordGender.Feminine:
									return "шестисотой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестисотому";
								case WordGender.Neuter:
									return "шестисотому";
								case WordGender.Feminine:
									return "шестисотой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестисотый";
								case WordGender.Neuter:
									return "шестисотое";
								case WordGender.Feminine:
									return "шестисотую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестисотого";
								case WordGender.Neuter:
									return "шестисотое";
								case WordGender.Feminine:
									return "шестисотую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестисотым";
								case WordGender.Neuter:
									return "шестисотым";
								case WordGender.Feminine:
									return "шестисотой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестисотом";
								case WordGender.Neuter:
									return "шестисотом";
								case WordGender.Feminine:
									return "шестисотой";
							}

							break;
					}

					break;

				#endregion

				#region case 700:

				case 700:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семисотый";
								case WordGender.Neuter:
									return "семисотое";
								case WordGender.Feminine:
									return "семисотая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семисотого";
								case WordGender.Neuter:
									return "семисотого";
								case WordGender.Feminine:
									return "семисотой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семисотому";
								case WordGender.Neuter:
									return "семисотому";
								case WordGender.Feminine:
									return "семисотой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семисотый";
								case WordGender.Neuter:
									return "семисотое";
								case WordGender.Feminine:
									return "семисотую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семисотого";
								case WordGender.Neuter:
									return "семисотое";
								case WordGender.Feminine:
									return "семисотую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семисотым";
								case WordGender.Neuter:
									return "семисотым";
								case WordGender.Feminine:
									return "семисотой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семисотом";
								case WordGender.Neuter:
									return "семисотом";
								case WordGender.Feminine:
									return "семисотой";
							}

							break;
					}

					break;

				#endregion

				#region case 800:

				case 800:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмисотый";
								case WordGender.Neuter:
									return "восьмисотое";
								case WordGender.Feminine:
									return "восьмисотая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмисотого";
								case WordGender.Neuter:
									return "восьмисотого";
								case WordGender.Feminine:
									return "восьмисотой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмисотому";
								case WordGender.Neuter:
									return "восьмисотому";
								case WordGender.Feminine:
									return "восьмисотой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмисотый";
								case WordGender.Neuter:
									return "восьмисотое";
								case WordGender.Feminine:
									return "восьмисотую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмисотого";
								case WordGender.Neuter:
									return "восьмисотое";
								case WordGender.Feminine:
									return "восьмисотую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмисотым";
								case WordGender.Neuter:
									return "восьмисотым";
								case WordGender.Feminine:
									return "восьмисотой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмисотом";
								case WordGender.Neuter:
									return "восьмисотом";
								case WordGender.Feminine:
									return "восьмисотой";
							}

							break;
					}

					break;

				#endregion

				#region case 900:

				case 900:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятисотый";
								case WordGender.Neuter:
									return "девятисотое";
								case WordGender.Feminine:
									return "девятисотая";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятисотого";
								case WordGender.Neuter:
									return "девятисотого";
								case WordGender.Feminine:
									return "девятисотой";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятисотому";
								case WordGender.Neuter:
									return "девятисотому";
								case WordGender.Feminine:
									return "девятисотой";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятисотый";
								case WordGender.Neuter:
									return "девятисотое";
								case WordGender.Feminine:
									return "девятисотую";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятисотого";
								case WordGender.Neuter:
									return "девятисотое";
								case WordGender.Feminine:
									return "девятисотую";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятисотым";
								case WordGender.Neuter:
									return "девятисотым";
								case WordGender.Feminine:
									return "девятисотой";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятисотом";
								case WordGender.Neuter:
									return "девятисотом";
								case WordGender.Feminine:
									return "девятисотой";
							}

							break;
					}

					break;

				#endregion

				#region case 1000:

				case 1000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тысячный";
								case WordGender.Neuter:
									return "тысячное";
								case WordGender.Feminine:
									return "тысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тысячного";
								case WordGender.Neuter:
									return "тысячного";
								case WordGender.Feminine:
									return "тысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тысячному";
								case WordGender.Neuter:
									return "тысячному";
								case WordGender.Feminine:
									return "тысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тысячный";
								case WordGender.Neuter:
									return "тысячное";
								case WordGender.Feminine:
									return "тысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тысячного";
								case WordGender.Neuter:
									return "тысячное";
								case WordGender.Feminine:
									return "тысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тысячным";
								case WordGender.Neuter:
									return "тысячным";
								case WordGender.Feminine:
									return "тысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тысячном";
								case WordGender.Neuter:
									return "тысячном";
								case WordGender.Feminine:
									return "тысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 2000:

				case 2000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухтысячный";
								case WordGender.Neuter:
									return "двухтысячное";
								case WordGender.Feminine:
									return "двухтысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухтысячного";
								case WordGender.Neuter:
									return "двухтысячного";
								case WordGender.Feminine:
									return "двухтысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухтысячному";
								case WordGender.Neuter:
									return "двухтысячному";
								case WordGender.Feminine:
									return "двухтысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухтысячный";
								case WordGender.Neuter:
									return "двухтысячное";
								case WordGender.Feminine:
									return "двухтысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухтысячного";
								case WordGender.Neuter:
									return "двухтысячное";
								case WordGender.Feminine:
									return "двухтысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухтысячным";
								case WordGender.Neuter:
									return "двухтысячным";
								case WordGender.Feminine:
									return "двухтысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двухтысячном";
								case WordGender.Neuter:
									return "двухтысячном";
								case WordGender.Feminine:
									return "двухтысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 3000:

				case 3000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхтысячный";
								case WordGender.Neuter:
									return "трёхтысячное";
								case WordGender.Feminine:
									return "трёхтысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхтысячного";
								case WordGender.Neuter:
									return "трёхтысячного";
								case WordGender.Feminine:
									return "трёхтысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхтысячному";
								case WordGender.Neuter:
									return "трёхтысячному";
								case WordGender.Feminine:
									return "трёхтысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхтысячный";
								case WordGender.Neuter:
									return "трёхтысячное";
								case WordGender.Feminine:
									return "трёхтысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхтысячного";
								case WordGender.Neuter:
									return "трёхтысячное";
								case WordGender.Feminine:
									return "трёхтысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхтысячным";
								case WordGender.Neuter:
									return "трёхтысячным";
								case WordGender.Feminine:
									return "трёхтысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "трёхтысячном";
								case WordGender.Neuter:
									return "трёхтысячном";
								case WordGender.Feminine:
									return "трёхтысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 4000:

				case 4000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхтысячный";
								case WordGender.Neuter:
									return "четырёхтысячное";
								case WordGender.Feminine:
									return "четырёхтысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхтысячного";
								case WordGender.Neuter:
									return "четырёхтысячного";
								case WordGender.Feminine:
									return "четырёхтысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхтысячному";
								case WordGender.Neuter:
									return "четырёхтысячному";
								case WordGender.Feminine:
									return "четырёхтысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхтысячный";
								case WordGender.Neuter:
									return "четырёхтысячное";
								case WordGender.Feminine:
									return "четырёхтысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхтысячного";
								case WordGender.Neuter:
									return "четырёхтысячное";
								case WordGender.Feminine:
									return "четырёхтысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхтысячным";
								case WordGender.Neuter:
									return "четырёхтысячным";
								case WordGender.Feminine:
									return "четырёхтысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "четырёхтысячном";
								case WordGender.Neuter:
									return "четырёхтысячном";
								case WordGender.Feminine:
									return "четырёхтысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 5000:

				case 5000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятитысячный";
								case WordGender.Neuter:
									return "пятитысячное";
								case WordGender.Feminine:
									return "пятитысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятитысячного";
								case WordGender.Neuter:
									return "пятитысячного";
								case WordGender.Feminine:
									return "пятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятитысячному";
								case WordGender.Neuter:
									return "пятитысячному";
								case WordGender.Feminine:
									return "пятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятитысячный";
								case WordGender.Neuter:
									return "пятитысячное";
								case WordGender.Feminine:
									return "пятитысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятитысячного";
								case WordGender.Neuter:
									return "пятитысячное";
								case WordGender.Feminine:
									return "пятитысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятитысячным";
								case WordGender.Neuter:
									return "пятитысячным";
								case WordGender.Feminine:
									return "пятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятитысячном";
								case WordGender.Neuter:
									return "пятитысячном";
								case WordGender.Feminine:
									return "пятитысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 6000:

				case 6000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шеститысячный";
								case WordGender.Neuter:
									return "шеститысячное";
								case WordGender.Feminine:
									return "шеститысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шеститысячного";
								case WordGender.Neuter:
									return "шеститысячного";
								case WordGender.Feminine:
									return "шеститысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шеститысячному";
								case WordGender.Neuter:
									return "шеститысячному";
								case WordGender.Feminine:
									return "шеститысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шеститысячный";
								case WordGender.Neuter:
									return "шеститысячное";
								case WordGender.Feminine:
									return "шеститысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шеститысячного";
								case WordGender.Neuter:
									return "шеститысячное";
								case WordGender.Feminine:
									return "шеститысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шеститысячным";
								case WordGender.Neuter:
									return "шеститысячным";
								case WordGender.Feminine:
									return "шеститысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шеститысячном";
								case WordGender.Neuter:
									return "шеститысячном";
								case WordGender.Feminine:
									return "шеститысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 7000:

				case 7000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семитысячный";
								case WordGender.Neuter:
									return "семитысячное";
								case WordGender.Feminine:
									return "семитысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семитысячного";
								case WordGender.Neuter:
									return "семитысячного";
								case WordGender.Feminine:
									return "семитысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семитысячному";
								case WordGender.Neuter:
									return "семитысячному";
								case WordGender.Feminine:
									return "семитысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семитысячный";
								case WordGender.Neuter:
									return "семитысячное";
								case WordGender.Feminine:
									return "семитысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семитысячного";
								case WordGender.Neuter:
									return "семитысячное";
								case WordGender.Feminine:
									return "семитысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семитысячным";
								case WordGender.Neuter:
									return "семитысячным";
								case WordGender.Feminine:
									return "семитысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семитысячном";
								case WordGender.Neuter:
									return "семитысячном";
								case WordGender.Feminine:
									return "семитысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 8000:

				case 8000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмитысячный";
								case WordGender.Neuter:
									return "восьмитысячное";
								case WordGender.Feminine:
									return "восьмитысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмитысячного";
								case WordGender.Neuter:
									return "восьмитысячного";
								case WordGender.Feminine:
									return "восьмитысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмитысячному";
								case WordGender.Neuter:
									return "восьмитысячному";
								case WordGender.Feminine:
									return "восьмитысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмитысячный";
								case WordGender.Neuter:
									return "восьмитысячное";
								case WordGender.Feminine:
									return "восьмитысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмитысячного";
								case WordGender.Neuter:
									return "восьмитысячное";
								case WordGender.Feminine:
									return "восьмитысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмитысячным";
								case WordGender.Neuter:
									return "восьмитысячным";
								case WordGender.Feminine:
									return "восьмитысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмитысячном";
								case WordGender.Neuter:
									return "восьмитысячном";
								case WordGender.Feminine:
									return "восьмитысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 9000:

				case 9000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятитысячный";
								case WordGender.Neuter:
									return "девятитысячное";
								case WordGender.Feminine:
									return "девятитысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятитысячного";
								case WordGender.Neuter:
									return "девятитысячного";
								case WordGender.Feminine:
									return "девятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятитысячному";
								case WordGender.Neuter:
									return "девятитысячному";
								case WordGender.Feminine:
									return "девятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятитысячный";
								case WordGender.Neuter:
									return "девятитысячное";
								case WordGender.Feminine:
									return "девятитысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятитысячного";
								case WordGender.Neuter:
									return "девятитысячное";
								case WordGender.Feminine:
									return "девятитысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятитысячным";
								case WordGender.Neuter:
									return "девятитысячным";
								case WordGender.Feminine:
									return "девятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девятитысячном";
								case WordGender.Neuter:
									return "девятитысячном";
								case WordGender.Feminine:
									return "девятитысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 10000:

				case 10000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятитысячный";
								case WordGender.Neuter:
									return "десятитысячное";
								case WordGender.Feminine:
									return "десятитысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятитысячного";
								case WordGender.Neuter:
									return "десятитысячного";
								case WordGender.Feminine:
									return "десятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятитысячному";
								case WordGender.Neuter:
									return "десятитысячному";
								case WordGender.Feminine:
									return "десятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятитысячный";
								case WordGender.Neuter:
									return "десятитысячное";
								case WordGender.Feminine:
									return "десятитысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятитысячного";
								case WordGender.Neuter:
									return "десятитысячное";
								case WordGender.Feminine:
									return "десятитысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятитысячным";
								case WordGender.Neuter:
									return "десятитысячным";
								case WordGender.Feminine:
									return "десятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "десятитысячном";
								case WordGender.Neuter:
									return "десятитысячном";
								case WordGender.Feminine:
									return "десятитысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 20000:

				case 20000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатитысячный";
								case WordGender.Neuter:
									return "двадцатитысячное";
								case WordGender.Feminine:
									return "двадцатитысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатитысячного";
								case WordGender.Neuter:
									return "двадцатитысячного";
								case WordGender.Feminine:
									return "двадцатитысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатитысячному";
								case WordGender.Neuter:
									return "двадцатитысячному";
								case WordGender.Feminine:
									return "двадцатитысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатитысячный";
								case WordGender.Neuter:
									return "двадцатитысячное";
								case WordGender.Feminine:
									return "двадцатитысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатитысячного";
								case WordGender.Neuter:
									return "двадцатитысячное";
								case WordGender.Feminine:
									return "двадцатитысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатитысячным";
								case WordGender.Neuter:
									return "двадцатитысячным";
								case WordGender.Feminine:
									return "двадцатитысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "двадцатитысячном";
								case WordGender.Neuter:
									return "двадцатитысячном";
								case WordGender.Feminine:
									return "двадцатитысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 30000:

				case 30000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатитысячный";
								case WordGender.Neuter:
									return "тридцатитысячное";
								case WordGender.Feminine:
									return "тридцатитысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатитысячного";
								case WordGender.Neuter:
									return "тридцатитысячного";
								case WordGender.Feminine:
									return "тридцатитысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатитысячному";
								case WordGender.Neuter:
									return "тридцатитысячному";
								case WordGender.Feminine:
									return "тридцатитысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатитысячный";
								case WordGender.Neuter:
									return "тридцатитысячное";
								case WordGender.Feminine:
									return "тридцатитысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатитысячного";
								case WordGender.Neuter:
									return "тридцатитысячное";
								case WordGender.Feminine:
									return "тридцатитысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатитысячным";
								case WordGender.Neuter:
									return "тридцатитысячным";
								case WordGender.Feminine:
									return "тридцатитысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "тридцатитысячном";
								case WordGender.Neuter:
									return "тридцатитысячном";
								case WordGender.Feminine:
									return "тридцатитысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 40000:

				case 40000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сорокатысячный";
								case WordGender.Neuter:
									return "сорокатысячное";
								case WordGender.Feminine:
									return "сорокатысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сорокатысячного";
								case WordGender.Neuter:
									return "сорокатысячного";
								case WordGender.Feminine:
									return "сорокатысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сорокатысячному";
								case WordGender.Neuter:
									return "сорокатысячному";
								case WordGender.Feminine:
									return "сорокатысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сорокатысячный";
								case WordGender.Neuter:
									return "сорокатысячное";
								case WordGender.Feminine:
									return "сорокатысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сорокатысячного";
								case WordGender.Neuter:
									return "сорокатысячное";
								case WordGender.Feminine:
									return "сорокатысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сорокатысячным";
								case WordGender.Neuter:
									return "сорокатысячным";
								case WordGender.Feminine:
									return "сорокатысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "сорокатысячном";
								case WordGender.Neuter:
									return "сорокатысячном";
								case WordGender.Feminine:
									return "сорокатысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 50000:

				case 50000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятитысячный";
								case WordGender.Neuter:
									return "пятидесятитысячное";
								case WordGender.Feminine:
									return "пятидесятитысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятитысячного";
								case WordGender.Neuter:
									return "пятидесятитысячного";
								case WordGender.Feminine:
									return "пятидесятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятитысячному";
								case WordGender.Neuter:
									return "пятидесятитысячному";
								case WordGender.Feminine:
									return "пятидесятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятитысячный";
								case WordGender.Neuter:
									return "пятидесятитысячное";
								case WordGender.Feminine:
									return "пятидесятитысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятитысячного";
								case WordGender.Neuter:
									return "пятидесятитысячное";
								case WordGender.Feminine:
									return "пятидесятитысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятитысячным";
								case WordGender.Neuter:
									return "пятидесятитысячным";
								case WordGender.Feminine:
									return "пятидесятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "пятидесятитысячном";
								case WordGender.Neuter:
									return "пятидесятитысячном";
								case WordGender.Feminine:
									return "пятидесятитысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 60000:

				case 60000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятитысячный";
								case WordGender.Neuter:
									return "шестидесятитысячное";
								case WordGender.Feminine:
									return "шестидесятитысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятитысячного";
								case WordGender.Neuter:
									return "шестидесятитысячного";
								case WordGender.Feminine:
									return "шестидесятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятитысячному";
								case WordGender.Neuter:
									return "шестидесятитысячному";
								case WordGender.Feminine:
									return "шестидесятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятитысячный";
								case WordGender.Neuter:
									return "шестидесятитысячное";
								case WordGender.Feminine:
									return "шестидесятитысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятитысячного";
								case WordGender.Neuter:
									return "шестидесятитысячное";
								case WordGender.Feminine:
									return "шестидесятитысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятитысячным";
								case WordGender.Neuter:
									return "шестидесятитысячным";
								case WordGender.Feminine:
									return "шестидесятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "шестидесятитысячном";
								case WordGender.Neuter:
									return "шестидесятитысячном";
								case WordGender.Feminine:
									return "шестидесятитысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 70000:

				case 70000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятитысячный";
								case WordGender.Neuter:
									return "семидесятитысячное";
								case WordGender.Feminine:
									return "семидесятитысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятитысячного";
								case WordGender.Neuter:
									return "семидесятитысячного";
								case WordGender.Feminine:
									return "семидесятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятитысячному";
								case WordGender.Neuter:
									return "семидесятитысячному";
								case WordGender.Feminine:
									return "семидесятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятитысячный";
								case WordGender.Neuter:
									return "семидесятитысячное";
								case WordGender.Feminine:
									return "семидесятитысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятитысячного";
								case WordGender.Neuter:
									return "семидесятитысячное";
								case WordGender.Feminine:
									return "семидесятитысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятитысячным";
								case WordGender.Neuter:
									return "семидесятитысячным";
								case WordGender.Feminine:
									return "семидесятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "семидесятитысячном";
								case WordGender.Neuter:
									return "семидесятитысячном";
								case WordGender.Feminine:
									return "семидесятитысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 80000:

				case 80000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятитысячный";
								case WordGender.Neuter:
									return "восьмидесятитысячное";
								case WordGender.Feminine:
									return "восьмидесятитысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятитысячного";
								case WordGender.Neuter:
									return "восьмидесятитысячного";
								case WordGender.Feminine:
									return "восьмидесятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятитысячному";
								case WordGender.Neuter:
									return "восьмидесятитысячному";
								case WordGender.Feminine:
									return "восьмидесятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятитысячный";
								case WordGender.Neuter:
									return "восьмидесятитысячное";
								case WordGender.Feminine:
									return "восьмидесятитысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятитысячного";
								case WordGender.Neuter:
									return "восьмидесятитысячное";
								case WordGender.Feminine:
									return "восьмидесятитысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятитысячным";
								case WordGender.Neuter:
									return "восьмидесятитысячным";
								case WordGender.Feminine:
									return "восьмидесятитысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "восьмидесятитысячном";
								case WordGender.Neuter:
									return "восьмидесятитысячном";
								case WordGender.Feminine:
									return "восьмидесятитысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 90000:

				case 90000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностотысячный";
								case WordGender.Neuter:
									return "девяностотысячное";
								case WordGender.Feminine:
									return "девяностотысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностотысячного";
								case WordGender.Neuter:
									return "девяностотысячного";
								case WordGender.Feminine:
									return "девяностотысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностотысячному";
								case WordGender.Neuter:
									return "девяностотысячному";
								case WordGender.Feminine:
									return "девяностотысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностотысячный";
								case WordGender.Neuter:
									return "девяностотысячное";
								case WordGender.Feminine:
									return "девяностотысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностотысячного";
								case WordGender.Neuter:
									return "девяностотысячное";
								case WordGender.Feminine:
									return "девяностотысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностотысячным";
								case WordGender.Neuter:
									return "девяностотысячным";
								case WordGender.Feminine:
									return "девяностотысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "девяностотысячном";
								case WordGender.Neuter:
									return "девяностотысячном";
								case WordGender.Feminine:
									return "девяностотысячной";
							}

							break;
					}

					break;

				#endregion

				#region case 100000:

				case 100000:
					switch (numeralForm)
					{
						case WordForms.RU_Inanimate_SubjectiveCase:
						case WordForms.RU_Animate_SubjectiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "стотысячный";
								case WordGender.Neuter:
									return "стотысячное";
								case WordGender.Feminine:
									return "стотысячная";
							}

							break;
						case WordForms.RU_Inanimate_GenitiveCase:
						case WordForms.RU_Animate_GenitiveCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "стотысячного";
								case WordGender.Neuter:
									return "стотысячного";
								case WordGender.Feminine:
									return "стотысячной";
							}

							break;
						case WordForms.RU_Inanimate_DativeCase:
						case WordForms.RU_Animate_DativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "стотысячному";
								case WordGender.Neuter:
									return "стотысячному";
								case WordGender.Feminine:
									return "стотысячной";
							}

							break;
						case WordForms.RU_Inanimate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "стотысячный";
								case WordGender.Neuter:
									return "стотысячное";
								case WordGender.Feminine:
									return "стотысячную";
							}

							break;
						case WordForms.RU_Animate_AccusativeCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "стотысячного";
								case WordGender.Neuter:
									return "стотысячное";
								case WordGender.Feminine:
									return "стотысячную";
							}

							break;
						case WordForms.RU_Inanimate_InstrumentalCase:
						case WordForms.RU_Animate_InstrumentalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "стотысячным";
								case WordGender.Neuter:
									return "стотысячным";
								case WordGender.Feminine:
									return "стотысячной";
							}

							break;
						case WordForms.RU_Inanimate_PrepositionalCase:
						case WordForms.RU_Animate_PrepositionalCase:
							switch (numeralGender)
							{
								case WordGender.Masculine:
									return "стотысячном";
								case WordGender.Neuter:
									return "стотысячном";
								case WordGender.Feminine:
									return "стотысячной";
							}

							break;
					}

					break;

				#endregion
			}

			return string.Format(
				this._numberProvider,
				"[{0}]",
				numeralValue);
		}

		[SuppressMessage("Microsoft.Usage", "CA1801", MessageId = "suppressExtraNumbers")]
		[SuppressMessage("Microsoft.Performance", "CA1822")]
		private string OrdinalNameFromValueInternal(
			long numeralValue,
			WordGender numeralGender,
			WordForms numeralForm,
			bool suppressExtraNumbers,
			long scale)
		{
			string result = string.Format(
				this._numberProvider,
				"[{0}]",
				numeralValue);
			long numeralValuePart0 = (numeralValue / 1) % 10;
			long numeralValuePart1 = (numeralValue / 10) % 10;
			long numeralValuePart2 = (numeralValue / 100) % 10;

			if (numeralValuePart2 == 0)
			{
				if (numeralValuePart1 == 0)
				{
					result = OrdinalNameFromValue(scale * numeralValuePart0, numeralGender, numeralForm);
				}
				else if (numeralValuePart1 == 1)
				{
					result = OrdinalNameFromValue(10 + numeralValuePart0, numeralGender, numeralForm);
				}
				else
				{
					if (numeralValuePart0 == 0)
					{
						result = OrdinalNameFromValue(10 * numeralValuePart1, numeralGender, numeralForm);
					}
					else
					{
						result =
							CardinalNameFromValue(10 * numeralValuePart1, numeralGender, numeralForm) +
							" " +
							OrdinalNameFromValue(numeralValuePart0, numeralGender, numeralForm);
					}
				}
			}
			else
			{
				if (numeralValuePart1 == 0)
				{
					if (numeralValuePart0 == 0)
					{
						result = OrdinalNameFromValue(100 * numeralValuePart2, numeralGender, numeralForm);
					}
					else
					{
						result =
							CardinalNameFromValue(100 * numeralValuePart2, numeralGender, numeralForm) +
							" " +
							OrdinalNameFromValue(numeralValuePart0, numeralGender, numeralForm);
					}
				}
				else if (numeralValuePart1 == 1)
				{
					result =
						CardinalNameFromValue(100 * numeralValuePart2, numeralGender, numeralForm) +
						" " +
						OrdinalNameFromValue(10 + numeralValuePart0, numeralGender, numeralForm);
				}
				else
				{
					if (numeralValuePart0 == 0)
					{
						result =
							CardinalNameFromValue(100 * numeralValuePart2, numeralGender, numeralForm) +
							" " +
							OrdinalNameFromValue(10 * numeralValuePart1, numeralGender, numeralForm);
					}
					else
					{
						result =
							CardinalNameFromValue(100 * numeralValuePart2, numeralGender, numeralForm) +
							" " +
							CardinalNameFromValue(10 * numeralValuePart1, numeralGender, numeralForm) +
							" " +
							OrdinalNameFromValue(numeralValuePart0, numeralGender, numeralForm);
					}
				}
			}

			return result;
		}

		public string OrdinalNameFromValue(
			long numeralValue,
			WordGender numeralGender,
			WordForms numeralForm,
			bool suppressExtraNumbers)
		{
			long numeralValuePart0 = (numeralValue / 1) % 1000;
			long numeralValuePart3 = (numeralValue / 1000) % 1000;
			long numeralValuePart6 = (numeralValue / 1000000) % 1000;
			string result = string.Empty;

			if (numeralValuePart6 > 0)
			{
				if (
					(numeralValuePart3 > 0) ||
					(numeralValuePart0 > 0))
				{
					result +=
						((result.Length == 0) ? string.Empty : " ") +
						CardinalNameFromValueInternal(
							numeralValuePart6,
							numeralGender,
							numeralForm,
							suppressExtraNumbers,
							UnitFormsMillion);
				}
				else
				{
					result +=
						((result.Length == 0) ? string.Empty : " ") +
						OrdinalNameFromValueInternal(
							numeralValuePart6,
							numeralGender,
							numeralForm,
							suppressExtraNumbers,
							1000000);
				}

				suppressExtraNumbers = false;
			}

			if (numeralValuePart3 > 0)
			{
				if (numeralValuePart0 > 0)
				{
					result +=
						((result.Length == 0) ? string.Empty : " ") +
						CardinalNameFromValueInternal(
							numeralValuePart3,
							numeralGender,
							numeralForm,
							suppressExtraNumbers,
							UnitFormsThousand);
				}
				else
				{
					result +=
						((result.Length == 0) ? string.Empty : " ") +
						OrdinalNameFromValueInternal(
							numeralValuePart3,
							numeralGender,
							numeralForm,
							suppressExtraNumbers,
							1000);
				}

				suppressExtraNumbers = false;
			}

			if ((numeralValuePart0 > 0) || (result.Length == 0))
			{
				result +=
					((result.Length == 0) ? string.Empty : " ") +
					OrdinalNameFromValueInternal(
						numeralValuePart0,
						numeralGender,
						numeralForm,
						suppressExtraNumbers,
						1);
			}

			return result;
		}

		#endregion

		#endregion

		#endregion

		#region NumeralFormatter

		public override string NumeralNameFromValue(
			long numeralValue,
			NumeralType numeralType,
			WordGender numeralGender,
			WordForms numeralForm,
			bool suppressExtraNumbers)
		{
			switch (numeralType)
			{
				case NumeralType.Cardinal:
					return CardinalStringFromNumber(
						numeralValue,
						numeralGender,
						numeralForm,
						suppressExtraNumbers);
				case NumeralType.Ordinal:
					return OrdinalNameFromValue(
						numeralValue,
						numeralGender,
						numeralForm,
						suppressExtraNumbers);
			}

			return string.Format(
				this._numberProvider,
				"[{0}]",
				numeralValue);
		}

		public override WordForms NounFormFromNumeral(
			long numeralValue,
			NumeralType numeralType,
			WordGender numeralGender,
			WordForms numeralForm)
		{
			switch (numeralType)
			{
				case NumeralType.Cardinal:
					return CardinalNounFormFromValueInternal(
						(numeralValue / 1) % 10,
						(numeralValue / 10) % 10,
						numeralForm);
				case NumeralType.Ordinal:
					return numeralForm;
			}

			return WordForms.RU_Default;
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
