﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Text Library                             */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;

namespace Nabu.Text
{
	/*
	|------------------------------------------------|
	| Именительный | Subjective    | Кто?   | Что?   |
	|------------------------------|--------|--------|
	| Родительный  | Genitive      | Кого?  | Чего?  |
	|------------------------------|--------|--------|
	| Дательный    | Dative        | Кому?  | Чему?  |
	|------------------------------|--------|--------|
	| Винительный  | Accusative    | Кого?  | Что?   |
	|------------------------------|--------|--------|
	| Творительный | Instrumental  | Кем?   | Чем?   |
	|------------------------------|--------|--------|
	| Предложный   | Prepositional | О ком? | О чём? |
	|------------------------------------------------|
	*/

	[Flags]
	public enum WordForms : int
	{
		None = 0x0000,

		Unused1 = 0x0001,
		Unused2 = 0x0002,
		Unused3 = 0x0004,
		Unused4 = 0x0008,
		Unused5 = 0x0010,
		Unused6 = 0x0020,
		Unused7 = 0x0040,
		Unused8 = 0x0080,

		EN_Default /*  */ = 0x0001,
		EN_Singular /* */ = 0x0001,
		EN_Plural /*   */ = 0x0002,

		RU_Default = 0x0001,

		RU_Case_Mask = 0x000F,

		RU_SubjectiveCase /*    */ = 0x0001,
		RU_GenitiveCase /*      */ = 0x0002,
		RU_DativeCase /*        */ = 0x0003,
		RU_AccusativeCase /*    */ = 0x0004,
		RU_InstrumentalCase /*  */ = 0x0005,
		RU_PrepositionalCase /* */ = 0x0006,

		RU_Singular_SubjectiveCase /*    */ = RU_SubjectiveCase,
		RU_Singular_GenitiveCase /*      */ = RU_GenitiveCase,
		RU_Singular_DativeCase /*        */ = RU_DativeCase,
		RU_Singular_AccusativeCase /*    */ = RU_AccusativeCase,
		RU_Singular_InstrumentalCase /*  */ = RU_InstrumentalCase,
		RU_Singular_PrepositionalCase /* */ = RU_PrepositionalCase,

		RU_Plural_SubjectiveCase /*    */ = 0x0010 | RU_SubjectiveCase,
		RU_Plural_GenitiveCase /*      */ = 0x0010 | RU_GenitiveCase,
		RU_Plural_DativeCase /*        */ = 0x0010 | RU_DativeCase,
		RU_Plural_AccusativeCase /*    */ = 0x0010 | RU_AccusativeCase,
		RU_Plural_InstrumentalCase /*  */ = 0x0010 | RU_InstrumentalCase,
		RU_Plural_PrepositionalCase /* */ = 0x0010 | RU_PrepositionalCase,

		RU_Inanimate_SubjectiveCase /*    */ = RU_SubjectiveCase,
		RU_Inanimate_GenitiveCase /*      */ = RU_GenitiveCase,
		RU_Inanimate_DativeCase /*        */ = RU_DativeCase,
		RU_Inanimate_AccusativeCase /*    */ = RU_AccusativeCase,
		RU_Inanimate_InstrumentalCase /*  */ = RU_InstrumentalCase,
		RU_Inanimate_PrepositionalCase /* */ = RU_PrepositionalCase,

		RU_Animate_SubjectiveCase /*    */ = 0x0010 | RU_SubjectiveCase,
		RU_Animate_GenitiveCase /*      */ = 0x0010 | RU_GenitiveCase,
		RU_Animate_DativeCase /*        */ = 0x0010 | RU_DativeCase,
		RU_Animate_AccusativeCase /*    */ = 0x0010 | RU_AccusativeCase,
		RU_Animate_InstrumentalCase /*  */ = 0x0010 | RU_InstrumentalCase,
		RU_Animate_PrepositionalCase /* */ = 0x0010 | RU_PrepositionalCase,
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
