﻿using System;
using System.Text.RegularExpressions;

using Nabu.Diagnostics;

namespace Nabu.GacCleanUp
{
	public sealed class Program
	{
		public static void Main(string[] args)
		{
			CommandLine commandLine = new CommandLine(args);

			commandLine.AddNamed(
				"p",
				"Regular expression pattern used to match assembly name.",
				true,
				new CommandLineParameterDescriptor("pattern"));
			commandLine.AddNamed(
				"q",
				"Quiet mode.");
			commandLine.AddNamed(
				"s",
				"Suppress confirmations.");

			if (commandLine.Parse())
			{
				string pattern = (string)commandLine.GetNamedValue("p").ParameterValues[0];
				bool isQuiet = commandLine.GetNamedValueAction("q", false);
				bool isSuppessed = commandLine.GetNamedValueAction("s", false);

				IAssemblyCache assemblyCache = FusionWrapper.CreateAssemblyCache();
				IAssemblyEnum assemblyEnum = FusionWrapper.CreateGacEnum();
				IAssemblyName assemblyName;

				Regex regex = new Regex(pattern, RegexOptions.Compiled | RegexOptions.IgnoreCase);

				while (FusionWrapper.GetNextAssembly(assemblyEnum, out assemblyName) == ComResult.S_OK)
				{
					string assemblyFullName = FusionWrapper.GetDisplayName(
						assemblyName,
						AssemblyDisplayModes.Version | AssemblyDisplayModes.Culture | AssemblyDisplayModes.PublicKeyToken | AssemblyDisplayModes.ProcessorArchitecture);

					if (regex.IsMatch(assemblyFullName))
					{
						bool userAnswer;

						if (isSuppessed)
						{
							userAnswer = true;
						}
						else
						{
							for (; ; )
							{
								Console.WriteLine(
									"Are you sure you want to uninstall '{0}'?",
									assemblyFullName);
								Console.Write("(Y/N): ");

								string userAnswerLine = Console.ReadLine();

								if ((userAnswerLine == "Y") || (userAnswerLine == "y"))
								{
									userAnswer = true;
									break;
								}
								else if ((userAnswerLine == "N") || (userAnswerLine == "n"))
								{
									userAnswer = false;
									break;
								}
							}
						}

						if (userAnswer)
						{
							AssemblyCacheUninstallDisposition disposition;
							ComResult result = assemblyCache.UninstallAssembly(0, assemblyFullName, null, out disposition);

							if (NativeResult.ComSucceeded(result))
							{
								if (!isQuiet)
								{
									Console.WriteLine("{0} {1}", disposition.ToString(), assemblyFullName);
								}
							}
							else
							{
								Console.WriteLine("FAILURE (0x{0:X}) '{1}'", result, assemblyFullName);
							}
						}
					}
				}
			}
		}
	}
}
