﻿using System;
using System.Collections.Generic;
using System.IO;


namespace Nabu.SvnToVersion
{
	public sealed class Program
	{
		public static void Main(string[] args)
		{
			try
			{
				SvnWrapper svnWrapper = new SvnWrapper();

				CommandLine commandLine = new CommandLine(args);

				commandLine.AddNamed(
					"pd",
					"Path of directory, where revision number determines product version.",
					true,
					new CommandLineParameterDescriptor("path"));
				commandLine.AddNamed(
					"md",
					"Path of directory, where revision number determines module version.",
					false,
					new CommandLineParameterDescriptor("path"));
				commandLine.AddNamed(
					"td",
					"Path of target directory containing files to update.",
					false,
					new CommandLineParameterDescriptor("path"));
				commandLine.AddNamed(
					"tf",
					"Name or mask of target files to update.",
					true,
					new CommandLineParameterDescriptor("path"));
				commandLine.AddNamed(
					"te",
					"Target file encoding name.",
					false,
					new CommandLineParameterDescriptor("encoding"));
				commandLine.AddNamed(
					"re",
					"Path of file with regular expressions or one of predefined names: ?.net, ?rc.",
					1, int.MaxValue,
					new CommandLineParameterDescriptor("path"));

				if (commandLine.Parse())
				{
					string productDirectoryPath = Path.GetFullPath((string)commandLine.GetNamedValueParameter("pd"));
					string moduleDirectoryPath = (string)commandLine.GetNamedValueParameter("md");
					string targetDirectoryPath = (string)commandLine.GetNamedValueParameter("td");
					string targetFilePath = (string)commandLine.GetNamedValueParameter("tf");
					string targetFileEncoding = (string)commandLine.GetNamedValueParameter("te");

					if (moduleDirectoryPath != null)
					{
						moduleDirectoryPath = Path.GetFullPath(moduleDirectoryPath);
					}

					if (targetDirectoryPath != null)
					{
						targetDirectoryPath = Path.GetFullPath(targetDirectoryPath);
					}

					int productRevision = svnWrapper.GetRevisionNumber(productDirectoryPath);
					int moduleRevision = (moduleDirectoryPath == null) ? productRevision : svnWrapper.GetRevisionNumber(moduleDirectoryPath);

					Console.WriteLine("Product Revision: {0}", productRevision.ToString().PadLeft(6));
					Console.WriteLine("Module Revision:  {0}", moduleRevision.ToString().PadLeft(6));

					FileProcessor fileProcessor = new FileProcessor(targetFileEncoding);

					IList<CommandLineNamedArgument> regularExpressions = commandLine.GetNamedValues("re");

					foreach (CommandLineNamedArgument regularExpression in regularExpressions)
					{
						fileProcessor.LoadConfiguration((string)regularExpression.ParameterValues[0]);
					}

					if ((targetFilePath.IndexOfAny(new char[] { '?', '*' }) != -1) || (targetDirectoryPath != null))
					{
						if (targetDirectoryPath == null)
						{
							targetDirectoryPath = Directory.GetCurrentDirectory();
						}

						fileProcessor.ProcessDirectory(
							targetDirectoryPath,
							targetFilePath,
							productRevision,
							moduleRevision);
					}
					else
					{
						fileProcessor.ProcessFile(
							targetFilePath,
							productRevision,
							moduleRevision);
					}
				}
			}
			catch (Exception exception)
			{
				Console.WriteLine("Error:");
				Console.WriteLine(exception.Message);
			}
		}
	}
}
