﻿using System;
using System.Diagnostics;
using System.Text;

namespace Nabu.SvnToVersion
{
	public sealed class SvnWrapper
	{
		#region Fields

		private string _svnPath;

		#endregion

		#region Methods

		public SvnWrapper()
		{
			this._svnPath = Environment.GetEnvironmentVariable("SVN_PATH");

			if (this._svnPath == null)
			{
				throw new ApplicationException("Environment variable %SVN_PATH% with value of path to svn.exe must be defined.");
			}
		}

		public int GetRevisionNumber(string directoryPath)
		{
			Process process = new Process();

			process.StartInfo.Arguments = string.Format("info \"{0}\"", directoryPath);
			process.StartInfo.CreateNoWindow = true;
			process.StartInfo.FileName = this._svnPath;
			process.StartInfo.RedirectStandardError = true;
			process.StartInfo.RedirectStandardInput = true;
			process.StartInfo.RedirectStandardOutput = true;
			process.StartInfo.StandardErrorEncoding = Encoding.Default;
			process.StartInfo.StandardOutputEncoding = Encoding.Default;
			process.StartInfo.UseShellExecute = false;
			process.StartInfo.WorkingDirectory = Process.GetCurrentProcess().StartInfo.WorkingDirectory;

			if (process.Start())
			{
				process.WaitForExit();

				string stdError = process.StandardError.ReadToEnd();
				string stdOutput = process.StandardOutput.ReadToEnd();

				if (stdError.Length != 0)
				{
					throw new ApplicationException(string.Format("Execution of {0} failed\r\n.{1}", this._svnPath, stdError));
				}
				else
				{
					foreach (string stdOutputLine in stdOutput.Split(new char[] { '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries))
					{
						if (stdOutputLine.StartsWith("Last Changed Rev:"))
						{
							return int.Parse(stdOutputLine.Substring(18)) + 1;
						}
					}
				}
			}

			throw new ApplicationException(string.Format("Folder '{0}' revision can not be retrieved.", directoryPath));
		}

		#endregion
	}
}
