﻿/******************************************************************************/
/*                                                                            */
/*    _                ___        _..-._                                      */
/*    \`.|\..----...-'`   `-._.-'' _.-..'                                     */
/*    /  ' `         ,       __.-''                                           */
/*    )/` _/     \   `-_,   /                                                 */
/*    `-'" `"\_  ,_.-;_.-\_ ',  NABU Project                                  */
/*        _.-'_./   {_.'   ; /  Nabu.Forms Library                            */
/*       {_.-``-'         {_/   Copyright © 2005-2009, TrifleSoft             */
/*                                                                            */
/******************************************************************************/
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Globalization;
using System.IO;
using System.Resources;
using System.Runtime.InteropServices;
using System.Windows.Forms;

using Nabu.Drawing;

namespace Nabu.Forms
{
	public partial class FormColorPicker : Form
	{
		#region FormColorPicker

		#region Constants

		private static ColorPickerColorPalette[] _palletes =
		{
			#region VGA

			new ColorPickerColorPalette(
				"VGA", "vga",
				new ColorPickerColorGroup[]
				{
					new ColorPickerColorGroup(
						"Dark",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Black", Color.FromArgb(0x00, 0x00, 0x00)),
							new ColorPickerColor("Navy", Color.FromArgb(0x00, 0x00, 0xAA)),
							new ColorPickerColor("Green", Color.FromArgb(0x00, 0xAA, 0x00)),
							new ColorPickerColor("Teal", Color.FromArgb(0x00, 0xAA, 0xAA)),
							new ColorPickerColor("Red", Color.FromArgb(0xAA, 0x00, 0x00)),
							new ColorPickerColor("Purple", Color.FromArgb(0xAA, 0x00, 0xAA)),
							new ColorPickerColor("Brown", Color.FromArgb(0xAA, 0x55, 0x00)),
							new ColorPickerColor("Light Gray", Color.FromArgb(0xAA, 0xAA, 0xAA)),
						}),
					new ColorPickerColorGroup(
						"Bright",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Gray", Color.FromArgb(0x55, 0x55, 0x55)),
							new ColorPickerColor("Blue", Color.FromArgb(0x55, 0x55, 0xFF)),
							new ColorPickerColor("Lime", Color.FromArgb(0x55, 0xFF, 0x55)),
							new ColorPickerColor("Cyan", Color.FromArgb(0x55, 0xFF, 0xFF)),
							new ColorPickerColor("Pink", Color.FromArgb(0xFF, 0x55, 0x55)),
							new ColorPickerColor("Magenta", Color.FromArgb(0xFF, 0x55, 0xFF)),
							new ColorPickerColor("Yellow", Color.FromArgb(0xFF, 0xFF, 0x55)),
							new ColorPickerColor("White", Color.FromArgb(0xFF, 0xFF, 0xFF)),
						}),
				}),

			#endregion

			#region Crayola
/*
			new ColorPickerColorPalette(
				"Crayola", "crayola",
				new ColorPickerColorGroup[]
				{
					new ColorPickerColorGroup(
						"8 Pack",
						new ColorPickerColor[]
							{
								new ColorPickerColor("Black", Color.FromArgb(0x00, 0x00, 0x00)),
								new ColorPickerColor("Blue", Color.FromArgb(0x00, 0x66, 0xFF)),
								new ColorPickerColor("Brown", Color.FromArgb(0xAF, 0x59, 0x3E)),
								new ColorPickerColor("Green", Color.FromArgb(0x01, 0xA3, 0x68)),
								new ColorPickerColor("Orange", Color.FromArgb(0xFF, 0x68, 0x1F)),
								new ColorPickerColor("Red", Color.FromArgb(0xED, 0x0A, 0x3F)),
								new ColorPickerColor("Violet", Color.FromArgb(0x83, 0x59, 0xA3)),
								new ColorPickerColor("Yellow", Color.FromArgb(0xFB, 0xE8, 0x70)),
							}),
						new ColorPickerColorGroup(
						"16 Pack",
							new ColorPickerColor[]
							{
								new ColorPickerColor("Blue Green", Color.FromArgb(0x00, 0x95, 0xB6)),
								new ColorPickerColor("Blue Violet", Color.FromArgb(0x64, 0x56, 0xB7)),
								new ColorPickerColor("Carnation Pink", Color.FromArgb(0xFF, 0xA6, 0xC9)),
								new ColorPickerColor("Red Orange", Color.FromArgb(0xFF, 0x3F, 0x34)),
								new ColorPickerColor("Red Violet", Color.FromArgb(0xBB, 0x33, 0x85)),
								new ColorPickerColor("White", Color.FromArgb(0xFF, 0xFF, 0xFF)),
								new ColorPickerColor("Yellow Green", Color.FromArgb(0xC5, 0xE1, 0x7A)),
								new ColorPickerColor("Yellow Orange", Color.FromArgb(0xFF, 0xAE, 0x42)),
							}),
						new ColorPickerColorGroup(
						"24 Pack",
						new ColorPickerColor[]
							{
								new ColorPickerColor("Apricot", Color.FromArgb(0xFD, 0xD5, 0xB1)),
								new ColorPickerColor("Cerulean", Color.FromArgb(0x02, 0xA4, 0xD3)),
								new ColorPickerColor("Dandelion", Color.FromArgb(0xFE, 0xD8, 0x5D)),
								new ColorPickerColor("Gray", Color.FromArgb(0x8B, 0x86, 0x80)),
								new ColorPickerColor("Green Yellow", Color.FromArgb(0xF1, 0xE7, 0x88)),
								new ColorPickerColor("Indigo", Color.FromArgb(0x4F, 0x69, 0xC6)),
								new ColorPickerColor("Scarlet", Color.FromArgb(0xFD, 0x0E, 0x35)),
								new ColorPickerColor("Violet Red", Color.FromArgb(0xF7, 0x46, 0x8A)),
							}),
				}),
*/
			#endregion

			#region Apple IIe
/*
			new ColorPickerColorPalette(
				"Apple IIe", "apple_iie",
				new ColorPickerColorGroup[]
				{
					new ColorPickerColorGroup(
						"",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Black", Color.FromArgb(0x00, 0x00, 0x00)),
							new ColorPickerColor("Magenta", Color.FromArgb(0xE3, 0x1E, 0x60)),
							new ColorPickerColor("Dark Blue", Color.FromArgb(0x60, 0x4E, 0xBD)),
							new ColorPickerColor("Purple", Color.FromArgb(0xFF, 0x44, 0xFD)),
							new ColorPickerColor("Dark Green", Color.FromArgb(0x00, 0xA3, 0x60)),
							new ColorPickerColor("Grey", Color.FromArgb(0x9C, 0x9C, 0x9C)),
							new ColorPickerColor("Medium Blue", Color.FromArgb(0x14, 0xCF, 0xFD)),
							new ColorPickerColor("Light Blue", Color.FromArgb(0xD0, 0xC3, 0xFF)),
							new ColorPickerColor("Brown", Color.FromArgb(0x60, 0x72, 0x03)),
							new ColorPickerColor("Orange", Color.FromArgb(0xFF, 0x6A, 0x3C)),
							new ColorPickerColor("Pink", Color.FromArgb(0xFF, 0xA0, 0xD0)),
							new ColorPickerColor("Green", Color.FromArgb(0x14, 0xF5, 0x3C)),
							new ColorPickerColor("Yellow", Color.FromArgb(0xD0, 0xDD, 0x8D)),
							new ColorPickerColor("Aqua", Color.FromArgb(0x72, 0xFF, 0xD0)),
							new ColorPickerColor("White", Color.FromArgb(0xFF, 0xFF, 0xFF)),
						}),
				}),
*/
			#endregion

			#region MOS
/*
			new ColorPickerColorPalette(
				"MOS", "mos",
				new ColorPickerColorGroup[]
				{
					new ColorPickerColorGroup(
						"VIC - I",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Black", Color.FromArgb(0x00, 0x00, 0x00)),
							new ColorPickerColor("White", Color.FromArgb(0xFF, 0xFF, 0xFF)),
							new ColorPickerColor("Red", Color.FromArgb(0xF0, 0x00, 0x00)),
							new ColorPickerColor("Cyan", Color.FromArgb(0x00, 0xF0, 0xF0)),
							new ColorPickerColor("Purple", Color.FromArgb(0x60, 0x00, 0x60)),
							new ColorPickerColor("Green", Color.FromArgb(0x00, 0xA0, 0x00)),
							new ColorPickerColor("Blue", Color.FromArgb(0x00, 0x00, 0xF0)),
							new ColorPickerColor("Yellow", Color.FromArgb(0xD0, 0xD0, 0x00)),
							new ColorPickerColor("Orange", Color.FromArgb(0xC0, 0xA0, 0x00)),
							new ColorPickerColor("Light Orange", Color.FromArgb(0xFF, 0xA0, 0x00)),
							new ColorPickerColor("Light Red", Color.FromArgb(0xF0, 0x80, 0x80)),
							new ColorPickerColor("Light Cyan", Color.FromArgb(0x00, 0xFF, 0xFF)),
							new ColorPickerColor("Light Purple", Color.FromArgb(0xFF, 0x00, 0xFF)),
							new ColorPickerColor("Light Green", Color.FromArgb(0x00, 0xFF, 0x00)),
							new ColorPickerColor("Light Blue", Color.FromArgb(0x00, 0xA0, 0xFF)),
							new ColorPickerColor("Light Yellow", Color.FromArgb(0xFF, 0xFF, 0x00)),
						}),
					new ColorPickerColorGroup(
						"VIC - II",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Black", Color.FromArgb(0x00, 0x00, 0x00)),
							new ColorPickerColor("White", Color.FromArgb(0xFF, 0xFF, 0xFF)),
							new ColorPickerColor("Red", Color.FromArgb(0x68, 0x37, 0x2B)),
							new ColorPickerColor("Cyan", Color.FromArgb(0x70, 0xA4, 0xB2)),
							new ColorPickerColor("Purple", Color.FromArgb(0x6F, 0x3D, 0x86)),
							new ColorPickerColor("Green", Color.FromArgb(0x58, 0x8D, 0x43)),
							new ColorPickerColor("Blue", Color.FromArgb(0x35, 0x28, 0x79)),
							new ColorPickerColor("Yellow", Color.FromArgb(0xB8, 0xC7, 0x6F)),
							new ColorPickerColor("Orange", Color.FromArgb(0x6F, 0x4F, 0x25)),
							new ColorPickerColor("Brown", Color.FromArgb(0x43, 0x39, 0x00)),
							new ColorPickerColor("Light Red", Color.FromArgb(0x9A, 0x67, 0x59)),
							new ColorPickerColor("Dark Grey", Color.FromArgb(0x44, 0x44, 0x44)),
							new ColorPickerColor("Grey", Color.FromArgb(0x6C, 0x6C, 0x6C)),
							new ColorPickerColor("Light Green", Color.FromArgb(0x9A, 0xD2, 0x84)),
							new ColorPickerColor("Light Blue", Color.FromArgb(0x6C, 0x5E, 0xB5)),
							new ColorPickerColor("Light Grey", Color.FromArgb(0x95, 0x95, 0x95)),
						}),
					new ColorPickerColorGroup(
						"TED - Black",
						new ColorPickerColor[]
						{
							new ColorPickerColor("1,0", Color.FromArgb(0x03, 0x03, 0x03)),
						}),
					new ColorPickerColorGroup(
						"TED - White",
						new ColorPickerColor[]
						{
							new ColorPickerColor("2,0", Color.FromArgb(0x2F, 0x2F, 0x2F)),
							new ColorPickerColor("2,1", Color.FromArgb(0x3D, 0x3D, 0x3D)),
							new ColorPickerColor("2,2", Color.FromArgb(0x42, 0x42, 0x42)),
							new ColorPickerColor("2,3", Color.FromArgb(0x56, 0x55, 0x5A)),
							new ColorPickerColor("2,4", Color.FromArgb(0x84, 0x7E, 0x85)),
							new ColorPickerColor("2,5", Color.FromArgb(0xB2, 0xAC, 0xB3)),
							new ColorPickerColor("2,6", Color.FromArgb(0xCA, 0xCA, 0xCA)),
							new ColorPickerColor("2,7", Color.FromArgb(0xFF, 0xFF, 0xFF)),
						}),
					new ColorPickerColorGroup(
						"TED - Red",
						new ColorPickerColor[]
						{
							new ColorPickerColor("3,0", Color.FromArgb(0x68, 0x10, 0x10)),
							new ColorPickerColor("3,1", Color.FromArgb(0x75, 0x1E, 0x20)),
							new ColorPickerColor("3,2", Color.FromArgb(0x7B, 0x28, 0x20)),
							new ColorPickerColor("3,3", Color.FromArgb(0x90, 0x3C, 0x3B)),
							new ColorPickerColor("3,4", Color.FromArgb(0xBB, 0x67, 0x68)),
							new ColorPickerColor("3,5", Color.FromArgb(0xE9, 0x92, 0x92)),
							new ColorPickerColor("3,6", Color.FromArgb(0xFF, 0xAC, 0xAC)),
							new ColorPickerColor("3,7", Color.FromArgb(0xFF, 0xF6, 0xF2)),
						}),
					new ColorPickerColorGroup(
						"TED - Cyan",
						new ColorPickerColor[]
						{
							new ColorPickerColor("4,0", Color.FromArgb(0x00, 0x42, 0x42)),
							new ColorPickerColor("4,1", Color.FromArgb(0x00, 0x50, 0x4F)),
							new ColorPickerColor("4,2", Color.FromArgb(0x02, 0x56, 0x59)),
							new ColorPickerColor("4,3", Color.FromArgb(0x17, 0x6D, 0x72)),
							new ColorPickerColor("4,4", Color.FromArgb(0x45, 0x96, 0x96)),
							new ColorPickerColor("4,5", Color.FromArgb(0x6C, 0xC3, 0xC1)),
							new ColorPickerColor("4,6", Color.FromArgb(0x85, 0xD8, 0xE0)),
							new ColorPickerColor("4,7", Color.FromArgb(0xD1, 0xFF, 0xFF)),
						}),
					new ColorPickerColorGroup(
						"TED - Purple",
						new ColorPickerColor[]
						{
							new ColorPickerColor("5,0", Color.FromArgb(0x58, 0x00, 0x6D)),
							new ColorPickerColor("5,1", Color.FromArgb(0x6A, 0x10, 0x78)),
							new ColorPickerColor("5,2", Color.FromArgb(0x6F, 0x1A, 0x82)),
							new ColorPickerColor("5,3", Color.FromArgb(0x87, 0x2D, 0x99)),
							new ColorPickerColor("5,4", Color.FromArgb(0xAF, 0x58, 0xC3)),
							new ColorPickerColor("5,5", Color.FromArgb(0xD9, 0x86, 0xF0)),
							new ColorPickerColor("5,6", Color.FromArgb(0xF3, 0x9C, 0xFF)),
							new ColorPickerColor("5,7", Color.FromArgb(0xFF, 0xE9, 0xFF)),
						}),
					new ColorPickerColorGroup(
						"TED - Green",
						new ColorPickerColor[]
						{
							new ColorPickerColor("6,0", Color.FromArgb(0x00, 0x4E, 0x00)),
							new ColorPickerColor("6,1", Color.FromArgb(0x04, 0x5C, 0x00)),
							new ColorPickerColor("6,2", Color.FromArgb(0x0A, 0x65, 0x09)),
							new ColorPickerColor("6,3", Color.FromArgb(0x1F, 0x7B, 0x15)),
							new ColorPickerColor("6,4", Color.FromArgb(0x4A, 0xA7, 0x3E)),
							new ColorPickerColor("6,5", Color.FromArgb(0x79, 0xD1, 0x76)),
							new ColorPickerColor("6,6", Color.FromArgb(0x92, 0xEA, 0x8A)),
							new ColorPickerColor("6,7", Color.FromArgb(0xDB, 0xFF, 0xD3)),
						}),
					new ColorPickerColorGroup(
						"TED - Blue",
						new ColorPickerColor[]
						{
							new ColorPickerColor("7,0", Color.FromArgb(0x19, 0x1C, 0x94)),
							new ColorPickerColor("7,1", Color.FromArgb(0x2A, 0x2A, 0xA3)),
							new ColorPickerColor("7,2", Color.FromArgb(0x30, 0x34, 0xA7)),
							new ColorPickerColor("7,3", Color.FromArgb(0x46, 0x49, 0xC1)),
							new ColorPickerColor("7,4", Color.FromArgb(0x73, 0x73, 0xEC)),
							new ColorPickerColor("7,5", Color.FromArgb(0x9D, 0xA1, 0xFF)),
							new ColorPickerColor("7,6", Color.FromArgb(0xB7, 0xBA, 0xFF)),
							new ColorPickerColor("7,7", Color.FromArgb(0xE8, 0xE8, 0xFF)),
						}),
					new ColorPickerColorGroup(
						"TED - Yellow",
						new ColorPickerColor[]
						{
							new ColorPickerColor("8,0", Color.FromArgb(0x38, 0x38, 0x00)),
							new ColorPickerColor("8,1", Color.FromArgb(0x4C, 0x47, 0x00)),
							new ColorPickerColor("8,2", Color.FromArgb(0x50, 0x51, 0x00)),
							new ColorPickerColor("8,3", Color.FromArgb(0x66, 0x63, 0x00)),
							new ColorPickerColor("8,4", Color.FromArgb(0x92, 0x8D, 0x11)),
							new ColorPickerColor("8,5", Color.FromArgb(0xBD, 0xBE, 0x40)),
							new ColorPickerColor("8,6", Color.FromArgb(0xD6, 0xD3, 0x5B)),
							new ColorPickerColor("8,7", Color.FromArgb(0xFF, 0xFF, 0xA3)),
						}),
					new ColorPickerColorGroup(
						"TED - Orange",
						new ColorPickerColor[]
						{
							new ColorPickerColor("9,0", Color.FromArgb(0x56, 0x20, 0x00)),
							new ColorPickerColor("9,1", Color.FromArgb(0x69, 0x2F, 0x00)),
							new ColorPickerColor("9,2", Color.FromArgb(0x6E, 0x36, 0x00)),
							new ColorPickerColor("9,3", Color.FromArgb(0x84, 0x4C, 0x0D)),
							new ColorPickerColor("9,4", Color.FromArgb(0xAF, 0x78, 0x32)),
							new ColorPickerColor("9,5", Color.FromArgb(0xDC, 0xA2, 0x61)),
							new ColorPickerColor("9,6", Color.FromArgb(0xF3, 0xBE, 0x79)),
							new ColorPickerColor("9,7", Color.FromArgb(0xFF, 0xFF, 0xC1)),
						}),
					new ColorPickerColorGroup(
						"TED - Brown",
						new ColorPickerColor[]
						{
							new ColorPickerColor("10,0", Color.FromArgb(0x4B, 0x28, 0x00)),
							new ColorPickerColor("10,1", Color.FromArgb(0x59, 0x38, 0x00)),
							new ColorPickerColor("10,2", Color.FromArgb(0x65, 0x40, 0x00)),
							new ColorPickerColor("10,3", Color.FromArgb(0x73, 0x55, 0x00)),
							new ColorPickerColor("10,4", Color.FromArgb(0xA1, 0x80, 0x20)),
							new ColorPickerColor("10,5", Color.FromArgb(0xD1, 0xA9, 0x4C)),
							new ColorPickerColor("10,6", Color.FromArgb(0xE6, 0xC5, 0x65)),
							new ColorPickerColor("10,7", Color.FromArgb(0xFF, 0xFF, 0xB2)),
						}),
					new ColorPickerColorGroup(
						"TED - Yellow Green",
						new ColorPickerColor[]
						{
							new ColorPickerColor("11,0", Color.FromArgb(0x16, 0x48, 0x00)),
							new ColorPickerColor("11,1", Color.FromArgb(0x26, 0x56, 0x00)),
							new ColorPickerColor("11,2", Color.FromArgb(0x2C, 0x5C, 0x00)),
							new ColorPickerColor("11,3", Color.FromArgb(0x40, 0x72, 0x00)),
							new ColorPickerColor("11,4", Color.FromArgb(0x6C, 0x9E, 0x12)),
							new ColorPickerColor("11,5", Color.FromArgb(0x93, 0xC8, 0x3D)),
							new ColorPickerColor("11,6", Color.FromArgb(0xB0, 0xE0, 0x57)),
							new ColorPickerColor("11,7", Color.FromArgb(0xFC, 0xFF, 0xA2)),
						}),
					new ColorPickerColorGroup(
						"TED - Pink",
						new ColorPickerColor[]
						{
							new ColorPickerColor("12,0", Color.FromArgb(0x69, 0x07, 0x2F)),
							new ColorPickerColor("12,1", Color.FromArgb(0x75, 0x15, 0x41)),
							new ColorPickerColor("12,2", Color.FromArgb(0x7D, 0x1E, 0x45)),
							new ColorPickerColor("12,3", Color.FromArgb(0x91, 0x33, 0x5E)),
							new ColorPickerColor("12,4", Color.FromArgb(0xBA, 0x5F, 0x89)),
							new ColorPickerColor("12,5", Color.FromArgb(0xE9, 0x8A, 0xB1)),
							new ColorPickerColor("12,6", Color.FromArgb(0xFF, 0xA4, 0xCF)),
							new ColorPickerColor("12,7", Color.FromArgb(0xFF, 0xEE, 0xFF)),
						}),
					new ColorPickerColorGroup(
						"TED - Blue Green",
						new ColorPickerColor[]
						{
							new ColorPickerColor("13,0", Color.FromArgb(0x00, 0x46, 0x26)),
							new ColorPickerColor("13,1", Color.FromArgb(0x00, 0x58, 0x3D)),
							new ColorPickerColor("13,2", Color.FromArgb(0x01, 0x61, 0x45)),
							new ColorPickerColor("13,3", Color.FromArgb(0x19, 0x74, 0x5C)),
							new ColorPickerColor("13,4", Color.FromArgb(0x46, 0x9F, 0x83)),
							new ColorPickerColor("13,5", Color.FromArgb(0x6F, 0xCD, 0xAB)),
							new ColorPickerColor("13,6", Color.FromArgb(0x89, 0xE5, 0xC8)),
							new ColorPickerColor("13,7", Color.FromArgb(0xD1, 0xFF, 0xFF)),
						}),
					new ColorPickerColorGroup(
						"TED - Light Blue",
						new ColorPickerColor[]
						{
							new ColorPickerColor("14,0", Color.FromArgb(0x06, 0x2A, 0x80)),
							new ColorPickerColor("14,1", Color.FromArgb(0x15, 0x3D, 0x8F)),
							new ColorPickerColor("14,2", Color.FromArgb(0x1C, 0x45, 0x99)),
							new ColorPickerColor("14,3", Color.FromArgb(0x32, 0x59, 0xAE)),
							new ColorPickerColor("14,4", Color.FromArgb(0x61, 0x85, 0xDD)),
							new ColorPickerColor("14,5", Color.FromArgb(0x8A, 0xB4, 0xFF)),
							new ColorPickerColor("14,6", Color.FromArgb(0xA4, 0xCA, 0xFF)),
							new ColorPickerColor("14,7", Color.FromArgb(0xEB, 0xFF, 0xFF)),
						}),
					new ColorPickerColorGroup(
						"TED - Dark Blue",
						new ColorPickerColor[]
						{
							new ColorPickerColor("15,0", Color.FromArgb(0x2A, 0x14, 0x9B)),
							new ColorPickerColor("15,1", Color.FromArgb(0x39, 0x22, 0xAE)),
							new ColorPickerColor("15,2", Color.FromArgb(0x42, 0x2D, 0xAD)),
							new ColorPickerColor("15,3", Color.FromArgb(0x59, 0x3F, 0xC3)),
							new ColorPickerColor("15,4", Color.FromArgb(0x84, 0x6C, 0xEF)),
							new ColorPickerColor("15,5", Color.FromArgb(0xB2, 0x9A, 0xFF)),
							new ColorPickerColor("15,6", Color.FromArgb(0xC8, 0xB8, 0xFF)),
							new ColorPickerColor("15,7", Color.FromArgb(0xFF, 0xF0, 0xFF)),
						}),
					new ColorPickerColorGroup(
						"TED - Light Green",
						new ColorPickerColor[]
						{
							new ColorPickerColor("16,0", Color.FromArgb(0x0B, 0x49, 0x00)),
							new ColorPickerColor("16,1", Color.FromArgb(0x19, 0x59, 0x00)),
							new ColorPickerColor("16,2", Color.FromArgb(0x1D, 0x62, 0x00)),
							new ColorPickerColor("16,3", Color.FromArgb(0x32, 0x76, 0x00)),
							new ColorPickerColor("16,4", Color.FromArgb(0x5D, 0xA3, 0x29)),
							new ColorPickerColor("16,5", Color.FromArgb(0x88, 0xCB, 0x59)),
							new ColorPickerColor("16,6", Color.FromArgb(0xA2, 0xE5, 0x7A)),
							new ColorPickerColor("16,7", Color.FromArgb(0xED, 0xFF, 0xBC)),
						}),
				}),
*/
			#endregion

			#region X11 / Web

			new ColorPickerColorPalette(
				"X11 / Web", "x11_web",
				new ColorPickerColorGroup[]
				{
					new ColorPickerColorGroup(
						"Red colors",
						new ColorPickerColor[]
						{
							new ColorPickerColor("IndianRed", Color.FromArgb(0xCD, 0x5C, 0x5C)),
							new ColorPickerColor("LightCoral", Color.FromArgb(0xF0, 0x80, 0x80)),
							new ColorPickerColor("Salmon", Color.FromArgb(0xFA, 0x80, 0x72)),
							new ColorPickerColor("DarkSalmon", Color.FromArgb(0xE9, 0x96, 0x7A)),
							new ColorPickerColor("LightSalmon", Color.FromArgb(0xFF, 0xA0, 0x7A)),
							new ColorPickerColor("Crimson", Color.FromArgb(0xDC, 0x14, 0x3C)),
							new ColorPickerColor("Red", Color.FromArgb(0xFF, 0x00, 0x00)),
							new ColorPickerColor("FireBrick", Color.FromArgb(0xB2, 0x22, 0x22)),
							new ColorPickerColor("DarkRed", Color.FromArgb(0x8B, 0x00, 0x00)),
						}),
					new ColorPickerColorGroup(
						"Pink colors",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Pink", Color.FromArgb(0xFF, 0xC0, 0xCB)),
							new ColorPickerColor("LightPink", Color.FromArgb(0xFF, 0xB6, 0xC1)),
							new ColorPickerColor("HotPink", Color.FromArgb(0xFF, 0x69, 0xB4)),
							new ColorPickerColor("DeepPink", Color.FromArgb(0xFF, 0x14, 0x93)),
							new ColorPickerColor("MediumVioletRed", Color.FromArgb(0xC7, 0x15, 0x85)),
							new ColorPickerColor("PaleVioletRed", Color.FromArgb(0xDB, 0x70, 0x93)),
						}),
					new ColorPickerColorGroup(
						"Orange colors",
						new ColorPickerColor[]
						{
							new ColorPickerColor("LightSalmon", Color.FromArgb(0xFF, 0xA0, 0x7A)),
							new ColorPickerColor("Coral", Color.FromArgb(0xFF, 0x7F, 0x50)),
							new ColorPickerColor("Tomato", Color.FromArgb(0xFF, 0x63, 0x47)),
							new ColorPickerColor("OrangeRed", Color.FromArgb(0xFF, 0x45, 0x00)),
							new ColorPickerColor("DarkOrange", Color.FromArgb(0xFF, 0x8C, 0x00)),
							new ColorPickerColor("Orange", Color.FromArgb(0xFF, 0xA5, 0x00)),
						}),
					new ColorPickerColorGroup(
						"Yellow colors",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Gold", Color.FromArgb(0xFF, 0xD7, 0x00)),
							new ColorPickerColor("Yellow", Color.FromArgb(0xFF, 0xFF, 0x00)),
							new ColorPickerColor("LightYellow", Color.FromArgb(0xFF, 0xFF, 0xE0)),
							new ColorPickerColor("LemonChiffon", Color.FromArgb(0xFF, 0xFA, 0xCD)),
							new ColorPickerColor("LightGoldenrodYellow", Color.FromArgb(0xFA, 0xFA, 0xD2)),
							new ColorPickerColor("PapayaWhip", Color.FromArgb(0xFF, 0xEF, 0xD5)),
							new ColorPickerColor("Moccasin", Color.FromArgb(0xFF, 0xE4, 0xB5)),
							new ColorPickerColor("PeachPuff", Color.FromArgb(0xFF, 0xDA, 0xB9)),
							new ColorPickerColor("PaleGoldenrod", Color.FromArgb(0xEE, 0xE8, 0xAA)),
							new ColorPickerColor("Khaki", Color.FromArgb(0xF0, 0xE6, 0x8C)),
							new ColorPickerColor("DarkKhaki", Color.FromArgb(0xBD, 0xB7, 0x6B)),
						}),
					new ColorPickerColorGroup(
						"Purple colors",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Lavender", Color.FromArgb(0xE6, 0xE6, 0xFA)),
							new ColorPickerColor("Thistle", Color.FromArgb(0xD8, 0xBF, 0xD8)),
							new ColorPickerColor("Plum", Color.FromArgb(0xDD, 0xA0, 0xDD)),
							new ColorPickerColor("Violet", Color.FromArgb(0xEE, 0x82, 0xEE)),
							new ColorPickerColor("Orchid", Color.FromArgb(0xDA, 0x70, 0xD6)),
							new ColorPickerColor("Magenta", Color.FromArgb(0xFF, 0x00, 0xFF)),
							new ColorPickerColor("MediumOrchid", Color.FromArgb(0xBA, 0x55, 0xD3)),
							new ColorPickerColor("MediumPurple", Color.FromArgb(0x93, 0x70, 0xDB)),
							new ColorPickerColor("BlueViolet", Color.FromArgb(0x8A, 0x2B, 0xE2)),
							new ColorPickerColor("DarkViolet", Color.FromArgb(0x94, 0x00, 0xD3)),
							new ColorPickerColor("DarkOrchid", Color.FromArgb(0x99, 0x32, 0xCC)),
							new ColorPickerColor("DarkMagenta", Color.FromArgb(0x8B, 0x00, 0x8B)),
							new ColorPickerColor("Purple", Color.FromArgb(0x80, 0x00, 0x80)),
							new ColorPickerColor("Indigo", Color.FromArgb(0x4B, 0x00, 0x82)),
							new ColorPickerColor("SlateBlue", Color.FromArgb(0x6A, 0x5A, 0xCD)),
							new ColorPickerColor("DarkSlateBlue", Color.FromArgb(0x48, 0x3D, 0x8B)),
						}),
					new ColorPickerColorGroup(
						"Green colors",
						new ColorPickerColor[]
						{
							new ColorPickerColor("GreenYellow", Color.FromArgb(0xAD, 0xFF, 0x2F)),
							new ColorPickerColor("Chartreuse", Color.FromArgb(0x7F, 0xFF, 0x00)),
							new ColorPickerColor("LawnGreen", Color.FromArgb(0x7C, 0xFC, 0x00)),
							new ColorPickerColor("Lime", Color.FromArgb(0x00, 0xFF, 0x00)),
							new ColorPickerColor("LimeGreen", Color.FromArgb(0x32, 0xCD, 0x32)),
							new ColorPickerColor("PaleGreen", Color.FromArgb(0x98, 0xFB, 0x98)),
							new ColorPickerColor("LightGreen", Color.FromArgb(0x90, 0xEE, 0x90)),
							new ColorPickerColor("MediumSpringGreen", Color.FromArgb(0x00, 0xFA, 0x9A)),
							new ColorPickerColor("SpringGreen", Color.FromArgb(0x00, 0xFF, 0x7F)),
							new ColorPickerColor("MediumSeaGreen", Color.FromArgb(0x3C, 0xB3, 0x71)),
							new ColorPickerColor("SeaGreen", Color.FromArgb(0x2E, 0x8B, 0x57)),
							new ColorPickerColor("ForestGreen", Color.FromArgb(0x22, 0x8B, 0x22)),
							new ColorPickerColor("Green", Color.FromArgb(0x00, 0x80, 0x00)),
							new ColorPickerColor("DarkGreen", Color.FromArgb(0x00, 0x64, 0x00)),
							new ColorPickerColor("YellowGreen", Color.FromArgb(0x9A, 0xCD, 0x32)),
							new ColorPickerColor("OliveDrab", Color.FromArgb(0x6B, 0x8E, 0x23)),
							new ColorPickerColor("Olive", Color.FromArgb(0x80, 0x80, 0x00)),
							new ColorPickerColor("DarkOliveGreen", Color.FromArgb(0x55, 0x6B, 0x2F)),
							new ColorPickerColor("MediumAquamarine", Color.FromArgb(0x66, 0xCD, 0xAA)),
							new ColorPickerColor("DarkSeaGreen", Color.FromArgb(0x8F, 0xBC, 0x8F)),
							new ColorPickerColor("LightSeaGreen", Color.FromArgb(0x20, 0xB2, 0xAA)),
							new ColorPickerColor("DarkCyan", Color.FromArgb(0x00, 0x8B, 0x8B)),
							new ColorPickerColor("Teal", Color.FromArgb(0x00, 0x80, 0x80)),
						}),
					new ColorPickerColorGroup(
						"Blue colors",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Aqua", Color.FromArgb(0x00, 0xFF, 0xFF)),
							new ColorPickerColor("Cyan", Color.FromArgb(0x00, 0xFF, 0xFF)),
							new ColorPickerColor("LightCyan", Color.FromArgb(0xE0, 0xFF, 0xFF)),
							new ColorPickerColor("PaleTurquoise", Color.FromArgb(0xAF, 0xEE, 0xEE)),
							new ColorPickerColor("Aquamarine", Color.FromArgb(0x7F, 0xFF, 0xD4)),
							new ColorPickerColor("Turquoise", Color.FromArgb(0x40, 0xE0, 0xD0)),
							new ColorPickerColor("MediumTurquoise", Color.FromArgb(0x48, 0xD1, 0xCC)),
							new ColorPickerColor("DarkTurquoise", Color.FromArgb(0x00, 0xCE, 0xD1)),
							new ColorPickerColor("CadetBlue", Color.FromArgb(0x5F, 0x9E, 0xA0)),
							new ColorPickerColor("SteelBlue", Color.FromArgb(0x46, 0x82, 0xB4)),
							new ColorPickerColor("LightSteelBlue", Color.FromArgb(0xB0, 0xC4, 0xDE)),
							new ColorPickerColor("PowderBlue", Color.FromArgb(0xB0, 0xE0, 0xE6)),
							new ColorPickerColor("LightBlue", Color.FromArgb(0xAD, 0xD8, 0xE6)),
							new ColorPickerColor("SkyBlue", Color.FromArgb(0x87, 0xCE, 0xEB)),
							new ColorPickerColor("LightSkyBlue", Color.FromArgb(0x87, 0xCE, 0xFA)),
							new ColorPickerColor("DeepSkyBlue", Color.FromArgb(0x00, 0xBF, 0xFF)),
							new ColorPickerColor("DodgerBlue", Color.FromArgb(0x1E, 0x90, 0xFF)),
							new ColorPickerColor("CornflowerBlue", Color.FromArgb(0x64, 0x95, 0xED)),
							new ColorPickerColor("MediumSlateBlue", Color.FromArgb(0x7B, 0x68, 0xEE)),
							new ColorPickerColor("RoyalBlue", Color.FromArgb(0x41, 0x69, 0xE1)),
							new ColorPickerColor("Blue", Color.FromArgb(0x00, 0x00, 0xFF)),
							new ColorPickerColor("MediumBlue", Color.FromArgb(0x00, 0x00, 0xCD)),
							new ColorPickerColor("DarkBlue", Color.FromArgb(0x00, 0x00, 0x8B)),
							new ColorPickerColor("Navy", Color.FromArgb(0x00, 0x00, 0x80)),
							new ColorPickerColor("MidnightBlue", Color.FromArgb(0x19, 0x19, 0x70)),
						}),
					new ColorPickerColorGroup(
						"Brown colors",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Cornsilk", Color.FromArgb(0xFF, 0xF8, 0xDC)),
							new ColorPickerColor("BlanchedAlmond", Color.FromArgb(0xFF, 0xEB, 0xCD)),
							new ColorPickerColor("Bisque", Color.FromArgb(0xFF, 0xE4, 0xC4)),
							new ColorPickerColor("NavajoWhite", Color.FromArgb(0xFF, 0xDE, 0xAD)),
							new ColorPickerColor("Wheat", Color.FromArgb(0xF5, 0xDE, 0xB3)),
							new ColorPickerColor("BurlyWood", Color.FromArgb(0xDE, 0xB8, 0x87)),
							new ColorPickerColor("Tan", Color.FromArgb(0xD2, 0xB4, 0x8C)),
							new ColorPickerColor("RosyBrown", Color.FromArgb(0xBC, 0x8F, 0x8F)),
							new ColorPickerColor("SandyBrown", Color.FromArgb(0xF4, 0xA4, 0x60)),
							new ColorPickerColor("Goldenrod", Color.FromArgb(0xDA, 0xA5, 0x20)),
							new ColorPickerColor("DarkGoldenrod", Color.FromArgb(0xB8, 0x86, 0x0B)),
							new ColorPickerColor("Peru", Color.FromArgb(0xCD, 0x85, 0x3F)),
							new ColorPickerColor("Chocolate", Color.FromArgb(0xD2, 0x69, 0x1E)),
							new ColorPickerColor("SaddleBrown", Color.FromArgb(0x8B, 0x45, 0x13)),
							new ColorPickerColor("Sienna", Color.FromArgb(0xA0, 0x52, 0x2D)),
							new ColorPickerColor("Brown", Color.FromArgb(0xA5, 0x2A, 0x2A)),
							new ColorPickerColor("Maroon", Color.FromArgb(0x80, 0x00, 0x00)),
						}),
					new ColorPickerColorGroup(
						"White colors",
						new ColorPickerColor[]
						{
							new ColorPickerColor("White", Color.FromArgb(0xFF, 0xFF, 0xFF)),
							new ColorPickerColor("Snow", Color.FromArgb(0xFF, 0xFA, 0xFA)),
							new ColorPickerColor("Honeydew", Color.FromArgb(0xF0, 0xFF, 0xF0)),
							new ColorPickerColor("MintCream", Color.FromArgb(0xF5, 0xFF, 0xFA)),
							new ColorPickerColor("Azure", Color.FromArgb(0xF0, 0xFF, 0xFF)),
							new ColorPickerColor("AliceBlue", Color.FromArgb(0xF0, 0xF8, 0xFF)),
							new ColorPickerColor("GhostWhite", Color.FromArgb(0xF8, 0xF8, 0xFF)),
							new ColorPickerColor("WhiteSmoke", Color.FromArgb(0xF5, 0xF5, 0xF5)),
							new ColorPickerColor("Seashell", Color.FromArgb(0xFF, 0xF5, 0xEE)),
							new ColorPickerColor("Beige", Color.FromArgb(0xF5, 0xF5, 0xDC)),
							new ColorPickerColor("OldLace", Color.FromArgb(0xFD, 0xF5, 0xE6)),
							new ColorPickerColor("FloralWhite", Color.FromArgb(0xFF, 0xFA, 0xF0)),
							new ColorPickerColor("Ivory", Color.FromArgb(0xFF, 0xFF, 0xF0)),
							new ColorPickerColor("AntiqueWhite", Color.FromArgb(0xFA, 0xEB, 0xD7)),
							new ColorPickerColor("Linen", Color.FromArgb(0xFA, 0xF0, 0xE6)),
							new ColorPickerColor("LavenderBlush", Color.FromArgb(0xFF, 0xF0, 0xF5)),
							new ColorPickerColor("MistyRose", Color.FromArgb(0xFF, 0xE4, 0xE1)),
						}),
					new ColorPickerColorGroup(
						"Grey colors",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Gainsboro", Color.FromArgb(0xDC, 0xDC, 0xDC)),
							new ColorPickerColor("LightGrey", Color.FromArgb(0xD3, 0xD3, 0xD3)),
							new ColorPickerColor("Silver", Color.FromArgb(0xC0, 0xC0, 0xC0)),
							new ColorPickerColor("DarkGray", Color.FromArgb(0xA9, 0xA9, 0xA9)),
							new ColorPickerColor("Gray", Color.FromArgb(0x80, 0x80, 0x80)),
							new ColorPickerColor("DimGray", Color.FromArgb(0x69, 0x69, 0x69)),
							new ColorPickerColor("LightSlateGray", Color.FromArgb(0x77, 0x88, 0x99)),
							new ColorPickerColor("SlateGray", Color.FromArgb(0x70, 0x80, 0x90)),
							new ColorPickerColor("DarkSlateGray", Color.FromArgb(0x2F, 0x4F, 0x4F)),
							new ColorPickerColor("Black", Color.FromArgb(0x00, 0x00, 0x00)),
						}),
				}),

			#endregion

			#region Millennium
/*
			new ColorPickerColorPalette(
				"Millennium", "cbn_millenium",
				new ColorPickerColorGroup[]
				{
					new ColorPickerColorGroup(
						"",
						new ColorPickerColor[]
						{
							new ColorPickerColor("7000W White Lilac", Color.FromArgb(0xE5, 0xDD, 0xE1)),
							new ColorPickerColor("7001W Lilac Glaze", Color.FromArgb(0xDD, 0xD3, 0xDF)),
							new ColorPickerColor("7002W Lavender Bubble", Color.FromArgb(0xCD, 0xC1, 0xD7)),
							new ColorPickerColor("7003M Lively Lavender", Color.FromArgb(0xBD, 0xAD, 0xCB)),
							new ColorPickerColor("7004M Candy Violet", Color.FromArgb(0xAB, 0x98, 0xBE)),
							new ColorPickerColor("7005D Purple Sequel", Color.FromArgb(0x87, 0x72, 0x9E)),
							new ColorPickerColor("7006N Purple Passion", Color.FromArgb(0x67, 0x52, 0x7A)),
							new ColorPickerColor("7010W Phantom Violet", Color.FromArgb(0xDC, 0xDA, 0xE1)),
							new ColorPickerColor("7011W Lilac Petal", Color.FromArgb(0xD4, 0xCF, 0xDE)),
							new ColorPickerColor("7012W Liolet Veil", Color.FromArgb(0xC8, 0xC3, 0xD9)),
							new ColorPickerColor("7013M Peruvian Violet", Color.FromArgb(0xB6, 0xAF, 0xCF)),
							new ColorPickerColor("7014M Sugarplum", Color.FromArgb(0xA5, 0x9E, 0xC3)),
							new ColorPickerColor("7015D Super Violet", Color.FromArgb(0x7B, 0x73, 0xA3)),
							new ColorPickerColor("7016A Purple Banner", Color.FromArgb(0x60, 0x59, 0x86)),
							new ColorPickerColor("7020W Frost Violet", Color.FromArgb(0xD6, 0xDA, 0xE2)),
							new ColorPickerColor("7021W Cool Lilac", Color.FromArgb(0xCC, 0xD2, 0xE0)),
							new ColorPickerColor("7022W Spring Crocus", Color.FromArgb(0xBE, 0xC4, 0xDA)),
							new ColorPickerColor("7023M Lupine", Color.FromArgb(0xAA, 0xB1, 0xD0)),
							new ColorPickerColor("7024M Poet's Purple", Color.FromArgb(0x99, 0xA0, 0xC6)),
							new ColorPickerColor("7025D Paradox", Color.FromArgb(0x6E, 0x77, 0xA6)),
							new ColorPickerColor("7026N Purple Eden", Color.FromArgb(0x4E, 0x55, 0x82)),
							new ColorPickerColor("7030W White Finesse", Color.FromArgb(0xE4, 0xE7, 0xE3)),
							new ColorPickerColor("7031W Lavender Star", Color.FromArgb(0xC7, 0xD0, 0xDF)),
							new ColorPickerColor("7032W Lavender Glow", Color.FromArgb(0xBB, 0xC7, 0xDD)),
							new ColorPickerColor("7033M Lilac Luxury", Color.FromArgb(0xA6, 0xB3, 0xD4)),
							new ColorPickerColor("7034M Blue Iris", Color.FromArgb(0x90, 0xA0, 0xC9)),
							new ColorPickerColor("7035D Lavender Blue", Color.FromArgb(0x66, 0x79, 0xAB)),
							new ColorPickerColor("7036A Blue Foxglove", Color.FromArgb(0x4E, 0x5F, 0x8F)),
							new ColorPickerColor("7040W Sedum Flower", Color.FromArgb(0xDB, 0xE4, 0xE4)),
							new ColorPickerColor("7041W Woodflower", Color.FromArgb(0xC1, 0xD2, 0xE1)),
							new ColorPickerColor("7042W Parlor Blue", Color.FromArgb(0xB1, 0xC7, 0xDD)),
							new ColorPickerColor("7043M Blue Feather", Color.FromArgb(0x9D, 0xB6, 0xD6)),
							new ColorPickerColor("7044M Brewster Blue", Color.FromArgb(0x85, 0xA3, 0xCC)),
							new ColorPickerColor("7045D Blue Epic", Color.FromArgb(0x5A, 0x7C, 0xAF)),
							new ColorPickerColor("7046A Etheria Blue", Color.FromArgb(0x41, 0x61, 0x93)),
							new ColorPickerColor("7050W Blue Chill", Color.FromArgb(0xD2, 0xE2, 0xE4)),
							new ColorPickerColor("7051W Blue Bouquet", Color.FromArgb(0xC8, 0xDC, 0xE4)),
							new ColorPickerColor("7052W Brisk Blue", Color.FromArgb(0xA7, 0xC9, 0xDF)),
							new ColorPickerColor("7053M Jonathan", Color.FromArgb(0x8E, 0xBA, 0xDA)),
							new ColorPickerColor("7054M Venetian Blue", Color.FromArgb(0x76, 0xA9, 0xD1)),
							new ColorPickerColor("7055D Pompeii", Color.FromArgb(0x49, 0x86, 0xB9)),
							new ColorPickerColor("7056N Endless Blue", Color.FromArgb(0x33, 0x63, 0x97)),
							new ColorPickerColor("7060W Cool Vista", Color.FromArgb(0xE4, 0xEC, 0xE4)),
							new ColorPickerColor("7061W Soft Sky", Color.FromArgb(0xC5, 0xDD, 0xE3)),
							new ColorPickerColor("7062W Carolina Blue", Color.FromArgb(0xA8, 0xD1, 0xE2)),
							new ColorPickerColor("7063M Blue Palace", Color.FromArgb(0x87, 0xC0, 0xDD)),
							new ColorPickerColor("7064M Water Jet", Color.FromArgb(0x70, 0xB2, 0xD7)),
							new ColorPickerColor("7065D Triumph Blue", Color.FromArgb(0x32, 0x8B, 0xC0)),
							new ColorPickerColor("7066N Stars Forever", Color.FromArgb(0x10, 0x69, 0x9E)),
							new ColorPickerColor("7070W Bridgewater", Color.FromArgb(0xDC, 0xEA, 0xE5)),
							new ColorPickerColor("7071W Meltwater", Color.FromArgb(0xB8, 0xDC, 0xE4)),
							new ColorPickerColor("7072W Heron Blue", Color.FromArgb(0xAD, 0xD8, 0xE4)),
							new ColorPickerColor("7073M Bluejay", Color.FromArgb(0x7B, 0xC1, 0xDE)),
							new ColorPickerColor("7074M Brandon's Blue", Color.FromArgb(0x64, 0xB6, 0xDC)),
							new ColorPickerColor("7075D Electron Blue", Color.FromArgb(0x11, 0x93, 0xC7)),
							new ColorPickerColor("7076A Neon Blue", Color.FromArgb(0x00, 0x79, 0xB4)),
							new ColorPickerColor("7080W Sistine Blue", Color.FromArgb(0xCA, 0xE4, 0xE4)),
							new ColorPickerColor("7081W Clear Day", Color.FromArgb(0xB9, 0xDE, 0xE1)),
							new ColorPickerColor("7082W Sky Delight", Color.FromArgb(0xA1, 0xD5, 0xE0)),
							new ColorPickerColor("7083M Blue Yonder", Color.FromArgb(0x7E, 0xC5, 0xDB)),
							new ColorPickerColor("7084M Parade Blue", Color.FromArgb(0x5F, 0xB8, 0xD4)),
							new ColorPickerColor("7085D Atlas Blue", Color.FromArgb(0x00, 0x97, 0xBD)),
							new ColorPickerColor("7086A Gulf Coast", Color.FromArgb(0x00, 0x7A, 0xA5)),
							new ColorPickerColor("7090W Blue Cool", Color.FromArgb(0xCF, 0xE7, 0xE3)),
							new ColorPickerColor("7091W Ondine Blue", Color.FromArgb(0xC1, 0xE3, 0xE3)),
							new ColorPickerColor("7092W Inspiration Blue", Color.FromArgb(0xA1, 0xD7, 0xDF)),
							new ColorPickerColor("7093M Baby Blue Eyes", Color.FromArgb(0x7D, 0xC8, 0xD7)),
							new ColorPickerColor("7094M Blue Stencil", Color.FromArgb(0x5D, 0xB9, 0xCE)),
							new ColorPickerColor("7095D Grandma Blue", Color.FromArgb(0x03, 0x99, 0xB4)),
							new ColorPickerColor("7096A Deep Marine", Color.FromArgb(0x00, 0x7D, 0x9A)),
							new ColorPickerColor("7100W Blue Facets", Color.FromArgb(0xDB, 0xEC, 0xE3)),
							new ColorPickerColor("7101W Blue Gossamer", Color.FromArgb(0xC1, 0xE5, 0xE2)),
							new ColorPickerColor("7102W Galactic Blue", Color.FromArgb(0xA1, 0xDB, 0xDB)),
							new ColorPickerColor("7103M Aqua Whisper", Color.FromArgb(0x7B, 0xCD, 0xD1)),
							new ColorPickerColor("7104M Carribean Waters", Color.FromArgb(0x5B, 0xC0, 0xC7)),
							new ColorPickerColor("7105D Matisse Blue", Color.FromArgb(0x00, 0x9D, 0xA7)),
							new ColorPickerColor("7106A Aqua Fantasy", Color.FromArgb(0x00, 0x82, 0x8A)),
							new ColorPickerColor("7110W Lithesome Blue", Color.FromArgb(0xCF, 0xEB, 0xE2)),
							new ColorPickerColor("7111W Intricate Aqua", Color.FromArgb(0xC1, 0xE7, 0xE0)),
							new ColorPickerColor("7112W Sea Port", Color.FromArgb(0xAD, 0xE2, 0xDA)),
							new ColorPickerColor("7113M Birthstone", Color.FromArgb(0x7C, 0xD1, 0xCB)),
							new ColorPickerColor("7114M Noblesse", Color.FromArgb(0x5B, 0xC4, 0xBF)),
							new ColorPickerColor("7115D Calypso", Color.FromArgb(0x00, 0xA7, 0xA0)),
							new ColorPickerColor("7116A Bloodstone", Color.FromArgb(0x00, 0x8B, 0x83)),
							new ColorPickerColor("7120W Shallow Pool", Color.FromArgb(0xD6, 0xEE, 0xE2)),
							new ColorPickerColor("7121W Cupid Green", Color.FromArgb(0xB8, 0xE7, 0xDC)),
							new ColorPickerColor("7122W Water Dance", Color.FromArgb(0xA1, 0xE1, 0xD4)),
							new ColorPickerColor("7123M Jadecast", Color.FromArgb(0x7D, 0xD5, 0xC6)),
							new ColorPickerColor("7124M Aqua Fresco", Color.FromArgb(0x5A, 0xCA, 0xB9)),
							new ColorPickerColor("7125D Indian Bead", Color.FromArgb(0x00, 0xAD, 0x99)),
							new ColorPickerColor("7126N Subterranean", Color.FromArgb(0x00, 0x8C, 0x77)),
							new ColorPickerColor("7130W Aquarill", Color.FromArgb(0xDC, 0xEF, 0xE2)),
							new ColorPickerColor("7131W Green Aqua", Color.FromArgb(0xB6, 0xE8, 0xDB)),
							new ColorPickerColor("7132W Cyan Waters", Color.FromArgb(0xA4, 0xE4, 0xD6)),
							new ColorPickerColor("7133M Tahitian Breeze", Color.FromArgb(0x7C, 0xD7, 0xC5)),
							new ColorPickerColor("7134D Jade Jewel", Color.FromArgb(0x39, 0xC1, 0xA9)),
							new ColorPickerColor("7135A Paradise Waters", Color.FromArgb(0x00, 0xA2, 0x87)),
							new ColorPickerColor("7136N Mayan Green", Color.FromArgb(0x00, 0x8F, 0x73)),
							new ColorPickerColor("7140W Nile Mist", Color.FromArgb(0xD5, 0xED, 0xDB)),
							new ColorPickerColor("7141W Tropic Sealight", Color.FromArgb(0xB3, 0xE5, 0xD3)),
							new ColorPickerColor("7142W Beau Monde", Color.FromArgb(0x9F, 0xDF, 0xCA)),
							new ColorPickerColor("7143M Wistfull Green", Color.FromArgb(0x8A, 0xD3, 0xBC)),
							new ColorPickerColor("7144D Island Dream", Color.FromArgb(0x56, 0xB6, 0x98)),
							new ColorPickerColor("7145D Green Melody", Color.FromArgb(0x3A, 0xA6, 0x85)),
							new ColorPickerColor("7146N Pageant Green", Color.FromArgb(0x13, 0x7F, 0x5E)),
							new ColorPickerColor("7150W Hint O' Green", Color.FromArgb(0xD7, 0xEC, 0xD8)),
							new ColorPickerColor("7151W Mint Frost", Color.FromArgb(0xBE, 0xE6, 0xC9)),
							new ColorPickerColor("7152W Tender Green", Color.FromArgb(0xB3, 0xE3, 0xC1)),
							new ColorPickerColor("7153M Summer Green", Color.FromArgb(0x9B, 0xD3, 0xAD)),
							new ColorPickerColor("7154D Dutch Bells", Color.FromArgb(0x6E, 0xB7, 0x87)),
							new ColorPickerColor("7155D Green Glory", Color.FromArgb(0x56, 0xA9, 0x72)),
							new ColorPickerColor("7156N Patrick Green", Color.FromArgb(0x35, 0x84, 0x51)),
							new ColorPickerColor("7160W Spring Mint", Color.FromArgb(0xD9, 0xEC, 0xD1)),
							new ColorPickerColor("7161W Wispy Green", Color.FromArgb(0xCE, 0xE8, 0xC7)),
							new ColorPickerColor("7162W Green Gulf", Color.FromArgb(0xBE, 0xE3, 0xB7)),
							new ColorPickerColor("7163M Bailey's Beads", Color.FromArgb(0x99, 0xD2, 0x94)),
							new ColorPickerColor("7164D Shocking Green", Color.FromArgb(0x6C, 0xB8, 0x68)),
							new ColorPickerColor("7165A Lucky Shamrock", Color.FromArgb(0x4A, 0xA6, 0x5A)),
							new ColorPickerColor("7166N Isle O' Green", Color.FromArgb(0x35, 0x8E, 0x44)),
							new ColorPickerColor("7170W Bon Bon", Color.FromArgb(0xDF, 0xEE, 0xD4)),
							new ColorPickerColor("7171W Willow Sweeps", Color.FromArgb(0xD0, 0xE6, 0xC1)),
							new ColorPickerColor("7172W Duet Green", Color.FromArgb(0xC3, 0xE0, 0xB2)),
							new ColorPickerColor("7173M Applejack", Color.FromArgb(0xA8, 0xCC, 0x90)),
							new ColorPickerColor("7174D Canopy Green", Color.FromArgb(0x82, 0xAE, 0x65)),
							new ColorPickerColor("7175A Belladonna", Color.FromArgb(0x6D, 0x9A, 0x50)),
							new ColorPickerColor("7176N Green Hedge", Color.FromArgb(0x59, 0x87, 0x42)),
							new ColorPickerColor("7180W Pale Parsnip", Color.FromArgb(0xE7, 0xED, 0xCE)),
							new ColorPickerColor("7181W Green Pear", Color.FromArgb(0xDB, 0xE3, 0xB4)),
							new ColorPickerColor("7182W Green Muslin", Color.FromArgb(0xCF, 0xDB, 0xA1)),
							new ColorPickerColor("7183M Candy Lime", Color.FromArgb(0xBC, 0xCD, 0x88)),
							new ColorPickerColor("7184D Green Balsam", Color.FromArgb(0x9B, 0xB3, 0x5F)),
							new ColorPickerColor("7185A Broadleaf", Color.FromArgb(0x8D, 0xA7, 0x4F)),
							new ColorPickerColor("7186N Bonzai Green", Color.FromArgb(0x7F, 0x95, 0x3E)),
							new ColorPickerColor("7190W Tiffany Glass", Color.FromArgb(0xE8, 0xE8, 0xC6)),
							new ColorPickerColor("7191W Rain Reflection", Color.FromArgb(0xE5, 0xE2, 0xB0)),
							new ColorPickerColor("7192W Tangy Green", Color.FromArgb(0xDE, 0xDC, 0x9D)),
							new ColorPickerColor("7193M Green Moth", Color.FromArgb(0xD4, 0xD2, 0x86)),
							new ColorPickerColor("7194D Spring Grass", Color.FromArgb(0xC2, 0xBC, 0x5B)),
							new ColorPickerColor("7195A Picnic Green", Color.FromArgb(0xB0, 0xAC, 0x45)),
							new ColorPickerColor("7196N Bistro Green", Color.FromArgb(0x96, 0x98, 0x3C)),
							new ColorPickerColor("7200W Shower Mint", Color.FromArgb(0xEA, 0xEA, 0xC0)),
							new ColorPickerColor("7201W Sparkle Green", Color.FromArgb(0xEA, 0xE6, 0xAE)),
							new ColorPickerColor("7202W Lilting Green", Color.FromArgb(0xE5, 0xE2, 0x9C)),
							new ColorPickerColor("7203M Breadfruit", Color.FromArgb(0xD9, 0xD8, 0x7D)),
							new ColorPickerColor("7204D Spring Chartreuse", Color.FromArgb(0xCE, 0xCA, 0x63)),
							new ColorPickerColor("7205A Limeburst", Color.FromArgb(0xBC, 0xB9, 0x3F)),
							new ColorPickerColor("7206N Pelingese", Color.FromArgb(0xA4, 0xA8, 0x30)),
							new ColorPickerColor("7210W Greenlet", Color.FromArgb(0xE8, 0xEC, 0xBE)),
							new ColorPickerColor("7211W New Spring", Color.FromArgb(0xEB, 0xE7, 0xA8)),
							new ColorPickerColor("7212W Spiney Green", Color.FromArgb(0xE5, 0xE2, 0x8C)),
							new ColorPickerColor("7213D Yellow CItrus", Color.FromArgb(0xDF, 0xD6, 0x5E)),
							new ColorPickerColor("7214D Lime Tart", Color.FromArgb(0xE2, 0xD4, 0x4E)),
							new ColorPickerColor("7215A Lime Flip", Color.FromArgb(0xDB, 0xCC, 0x38)),
							new ColorPickerColor("7216N Brite Lime", Color.FromArgb(0xC8, 0xBC, 0x18)),
							new ColorPickerColor("7220W Yellow Buff", Color.FromArgb(0xF5, 0xEB, 0xC1)),
							new ColorPickerColor("7221W Fresh Lemon", Color.FromArgb(0xFB, 0xEC, 0xB6)),
							new ColorPickerColor("7222W Daisy Petal", Color.FromArgb(0xFE, 0xE7, 0x98)),
							new ColorPickerColor("7223M Glittering Sun", Color.FromArgb(0xFF, 0xE4, 0x83)),
							new ColorPickerColor("7224D Yellow Jubilee", Color.FromArgb(0xFE, 0xD8, 0x4D)),
							new ColorPickerColor("7225A Competition Yellow", Color.FromArgb(0xFD, 0xD0, 0x38)),
							new ColorPickerColor("7226N Sun Valley", Color.FromArgb(0xF9, 0xC0, 0x00)),
							new ColorPickerColor("7230W Translucence", Color.FromArgb(0xF9, 0xED, 0xC9)),
							new ColorPickerColor("7231W Afternoon Delight", Color.FromArgb(0xFA, 0xE9, 0xAC)),
							new ColorPickerColor("7232W Mineral Yellow", Color.FromArgb(0xFC, 0xE3, 0x98)),
							new ColorPickerColor("7233M Southern Sun", Color.FromArgb(0xFD, 0xDC, 0x80)),
							new ColorPickerColor("7234D Yellow Tulip", Color.FromArgb(0xFB, 0xCF, 0x59)),
							new ColorPickerColor("7235A Ornate Yellow", Color.FromArgb(0xF9, 0xC4, 0x3E)),
							new ColorPickerColor("7236N Imperial Yellow", Color.FromArgb(0xF4, 0xB6, 0x1F)),
							new ColorPickerColor("7240W Chaste White", Color.FromArgb(0xF7, 0xEC, 0xCD)),
							new ColorPickerColor("7241W Sun Shimmer", Color.FromArgb(0xFD, 0xE7, 0xAB)),
							new ColorPickerColor("7242W Yellow Begonia", Color.FromArgb(0xFF, 0xE2, 0x95)),
							new ColorPickerColor("7243M Forsythia", Color.FromArgb(0xFF, 0xD8, 0x7A)),
							new ColorPickerColor("7244D Golden Ray", Color.FromArgb(0xFF, 0xCA, 0x4C)),
							new ColorPickerColor("7245A Glimmer Gold", Color.FromArgb(0xFF, 0xC2, 0x35)),
							new ColorPickerColor("7246N Golden Dome", Color.FromArgb(0xFF, 0xB7, 0x21)),
							new ColorPickerColor("7250W Serenity", Color.FromArgb(0xF4, 0xEC, 0xD1)),
							new ColorPickerColor("7251W Lemon Meringue", Color.FromArgb(0xFC, 0xE5, 0xAE)),
							new ColorPickerColor("7252W Sundew", Color.FromArgb(0xFF, 0xE1, 0x9C)),
							new ColorPickerColor("7253M Golden Corn", Color.FromArgb(0xFF, 0xD7, 0x7C)),
							new ColorPickerColor("7254D Black-Eyed Susan", Color.FromArgb(0xFF, 0xC7, 0x48)),
							new ColorPickerColor("7255A High Sun", Color.FromArgb(0xFF, 0xBE, 0x30)),
							new ColorPickerColor("7256N Coronation Gold", Color.FromArgb(0xFF, 0xAA, 0x00)),
							new ColorPickerColor("7260W Amber White", Color.FromArgb(0xFB, 0xEB, 0xCA)),
							new ColorPickerColor("7261W Sunny Beach", Color.FromArgb(0xFD, 0xE7, 0xBB)),
							new ColorPickerColor("7262W Warm Rays", Color.FromArgb(0xFF, 0xDD, 0x97)),
							new ColorPickerColor("7263M Sunspot", Color.FromArgb(0xFF, 0xD1, 0x74)),
							new ColorPickerColor("7264D Sardonyx", Color.FromArgb(0xFF, 0xBF, 0x43)),
							new ColorPickerColor("7265A Golden Glisten", Color.FromArgb(0xFF, 0xB7, 0x31)),
							new ColorPickerColor("7266N Garden Marigold", Color.FromArgb(0xFF, 0x9E, 0x00)),
							new ColorPickerColor("7270W Filtered Light", Color.FromArgb(0xFA, 0xEC, 0xCE)),
							new ColorPickerColor("7271W Morning Moon", Color.FromArgb(0xFD, 0xE6, 0xB9)),
							new ColorPickerColor("7272W Honey Pear", Color.FromArgb(0xFF, 0xDA, 0x93)),
							new ColorPickerColor("7273M Amber Light", Color.FromArgb(0xFF, 0xCC, 0x72)),
							new ColorPickerColor("7274D Scotch Yellow", Color.FromArgb(0xFF, 0xBA, 0x3E)),
							new ColorPickerColor("7275A Imperial Palace", Color.FromArgb(0xFF, 0xB1, 0x2D)),
							new ColorPickerColor("7276N Palace Gold", Color.FromArgb(0xFF, 0x9C, 0x07)),
							new ColorPickerColor("7280W Golden Blonde", Color.FromArgb(0xFA, 0xE5, 0xC6)),
							new ColorPickerColor("7281W Palest Sunset", Color.FromArgb(0xFB, 0xE0, 0xB8)),
							new ColorPickerColor("7282W Candleglow", Color.FromArgb(0xFF, 0xD2, 0x92)),
							new ColorPickerColor("7283M Matisse Gold", Color.FromArgb(0xFF, 0xC3, 0x74)),
							new ColorPickerColor("7284D Fireside", Color.FromArgb(0xFF, 0xB5, 0x5B)),
							new ColorPickerColor("7285A Marsh Marigold", Color.FromArgb(0xFF, 0xA3, 0x34)),
							new ColorPickerColor("7286N Brilliant Orange", Color.FromArgb(0xFD, 0x8F, 0x14)),
							new ColorPickerColor("7290W Natural Wax", Color.FromArgb(0xFB, 0xE1, 0xBE)),
							new ColorPickerColor("7291W Runnymede", Color.FromArgb(0xFE, 0xDA, 0xAD)),
							new ColorPickerColor("7292W Freestone", Color.FromArgb(0xFF, 0xD3, 0x9A)),
							new ColorPickerColor("7293M Summer Melon", Color.FromArgb(0xFD, 0xBE, 0x7A)),
							new ColorPickerColor("7294D Pom Pom Orange", Color.FromArgb(0xF9, 0xAF, 0x60)),
							new ColorPickerColor("7295D Malaga", Color.FromArgb(0xF4, 0xA0, 0x4E)),
							new ColorPickerColor("7296N Barcelona", Color.FromArgb(0xE2, 0x82, 0x2E)),
							new ColorPickerColor("7300W Pale Chamois", Color.FromArgb(0xFB, 0xE4, 0xC7)),
							new ColorPickerColor("7301W Brushed Peach", Color.FromArgb(0xFC, 0xD9, 0xB2)),
							new ColorPickerColor("7302W Carmelite", Color.FromArgb(0xF8, 0xC9, 0x98)),
							new ColorPickerColor("7303M Aztec Dawn", Color.FromArgb(0xF2, 0xB3, 0x7A)),
							new ColorPickerColor("7304D Rice Curry", Color.FromArgb(0xF1, 0xA7, 0x66)),
							new ColorPickerColor("7305D Sundance", Color.FromArgb(0xE5, 0x97, 0x54)),
							new ColorPickerColor("7306N Cinnamon Cider", Color.FromArgb(0xD5, 0x7C, 0x3A)),
							new ColorPickerColor("7310W Camisole Silk", Color.FromArgb(0xFB, 0xE5, 0xC7)),
							new ColorPickerColor("7311W Peach Mallow", Color.FromArgb(0xFA, 0xD6, 0xB1)),
							new ColorPickerColor("7312W Filtered Moon", Color.FromArgb(0xF9, 0xCD, 0xA6)),
							new ColorPickerColor("7313M New Penny", Color.FromArgb(0xF2, 0xB0, 0x7D)),
							new ColorPickerColor("7314D Octoberfest", Color.FromArgb(0xF0, 0xA6, 0x6C)),
							new ColorPickerColor("7315D Rustica", Color.FromArgb(0xE5, 0x94, 0x5A)),
							new ColorPickerColor("7316N Iron Orange", Color.FromArgb(0xCF, 0x74, 0x3B)),
							new ColorPickerColor("7320W Flemish Lace", Color.FromArgb(0xF7, 0xE5, 0xCC)),
							new ColorPickerColor("7321W Apricot Lily", Color.FromArgb(0xFA, 0xD8, 0xB5)),
							new ColorPickerColor("7322W Petite Peach", Color.FromArgb(0xFF, 0xC9, 0x9A)),
							new ColorPickerColor("7323M Peruvian Orange", Color.FromArgb(0xFE, 0xB3, 0x7B)),
							new ColorPickerColor("7324D Indian Turnip", Color.FromArgb(0xF9, 0xA3, 0x63)),
							new ColorPickerColor("7325D Diced Carrot", Color.FromArgb(0xF4, 0x93, 0x50)),
							new ColorPickerColor("7326A Glazed Carrot", Color.FromArgb(0xE9, 0x7D, 0x3B)),
							new ColorPickerColor("7330W Silky White", Color.FromArgb(0xF9, 0xE2, 0xCA)),
							new ColorPickerColor("7331W Peach Veil", Color.FromArgb(0xFB, 0xD6, 0xB8)),
							new ColorPickerColor("7332W Peach Satin", Color.FromArgb(0xFD, 0xC7, 0xA0)),
							new ColorPickerColor("7333M Endless Peach", Color.FromArgb(0xFA, 0xB1, 0x81)),
							new ColorPickerColor("7334D Orangeade", Color.FromArgb(0xF7, 0xA2, 0x6C)),
							new ColorPickerColor("7335D Golden Poppy", Color.FromArgb(0xF3, 0x91, 0x58)),
							new ColorPickerColor("7336A Tahitian Sun", Color.FromArgb(0xE7, 0x77, 0x41)),
							new ColorPickerColor("7340W Ocean Shell", Color.FromArgb(0xF9, 0xE5, 0xD5)),
							new ColorPickerColor("7341W Coral Almond", Color.FromArgb(0xFA, 0xD7, 0xBE)),
							new ColorPickerColor("7342W Apricot Nector", Color.FromArgb(0xFC, 0xC2, 0xA2)),
							new ColorPickerColor("7343M Salted Shrimp", Color.FromArgb(0xF8, 0xAA, 0x85)),
							new ColorPickerColor("7344D Shrimp Peel", Color.FromArgb(0xF3, 0x99, 0x6D)),
							new ColorPickerColor("7345D Fired Shrimp", Color.FromArgb(0xEA, 0x86, 0x5A)),
							new ColorPickerColor("7346A Mikado", Color.FromArgb(0xE0, 0x71, 0x45)),
							new ColorPickerColor("7350W Orange Charm", Color.FromArgb(0xF1, 0xE5, 0xD2)),
							new ColorPickerColor("7351W Always Apricot", Color.FromArgb(0xFA, 0xDC, 0xC7)),
							new ColorPickerColor("7352W Cayman Peach", Color.FromArgb(0xFB, 0xCB, 0xAE)),
							new ColorPickerColor("7353M Peach Darling", Color.FromArgb(0xF9, 0xB1, 0x8B)),
							new ColorPickerColor("7354D Orange Buff", Color.FromArgb(0xF3, 0x94, 0x67)),
							new ColorPickerColor("7355A Fiesta Fire", Color.FromArgb(0xEC, 0x7C, 0x4E)),
							new ColorPickerColor("7356N Autumn Blaze", Color.FromArgb(0xE5, 0x68, 0x3E)),
							new ColorPickerColor("7360W Coral Sand", Color.FromArgb(0xF5, 0xE6, 0xD9)),
							new ColorPickerColor("7361W Apricot Glove", Color.FromArgb(0xF9, 0xD6, 0xC2)),
							new ColorPickerColor("7362W Malta Pink", Color.FromArgb(0xFA, 0xCC, 0xB6)),
							new ColorPickerColor("7363M Peach Parfait", Color.FromArgb(0xF8, 0xA6, 0x8A)),
							new ColorPickerColor("7364D Coral Azalea", Color.FromArgb(0xF0, 0x8D, 0x6E)),
							new ColorPickerColor("7365A New Orange", Color.FromArgb(0xE4, 0x74, 0x51)),
							new ColorPickerColor("7366N Orange Flambe'", Color.FromArgb(0xCD, 0x57, 0x38)),
							new ColorPickerColor("7370W Angel Peach", Color.FromArgb(0xF7, 0xDF, 0xD2)),
							new ColorPickerColor("7371W Peach Opal", Color.FromArgb(0xF8, 0xD7, 0xC8)),
							new ColorPickerColor("7372W Coral Haze", Color.FromArgb(0xFA, 0xC9, 0xBB)),
							new ColorPickerColor("7373M Corallina", Color.FromArgb(0xF5, 0xA3, 0x90)),
							new ColorPickerColor("7374D Phoenix Sun", Color.FromArgb(0xE7, 0x84, 0x6C)),
							new ColorPickerColor("7375A Tangerine", Color.FromArgb(0xD8, 0x6B, 0x52)),
							new ColorPickerColor("7376N Fired Orange", Color.FromArgb(0xBB, 0x49, 0x34)),
							new ColorPickerColor("7380W Creamy Pink", Color.FromArgb(0xF8, 0xE3, 0xD9)),
							new ColorPickerColor("7381W Crushed Shell", Color.FromArgb(0xF8, 0xD4, 0xCA)),
							new ColorPickerColor("7382W Granite Rose", Color.FromArgb(0xF8, 0xC9, 0xBE)),
							new ColorPickerColor("7383M Coral Fantasy", Color.FromArgb(0xF3, 0xA2, 0x96)),
							new ColorPickerColor("7384D Coral Gate", Color.FromArgb(0xE6, 0x80, 0x70)),
							new ColorPickerColor("7385A Coral Flower", Color.FromArgb(0xD7, 0x69, 0x58)),
							new ColorPickerColor("7386N Red Flare", Color.FromArgb(0xB8, 0x4C, 0x3F)),
							new ColorPickerColor("7390W April Blush", Color.FromArgb(0xF5, 0xE6, 0xDC)),
							new ColorPickerColor("7391W Peaking Pink", Color.FromArgb(0xF7, 0xD6, 0xD1)),
							new ColorPickerColor("7392W Peach Bloom", Color.FromArgb(0xF8, 0xCB, 0xC6)),
							new ColorPickerColor("7393M Camellia Pink", Color.FromArgb(0xF2, 0xA1, 0x9E)),
							new ColorPickerColor("7394D Peach Pink", Color.FromArgb(0xE2, 0x7B, 0x77)),
							new ColorPickerColor("7395A Lobster Tail", Color.FromArgb(0xDA, 0x69, 0x62)),
							new ColorPickerColor("7396N Fire Princess", Color.FromArgb(0xBB, 0x43, 0x3B)),
							new ColorPickerColor("7400W Ceramic Pink", Color.FromArgb(0xF7, 0xDF, 0xD6)),
							new ColorPickerColor("7401W Melon Ice", Color.FromArgb(0xF6, 0xD5, 0xD1)),
							new ColorPickerColor("7402W Pink Crest", Color.FromArgb(0xF5, 0xC8, 0xC5)),
							new ColorPickerColor("7403M Exquisite", Color.FromArgb(0xF2, 0x9E, 0xA4)),
							new ColorPickerColor("7404D Melon Shine", Color.FromArgb(0xE8, 0x7A, 0x7F)),
							new ColorPickerColor("7405A Fire Pink", Color.FromArgb(0xDF, 0x65, 0x67)),
							new ColorPickerColor("7406N Real Red", Color.FromArgb(0xBF, 0x42, 0x3E)),
							new ColorPickerColor("7410W Ballet", Color.FromArgb(0xF4, 0xE7, 0xE0)),
							new ColorPickerColor("7411W Pink Ribbon", Color.FromArgb(0xF6, 0xD5, 0xD9)),
							new ColorPickerColor("7412W Timeless Pink", Color.FromArgb(0xF7, 0xC7, 0xD0)),
							new ColorPickerColor("7413M Pricilla Pink", Color.FromArgb(0xF5, 0x9F, 0xB3)),
							new ColorPickerColor("7414D Carnation Kiss", Color.FromArgb(0xE9, 0x76, 0x8B)),
							new ColorPickerColor("7415A Valentine", Color.FromArgb(0xD9, 0x5A, 0x6E)),
							new ColorPickerColor("7416N Chinese Pagoda", Color.FromArgb(0xC8, 0x45, 0x4E)),
							new ColorPickerColor("7420W Pink Pomander", Color.FromArgb(0xF6, 0xDE, 0xDE)),
							new ColorPickerColor("7421W Romantic Lace", Color.FromArgb(0xF7, 0xD4, 0xD8)),
							new ColorPickerColor("7422W Porcelain Pink", Color.FromArgb(0xF5, 0xC8, 0xD0)),
							new ColorPickerColor("7423M Peony Petal", Color.FromArgb(0xF2, 0x9D, 0xB2)),
							new ColorPickerColor("7424D Strawberry Swirl", Color.FromArgb(0xE5, 0x77, 0x8E)),
							new ColorPickerColor("7425A Precious Rose", Color.FromArgb(0xDA, 0x62, 0x77)),
							new ColorPickerColor("7426N Zinnia Scarlet", Color.FromArgb(0xBF, 0x3D, 0x46)),
							new ColorPickerColor("7430W Pink Icing", Color.FromArgb(0xF4, 0xE2, 0xE0)),
							new ColorPickerColor("7431W Ashley Pink", Color.FromArgb(0xF3, 0xD0, 0xD7)),
							new ColorPickerColor("7432W Pink Crown", Color.FromArgb(0xF2, 0xC6, 0xD1)),
							new ColorPickerColor("7433M Rose Freeze", Color.FromArgb(0xE8, 0x9A, 0xB4)),
							new ColorPickerColor("7434D Fasion Pink", Color.FromArgb(0xD6, 0x72, 0x92)),
							new ColorPickerColor("7435A Veronica", Color.FromArgb(0xCB, 0x5D, 0x7C)),
							new ColorPickerColor("7436N Brilliant Pink", Color.FromArgb(0xAF, 0x3D, 0x50)),
							new ColorPickerColor("7440W Lilium", Color.FromArgb(0xF1, 0xDF, 0xDF)),
							new ColorPickerColor("7441W Pink Tiara", Color.FromArgb(0xF1, 0xD2, 0xDA)),
							new ColorPickerColor("7442W Angel Wing", Color.FromArgb(0xEF, 0xC5, 0xD5)),
							new ColorPickerColor("7443M Cerise Pink", Color.FromArgb(0xE1, 0x98, 0xB8)),
							new ColorPickerColor("7444D Fuschia Flock", Color.FromArgb(0xCC, 0x6E, 0x95)),
							new ColorPickerColor("7445A Fuschia Kiss", Color.FromArgb(0xBF, 0x5C, 0x82)),
							new ColorPickerColor("7446N Sizzling Pink", Color.FromArgb(0xAA, 0x45, 0x63)),
							new ColorPickerColor("7450W Trailing Pink", Color.FromArgb(0xF2, 0xE4, 0xE2)),
							new ColorPickerColor("7451W Early Pink", Color.FromArgb(0xED, 0xD1, 0xDD)),
							new ColorPickerColor("7452W Pink Overlay", Color.FromArgb(0xE8, 0xC3, 0xD6)),
							new ColorPickerColor("7453M Cut Lavender", Color.FromArgb(0xE0, 0xAC, 0xCA)),
							new ColorPickerColor("7454D Souvenir Rose", Color.FromArgb(0xC7, 0x78, 0xA3)),
							new ColorPickerColor("7455A Rose Luster", Color.FromArgb(0xB6, 0x5E, 0x89)),
							new ColorPickerColor("7456N Majestic Magenta", Color.FromArgb(0x9C, 0x47, 0x6B)),
							new ColorPickerColor("7460W Rose Morn", Color.FromArgb(0xEF, 0xE4, 0xDF)),
							new ColorPickerColor("7461W Island Dawn", Color.FromArgb(0xE7, 0xD9, 0xDF)),
							new ColorPickerColor("7462W Fresh Bud", Color.FromArgb(0xDE, 0xC3, 0xD7)),
							new ColorPickerColor("7463M Laurel Blossom", Color.FromArgb(0xC5, 0x99, 0xBD)),
							new ColorPickerColor("7464D Violet Cluster", Color.FromArgb(0xB7, 0x84, 0xAD)),
							new ColorPickerColor("7465D Imperial Purple", Color.FromArgb(0xA6, 0x71, 0x9D)),
							new ColorPickerColor("7466N Royal Robe", Color.FromArgb(0x81, 0x4D, 0x74)),
							new ColorPickerColor("7470W Iced Orchid", Color.FromArgb(0xE5, 0xDC, 0xE0)),
							new ColorPickerColor("7471W Lavender Posy", Color.FromArgb(0xE0, 0xD2, 0xDF)),
							new ColorPickerColor("7472W Orchid Scent", Color.FromArgb(0xD4, 0xC2, 0xD7)),
							new ColorPickerColor("7473M Lavender Mist", Color.FromArgb(0xB8, 0x9A, 0xBF)),
							new ColorPickerColor("7474D Salina Grape", Color.FromArgb(0xAA, 0x88, 0xB1)),
							new ColorPickerColor("7475D Wild Orchid", Color.FromArgb(0x96, 0x73, 0x9F)),
							new ColorPickerColor("7476N Purplestone", Color.FromArgb(0x71, 0x50, 0x79)),
							new ColorPickerColor("7480W Demure", Color.FromArgb(0xE2, 0xDC, 0xDF)),
							new ColorPickerColor("7481W Violet Dust", Color.FromArgb(0xDA, 0xD2, 0xDC)),
							new ColorPickerColor("7482W Early Purple", Color.FromArgb(0xCF, 0xC8, 0xD6)),
							new ColorPickerColor("7483M Violet Petal", Color.FromArgb(0xBB, 0xAD, 0xC7)),
							new ColorPickerColor("7484M Wild Iris", Color.FromArgb(0xA1, 0x92, 0xB3)),
							new ColorPickerColor("7485D Violet Bouquet", Color.FromArgb(0x82, 0x73, 0x98)),
							new ColorPickerColor("7486N Prophetic Purple", Color.FromArgb(0x5D, 0x50, 0x6D)),
							new ColorPickerColor("7490W Faded Lilac", Color.FromArgb(0xDD, 0xDC, 0xE1)),
							new ColorPickerColor("7491W Lilac Lace", Color.FromArgb(0xD4, 0xD2, 0xDC)),
							new ColorPickerColor("7492W Lilac Sachet", Color.FromArgb(0xC8, 0xC6, 0xD7)),
							new ColorPickerColor("7493M Pastoral Purple", Color.FromArgb(0xB2, 0xAD, 0xCA)),
							new ColorPickerColor("7494M Monet's Purple", Color.FromArgb(0x9F, 0x9B, 0xBE)),
							new ColorPickerColor("7495D Princely Purple", Color.FromArgb(0x76, 0x76, 0xA0)),
							new ColorPickerColor("7496N Ruling Royalty", Color.FromArgb(0x4C, 0x49, 0x6C)),
							new ColorPickerColor("7500W Light Lilac", Color.FromArgb(0xDE, 0xE0, 0xE3)),
							new ColorPickerColor("7501W Violet Charm", Color.FromArgb(0xD3, 0xD5, 0xDC)),
							new ColorPickerColor("7502W Lyric Iris", Color.FromArgb(0xC8, 0xC8, 0xD5)),
							new ColorPickerColor("7503M Purple Aster", Color.FromArgb(0xAE, 0xB0, 0xC8)),
							new ColorPickerColor("7504M Sparkling Grape", Color.FromArgb(0x9A, 0x9E, 0xBF)),
							new ColorPickerColor("7505D Night Shade", Color.FromArgb(0x74, 0x76, 0xA3)),
							new ColorPickerColor("7506N Deep Purple", Color.FromArgb(0x48, 0x4B, 0x6E)),
							new ColorPickerColor("7510W Snowlight", Color.FromArgb(0xDE, 0xE2, 0xE4)),
							new ColorPickerColor("7511W Pastel Periwinkle", Color.FromArgb(0xC9, 0xD6, 0xE1)),
							new ColorPickerColor("7512W Periwinkle Promise", Color.FromArgb(0xB5, 0xC3, 0xD7)),
							new ColorPickerColor("7513W Autumn Blue", Color.FromArgb(0xA5, 0xB8, 0xD3)),
							new ColorPickerColor("7514M Sedona Shadow", Color.FromArgb(0x89, 0x9B, 0xC1)),
							new ColorPickerColor("7515D Mysteria", Color.FromArgb(0x6D, 0x81, 0xAB)),
							new ColorPickerColor("7516N Pharaoh's Blue", Color.FromArgb(0x45, 0x54, 0x7D)),
							new ColorPickerColor("7520W Blue Dash", Color.FromArgb(0xDB, 0xE2, 0xE0)),
							new ColorPickerColor("7521W River Run Blue", Color.FromArgb(0xC7, 0xD6, 0xE0)),
							new ColorPickerColor("7522W Fantasia Blue", Color.FromArgb(0xB8, 0xCB, 0xDB)),
							new ColorPickerColor("7523M Blue Hills", Color.FromArgb(0x96, 0xB0, 0xCE)),
							new ColorPickerColor("7524M Wonder Blue", Color.FromArgb(0x84, 0xA1, 0xC3)),
							new ColorPickerColor("7525D Diva Blue", Color.FromArgb(0x5C, 0x7D, 0xA7)),
							new ColorPickerColor("7526N Enchanted Blue", Color.FromArgb(0x39, 0x4A, 0x6C)),
							new ColorPickerColor("7530W Frosted Glass", Color.FromArgb(0xD2, 0xE2, 0xE4)),
							new ColorPickerColor("7531W Milano Blue", Color.FromArgb(0xC2, 0xD6, 0xDF)),
							new ColorPickerColor("7532W Botticelli Blue", Color.FromArgb(0xB0, 0xCC, 0xDE)),
							new ColorPickerColor("7533M Magic Blue", Color.FromArgb(0x98, 0xBE, 0xD8)),
							new ColorPickerColor("7534M Toscana Blue", Color.FromArgb(0x75, 0xA7, 0xCC)),
							new ColorPickerColor("7535D Cobalt Blue", Color.FromArgb(0x46, 0x80, 0xAF)),
							new ColorPickerColor("7536N Deep Space", Color.FromArgb(0x2D, 0x54, 0x7E)),
							new ColorPickerColor("7540W Snowdrop", Color.FromArgb(0xCB, 0xDF, 0xE4)),
							new ColorPickerColor("7541W Lovebird Feather", Color.FromArgb(0xBD, 0xD7, 0xE1)),
							new ColorPickerColor("7542W Wishful Blue", Color.FromArgb(0xAE, 0xCD, 0xDD)),
							new ColorPickerColor("7543M Flaxflower", Color.FromArgb(0x8F, 0xB7, 0xCF)),
							new ColorPickerColor("7544M Fresh Water", Color.FromArgb(0x7A, 0xA7, 0xC5)),
							new ColorPickerColor("7545D Norwegian Blue", Color.FromArgb(0x48, 0x81, 0xA7)),
							new ColorPickerColor("7546N Baltic Blue", Color.FromArgb(0x31, 0x5A, 0x7D)),
							new ColorPickerColor("7550W Iceland Blue", Color.FromArgb(0xCE, 0xE1, 0xE1)),
							new ColorPickerColor("7551W Bunting Blue", Color.FromArgb(0xB1, 0xD5, 0xDF)),
							new ColorPickerColor("7552W Blue Lady", Color.FromArgb(0xA5, 0xCC, 0xD9)),
							new ColorPickerColor("7553M Nautilus", Color.FromArgb(0x8C, 0xBD, 0xD0)),
							new ColorPickerColor("7554M Castaway Blue", Color.FromArgb(0x6A, 0xA9, 0xC4)),
							new ColorPickerColor("7555D Sea Route", Color.FromArgb(0x37, 0x81, 0xA2)),
							new ColorPickerColor("7556A Mariner", Color.FromArgb(0x26, 0x68, 0x88)),
							new ColorPickerColor("7560W Ice Crystal", Color.FromArgb(0xC7, 0xE1, 0xDF)),
							new ColorPickerColor("7561W Blue Current", Color.FromArgb(0xB7, 0xDA, 0xDB)),
							new ColorPickerColor("7562W Crescent Blue", Color.FromArgb(0xA1, 0xCE, 0xD5)),
							new ColorPickerColor("7563M Low Tide", Color.FromArgb(0x87, 0xBF, 0xC9)),
							new ColorPickerColor("7564M Blue Seclusion", Color.FromArgb(0x6D, 0xAE, 0xBD)),
							new ColorPickerColor("7565D Cove Blue", Color.FromArgb(0x3D, 0x88, 0x9A)),
							new ColorPickerColor("7566A Voyager Blue", Color.FromArgb(0x20, 0x6C, 0x82)),
							new ColorPickerColor("7570W Heaven Scent", Color.FromArgb(0xCD, 0xE0, 0xDC)),
							new ColorPickerColor("7571W Blue Inlet", Color.FromArgb(0xBA, 0xDF, 0xDC)),
							new ColorPickerColor("7572W Placid Sea", Color.FromArgb(0xA6, 0xD6, 0xD8)),
							new ColorPickerColor("7573M Waterside", Color.FromArgb(0x7F, 0xC3, 0xC9)),
							new ColorPickerColor("7574M Perry Blue", Color.FromArgb(0x62, 0xB4, 0xBC)),
							new ColorPickerColor("7575D Velvet Teal", Color.FromArgb(0x36, 0x92, 0x9C)),
							new ColorPickerColor("7576A Aqueduct Blue", Color.FromArgb(0x15, 0x72, 0x7C)),
							new ColorPickerColor("7580W Mermaid Tear", Color.FromArgb(0xC3, 0xE2, 0xDC)),
							new ColorPickerColor("7581W Ckarity Blue", Color.FromArgb(0xB4, 0xDC, 0xD8)),
							new ColorPickerColor("7582W Aqua Edge", Color.FromArgb(0x9F, 0xD2, 0xD0)),
							new ColorPickerColor("7583M Aqua Sweeps", Color.FromArgb(0x86, 0xC5, 0xC3)),
							new ColorPickerColor("7584m Galleon", Color.FromArgb(0x6C, 0xB4, 0xB4)),
							new ColorPickerColor("7585D Sea Fascination", Color.FromArgb(0x3E, 0x8E, 0x90)),
							new ColorPickerColor("7586A Mighty Man", Color.FromArgb(0x22, 0x70, 0x72)),
							new ColorPickerColor("7590W Distant Blue", Color.FromArgb(0xCA, 0xE4, 0xDD)),
							new ColorPickerColor("7591W Cruising Blue", Color.FromArgb(0xBC, 0xE1, 0xDB)),
							new ColorPickerColor("7592W Abstract Aqua", Color.FromArgb(0xAA, 0xDC, 0xD5)),
							new ColorPickerColor("7593M Aquifer", Color.FromArgb(0x88, 0xCA, 0xC5)),
							new ColorPickerColor("7594M Etruscan Blue", Color.FromArgb(0x6A, 0xBA, 0xB5)),
							new ColorPickerColor("7595D Sea Sphinx", Color.FromArgb(0x36, 0x95, 0x91)),
							new ColorPickerColor("7596A Aqua Dazzle", Color.FromArgb(0x1E, 0x7A, 0x77)),
							new ColorPickerColor("7600W Cistern Blue", Color.FromArgb(0xCD, 0xE5, 0xDC)),
							new ColorPickerColor("7601W Smooth Sailing", Color.FromArgb(0xB8, 0xDF, 0xD5)),
							new ColorPickerColor("7602W Sea Lyric", Color.FromArgb(0xA4, 0xD7, 0xCE)),
							new ColorPickerColor("7603M Bali Hi", Color.FromArgb(0x8C, 0xCB, 0xC2)),
							new ColorPickerColor("7604M Yama Light", Color.FromArgb(0x68, 0xB4, 0xAB)),
							new ColorPickerColor("7605D Turquoise Mosaic", Color.FromArgb(0x41, 0x94, 0x8A)),
							new ColorPickerColor("7606A Sea Venture", Color.FromArgb(0x29, 0x78, 0x6E)),
							new ColorPickerColor("7610W Poolside", Color.FromArgb(0xD0, 0xE9, 0xD9)),
							new ColorPickerColor("7611W Falling Water", Color.FromArgb(0xC1, 0xE2, 0xD4)),
							new ColorPickerColor("7612W Headwaters", Color.FromArgb(0xAE, 0xD9, 0xC9)),
							new ColorPickerColor("7613M Sea Spray", Color.FromArgb(0x91, 0xC9, 0xB8)),
							new ColorPickerColor("7614M Sea Quarry", Color.FromArgb(0x78, 0xBD, 0xA8)),
							new ColorPickerColor("7615 Sea Queen", Color.FromArgb(0x4D, 0x9A, 0x82)),
							new ColorPickerColor("7616N Ultra Geen", Color.FromArgb(0x31, 0x71, 0x59)),
							new ColorPickerColor("7620W Opal Green", Color.FromArgb(0xD5, 0xE9, 0xD9)),
							new ColorPickerColor("7621W Filtered Green", Color.FromArgb(0xC3, 0xE3, 0xCF)),
							new ColorPickerColor("7622W Chiffon Mist", Color.FromArgb(0xB3, 0xDD, 0xC5)),
							new ColorPickerColor("7623M Spun Green", Color.FromArgb(0x99, 0xCE, 0xB0)),
							new ColorPickerColor("7624M Porch View", Color.FromArgb(0x82, 0xBE, 0x9E)),
							new ColorPickerColor("7625D Maidenhair Fern", Color.FromArgb(0x5E, 0xA0, 0x79)),
							new ColorPickerColor("7626N Irish Juniper", Color.FromArgb(0x3D, 0x6F, 0x4E)),
							new ColorPickerColor("7630W Dewmist", Color.FromArgb(0xE0, 0xEC, 0xDB)),
							new ColorPickerColor("7631W Slender Stem", Color.FromArgb(0xC4, 0xDE, 0xC6)),
							new ColorPickerColor("7632W Green Pond", Color.FromArgb(0xB4, 0xD6, 0xBA)),
							new ColorPickerColor("7633M Water Cress", Color.FromArgb(0xA7, 0xCE, 0xAB)),
							new ColorPickerColor("7634M Green Summit", Color.FromArgb(0x92, 0xC0, 0x99)),
							new ColorPickerColor("7635D Artemis", Color.FromArgb(0x67, 0x9E, 0x71)),
							new ColorPickerColor("7636N Lush Veranda", Color.FromArgb(0x39, 0x6E, 0x46)),
							new ColorPickerColor("7640W Snowy Pine", Color.FromArgb(0xE0, 0xEB, 0xD9)),
							new ColorPickerColor("7641W Clover Tip", Color.FromArgb(0xCB, 0xE1, 0xC7)),
							new ColorPickerColor("7642W Impatient Green", Color.FromArgb(0xB9, 0xD6, 0xB8)),
							new ColorPickerColor("7643M Weeping Willow", Color.FromArgb(0xAC, 0xCA, 0xA7)),
							new ColorPickerColor("7644M Shaded Green", Color.FromArgb(0x93, 0xB8, 0x90)),
							new ColorPickerColor("7645D Fairway", Color.FromArgb(0x78, 0x9D, 0x72)),
							new ColorPickerColor("7646N Wild Mint", Color.FromArgb(0x54, 0x74, 0x4C)),
							new ColorPickerColor("7650W Creamy Cucumber", Color.FromArgb(0xE3, 0xEA, 0xD6)),
							new ColorPickerColor("7651W Garden Fragrance", Color.FromArgb(0xCE, 0xDD, 0xBF)),
							new ColorPickerColor("7652W Summer Lullaby", Color.FromArgb(0xC1, 0xD4, 0xAF)),
							new ColorPickerColor("7653M Village Green", Color.FromArgb(0xB5, 0xC9, 0xA1)),
							new ColorPickerColor("7654M Garland Green", Color.FromArgb(0x9E, 0xB8, 0x8B)),
							new ColorPickerColor("7655D Barbados Green", Color.FromArgb(0x83, 0x9C, 0x69)),
							new ColorPickerColor("7656N Dandelion", Color.FromArgb(0x62, 0x77, 0x47)),
							new ColorPickerColor("7660W Willow Wind", Color.FromArgb(0xE9, 0xEA, 0xD6)),
							new ColorPickerColor("7661W Delicate Vine", Color.FromArgb(0xCE, 0xD7, 0xB6)),
							new ColorPickerColor("7662W Cypress Garden", Color.FromArgb(0xC4, 0xCF, 0xAB)),
							new ColorPickerColor("7663M Aspen Meadows", Color.FromArgb(0xBE, 0xC9, 0xA0)),
							new ColorPickerColor("7664M Thin Asparagus", Color.FromArgb(0xA7, 0xB5, 0x85)),
							new ColorPickerColor("7665D Aloe Vera", Color.FromArgb(0x8B, 0x9B, 0x68)),
							new ColorPickerColor("7666N Cactus Pete", Color.FromArgb(0x6D, 0x78, 0x46)),
							new ColorPickerColor("7670W Minced Onion", Color.FromArgb(0xEA, 0xE8, 0xCB)),
							new ColorPickerColor("7671W Boc Choy", Color.FromArgb(0xDD, 0xD9, 0xB1)),
							new ColorPickerColor("7672W Honeydew", Color.FromArgb(0xD3, 0xCE, 0x9F)),
							new ColorPickerColor("7673M Pekoe Tea", Color.FromArgb(0xC1, 0xBB, 0x82)),
							new ColorPickerColor("7674D Dried Basil", Color.FromArgb(0xA9, 0xA1, 0x60)),
							new ColorPickerColor("7675A Romaine", Color.FromArgb(0x98, 0x91, 0x51)),
							new ColorPickerColor("7676N Jungle Moss", Color.FromArgb(0x7E, 0x79, 0x3D)),
							new ColorPickerColor("7680W Evasive White", Color.FromArgb(0xE8, 0xE2, 0xC3)),
							new ColorPickerColor("7681W Dewy Twig", Color.FromArgb(0xE1, 0xDA, 0xB4)),
							new ColorPickerColor("7682W Lime Ice", Color.FromArgb(0xD9, 0xD0, 0xA2)),
							new ColorPickerColor("7683M Sweet Olive", Color.FromArgb(0xCD, 0xC2, 0x86)),
							new ColorPickerColor("7684D Spring Sprout", Color.FromArgb(0xBD, 0xAE, 0x5F)),
							new ColorPickerColor("7685A Liverpool", Color.FromArgb(0xA8, 0x9B, 0x49)),
							new ColorPickerColor("7686N Cummer Olive", Color.FromArgb(0x8F, 0x85, 0x3A)),
							new ColorPickerColor("7690W Liming White", Color.FromArgb(0xF4, 0xEC, 0xD0)),
							new ColorPickerColor("7691W Flora Ivory", Color.FromArgb(0xED, 0xE3, 0xB9)),
							new ColorPickerColor("7692W Escarole", Color.FromArgb(0xEA, 0xDD, 0xA9)),
							new ColorPickerColor("7693M Emeraude Gold", Color.FromArgb(0xE2, 0xD0, 0x8C)),
							new ColorPickerColor("7694D Fife Gold", Color.FromArgb(0xCD, 0xB5, 0x5D)),
							new ColorPickerColor("7695A Antebellum", Color.FromArgb(0xBD, 0xA3, 0x47)),
							new ColorPickerColor("7696N Kiwi Fruit", Color.FromArgb(0xA9, 0x90, 0x38)),
							new ColorPickerColor("7700W Downy Duckling", Color.FromArgb(0xF7, 0xEB, 0xC2)),
							new ColorPickerColor("7701W Ivory Paper", Color.FromArgb(0xF7, 0xE7, 0xB4)),
							new ColorPickerColor("7702W Yellow Sash", Color.FromArgb(0xF2, 0xE0, 0xA4)),
							new ColorPickerColor("7703M Cool Yellow", Color.FromArgb(0xEF, 0xD7, 0x84)),
							new ColorPickerColor("7704D Cat;s Eye", Color.FromArgb(0xE3, 0xC2, 0x56)),
							new ColorPickerColor("7705A Pyrite", Color.FromArgb(0xD8, 0xB4, 0x3B)),
							new ColorPickerColor("7706N Green Citrus", Color.FromArgb(0xBB, 0x96, 0x27)),
							new ColorPickerColor("7710W Star Magnolia", Color.FromArgb(0xF6, 0xEA, 0xCB)),
							new ColorPickerColor("7711W Angelica Blossom", Color.FromArgb(0xF4, 0xE1, 0xB4)),
							new ColorPickerColor("7712W Dijon", Color.FromArgb(0xF2, 0xDA, 0xA2)),
							new ColorPickerColor("7713M Mustard Bloom", Color.FromArgb(0xF0, 0xCD, 0x82)),
							new ColorPickerColor("7714D Honey Gold", Color.FromArgb(0xEB, 0xC2, 0x6C)),
							new ColorPickerColor("7715D Golden Rotunda", Color.FromArgb(0xE2, 0xB3, 0x57)),
							new ColorPickerColor("7716N Evergold", Color.FromArgb(0xD0, 0x99, 0x36)),
							new ColorPickerColor("7720W Delicate Cream", Color.FromArgb(0xFA, 0xED, 0xD0)),
							new ColorPickerColor("7721W Light Honey", Color.FromArgb(0xFA, 0xE3, 0xB4)),
							new ColorPickerColor("7722W Roman Chamomile", Color.FromArgb(0xF9, 0xDD, 0xA2)),
							new ColorPickerColor("7723M Daisetta", Color.FromArgb(0xF5, 0xCE, 0x83)),
							new ColorPickerColor("7724D Golden Horn", Color.FromArgb(0xEB, 0xB5, 0x58)),
							new ColorPickerColor("7725A Cortez Gold", Color.FromArgb(0xD0, 0x94, 0x32)),
							new ColorPickerColor("7726N Gold Rush", Color.FromArgb(0xD1, 0x95, 0x34)),
							new ColorPickerColor("7730W Corinthian", Color.FromArgb(0xF5, 0xEB, 0xD4)),
							new ColorPickerColor("7731W Light Topaz", Color.FromArgb(0xF9, 0xDC, 0xAF)),
							new ColorPickerColor("7732W Butter Pecan", Color.FromArgb(0xF7, 0xD1, 0x9B)),
							new ColorPickerColor("7733M Sahara Gold", Color.FromArgb(0xF5, 0xCB, 0x91)),
							new ColorPickerColor("7734M Cane", Color.FromArgb(0xEF, 0xBE, 0x7B)),
							new ColorPickerColor("7735D Topaz", Color.FromArgb(0xE0, 0xA3, 0x54)),
							new ColorPickerColor("7736N Butterbrickle", Color.FromArgb(0xC8, 0x88, 0x37)),
							new ColorPickerColor("7740W Snow Gum", Color.FromArgb(0xF6, 0xED, 0xD9)),
							new ColorPickerColor("7741W Silvered Pecan", Color.FromArgb(0xF5, 0xD9, 0xB2)),
							new ColorPickerColor("7742W Light Toast", Color.FromArgb(0xF2, 0xCF, 0x9F)),
							new ColorPickerColor("7743M Cupola Yellow", Color.FromArgb(0xF0, 0xC9, 0x95)),
							new ColorPickerColor("7744M September Leaf", Color.FromArgb(0xE8, 0xBB, 0x82)),
							new ColorPickerColor("7745D Spanish Gold", Color.FromArgb(0xD6, 0x9F, 0x5A)),
							new ColorPickerColor("7746N Golden Glove", Color.FromArgb(0xB6, 0x7D, 0x3B)),
							new ColorPickerColor("7750W Beach Basket", Color.FromArgb(0xF4, 0xE6, 0xCE)),
							new ColorPickerColor("7751W Beach Grass", Color.FromArgb(0xEA, 0xD2, 0xB4)),
							new ColorPickerColor("7752W Oakbuff", Color.FromArgb(0xE2, 0xC4, 0xA1)),
							new ColorPickerColor("7753M Harvest Tan", Color.FromArgb(0xDF, 0xBD, 0x98)),
							new ColorPickerColor("7754M Safari Tan", Color.FromArgb(0xD5, 0xAF, 0x84)),
							new ColorPickerColor("7755D Tannery", Color.FromArgb(0xC2, 0x95, 0x66)),
							new ColorPickerColor("7756N Cockatoo Gold", Color.FromArgb(0xA0, 0x72, 0x46)),
							new ColorPickerColor("7760W Weaverbird", Color.FromArgb(0xF1, 0xE9, 0xD7)),
							new ColorPickerColor("7761W Nutmeg Dust", Color.FromArgb(0xED, 0xD7, 0xBB)),
							new ColorPickerColor("7762W Western Sand", Color.FromArgb(0xEE, 0xCE, 0xAB)),
							new ColorPickerColor("7763M Highland Buff", Color.FromArgb(0xE6, 0xBB, 0x8F)),
							new ColorPickerColor("7764D Coffee Cream", Color.FromArgb(0xD6, 0xA4, 0x72)),
							new ColorPickerColor("7765D Terracotta Tile", Color.FromArgb(0xC6, 0x8E, 0x5A)),
							new ColorPickerColor("7766A Rich Amber", Color.FromArgb(0xA8, 0x6D, 0x3D)),
							new ColorPickerColor("7770W Frosty Toffee", Color.FromArgb(0xF2, 0xE1, 0xCC)),
							new ColorPickerColor("7771W Ceramic Beige", Color.FromArgb(0xF6, 0xDB, 0xC1)),
							new ColorPickerColor("7772W Pale Adobe", Color.FromArgb(0xF2, 0xCF, 0xB0)),
							new ColorPickerColor("7773M Desert Sunset", Color.FromArgb(0xE9, 0xB9, 0x92)),
							new ColorPickerColor("7774D Desert Vista", Color.FromArgb(0xD6, 0x97, 0x65)),
							new ColorPickerColor("7775D Canyon Wall", Color.FromArgb(0xC9, 0x88, 0x56)),
							new ColorPickerColor("7776N Galena Gold", Color.FromArgb(0xB4, 0x72, 0x40)),
							new ColorPickerColor("7780W Blushed Cotton", Color.FromArgb(0xF6, 0xE6, 0xD4)),
							new ColorPickerColor("7781W Enchilada", Color.FromArgb(0xF1, 0xD9, 0xC0)),
							new ColorPickerColor("7782W Peach Beige", Color.FromArgb(0xEF, 0xCE, 0xB1)),
							new ColorPickerColor("7783M Toasted Tan", Color.FromArgb(0xDE, 0xB3, 0x92)),
							new ColorPickerColor("7784M Nullabor Sand", Color.FromArgb(0xD3, 0xA3, 0x7E)),
							new ColorPickerColor("7785D Heathcote", Color.FromArgb(0xBB, 0x82, 0x5C)),
							new ColorPickerColor("7786N Contemporary Maple", Color.FromArgb(0xA3, 0x68, 0x44)),
							new ColorPickerColor("7790W Opalescence", Color.FromArgb(0xF5, 0xE4, 0xCF)),
							new ColorPickerColor("7791W Ivory Peach", Color.FromArgb(0xF6, 0xDA, 0xC3)),
							new ColorPickerColor("7792W Dusky Coral", Color.FromArgb(0xF6, 0xD2, 0xBA)),
							new ColorPickerColor("7793M Taffy Crunch", Color.FromArgb(0xEC, 0xB0, 0x8A)),
							new ColorPickerColor("7794D Appealing Apricot", Color.FromArgb(0xDA, 0x8E, 0x61)),
							new ColorPickerColor("7795A Southwest Amber", Color.FromArgb(0xCF, 0x83, 0x55)),
							new ColorPickerColor("7796N Mythic Fire", Color.FromArgb(0xB3, 0x67, 0x3F)),
							new ColorPickerColor("7800W Ivory Powder", Color.FromArgb(0xEE, 0xE0, 0xCF)),
							new ColorPickerColor("7801W Strata Beige", Color.FromArgb(0xF1, 0xDB, 0xC6)),
							new ColorPickerColor("7802W Westerly Tan", Color.FromArgb(0xEE, 0xD1, 0xBA)),
							new ColorPickerColor("7803M Mission Beige", Color.FromArgb(0xE0, 0xB3, 0x94)),
							new ColorPickerColor("7804D Clay Urn", Color.FromArgb(0xC9, 0x90, 0x6B)),
							new ColorPickerColor("7805A Copper Gold", Color.FromArgb(0xB2, 0x73, 0x50)),
							new ColorPickerColor("7806N Roaring Hearth", Color.FromArgb(0x9C, 0x5E, 0x3F)),
							new ColorPickerColor("7810W Taffy Tint", Color.FromArgb(0xF5, 0xE3, 0xD2)),
							new ColorPickerColor("7811W Golden Apricot", Color.FromArgb(0xF0, 0xD1, 0xBD)),
							new ColorPickerColor("7812W Confection Tan", Color.FromArgb(0xE6, 0xC1, 0xAA)),
							new ColorPickerColor("7813M Apple Taffy", Color.FromArgb(0xD9, 0xA6, 0x8A)),
							new ColorPickerColor("7814D Honeysuckle Coral", Color.FromArgb(0xC5, 0x89, 0x6C)),
							new ColorPickerColor("7815A Paprika", Color.FromArgb(0xB8, 0x79, 0x5C)),
							new ColorPickerColor("7816N Tasmanian Myrtle", Color.FromArgb(0xA6, 0x67, 0x4C)),
							new ColorPickerColor("7820W Sheer Natural", Color.FromArgb(0xF1, 0xDD, 0xCC)),
							new ColorPickerColor("7821W Bush Beige", Color.FromArgb(0xEB, 0xCC, 0xB8)),
							new ColorPickerColor("7822W Tyrian Tan", Color.FromArgb(0xE4, 0xBE, 0xA8)),
							new ColorPickerColor("7823M Mayan Sun", Color.FromArgb(0xDA, 0xA8, 0x91)),
							new ColorPickerColor("7824M Desert Edge", Color.FromArgb(0xC8, 0x8F, 0x76)),
							new ColorPickerColor("7825D Tawny Copper", Color.FromArgb(0xB7, 0x77, 0x5E)),
							new ColorPickerColor("7826A Ayers Rock", Color.FromArgb(0x9C, 0x5E, 0x48)),
							new ColorPickerColor("7830W Apple Blush", Color.FromArgb(0xED, 0xDA, 0xD0)),
							new ColorPickerColor("7831W French Floral", Color.FromArgb(0xF5, 0xDA, 0xCE)),
							new ColorPickerColor("7832W Pale Rose", Color.FromArgb(0xF4, 0xD0, 0xC4)),
							new ColorPickerColor("7833M Matador Pink", Color.FromArgb(0xEA, 0xAE, 0x9D)),
							new ColorPickerColor("7834D Breccia", Color.FromArgb(0xE1, 0x96, 0x81)),
							new ColorPickerColor("7835D Reef Rust", Color.FromArgb(0xD5, 0x83, 0x6D)),
							new ColorPickerColor("7836N Copper Canyon", Color.FromArgb(0xA9, 0x5A, 0x44)),
							new ColorPickerColor("7840W Pink Candle", Color.FromArgb(0xF6, 0xE1, 0xD5)),
							new ColorPickerColor("7841W Candied Apricot", Color.FromArgb(0xF5, 0xDB, 0xD2)),
							new ColorPickerColor("7842W Misted Coral", Color.FromArgb(0xF3, 0xCD, 0xC3)),
							new ColorPickerColor("7843M Terra Cotta Tint", Color.FromArgb(0xEA, 0xAB, 0xA1)),
							new ColorPickerColor("7844D Rockies Red", Color.FromArgb(0xD7, 0x86, 0x77)),
							new ColorPickerColor("7845D Starburst", Color.FromArgb(0xC9, 0x73, 0x64)),
							new ColorPickerColor("7846N Eldorado", Color.FromArgb(0xA5, 0x4F, 0x40)),
							new ColorPickerColor("7850W Cosmetic Pink", Color.FromArgb(0xF3, 0xE0, 0xD6)),
							new ColorPickerColor("7851W Rose Damask", Color.FromArgb(0xF2, 0xD7, 0xCD)),
							new ColorPickerColor("7852W Dutchess Dawn", Color.FromArgb(0xEB, 0xC3, 0xB9)),
							new ColorPickerColor("7853W Muted Mesa", Color.FromArgb(0xDC, 0xA3, 0x98)),
							new ColorPickerColor("7854M Rose Pueblo", Color.FromArgb(0xBC, 0x74, 0x68)),
							new ColorPickerColor("7855D Cajun Spice", Color.FromArgb(0xAF, 0x63, 0x57)),
							new ColorPickerColor("7856A Tomahawk", Color.FromArgb(0x94, 0x4C, 0x40)),
							new ColorPickerColor("7860W Pink Shine", Color.FromArgb(0xF3, 0xE4, 0xDA)),
							new ColorPickerColor("7861W Oregon Rose", Color.FromArgb(0xEE, 0xD6, 0xD0)),
							new ColorPickerColor("7862W Wild Carnation", Color.FromArgb(0xE8, 0xC7, 0xC4)),
							new ColorPickerColor("7863M Opening Bud", Color.FromArgb(0xE3, 0xA1, 0x9C)),
							new ColorPickerColor("7864 Coral Berry", Color.FromArgb(0xCF, 0x7E, 0x75)),
							new ColorPickerColor("7865A Field Poppy", Color.FromArgb(0xB9, 0x68, 0x60)),
							new ColorPickerColor("7866N Baked Apple", Color.FromArgb(0x9B, 0x4A, 0x43)),
							new ColorPickerColor("7870W Pinafore", Color.FromArgb(0xF5, 0xDF, 0xD8)),
							new ColorPickerColor("7871W Pink Light", Color.FromArgb(0xEF, 0xCB, 0xC9)),
							new ColorPickerColor("7872W Berry Mousse", Color.FromArgb(0xE9, 0xBC, 0xBC)),
							new ColorPickerColor("7873M Cashmere Rose", Color.FromArgb(0xDE, 0x9B, 0x9D)),
							new ColorPickerColor("7874D Canterbury Rose", Color.FromArgb(0xCF, 0x7A, 0x79)),
							new ColorPickerColor("7875A Polished Coral", Color.FromArgb(0xBC, 0x67, 0x63)),
							new ColorPickerColor("7876N Briar Berry", Color.FromArgb(0x9B, 0x4F, 0x47)),
							new ColorPickerColor("7880W Softly Pink", Color.FromArgb(0xEF, 0xD4, 0xCF)),
							new ColorPickerColor("7881W Dawn Glow", Color.FromArgb(0xF3, 0xCC, 0xCA)),
							new ColorPickerColor("7882W Rouge", Color.FromArgb(0xEE, 0xBB, 0xBD)),
							new ColorPickerColor("7883M Charming Pink", Color.FromArgb(0xE6, 0x9E, 0xA3)),
							new ColorPickerColor("7884D Rambler Rose", Color.FromArgb(0xD7, 0x80, 0x84)),
							new ColorPickerColor("7885A Crimson Barberry", Color.FromArgb(0xBA, 0x5C, 0x5D)),
							new ColorPickerColor("7886N Rio Red", Color.FromArgb(0xA4, 0x4A, 0x46)),
							new ColorPickerColor("7890W Pink Clarity", Color.FromArgb(0xF2, 0xDD, 0xDA)),
							new ColorPickerColor("7891W Ginger Pink", Color.FromArgb(0xE7, 0xC0, 0xBF)),
							new ColorPickerColor("7892W Cherokee Rose", Color.FromArgb(0xE1, 0xB1, 0xB1)),
							new ColorPickerColor("7893M Gaiety Rose", Color.FromArgb(0xD2, 0x94, 0x97)),
							new ColorPickerColor("7894D Sun Poppy", Color.FromArgb(0xC5, 0x80, 0x80)),
							new ColorPickerColor("7895A Blazen Pink", Color.FromArgb(0xAA, 0x5F, 0x5F)),
							new ColorPickerColor("7896N Duberry Red", Color.FromArgb(0x8C, 0x4B, 0x48)),
							new ColorPickerColor("7900W Tatiana Pink", Color.FromArgb(0xF2, 0xDE, 0xDA)),
							new ColorPickerColor("7901W Cameo Glass", Color.FromArgb(0xEA, 0xC7, 0xC8)),
							new ColorPickerColor("7902W Rose Memory", Color.FromArgb(0xE3, 0xB9, 0xBC)),
							new ColorPickerColor("7903M Autumn Rose", Color.FromArgb(0xD5, 0x9A, 0xA2)),
							new ColorPickerColor("7904D Rose Shade", Color.FromArgb(0xC8, 0x86, 0x8D)),
							new ColorPickerColor("7905D Chinaberry", Color.FromArgb(0xB0, 0x6B, 0x70)),
							new ColorPickerColor("7906N Remember Rose", Color.FromArgb(0x89, 0x49, 0x4F)),
							new ColorPickerColor("7910W Rose Thought", Color.FromArgb(0xF1, 0xD4, 0xD8)),
							new ColorPickerColor("7911W Sweet Columine", Color.FromArgb(0xF1, 0xC7, 0xD0)),
							new ColorPickerColor("7912W Sweetrose", Color.FromArgb(0xEC, 0xB9, 0xC6)),
							new ColorPickerColor("7913M Cactus Bloom", Color.FromArgb(0xE2, 0x9D, 0xB0)),
							new ColorPickerColor("7914D Pink Gardenia", Color.FromArgb(0xC6, 0x73, 0x88)),
							new ColorPickerColor("7915A Miniature Rose", Color.FromArgb(0xB8, 0x61, 0x72)),
							new ColorPickerColor("7916N Regensy Red", Color.FromArgb(0x8A, 0x40, 0x46)),
							new ColorPickerColor("7920W Swiss Pink", Color.FromArgb(0xEC, 0xCE, 0xD5)),
							new ColorPickerColor("7921W Windflower", Color.FromArgb(0xE5, 0xBF, 0xCB)),
							new ColorPickerColor("7922W Viva La Pink", Color.FromArgb(0xDE, 0xB0, 0xC2)),
							new ColorPickerColor("7923M Madeline", Color.FromArgb(0xD4, 0x95, 0xAB)),
							new ColorPickerColor("7924D Floral Plum", Color.FromArgb(0xB8, 0x73, 0x8C)),
							new ColorPickerColor("7925A Rose Velvet", Color.FromArgb(0xA3, 0x5B, 0x71)),
							new ColorPickerColor("7926N Magenta Royale", Color.FromArgb(0x7D, 0x45, 0x54)),
							new ColorPickerColor("7930W Raspberry Pink", Color.FromArgb(0xF2, 0xDF, 0xDF)),
							new ColorPickerColor("7931W May Blossom", Color.FromArgb(0xE0, 0xC0, 0xCF)),
							new ColorPickerColor("7932W Garden Paradise", Color.FromArgb(0xD9, 0xAF, 0xC3)),
							new ColorPickerColor("7933M Viola Phlox", Color.FromArgb(0xCB, 0x93, 0xB0)),
							new ColorPickerColor("7934D Geisha", Color.FromArgb(0xB5, 0x71, 0x91)),
							new ColorPickerColor("7935A Exotic Plum", Color.FromArgb(0x9A, 0x5A, 0x77)),
							new ColorPickerColor("7936N Bright Bordeaux", Color.FromArgb(0x79, 0x43, 0x56)),
							new ColorPickerColor("7940W Cherish", Color.FromArgb(0xE7, 0xDA, 0xDD)),
							new ColorPickerColor("7941W Romantic Violet", Color.FromArgb(0xDA, 0xC4, 0xD2)),
							new ColorPickerColor("7942W Violet Trace", Color.FromArgb(0xD3, 0xB5, 0xC9)),
							new ColorPickerColor("7943D Violet Aero", Color.FromArgb(0xB4, 0x89, 0xA8)),
							new ColorPickerColor("7944D Rose Amaranth", Color.FromArgb(0x9F, 0x73, 0x94)),
							new ColorPickerColor("7945A Fresh Berry", Color.FromArgb(0x8C, 0x5C, 0x7B)),
							new ColorPickerColor("7946N Blackberry Swirl", Color.FromArgb(0x5B, 0x41, 0x50)),
							new ColorPickerColor("7950W Plum Frost", Color.FromArgb(0xE9, 0xDC, 0xDC)),
							new ColorPickerColor("7951W Ophelia Violet", Color.FromArgb(0xDB, 0xC6, 0xD1)),
							new ColorPickerColor("7952W Captivating Violet", Color.FromArgb(0xD0, 0xB5, 0xC6)),
							new ColorPickerColor("7953D Plum Sparkle", Color.FromArgb(0xAB, 0x8B, 0xA6)),
							new ColorPickerColor("7954D Perfect Plum", Color.FromArgb(0x96, 0x75, 0x92)),
							new ColorPickerColor("7955A Regal Bearing", Color.FromArgb(0x80, 0x60, 0x7C)),
							new ColorPickerColor("7956N Bacchus", Color.FromArgb(0x59, 0x46, 0x57)),
							new ColorPickerColor("7960W Embracing Lilac", Color.FromArgb(0xDA, 0xD9, 0xDB)),
							new ColorPickerColor("7961W Spring Parasol", Color.FromArgb(0xC9, 0xCA, 0xD0)),
							new ColorPickerColor("7962W Venice Lavender", Color.FromArgb(0xBD, 0xBE, 0xC7)),
							new ColorPickerColor("7963M Slow Dancer", Color.FromArgb(0xA6, 0xA6, 0xB7)),
							new ColorPickerColor("7964M Rhinestone", Color.FromArgb(0x8F, 0x8F, 0xA7)),
							new ColorPickerColor("7965D Prestige Purple", Color.FromArgb(0x6F, 0x6F, 0x8A)),
							new ColorPickerColor("7966A Valhalla", Color.FromArgb(0x55, 0x55, 0x6B)),
							new ColorPickerColor("7970W Sheer Lilac", Color.FromArgb(0xDD, 0xDE, 0xDF)),
							new ColorPickerColor("7971W Canterbury Bell", Color.FromArgb(0xD0, 0xD1, 0xDA)),
							new ColorPickerColor("7972W Crushed Heather", Color.FromArgb(0xC2, 0xC4, 0xD3)),
							new ColorPickerColor("7973M Pretty Purple", Color.FromArgb(0xA8, 0xAD, 0xC6)),
							new ColorPickerColor("7974M Purple Rain", Color.FromArgb(0x90, 0x96, 0xB8)),
							new ColorPickerColor("7975D Stargazing Night", Color.FromArgb(0x6E, 0x74, 0x98)),
							new ColorPickerColor("7976N Planetary Plume", Color.FromArgb(0x4C, 0x50, 0x6F)),
							new ColorPickerColor("7980W Iced Blue", Color.FromArgb(0xD3, 0xDC, 0xE2)),
							new ColorPickerColor("7981W Glacier Wall", Color.FromArgb(0xC6, 0xD2, 0xDE)),
							new ColorPickerColor("7982W Blue Sash", Color.FromArgb(0xB5, 0xC4, 0xD6)),
							new ColorPickerColor("7983M bracing Blue", Color.FromArgb(0xA4, 0xB4, 0xCB)),
							new ColorPickerColor("7984M Tapestry Blue", Color.FromArgb(0x89, 0x9D, 0xBB)),
							new ColorPickerColor("7985D Blue Border", Color.FromArgb(0x64, 0x7B, 0x9F)),
							new ColorPickerColor("7986N Indigo Night", Color.FromArgb(0x43, 0x50, 0x6C)),
							new ColorPickerColor("7990W Pearl Violet", Color.FromArgb(0xD8, 0xE0, 0xE1)),
							new ColorPickerColor("7991W Basin Blue", Color.FromArgb(0xC2, 0xD0, 0xDA)),
							new ColorPickerColor("7992W Studio Blue", Color.FromArgb(0xB3, 0xC6, 0xD8)),
							new ColorPickerColor("7993M Perfect Blue", Color.FromArgb(0x9C, 0xB5, 0xCE)),
							new ColorPickerColor("7994M Ceremonial Blue", Color.FromArgb(0x87, 0xA3, 0xC3)),
							new ColorPickerColor("7995D Valor Blue", Color.FromArgb(0x5E, 0x7D, 0xA0)),
							new ColorPickerColor("7996N Twilight Zone", Color.FromArgb(0x3C, 0x51, 0x6C)),
							new ColorPickerColor("8000W Airland Blue", Color.FromArgb(0xD3, 0xDF, 0xE2)),
							new ColorPickerColor("8001W Nimbus Blue", Color.FromArgb(0xC6, 0xD6, 0xDF)),
							new ColorPickerColor("8002W Floss Flower", Color.FromArgb(0xB1, 0xC8, 0xD7)),
							new ColorPickerColor("8003M Blue Wainscot", Color.FromArgb(0x98, 0xB7, 0xCD)),
							new ColorPickerColor("8004M Blue Room", Color.FromArgb(0x7B, 0xA1, 0xC1)),
							new ColorPickerColor("8005D Wilshire Blue", Color.FromArgb(0x55, 0x7F, 0xA3)),
							new ColorPickerColor("8006N Academy Blue", Color.FromArgb(0x3A, 0x5B, 0x7A)),
							new ColorPickerColor("8010W Blue Edge", Color.FromArgb(0xC9, 0xDC, 0xE0)),
							new ColorPickerColor("8011W Corn Flower", Color.FromArgb(0xBA, 0xD3, 0xDB)),
							new ColorPickerColor("8012W Jeffrey Blue", Color.FromArgb(0xAA, 0xC8, 0xD6)),
							new ColorPickerColor("8013W Blue Hat", Color.FromArgb(0x8B, 0xB3, 0xCA)),
							new ColorPickerColor("8014M Blue Surf", Color.FromArgb(0x72, 0x9F, 0xBB)),
							new ColorPickerColor("8015D Water Works", Color.FromArgb(0x4B, 0x7D, 0x9C)),
							new ColorPickerColor("8016N Deep Waters", Color.FromArgb(0x37, 0x59, 0x71)),
							new ColorPickerColor("8020W Tender Blue", Color.FromArgb(0xD7, 0xE1, 0xDF)),
							new ColorPickerColor("8021W Blue Waltz", Color.FromArgb(0xBF, 0xD9, 0xE0)),
							new ColorPickerColor("8022W Cool Starlight", Color.FromArgb(0xAA, 0xCE, 0xDA)),
							new ColorPickerColor("8023M Clear Sky", Color.FromArgb(0x8C, 0xBB, 0xCD)),
							new ColorPickerColor("8024M Capri Grotto", Color.FromArgb(0x68, 0xA2, 0xBC)),
							new ColorPickerColor("8025D Florentine Blue", Color.FromArgb(0x3D, 0x7C, 0x98)),
							new ColorPickerColor("8026N Italian Tile", Color.FromArgb(0x2A, 0x58, 0x71)),
							new ColorPickerColor("8030W Song Blue", Color.FromArgb(0xCD, 0xE2, 0xDF)),
							new ColorPickerColor("8031W Bungle Blue", Color.FromArgb(0xBB, 0xD9, 0xDC)),
							new ColorPickerColor("8032W High Flyer", Color.FromArgb(0xA5, 0xCE, 0xD7)),
							new ColorPickerColor("8033M Sky Dive", Color.FromArgb(0x88, 0xBE, 0xCD)),
							new ColorPickerColor("8034M Pacific Ocean", Color.FromArgb(0x6E, 0xAC, 0xC0)),
							new ColorPickerColor("8035D Seaquest", Color.FromArgb(0x3B, 0x86, 0x9E)),
							new ColorPickerColor("8036N Undersea Blue", Color.FromArgb(0x29, 0x59, 0x6C)),
							new ColorPickerColor("8040W Ice Cavern", Color.FromArgb(0xD1, 0xE6, 0xE1)),
							new ColorPickerColor("8041W Polar Blue", Color.FromArgb(0xBD, 0xDB, 0xDC)),
							new ColorPickerColor("8042W Sweeping Blue", Color.FromArgb(0xA4, 0xCF, 0xD4)),
							new ColorPickerColor("8043M Beaux Blue", Color.FromArgb(0x8E, 0xBA, 0xC3)),
							new ColorPickerColor("8044M Dutch Aqua", Color.FromArgb(0x76, 0xAA, 0xB6)),
							new ColorPickerColor("8045D Ocean Depths", Color.FromArgb(0x48, 0x83, 0x91)),
							new ColorPickerColor("8046N Mutiny", Color.FromArgb(0x2F, 0x58, 0x63)),
							new ColorPickerColor("8050W Zenith Blue", Color.FromArgb(0xD3, 0xE5, 0xDE)),
							new ColorPickerColor("8051W Seadrifter", Color.FromArgb(0xC6, 0xDF, 0xDB)),
							new ColorPickerColor("8052W West Keys", Color.FromArgb(0xAB, 0xD0, 0xCF)),
							new ColorPickerColor("8053M Sea View", Color.FromArgb(0x8D, 0xBC, 0xBE)),
							new ColorPickerColor("8054M Ocean Grove", Color.FromArgb(0x74, 0xAA, 0xAE)),
							new ColorPickerColor("8055D Poseidon", Color.FromArgb(0x49, 0x82, 0x87)),
							new ColorPickerColor("8056N Lost Atlantis", Color.FromArgb(0x3A, 0x5A, 0x5F)),
							new ColorPickerColor("8060W Retreat", Color.FromArgb(0xCF, 0xE5, 0xE0)),
							new ColorPickerColor("8061W Noonday Tide", Color.FromArgb(0xC6, 0xDF, 0xD9)),
							new ColorPickerColor("8062W Sea Silence", Color.FromArgb(0xAE, 0xD0, 0xCE)),
							new ColorPickerColor("8063M Wayward Wind", Color.FromArgb(0x8F, 0xBB, 0xB9)),
							new ColorPickerColor("8064M Cape Barran", Color.FromArgb(0x79, 0xA9, 0xA8)),
							new ColorPickerColor("8065D Aqualogic", Color.FromArgb(0x4E, 0x7F, 0x80)),
							new ColorPickerColor("8066A Abyss", Color.FromArgb(0x3D, 0x5E, 0x5F)),
							new ColorPickerColor("8070W Bermuda Shoal", Color.FromArgb(0xD3, 0xE8, 0xDE)),
							new ColorPickerColor("8071W Cool Aqua", Color.FromArgb(0xC1, 0xDE, 0xD6)),
							new ColorPickerColor("8072W Scuba", Color.FromArgb(0xAD, 0xD3, 0xCC)),
							new ColorPickerColor("8073M Seabeck", Color.FromArgb(0x8E, 0xC1, 0xBB)),
							new ColorPickerColor("8074M Sea Meadow", Color.FromArgb(0x77, 0xB0, 0xAA)),
							new ColorPickerColor("8075D Aqua Sea", Color.FromArgb(0x4C, 0x85, 0x81)),
							new ColorPickerColor("8076A Deep Teal", Color.FromArgb(0x3A, 0x6A, 0x66)),
							new ColorPickerColor("8080W Dissolving Blue", Color.FromArgb(0xDD, 0xEB, 0xDE)),
							new ColorPickerColor("8081W Blue Bauble", Color.FromArgb(0xC8, 0xDB, 0xD1)),
							new ColorPickerColor("8082W Alaskan Jade", Color.FromArgb(0xB6, 0xD3, 0xC9)),
							new ColorPickerColor("8083M Blue Ridge", Color.FromArgb(0x9B, 0xBE, 0xB3)),
							new ColorPickerColor("8084M Kentucky Grass", Color.FromArgb(0x85, 0xAC, 0xA1)),
							new ColorPickerColor("8085D India Green", Color.FromArgb(0x5F, 0x86, 0x79)),
							new ColorPickerColor("8086A Green Wharf", Color.FromArgb(0x4D, 0x69, 0x60)),
							new ColorPickerColor("8090W Winter Whisper", Color.FromArgb(0xE3, 0xEB, 0xDE)),
							new ColorPickerColor("8091W Drifting Green", Color.FromArgb(0xC7, 0xD8, 0xCD)),
							new ColorPickerColor("8092W Cool Comfort", Color.FromArgb(0xB9, 0xCE, 0xC2)),
							new ColorPickerColor("8093W Teal Suede", Color.FromArgb(0xA8, 0xC1, 0xB4)),
							new ColorPickerColor("8094M Himalaya", Color.FromArgb(0x91, 0xAD, 0x9E)),
							new ColorPickerColor("8095D Cool Forest", Color.FromArgb(0x6F, 0x89, 0x7B)),
							new ColorPickerColor("8096N Bering Green", Color.FromArgb(0x50, 0x65, 0x59)),
							new ColorPickerColor("8100W Micro Mint", Color.FromArgb(0xDB, 0xE8, 0xDD)),
							new ColorPickerColor("8101W Green Sprinkle", Color.FromArgb(0xC7, 0xD9, 0xCE)),
							new ColorPickerColor("8102W Green Sweeps", Color.FromArgb(0xBA, 0xD2, 0xC5)),
							new ColorPickerColor("8103W Sweet Grass", Color.FromArgb(0xA5, 0xC0, 0xB4)),
							new ColorPickerColor("8104M Green Plaza", Color.FromArgb(0x8F, 0xAE, 0xA0)),
							new ColorPickerColor("8105D Acanthus Leaf", Color.FromArgb(0x6C, 0x8C, 0x7D)),
							new ColorPickerColor("8106N Caulfield Green", Color.FromArgb(0x48, 0x60, 0x54)),
							new ColorPickerColor("8110W Gentle Rain", Color.FromArgb(0xD8, 0xE2, 0xD4)),
							new ColorPickerColor("8111W Shy Green", Color.FromArgb(0xC3, 0xDB, 0xCB)),
							new ColorPickerColor("8112W Green Mist", Color.FromArgb(0xB5, 0xD3, 0xC1)),
							new ColorPickerColor("8113M Fragrant Spruce", Color.FromArgb(0x9A, 0xBF, 0xAA)),
							new ColorPickerColor("8114M Vineleaf", Color.FromArgb(0x84, 0xAD, 0x96)),
							new ColorPickerColor("8115D Ivy Wreath", Color.FromArgb(0x62, 0x8C, 0x73)),
							new ColorPickerColor("8116N Casino Green", Color.FromArgb(0x48, 0x63, 0x4D)),
							new ColorPickerColor("8120W Frosted Pine", Color.FromArgb(0xDF, 0xE9, 0xD6)),
							new ColorPickerColor("8121W Garden Herb", Color.FromArgb(0xC5, 0xD7, 0xC2)),
							new ColorPickerColor("8122W Light Jade", Color.FromArgb(0xC0, 0xD6, 0xBE)),
							new ColorPickerColor("8123W Chapparel Pea", Color.FromArgb(0xA4, 0xC2, 0xA3)),
							new ColorPickerColor("8124M Green Terrace", Color.FromArgb(0x91, 0xB0, 0x8F)),
							new ColorPickerColor("8125D Water Hawthorne", Color.FromArgb(0x6F, 0x8E, 0x6B)),
							new ColorPickerColor("8126N Racing Green", Color.FromArgb(0x4D, 0x60, 0x45)),
							new ColorPickerColor("8130W Morning Dew", Color.FromArgb(0xE2, 0xE4, 0xD0)),
							new ColorPickerColor("8131W Night Hush", Color.FromArgb(0xD1, 0xD7, 0xC1)),
							new ColorPickerColor("8132W Lunar Green", Color.FromArgb(0xC0, 0xCA, 0xB1)),
							new ColorPickerColor("8133W Green Knoll", Color.FromArgb(0xB2, 0xBF, 0xA2)),
							new ColorPickerColor("8134M Glenhaven", Color.FromArgb(0x9E, 0xAA, 0x89)),
							new ColorPickerColor("8135D Hollyhock", Color.FromArgb(0x80, 0x8D, 0x6A)),
							new ColorPickerColor("8136N Shaded Moss", Color.FromArgb(0x59, 0x62, 0x45)),
							new ColorPickerColor("8140W Misted Pollen", Color.FromArgb(0xE8, 0xE9, 0xD4)),
							new ColorPickerColor("8141W Aviary", Color.FromArgb(0xD5, 0xD8, 0xBC)),
							new ColorPickerColor("8142W Bermuda Grass", Color.FromArgb(0xC9, 0xCD, 0xAE)),
							new ColorPickerColor("8143M Grazing Field", Color.FromArgb(0xC0, 0xC3, 0x9D)),
							new ColorPickerColor("8144M Revere Green", Color.FromArgb(0xAF, 0xB3, 0x8A)),
							new ColorPickerColor("8145D Green Acre", Color.FromArgb(0x8D, 0x91, 0x65)),
							new ColorPickerColor("8146N Green Gables", Color.FromArgb(0x61, 0x65, 0x41)),
							new ColorPickerColor("8150W Scrimshaw", Color.FromArgb(0xE3, 0xE2, 0xCB)),
							new ColorPickerColor("8151W Cut Hay", Color.FromArgb(0xDC, 0xDA, 0xBF)),
							new ColorPickerColor("8152W Rock Cress", Color.FromArgb(0xD2, 0xCF, 0xB0)),
							new ColorPickerColor("8153M Hiking Path", Color.FromArgb(0xC5, 0xC1, 0x9E)),
							new ColorPickerColor("8154M Marshland", Color.FromArgb(0xB8, 0xB3, 0x8B)),
							new ColorPickerColor("8155D Ozark Green", Color.FromArgb(0x94, 0x8C, 0x60)),
							new ColorPickerColor("8156N Florida Palm", Color.FromArgb(0x69, 0x64, 0x41)),
							new ColorPickerColor("8160W White Leek", Color.FromArgb(0xDE, 0xDB, 0xC4)),
							new ColorPickerColor("8161W Sulphur Springs", Color.FromArgb(0xD8, 0xD3, 0xB9)),
							new ColorPickerColor("8162W Omar", Color.FromArgb(0xD0, 0xCC, 0xAF)),
							new ColorPickerColor("8163W Olive Branch", Color.FromArgb(0xC3, 0xBD, 0x9B)),
							new ColorPickerColor("8164M Gumleaf", Color.FromArgb(0xAE, 0xA6, 0x83)),
							new ColorPickerColor("8165D Antique Pine", Color.FromArgb(0x92, 0x88, 0x63)),
							new ColorPickerColor("8166N Hawthorne Valley", Color.FromArgb(0x66, 0x60, 0x46)),
							new ColorPickerColor("8170W Owl Wing", Color.FromArgb(0xE3, 0xE0, 0xCF)),
							new ColorPickerColor("8171W Olive Fog", Color.FromArgb(0xD1, 0xCB, 0xB8)),
							new ColorPickerColor("8172W Shire Fields", Color.FromArgb(0xCA, 0xC3, 0xAE)),
							new ColorPickerColor("8173M Herbivore", Color.FromArgb(0xBD, 0xB5, 0x9C)),
							new ColorPickerColor("8174M Fenland", Color.FromArgb(0xAE, 0xA4, 0x89)),
							new ColorPickerColor("8175D Wild Grasses", Color.FromArgb(0x89, 0x7F, 0x64)),
							new ColorPickerColor("8176N Pinon Green", Color.FromArgb(0x5E, 0x56, 0x43)),
							new ColorPickerColor("8180W Almond White", Color.FromArgb(0xE5, 0xDF, 0xCB)),
							new ColorPickerColor("8181W Maison Blanche", Color.FromArgb(0xD4, 0xCB, 0xB5)),
							new ColorPickerColor("8182W Candlebark", Color.FromArgb(0xCE, 0xC3, 0xAB)),
							new ColorPickerColor("8183M Saltbush", Color.FromArgb(0xC5, 0xB8, 0x9D)),
							new ColorPickerColor("8184M Moselle", Color.FromArgb(0xB4, 0xA5, 0x88)),
							new ColorPickerColor("8185D Ardmore Green", Color.FromArgb(0x91, 0x80, 0x61)),
							new ColorPickerColor("8186N Scrubgum Green", Color.FromArgb(0x68, 0x5A, 0x42)),
							new ColorPickerColor("8190W Impressive Ivory", Color.FromArgb(0xEC, 0xE3, 0xCE)),
							new ColorPickerColor("8191W Madonna Lily", Color.FromArgb(0xDF, 0xD1, 0xB5)),
							new ColorPickerColor("8192W Graceful Gold", Color.FromArgb(0xD5, 0xC4, 0xA3)),
							new ColorPickerColor("8193M Winter Garden", Color.FromArgb(0xD0, 0xBD, 0x99)),
							new ColorPickerColor("8194M Capertree", Color.FromArgb(0xC1, 0xAD, 0x88)),
							new ColorPickerColor("8195D Eddystone", Color.FromArgb(0x9D, 0x88, 0x61)),
							new ColorPickerColor("8196N Wood Sorrel", Color.FromArgb(0x78, 0x65, 0x43)),
							new ColorPickerColor("8200W Bleached Sand", Color.FromArgb(0xE9, 0xDD, 0xC5)),
							new ColorPickerColor("8201W Burma Buff", Color.FromArgb(0xE3, 0xD4, 0xB8)),
							new ColorPickerColor("8202W Coast Pointe", Color.FromArgb(0xDB, 0xC8, 0xA6)),
							new ColorPickerColor("8203M Gobi Beige", Color.FromArgb(0xD6, 0xC1, 0x9E)),
							new ColorPickerColor("8204M Spice Bounty", Color.FromArgb(0xC3, 0xAB, 0x83)),
							new ColorPickerColor("8205D Brass Bucket", Color.FromArgb(0xAA, 0x8E, 0x63)),
							new ColorPickerColor("8206N Tarrytown", Color.FromArgb(0x89, 0x6E, 0x47)),
							new ColorPickerColor("8210W Whispering Birch", Color.FromArgb(0xF2, 0xEB, 0xD7)),
							new ColorPickerColor("8211W Sandy Lane", Color.FromArgb(0xE9, 0xD9, 0xBD)),
							new ColorPickerColor("8212W Millet", Color.FromArgb(0xE0, 0xCC, 0xAD)),
							new ColorPickerColor("8213M Plantation Beige", Color.FromArgb(0xD9, 0xC2, 0x9E)),
							new ColorPickerColor("8214M Tobacco Road", Color.FromArgb(0xCD, 0xAF, 0x85)),
							new ColorPickerColor("8215D Tattersall Brown", Color.FromArgb(0xB2, 0x93, 0x6A)),
							new ColorPickerColor("8216N Dried Tobacco", Color.FromArgb(0x8D, 0x6B, 0x43)),
							new ColorPickerColor("8220W Hayseed", Color.FromArgb(0xE8, 0xDF, 0xCD)),
							new ColorPickerColor("8221W Botany Beige", Color.FromArgb(0xDD, 0xCF, 0xBA)),
							new ColorPickerColor("8222W Desert Fawn", Color.FromArgb(0xD8, 0xC8, 0xB1)),
							new ColorPickerColor("8223M Sienna Sand", Color.FromArgb(0xCE, 0xB7, 0x98)),
							new ColorPickerColor("8224M Balsam Bark", Color.FromArgb(0xBA, 0x9F, 0x7E)),
							new ColorPickerColor("8225D Autumn Wheat", Color.FromArgb(0xA0, 0x82, 0x5F)),
							new ColorPickerColor("8226N October Oak", Color.FromArgb(0x84, 0x67, 0x47)),
							new ColorPickerColor("8230W Sunbaked Sand", Color.FromArgb(0xF1, 0xE4, 0xD2)),
							new ColorPickerColor("8231W Sawyer's Fence", Color.FromArgb(0xDF, 0xCE, 0xB9)),
							new ColorPickerColor("8232W Lulled Beige", Color.FromArgb(0xD5, 0xC3, 0xAA)),
							new ColorPickerColor("8233M Crisp Khaki", Color.FromArgb(0xC7, 0xAF, 0x94)),
							new ColorPickerColor("8234M Daplin", Color.FromArgb(0xBD, 0xA1, 0x83)),
							new ColorPickerColor("8235D Brush Box", Color.FromArgb(0x9D, 0x7E, 0x5F)),
							new ColorPickerColor("8236N Gumnut", Color.FromArgb(0x80, 0x61, 0x45)),
							new ColorPickerColor("8240W Windrift", Color.FromArgb(0xF3, 0xE3, 0xCE)),
							new ColorPickerColor("8241W Tenderfoot", Color.FromArgb(0xE3, 0xCB, 0xB2)),
							new ColorPickerColor("8242W Northern Plains", Color.FromArgb(0xDA, 0xBC, 0x9F)),
							new ColorPickerColor("8243M Amber Waves", Color.FromArgb(0xCF, 0xAB, 0x8B)),
							new ColorPickerColor("8244D Copper Springs", Color.FromArgb(0xBB, 0x90, 0x6D)),
							new ColorPickerColor("8245A Ochre Rust", Color.FromArgb(0x9D, 0x71, 0x51)),
							new ColorPickerColor("8246N Queensland Walnut", Color.FromArgb(0x81, 0x5A, 0x3E)),
							new ColorPickerColor("8250W Ivory Sampler", Color.FromArgb(0xEF, 0xE0, 0xCF)),
							new ColorPickerColor("8251W Plaza Buff", Color.FromArgb(0xE3, 0xCC, 0xB8)),
							new ColorPickerColor("8252W Strata Light", Color.FromArgb(0xD9, 0xBD, 0xA7)),
							new ColorPickerColor("8253M Swept Plains", Color.FromArgb(0xCD, 0xAB, 0x92)),
							new ColorPickerColor("8254M Cinnamon Spice", Color.FromArgb(0xBD, 0x96, 0x7B)),
							new ColorPickerColor("8255D Jamocha", Color.FromArgb(0xA1, 0x77, 0x5C)),
							new ColorPickerColor("8256N Sweet Earth", Color.FromArgb(0x80, 0x58, 0x41)),
							new ColorPickerColor("8260W Pavillion", Color.FromArgb(0xEA, 0xD6, 0xC3)),
							new ColorPickerColor("8261W Traditional Tan", Color.FromArgb(0xE2, 0xCA, 0xB5)),
							new ColorPickerColor("8262W Peanut Shell", Color.FromArgb(0xD9, 0xBD, 0xA4)),
							new ColorPickerColor("8263M Deerfield", Color.FromArgb(0xC8, 0xA1, 0x86)),
							new ColorPickerColor("8264D Tucson Clay", Color.FromArgb(0xBA, 0x8E, 0x70)),
							new ColorPickerColor("8265A Basque Brown", Color.FromArgb(0x9C, 0x6E, 0x52)),
							new ColorPickerColor("8266N Brazillian Brown", Color.FromArgb(0x84, 0x5A, 0x41)),
							new ColorPickerColor("8270W Renoir Bisque", Color.FromArgb(0xE6, 0xD6, 0xC8)),
							new ColorPickerColor("8271W Palacio", Color.FromArgb(0xE3, 0xCB, 0xBB)),
							new ColorPickerColor("8272W La Mesa", Color.FromArgb(0xDB, 0xBC, 0xA9)),
							new ColorPickerColor("8273M Spiced Cognac", Color.FromArgb(0xCD, 0xA5, 0x8F)),
							new ColorPickerColor("8274M Festoon", Color.FromArgb(0xBB, 0x8D, 0x76)),
							new ColorPickerColor("8275D Round-Up", Color.FromArgb(0xA0, 0x6E, 0x57)),
							new ColorPickerColor("8276A Mesquite Canyon", Color.FromArgb(0x90, 0x5E, 0x49)),
							new ColorPickerColor("8280W Sand Motif", Color.FromArgb(0xED, 0xE0, 0xD4)),
							new ColorPickerColor("8281W Pink Mocha", Color.FromArgb(0xE1, 0xCD, 0xBD)),
							new ColorPickerColor("8282W Nutmeat", Color.FromArgb(0xD8, 0xBF, 0xAE)),
							new ColorPickerColor("8283W Indian Painting", Color.FromArgb(0xC2, 0xA2, 0x8F)),
							new ColorPickerColor("8284D Falling Leaf", Color.FromArgb(0xA7, 0x81, 0x6C)),
							new ColorPickerColor("8285A Monk's Hood", Color.FromArgb(0x8B, 0x63, 0x50)),
							new ColorPickerColor("8286N Burntside", Color.FromArgb(0x74, 0x50, 0x40)),
							new ColorPickerColor("8290W Water Chestnut", Color.FromArgb(0xEA, 0xDD, 0xD2)),
							new ColorPickerColor("8291W Mallow", Color.FromArgb(0xE5, 0xD4, 0xC7)),
							new ColorPickerColor("8292W Coral Brown", Color.FromArgb(0xD0, 0xB6, 0xA8)),
							new ColorPickerColor("8293M Clove Dust", Color.FromArgb(0xC3, 0xA3, 0x93)),
							new ColorPickerColor("8294M Red Dust", Color.FromArgb(0xAC, 0x89, 0x79)),
							new ColorPickerColor("8295D Copper Nail", Color.FromArgb(0x8F, 0x6A, 0x5A)),
							new ColorPickerColor("8296N Treasure Chest", Color.FromArgb(0x6C, 0x4A, 0x3C)),
							new ColorPickerColor("8300W Nomadic Ivory", Color.FromArgb(0xE3, 0xD8, 0xCC)),
							new ColorPickerColor("8301W Bedouin Beige", Color.FromArgb(0xDB, 0xCD, 0xC2)),
							new ColorPickerColor("8302W Mineral Mauve", Color.FromArgb(0xCE, 0xBE, 0xB2)),
							new ColorPickerColor("8303W Lapstone", Color.FromArgb(0xBA, 0xA4, 0x98)),
							new ColorPickerColor("8304M Washed Suede", Color.FromArgb(0xA4, 0x8A, 0x7B)),
							new ColorPickerColor("8305D Jericho Brown", Color.FromArgb(0x85, 0x6A, 0x5B)),
							new ColorPickerColor("8306N Alcazar Brown", Color.FromArgb(0x66, 0x50, 0x42)),
							new ColorPickerColor("8310W Pink Reed", Color.FromArgb(0xDE, 0xCF, 0xC4)),
							new ColorPickerColor("8311W Ambient Sound", Color.FromArgb(0xD5, 0xC6, 0xBC)),
							new ColorPickerColor("8312W Misted Beige", Color.FromArgb(0xC9, 0xB7, 0xAD)),
							new ColorPickerColor("8313M Cast Pebble", Color.FromArgb(0xB2, 0x9C, 0x90)),
							new ColorPickerColor("8314M Silvered Bark", Color.FromArgb(0x9F, 0x88, 0x7B)),
							new ColorPickerColor("8315D Ravenwood", Color.FromArgb(0x79, 0x61, 0x56)),
							new ColorPickerColor("8316N Black Burgundy", Color.FromArgb(0x61, 0x4C, 0x43)),
							new ColorPickerColor("8320W Thin Pink", Color.FromArgb(0xEB, 0xDE, 0xD4)),
							new ColorPickerColor("8321W Quiet Alcove", Color.FromArgb(0xDD, 0xC9, 0xBF)),
							new ColorPickerColor("8322W Auburn Sand", Color.FromArgb(0xD2, 0xB9, 0xAF)),
							new ColorPickerColor("8323M Pancho Brown", Color.FromArgb(0xBD, 0x9D, 0x92)),
							new ColorPickerColor("8324M Smooth Brown", Color.FromArgb(0xA5, 0x83, 0x7A)),
							new ColorPickerColor("8325D Timber Tan", Color.FromArgb(0x89, 0x64, 0x5A)),
							new ColorPickerColor("8326N Brick Shadow", Color.FromArgb(0x6C, 0x4B, 0x41)),
							new ColorPickerColor("8330W Sachet Rose", Color.FromArgb(0xE7, 0xD9, 0xCE)),
							new ColorPickerColor("8331W Mauve Blush", Color.FromArgb(0xE2, 0xCA, 0xBD)),
							new ColorPickerColor("8332W Spice Beige", Color.FromArgb(0xDB, 0xBC, 0xAF)),
							new ColorPickerColor("8333M Incense", Color.FromArgb(0xC8, 0xA2, 0x93)),
							new ColorPickerColor("8334M Taupe Rose", Color.FromArgb(0xB6, 0x8D, 0x7F)),
							new ColorPickerColor("8335D Terrawood", Color.FromArgb(0x96, 0x6A, 0x5C)),
							new ColorPickerColor("8336N Bavarian Chalet", Color.FromArgb(0x74, 0x4C, 0x40)),
							new ColorPickerColor("8340W Pale Coral", Color.FromArgb(0xEC, 0xDD, 0xD3)),
							new ColorPickerColor("8341W Prairie Tan", Color.FromArgb(0xE1, 0xC6, 0xBA)),
							new ColorPickerColor("8342W Sanibel Sunset", Color.FromArgb(0xD6, 0xB4, 0xA7)),
							new ColorPickerColor("8343M Condo Coral", Color.FromArgb(0xC1, 0x95, 0x86)),
							new ColorPickerColor("8344M Sane Fe Brown", Color.FromArgb(0xAF, 0x81, 0x73)),
							new ColorPickerColor("8345D Western Cedar", Color.FromArgb(0x8E, 0x63, 0x56)),
							new ColorPickerColor("8346N Taiga", Color.FromArgb(0x72, 0x4A, 0x3D)),
							new ColorPickerColor("8350W Miami Sand", Color.FromArgb(0xEB, 0xD4, 0xC8)),
							new ColorPickerColor("8351W Tan Blush", Color.FromArgb(0xE6, 0xC9, 0xBC)),
							new ColorPickerColor("8352W Beaux Beige", Color.FromArgb(0xDF, 0xBA, 0xAC)),
							new ColorPickerColor("8353M Fire Haze", Color.FromArgb(0xCF, 0xA1, 0x90)),
							new ColorPickerColor("8354M Parched Earth", Color.FromArgb(0xBC, 0x89, 0x79)),
							new ColorPickerColor("8355D Burnt Copper", Color.FromArgb(0x9F, 0x68, 0x58)),
							new ColorPickerColor("8356N Mackay Cedar", Color.FromArgb(0x7B, 0x4A, 0x3B)),
							new ColorPickerColor("8360W Bubble Bath", Color.FromArgb(0xE6, 0xD3, 0xCA)),
							new ColorPickerColor("8361W Baby Bouquet", Color.FromArgb(0xDF, 0xC3, 0xBA)),
							new ColorPickerColor("8362W Franciscan Pink", Color.FromArgb(0xD4, 0xB1, 0xA7)),
							new ColorPickerColor("8363M Cinnabar", Color.FromArgb(0xC0, 0x95, 0x8B)),
							new ColorPickerColor("8364M Olde Town", Color.FromArgb(0xAF, 0x82, 0x77)),
							new ColorPickerColor("8365D Colonial Brown", Color.FromArgb(0x8E, 0x5F, 0x54)),
							new ColorPickerColor("8366N Heartwood", Color.FromArgb(0x72, 0x46, 0x3B)),
							new ColorPickerColor("8370W Novette", Color.FromArgb(0xF3, 0xDE, 0xD6)),
							new ColorPickerColor("8371W Ancient Pink", Color.FromArgb(0xE4, 0xC5, 0xC0)),
							new ColorPickerColor("8372W Cinnaswirl", Color.FromArgb(0xDC, 0xB5, 0xB0)),
							new ColorPickerColor("8373M Norman Rose", Color.FromArgb(0xCC, 0x9E, 0x9C)),
							new ColorPickerColor("8374M Forest Berry", Color.FromArgb(0xB4, 0x83, 0x82)),
							new ColorPickerColor("8375D Ruby River", Color.FromArgb(0x95, 0x63, 0x60)),
							new ColorPickerColor("8376N Deep Henna", Color.FromArgb(0x79, 0x48, 0x41)),
							new ColorPickerColor("8380W Pink Shadow", Color.FromArgb(0xEA, 0xD9, 0xD2)),
							new ColorPickerColor("8381W Rosewater", Color.FromArgb(0xE0, 0xC7, 0xC2)),
							new ColorPickerColor("8382W Victorian Heath", Color.FromArgb(0xD6, 0xB6, 0xB2)),
							new ColorPickerColor("8383M Melodious Mauve", Color.FromArgb(0xC7, 0xA4, 0xA1)),
							new ColorPickerColor("8384M Broadmoor Rose", Color.FromArgb(0xB0, 0x89, 0x86)),
							new ColorPickerColor("8385D Brick Rose", Color.FromArgb(0x97, 0x6B, 0x67)),
							new ColorPickerColor("8386N Winestain", Color.FromArgb(0x75, 0x4D, 0x47)),
							new ColorPickerColor("8390W Rose Pearl", Color.FromArgb(0xF3, 0xE6, 0xE3)),
							new ColorPickerColor("8391W Fragile Pink", Color.FromArgb(0xE3, 0xC7, 0xC5)),
							new ColorPickerColor("8392W Fetching Pink", Color.FromArgb(0xDA, 0xB7, 0xB7)),
							new ColorPickerColor("8393M Rose Areys", Color.FromArgb(0xC8, 0x9D, 0xA0)),
							new ColorPickerColor("8394M Montrose", Color.FromArgb(0xB8, 0x88, 0x89)),
							new ColorPickerColor("8395D Monterey Rose", Color.FromArgb(0x8E, 0x60, 0x62)),
							new ColorPickerColor("8396N Cedar Rose", Color.FromArgb(0x6D, 0x44, 0x40)),
							new ColorPickerColor("8400W Sunset Snow", Color.FromArgb(0xF0, 0xD6, 0xD4)),
							new ColorPickerColor("8401W Strawberry Cream", Color.FromArgb(0xE7, 0xCA, 0xCB)),
							new ColorPickerColor("8402W Pinkwater", Color.FromArgb(0xE0, 0xBC, 0xBE)),
							new ColorPickerColor("8403M Moon Rose", Color.FromArgb(0xD0, 0xA1, 0xA6)),
							new ColorPickerColor("8404M Wine Cordial", Color.FromArgb(0xBC, 0x85, 0x8F)),
							new ColorPickerColor("8405D Red Valerian", Color.FromArgb(0x9F, 0x65, 0x6E)),
							new ColorPickerColor("8406N Red Jarrah", Color.FromArgb(0x77, 0x46, 0x47)),
							new ColorPickerColor("8410W Pink Gown", Color.FromArgb(0xEA, 0xE0, 0xDC)),
							new ColorPickerColor("8411W Debutante Pink", Color.FromArgb(0xE2, 0xCD, 0xCF)),
							new ColorPickerColor("8412W Satin Lining", Color.FromArgb(0xD8, 0xBD, 0xC5)),
							new ColorPickerColor("8413M Marjory Phlox", Color.FromArgb(0xC4, 0xA0, 0xAD)),
							new ColorPickerColor("8414M Wineberry", Color.FromArgb(0xB7, 0x8F, 0x9E)),
							new ColorPickerColor("8415D Rich Plum", Color.FromArgb(0x8C, 0x67, 0x77)),
							new ColorPickerColor("8416N Royal Garnet", Color.FromArgb(0x62, 0x41, 0x45)),
							new ColorPickerColor("8420W Tranquility", Color.FromArgb(0xDF, 0xD8, 0xD6)),
							new ColorPickerColor("8421W Meadow Sweet", Color.FromArgb(0xDF, 0xCF, 0xD4)),
							new ColorPickerColor("8422W Violet Fluff", Color.FromArgb(0xD3, 0xBF, 0xC7)),
							new ColorPickerColor("8423M Fading Amethyst", Color.FromArgb(0xC3, 0xAA, 0xB8)),
							new ColorPickerColor("8424M Viola Satin", Color.FromArgb(0xB2, 0x98, 0xA9)),
							new ColorPickerColor("8425D Exotic Bloom", Color.FromArgb(0x8A, 0x6D, 0x81)),
							new ColorPickerColor("8426N Purple Empire", Color.FromArgb(0x52, 0x43, 0x4C)),
							new ColorPickerColor("8430W Plum Light", Color.FromArgb(0xE8, 0xE1, 0xDC)),
							new ColorPickerColor("8431W Dusted Violet", Color.FromArgb(0xE1, 0xD5, 0xD5)),
							new ColorPickerColor("8432W Odessey Orchid", Color.FromArgb(0xD7, 0xC7, 0xCA)),
							new ColorPickerColor("8433M Plum Vine", Color.FromArgb(0xC2, 0xAC, 0xB5)),
							new ColorPickerColor("8434M Moss Phlox", Color.FromArgb(0xB0, 0x99, 0xA5)),
							new ColorPickerColor("8435D Blackberry Wine", Color.FromArgb(0x85, 0x6D, 0x7A)),
							new ColorPickerColor("8436N Abundant Wine", Color.FromArgb(0x55, 0x41, 0x49)),
							new ColorPickerColor("8440W First Light", Color.FromArgb(0xD7, 0xDA, 0xDA)),
							new ColorPickerColor("8441W Purple Pearl", Color.FromArgb(0xCA, 0xCD, 0xD0)),
							new ColorPickerColor("8442W Moon Violet", Color.FromArgb(0xB8, 0xBD, 0xC5)),
							new ColorPickerColor("8443M Tribute", Color.FromArgb(0xA4, 0xA8, 0xB4)),
							new ColorPickerColor("8444M Illusive Lilac", Color.FromArgb(0x88, 0x8E, 0x9E)),
							new ColorPickerColor("8445D Violet Kohl", Color.FromArgb(0x65, 0x6B, 0x7A)),
							new ColorPickerColor("8446N Purple Fable", Color.FromArgb(0x4A, 0x4D, 0x58)),
							new ColorPickerColor("8450W Twi-Night", Color.FromArgb(0xD2, 0xD8, 0xD9)),
							new ColorPickerColor("8451W Violet Dimensions", Color.FromArgb(0xC5, 0xCE, 0xD4)),
							new ColorPickerColor("8452W Aragon Violet", Color.FromArgb(0xB8, 0xC1, 0xCA)),
							new ColorPickerColor("8453W Cinder Grey", Color.FromArgb(0xA8, 0xB2, 0xBE)),
							new ColorPickerColor("8454M Smoky Mountain", Color.FromArgb(0x8B, 0x95, 0xA4)),
							new ColorPickerColor("8455D Old Amethyst", Color.FromArgb(0x62, 0x6D, 0x7D)),
							new ColorPickerColor("8456A Odessy Blue", Color.FromArgb(0x4A, 0x52, 0x60)),
							new ColorPickerColor("8460W Violet Cherub", Color.FromArgb(0xD5, 0xDD, 0xDF)),
							new ColorPickerColor("8461W Soft Ballad", Color.FromArgb(0xCA, 0xD5, 0xDB)),
							new ColorPickerColor("8462W Brushed Blue", Color.FromArgb(0xB4, 0xC0, 0xCC)),
							new ColorPickerColor("8463M Moonless Morn", Color.FromArgb(0x9E, 0xAD, 0xC0)),
							new ColorPickerColor("8464M Paris Evening", Color.FromArgb(0x87, 0x98, 0xAF)),
							new ColorPickerColor("8465D Indigo Violet", Color.FromArgb(0x5C, 0x70, 0x8B)),
							new ColorPickerColor("8466N Night Magic", Color.FromArgb(0x44, 0x52, 0x67)),
							new ColorPickerColor("8470W Diamond Minuet", Color.FromArgb(0xD1, 0xDD, 0xDE)),
							new ColorPickerColor("8471W Silk Whisper", Color.FromArgb(0xC4, 0xD3, 0xD7)),
							new ColorPickerColor("8472W Handsome Blue", Color.FromArgb(0xB3, 0xC4, 0xCC)),
							new ColorPickerColor("8473M Toccata", Color.FromArgb(0x99, 0xAE, 0xBB)),
							new ColorPickerColor("8474M Autumn Evening", Color.FromArgb(0x84, 0x9A, 0xAB)),
							new ColorPickerColor("8475D Cavalier", Color.FromArgb(0x5C, 0x72, 0x84)),
							new ColorPickerColor("8476N Symphony Blue", Color.FromArgb(0x43, 0x53, 0x64)),
							new ColorPickerColor("8480W Fluid Blue", Color.FromArgb(0xD5, 0xE1, 0xE1)),
							new ColorPickerColor("8481W Lingering Blue", Color.FromArgb(0xC5, 0xD6, 0xDB)),
							new ColorPickerColor("8482W Blue Visage", Color.FromArgb(0xB4, 0xC9, 0xD2)),
							new ColorPickerColor("8483M Antiock Blue", Color.FromArgb(0x99, 0xB2, 0xC2)),
							new ColorPickerColor("8484M Huckleberry", Color.FromArgb(0x85, 0xA0, 0xB4)),
							new ColorPickerColor("8485D Napoleon", Color.FromArgb(0x58, 0x76, 0x8E)),
							new ColorPickerColor("8486N Beethoven Blue", Color.FromArgb(0x41, 0x55, 0x68)),
							new ColorPickerColor("8490W Divine", Color.FromArgb(0xD9, 0xE6, 0xE2)),
							new ColorPickerColor("8491W Domicile Blue", Color.FromArgb(0xC6, 0xD7, 0xD9)),
							new ColorPickerColor("8492W Young Cadet", Color.FromArgb(0xAF, 0xC7, 0xCE)),
							new ColorPickerColor("8493M Endicott Bay", Color.FromArgb(0x8F, 0xAC, 0xB7)),
							new ColorPickerColor("8494M Rebel Blue", Color.FromArgb(0x7A, 0x9B, 0xA9)),
							new ColorPickerColor("8495D Twain Blue", Color.FromArgb(0x51, 0x70, 0x80)),
							new ColorPickerColor("8496A Dark Quest", Color.FromArgb(0x3E, 0x56, 0x64)),
							new ColorPickerColor("8500W Windstar", Color.FromArgb(0xDC, 0xE3, 0xDD)),
							new ColorPickerColor("8501W Blue Frost", Color.FromArgb(0xC8, 0xD4, 0xD5)),
							new ColorPickerColor("8502W Dawn Wonder", Color.FromArgb(0xB3, 0xC5, 0xCA)),
							new ColorPickerColor("8503M Faded Jeans", Color.FromArgb(0x95, 0xAC, 0xB6)),
							new ColorPickerColor("8504M Pottery Blue", Color.FromArgb(0x7F, 0x99, 0xA5)),
							new ColorPickerColor("8505D Bluecoat", Color.FromArgb(0x55, 0x6E, 0x7B)),
							new ColorPickerColor("8506A Valley Forge Blue", Color.FromArgb(0x41, 0x55, 0x60)),
							new ColorPickerColor("8510W Clean Vapor", Color.FromArgb(0xDA, 0xE0, 0xDD)),
							new ColorPickerColor("8511W Snowcloud", Color.FromArgb(0xC2, 0xCC, 0xCD)),
							new ColorPickerColor("8512W Hidden Brook", Color.FromArgb(0xB5, 0xC1, 0xC5)),
							new ColorPickerColor("8513M Still Creek", Color.FromArgb(0x98, 0xA8, 0xAF)),
							new ColorPickerColor("8514M Grey Blue", Color.FromArgb(0x84, 0x95, 0x9E)),
							new ColorPickerColor("8515D Thunder Gray", Color.FromArgb(0x59, 0x68, 0x71)),
							new ColorPickerColor("8516A Temptress", Color.FromArgb(0x45, 0x52, 0x5A)),
							new ColorPickerColor("8520W Glamour Gray", Color.FromArgb(0xD9, 0xDB, 0xD6)),
							new ColorPickerColor("8521W Halation", Color.FromArgb(0xCD, 0xD1, 0xCE)),
							new ColorPickerColor("8522W Gray Dignity", Color.FromArgb(0xB7, 0xBD, 0xBF)),
							new ColorPickerColor("8523M Downing Slate", Color.FromArgb(0x9C, 0xA4, 0xA8)),
							new ColorPickerColor("8524M Vanderbilt Gray", Color.FromArgb(0x87, 0x90, 0x96)),
							new ColorPickerColor("8525D Rock Garden", Color.FromArgb(0x5E, 0x66, 0x6C)),
							new ColorPickerColor("8526A Dark Colossus", Color.FromArgb(0x48, 0x4D, 0x52)),
							new ColorPickerColor("8530W Blizzard White", Color.FromArgb(0xD9, 0xDB, 0xD4)),
							new ColorPickerColor("8531W Lombardy Mist", Color.FromArgb(0xCF, 0xD3, 0xCF)),
							new ColorPickerColor("8532W Light Showers", Color.FromArgb(0xB7, 0xBC, 0xBA)),
							new ColorPickerColor("8533M Summer Fog", Color.FromArgb(0xA0, 0xA5, 0xA5)),
							new ColorPickerColor("8534M Solid Gray", Color.FromArgb(0x8C, 0x92, 0x93)),
							new ColorPickerColor("8535D Wayfarer Gray", Color.FromArgb(0x64, 0x6A, 0x6C)),
							new ColorPickerColor("8536N Blackmar", Color.FromArgb(0x45, 0x49, 0x4A)),
							new ColorPickerColor("8540W Windstream", Color.FromArgb(0xD6, 0xDE, 0xD9)),
							new ColorPickerColor("8541W Minimal Gray", Color.FromArgb(0xCC, 0xD3, 0xD1)),
							new ColorPickerColor("8542W Lido Bay", Color.FromArgb(0xB6, 0xBF, 0xC0)),
							new ColorPickerColor("8543M Frosted Denim", Color.FromArgb(0x99, 0xA3, 0xA6)),
							new ColorPickerColor("8544M Blue Comet", Color.FromArgb(0x84, 0x8F, 0x95)),
							new ColorPickerColor("8545D Quasar Blue", Color.FromArgb(0x5A, 0x64, 0x6A)),
							new ColorPickerColor("8546N Sable Night", Color.FromArgb(0x42, 0x47, 0x4B)),
							new ColorPickerColor("8550W Refined Blue", Color.FromArgb(0xD2, 0xD9, 0xD5)),
							new ColorPickerColor("8551W Crackling Blue", Color.FromArgb(0xC4, 0xCE, 0xCC)),
							new ColorPickerColor("8552W Blue of January", Color.FromArgb(0xB4, 0xC1, 0xC2)),
							new ColorPickerColor("8553M Myth Gray", Color.FromArgb(0x99, 0xA8, 0xAB)),
							new ColorPickerColor("8554M Rising Smoke", Color.FromArgb(0x84, 0x95, 0x99)),
							new ColorPickerColor("8555D Alden Charcoal", Color.FromArgb(0x5B, 0x6A, 0x6F)),
							new ColorPickerColor("8556A Peking Blue", Color.FromArgb(0x46, 0x51, 0x55)),
							new ColorPickerColor("8560W Moon River", Color.FromArgb(0xDB, 0xE5, 0xDD)),
							new ColorPickerColor("8561W Blue Emblem", Color.FromArgb(0xC2, 0xD2, 0xCE)),
							new ColorPickerColor("8562W Impressionist Sky", Color.FromArgb(0xB2, 0xC5, 0xC4)),
							new ColorPickerColor("8563M Gorse Blue", Color.FromArgb(0x95, 0xAC, 0xAD)),
							new ColorPickerColor("8564M Old Blue Sea", Color.FromArgb(0x80, 0x99, 0x9C)),
							new ColorPickerColor("8565D Winter Sea", Color.FromArgb(0x5B, 0x70, 0x73)),
							new ColorPickerColor("8566N Night Sail", Color.FromArgb(0x3F, 0x4D, 0x4F)),
							new ColorPickerColor("8570W Aqua Pura", Color.FromArgb(0xDB, 0xE3, 0xD9)),
							new ColorPickerColor("8571W Rushing River", Color.FromArgb(0xCB, 0xD5, 0xCD)),
							new ColorPickerColor("8572W San Fran Fog", Color.FromArgb(0xB4, 0xC3, 0xBE)),
							new ColorPickerColor("8573M Snug Harbor", Color.FromArgb(0x9A, 0xAC, 0xA9)),
							new ColorPickerColor("8574M Fog Buoy", Color.FromArgb(0x87, 0x9C, 0x99)),
							new ColorPickerColor("8575D Bay Bridge", Color.FromArgb(0x5F, 0x72, 0x70)),
							new ColorPickerColor("8576N Briny Deep", Color.FromArgb(0x47, 0x53, 0x52)),
							new ColorPickerColor("8580W Gray Etching", Color.FromArgb(0xD7, 0xDB, 0xD1)),
							new ColorPickerColor("8581W Designer Gray", Color.FromArgb(0xCC, 0xD2, 0xCA)),
							new ColorPickerColor("8582W Terrazzo Gray", Color.FromArgb(0xBC, 0xC4, 0xBB)),
							new ColorPickerColor("8583W Gray Nuance", Color.FromArgb(0xAC, 0xB5, 0xAD)),
							new ColorPickerColor("8584M Grey Expectations", Color.FromArgb(0x8E, 0x98, 0x92)),
							new ColorPickerColor("8585D Smokebrush", Color.FromArgb(0x67, 0x6F, 0x69)),
							new ColorPickerColor("8586A Metropolis", Color.FromArgb(0x51, 0x58, 0x54)),
							new ColorPickerColor("8590W Seed Pearl", Color.FromArgb(0xDD, 0xE4, 0xD9)),
							new ColorPickerColor("8591W Salt Water", Color.FromArgb(0xC7, 0xD2, 0xC7)),
							new ColorPickerColor("8592W Seahawk", Color.FromArgb(0xB6, 0xC4, 0xB9)),
							new ColorPickerColor("8593M Bayou Waters", Color.FromArgb(0xA2, 0xB2, 0xA9)),
							new ColorPickerColor("8594M Mystic Green", Color.FromArgb(0x86, 0x98, 0x90)),
							new ColorPickerColor("8595D Elm Court", Color.FromArgb(0x66, 0x76, 0x6E)),
							new ColorPickerColor("8596N Olympic Pine", Color.FromArgb(0x48, 0x52, 0x4C)),
							new ColorPickerColor("8600W Glass Block", Color.FromArgb(0xD7, 0xDC, 0xD3)),
							new ColorPickerColor("8601W Metropolitan Gray", Color.FromArgb(0xCB, 0xD3, 0xC9)),
							new ColorPickerColor("8602W Green Granule", Color.FromArgb(0xBE, 0xC5, 0xB9)),
							new ColorPickerColor("8603M Gray Nut", Color.FromArgb(0xA6, 0xB1, 0xA5)),
							new ColorPickerColor("8604M Victoria Falls", Color.FromArgb(0x91, 0x9D, 0x92)),
							new ColorPickerColor("8605D English Pine", Color.FromArgb(0x6A, 0x76, 0x6B)),
							new ColorPickerColor("8606N Spruce Island", Color.FromArgb(0x48, 0x4F, 0x47)),
							new ColorPickerColor("8610W Garden Light", Color.FromArgb(0xD9, 0xDD, 0xD1)),
							new ColorPickerColor("8611W Rainforest Dew", Color.FromArgb(0xCD, 0xD2, 0xC6)),
							new ColorPickerColor("8612W Silver Sage", Color.FromArgb(0xC2, 0xC8, 0xBA)),
							new ColorPickerColor("8613M Shenandoah", Color.FromArgb(0xAB, 0xB3, 0xA3)),
							new ColorPickerColor("8614M Shade Tree", Color.FromArgb(0x95, 0xA0, 0x8F)),
							new ColorPickerColor("8615D Peppercorn", Color.FromArgb(0x70, 0x7A, 0x6A)),
							new ColorPickerColor("8616N Clermont Green", Color.FromArgb(0x4F, 0x56, 0x49)),
							new ColorPickerColor("8620W Limpet Shell", Color.FromArgb(0xDA, 0xDA, 0xD2)),
							new ColorPickerColor("8621W Chromox Green", Color.FromArgb(0xCE, 0xD2, 0xC9)),
							new ColorPickerColor("8622W Lunar Stone", Color.FromArgb(0xC7, 0xCB, 0xBD)),
							new ColorPickerColor("8623M Silent Path", Color.FromArgb(0xA7, 0xAC, 0xA1)),
							new ColorPickerColor("8624M Steeple Gray", Color.FromArgb(0x95, 0x99, 0x8E)),
							new ColorPickerColor("8625D Shipyard", Color.FromArgb(0x6F, 0x73, 0x68)),
							new ColorPickerColor("8626N Wilderness Green", Color.FromArgb(0x51, 0x53, 0x4A)),
							new ColorPickerColor("8630W Prism White", Color.FromArgb(0xDC, 0xDD, 0xD4)),
							new ColorPickerColor("8631W European Gray", Color.FromArgb(0xD2, 0xD2, 0xC8)),
							new ColorPickerColor("8632W Timeless Gray", Color.FromArgb(0xC7, 0xC6, 0xBC)),
							new ColorPickerColor("8633W Smoky Candle", Color.FromArgb(0xB8, 0xB7, 0xAE)),
							new ColorPickerColor("8634M Dusty Miller", Color.FromArgb(0x9C, 0x9C, 0x93)),
							new ColorPickerColor("8635D Mansard Stone", Color.FromArgb(0x73, 0x73, 0x6A)),
							new ColorPickerColor("8636N Old Porch", Color.FromArgb(0x4E, 0x4E, 0x47)),
							new ColorPickerColor("8640W Creative White", Color.FromArgb(0xE3, 0xE4, 0xD9)),
							new ColorPickerColor("8641W Tradewind", Color.FromArgb(0xD5, 0xD5, 0xCA)),
							new ColorPickerColor("8642W Foggy London", Color.FromArgb(0xCA, 0xC9, 0xBE)),
							new ColorPickerColor("8643M Tram Gray", Color.FromArgb(0xAE, 0xAE, 0xA3)),
							new ColorPickerColor("8644M Seal Beach", Color.FromArgb(0x9A, 0x99, 0x8E)),
							new ColorPickerColor("8645D Greylock", Color.FromArgb(0x74, 0x73, 0x68)),
							new ColorPickerColor("8646N Blackened Beam", Color.FromArgb(0x53, 0x52, 0x47)),
							new ColorPickerColor("8650W Paraffin", Color.FromArgb(0xE2, 0xDF, 0xD1)),
							new ColorPickerColor("8651W Canvasback", Color.FromArgb(0xD6, 0xD3, 0xC5)),
							new ColorPickerColor("8652W Whippoorwill", Color.FromArgb(0xCD, 0xC8, 0xB8)),
							new ColorPickerColor("8653M Stone Lion", Color.FromArgb(0xB6, 0xB1, 0xA0)),
							new ColorPickerColor("8654M Elkhorn Cactus", Color.FromArgb(0xA2, 0x9E, 0x8D)),
							new ColorPickerColor("8655D Pinedale", Color.FromArgb(0x7C, 0x77, 0x67)),
							new ColorPickerColor("8656N Greek Olive", Color.FromArgb(0x57, 0x53, 0x46)),
							new ColorPickerColor("8660W Grecian Ivory", Color.FromArgb(0xE2, 0xE0, 0xD2)),
							new ColorPickerColor("8661W Light Almond", Color.FromArgb(0xD6, 0xD2, 0xC5)),
							new ColorPickerColor("8662W Jogging Path", Color.FromArgb(0xCB, 0xC6, 0xB7)),
							new ColorPickerColor("8663W Rugged Trail", Color.FromArgb(0xBC, 0xB6, 0xA5)),
							new ColorPickerColor("8664M Backpack", Color.FromArgb(0xA4, 0x9C, 0x8B)),
							new ColorPickerColor("8665D Gristmill", Color.FromArgb(0x7C, 0x74, 0x63)),
							new ColorPickerColor("8666N Potting Shed", Color.FromArgb(0x59, 0x52, 0x45)),
							new ColorPickerColor("8670W Walrus Tusk", Color.FromArgb(0xE1, 0xDB, 0xCA)),
							new ColorPickerColor("8671W Burbury Beige", Color.FromArgb(0xDD, 0xD5, 0xC3)),
							new ColorPickerColor("8672W Tequila", Color.FromArgb(0xD5, 0xCB, 0xB7)),
							new ColorPickerColor("8673M Tavern Taupe", Color.FromArgb(0xBE, 0xB0, 0x9B)),
							new ColorPickerColor("8674M Muddy River", Color.FromArgb(0xAD, 0x9E, 0x89)),
							new ColorPickerColor("8675D Wooden Oar", Color.FromArgb(0x88, 0x77, 0x62)),
							new ColorPickerColor("8676N Turned Earth", Color.FromArgb(0x68, 0x59, 0x47)),
							new ColorPickerColor("8680W Aria Ivory", Color.FromArgb(0xE2, 0xDC, 0xCF)),
							new ColorPickerColor("8681W Artists Canvas", Color.FromArgb(0xDA, 0xD2, 0xC2)),
							new ColorPickerColor("8682W Estate Greige", Color.FromArgb(0xD0, 0xC6, 0xB6)),
							new ColorPickerColor("8683W Tinderbox", Color.FromArgb(0xC1, 0xB5, 0xA4)),
							new ColorPickerColor("8684M Oak Flats", Color.FromArgb(0xA9, 0x9A, 0x86)),
							new ColorPickerColor("8685D Woodlet", Color.FromArgb(0x84, 0x74, 0x61)),
							new ColorPickerColor("8686N Northwest Trail", Color.FromArgb(0x64, 0x55, 0x45)),
							new ColorPickerColor("8690W Snow Ballet", Color.FromArgb(0xEA, 0xE8, 0xDB)),
							new ColorPickerColor("8691W City Loft", Color.FromArgb(0xDB, 0xD4, 0xC6)),
							new ColorPickerColor("8692W Bauhaus Buff", Color.FromArgb(0xD0, 0xC7, 0xB8)),
							new ColorPickerColor("8693M Stucco Greige", Color.FromArgb(0xB6, 0xAB, 0x9C)),
							new ColorPickerColor("8694M Wood Cut", Color.FromArgb(0xA4, 0x98, 0x88)),
							new ColorPickerColor("8695D Thatch Roof", Color.FromArgb(0x80, 0x73, 0x62)),
							new ColorPickerColor("8696N Twisted Branch", Color.FromArgb(0x5E, 0x52, 0x45)),
							new ColorPickerColor("8700W Powdering Snow", Color.FromArgb(0xEA, 0xE6, 0xDA)),
							new ColorPickerColor("8701W Softly Chiming", Color.FromArgb(0xDB, 0xD5, 0xC9)),
							new ColorPickerColor("8702W Beige Pediment", Color.FromArgb(0xCA, 0xC3, 0xB7)),
							new ColorPickerColor("8703M Rockbridge", Color.FromArgb(0xBB, 0xB2, 0xA4)),
							new ColorPickerColor("8704D Stratford Brown", Color.FromArgb(0xA3, 0x97, 0x87)),
							new ColorPickerColor("8705D Barn Rafter", Color.FromArgb(0x7C, 0x6F, 0x62)),
							new ColorPickerColor("8706N Saddlebury", Color.FromArgb(0x5E, 0x54, 0x4A)),
							new ColorPickerColor("8710W Discovery", Color.FromArgb(0xE2, 0xDD, 0xD1)),
							new ColorPickerColor("8711W Misty Mica", Color.FromArgb(0xDA, 0xD1, 0xC3)),
							new ColorPickerColor("8712W Mushroom Basket", Color.FromArgb(0xCF, 0xC4, 0xB5)),
							new ColorPickerColor("8713W Kindling Wood", Color.FromArgb(0xBE, 0xB2, 0xA2)),
							new ColorPickerColor("8714M Wildcat", Color.FromArgb(0xA5, 0x97, 0x86)),
							new ColorPickerColor("8715D Moose Point", Color.FromArgb(0x7F, 0x70, 0x5F)),
							new ColorPickerColor("8716N Western Reserve", Color.FromArgb(0x5F, 0x51, 0x44)),
							new ColorPickerColor("8720W Pongee Tint", Color.FromArgb(0xE6, 0xDC, 0xCD)),
							new ColorPickerColor("8721W Clay Beige", Color.FromArgb(0xDB, 0xCC, 0xBA)),
							new ColorPickerColor("8722W Desert Castle", Color.FromArgb(0xD8, 0xC7, 0xB2)),
							new ColorPickerColor("8723M Deset Tumbleweed", Color.FromArgb(0xC9, 0xB7, 0xA3)),
							new ColorPickerColor("8724M Meadowlark", Color.FromArgb(0xB7, 0xA1, 0x8A)),
							new ColorPickerColor("8725A Coach House", Color.FromArgb(0x89, 0x6F, 0x59)),
							new ColorPickerColor("8726N Mocha Brown", Color.FromArgb(0x69, 0x54, 0x43)),
							new ColorPickerColor("8730W Loggia", Color.FromArgb(0xEB, 0xE2, 0xD2)),
							new ColorPickerColor("8731W Staghorn", Color.FromArgb(0xDB, 0xCA, 0xB9)),
							new ColorPickerColor("8732W Frontier Tan", Color.FromArgb(0xCB, 0xB9, 0xA6)),
							new ColorPickerColor("8733M Walnut Wash", Color.FromArgb(0xBC, 0xA7, 0x93)),
							new ColorPickerColor("8734M Bark Mulch", Color.FromArgb(0xAB, 0x94, 0x80)),
							new ColorPickerColor("8735D Wild Country", Color.FromArgb(0x87, 0x70, 0x5D)),
							new ColorPickerColor("8736N Little Bighorn", Color.FromArgb(0x6F, 0x58, 0x47)),
							new ColorPickerColor("8740W Barrister White", Color.FromArgb(0xE9, 0xDC, 0xCD)),
							new ColorPickerColor("8741W Historic Tan", Color.FromArgb(0xD6, 0xC6, 0xB8)),
							new ColorPickerColor("8742W Brickdust", Color.FromArgb(0xCC, 0xBB, 0xAC)),
							new ColorPickerColor("8743M Polished Oak", Color.FromArgb(0xB7, 0xA0, 0x90)),
							new ColorPickerColor("8744D Sweetwood", Color.FromArgb(0xA6, 0x8B, 0x79)),
							new ColorPickerColor("8745A Kayak Brown", Color.FromArgb(0x83, 0x68, 0x58)),
							new ColorPickerColor("8746N Timber Trail", Color.FromArgb(0x5D, 0x47, 0x3B)),
							new ColorPickerColor("8750W Swiss Almond", Color.FromArgb(0xE2, 0xDB, 0xD1)),
							new ColorPickerColor("8751W Alencon Lace", Color.FromArgb(0xD2, 0xC7, 0xBC)),
							new ColorPickerColor("8752W Stonehenge Shadow", Color.FromArgb(0xC9, 0xBD, 0xB1)),
							new ColorPickerColor("8753M Cornico Beige", Color.FromArgb(0xB4, 0xA4, 0x97)),
							new ColorPickerColor("8754D Tomorrow's Taupe", Color.FromArgb(0xA2, 0x8F, 0x81)),
							new ColorPickerColor("8755A Spicenut", Color.FromArgb(0x7D, 0x69, 0x5C)),
							new ColorPickerColor("8756n Cordwood", Color.FromArgb(0x59, 0x49, 0x40)),
							new ColorPickerColor("8760W Gray Ashlar", Color.FromArgb(0xDB, 0xD6, 0xCC)),
							new ColorPickerColor("8761W Kennet Square", Color.FromArgb(0xD4, 0xCD, 0xC4)),
							new ColorPickerColor("8762W Touchstone", Color.FromArgb(0xC8, 0xBF, 0xB5)),
							new ColorPickerColor("8763M Pewter Works", Color.FromArgb(0xB2, 0xA6, 0x9B)),
							new ColorPickerColor("8764M Lava Rock", Color.FromArgb(0x9F, 0x94, 0x8A)),
							new ColorPickerColor("8765D Beaver Creek", Color.FromArgb(0x71, 0x68, 0x5F)),
							new ColorPickerColor("8766N Caribou", Color.FromArgb(0x5E, 0x51, 0x48)),
							new ColorPickerColor("8770W Silver Slipper", Color.FromArgb(0xD9, 0xDB, 0xD3)),
							new ColorPickerColor("8771W Silver Shimmer", Color.FromArgb(0xCD, 0xCC, 0xC6)),
							new ColorPickerColor("8772W Siberian Ice", Color.FromArgb(0xC4, 0xC2, 0xBB)),
							new ColorPickerColor("8773M Dauphin Gray", Color.FromArgb(0xA9, 0xA4, 0x9C)),
							new ColorPickerColor("8774M Tankard", Color.FromArgb(0x97, 0x92, 0x8B)),
							new ColorPickerColor("8775D Metal Shavings", Color.FromArgb(0x70, 0x6B, 0x64)),
							new ColorPickerColor("8776N Blacksmith", Color.FromArgb(0x50, 0x4C, 0x47)),
							new ColorPickerColor("8780W Crystal Ball", Color.FromArgb(0xE0, 0xDF, 0xD6)),
							new ColorPickerColor("8781W Silver Chime", Color.FromArgb(0xD4, 0xD4, 0xCC)),
							new ColorPickerColor("8782W Sterling Coin", Color.FromArgb(0xC9, 0xC9, 0xC3)),
							new ColorPickerColor("8783M Stonewall", Color.FromArgb(0xA9, 0xA8, 0xA3)),
							new ColorPickerColor("8784D Blackthorn", Color.FromArgb(0x8D, 0x8B, 0x85)),
							new ColorPickerColor("8785A Iron Horse", Color.FromArgb(0x65, 0x63, 0x5F)),
							new ColorPickerColor("8786N Black Finish", Color.FromArgb(0x40, 0x3F, 0x3D)),
							new ColorPickerColor("8790W Intercoastal Gray", Color.FromArgb(0xDD, 0xDB, 0xD0)),
							new ColorPickerColor("8791W Gray Pennant", Color.FromArgb(0xCF, 0xCC, 0xC6)),
							new ColorPickerColor("8792W Monorail Silver", Color.FromArgb(0xC2, 0xBD, 0xB7)),
							new ColorPickerColor("8793M Grey Cliff", Color.FromArgb(0xA3, 0x9F, 0x9C)),
							new ColorPickerColor("8794M Fired Steel", Color.FromArgb(0x8F, 0x8D, 0x8A)),
							new ColorPickerColor("8795D Magnet", Color.FromArgb(0x67, 0x65, 0x63)),
							new ColorPickerColor("8796N Black Metal", Color.FromArgb(0x46, 0x44, 0x41)),
							new ColorPickerColor("8800W Portico", Color.FromArgb(0xD5, 0xD4, 0xCC)),
							new ColorPickerColor("8801W Studio Gray", Color.FromArgb(0xCB, 0xC9, 0xC2)),
							new ColorPickerColor("8802W Threshold Gray", Color.FromArgb(0xC1, 0xBE, 0xB8)),
							new ColorPickerColor("8803M Olde Fashion", Color.FromArgb(0xA9, 0xA6, 0xA0)),
							new ColorPickerColor("8804M Pavestone", Color.FromArgb(0x95, 0x91, 0x8D)),
							new ColorPickerColor("8805D Stonington", Color.FromArgb(0x6D, 0x69, 0x64)),
							new ColorPickerColor("8806N Tudor House", Color.FromArgb(0x4F, 0x49, 0x45)),
							new ColorPickerColor("8810W Siver Seed", Color.FromArgb(0xDB, 0xD8, 0xD2)),
							new ColorPickerColor("8811W Daytona Gray", Color.FromArgb(0xD0, 0xCC, 0xC5)),
							new ColorPickerColor("8812W Deville", Color.FromArgb(0xC2, 0xBC, 0xB6)),
							new ColorPickerColor("8813W Stone Cistern", Color.FromArgb(0xB2, 0xAA, 0xA4)),
							new ColorPickerColor("8814M Brindel", Color.FromArgb(0x96, 0x8D, 0x8A)),
							new ColorPickerColor("8815D Cultivated Fields", Color.FromArgb(0x72, 0x68, 0x62)),
							new ColorPickerColor("8816N Burnt Hickory", Color.FromArgb(0x54, 0x4A, 0x46)),
							new ColorPickerColor("8820W Cassis Ice", Color.FromArgb(0xDE, 0xD5, 0xCB)),
							new ColorPickerColor("8821W Mogul Mauve", Color.FromArgb(0xD4, 0xCB, 0xC2)),
							new ColorPickerColor("8822W Smoked Glass", Color.FromArgb(0xC6, 0xBA, 0xB1)),
							new ColorPickerColor("8823W Warm Shadows", Color.FromArgb(0xB4, 0xA9, 0xA1)),
							new ColorPickerColor("8824M Stone Freize", Color.FromArgb(0x99, 0x8C, 0x84)),
							new ColorPickerColor("8825D Jacaranda Brown", Color.FromArgb(0x74, 0x66, 0x5F)),
							new ColorPickerColor("8826N Manganese", Color.FromArgb(0x59, 0x4C, 0x45)),
							new ColorPickerColor("8830W Post Modern White", Color.FromArgb(0xDD, 0xD5, 0xCD)),
							new ColorPickerColor("8831W Bohemian Tint", Color.FromArgb(0xD4, 0xC8, 0xC1)),
							new ColorPickerColor("8832W Hourglass", Color.FromArgb(0xC7, 0xBA, 0xB3)),
							new ColorPickerColor("8833M Canberra", Color.FromArgb(0xB0, 0x9E, 0x98)),
							new ColorPickerColor("8834M Faux Marble", Color.FromArgb(0x9D, 0x8A, 0x84)),
							new ColorPickerColor("8835D Pawnee", Color.FromArgb(0x77, 0x63, 0x5D)),
							new ColorPickerColor("8836A Wagon Wheel", Color.FromArgb(0x62, 0x50, 0x4A)),
							new ColorPickerColor("8840W Rosevale", Color.FromArgb(0xE4, 0xDA, 0xD2)),
							new ColorPickerColor("8841W Sugar Blush", Color.FromArgb(0xD4, 0xC7, 0xC1)),
							new ColorPickerColor("8842W Retro Rose", Color.FromArgb(0xC7, 0xB7, 0xB1)),
							new ColorPickerColor("8843M Shallow Root", Color.FromArgb(0xAF, 0x9B, 0x96)),
							new ColorPickerColor("8844M Brown Bark", Color.FromArgb(0x9C, 0x88, 0x83)),
							new ColorPickerColor("8845D Colony Brick", Color.FromArgb(0x76, 0x60, 0x5B)),
							new ColorPickerColor("8846N Bernard Brown", Color.FromArgb(0x5A, 0x47, 0x41)),
							new ColorPickerColor("8850W Blackberry Ice", Color.FromArgb(0xDA, 0xD2, 0xCE)),
							new ColorPickerColor("8851W Motif Mauve", Color.FromArgb(0xD0, 0xC7, 0xC5)),
							new ColorPickerColor("8852W Medieval Mauve", Color.FromArgb(0xC2, 0xB7, 0xB6)),
							new ColorPickerColor("8853M Grey Heather", Color.FromArgb(0xA9, 0x97, 0x95)),
							new ColorPickerColor("8854M Drifting Spirit", Color.FromArgb(0x96, 0x84, 0x84)),
							new ColorPickerColor("8855D Cranbrook", Color.FromArgb(0x70, 0x5E, 0x5C)),
							new ColorPickerColor("8856N Badlands", Color.FromArgb(0x5B, 0x49, 0x46)),
							new ColorPickerColor("8860W Silver Mink", Color.FromArgb(0xE2, 0xDE, 0xD8)),
							new ColorPickerColor("8861W Icon Gray", Color.FromArgb(0xD8, 0xD5, 0xCF)),
							new ColorPickerColor("8862W Cliffside", Color.FromArgb(0xCE, 0xCA, 0xC7)),
							new ColorPickerColor("8863M Gray Archive", Color.FromArgb(0xA5, 0x9E, 0x9E)),
							new ColorPickerColor("8864D Philosophical Gray", Color.FromArgb(0x92, 0x8A, 0x8A)),
							new ColorPickerColor("8865D Zodiac Gray", Color.FromArgb(0x7B, 0x73, 0x74)),
							new ColorPickerColor("8866A Curtain Call", Color.FromArgb(0x5D, 0x56, 0x58)),
							new ColorPickerColor("8870W Fanlight", Color.FromArgb(0xE7, 0xE3, 0xDC)),
							new ColorPickerColor("8871W Classical Mauve", Color.FromArgb(0xD6, 0xCC, 0xCA)),
							new ColorPickerColor("8872W Mauvedust", Color.FromArgb(0xCA, 0xB9, 0xB9)),
							new ColorPickerColor("8873M Sassy Mauve", Color.FromArgb(0xB2, 0x9E, 0x9F)),
							new ColorPickerColor("8874M Plum Illusion", Color.FromArgb(0xA5, 0x8F, 0x93)),
							new ColorPickerColor("8875D Pigeon Isle", Color.FromArgb(0x7C, 0x65, 0x67)),
							new ColorPickerColor("8876N Spiced Plum", Color.FromArgb(0x5B, 0x47, 0x46)),
							new ColorPickerColor("8880W Bashful", Color.FromArgb(0xDF, 0xD2, 0xCF)),
							new ColorPickerColor("8881W Charlotte Rose", Color.FromArgb(0xD8, 0xC7, 0xC7)),
							new ColorPickerColor("8882W Mauvelous", Color.FromArgb(0xD0, 0xBA, 0xBC)),
							new ColorPickerColor("8883M Country Berry", Color.FromArgb(0xB8, 0x9F, 0xA3)),
							new ColorPickerColor("8884M Moonstone Mauve", Color.FromArgb(0xA3, 0x89, 0x8E)),
							new ColorPickerColor("8885D Velvet Maroon", Color.FromArgb(0x7C, 0x60, 0x63)),
							new ColorPickerColor("8886N Best Burgundy", Color.FromArgb(0x5B, 0x44, 0x43)),
							new ColorPickerColor("8890W Angel Flutter", Color.FromArgb(0xDF, 0xDA, 0xD7)),
							new ColorPickerColor("8891W Pinkish Plum", Color.FromArgb(0xD6, 0xCD, 0xCC)),
							new ColorPickerColor("8892W Heather Ashes", Color.FromArgb(0xCA, 0xBF, 0xC0)),
							new ColorPickerColor("8893M Mauve Silk", Color.FromArgb(0xB5, 0xA7, 0xAA)),
							new ColorPickerColor("8894M Smoked Mauve", Color.FromArgb(0xA1, 0x91, 0x96)),
							new ColorPickerColor("8895D Aged Plum", Color.FromArgb(0x78, 0x68, 0x6D)),
							new ColorPickerColor("8896N Black Bordeaux", Color.FromArgb(0x57, 0x48, 0x4A)),
							new ColorPickerColor("8900W Lavender Bouquet", Color.FromArgb(0xE3, 0xDF, 0xDC)),
							new ColorPickerColor("8901W Lavender Design", Color.FromArgb(0xD5, 0xCC, 0xD0)),
							new ColorPickerColor("8902W Secret Lilac", Color.FromArgb(0xC9, 0xBE, 0xC4)),
							new ColorPickerColor("8903W Violet Villa", Color.FromArgb(0xB9, 0xAC, 0xB6)),
							new ColorPickerColor("8904M Lilac Memento", Color.FromArgb(0xA3, 0x95, 0xA1)),
							new ColorPickerColor("8905D Glorious Plum", Color.FromArgb(0x79, 0x6B, 0x78)),
							new ColorPickerColor("8906N Purple Honor", Color.FromArgb(0x51, 0x47, 0x4F)),
							new ColorPickerColor("8910W Г%tude Lilac", Color.FromArgb(0xDB, 0xDA, 0xD8)),
							new ColorPickerColor("8911W Pale Thistle", Color.FromArgb(0xD1, 0xCD, 0xD0)),
							new ColorPickerColor("8912W Pale Amethyst", Color.FromArgb(0xC8, 0xC4, 0xCA)),
							new ColorPickerColor("8913W Vogue Violet", Color.FromArgb(0xB8, 0xB1, 0xBB)),
							new ColorPickerColor("8914M Grape Harvest", Color.FromArgb(0x9B, 0x94, 0xA2)),
							new ColorPickerColor("8915D Crushed Violets", Color.FromArgb(0x72, 0x6B, 0x7A)),
							new ColorPickerColor("8916N Purple Vignette", Color.FromArgb(0x52, 0x4C, 0x57)),
							new ColorPickerColor("AC073N Purple Spectacle", Color.FromArgb(0x4C, 0x46, 0x69)),
							new ColorPickerColor("AC074N Nocturne", Color.FromArgb(0x40, 0x42, 0x65)),
							new ColorPickerColor("AC075N Galaxy Blue", Color.FromArgb(0x39, 0x42, 0x5E)),
							new ColorPickerColor("AC076N Harbour Blue", Color.FromArgb(0x29, 0x4B, 0x7A)),
							new ColorPickerColor("AC077N Rendevous Blue", Color.FromArgb(0x30, 0x43, 0x67)),
							new ColorPickerColor("AC078N Dark Batik", Color.FromArgb(0x34, 0x42, 0x60)),
							new ColorPickerColor("AC079N Blue Brilliance", Color.FromArgb(0x00, 0x68, 0x9F)),
							new ColorPickerColor("AC080N Delphinium", Color.FromArgb(0x1E, 0x55, 0x82)),
							new ColorPickerColor("AC081N Ship Ahoy", Color.FromArgb(0x28, 0x4C, 0x72)),
							new ColorPickerColor("AC082N Coastal Shores", Color.FromArgb(0x00, 0x62, 0x8E)),
							new ColorPickerColor("AC083N Incan Sapphire", Color.FromArgb(0x25, 0x4D, 0x72)),
							new ColorPickerColor("AC084N Night Navy", Color.FromArgb(0x3A, 0x4A, 0x5F)),
							new ColorPickerColor("AC085N Tropical Surf", Color.FromArgb(0x00, 0x73, 0x85)),
							new ColorPickerColor("AC086N Teal Edge", Color.FromArgb(0x00, 0x5B, 0x67)),
							new ColorPickerColor("AC087N Architectural Teal", Color.FromArgb(0x23, 0x54, 0x63)),
							new ColorPickerColor("AC088N Ocean Reef", Color.FromArgb(0x00, 0x7A, 0x6C)),
							new ColorPickerColor("AC089N Green Luster", Color.FromArgb(0x00, 0x6D, 0x5B)),
							new ColorPickerColor("AC090N Rainforest Teal", Color.FromArgb(0x1D, 0x53, 0x54)),
							new ColorPickerColor("AC091N Earthbound Green", Color.FromArgb(0x33, 0x46, 0x44)),
							new ColorPickerColor("AC092N Strong Hunter", Color.FromArgb(0x2E, 0x4C, 0x47)),
							new ColorPickerColor("AC093N Alligator", Color.FromArgb(0x34, 0x44, 0x44)),
							new ColorPickerColor("AC094N Holiday Green", Color.FromArgb(0x2E, 0x51, 0x4C)),
							new ColorPickerColor("AC095N Green Ideal", Color.FromArgb(0x0A, 0x70, 0x58)),
							new ColorPickerColor("AC096n Golfing Green", Color.FromArgb(0x23, 0x71, 0x56)),
							new ColorPickerColor("AC097N Kryptonite", Color.FromArgb(0x44, 0x81, 0x41)),
							new ColorPickerColor("AC098N Java Green", Color.FromArgb(0x2B, 0x88, 0x4A)),
							new ColorPickerColor("AC099N Dragonfly", Color.FromArgb(0x26, 0x7F, 0x4D)),
							new ColorPickerColor("AC100N Emeraude", Color.FromArgb(0x26, 0x6A, 0x49)),
							new ColorPickerColor("AC101N New Meadow", Color.FromArgb(0x37, 0x5C, 0x42)),
							new ColorPickerColor("AC102N Raintree", Color.FromArgb(0x31, 0x53, 0x46)),
							new ColorPickerColor("AC103Y Sun Coast", Color.FromArgb(0xFF, 0xB8, 0x16)),
							new ColorPickerColor("AC104Y Southern Phoenix", Color.FromArgb(0xEE, 0xA9, 0x1F)),
							new ColorPickerColor("AC105Y Sunbury", Color.FromArgb(0xFF, 0xC7, 0x0F)),
							new ColorPickerColor("AC106Y El Sol", Color.FromArgb(0xFF, 0xA2, 0x1E)),
							new ColorPickerColor("AC107Y Orange Fiesta", Color.FromArgb(0xF0, 0x86, 0x2F)),
							new ColorPickerColor("AC108Y Serengeti Sun", Color.FromArgb(0xD6, 0x6B, 0x3B)),
							new ColorPickerColor("AC109Y Mandarin Melody", Color.FromArgb(0xD7, 0x5C, 0x42)),
							new ColorPickerColor("AC110N Fireking Red", Color.FromArgb(0xBC, 0x41, 0x3C)),
							new ColorPickerColor("AC111R Arresting Red", Color.FromArgb(0xB9, 0x39, 0x3B)),
							new ColorPickerColor("AC112N Firebush", Color.FromArgb(0xB4, 0x53, 0x43)),
							new ColorPickerColor("AC113N Spiced Rum", Color.FromArgb(0x8F, 0x47, 0x3A)),
							new ColorPickerColor("AC114N Robust Red", Color.FromArgb(0x63, 0x3C, 0x3A)),
							new ColorPickerColor("AC115N Tomatillo", Color.FromArgb(0x96, 0x43, 0x42)),
							new ColorPickerColor("AC116N Roasted Pepper", Color.FromArgb(0x84, 0x3B, 0x39)),
							new ColorPickerColor("AC117N French Red", Color.FromArgb(0x74, 0x40, 0x42)),
							new ColorPickerColor("AC118R Red Alert", Color.FromArgb(0xAC, 0x35, 0x37)),
							new ColorPickerColor("AC119R Sizzling Haute", Color.FromArgb(0x96, 0x38, 0x37)),
							new ColorPickerColor("AC120R Wright Red", Color.FromArgb(0xA9, 0x3A, 0x46)),
							new ColorPickerColor("AC121R Ruby Red", Color.FromArgb(0x9C, 0x3B, 0x45)),
							new ColorPickerColor("AC122N Watermelon Crush", Color.FromArgb(0x9A, 0x3F, 0x55)),
							new ColorPickerColor("AC123N Velvet Rose", Color.FromArgb(0x84, 0x39, 0x47)),
							new ColorPickerColor("AC124N Shocking Fuschia", Color.FromArgb(0x9D, 0x42, 0x62)),
							new ColorPickerColor("AC125N Berry Inspiration", Color.FromArgb(0x91, 0x3E, 0x5A)),
							new ColorPickerColor("AC126N New Berry", Color.FromArgb(0x8B, 0x45, 0x59)),
							new ColorPickerColor("AC127N Cherry Rose", Color.FromArgb(0x7D, 0x44, 0x4E)),
							new ColorPickerColor("AC128N Rich Burgundy", Color.FromArgb(0x72, 0x3D, 0x46)),
							new ColorPickerColor("AC129N Pompadour", Color.FromArgb(0x62, 0x3C, 0x44)),
							new ColorPickerColor("AC130N Rose Bouquet", Color.FromArgb(0x7B, 0x45, 0x5E)),
							new ColorPickerColor("AC131N Cassis", Color.FromArgb(0x6A, 0x3E, 0x59)),
							new ColorPickerColor("AC132N Purple Opulence", Color.FromArgb(0x68, 0x44, 0x5A)),
							new ColorPickerColor("AC133N Royal Crest", Color.FromArgb(0x6B, 0x48, 0x6D)),
							new ColorPickerColor("AC134N Wild Lilac", Color.FromArgb(0x67, 0x4E, 0x6E)),
							new ColorPickerColor("AC135N Purple Elegance", Color.FromArgb(0x56, 0x3E, 0x53)),
							new ColorPickerColor("AC136N Regal Purple", Color.FromArgb(0x4A, 0x3C, 0x49)),
							new ColorPickerColor("AC137N Plumberry", Color.FromArgb(0x4F, 0x3D, 0x45)),
							new ColorPickerColor("AC138N Plummit", Color.FromArgb(0x49, 0x3C, 0x45)),
							new ColorPickerColor("AC139N Aubergine", Color.FromArgb(0x45, 0x39, 0x40)),
							new ColorPickerColor("AC140N Mission Brown", Color.FromArgb(0x44, 0x3B, 0x39)),
							new ColorPickerColor("AC141N Momino", Color.FromArgb(0x34, 0x38, 0x39)),
							new ColorPickerColor("AC142N Purple Jazz", Color.FromArgb(0x3B, 0x38, 0x4B)),
							new ColorPickerColor("AC143N Night Mission", Color.FromArgb(0x33, 0x39, 0x4A)),
							new ColorPickerColor("AC144N Black Deco", Color.FromArgb(0x38, 0x39, 0x39)),
							new ColorPickerColor("CW001W Soft Whisper", Color.FromArgb(0xD9, 0xDC, 0xDF)),
							new ColorPickerColor("CW002W Rain Shimmer", Color.FromArgb(0xE8, 0xE9, 0xE4)),
							new ColorPickerColor("CW003W Mysterious", Color.FromArgb(0xE9, 0xE9, 0xE0)),
							new ColorPickerColor("CW004W Delicate Blue", Color.FromArgb(0xE0, 0xE7, 0xE4)),
							new ColorPickerColor("CW005W Winterscape", Color.FromArgb(0xDE, 0xE0, 0xDA)),
							new ColorPickerColor("CW006W Shaded Ice", Color.FromArgb(0xDE, 0xDE, 0xD8)),
							new ColorPickerColor("CW007W Blue Twinkle", Color.FromArgb(0xD7, 0xE3, 0xE3)),
							new ColorPickerColor("CW008W Blue Glance", Color.FromArgb(0xE1, 0xE8, 0xE2)),
							new ColorPickerColor("CW009W Barely Blue", Color.FromArgb(0xE5, 0xEC, 0xE4)),
							new ColorPickerColor("CW010W Blueberry Haze", Color.FromArgb(0xD9, 0xE2, 0xDB)),
							new ColorPickerColor("CW011W Serene White", Color.FromArgb(0xDA, 0xDF, 0xD8)),
							new ColorPickerColor("CW012W Blueberry Mist", Color.FromArgb(0xDD, 0xE3, 0xDC)),
							new ColorPickerColor("CW013W Blueberry Hint", Color.FromArgb(0xDC, 0xE7, 0xDE)),
							new ColorPickerColor("CW014W Blueberry Frost", Color.FromArgb(0xE4, 0xEA, 0xDE)),
							new ColorPickerColor("CW015W Perfect Mint", Color.FromArgb(0xE6, 0xED, 0xE0)),
							new ColorPickerColor("CW016W Lime Squeeze", Color.FromArgb(0xE3, 0xE8, 0xD9)),
							new ColorPickerColor("CW017W Limon", Color.FromArgb(0xE2, 0xE5, 0xD4)),
							new ColorPickerColor("CW018W Lime Treat", Color.FromArgb(0xE9, 0xEE, 0xDD)),
							new ColorPickerColor("CW019W Lemonade", Color.FromArgb(0xF0, 0xE8, 0xCB)),
							new ColorPickerColor("CW020W Lemon White", Color.FromArgb(0xEF, 0xEA, 0xD5)),
							new ColorPickerColor("CW021W Lemon Peel", Color.FromArgb(0xED, 0xE9, 0xD5)),
							new ColorPickerColor("CW022W Soft Gold", Color.FromArgb(0xF8, 0xE8, 0xCD)),
							new ColorPickerColor("CW023W Lis Creme", Color.FromArgb(0xF8, 0xEE, 0xD7)),
							new ColorPickerColor("CW024W Soaring White", Color.FromArgb(0xF6, 0xED, 0xD8)),
							new ColorPickerColor("CW025W Restorative", Color.FromArgb(0xEC, 0xDF, 0xC7)),
							new ColorPickerColor("CW026W Lemon Chiffon", Color.FromArgb(0xEE, 0xE7, 0xD3)),
							new ColorPickerColor("CW027W Sensitive White", Color.FromArgb(0xF0, 0xE8, 0xD3)),
							new ColorPickerColor("CW028W Orange Whip", Color.FromArgb(0xEF, 0xE1, 0xCD)),
							new ColorPickerColor("CW029W Orange Peel", Color.FromArgb(0xF0, 0xE5, 0xD2)),
							new ColorPickerColor("CW030W Apple Peel", Color.FromArgb(0xEF, 0xE8, 0xDA)),
							new ColorPickerColor("CW031W Coconut Spirit", Color.FromArgb(0xE9, 0xDC, 0xCC)),
							new ColorPickerColor("CW032W Orange Foam", Color.FromArgb(0xF1, 0xE7, 0xD4)),
							new ColorPickerColor("CW033W Floral White", Color.FromArgb(0xF1, 0xEC, 0xDD)),
							new ColorPickerColor("CW034W Coconut Milk", Color.FromArgb(0xEB, 0xE1, 0xD0)),
							new ColorPickerColor("CW035W Coconut Twist", Color.FromArgb(0xEA, 0xDF, 0xCF)),
							new ColorPickerColor("CW036W Apple Tint", Color.FromArgb(0xEF, 0xE6, 0xD8)),
							new ColorPickerColor("CW037W White Filigree", Color.FromArgb(0xE3, 0xD8, 0xCD)),
							new ColorPickerColor("CW038W Apple Delight", Color.FromArgb(0xEB, 0xE2, 0xD7)),
							new ColorPickerColor("CW039W Coconut Colada", Color.FromArgb(0xEE, 0xE8, 0xDB)),
							new ColorPickerColor("CW040W Essence", Color.FromArgb(0xE2, 0xD6, 0xCF)),
							new ColorPickerColor("CW041W Poetic White", Color.FromArgb(0xDF, 0xD7, 0xCF)),
							new ColorPickerColor("CW042W Grape Freeze", Color.FromArgb(0xEF, 0xE7, 0xDD)),
							new ColorPickerColor("CW043W Grape Reflection", Color.FromArgb(0xE8, 0xE4, 0xDE)),
							new ColorPickerColor("CW044W Grape OlГc", Color.FromArgb(0xE4, 0xE0, 0xD7)),
							new ColorPickerColor("CW045W Grape Beginnings", Color.FromArgb(0xEB, 0xE7, 0xDD)),
							new ColorPickerColor("CW046W Enchanting", Color.FromArgb(0xE1, 0xDC, 0xD4)),
							new ColorPickerColor("CW047W Hidden White", Color.FromArgb(0xDF, 0xDB, 0xD1)),
							new ColorPickerColor("CW049W Composed", Color.FromArgb(0xE7, 0xE7, 0xDE)),
							new ColorPickerColor("CW049W Light Antique", Color.FromArgb(0xD9, 0xD0, 0xC2)),
							new ColorPickerColor("CW050W Plum Black White", Color.FromArgb(0xE3, 0xDF, 0xD3)),
							new ColorPickerColor("CW051W Solo White", Color.FromArgb(0xE7, 0xE6, 0xDD)),
							new ColorPickerColor("CW052W Subtly Natural", Color.FromArgb(0xDD, 0xD6, 0xC6)),
							new ColorPickerColor("CW053W Natural Bisque", Color.FromArgb(0xE1, 0xDC, 0xCB)),
							new ColorPickerColor("CW054W Natural Echo", Color.FromArgb(0xF1, 0xED, 0xDC)),
							new ColorPickerColor("CW055W Honeywind", Color.FromArgb(0xDF, 0xD6, 0xC3)),
							new ColorPickerColor("CW056W Moonlit", Color.FromArgb(0xE4, 0xDD, 0xCA)),
							new ColorPickerColor("CW057W White Solitude", Color.FromArgb(0xE7, 0xE3, 0xD4)),
							new ColorPickerColor("CW058W Dove Wing", Color.FromArgb(0xDE, 0xD8, 0xC6)),
							new ColorPickerColor("CW059W Fog Mist", Color.FromArgb(0xE6, 0xE0, 0xCD)),
							new ColorPickerColor("CW060W Sea Cloud", Color.FromArgb(0xEA, 0xE6, 0xD6)),
							new ColorPickerColor("CW061W Lime Peel", Color.FromArgb(0xE0, 0xDE, 0xCE)),
							new ColorPickerColor("CW062W Delicate Aire", Color.FromArgb(0xE5, 0xE3, 0xD4)),
							new ColorPickerColor("CW063W Juliet's White", Color.FromArgb(0xE9, 0xE9, 0xD9)),
							new ColorPickerColor("CW064W White Pebble", Color.FromArgb(0xD9, 0xD5, 0xC8)),
							new ColorPickerColor("CW065W Camelle", Color.FromArgb(0xED, 0xEB, 0xDC)),
							new ColorPickerColor("CW066W Picket White", Color.FromArgb(0xE8, 0xE6, 0xDA)),
							new ColorPickerColor("CW067W Soft Mist", Color.FromArgb(0xD5, 0xD4, 0xC9)),
							new ColorPickerColor("CW068W Intimate White", Color.FromArgb(0xE1, 0xDE, 0xCF)),
							new ColorPickerColor("CW069W Respectfully White", Color.FromArgb(0xDE, 0xDC, 0xD0)),
							new ColorPickerColor("CW070W Plum Black Ashes", Color.FromArgb(0xE0, 0xDD, 0xD2)),
							new ColorPickerColor("CW071W Plum Black Absence", Color.FromArgb(0xE1, 0xE0, 0xD7)),
							new ColorPickerColor("CW072W Plum Black Whisper", Color.FromArgb(0xE6, 0xE6, 0xDA)),
						}),
				}),
*/
			#endregion

			#region Harmonies
/*
			new ColorPickerColorPalette(
				"Harmonies", "cbn_harmonies",
				new ColorPickerColorGroup[]
				{
					new ColorPickerColorGroup(
						"",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Hollyhock Red A01", Color.FromArgb(0x8D, 0x4A, 0x4D)),
							new ColorPickerColor("Chenille A18", Color.FromArgb(0xEE, 0xD2, 0xB7)),
							new ColorPickerColor("Pink Bonnet A31", Color.FromArgb(0xE8, 0xDA, 0xE0)),
							new ColorPickerColor("Wisteria B13", Color.FromArgb(0xB2, 0xC0, 0xD2)),
							new ColorPickerColor("Lily Of The Valley B21", Color.FromArgb(0xE6, 0xE5, 0xDF)),
							new ColorPickerColor("Atlantic B29", Color.FromArgb(0xB5, 0xC1, 0xB6)),
							new ColorPickerColor("Hosta Green C16", Color.FromArgb(0x66, 0x84, 0x81)),
							new ColorPickerColor("Venetian Glass C24", Color.FromArgb(0x9E, 0xCA, 0xCA)),
							new ColorPickerColor("Shadow Box Blue C32", Color.FromArgb(0xA9, 0xD4, 0xDE)),
							new ColorPickerColor("Dried Hydrangea D15", Color.FromArgb(0xB8, 0xBB, 0x93)),
							new ColorPickerColor("Water Green D22", Color.FromArgb(0xE2, 0xEA, 0xD2)),
							new ColorPickerColor("Tall Taper Green D31", Color.FromArgb(0xA8, 0xC1, 0xAF)),
							new ColorPickerColor("Snapdragon E12", Color.FromArgb(0xF5, 0xD4, 0x8A)),
							new ColorPickerColor("Climbing Rose E21", Color.FromArgb(0xED, 0xDF, 0xAB)),
							new ColorPickerColor("Corn Husk E32", Color.FromArgb(0xD9, 0xCD, 0x9A)),
							new ColorPickerColor("Timber F15", Color.FromArgb(0x8F, 0x78, 0x5D)),
							new ColorPickerColor("Washed Khaki F25", Color.FromArgb(0xB5, 0xA9, 0x85)),
							new ColorPickerColor("Westport Tan G01", Color.FromArgb(0xA7, 0x9D, 0x86)),
							new ColorPickerColor("Granite G11", Color.FromArgb(0x9A, 0x9B, 0x97)),
							new ColorPickerColor("Pongee G20", Color.FromArgb(0xC1, 0xB1, 0x93)),
							new ColorPickerColor("Chalk G32", Color.FromArgb(0xDA, 0xD7, 0xCD)),
							new ColorPickerColor("Totilla H07", Color.FromArgb(0xD7, 0xCC, 0xB9)),
							new ColorPickerColor("Bee Balm Red A02", Color.FromArgb(0x97, 0x4A, 0x4B)),
							new ColorPickerColor("Magnolia A19", Color.FromArgb(0xF0, 0xDD, 0xC8)),
							new ColorPickerColor("Aster Pink A32", Color.FromArgb(0xD7, 0xCC, 0xD7)),
							new ColorPickerColor("Bluestar B14", Color.FromArgb(0xB4, 0xCB, 0xDC)),
							new ColorPickerColor("Pale Sky B22", Color.FromArgb(0xD9, 0xE3, 0xDF)),
							new ColorPickerColor("Siamese Eyes B31", Color.FromArgb(0xAF, 0xCD, 0xCF)),
							new ColorPickerColor("Glider Green C17", Color.FromArgb(0x77, 0xA0, 0xA0)),
							new ColorPickerColor("Robin's Egg Blue C25", Color.FromArgb(0xA6, 0xD0, 0xD1)),
							new ColorPickerColor("Lily Pond Blue D04", Color.FromArgb(0x86, 0xB5, 0xAB)),
							new ColorPickerColor("Seeding D16", Color.FromArgb(0xBC, 0xBD, 0x8C)),
							new ColorPickerColor("First Frost D23", Color.FromArgb(0xDC, 0xE0, 0xD2)),
							new ColorPickerColor("Sundial E01", Color.FromArgb(0xAD, 0x90, 0x65)),
							new ColorPickerColor("Goat's Beard E14", Color.FromArgb(0xE2, 0xCA, 0x98)),
							new ColorPickerColor("Primrose E22", Color.FromArgb(0xED, 0xDE, 0x9D)),
							new ColorPickerColor("Morel F03", Color.FromArgb(0x5C, 0x57, 0x51)),
							new ColorPickerColor("Suede Gray F16", Color.FromArgb(0x8A, 0x84, 0x73)),
							new ColorPickerColor("Luminaria F26", Color.FromArgb(0xB7, 0xA5, 0x78)),
							new ColorPickerColor("Clay G02", Color.FromArgb(0x9F, 0x9E, 0x8D)),
							new ColorPickerColor("Bluestone G12", Color.FromArgb(0x8C, 0x96, 0x91)),
							new ColorPickerColor("Deep Linen G22", Color.FromArgb(0xCC, 0xB6, 0x90)),
							new ColorPickerColor("Georgica Beach H01", Color.FromArgb(0xC1, 0xB5, 0x9B)),
							new ColorPickerColor("Sandcastle H08", Color.FromArgb(0xE1, 0xD1, 0xB4)),
							new ColorPickerColor("Brick Path A03", Color.FromArgb(0x9A, 0x54, 0x4C)),
							new ColorPickerColor("Viburnum A20", Color.FromArgb(0xEA, 0xD5, 0xC3)),
							new ColorPickerColor("Copper Beech B02", Color.FromArgb(0x67, 0x49, 0x45)),
							new ColorPickerColor("Lilac B15", Color.FromArgb(0xB6, 0xC7, 0xDF)),
							new ColorPickerColor("Full Moon B23", Color.FromArgb(0xDB, 0xE6, 0xE2)),
							new ColorPickerColor("Aquatic Green B32", Color.FromArgb(0xAB, 0xBF, 0xBC)),
							new ColorPickerColor("Tree Swing Green C18", Color.FromArgb(0x77, 0x99, 0x97)),
							new ColorPickerColor("Porch Ceiling Blue C26", Color.FromArgb(0xB0, 0xC9, 0xD0)),
							new ColorPickerColor("Strawberry Pot Blue D05", Color.FromArgb(0x93, 0xAE, 0xA3)),
							new ColorPickerColor("Shagreen D17", Color.FromArgb(0xC4, 0xC7, 0xA7)),
							new ColorPickerColor("Beach Glass D24", Color.FromArgb(0xD7, 0xE9, 0xE0)),
							new ColorPickerColor("Teasel E02", Color.FromArgb(0xBD, 0x96, 0x69)),
							new ColorPickerColor("Oat Straw E15", Color.FromArgb(0xEC, 0xCD, 0x9A)),
							new ColorPickerColor("Daylily E23", Color.FromArgb(0xF1, 0xD9, 0x8B)),
							new ColorPickerColor("Peat F05", Color.FromArgb(0x5E, 0x5C, 0x50)),
							new ColorPickerColor("Pinon Seed F18", Color.FromArgb(0xA6, 0x8C, 0x74)),
							new ColorPickerColor("Tea Bath F27", Color.FromArgb(0xC1, 0xAA, 0x83)),
							new ColorPickerColor("Comfrey G03", Color.FromArgb(0x8D, 0x98, 0x86)),
							new ColorPickerColor("Twine G14", Color.FromArgb(0xB0, 0x9D, 0x87)),
							new ColorPickerColor("Winter Surf G23", Color.FromArgb(0xC9, 0xC3, 0xAA)),
							new ColorPickerColor("Buckwheat H02", Color.FromArgb(0xC5, 0xB6, 0x9C)),
							new ColorPickerColor("Caneware H09", Color.FromArgb(0xE8, 0xDF, 0xBD)),
							new ColorPickerColor("Flower Pot A04", Color.FromArgb(0xB6, 0x69, 0x56)),
							new ColorPickerColor("Wild Rose A21", Color.FromArgb(0xEE, 0xCC, 0xB7)),
							new ColorPickerColor("Hydrangea Purple B06", Color.FromArgb(0x76, 0x85, 0x9A)),
							new ColorPickerColor("Blue Phlox B16", Color.FromArgb(0xC7, 0xD3, 0xE0)),
							new ColorPickerColor("Whitecap B24", Color.FromArgb(0xD8, 0xDF, 0xD8)),
							new ColorPickerColor("Silver Blue C01", Color.FromArgb(0x77, 0x8F, 0x98)),
							new ColorPickerColor("Delphinium C19", Color.FromArgb(0x79, 0xA6, 0xBB)),
							new ColorPickerColor("Forget-me-not C27", Color.FromArgb(0xB1, 0xCB, 0xD6)),
							new ColorPickerColor("Old Green Enamel D06", Color.FromArgb(0x88, 0xAA, 0x88)),
							new ColorPickerColor("Hooked Rug Green D18", Color.FromArgb(0xCC, 0xCD, 0xB5)),
							new ColorPickerColor("Sunshower D25", Color.FromArgb(0xD8, 0xE7, 0xDB)),
							new ColorPickerColor("Beeswax E05", Color.FromArgb(0xD5, 0xAF, 0x82)),
							new ColorPickerColor("Corn Silk E16", Color.FromArgb(0xF2, 0xD6, 0xAA)),
							new ColorPickerColor("Lemonade E24", Color.FromArgb(0xED, 0xDF, 0x95)),
							new ColorPickerColor("Mesquite F06", Color.FromArgb(0x66, 0x5E, 0x4E)),
							new ColorPickerColor("Corkscrew Willow F19", Color.FromArgb(0xAD, 0x93, 0x79)),
							new ColorPickerColor("Fresh Hay F28", Color.FromArgb(0xC9, 0xB6, 0x87)),
							new ColorPickerColor("Rue G04", Color.FromArgb(0xA2, 0xAE, 0x9C)),
							new ColorPickerColor("Weimaraner G15", Color.FromArgb(0xAA, 0x9C, 0x8C)),
							new ColorPickerColor("Conch G24", Color.FromArgb(0xD3, 0xB4, 0x9B)),
							new ColorPickerColor("Homespun H03", Color.FromArgb(0xCA, 0xC2, 0xAD)),
							new ColorPickerColor("Ivory H10", Color.FromArgb(0xED, 0xEA, 0xD2)),
							new ColorPickerColor("Sandstone A05", Color.FromArgb(0xB5, 0x78, 0x65)),
							new ColorPickerColor("Quince Blossom A23", Color.FromArgb(0xE6, 0xB5, 0xA3)),
							new ColorPickerColor("Thyme Flower B09", Color.FromArgb(0x96, 0x8B, 0x92)),
							new ColorPickerColor("First Crocus B17", Color.FromArgb(0xD1, 0xD5, 0xDD)),
							new ColorPickerColor("Drop Of Blue B25", Color.FromArgb(0xCB, 0xE0, 0xE2)),
							new ColorPickerColor("North Sky Blue C02", Color.FromArgb(0x6C, 0x85, 0x8F)),
							new ColorPickerColor("Cornflower C20", Color.FromArgb(0x83, 0xAB, 0xC0)),
							new ColorPickerColor("Bluebell C28", Color.FromArgb(0xB9, 0xCC, 0xD6)),
							new ColorPickerColor("Verdigris D07", Color.FromArgb(0x93, 0xB1, 0x9A)),
							new ColorPickerColor("Hooked Rug D18", Color.FromArgb(0xCC, 0xCD, 0xB5)),
							new ColorPickerColor("Lichen D26", Color.FromArgb(0xCF, 0xE2, 0xD1)),
							new ColorPickerColor("Pebblestone E06", Color.FromArgb(0xDD, 0xBE, 0x95)),
							new ColorPickerColor("Firefly E17", Color.FromArgb(0xF0, 0xE0, 0xAE)),
							new ColorPickerColor("Hellebore E27", Color.FromArgb(0xC9, 0xBF, 0x88)),
							new ColorPickerColor("Goose Down F09", Color.FromArgb(0x85, 0x77, 0x6A)),
							new ColorPickerColor("Coffee F20", Color.FromArgb(0xA3, 0x8D, 0x6C)),
							new ColorPickerColor("Sea Grass F29", Color.FromArgb(0xC7, 0xBB, 0x98)),
							new ColorPickerColor("Willow G05", Color.FromArgb(0x9C, 0xA3, 0x94)),
							new ColorPickerColor("Tumbleweed G16", Color.FromArgb(0xAE, 0x9E, 0x88)),
							new ColorPickerColor("Quahaug G27", Color.FromArgb(0xC7, 0xBF, 0xB2)),
							new ColorPickerColor("Stone H04", Color.FromArgb(0xC8, 0xBC, 0xA3)),
							new ColorPickerColor("Muslin H11", Color.FromArgb(0xE4, 0xDA, 0xC7)),
							new ColorPickerColor("Peachtree A13", Color.FromArgb(0xE9, 0xC2, 0xA1)),
							new ColorPickerColor("Pearl Pink A26", Color.FromArgb(0xE9, 0xC9, 0xC5)),
							new ColorPickerColor("June Iris B10", Color.FromArgb(0x8B, 0x9D, 0xB5)),
							new ColorPickerColor("Lupine B18", Color.FromArgb(0xD0, 0xD6, 0xE1)),
							new ColorPickerColor("Ice Water B26", Color.FromArgb(0xCC, 0xE1, 0xE6)),
							new ColorPickerColor("Mexican Tile Green C11", Color.FromArgb(0x51, 0x86, 0x75)),
							new ColorPickerColor("Hydrangea Blue C21", Color.FromArgb(0x90, 0xB0, 0xC5)),
							new ColorPickerColor("Bluepoint C29", Color.FromArgb(0xB6, 0xD5, 0xE1)),
							new ColorPickerColor("American Green D11", Color.FromArgb(0xA6, 0xBB, 0x9B)),
							new ColorPickerColor("Celadon D19", Color.FromArgb(0xC4, 0xCC, 0xB5)),
							new ColorPickerColor("Batter Bowl Green D27", Color.FromArgb(0xBC, 0xD7, 0xC8)),
							new ColorPickerColor("Yarrow E08", Color.FromArgb(0xDA, 0xAE, 0x62)),
							new ColorPickerColor("Heirloom Rose E18", Color.FromArgb(0xED, 0xE0, 0xB4)),
							new ColorPickerColor("Dill Flower E28", Color.FromArgb(0xCF, 0xC0, 0x87)),
							new ColorPickerColor("Late Summer Leaf F12", Color.FromArgb(0x75, 0x76, 0x5E)),
							new ColorPickerColor("New Khaki F21", Color.FromArgb(0xA2, 0x95, 0x72)),
							new ColorPickerColor("Marzipan F30", Color.FromArgb(0xC3, 0xB9, 0x92)),
							new ColorPickerColor("Jasper G06", Color.FromArgb(0xA1, 0xA4, 0x8C)),
							new ColorPickerColor("Pearl Gray G17", Color.FromArgb(0xC0, 0xB4, 0xA4)),
							new ColorPickerColor("Driftwood G28", Color.FromArgb(0xB6, 0xAF, 0xA7)),
							new ColorPickerColor("Travertine H05", Color.FromArgb(0xCF, 0xC1, 0xAC)),
							new ColorPickerColor("Raw Silk H12", Color.FromArgb(0xF6, 0xE4, 0xC5)),
							new ColorPickerColor("Plaster Pink A14", Color.FromArgb(0xE3, 0xC4, 0xA9)),
							new ColorPickerColor("Cherry Blossom A27", Color.FromArgb(0xF3, 0xCE, 0xBF)),
							new ColorPickerColor("Pale Violet B11", Color.FromArgb(0x91, 0x9E, 0xAB)),
							new ColorPickerColor("Veronica B19", Color.FromArgb(0xDA, 0xD9, 0xDC)),
							new ColorPickerColor("Kitchen Ceiling Blue B27", Color.FromArgb(0xB7, 0xD4, 0xCE)),
							new ColorPickerColor("Thunderclap C14", Color.FromArgb(0x6D, 0x7E, 0x74)),
							new ColorPickerColor("Morning Glory C22", Color.FromArgb(0x8F, 0xB6, 0xD5)),
							new ColorPickerColor("Himalayan Eyes C30", Color.FromArgb(0xB4, 0xD4, 0xDD)),
							new ColorPickerColor("Seaweed D13", Color.FromArgb(0xAE, 0xB0, 0x88)),
							new ColorPickerColor("Glass Green D20", Color.FromArgb(0xDC, 0xE3, 0xCC)),
							new ColorPickerColor("Plate Green D29", Color.FromArgb(0x9D, 0xBE, 0xAD)),
							new ColorPickerColor("Sunflower E10", Color.FromArgb(0xFD, 0xC2, 0x59)),
							new ColorPickerColor("Yellow Rose E19", Color.FromArgb(0xF4, 0xE3, 0xA1)),
							new ColorPickerColor("New Bamboo E30", Color.FromArgb(0xD8, 0xBF, 0x86)),
							new ColorPickerColor("Pine Needle F13", Color.FromArgb(0x85, 0x84, 0x6F)),
							new ColorPickerColor("Arbor Green F23", Color.FromArgb(0xA1, 0x96, 0x77)),
							new ColorPickerColor("Everyday Green F31", Color.FromArgb(0xCB, 0xC3, 0x9E)),
							new ColorPickerColor("Quaking Aspen G07", Color.FromArgb(0xAF, 0xB2, 0xA0)),
							new ColorPickerColor("Light Cocoa G18", Color.FromArgb(0xC6, 0xB5, 0xA0)),
							new ColorPickerColor("Georgica Mist G30", Color.FromArgb(0xC9, 0xC7, 0xBB)),
							new ColorPickerColor("Scone H06", Color.FromArgb(0xD7, 0xC7, 0xAB)),
							new ColorPickerColor("Pale Naples Yellow H13", Color.FromArgb(0xF5, 0xE3, 0xB9)),
							new ColorPickerColor("Honeysuckle A15", Color.FromArgb(0xE6, 0xC5, 0x9E)),
							new ColorPickerColor("Paris Pink A29", Color.FromArgb(0xF6, 0xDE, 0xD4)),
							new ColorPickerColor("Lacecap B12", Color.FromArgb(0xAF, 0xBC, 0xC5)),
							new ColorPickerColor("Edelweiss B20", Color.FromArgb(0xD9, 0xDE, 0xE3)),
							new ColorPickerColor("Mineral Green B28", Color.FromArgb(0xBF, 0xCB, 0xC2)),
							new ColorPickerColor("Book Binding Green C15", Color.FromArgb(0x81, 0x94, 0x86)),
							new ColorPickerColor("Light Blue Grosgrain C23", Color.FromArgb(0xA1, 0xC8, 0xD0)),
							new ColorPickerColor("Morning Room Blue C31", Color.FromArgb(0xA9, 0xCD, 0xD7)),
							new ColorPickerColor("Green Tea D14", Color.FromArgb(0xAA, 0xAE, 0x8D)),
							new ColorPickerColor("Drop of Green D21", Color.FromArgb(0xE2, 0xEC, 0xDF)),
							new ColorPickerColor("Lamb's Ear D30", Color.FromArgb(0xAD, 0xBC, 0xAB)),
							new ColorPickerColor("Cornmeal E11", Color.FromArgb(0xEB, 0xCA, 0x85)),
							new ColorPickerColor("Tea Rose E20", Color.FromArgb(0xF1, 0xDC, 0xA5)),
							new ColorPickerColor("Squash Blossom E31", Color.FromArgb(0xE0, 0xCC, 0x97)),
							new ColorPickerColor("Potato Peel F14", Color.FromArgb(0x85, 0x73, 0x5F)),
							new ColorPickerColor("Westport Green F24", Color.FromArgb(0xAA, 0xA4, 0x8D)),
							new ColorPickerColor("Meadow Way F32", Color.FromArgb(0xDD, 0xDC, 0xB8)),
							new ColorPickerColor("Artemisia G08", Color.FromArgb(0x97, 0xA8, 0xA4)),
							new ColorPickerColor("Coir G19", Color.FromArgb(0xB5, 0xA6, 0x89)),
							new ColorPickerColor("Mercury Glass G31", Color.FromArgb(0xC9, 0xC7, 0xBF)),
							new ColorPickerColor("Tortilla H07", Color.FromArgb(0xD4, 0xCA, 0xB8)),
							new ColorPickerColor("Buttercream H14", Color.FromArgb(0xF2, 0xE5, 0xC5)),
							new ColorPickerColor("Sand Dollar H15", Color.FromArgb(0xF1, 0xE1, 0xC7)),
							new ColorPickerColor("Vellum H22", Color.FromArgb(0xEE, 0xE7, 0xD0)),
							new ColorPickerColor("Linen White H29", Color.FromArgb(0xF0, 0xE4, 0xC6)),
							new ColorPickerColor("Grandiflora H16", Color.FromArgb(0xEF, 0xDE, 0xC7)),
							new ColorPickerColor("Ursa Major H23", Color.FromArgb(0xEF, 0xE2, 0xC4)),
							new ColorPickerColor("Magnolia White H30", Color.FromArgb(0xF3, 0xEC, 0xDB)),
							new ColorPickerColor("Dogwood Blossom H17", Color.FromArgb(0xF0, 0xE2, 0xCC)),
							new ColorPickerColor("Butter White H24", Color.FromArgb(0xF2, 0xE9, 0xD0)),
							new ColorPickerColor("Ironston White H31", Color.FromArgb(0xE9, 0xE8, 0xDD)),
							new ColorPickerColor("Ursa Minor H18", Color.FromArgb(0xEB, 0xDF, 0xC3)),
							new ColorPickerColor("Sugar White H25", Color.FromArgb(0xED, 0xE9, 0xDA)),
							new ColorPickerColor("Ironstone White H31", Color.FromArgb(0xE8, 0xE7, 0xDE)),
							new ColorPickerColor("Camellia H19", Color.FromArgb(0xED, 0xE4, 0xD0)),
							new ColorPickerColor("Daisy White H26", Color.FromArgb(0xF2, 0xED, 0xDF)),
							new ColorPickerColor("Bright White H32", Color.FromArgb(0xEE, 0xEE, 0xE4)),
							new ColorPickerColor("Autumn Clematis H20", Color.FromArgb(0xEE, 0xE9, 0xD9)),
							new ColorPickerColor("Rose Tinted White H27", Color.FromArgb(0xF4, 0xE8, 0xD9)),
							new ColorPickerColor("Bright Light H32", Color.FromArgb(0xEE, 0xEE, 0xE5)),
							new ColorPickerColor("Wicker White H21", Color.FromArgb(0xEF, 0xE9, 0xD4)),
							new ColorPickerColor("Fine White H28", Color.FromArgb(0xF5, 0xF0, 0xE1)),
						}),
				}),
*/
			#endregion

			#region Disney
/*
			new ColorPickerColorPalette(
				"Disney", "cbn_disney",
				new ColorPickerColorGroup[]
				{
					new ColorPickerColorGroup(
						"",
						new ColorPickerColor[]
						{
							new ColorPickerColor("1A-10-1 New Day Dew", Color.FromArgb(0xF8, 0xF4, 0xE3)),
							new ColorPickerColor("1A-10-2 Dandelion for Roo", Color.FromArgb(0xF9, 0xE9, 0xC1)),
							new ColorPickerColor("1A-20-1 Brand New Morning", Color.FromArgb(0xFA, 0xF0, 0xD5)),
							new ColorPickerColor("1A-20-2 Baby Bee Yellow", Color.FromArgb(0xF8, 0xE5, 0xB7)),
							new ColorPickerColor("1A-30-1 Hums in the Sun", Color.FromArgb(0xFA, 0xF1, 0xDB)),
							new ColorPickerColor("1A-30-2 Pooh Bear Yellow", Color.FromArgb(0xFF, 0xDE, 0xA4)),
							new ColorPickerColor("1A-40-1 100 Acre Snowfall", Color.FromArgb(0xF8, 0xEC, 0xE0)),
							new ColorPickerColor("1A-40-2 Fluff and Stuff", Color.FromArgb(0xED, 0xCE, 0xB5)),
							new ColorPickerColor("1A-50-1 Baby Roo", Color.FromArgb(0xF8, 0xDC, 0xC5)),
							new ColorPickerColor("1A-50-2 Buzz Beta Sector", Color.FromArgb(0xD1, 0xA3, 0x80)),
							new ColorPickerColor("1A-60-1 Kissed Awake", Color.FromArgb(0xF8, 0xE6, 0xDA)),
							new ColorPickerColor("1A-60-2 Blushing Belle", Color.FromArgb(0xFE, 0xD9, 0xC3)),
							new ColorPickerColor("1A-70-1 Velvet Glove", Color.FromArgb(0xE7, 0xD7, 0xCF)),
							new ColorPickerColor("1A-70-2 Glamorous Glow", Color.FromArgb(0xFE, 0xB8, 0x9C)),
							new ColorPickerColor("1A-80-1 Whole New World", Color.FromArgb(0xF8, 0xEB, 0xE2)),
							new ColorPickerColor("1A-80-2 Blushing Princess", Color.FromArgb(0xFD, 0xC3, 0xB3)),
							new ColorPickerColor("1B-10 Sunny Spot", Color.FromArgb(0xFF, 0xD3, 0x45)),
							new ColorPickerColor("1B-20 Laser Beam", Color.FromArgb(0xFF, 0xBB, 0x00)),
							new ColorPickerColor("1B-30 Smackeral of Hunny", Color.FromArgb(0xFF, 0xB4, 0x49)),
							new ColorPickerColor("1B-40 Rumbly Tumbly", Color.FromArgb(0xFD, 0x87, 0x26)),
							new ColorPickerColor("1B-50 Orange Blast", Color.FromArgb(0xF5, 0x7C, 0x35)),
							new ColorPickerColor("1B-60 Bouncy Councy", Color.FromArgb(0xEA, 0x63, 0x2B)),
							new ColorPickerColor("1B-70 Light Speed Orange", Color.FromArgb(0xDB, 0x4F, 0x2F)),
							new ColorPickerColor("1B-80 Gamma Rays", Color.FromArgb(0xB7, 0x4C, 0x36)),
							new ColorPickerColor("1C-10-3 Main Street Lights", Color.FromArgb(0xFF, 0xE6, 0x92)),
							new ColorPickerColor("1C-10-4 Blast Off Yellow", Color.FromArgb(0xFC, 0xD0, 0x18)),
							new ColorPickerColor("1C-20-3 Tiggers Tummy", Color.FromArgb(0xFF, 0xDE, 0x92)),
							new ColorPickerColor("1C-20-4 Bee a Friend", Color.FromArgb(0xFF, 0xC6, 0x46)),
							new ColorPickerColor("1C-30-3 Ho Hum Yellow", Color.FromArgb(0xF8, 0xBA, 0x61)),
							new ColorPickerColor("1C-30-4 Get Goofy", Color.FromArgb(0xDD, 0x92, 0x29)),
							new ColorPickerColor("1C-40-3 Pluto", Color.FromArgb(0xF6, 0xBA, 0x7C)),
							new ColorPickerColor("1C-40-4 Oh Boing", Color.FromArgb(0xEF, 0x8E, 0x20)),
							new ColorPickerColor("1C-50-3 Muddlement", Color.FromArgb(0xB1, 0x79, 0x50)),
							new ColorPickerColor("1C-50-4 Haycorn Brown", Color.FromArgb(0x94, 0x65, 0x45)),
							new ColorPickerColor("1C-60-3 Donald Orange", Color.FromArgb(0xF3, 0x95, 0x52)),
							new ColorPickerColor("1C-60-4 Kanga Orange", Color.FromArgb(0xBD, 0x6A, 0x2F)),
							new ColorPickerColor("1C-70-3 Forest Sunset", Color.FromArgb(0xFD, 0x9D, 0x72)),
							new ColorPickerColor("1C-70-4 Tiggerific", Color.FromArgb(0xE2, 0x5C, 0x30)),
							new ColorPickerColor("1C-80-3 100 Acre Woods", Color.FromArgb(0xD3, 0x6B, 0x50)),
							new ColorPickerColor("1C-80-4 Deep in the Woods", Color.FromArgb(0x70, 0x44, 0x38)),
							new ColorPickerColor("2A-10-1 Butterfly Flutter By", Color.FromArgb(0xDB, 0xDB, 0xE6)),
							new ColorPickerColor("2A-10-2 One Enchanted Evening", Color.FromArgb(0xB2, 0xAE, 0xD4)),
							new ColorPickerColor("2A-20-1 Clock Strikes 12", Color.FromArgb(0xE0, 0xD4, 0xE4)),
							new ColorPickerColor("2A-20-2 A Wave of the Wand", Color.FromArgb(0xCB, 0xAB, 0xCB)),
							new ColorPickerColor("2A-30-1 Pixie Dust", Color.FromArgb(0xEF, 0xE4, 0xE9)),
							new ColorPickerColor("2A-30-2 Enchanted Princess", Color.FromArgb(0xEC, 0xC2, 0xDD)),
							new ColorPickerColor("2A-40-1 Fairest of Them All", Color.FromArgb(0xF2, 0xE1, 0xE9)),
							new ColorPickerColor("2A-40-2 Pretty in Pink", Color.FromArgb(0xED, 0xC1, 0xD6)),
							new ColorPickerColor("2A-50-1 Princess Tiara", Color.FromArgb(0xF6, 0xEE, 0xED)),
							new ColorPickerColor("2A-50-2 Fairytale Pink", Color.FromArgb(0xF3, 0xB9, 0xCB)),
							new ColorPickerColor("2A-60-1 Shy Little Piglet", Color.FromArgb(0xEB, 0xBE, 0xC1)),
							new ColorPickerColor("2A-60-2 Snow White Song", Color.FromArgb(0xCC, 0x82, 0x8D)),
							new ColorPickerColor("2A-70-1 Princess Bouquet", Color.FromArgb(0xEE, 0xDA, 0xDB)),
							new ColorPickerColor("2A-70-2 Blushing Piglet", Color.FromArgb(0xEB, 0x80, 0x8C)),
							new ColorPickerColor("2A-80-1 Belle of the Ball", Color.FromArgb(0xF4, 0xE2, 0xDB)),
							new ColorPickerColor("2A-80-2 Make Believe", Color.FromArgb(0xF7, 0xAC, 0x9E)),
							new ColorPickerColor("2B-10 Prince Charming", Color.FromArgb(0x49, 0x3F, 0x5E)),
							new ColorPickerColor("2B-20 Princess Purple", Color.FromArgb(0x98, 0x66, 0x98)),
							new ColorPickerColor("2B-30 Happily Ever After", Color.FromArgb(0xA9, 0x50, 0x7F)),
							new ColorPickerColor("2B-40 Grand Ball", Color.FromArgb(0x6B, 0x3D, 0x59)),
							new ColorPickerColor("2B-50 Poohs Kite", Color.FromArgb(0x6D, 0x38, 0x4D)),
							new ColorPickerColor("2B-60 Three Cheers for Pooh", Color.FromArgb(0xB1, 0x3B, 0x4B)),
							new ColorPickerColor("2B-70 Code Red", Color.FromArgb(0xBD, 0x35, 0x3B)),
							new ColorPickerColor("2B-80 Mickeys Pants", Color.FromArgb(0xC4, 0x3A, 0x36)),
							new ColorPickerColor("2C-10-3 Dreams Come True", Color.FromArgb(0x8B, 0x7B, 0xA7)),
							new ColorPickerColor("2C-10-4 Robes for the Queen", Color.FromArgb(0x60, 0x4F, 0x7F)),
							new ColorPickerColor("2C-20-3 Enchanted Violet", Color.FromArgb(0xAE, 0x7D, 0xAB)),
							new ColorPickerColor("2C-20-4 Magical Evening", Color.FromArgb(0x85, 0x65, 0x86)),
							new ColorPickerColor("2C-30-3 Sleeping Beauty Pink", Color.FromArgb(0xE3, 0x9C, 0xBE)),
							new ColorPickerColor("2C-30-4 Pretty as a Princess", Color.FromArgb(0xCE, 0x75, 0xA7)),
							new ColorPickerColor("2C-40-3 Princess Power", Color.FromArgb(0xD8, 0x97, 0xAE)),
							new ColorPickerColor("2C-40-4 Enchanted Coach Ride", Color.FromArgb(0x8E, 0x4C, 0x70)),
							new ColorPickerColor("2C-50-3 Tink Pink", Color.FromArgb(0xCD, 0x67, 0x8B)),
							new ColorPickerColor("2C-50-4 Jasmines Gems", Color.FromArgb(0x8B, 0x45, 0x5C)),
							new ColorPickerColor("2C-60-3 Satin Ribbon", Color.FromArgb(0xCF, 0x5A, 0x75)),
							new ColorPickerColor("2C-60-4 Invitation to a Princess", Color.FromArgb(0xB8, 0x44, 0x60)),
							new ColorPickerColor("2C-70-3 Loves First Kiss", Color.FromArgb(0xD2, 0x51, 0x63)),
							new ColorPickerColor("2C-70-4 Fireflies Glow", Color.FromArgb(0x9F, 0x44, 0x54)),
							new ColorPickerColor("2C-80-3 Rabbits Garden", Color.FromArgb(0xEC, 0x79, 0x67)),
							new ColorPickerColor("2C-80-4 Owls House", Color.FromArgb(0x9D, 0x4F, 0x42)),
							new ColorPickerColor("3A-10-1 Cinderellas Glass Slipper", Color.FromArgb(0xD7, 0xDD, 0xEC)),
							new ColorPickerColor("3A-10-2 Moonlight Waltz", Color.FromArgb(0xA5, 0xB4, 0xD3)),
							new ColorPickerColor("3A-20-1 Heffalump Purple", Color.FromArgb(0xC7, 0xD4, 0xE9)),
							new ColorPickerColor("3A-20-2 Dragon Fly", Color.FromArgb(0x84, 0x9E, 0xCD)),
							new ColorPickerColor("3A-30-1 Mirror Mirror", Color.FromArgb(0xD4, 0xE1, 0xEC)),
							new ColorPickerColor("3A-30-2 Eeyore", Color.FromArgb(0xAB, 0xBC, 0xD0)),
							new ColorPickerColor("3A-40-1 Blustery Day", Color.FromArgb(0xD7, 0xE5, 0xEE)),
							new ColorPickerColor("3A-40-2 Bibbidi Bobbidi Blue", Color.FromArgb(0xB8, 0xD6, 0xE9)),
							new ColorPickerColor("3A-50-1 Silly Old Bluebirds", Color.FromArgb(0xC4, 0xE1, 0xE8)),
							new ColorPickerColor("3A-50-2 100 Acre Sky", Color.FromArgb(0x93, 0xC6, 0xDF)),
							new ColorPickerColor("3A-60-1 Mickey Sailor", Color.FromArgb(0xB8, 0xE1, 0xEF)),
							new ColorPickerColor("3A-60-2 100 Acre Morning", Color.FromArgb(0x90, 0xCC, 0xE2)),
							new ColorPickerColor("3A-70-1 Thoughtful Spot", Color.FromArgb(0xAA, 0xD8, 0xDE)),
							new ColorPickerColor("3A-70-2 Blue Balloon", Color.FromArgb(0x85, 0xC8, 0xCF)),
							new ColorPickerColor("3A-80-1 Bother Free Blue", Color.FromArgb(0xC6, 0xE8, 0xE5)),
							new ColorPickerColor("3A-80-2 Poohs Favorite Things", Color.FromArgb(0xA7, 0xDB, 0xD8)),
							new ColorPickerColor("3B-10 Warp Speed Blue", Color.FromArgb(0x47, 0x55, 0x89)),
							new ColorPickerColor("3B-20 Space Hero Blue", Color.FromArgb(0x33, 0x43, 0x67)),
							new ColorPickerColor("3B-30 Sorcerers Hat", Color.FromArgb(0x1C, 0x4E, 0x7A)),
							new ColorPickerColor("3B-40 Galactic Sky", Color.FromArgb(0x17, 0x5D, 0x95)),
							new ColorPickerColor("3B-50 Midnight Ball Blue", Color.FromArgb(0x0F, 0x66, 0x8E)),
							new ColorPickerColor("3B-60 Storytime Blue", Color.FromArgb(0x00, 0x6E, 0x99)),
							new ColorPickerColor("3B-70 Bottomless Blue", Color.FromArgb(0x17, 0x64, 0x78)),
							new ColorPickerColor("3B-80 Ariels Afternoon Swim", Color.FromArgb(0x00, 0x88, 0x95)),
							new ColorPickerColor("3C-10-3 Posies for Pooh", Color.FromArgb(0x91, 0x9F, 0xCC)),
							new ColorPickerColor("3C-10-4 Hunny Pot Purple", Color.FromArgb(0x6C, 0x78, 0xAD)),
							new ColorPickerColor("3C-20-3 Poohs Umbrella", Color.FromArgb(0x4A, 0x6D, 0xA6)),
							new ColorPickerColor("3C-20-4 Infinity and Beyond", Color.FromArgb(0x33, 0x5B, 0x92)),
							new ColorPickerColor("3C-30-3 Oh Bother Blue", Color.FromArgb(0x7C, 0xA5, 0xC5)),
							new ColorPickerColor("3C-30-4 Cast a Spell Blue", Color.FromArgb(0x39, 0x60, 0x8C)),
							new ColorPickerColor("3C-40-3 Cinderellas Ball Gown", Color.FromArgb(0x8B, 0xBE, 0xDD)),
							new ColorPickerColor("3C-40-4 Star Command Blue", Color.FromArgb(0x00, 0x7B, 0xB8)),
							new ColorPickerColor("3C-50-3 Fairy Godmother", Color.FromArgb(0x7A, 0xB0, 0xD1)),
							new ColorPickerColor("3C-50-4 Home Town Hero", Color.FromArgb(0x00, 0x88, 0xBC)),
							new ColorPickerColor("3C-60-3 Buzz Blaster Blue", Color.FromArgb(0x39, 0xAB, 0xCE)),
							new ColorPickerColor("3C-60-4 Blue Adventure", Color.FromArgb(0x00, 0x91, 0xB7)),
							new ColorPickerColor("3C-70-3 Blue Wonder", Color.FromArgb(0x28, 0xAF, 0xBB)),
							new ColorPickerColor("3C-70-4 Little Critters", Color.FromArgb(0x1A, 0x8D, 0x9A)),
							new ColorPickerColor("3C-80-3 Atta Boy Blue", Color.FromArgb(0x66, 0xB8, 0xB8)),
							new ColorPickerColor("3C-80-4 Underwater Wonders", Color.FromArgb(0x00, 0xA4, 0xA9)),
							new ColorPickerColor("4A-10-1 Eeyores Rain Cloud", Color.FromArgb(0xD7, 0xE3, 0xE1)),
							new ColorPickerColor("4A-10-2 Space Dust", Color.FromArgb(0xA5, 0xAF, 0xAE)),
							new ColorPickerColor("4A-20-1 Once Upon a Time", Color.FromArgb(0xEB, 0xEB, 0xE5)),
							new ColorPickerColor("4A-20-2 100 Acre Raindrops", Color.FromArgb(0xD0, 0xD3, 0xD1)),
							new ColorPickerColor("4A-30-1 Mickeys Gloves", Color.FromArgb(0xEB, 0xEB, 0xE6)),
							new ColorPickerColor("4A-30-2 Tiddely Pom", Color.FromArgb(0xE6, 0xE2, 0xD1)),
							new ColorPickerColor("4A-40-1 Skipping Stones", Color.FromArgb(0xF0, 0xF0, 0xD5)),
							new ColorPickerColor("4A-40-2 Christopher Robins Swing", Color.FromArgb(0xC2, 0xC5, 0xA0)),
							new ColorPickerColor("4A-50-1 Pooh Corner", Color.FromArgb(0xED, 0xF5, 0xDC)),
							new ColorPickerColor("4A-50-2 Tinker Bell", Color.FromArgb(0xD7, 0xEA, 0xB1)),
							new ColorPickerColor("4A-60-1 Silly Old Bear", Color.FromArgb(0xD5, 0xEB, 0xD1)),
							new ColorPickerColor("4A-60-2 By the Pond", Color.FromArgb(0xB5, 0xDC, 0xB4)),
							new ColorPickerColor("4A-70-1 Lullaby Green", Color.FromArgb(0xC6, 0xEE, 0xDB)),
							new ColorPickerColor("4A-70-2 Sea Song", Color.FromArgb(0x91, 0xD8, 0xBE)),
							new ColorPickerColor("4A-80-1 Woozle Green", Color.FromArgb(0xDD, 0xEC, 0xE4)),
							new ColorPickerColor("4A-80-2 Timeless Tales", Color.FromArgb(0xB7, 0xD9, 0xD0)),
							new ColorPickerColor("4B-10 Mouse Ears", Color.FromArgb(0x3B, 0x3B, 0x3B)),
							new ColorPickerColor("4B-20 BEHR Ultra Pure White", Color.FromArgb(0xF6, 0xF7, 0xF1)),
							new ColorPickerColor("4B-30 Alien Green", Color.FromArgb(0xC3, 0xC7, 0x28)),
							new ColorPickerColor("4B-40 Sonic Boom", Color.FromArgb(0x89, 0xB7, 0x37)),
							new ColorPickerColor("4B-50 Gamma Sector Green", Color.FromArgb(0x45, 0x9D, 0x43)),
							new ColorPickerColor("4B-60 Friendly Forest", Color.FromArgb(0x14, 0x72, 0x52)),
							new ColorPickerColor("4B-70 Blades of Grass", Color.FromArgb(0x05, 0x64, 0x59)),
							new ColorPickerColor("4B-80 Never Land", Color.FromArgb(0x00, 0x65, 0x66)),
							new ColorPickerColor("4C-10-3 Space Suit", Color.FromArgb(0x76, 0x83, 0x83)),
							new ColorPickerColor("4C-10-4 Galaxy Black", Color.FromArgb(0x34, 0x3C, 0x40)),
							new ColorPickerColor("4C-20-3 Mickeys Shadow", Color.FromArgb(0x90, 0x8F, 0x8C)),
							new ColorPickerColor("4C-20-4 Cenema Star", Color.FromArgb(0x42, 0x3F, 0x3C)),
							new ColorPickerColor("4C-30-3 Whistling Wind", Color.FromArgb(0xBC, 0xB5, 0xA0)),
							new ColorPickerColor("4C-30-4 Thistle Green", Color.FromArgb(0x8C, 0x87, 0x72)),
							new ColorPickerColor("4C-40-3 English Meadow", Color.FromArgb(0x92, 0xA0, 0x74)),
							new ColorPickerColor("4C-40-4 100 Acre Trees", Color.FromArgb(0x4D, 0x65, 0x3F)),
							new ColorPickerColor("4C-50-3 Friendly Dog", Color.FromArgb(0xAB, 0xD7, 0x8A)),
							new ColorPickerColor("4C-50-4 Springtime Grass", Color.FromArgb(0x70, 0xB1, 0x44)),
							new ColorPickerColor("4C-60-3 Green with Envy", Color.FromArgb(0x77, 0xCD, 0x94)),
							new ColorPickerColor("4C-60-4 Little Mermaid", Color.FromArgb(0x1F, 0x9A, 0x5B)),
							new ColorPickerColor("4C-70-3 Poohs Prairie", Color.FromArgb(0x00, 0x9D, 0x7B)),
							new ColorPickerColor("4C-70-4 Grand Day Green", Color.FromArgb(0x00, 0x90, 0x79)),
							new ColorPickerColor("4C-80-3 Ariels Song", Color.FromArgb(0x07, 0xAC, 0xA1)),
							new ColorPickerColor("4C-80-4 Mermaid Water Dance", Color.FromArgb(0x1C, 0x7C, 0x78)),
						}),
				}),
*/
			#endregion

			#region Ralph Lauren
/*
			new ColorPickerColorPalette(
				"Ralph Lauren", "cbn_ralph_lauren",
				new ColorPickerColorGroup[]
				{
					new ColorPickerColorGroup(
						"Desert Hollywood",
						new ColorPickerColor[]
						{
							new ColorPickerColor("DH02B Harlow's Slip", Color.FromArgb(0xD9, 0xC5, 0xAC)),
							new ColorPickerColor("DH02A San Simeon Grove", Color.FromArgb(0x7C, 0x65, 0x4E)),
							new ColorPickerColor("DH03B Mission Tile", Color.FromArgb(0x83, 0x45, 0x3C)),
							new ColorPickerColor("DH03C Desert Springs", Color.FromArgb(0xA4, 0xB1, 0xB1)),
							new ColorPickerColor("DH02D Director's Script", Color.FromArgb(0xCD, 0xBE, 0xA1)),
							new ColorPickerColor("DH01D Bankhead Blossom", Color.FromArgb(0x58, 0x40, 0x44)),
							new ColorPickerColor("DH04A Mulholland Terrace", Color.FromArgb(0x6E, 0x4C, 0x46)),
							new ColorPickerColor("DH01B Lombard's Rose", Color.FromArgb(0xA8, 0x84, 0x7B)),
							new ColorPickerColor("DH02C Sunset Drive", Color.FromArgb(0x97, 0x4E, 0x3C)),
							new ColorPickerColor("DH03A Pacific Balustrade", Color.FromArgb(0xBB, 0x94, 0x69)),
							new ColorPickerColor("DH03D Grant Gray", Color.FromArgb(0xA0, 0x93, 0x80)),
							new ColorPickerColor("DH04C Coldwater Canyon", Color.FromArgb(0x83, 0x66, 0x4D)),
							new ColorPickerColor("DH01A Dinner Jacket", Color.FromArgb(0xF1, 0xE2, 0xC8)),
							new ColorPickerColor("DH04B Coronado Dunes", Color.FromArgb(0xC6, 0x9B, 0x81)),
							new ColorPickerColor("DH04D Pavilion", Color.FromArgb(0xD0, 0xC1, 0xBC)),
							new ColorPickerColor("DH01C Courtyard Canopy", Color.FromArgb(0xB8, 0xAA, 0x94)),
						}),
					new ColorPickerColorGroup(
						"New England Lighthouse",
						new ColorPickerColor[]
						{
							new ColorPickerColor("LH02B Awning", Color.FromArgb(0x50, 0x5F, 0x48)),
							new ColorPickerColor("LH02D Sea Spray", Color.FromArgb(0x97, 0xBA, 0xC3)),
							new ColorPickerColor("LH04C Veranda", Color.FromArgb(0x5E, 0x7F, 0x6D)),
							new ColorPickerColor("LH01C Dockside", Color.FromArgb(0x58, 0x63, 0x6E)),
							new ColorPickerColor("LH01B Weathered Shingle", Color.FromArgb(0xC7, 0xC4, 0xBD)),
							new ColorPickerColor("LH01D Sundeck", Color.FromArgb(0xF5, 0xD3, 0x95)),
							new ColorPickerColor("LH02A Hammock", Color.FromArgb(0xEA, 0xDB, 0xC0)),
							new ColorPickerColor("LH04D Sand Pail", Color.FromArgb(0x80, 0x3E, 0x39)),
							new ColorPickerColor("LH03A Lakefront", Color.FromArgb(0x47, 0x68, 0x83)),
							new ColorPickerColor("LH01A Penny Candy", Color.FromArgb(0x98, 0x4D, 0x52)),
							new ColorPickerColor("LH04A Sail Boat", Color.FromArgb(0xC2, 0xCD, 0xC8)),
							new ColorPickerColor("LH03B Porch Swing", Color.FromArgb(0xC1, 0xB9, 0xAA)),
							new ColorPickerColor("LH03D Picnic Basket", Color.FromArgb(0xF1, 0xDE, 0xB2)),
							new ColorPickerColor("LH03C Yacht Club", Color.FromArgb(0x37, 0x45, 0x55)),
							new ColorPickerColor("LH04B Yarmouth", Color.FromArgb(0xE7, 0xAE, 0x59)),
							new ColorPickerColor("LH02C Bonfire", Color.FromArgb(0xE5, 0x84, 0x46)),
						}),
					new ColorPickerColorGroup(
						"New Zealand",
						new ColorPickerColor[]
						{
							new ColorPickerColor("NZ01C Lambswool", Color.FromArgb(0xC7, 0xB2, 0x92)),
							new ColorPickerColor("NZ04C Auckland", Color.FromArgb(0xA3, 0x99, 0x8A)),
							new ColorPickerColor("NZ04B Eucalyptus", Color.FromArgb(0x9C, 0x8D, 0x6F)),
							new ColorPickerColor("NZ03C Shearling", Color.FromArgb(0xCC, 0xB9, 0x95)),
							new ColorPickerColor("NZ04A Mountain Ice", Color.FromArgb(0xBF, 0xD0, 0xD3)),
							new ColorPickerColor("NZ04D Mount Cook", Color.FromArgb(0x50, 0x45, 0x3E)),
							new ColorPickerColor("NZ03D Forest Moss", Color.FromArgb(0x4B, 0x55, 0x3A)),
							new ColorPickerColor("NZ02D Anna", Color.FromArgb(0xDF, 0xCD, 0xAF)),
							new ColorPickerColor("NZ02A Myrtle", Color.FromArgb(0xA1, 0x81, 0x5C)),
							new ColorPickerColor("NZ02B Wellington", Color.FromArgb(0x92, 0x84, 0x74)),
							new ColorPickerColor("NZ0 Morning Shadow", Color.FromArgb(0x7E, 0x76, 0x6E)),
							new ColorPickerColor("NZ02C Morning Sky", Color.FromArgb(0xA9, 0xB9, 0xB7)),
							new ColorPickerColor("NZ01B Fern Canopy", Color.FromArgb(0x7A, 0x88, 0x5D)),
							new ColorPickerColor("NZ01D Timber Brown", Color.FromArgb(0x6C, 0x4F, 0x3F)),
							new ColorPickerColor("NZ03B Campion", Color.FromArgb(0xE9, 0xE2, 0xD3)),
							new ColorPickerColor("NZ03A Nelson", Color.FromArgb(0x53, 0x43, 0x3C)),
						}),
					new ColorPickerColorGroup(
						"River Rock",
						new ColorPickerColor[]
						{
							new ColorPickerColor("RR01B Yosemite Range", Color.FromArgb(0x6D, 0x52, 0x44)),
							new ColorPickerColor("RR05D Sandstone Cliff", Color.FromArgb(0xA1, 0x85, 0x5F)),
							new ColorPickerColor("RR01A Estate Limestone", Color.FromArgb(0xB6, 0xAC, 0x9E)),
							new ColorPickerColor("RR06B Pacific Bluffs", Color.FromArgb(0x4A, 0x56, 0x61)),
							new ColorPickerColor("RR03D Garden Wall", Color.FromArgb(0xCD, 0xC1, 0xAE)),
							new ColorPickerColor("RR04D Volcanic Stone", Color.FromArgb(0x76, 0x51, 0x48)),
							new ColorPickerColor("RR06D Steam Boat Basin", Color.FromArgb(0x8A, 0x80, 0x76)),
							new ColorPickerColor("RR02B Cavern Clay", Color.FromArgb(0x7C, 0x46, 0x45)),
							new ColorPickerColor("RR03B Rock Harbor", Color.FromArgb(0x84, 0x81, 0x74)),
							new ColorPickerColor("RR02C Atlantic Shoreline", Color.FromArgb(0xB6, 0x9E, 0x7C)),
							new ColorPickerColor("RR05C Stepping Stone", Color.FromArgb(0x73, 0x6B, 0x5D)),
							new ColorPickerColor("RR01D Hidden Peak", Color.FromArgb(0x48, 0x4D, 0x57)),
							new ColorPickerColor("RR05B Stormy Peak", Color.FromArgb(0xB3, 0xA9, 0x91)),
							new ColorPickerColor("RR04A Coastal Sand", Color.FromArgb(0xD1, 0xC8, 0xB6)),
							new ColorPickerColor("RR03C Pebble Beach", Color.FromArgb(0x9A, 0x8A, 0x77)),
							new ColorPickerColor("RR02D Somerset Moss", Color.FromArgb(0x60, 0x5C, 0x52)),
							new ColorPickerColor("RR04C Stone Chasm", Color.FromArgb(0x55, 0x58, 0x58)),
							new ColorPickerColor("RR02A Canyon Fossil", Color.FromArgb(0x73, 0x76, 0x74)),
							new ColorPickerColor("RR04B Lichen Boulder", Color.FromArgb(0x68, 0x62, 0x52)),
							new ColorPickerColor("RR01C Pediment", Color.FromArgb(0x95, 0x97, 0x90)),
							new ColorPickerColor("RR05A Grand Teton", Color.FromArgb(0x8C, 0x54, 0x49)),
							new ColorPickerColor("RR06A Glacial Rock", Color.FromArgb(0x73, 0x71, 0x6A)),
							new ColorPickerColor("RR06C Granite Falls", Color.FromArgb(0xC7, 0xC6, 0xBE)),
							new ColorPickerColor("RR03A Desert Trail", Color.FromArgb(0x80, 0x64, 0x4B)),
						}),
					new ColorPickerColorGroup(
						"Southport",
						new ColorPickerColor[]
						{
							new ColorPickerColor("SO02C Oar Lock", Color.FromArgb(0x73, 0x39, 0x3A)),
							new ColorPickerColor("SO03B Green Farms", Color.FromArgb(0x44, 0x53, 0x44)),
							new ColorPickerColor("SO02D Tallow Candle", Color.FromArgb(0xD7, 0xAD, 0x6C)),
							new ColorPickerColor("SO04C Driving Cap", Color.FromArgb(0xBC, 0x79, 0x3F)),
							new ColorPickerColor("SO02A Putting Green", Color.FromArgb(0x7C, 0x74, 0x5A)),
							new ColorPickerColor("SO01D Black Rock Harbor", Color.FromArgb(0x37, 0x4A, 0x57)),
							new ColorPickerColor("SO03C Cotillion Pink", Color.FromArgb(0xBF, 0x92, 0x8A)),
							new ColorPickerColor("SO04A University Red", Color.FromArgb(0xB3, 0x4C, 0x47)),
							new ColorPickerColor("SO01C Apple Butter", Color.FromArgb(0xAD, 0x93, 0x6F)),
							new ColorPickerColor("SO03D Hardtop Maroon", Color.FromArgb(0x54, 0x3A, 0x44)),
							new ColorPickerColor("SO04D Wisteria Bloom", Color.FromArgb(0x94, 0xA4, 0xB9)),
							new ColorPickerColor("SO01A Westminster Ribbon", Color.FromArgb(0x52, 0x5A, 0x7C)),
							new ColorPickerColor("SO01B Taffeta Bow", Color.FromArgb(0x6C, 0x47, 0x6D)),
							new ColorPickerColor("SO03A Shooting Range", Color.FromArgb(0x5A, 0x6C, 0x6D)),
							new ColorPickerColor("SO04B Beachside Drive", Color.FromArgb(0x44, 0x6C, 0x9F)),
							new ColorPickerColor("SO02B Hampshire Oak", Color.FromArgb(0x73, 0x5C, 0x3F)),
						}),
					new ColorPickerColorGroup(
						"Suedes",
						new ColorPickerColor[]
						{
							new ColorPickerColor("SU02 Snowdrift", Color.FromArgb(0xCA, 0xB0, 0x8E)),
							new ColorPickerColor("SU05 Fawns Leap", Color.FromArgb(0x9F, 0x82, 0x66)),
							new ColorPickerColor("SU09 Mochernut", Color.FromArgb(0xBB, 0x92, 0x6C)),
							new ColorPickerColor("SU13 Ayers Rock", Color.FromArgb(0xB2, 0x95, 0x6A)),
							new ColorPickerColor("SU17 Whistlewood", Color.FromArgb(0x87, 0x62, 0x43)),
							new ColorPickerColor("SU21 Serape", Color.FromArgb(0xB2, 0x84, 0x77)),
							new ColorPickerColor("SU25 Flowering Ash", Color.FromArgb(0xB3, 0x9F, 0x80)),
							new ColorPickerColor("SU29 Gunnison Plateau", Color.FromArgb(0x8C, 0x74, 0x56)),
							new ColorPickerColor("SU33 Tallgrass Prairie", Color.FromArgb(0x95, 0xA0, 0x92)),
							new ColorPickerColor("SU37 Blue Beech", Color.FromArgb(0x7F, 0x92, 0x94)),
							new ColorPickerColor("SU41 Bluegrass", Color.FromArgb(0x4A, 0x53, 0x5B)),
							new ColorPickerColor("SU01 Spitfire", Color.FromArgb(0xB8, 0xA4, 0x8A)),
							new ColorPickerColor("SU06 Silver Blaze", Color.FromArgb(0x9D, 0x8E, 0x7C)),
							new ColorPickerColor("SU10 Arrow Wood", Color.FromArgb(0x98, 0x75, 0x53)),
							new ColorPickerColor("SU15 Indian Bitter", Color.FromArgb(0x8B, 0x72, 0x5B)),
							new ColorPickerColor("SU18 Buckeye Cavern", Color.FromArgb(0x73, 0x56, 0x47)),
							new ColorPickerColor("SU22 Cheyenne Rock", Color.FromArgb(0x94, 0x55, 0x47)),
							new ColorPickerColor("SU26 Jasper", Color.FromArgb(0xA6, 0x98, 0x83)),
							new ColorPickerColor("SU30 Mulberry", Color.FromArgb(0x67, 0x60, 0x49)),
							new ColorPickerColor("SU34 Desert Cactus", Color.FromArgb(0x85, 0x8C, 0x77)),
							new ColorPickerColor("SU38 Indian Springs", Color.FromArgb(0x74, 0x8C, 0x91)),
							new ColorPickerColor("SU42 Gunflint", Color.FromArgb(0x48, 0x4F, 0x4F)),
							new ColorPickerColor("SU03 Desert Plateau", Color.FromArgb(0xBD, 0x9B, 0x78)),
							new ColorPickerColor("SU07 Doeskin Gray", Color.FromArgb(0x93, 0x80, 0x6E)),
							new ColorPickerColor("SU11 Pioneers Mesa", Color.FromArgb(0x8D, 0x67, 0x46)),
							new ColorPickerColor("SU14 Almond Willow", Color.FromArgb(0x9F, 0x85, 0x6B)),
							new ColorPickerColor("SU19 Shagbark", Color.FromArgb(0x61, 0x4C, 0x40)),
							new ColorPickerColor("SU23 Rowan Berry", Color.FromArgb(0x83, 0x48, 0x3C)),
							new ColorPickerColor("SU27 Prairie Dog", Color.FromArgb(0x91, 0x81, 0x6A)),
							new ColorPickerColor("SU31 Dusty Mountain", Color.FromArgb(0x6A, 0x5D, 0x4C)),
							new ColorPickerColor("SU35 Shadow Ridge", Color.FromArgb(0x5B, 0x5A, 0x46)),
							new ColorPickerColor("SU39 Buffalo Creek", Color.FromArgb(0x4F, 0x64, 0x73)),
							new ColorPickerColor("SU43 Zinc", Color.FromArgb(0x50, 0x4E, 0x4A)),
							new ColorPickerColor("SU04 Rattlebox", Color.FromArgb(0x98, 0x6C, 0x42)),
							new ColorPickerColor("SU08 Fossil Butte", Color.FromArgb(0x7E, 0x69, 0x54)),
							new ColorPickerColor("SU12 Bison", Color.FromArgb(0x6A, 0x51, 0x41)),
							new ColorPickerColor("SU16 Teton Range", Color.FromArgb(0x6C, 0x5C, 0x4B)),
							new ColorPickerColor("SU20 Black Shale", Color.FromArgb(0x54, 0x4C, 0x48)),
							new ColorPickerColor("SU24 Red Mountain", Color.FromArgb(0x6D, 0x4A, 0x3E)),
							new ColorPickerColor("SU28 Slate", Color.FromArgb(0x88, 0x6B, 0x44)),
							new ColorPickerColor("SU32 Sage", Color.FromArgb(0x61, 0x58, 0x48)),
							new ColorPickerColor("SU36 Lapis Lazuli", Color.FromArgb(0x5C, 0x5A, 0x4C)),
							new ColorPickerColor("SU40 Loess", Color.FromArgb(0x48, 0x60, 0x6D)),
							new ColorPickerColor("SU44 Cilia", Color.FromArgb(0x47, 0x48, 0x49)),
						}),
					new ColorPickerColorGroup(
						"Thoroughbred",
						new ColorPickerColor[]
						{
							new ColorPickerColor("TH01 - Atlantic Winter", Color.FromArgb(0xCF, 0xC5, 0xBC)),
							new ColorPickerColor("TH02 - Tudric Pewter", Color.FromArgb(0xB9, 0xB2, 0xA7)),
							new ColorPickerColor("TH03 - Polaris", Color.FromArgb(0xBF, 0xC1, 0xC0)),
							new ColorPickerColor("TH04 - Sweatshirt Grey", Color.FromArgb(0xA1, 0xA3, 0xA1)),
							new ColorPickerColor("TH05 - Masonry Grey", Color.FromArgb(0xB9, 0xB2, 0xAB)),
							new ColorPickerColor("TH06 - Forde Abbey", Color.FromArgb(0xAF, 0xA8, 0x9E)),
							new ColorPickerColor("TH07 - Washboard", Color.FromArgb(0x93, 0x90, 0x8A)),
							new ColorPickerColor("TH08 - Peached Teak", Color.FromArgb(0x80, 0x7C, 0x75)),
							new ColorPickerColor("TH09 - Cymric Silver", Color.FromArgb(0xB7, 0xAA, 0x99)),
							new ColorPickerColor("TH10 - Mt. Rainier", Color.FromArgb(0xB1, 0xA9, 0xA1)),
							new ColorPickerColor("TH11 - Mombasa Mist", Color.FromArgb(0x83, 0x7C, 0x73)),
							new ColorPickerColor("TH12 - Palais Royal", Color.FromArgb(0x63, 0x63, 0x60)),
							new ColorPickerColor("TH13 - Moonlight", Color.FromArgb(0xB1, 0xAE, 0xA6)),
							new ColorPickerColor("TH14 - Orion Grey", Color.FromArgb(0x83, 0x81, 0x7D)),
							new ColorPickerColor("TH15 - Garden Spigot", Color.FromArgb(0x56, 0x59, 0x5A)),
							new ColorPickerColor("TH16 - Bone Black", Color.FromArgb(0x36, 0x36, 0x36)),
							new ColorPickerColor("TH17 - Windsor", Color.FromArgb(0x3D, 0x55, 0x4B)),
							new ColorPickerColor("TH18 - British Racing Green", Color.FromArgb(0x2F, 0x55, 0x4A)),
							new ColorPickerColor("TH19 - Porch Awning", Color.FromArgb(0x37, 0x4B, 0x46)),
							new ColorPickerColor("TH20 - Sea", Color.FromArgb(0x34, 0x4A, 0x46)),
							new ColorPickerColor("TH21 - Gallery Green", Color.FromArgb(0x4B, 0x58, 0x4F)),
							new ColorPickerColor("TH22 - Hometown Green", Color.FromArgb(0x37, 0x56, 0x50)),
							new ColorPickerColor("TH23 - Boxwood", Color.FromArgb(0x39, 0x49, 0x45)),
							new ColorPickerColor("TH24 - Farmington Green", Color.FromArgb(0x3B, 0x43, 0x42)),
							new ColorPickerColor("TH25 - Scholar Green", Color.FromArgb(0x35, 0x40, 0x3B)),
							new ColorPickerColor("TH26 - Polo Green", Color.FromArgb(0x3B, 0x42, 0x43)),
							new ColorPickerColor("TH27 - Alistair Green", Color.FromArgb(0x44, 0x4B, 0x48)),
							new ColorPickerColor("TH28 - Surrey", Color.FromArgb(0x42, 0x43, 0x41)),
							new ColorPickerColor("TH29 - Cottonwood", Color.FromArgb(0xCB, 0xB0, 0x93)),
							new ColorPickerColor("TH30 - Thunder Bay", Color.FromArgb(0xC3, 0xA1, 0x7F)),
							new ColorPickerColor("TH31 - Kauai Jungle", Color.FromArgb(0x9A, 0x7A, 0x5C)),
							new ColorPickerColor("TH32 - Mayfield Common", Color.FromArgb(0x79, 0x5B, 0x48)),
							new ColorPickerColor("TH33 - Rufford Hall", Color.FromArgb(0x88, 0x5F, 0x44)),
							new ColorPickerColor("TH34 - Elm", Color.FromArgb(0x71, 0x54, 0x42)),
							new ColorPickerColor("TH35 - Desert Boot", Color.FromArgb(0x53, 0x41, 0x3D)),
							new ColorPickerColor("TH36 - Ebony", Color.FromArgb(0x37, 0x36, 0x37)),
							new ColorPickerColor("TH37 - Takahe", Color.FromArgb(0x7F, 0x4F, 0x3D)),
							new ColorPickerColor("TH38 - Bridle", Color.FromArgb(0x65, 0x4E, 0x45)),
							new ColorPickerColor("TH39 - Oxford Brown", Color.FromArgb(0x62, 0x4B, 0x43)),
							new ColorPickerColor("TH40 - Traditional", Color.FromArgb(0x47, 0x3A, 0x38)),
							new ColorPickerColor("TH41 - Dressage Red", Color.FromArgb(0xA0, 0x3E, 0x3E)),
							new ColorPickerColor("TH42 - Stadium Red", Color.FromArgb(0x94, 0x39, 0x40)),
							new ColorPickerColor("TH43 - Hunting Coat Red", Color.FromArgb(0x86, 0x3B, 0x3D)),
							new ColorPickerColor("TH44 - Balmoral Red", Color.FromArgb(0x71, 0x3B, 0x3F)),
							new ColorPickerColor("TH45 - Longjohn Red", Color.FromArgb(0x87, 0x43, 0x42)),
							new ColorPickerColor("TH46 - Chimayo Red", Color.FromArgb(0x89, 0x40, 0x3D)),
							new ColorPickerColor("TH47 - Maleya Red", Color.FromArgb(0x81, 0x3E, 0x41)),
							new ColorPickerColor("TH48 - Townhouse", Color.FromArgb(0x63, 0x3B, 0x38)),
							new ColorPickerColor("TH49 - Venetian Red", Color.FromArgb(0x77, 0x39, 0x3E)),
							new ColorPickerColor("TH50 - Canterbury", Color.FromArgb(0x5B, 0x41, 0x41)),
							new ColorPickerColor("TH51 - Brazilian Ruby", Color.FromArgb(0x5B, 0x3E, 0x41)),
							new ColorPickerColor("TH52 - Picture-Gallery Red", Color.FromArgb(0x59, 0x3A, 0x3A)),
							new ColorPickerColor("TH53 - Kilim Red", Color.FromArgb(0x66, 0x3F, 0x41)),
							new ColorPickerColor("TH54 - Claret", Color.FromArgb(0x57, 0x3B, 0x40)),
							new ColorPickerColor("TH55 - Mahogany", Color.FromArgb(0x53, 0x3F, 0x3F)),
							new ColorPickerColor("TH56 - Armagnac", Color.FromArgb(0x51, 0x3D, 0x3D)),
							new ColorPickerColor("TH57 - Burgundy", Color.FromArgb(0x4E, 0x3B, 0x42)),
							new ColorPickerColor("TH58 - Nairobi Dusk", Color.FromArgb(0x51, 0x42, 0x4F)),
							new ColorPickerColor("TH59 - Embassy Purple", Color.FromArgb(0x45, 0x3C, 0x46)),
							new ColorPickerColor("TH60 - Athletic Purple", Color.FromArgb(0x44, 0x45, 0x65)),
							new ColorPickerColor("TH61 - Black Watch Navy", Color.FromArgb(0x39, 0x3F, 0x54)),
							new ColorPickerColor("TH62 - Cardiff Blue", Color.FromArgb(0x38, 0x3C, 0x46)),
							new ColorPickerColor("TH63 - Tartan Blue", Color.FromArgb(0x3C, 0x42, 0x59)),
							new ColorPickerColor("TH64 - Yacht Blue", Color.FromArgb(0x37, 0x3E, 0x4F)),
							new ColorPickerColor("TH65 - Overalls", Color.FromArgb(0x39, 0x47, 0x5D)),
							new ColorPickerColor("TH66 - Adirondack Blue", Color.FromArgb(0x3B, 0x49, 0x5D)),
							new ColorPickerColor("TH67 - Club Navy", Color.FromArgb(0x34, 0x3D, 0x50)),
							new ColorPickerColor("TH68 - Edgartown Blue", Color.FromArgb(0x3F, 0x43, 0x51)),
							new ColorPickerColor("TH69 - Carson Blue", Color.FromArgb(0x3C, 0x52, 0x6A)),
							new ColorPickerColor("TH70 - Chatsworth Blue", Color.FromArgb(0x3D, 0x40, 0x49)),
							new ColorPickerColor("TH71 - Harbour", Color.FromArgb(0x32, 0x41, 0x59)),
							new ColorPickerColor("TH72 - Magistrate Black", Color.FromArgb(0x3B, 0x3F, 0x45)),
						}),
					new ColorPickerColorGroup(
						"Whites",
						new ColorPickerColor[]
						{
							new ColorPickerColor("WH01A Design Studio White", Color.FromArgb(0xF4, 0xF3, 0xE9)),
							new ColorPickerColor("WH02A Roadster White", Color.FromArgb(0xF3, 0xEA, 0xD4)),
							new ColorPickerColor("WH03A Avalanche", Color.FromArgb(0xE1, 0xE4, 0xDC)),
							new ColorPickerColor("WH04A Tuxedo Shirt", Color.FromArgb(0xF2, 0xEF, 0xDF)),
							new ColorPickerColor("WH05A Picket Fence White", Color.FromArgb(0xF3, 0xF2, 0xE9)),
							new ColorPickerColor("WH06A Flour Sack White", Color.FromArgb(0xE7, 0xD7, 0xBF)),
							new ColorPickerColor("WH07A Aspen Summit", Color.FromArgb(0xE6, 0xE4, 0xE3)),
							new ColorPickerColor("WH08A Breakwater White", Color.FromArgb(0xE6, 0xE9, 0xE2)),
							new ColorPickerColor("WH01B Pocket Watch White", Color.FromArgb(0xF0, 0xEE, 0xE3)),
							new ColorPickerColor("WH02B Riviera Terrace", Color.FromArgb(0xF2, 0xEA, 0xDA)),
							new ColorPickerColor("WH03B Spring Clover", Color.FromArgb(0xEA, 0xDE, 0xD5)),
							new ColorPickerColor("WH04B Candelabra White", Color.FromArgb(0xF3, 0xEB, 0xDC)),
							new ColorPickerColor("WH05B Country Stove White", Color.FromArgb(0xF1, 0xEF, 0xE3)),
							new ColorPickerColor("WH06B Writer's Parchment", Color.FromArgb(0xEE, 0xE7, 0xD3)),
							new ColorPickerColor("WH07B Killington Traverse", Color.FromArgb(0xE2, 0xE8, 0xD7)),
							new ColorPickerColor("WH08B Starched Apron", Color.FromArgb(0xEB, 0xE4, 0xD9)),
							new ColorPickerColor("WH01C Garden Rose White", Color.FromArgb(0xE6, 0xDF, 0xD8)),
							new ColorPickerColor("WH02C Sail White", Color.FromArgb(0xE8, 0xE1, 0xD2)),
							new ColorPickerColor("WH03C River Rapids", Color.FromArgb(0xD7, 0xDC, 0xDA)),
							new ColorPickerColor("WH04C Petticoat White", Color.FromArgb(0xF2, 0xEB, 0xE2)),
							new ColorPickerColor("WH05C Polo Mallet White", Color.FromArgb(0xEC, 0xEB, 0xDF)),
							new ColorPickerColor("WH06C Edwardian Linen", Color.FromArgb(0xF1, 0xE9, 0xD5)),
							new ColorPickerColor("WH07C Montauk Driftwood", Color.FromArgb(0xDA, 0xDC, 0xD3)),
							new ColorPickerColor("WH08C Country Diary", Color.FromArgb(0xE8, 0xE4, 0xD9)),
							new ColorPickerColor("WH01D Tennis Court White", Color.FromArgb(0xF0, 0xEC, 0xDD)),
							new ColorPickerColor("WH02D Dune White", Color.FromArgb(0xED, 0xE1, 0xCA)),
							new ColorPickerColor("WH03D Dover Cliffs", Color.FromArgb(0xDB, 0xDB, 0xD7)),
							new ColorPickerColor("WH04D Resort White", Color.FromArgb(0xF2, 0xE7, 0xDE)),
							new ColorPickerColor("WH05D Sneaker White", Color.FromArgb(0xEE, 0xED, 0xE3)),
							new ColorPickerColor("WH06D Journal White", Color.FromArgb(0xE8, 0xE4, 0xD8)),
							new ColorPickerColor("WH07D Tackroom White", Color.FromArgb(0xE6, 0xEA, 0xE4)),
							new ColorPickerColor("WH08D Sailor's Knot", Color.FromArgb(0xE0, 0xDD, 0xD7)),
						}),
				}),
*/
			#endregion

			#region Dream
/*
			new ColorPickerColorPalette(
				"Dream", "cbn_dream",
				new ColorPickerColorGroup[]
				{
					new ColorPickerColorGroup(
						"",
						new ColorPickerColor[]
						{
							new ColorPickerColor("Cascade 101-2", Color.FromArgb(0xD1, 0xEA, 0xE1)),
							new ColorPickerColor("Oasis 102-1", Color.FromArgb(0xD6, 0xEC, 0xE4)),
							new ColorPickerColor("Wintergreen 103-1", Color.FromArgb(0xD1, 0xE9, 0xDB)),
							new ColorPickerColor("Winterfresh 104-1", Color.FromArgb(0xD1, 0xE9, 0xD7)),
							new ColorPickerColor("Sea Mist 105-1", Color.FromArgb(0xD8, 0xEA, 0xD9)),
							new ColorPickerColor("Mint Essence 106-1", Color.FromArgb(0xE0, 0xEC, 0xDD)),
							new ColorPickerColor("Savannah Mist 106-2", Color.FromArgb(0xD2, 0xE8, 0xD3)),
							new ColorPickerColor("Hint Of Mint 107-1", Color.FromArgb(0xE2, 0xEC, 0xDB)),
							new ColorPickerColor("Dewmist Delight 107-2", Color.FromArgb(0xDB, 0xEB, 0xD5)),
							new ColorPickerColor("Applemint 107-3", Color.FromArgb(0xCC, 0xE6, 0xC6)),
							new ColorPickerColor("Alaskan Mist 108-1", Color.FromArgb(0xE9, 0xEC, 0xDF)),
							new ColorPickerColor("Spearmint Stick 108-2", Color.FromArgb(0xE8, 0xED, 0xDC)),
							new ColorPickerColor("Chilled Mint 108-3", Color.FromArgb(0xE3, 0xEC, 0xD9)),
							new ColorPickerColor("Lantana Lime 108-4", Color.FromArgb(0xD6, 0xE7, 0xC5)),
							new ColorPickerColor("Morning Glow 110-1", Color.FromArgb(0xEE, 0xEC, 0xD2)),
							new ColorPickerColor("Cool Yellow 110-2", Color.FromArgb(0xE9, 0xE9, 0xC6)),
							new ColorPickerColor("Buttermint 110-3", Color.FromArgb(0xEA, 0xE9, 0xC0)),
							new ColorPickerColor("Spring Blossom 110-4", Color.FromArgb(0xEA, 0xE7, 0xB5)),
							new ColorPickerColor("Lemon Pepper 110-5", Color.FromArgb(0xEE, 0xE3, 0x9E)),
							new ColorPickerColor("Sail Cloth 111-1", Color.FromArgb(0xED, 0xEB, 0xD9)),
							new ColorPickerColor("Blonde Beauty 111-2", Color.FromArgb(0xF2, 0xE9, 0xC5)),
							new ColorPickerColor("Butter 111-3", Color.FromArgb(0xF5, 0xE6, 0xB1)),
							new ColorPickerColor("Calabash 111-5", Color.FromArgb(0xFB, 0xE1, 0x8D)),
							new ColorPickerColor("Capri Cream 112-1", Color.FromArgb(0xF1, 0xEB, 0xCF)),
							new ColorPickerColor("Moonshine 113-1", Color.FromArgb(0xF2, 0xE9, 0xCD)),
							new ColorPickerColor("Shortbread 113-2", Color.FromArgb(0xF7, 0xE5, 0xB3)),
							new ColorPickerColor("Beeswax 113-3", Color.FromArgb(0xFA, 0xDE, 0x9B)),
							new ColorPickerColor("Yorkshire Pudding 113-4", Color.FromArgb(0xFE, 0xDA, 0x89)),
							new ColorPickerColor("Lotus Flower 114-1", Color.FromArgb(0xF4, 0xEB, 0xD2)),
							new ColorPickerColor("Silk Sails 114-2", Color.FromArgb(0xF7, 0xE9, 0xC6)),
							new ColorPickerColor("Flax 114-3", Color.FromArgb(0xFC, 0xE1, 0xA8)),
							new ColorPickerColor("Golden Slumber 114-4", Color.FromArgb(0xFF, 0xD9, 0x8F)),
							new ColorPickerColor("Combed Cotton 115-1", Color.FromArgb(0xF1, 0xEA, 0xD5)),
							new ColorPickerColor("Turban Shell 116-1", Color.FromArgb(0xF1, 0xED, 0xDB)),
							new ColorPickerColor("Spice Ivory 116-2", Color.FromArgb(0xF2, 0xE9, 0xD5)),
							new ColorPickerColor("Vanilla Wafer 116-3", Color.FromArgb(0xF3, 0xE5, 0xCA)),
							new ColorPickerColor("Shell Flower 116-4", Color.FromArgb(0xFC, 0xDC, 0xAD)),
							new ColorPickerColor("Spun Cotton 119-1", Color.FromArgb(0xF1, 0xE8, 0xD5)),
							new ColorPickerColor("Mimosa 119-2", Color.FromArgb(0xF3, 0xE4, 0xCE)),
							new ColorPickerColor("Sweet Sorbet 119-3", Color.FromArgb(0xF3, 0xE0, 0xC8)),
							new ColorPickerColor("Peach Dip 119-4", Color.FromArgb(0xF3, 0xD7, 0xB6)),
							new ColorPickerColor("Ginger Peach 119-5", Color.FromArgb(0xF8, 0xC7, 0x95)),
							new ColorPickerColor("Victorian Linen 120-1", Color.FromArgb(0xF2, 0xE7, 0xD6)),
							new ColorPickerColor("Croissant 120-2", Color.FromArgb(0xF3, 0xE4, 0xD1)),
							new ColorPickerColor("Sugar Sweet 120-3", Color.FromArgb(0xF4, 0xDE, 0xC8)),
							new ColorPickerColor("Honey Bird 120-4", Color.FromArgb(0xF7, 0xD4, 0xB0)),
							new ColorPickerColor("Tangelo Cream 122-1", Color.FromArgb(0xF1, 0xE6, 0xD8)),
							new ColorPickerColor("Light Peach 122-2", Color.FromArgb(0xF1, 0xDC, 0xCB)),
							new ColorPickerColor("Bisque 122-3", Color.FromArgb(0xF5, 0xD1, 0xB7)),
							new ColorPickerColor("Melon Ice 124-2", Color.FromArgb(0xF2, 0xD4, 0xC1)),
							new ColorPickerColor("Dreamsicle 124-3", Color.FromArgb(0xF3, 0xCF, 0xBA)),
							new ColorPickerColor("Peach Everlasting 125-2", Color.FromArgb(0xF2, 0xDE, 0xCE)),
							new ColorPickerColor("Opal Peach 126-1", Color.FromArgb(0xF1, 0xDE, 0xCF)),
							new ColorPickerColor("Light Rose Beige 126-2", Color.FromArgb(0xF1, 0xDC, 0xCD)),
							new ColorPickerColor("Sheraton Rose 126-3", Color.FromArgb(0xEF, 0xD0, 0xC2)),
							new ColorPickerColor("La Minuet 128-2", Color.FromArgb(0xF4, 0xE1, 0xD5)),
							new ColorPickerColor("Scalloped Shell 129-1", Color.FromArgb(0xF1, 0xE6, 0xDB)),
							new ColorPickerColor("Cupid Pink 130-1", Color.FromArgb(0xF1, 0xDB, 0xD4)),
							new ColorPickerColor("Rose Linen 130-2", Color.FromArgb(0xF3, 0xDB, 0xD4)),
							new ColorPickerColor("Floral Linen 131-1", Color.FromArgb(0xF1, 0xDD, 0xD7)),
							new ColorPickerColor("Plaster Pink 132-1", Color.FromArgb(0xF3, 0xE1, 0xD9)),
							new ColorPickerColor("Rockabye Baby 133-1", Color.FromArgb(0xF1, 0xDF, 0xD9)),
							new ColorPickerColor("Gaiety 135-1", Color.FromArgb(0xF1, 0xE1, 0xE0)),
							new ColorPickerColor("Candy Mix 137-1", Color.FromArgb(0xF0, 0xDC, 0xDE)),
							new ColorPickerColor("Brittany's Bow 137-2", Color.FromArgb(0xEF, 0xD5, 0xDA)),
							new ColorPickerColor("Bubble Bath 138-1", Color.FromArgb(0xEE, 0xE1, 0xE3)),
							new ColorPickerColor("Heaven Sent 140-2", Color.FromArgb(0xE9, 0xE0, 0xE6)),
							new ColorPickerColor("Desert Dawn 140-3", Color.FromArgb(0xE8, 0xD9, 0xE2)),
							new ColorPickerColor("Angora Pink 142-1", Color.FromArgb(0xE7, 0xDD, 0xE4)),
							new ColorPickerColor("Lilac Frost 142-2", Color.FromArgb(0xE3, 0xDC, 0xE4)),
							new ColorPickerColor("Syrian Violet 142-3", Color.FromArgb(0xD8, 0xC9, 0xDF)),
							new ColorPickerColor("Darling Lilac 145-1", Color.FromArgb(0xE1, 0xE3, 0xE7)),
							new ColorPickerColor("Astoria 145-2", Color.FromArgb(0xD7, 0xDD, 0xE6)),
							new ColorPickerColor("Monet's Lilies 145-3", Color.FromArgb(0xCD, 0xD5, 0xE5)),
							new ColorPickerColor("December Dawn 147-1", Color.FromArgb(0xDA, 0xE2, 0xE7)),
							new ColorPickerColor("Ocean Breeze 150-1", Color.FromArgb(0xCF, 0xE5, 0xE9)),
							new ColorPickerColor("Niagara Falls 150-2", Color.FromArgb(0xC4, 0xE1, 0xE8)),
							new ColorPickerColor("Fond Farewell 151-2", Color.FromArgb(0xC1, 0xE0, 0xE7)),
							new ColorPickerColor("Bashful 152-1", Color.FromArgb(0xE0, 0xEA, 0xE7)),
							new ColorPickerColor("Diamond Blue 152-2", Color.FromArgb(0xC9, 0xE2, 0xE7)),
							new ColorPickerColor("Ariel 152-3", Color.FromArgb(0xA6, 0xD6, 0xE5)),
							new ColorPickerColor("Blue Persuasion 153-2", Color.FromArgb(0xD1, 0xE6, 0xE6)),
							new ColorPickerColor("Breathless 154-1", Color.FromArgb(0xDA, 0xEA, 0xE7)),
							new ColorPickerColor("Enchanted Evening 154-2", Color.FromArgb(0xCF, 0xE7, 0xE6)),
							new ColorPickerColor("Surf's Up 154-3", Color.FromArgb(0xC0, 0xE2, 0xE4)),
							new ColorPickerColor("Himalayan Mist 155-1", Color.FromArgb(0xDE, 0xEA, 0xE4)),
							new ColorPickerColor("Fairytale 155-2", Color.FromArgb(0xD6, 0xE8, 0xE5)),
							new ColorPickerColor("Hallowed Hush 201-1", Color.FromArgb(0xE0, 0xE9, 0xE1)),
							new ColorPickerColor("Swan Lake 201-2", Color.FromArgb(0xC2, 0xE3, 0xDC)),
							new ColorPickerColor("Tropical Holiday 201-4", Color.FromArgb(0x8A, 0xC9, 0xBF)),
							new ColorPickerColor("Artesian Well 201-5", Color.FromArgb(0x58, 0xAC, 0x9F)),
							new ColorPickerColor("Azure Tide 201-6", Color.FromArgb(0x2B, 0x91, 0x85)),
							new ColorPickerColor("Maggie's Magic 204-1", Color.FromArgb(0xDA, 0xEA, 0xDC)),
							new ColorPickerColor("Jade Spell 204-2", Color.FromArgb(0xBE, 0xE1, 0xCE)),
							new ColorPickerColor("Pistachio 205-1", Color.FromArgb(0xD9, 0xE9, 0xD9)),
							new ColorPickerColor("Mint Majesty 205-2", Color.FromArgb(0xC8, 0xE5, 0xCD)),
							new ColorPickerColor("Sprite Twist 206-3", Color.FromArgb(0xB8, 0xD6, 0xBB)),
							new ColorPickerColor("Heath Green 206-4", Color.FromArgb(0x9A, 0xC6, 0x9F)),
							new ColorPickerColor("Elf Shoe 206-5", Color.FromArgb(0x6B, 0xA6, 0x76)),
							new ColorPickerColor("Blarney Stone 206-7", Color.FromArgb(0x1F, 0x6F, 0x3C)),
							new ColorPickerColor("Green Whisper 207-1", Color.FromArgb(0xE1, 0xEA, 0xDC)),
							new ColorPickerColor("Herbal Mist 207-2", Color.FromArgb(0xD0, 0xE2, 0xCB)),
							new ColorPickerColor("Rain Forest 207-3", Color.FromArgb(0xC5, 0xDB, 0xC2)),
							new ColorPickerColor("Herbal Green 207-4", Color.FromArgb(0xAA, 0xCA, 0xA5)),
							new ColorPickerColor("Ivy Vine 207-5", Color.FromArgb(0x80, 0xAD, 0x78)),
							new ColorPickerColor("Jester's Green 207-6", Color.FromArgb(0x66, 0x95, 0x5E)),
							new ColorPickerColor("Bell Pepper 207-7", Color.FromArgb(0x3A, 0x6C, 0x32)),
							new ColorPickerColor("Lime Meringue 209-1", Color.FromArgb(0xE5, 0xE8, 0xD0)),
							new ColorPickerColor("Honeydew Melon 209-2", Color.FromArgb(0xE5, 0xE7, 0xC4)),
							new ColorPickerColor("Lettuce Alone 209-4", Color.FromArgb(0xCE, 0xD4, 0x99)),
							new ColorPickerColor("Lime Green 209-5", Color.FromArgb(0xB0, 0xB9, 0x66)),
							new ColorPickerColor("Asparagus 209-6", Color.FromArgb(0x96, 0xA0, 0x48)),
							new ColorPickerColor("Sweet Honeydew 210-1", Color.FromArgb(0xEB, 0xEA, 0xD8)),
							new ColorPickerColor("Lime Twist 210-2", Color.FromArgb(0xE9, 0xE7, 0xC9)),
							new ColorPickerColor("Green Pastures 210-3", Color.FromArgb(0xE7, 0xE2, 0xB0)),
							new ColorPickerColor("Woodland Fern 210-4", Color.FromArgb(0xDF, 0xD9, 0x94)),
							new ColorPickerColor("Golden Cricket 210-5", Color.FromArgb(0xD4, 0xCA, 0x62)),
							new ColorPickerColor("Clear Yellow 211-1", Color.FromArgb(0xF0, 0xEF, 0xE1)),
							new ColorPickerColor("Pineapple Delight 211-3", Color.FromArgb(0xF1, 0xDD, 0x9F)),
							new ColorPickerColor("Pollination 211-7", Color.FromArgb(0xB5, 0x91, 0x23)),
							new ColorPickerColor("Golden Fleece 212-2", Color.FromArgb(0xEF, 0xE6, 0xCB)),
							new ColorPickerColor("Butterfly Bush 212-3", Color.FromArgb(0xF4, 0xDC, 0x9E)),
							new ColorPickerColor("Golden Field 212-7", Color.FromArgb(0xBF, 0x8F, 0x3A)),
							new ColorPickerColor("Midas Touch 213-7", Color.FromArgb(0xC7, 0x87, 0x30)),
							new ColorPickerColor("Gold Buff 215-4", Color.FromArgb(0xEA, 0xB8, 0x77)),
							new ColorPickerColor("Brown Mustard 215-5", Color.FromArgb(0xDD, 0x9D, 0x50)),
							new ColorPickerColor("Gothic Gold 215-7", Color.FromArgb(0xB8, 0x77, 0x2A)),
							new ColorPickerColor("Candlelit Beige 216-1", Color.FromArgb(0xEF, 0xEA, 0xDA)),
							new ColorPickerColor("Golden Mushroom 216-2", Color.FromArgb(0xF3, 0xE2, 0xC9)),
							new ColorPickerColor("Honey Beige 216-3", Color.FromArgb(0xF3, 0xDC, 0xBD)),
							new ColorPickerColor("Toffee Crunch 216-4", Color.FromArgb(0xED, 0xC5, 0x96)),
							new ColorPickerColor("Cape Honey Flower 218-1", Color.FromArgb(0xF2, 0xE8, 0xD6)),
							new ColorPickerColor("Vanilla Tan 219-1", Color.FromArgb(0xEF, 0xE5, 0xD6)),
							new ColorPickerColor("Peach Surprise 219-2", Color.FromArgb(0xF2, 0xDE, 0xCA)),
							new ColorPickerColor("Caramel Ice 219-4", Color.FromArgb(0xEC, 0xC1, 0x9F)),
							new ColorPickerColor("Buffalo Trail 219-5", Color.FromArgb(0xDD, 0x9F, 0x6C)),
							new ColorPickerColor("Ginger Beer 219-7", Color.FromArgb(0xBB, 0x71, 0x30)),
							new ColorPickerColor("Koala 221-3", Color.FromArgb(0xE8, 0xCC, 0xB4)),
							new ColorPickerColor("Tea Biscuit 222-2", Color.FromArgb(0xED, 0xD6, 0xC1)),
							new ColorPickerColor("Gentle Doe 222-4", Color.FromArgb(0xE3, 0xAD, 0x89)),
							new ColorPickerColor("Lucky Penny 222-7", Color.FromArgb(0xB0, 0x62, 0x32)),
							new ColorPickerColor("Siesta 224-3", Color.FromArgb(0xEC, 0xBB, 0x9E)),
							new ColorPickerColor("Georgian Leather 224-5", Color.FromArgb(0xC9, 0x7B, 0x53)),
							new ColorPickerColor("Mincemeat 224-7", Color.FromArgb(0xAE, 0x5E, 0x35)),
							new ColorPickerColor("Mr. Sandman 226-3", Color.FromArgb(0xEB, 0xC6, 0xB7)),
							new ColorPickerColor("Santa Fe Trail 226-4", Color.FromArgb(0xE7, 0xAD, 0x93)),
							new ColorPickerColor("Ginger Jar 226-6", Color.FromArgb(0xC6, 0x75, 0x53)),
							new ColorPickerColor("Summer Lily 229-1", Color.FromArgb(0xF2, 0xE9, 0xE1)),
							new ColorPickerColor("Peach Tone 229-2", Color.FromArgb(0xEF, 0xDE, 0xD5)),
							new ColorPickerColor("Texas Rose 230-3", Color.FromArgb(0xEC, 0xCC, 0xC1)),
							new ColorPickerColor("Peach Beige 230-4", Color.FromArgb(0xE7, 0xB5, 0xAA)),
							new ColorPickerColor("Holland Tile 230-5", Color.FromArgb(0xD8, 0x8F, 0x7F)),
							new ColorPickerColor("Sahara Sun 230-6", Color.FromArgb(0xBB, 0x6A, 0x59)),
							new ColorPickerColor("Limoge Pink 231-1", Color.FromArgb(0xEF, 0xDC, 0xD4)),
							new ColorPickerColor("Ballet Slipper 231-2", Color.FromArgb(0xF0, 0xD7, 0xD0)),
							new ColorPickerColor("Cradle Pink 236-2", Color.FromArgb(0xED, 0xCF, 0xD3)),
							new ColorPickerColor("Dream Dust 240-1", Color.FromArgb(0xE7, 0xE1, 0xE3)),
							new ColorPickerColor("Windsor Purple 242-4", Color.FromArgb(0xBF, 0xAB, 0xCA)),
							new ColorPickerColor("Pretty Purple 243-4", Color.FromArgb(0xBB, 0xB1, 0xCA)),
							new ColorPickerColor("Orchid Mist 244-1", Color.FromArgb(0xE5, 0xE5, 0xE3)),
							new ColorPickerColor("Guardian Angel 244-2", Color.FromArgb(0xE1, 0xE0, 0xE4)),
							new ColorPickerColor("Misty Violet 244-3", Color.FromArgb(0xD6, 0xD4, 0xDE)),
							new ColorPickerColor("Snow Ballet 245-1", Color.FromArgb(0xE5, 0xE5, 0xE2)),
							new ColorPickerColor("Pastel Violet 245-2", Color.FromArgb(0xE1, 0xE2, 0xE3)),
							new ColorPickerColor("Soft Iris 245-3", Color.FromArgb(0xCF, 0xD4, 0xE2)),
							new ColorPickerColor("Smoky Blue 247-2", Color.FromArgb(0xD0, 0xDA, 0xE0)),
							new ColorPickerColor("Twinberry Blossom 247-3", Color.FromArgb(0xB4, 0xC6, 0xD9)),
							new ColorPickerColor("Abyss 248-1", Color.FromArgb(0xDC, 0xE5, 0xE5)),
							new ColorPickerColor("Blue Silk 248-2", Color.FromArgb(0xCA, 0xD9, 0xE2)),
							new ColorPickerColor("Touch Of Blue 249-2", Color.FromArgb(0xBA, 0xD5, 0xE3)),
							new ColorPickerColor("Bridal Blue 251-1", Color.FromArgb(0xD2, 0xE5, 0xE6)),
							new ColorPickerColor("Airy 252-1", Color.FromArgb(0xD5, 0xE4, 0xE3)),
							new ColorPickerColor("Goddess 252-2", Color.FromArgb(0xCC, 0xE0, 0xE2)),
							new ColorPickerColor("Flemish Blue 252-3", Color.FromArgb(0xA5, 0xCD, 0xD9)),
							new ColorPickerColor("Harbor Sky 252-4", Color.FromArgb(0x75, 0xB4, 0xC9)),
							new ColorPickerColor("Hush-a-Bye 252-5", Color.FromArgb(0x47, 0x94, 0xAD)),
							new ColorPickerColor("Port Au Prince 252-7", Color.FromArgb(0x00, 0x6B, 0x89)),
							new ColorPickerColor("Baby Bird 253-1", Color.FromArgb(0xDA, 0xE6, 0xE3)),
							new ColorPickerColor("Kiss Me Kate 254-1", Color.FromArgb(0xE5, 0xEC, 0xE6)),
							new ColorPickerColor("Blue Phlox 254-2", Color.FromArgb(0xCF, 0xE4, 0xE3)),
							new ColorPickerColor("Galactica 254-3", Color.FromArgb(0xBF, 0xDB, 0xDC)),
							new ColorPickerColor("Idyllic Isle 254-4", Color.FromArgb(0x8C, 0xC5, 0xCA)),
							new ColorPickerColor("Blue Calypso 254-5", Color.FromArgb(0x4C, 0xA3, 0xAD)),
							new ColorPickerColor("Bimini Blue 254-7", Color.FromArgb(0x00, 0x79, 0x87)),
							new ColorPickerColor("Blue Yonder 255-1", Color.FromArgb(0xD6, 0xE7, 0xE3)),
							new ColorPickerColor("Sweet Rhapsody 255-2", Color.FromArgb(0xCB, 0xE2, 0xDE)),
							new ColorPickerColor("Blue Dusk 255-3", Color.FromArgb(0xBA, 0xDD, 0xDC)),
							new ColorPickerColor("Aqua Chiffon 255-4", Color.FromArgb(0x8A, 0xCC, 0xCD)),
							new ColorPickerColor("Schooner 255-6", Color.FromArgb(0x1F, 0x8B, 0x8E)),
							new ColorPickerColor("French Riviera 255-7", Color.FromArgb(0x00, 0x78, 0x7B)),
							new ColorPickerColor("Aqua Tint 256-1", Color.FromArgb(0xE3, 0xF0, 0xE9)),
							new ColorPickerColor("Plateau 256-2", Color.FromArgb(0xCF, 0xE5, 0xDF)),
							new ColorPickerColor("Aqua Whisper 256-3", Color.FromArgb(0xBB, 0xDC, 0xD8)),
							new ColorPickerColor("Fling Green 256-4", Color.FromArgb(0x88, 0xCA, 0xC7)),
							new ColorPickerColor("Biscay Bay 256-5", Color.FromArgb(0x4B, 0xAD, 0xA7)),
							new ColorPickerColor("Sea Fantasy 256-6", Color.FromArgb(0x1F, 0x90, 0x8C)),
							new ColorPickerColor("Hypnotic Sea 256-7", Color.FromArgb(0x00, 0x7C, 0x7A)),
							new ColorPickerColor("Peaceful Night 301-1", Color.FromArgb(0xD2, 0xE5, 0xE0)),
							new ColorPickerColor("Big Sky 301-2", Color.FromArgb(0xC9, 0xDF, 0xD8)),
							new ColorPickerColor("Veridian Green 301-3", Color.FromArgb(0xB7, 0xD2, 0xCC)),
							new ColorPickerColor("Pitter Patter 301-4", Color.FromArgb(0x95, 0xBB, 0xB4)),
							new ColorPickerColor("Jericho Jade 301-6", Color.FromArgb(0x48, 0x7E, 0x77)),
							new ColorPickerColor("Ebb Tide 304-1", Color.FromArgb(0xD7, 0xE7, 0xDA)),
							new ColorPickerColor("Splish Splash 304-2", Color.FromArgb(0xB7, 0xD7, 0xC5)),
							new ColorPickerColor("Clair De Lune 305-1", Color.FromArgb(0xD8, 0xE6, 0xD9)),
							new ColorPickerColor("Solitaire 305-2", Color.FromArgb(0xC3, 0xD9, 0xC7)),
							new ColorPickerColor("Salty Sea 306-3", Color.FromArgb(0xB5, 0xCF, 0xB9)),
							new ColorPickerColor("Phosphorus 306-5", Color.FromArgb(0x72, 0x95, 0x76)),
							new ColorPickerColor("Green Ripple 306-6", Color.FromArgb(0x53, 0x76, 0x56)),
							new ColorPickerColor("Rhododendron 306-7", Color.FromArgb(0x38, 0x5B, 0x3E)),
							new ColorPickerColor("Valley Mist 307-1", Color.FromArgb(0xE0, 0xE9, 0xDC)),
							new ColorPickerColor("Sweetbriar 307-2", Color.FromArgb(0xD1, 0xDD, 0xCD)),
							new ColorPickerColor("Lime Blossom 307-3", Color.FromArgb(0xC6, 0xD6, 0xC1)),
							new ColorPickerColor("Darlington 307-4", Color.FromArgb(0xA8, 0xC0, 0xA6)),
							new ColorPickerColor("Prime Lime 307-5", Color.FromArgb(0x7F, 0x97, 0x78)),
							new ColorPickerColor("Hummingbird Green 307-7", Color.FromArgb(0x41, 0x5B, 0x38)),
							new ColorPickerColor("Summer Rain 308-1", Color.FromArgb(0xDE, 0xE3, 0xD4)),
							new ColorPickerColor("Mint Wafer 308-2", Color.FromArgb(0xD9, 0xE1, 0xD1)),
							new ColorPickerColor("St. Augustine 308-3", Color.FromArgb(0xCD, 0xD8, 0xC5)),
							new ColorPickerColor("Lime Taffy 308-4", Color.FromArgb(0xB7, 0xCA, 0xAD)),
							new ColorPickerColor("Pear Cactus 308-5", Color.FromArgb(0x8F, 0xA4, 0x7D)),
							new ColorPickerColor("Moss Ring 308-6", Color.FromArgb(0x70, 0x85, 0x5E)),
							new ColorPickerColor("Mountain Forest 308-7", Color.FromArgb(0x48, 0x59, 0x35)),
							new ColorPickerColor("Milkweed 309-1", Color.FromArgb(0xE0, 0xE4, 0xD3)),
							new ColorPickerColor("Lime Spritz 309-2", Color.FromArgb(0xD8, 0xDD, 0xC7)),
							new ColorPickerColor("Pale Moss Green 309-3", Color.FromArgb(0xCE, 0xD6, 0xBB)),
							new ColorPickerColor("Quaking Grass 309-4", Color.FromArgb(0xB8, 0xBD, 0x99)),
							new ColorPickerColor("Guacamole 309-5", Color.FromArgb(0x96, 0x9C, 0x71)),
							new ColorPickerColor("Moss Point Green 309-6", Color.FromArgb(0x7A, 0x81, 0x56)),
							new ColorPickerColor("Globe Artichoke 309-7", Color.FromArgb(0x5B, 0x5F, 0x32)),
							new ColorPickerColor("White Smoke 310-1", Color.FromArgb(0xE7, 0xE5, 0xD4)),
							new ColorPickerColor("Lime Wash 310-2", Color.FromArgb(0xDE, 0xDE, 0xC8)),
							new ColorPickerColor("Pickling Spice 310-3", Color.FromArgb(0xCF, 0xCC, 0xAB)),
							new ColorPickerColor("Bahia Grass 310-4", Color.FromArgb(0xB6, 0xB3, 0x8C)),
							new ColorPickerColor("Dill 310-5", Color.FromArgb(0x9A, 0x94, 0x68)),
							new ColorPickerColor("Featherstone 311-1", Color.FromArgb(0xE6, 0xE2, 0xD0)),
							new ColorPickerColor("River Reed 311-2", Color.FromArgb(0xDC, 0xD5, 0xBB)),
							new ColorPickerColor("Canary Grass 311-3", Color.FromArgb(0xCF, 0xC3, 0x9F)),
							new ColorPickerColor("Cress Green 311-4", Color.FromArgb(0xB9, 0xAA, 0x7F)),
							new ColorPickerColor("Pea Soup 311-5", Color.FromArgb(0x9F, 0x91, 0x65)),
							new ColorPickerColor("Prairie Winds 312-1", Color.FromArgb(0xE7, 0xE3, 0xD3)),
							new ColorPickerColor("Bamboo 312-2", Color.FromArgb(0xE1, 0xD8, 0xBE)),
							new ColorPickerColor("Safari 312-3", Color.FromArgb(0xCF, 0xBD, 0x97)),
							new ColorPickerColor("Nettle 312-4", Color.FromArgb(0xB7, 0xA1, 0x72)),
							new ColorPickerColor("Pineapple Sage 312-5", Color.FromArgb(0x96, 0x81, 0x55)),
							new ColorPickerColor("Pickled Okra 312-6", Color.FromArgb(0x80, 0x68, 0x3C)),
							new ColorPickerColor("Ivory Tower 313-1", Color.FromArgb(0xEF, 0xE7, 0xD5)),
							new ColorPickerColor("Almond Paste 313-2", Color.FromArgb(0xE4, 0xD6, 0xBC)),
							new ColorPickerColor("Soft Suede 313-3", Color.FromArgb(0xD6, 0xC3, 0xA3)),
							new ColorPickerColor("Earthy Cane 313-4", Color.FromArgb(0xC1, 0xA7, 0x80)),
							new ColorPickerColor("Rattan 313-5", Color.FromArgb(0x9F, 0x80, 0x56)),
							new ColorPickerColor("Twig Basket 313-7", Color.FromArgb(0x70, 0x56, 0x32)),
							new ColorPickerColor("Milk Paint 314-1", Color.FromArgb(0xEE, 0xE6, 0xD2)),
							new ColorPickerColor("Heavy Cream 314-2", Color.FromArgb(0xE8, 0xD8, 0xBE)),
							new ColorPickerColor("Sand Fossil 314-3", Color.FromArgb(0xDE, 0xC9, 0xAA)),
							new ColorPickerColor("Brandied Pears 315-2", Color.FromArgb(0xE8, 0xDD, 0xCA)),
							new ColorPickerColor("Chicory 316-6", Color.FromArgb(0xA0, 0x7A, 0x50)),
							new ColorPickerColor("Madonna Lily 317-1", Color.FromArgb(0xEC, 0xE3, 0xD5)),
							new ColorPickerColor("Pearl 317-2", Color.FromArgb(0xED, 0xE2, 0xD2)),
							new ColorPickerColor("Antique Parchment 317-3", Color.FromArgb(0xE8, 0xD2, 0xB9)),
							new ColorPickerColor("Chalkware 317-4", Color.FromArgb(0xDE, 0xC8, 0xAF)),
							new ColorPickerColor("Cracker Bitz 317-5", Color.FromArgb(0xCD, 0xA6, 0x7E)),
							new ColorPickerColor("Bread Basket 317-6", Color.FromArgb(0xA4, 0x79, 0x50)),
							new ColorPickerColor("Chewy Caramel 317-7", Color.FromArgb(0x8F, 0x63, 0x3A)),
							new ColorPickerColor("Malibu Dune 318-4", Color.FromArgb(0xE5, 0xC7, 0xAC)),
							new ColorPickerColor("Elegant Ivory 319-1", Color.FromArgb(0xF0, 0xE2, 0xCF)),
							new ColorPickerColor("Covered Wagon 319-5", Color.FromArgb(0xC9, 0x9E, 0x7B)),
							new ColorPickerColor("Bronco Brown 319-6", Color.FromArgb(0xA8, 0x7A, 0x56)),
							new ColorPickerColor("Apple Butter 319-7", Color.FromArgb(0x92, 0x63, 0x3E)),
							new ColorPickerColor("Honeysuckle 320-1", Color.FromArgb(0xEC, 0xE0, 0xD1)),
							new ColorPickerColor("Shoreland 320-2", Color.FromArgb(0xE8, 0xD4, 0xC3)),
							new ColorPickerColor("Ice Cream Cone 320-3", Color.FromArgb(0xE1, 0xCA, 0xB7)),
							new ColorPickerColor("Weathered Sandstone 320-4", Color.FromArgb(0xDD, 0xB8, 0x9D)),
							new ColorPickerColor("Teddy Bear Tan 321-5", Color.FromArgb(0xBB, 0x8A, 0x67)),
							new ColorPickerColor("Caramel Sundae 321-7", Color.FromArgb(0x93, 0x60, 0x3A)),
							new ColorPickerColor("Blush Beige 323-2", Color.FromArgb(0xEA, 0xD0, 0xBE)),
							new ColorPickerColor("Nosegay 324-1", Color.FromArgb(0xEC, 0xD6, 0xC7)),
							new ColorPickerColor("Starfish 324-3", Color.FromArgb(0xE1, 0xB5, 0x9B)),
							new ColorPickerColor("Orange Maple 324-4", Color.FromArgb(0xCD, 0x95, 0x79)),
							new ColorPickerColor("Sombrero Tan 325-4", Color.FromArgb(0xCA, 0xA3, 0x90)),
							new ColorPickerColor("Sablewood 326-2", Color.FromArgb(0xEA, 0xDB, 0xCF)),
							new ColorPickerColor("Sultan Sand 326-3", Color.FromArgb(0xDE, 0xC1, 0xB5)),
							new ColorPickerColor("Suntan 326-4", Color.FromArgb(0xD4, 0xA7, 0x94)),
							new ColorPickerColor("Nutmeg 326-6", Color.FromArgb(0x9E, 0x66, 0x50)),
							new ColorPickerColor("Spiced Cider 326-7", Color.FromArgb(0x86, 0x50, 0x3A)),
							new ColorPickerColor("Polo Tan 328-1", Color.FromArgb(0xF2, 0xE2, 0xD5)),
							new ColorPickerColor("Siesta Sands 329-1", Color.FromArgb(0xF0, 0xE3, 0xDC)),
							new ColorPickerColor("Slightly Peach 329-2", Color.FromArgb(0xEF, 0xD9, 0xD2)),
							new ColorPickerColor("Cool Cantaloupe 329-3", Color.FromArgb(0xEE, 0xCE, 0xC3)),
							new ColorPickerColor("Spanish Sherry 329-4", Color.FromArgb(0xE3, 0xC0, 0xB5)),
							new ColorPickerColor("Amaretto 329-6", Color.FromArgb(0xA0, 0x65, 0x56)),
							new ColorPickerColor("Chantilly Lace 330-1", Color.FromArgb(0xEF, 0xE0, 0xD9)),
							new ColorPickerColor("Potpourri 331-1", Color.FromArgb(0xEF, 0xDD, 0xD5)),
							new ColorPickerColor("Tangy Taffy 331-3", Color.FromArgb(0xE3, 0xC4, 0xBB)),
							new ColorPickerColor("Mesa Pink 331-4", Color.FromArgb(0xD7, 0xA9, 0x9F)),
							new ColorPickerColor("Creamy Melon 333-1", Color.FromArgb(0xEB, 0xD5, 0xCE)),
							new ColorPickerColor("Sweet Bianca 336-1", Color.FromArgb(0xEA, 0xD5, 0xD6)),
							new ColorPickerColor("Pink Pail 336-2", Color.FromArgb(0xE6, 0xC9, 0xCC)),
							new ColorPickerColor("Summer Clover 340-3", Color.FromArgb(0xE1, 0xCC, 0xD9)),
							new ColorPickerColor("Water Iris 345-1", Color.FromArgb(0xDC, 0xE0, 0xE5)),
							new ColorPickerColor("Violet Shadow 345-3", Color.FromArgb(0xCB, 0xD1, 0xDC)),
							new ColorPickerColor("Spring Lilac 345-4", Color.FromArgb(0xA7, 0xAF, 0xC4)),
							new ColorPickerColor("Arctic Dawn 346-1", Color.FromArgb(0xDF, 0xE2, 0xE2)),
							new ColorPickerColor("Bright Star 347-1", Color.FromArgb(0xD9, 0xE0, 0xE0)),
							new ColorPickerColor("Aster Petal 347-2", Color.FromArgb(0xD1, 0xD9, 0xDC)),
							new ColorPickerColor("Harbor Light 348-1", Color.FromArgb(0xD4, 0xDF, 0xE2)),
							new ColorPickerColor("Stratosphere 348-2", Color.FromArgb(0xC1, 0xD1, 0xDA)),
							new ColorPickerColor("Blue Thistle 348-3", Color.FromArgb(0xA4, 0xBB, 0xCF)),
							new ColorPickerColor("Baby's Breath 350-1", Color.FromArgb(0xD5, 0xE3, 0xE4)),
							new ColorPickerColor("Magic Wand 350-2", Color.FromArgb(0xBC, 0xD6, 0xDE)),
							new ColorPickerColor("Blue Bows 350-3", Color.FromArgb(0x9A, 0xBF, 0xCF)),
							new ColorPickerColor("Cosmopolitan 350-5", Color.FromArgb(0x47, 0x86, 0xA1)),
							new ColorPickerColor("Stunning Sapphire 350-7", Color.FromArgb(0x00, 0x56, 0x7A)),
							new ColorPickerColor("Pale Innocence 352-1", Color.FromArgb(0xD1, 0xDF, 0xDE)),
							new ColorPickerColor("Limerick 352-4", Color.FromArgb(0x7D, 0xAB, 0xB8)),
							new ColorPickerColor("Maiden Voyage 352-5", Color.FromArgb(0x56, 0x8D, 0x9E)),
							new ColorPickerColor("Sea Isle 352-6", Color.FromArgb(0x38, 0x73, 0x87)),
							new ColorPickerColor("Blue Starfish 352-7", Color.FromArgb(0x00, 0x5F, 0x78)),
							new ColorPickerColor("Aqua Sparkle 354-1", Color.FromArgb(0xCE, 0xE1, 0xE0)),
							new ColorPickerColor("Water Lily 354-2", Color.FromArgb(0xC9, 0xDE, 0xDD)),
							new ColorPickerColor("Everglade Mist 354-3", Color.FromArgb(0xB2, 0xD5, 0xD7)),
							new ColorPickerColor("Kingston Aqua 354-4", Color.FromArgb(0x86, 0xB7, 0xBB)),
							new ColorPickerColor("Acapulco Cliffs 354-5", Color.FromArgb(0x45, 0x95, 0x9E)),
							new ColorPickerColor("Caribbean Splash 354-7", Color.FromArgb(0x00, 0x68, 0x72)),
							new ColorPickerColor("Kiwi Splash 355-1", Color.FromArgb(0xD9, 0xE7, 0xE2)),
							new ColorPickerColor("Aqua Frost 355-2", Color.FromArgb(0xCF, 0xE0, 0xDD)),
							new ColorPickerColor("Turquoise Mist 355-3", Color.FromArgb(0xBC, 0xD8, 0xD6)),
							new ColorPickerColor("Alpine Valley 355-4", Color.FromArgb(0x8C, 0xBB, 0xBC)),
							new ColorPickerColor("Teal Zeal 355-6", Color.FromArgb(0x3A, 0x78, 0x7A)),
							new ColorPickerColor("Caribbean Holiday 355-7", Color.FromArgb(0x16, 0x65, 0x68)),
							new ColorPickerColor("Cameo Green 356-1", Color.FromArgb(0xD8, 0xE4, 0xDF)),
							new ColorPickerColor("Mountain Dew 356-2", Color.FromArgb(0xCC, 0xE0, 0xDB)),
							new ColorPickerColor("Misty Aqua 356-3", Color.FromArgb(0xB7, 0xD8, 0xD4)),
							new ColorPickerColor("Aqua Fiesta 356-4", Color.FromArgb(0x86, 0xBE, 0xBA)),
							new ColorPickerColor("Teal Bayou 356-5", Color.FromArgb(0x51, 0x9A, 0x95)),
							new ColorPickerColor("Jade Jewel 356-6", Color.FromArgb(0x23, 0x78, 0x76)),
							new ColorPickerColor("Tahitian Treat 356-7", Color.FromArgb(0x00, 0x66, 0x65)),
							new ColorPickerColor("Morning Fog 401-1", Color.FromArgb(0xCD, 0xD8, 0xD2)),
							new ColorPickerColor("Babbling Brook 401-3", Color.FromArgb(0xBA, 0xCB, 0xC6)),
							new ColorPickerColor("Holly Glen 401-4", Color.FromArgb(0x9C, 0xB1, 0xAD)),
							new ColorPickerColor("Evening Emerald 401-6", Color.FromArgb(0x51, 0x6B, 0x66)),
							new ColorPickerColor("Delicate Mist 402-1", Color.FromArgb(0xDD, 0xE8, 0xE0)),
							new ColorPickerColor("Blue Lullaby 402-2", Color.FromArgb(0xC3, 0xD2, 0xCB)),
							new ColorPickerColor("Sea Sprite 402-3", Color.FromArgb(0xB2, 0xC7, 0xBF)),
							new ColorPickerColor("Parakeet Pete 402-4", Color.FromArgb(0x8E, 0xAA, 0x9F)),
							new ColorPickerColor("Yucca 402-5", Color.FromArgb(0x6F, 0x8F, 0x85)),
							new ColorPickerColor("Geyser 403-2", Color.FromArgb(0xC1, 0xD3, 0xC8)),
							new ColorPickerColor("Summer Breeze 404-1", Color.FromArgb(0xCF, 0xE1, 0xD4)),
							new ColorPickerColor("Malted Mint 404-2", Color.FromArgb(0xBB, 0xD1, 0xBF)),
							new ColorPickerColor("Green Silk 404-3", Color.FromArgb(0x9F, 0xBB, 0xA6)),
							new ColorPickerColor("Fountain Mist 405-1", Color.FromArgb(0xD5, 0xE1, 0xD5)),
							new ColorPickerColor("Venetian Dew 405-2", Color.FromArgb(0xC3, 0xD5, 0xC8)),
							new ColorPickerColor("La Paloma 405-3", Color.FromArgb(0xAA, 0xC3, 0xB0)),
							new ColorPickerColor("Bonsai 405-4", Color.FromArgb(0x88, 0xAA, 0x94)),
							new ColorPickerColor("Garden's Glow 405-5", Color.FromArgb(0x66, 0x88, 0x72)),
							new ColorPickerColor("Bottle Green 405-7", Color.FromArgb(0x31, 0x52, 0x3D)),
							new ColorPickerColor("Icicle 406-1", Color.FromArgb(0xD9, 0xE3, 0xD8)),
							new ColorPickerColor("Limelight 406-3", Color.FromArgb(0xB9, 0xCA, 0xBB)),
							new ColorPickerColor("Silver Leaf 406-4", Color.FromArgb(0x99, 0xAF, 0x9B)),
							new ColorPickerColor("Slate Green 406-5", Color.FromArgb(0x75, 0x8C, 0x78)),
							new ColorPickerColor("Delicate Petal 407-2", Color.FromArgb(0xCF, 0xD8, 0xCD)),
							new ColorPickerColor("Mesa Verde 407-5", Color.FromArgb(0x7E, 0x92, 0x78)),
							new ColorPickerColor("Chives 407-6", Color.FromArgb(0x62, 0x72, 0x59)),
							new ColorPickerColor("Shady Knoll 407-7", Color.FromArgb(0x41, 0x50, 0x3A)),
							new ColorPickerColor("Cloudy Day 408-1", Color.FromArgb(0xDA, 0xE1, 0xD4)),
							new ColorPickerColor("Hurricane 408-2", Color.FromArgb(0xD3, 0xD9, 0xCD)),
							new ColorPickerColor("Falling Star 408-3", Color.FromArgb(0xC6, 0xCF, 0xBF)),
							new ColorPickerColor("Kiwi Kissed 408-4", Color.FromArgb(0xAD, 0xB9, 0xA5)),
							new ColorPickerColor("Pacific Pine 408-5", Color.FromArgb(0x86, 0x91, 0x79)),
							new ColorPickerColor("Hidden Meadow 408-6", Color.FromArgb(0x71, 0x79, 0x60)),
							new ColorPickerColor("Camouflage 408-7", Color.FromArgb(0x4C, 0x53, 0x3C)),
							new ColorPickerColor("Touch of Lime 409-1", Color.FromArgb(0xDE, 0xE1, 0xCF)),
							new ColorPickerColor("Pale Vista 409-2", Color.FromArgb(0xD6, 0xD9, 0xC6)),
							new ColorPickerColor("Frosty Pine 409-3", Color.FromArgb(0xC4, 0xC9, 0xB5)),
							new ColorPickerColor("Cavern Moss 409-5", Color.FromArgb(0x8C, 0x8D, 0x72)),
							new ColorPickerColor("Dark Sage 409-6", Color.FromArgb(0x66, 0x6A, 0x50)),
							new ColorPickerColor("Grape Leaves 409-7", Color.FromArgb(0x51, 0x55, 0x3E)),
							new ColorPickerColor("White Sage 410-2", Color.FromArgb(0xCF, 0xCE, 0xBB)),
							new ColorPickerColor("Olive Sprig 410-4", Color.FromArgb(0xA6, 0xA5, 0x8A)),
							new ColorPickerColor("Hemlock 410-5", Color.FromArgb(0x89, 0x86, 0x68)),
							new ColorPickerColor("Silvery Moon 411-1", Color.FromArgb(0xE2, 0xE2, 0xD5)),
							new ColorPickerColor("Veil Of Dusk 411-2", Color.FromArgb(0xD6, 0xD2, 0xC0)),
							new ColorPickerColor("Olive Green 411-7", Color.FromArgb(0x56, 0x4C, 0x35)),
							new ColorPickerColor("Witch Hazel 412-1", Color.FromArgb(0xE3, 0xE1, 0xD4)),
							new ColorPickerColor("Enduring Ice 413-1", Color.FromArgb(0xEA, 0xE4, 0xD4)),
							new ColorPickerColor("Fossil Stone 413-2", Color.FromArgb(0xE1, 0xD8, 0xC5)),
							new ColorPickerColor("Saddle Soap 413-5", Color.FromArgb(0x99, 0x84, 0x63)),
							new ColorPickerColor("Colonial White 414-1", Color.FromArgb(0xE7, 0xE0, 0xD0)),
							new ColorPickerColor("Southern Breeze 414-2", Color.FromArgb(0xE2, 0xDA, 0xC9)),
							new ColorPickerColor("Wheat Sheaf 415-3", Color.FromArgb(0xDB, 0xCD, 0xBB)),
							new ColorPickerColor("Mountain Gray 416-1", Color.FromArgb(0xE6, 0xE0, 0xD6)),
							new ColorPickerColor("Gravel 416-2", Color.FromArgb(0xE1, 0xD7, 0xC9)),
							new ColorPickerColor("Gray Beige 416-3", Color.FromArgb(0xD6, 0xCC, 0xBF)),
							new ColorPickerColor("Oyster Shell 416-5", Color.FromArgb(0x99, 0x87, 0x73)),
							new ColorPickerColor("Diamond Mine 417-1", Color.FromArgb(0xE7, 0xE1, 0xD5)),
							new ColorPickerColor("Shalestone 417-2", Color.FromArgb(0xE1, 0xD6, 0xC7)),
							new ColorPickerColor("Drifting Dune 417-3", Color.FromArgb(0xDB, 0xCF, 0xC1)),
							new ColorPickerColor("Sauteed Mushroom 417-5", Color.FromArgb(0xA4, 0x88, 0x6F)),
							new ColorPickerColor("Hat Box Brown 417-6", Color.FromArgb(0x87, 0x6C, 0x53)),
							new ColorPickerColor("Kodiak 417-7", Color.FromArgb(0x60, 0x4A, 0x38)),
							new ColorPickerColor("Pearls and Lace 419-1", Color.FromArgb(0xEC, 0xE2, 0xD5)),
							new ColorPickerColor("Moroccan Moonlight 419-2", Color.FromArgb(0xE8, 0xDB, 0xCD)),
							new ColorPickerColor("Taupe Trivia 419-4", Color.FromArgb(0xC8, 0xB0, 0x9D)),
							new ColorPickerColor("Cobblestone 419-5", Color.FromArgb(0xA1, 0x86, 0x71)),
							new ColorPickerColor("Chipmunk 419-6", Color.FromArgb(0x86, 0x6B, 0x56)),
							new ColorPickerColor("Classic Brown 419-7", Color.FromArgb(0x6B, 0x50, 0x3C)),
							new ColorPickerColor("String Of Pearls 420-1", Color.FromArgb(0xE8, 0xE0, 0xD3)),
							new ColorPickerColor("South Peak 421-1", Color.FromArgb(0xE9, 0xDB, 0xCC)),
							new ColorPickerColor("Oyster 422-1", Color.FromArgb(0xEB, 0xE2, 0xD6)),
							new ColorPickerColor("Ancient Stone 422-2", Color.FromArgb(0xE3, 0xD1, 0xC1)),
							new ColorPickerColor("Sea Sponge 422-3", Color.FromArgb(0xD4, 0xBE, 0xAE)),
							new ColorPickerColor("Puppy Paws 422-4", Color.FromArgb(0xC2, 0xA4, 0x8E)),
							new ColorPickerColor("Iced Tea 422-5", Color.FromArgb(0x9F, 0x7B, 0x65)),
							new ColorPickerColor("Covered Bridge 422-7", Color.FromArgb(0x71, 0x4E, 0x3A)),
							new ColorPickerColor("Gourmet Mushroom 423-1", Color.FromArgb(0xEC, 0xDF, 0xD2)),
							new ColorPickerColor("Pale Taupe 424-2", Color.FromArgb(0xE0, 0xCC, 0xC0)),
							new ColorPickerColor("Pueblo 424-3", Color.FromArgb(0xCA, 0xB3, 0xA6)),
							new ColorPickerColor("Moose Antler 424-4", Color.FromArgb(0xB7, 0x9C, 0x8D)),
							new ColorPickerColor("Chocolate Curl 424-6", Color.FromArgb(0x6B, 0x50, 0x41)),
							new ColorPickerColor("Almond Roca 425-1", Color.FromArgb(0xEF, 0xE5, 0xDA)),
							new ColorPickerColor("Apricot Buff 426-1", Color.FromArgb(0xE8, 0xD6, 0xCA)),
							new ColorPickerColor("Bon Bon 426-3", Color.FromArgb(0xD7, 0xBC, 0xB0)),
							new ColorPickerColor("Taupe Tone 426-4", Color.FromArgb(0xC1, 0xA0, 0x92)),
							new ColorPickerColor("Tiny Teddy 426-5", Color.FromArgb(0x9B, 0x7B, 0x6D)),
							new ColorPickerColor("Baby Bear 427-5", Color.FromArgb(0xA6, 0x7C, 0x6C)),
							new ColorPickerColor("Root Beer 427-6", Color.FromArgb(0x80, 0x59, 0x4C)),
							new ColorPickerColor("Butter Icing 428-1", Color.FromArgb(0xEC, 0xDE, 0xD3)),
							new ColorPickerColor("Warm Wassail 428-7", Color.FromArgb(0x7B, 0x45, 0x35)),
							new ColorPickerColor("Winter Peach 430-1", Color.FromArgb(0xE8, 0xD4, 0xCA)),
							new ColorPickerColor("China Silk 430-2", Color.FromArgb(0xDF, 0xCC, 0xC5)),
							new ColorPickerColor("Kangaroo Paw 430-3", Color.FromArgb(0xD8, 0xC3, 0xBC)),
							new ColorPickerColor("Velveteen Crush 430-4", Color.FromArgb(0xC7, 0xA6, 0x9C)),
							new ColorPickerColor("Bedford Brown 430-5", Color.FromArgb(0xA2, 0x7F, 0x77)),
							new ColorPickerColor("Apricot Cream 431-1", Color.FromArgb(0xEA, 0xD9, 0xD1)),
							new ColorPickerColor("Reindeer 431-3", Color.FromArgb(0xD4, 0xB9, 0xB2)),
							new ColorPickerColor("Parfait 431-4", Color.FromArgb(0xC1, 0x9F, 0x98)),
							new ColorPickerColor("Canyon Stone 431-6", Color.FromArgb(0x88, 0x58, 0x51)),
							new ColorPickerColor("Romeo 432-2", Color.FromArgb(0xE0, 0xCD, 0xC7)),
							new ColorPickerColor("Sweet Truffle 433-1", Color.FromArgb(0xEC, 0xD6, 0xCF)),
							new ColorPickerColor("Whirligig 433-2", Color.FromArgb(0xE1, 0xC7, 0xC2)),
							new ColorPickerColor("Mesa Red 434-6", Color.FromArgb(0x86, 0x4E, 0x51)),
							new ColorPickerColor("Crushed Roses 435-3", Color.FromArgb(0xD4, 0xB1, 0xB3)),
							new ColorPickerColor("Silk Sheets 436-1", Color.FromArgb(0xEB, 0xD9, 0xD9)),
							new ColorPickerColor("Rose Cloud 436-2", Color.FromArgb(0xDD, 0xC5, 0xC7)),
							new ColorPickerColor("Tic Tac Toe 437-1", Color.FromArgb(0xE1, 0xD9, 0xD8)),
							new ColorPickerColor("Rose Lake 437-2", Color.FromArgb(0xDD, 0xCB, 0xCE)),
							new ColorPickerColor("Smoke Pink 437-3", Color.FromArgb(0xD9, 0xBB, 0xC0)),
							new ColorPickerColor("Smoky Orchid 439-2", Color.FromArgb(0xDE, 0xD5, 0xD5)),
							new ColorPickerColor("Silver Mauve 439-3", Color.FromArgb(0xD6, 0xC7, 0xCC)),
							new ColorPickerColor("Rose Marble 439-4", Color.FromArgb(0xC9, 0xB4, 0xBC)),
							new ColorPickerColor("Lavender Twilight 440-1", Color.FromArgb(0xE3, 0xDD, 0xE0)),
							new ColorPickerColor("Wine Frost 440-2", Color.FromArgb(0xE1, 0xD5, 0xDA)),
							new ColorPickerColor("Pale Mauve 440-3", Color.FromArgb(0xD7, 0xC7, 0xD2)),
							new ColorPickerColor("Soft Amethyst 440-4", Color.FromArgb(0xC9, 0xB3, 0xC3)),
							new ColorPickerColor("Aspiration 442-1", Color.FromArgb(0xDD, 0xDA, 0xDC)),
							new ColorPickerColor("Silver Smoke 442-2", Color.FromArgb(0xD8, 0xD3, 0xD8)),
							new ColorPickerColor("Pearl Violet 442-3", Color.FromArgb(0xCC, 0xC7, 0xCF)),
							new ColorPickerColor("Victorian Iris 442-4", Color.FromArgb(0xB3, 0xAD, 0xB8)),
							new ColorPickerColor("Silver Chalice 443-3", Color.FromArgb(0xC8, 0xC4, 0xCC)),
							new ColorPickerColor("Cloudberry 443-4", Color.FromArgb(0xB1, 0xAD, 0xBA)),
							new ColorPickerColor("Violet Echo 444-1", Color.FromArgb(0xD9, 0xDA, 0xDE)),
							new ColorPickerColor("February Frost 444-2", Color.FromArgb(0xDB, 0xDA, 0xDD)),
							new ColorPickerColor("Frosted Lilac 444-3", Color.FromArgb(0xCD, 0xCD, 0xD5)),
							new ColorPickerColor("Silverberry 444-4", Color.FromArgb(0xB7, 0xB7, 0xC1)),
							new ColorPickerColor("Rare Orchid 445-2", Color.FromArgb(0xD7, 0xD9, 0xDC)),
							new ColorPickerColor("Glacier Pearl 445-3", Color.FromArgb(0xCC, 0xCF, 0xD5)),
							new ColorPickerColor("Timeless Lilac 445-4", Color.FromArgb(0xA6, 0xAD, 0xBC)),
							new ColorPickerColor("Moon Lily 446-1", Color.FromArgb(0xE2, 0xE4, 0xE2)),
							new ColorPickerColor("Cumberland Fog 446-2", Color.FromArgb(0xD6, 0xD8, 0xD9)),
							new ColorPickerColor("January Dawn 447-1", Color.FromArgb(0xDC, 0xE0, 0xDF)),
							new ColorPickerColor("Northern Exposure 447-3", Color.FromArgb(0xB9, 0xC3, 0xCD)),
							new ColorPickerColor("Innuendo 447-4", Color.FromArgb(0x9B, 0xAB, 0xBD)),
							new ColorPickerColor("Elusive Blue 448-1", Color.FromArgb(0xD8, 0xE1, 0xE2)),
							new ColorPickerColor("Memory Lane 448-2", Color.FromArgb(0xC0, 0xCD, 0xD5)),
							new ColorPickerColor("Pageant Song 448-3", Color.FromArgb(0xAE, 0xBE, 0xCB)),
							new ColorPickerColor("Always Blue 451-3", Color.FromArgb(0x99, 0xB5, 0xC3)),
							new ColorPickerColor("Breakwater 452-2", Color.FromArgb(0xCC, 0xDC, 0xDE)),
							new ColorPickerColor("Graceful 452-3", Color.FromArgb(0xA0, 0xBC, 0xC7)),
							new ColorPickerColor("Americana 452-4", Color.FromArgb(0x76, 0x98, 0xA6)),
							new ColorPickerColor("Granite Falls 452-5", Color.FromArgb(0x59, 0x7B, 0x8A)),
							new ColorPickerColor("Tradewinds 453-2", Color.FromArgb(0xC9, 0xD9, 0xDA)),
							new ColorPickerColor("Meadowsweet Mist 455-1", Color.FromArgb(0xD0, 0xE0, 0xDC)),
							new ColorPickerColor("Pistachio Cream 455-2", Color.FromArgb(0xD2, 0xDF, 0xDB)),
							new ColorPickerColor("Cosmic Rays 455-3", Color.FromArgb(0xC5, 0xD6, 0xD4)),
							new ColorPickerColor("Colonial Aqua 455-4", Color.FromArgb(0x9A, 0xB7, 0xB6)),
							new ColorPickerColor("Cathedral Glass 455-5", Color.FromArgb(0x71, 0x91, 0x92)),
							new ColorPickerColor("Vining Ivy 455-6", Color.FromArgb(0x43, 0x6B, 0x6D)),
							new ColorPickerColor("Deep Emerald 455-7", Color.FromArgb(0x23, 0x50, 0x53)),
							new ColorPickerColor("Mild Mint 456-1", Color.FromArgb(0xD8, 0xE3, 0xDD)),
							new ColorPickerColor("Spring Mist 456-2", Color.FromArgb(0xCE, 0xDD, 0xD8)),
							new ColorPickerColor("Julep Green 456-3", Color.FromArgb(0xC3, 0xD8, 0xD2)),
							new ColorPickerColor("Frosty Glade 456-4", Color.FromArgb(0x9A, 0xBA, 0xB6)),
							new ColorPickerColor("Palmetto 456-5", Color.FromArgb(0x66, 0x93, 0x91)),
							new ColorPickerColor("Rain Drop 501-1", Color.FromArgb(0xE5, 0xEB, 0xE3)),
							new ColorPickerColor("Cave Pearl 501-3", Color.FromArgb(0xD3, 0xE3, 0xDC)),
							new ColorPickerColor("Scarborough 501-5", Color.FromArgb(0x78, 0x8A, 0x87)),
							new ColorPickerColor("Spring Rain 502-1", Color.FromArgb(0xEA, 0xEF, 0xE7)),
							new ColorPickerColor("Northern Lights 502-2", Color.FromArgb(0xE3, 0xEC, 0xE3)),
							new ColorPickerColor("Silver Laurel 502-5", Color.FromArgb(0x7C, 0x97, 0x8C)),
							new ColorPickerColor("Salal Leaves 502-6", Color.FromArgb(0x5E, 0x75, 0x68)),
							new ColorPickerColor("Sea Pearl 503-1", Color.FromArgb(0xDD, 0xE6, 0xDD)),
							new ColorPickerColor("Rainsong 503-2", Color.FromArgb(0xDB, 0xE6, 0xDD)),
							new ColorPickerColor("Morning Breeze 503-3", Color.FromArgb(0xD3, 0xE0, 0xD8)),
							new ColorPickerColor("Blue Shamrock 503-4", Color.FromArgb(0xB5, 0xC5, 0xBD)),
							new ColorPickerColor("Spruce Shade 503-5", Color.FromArgb(0x8B, 0x9C, 0x93)),
							new ColorPickerColor("Dark Green Velvet 503-7", Color.FromArgb(0x43, 0x51, 0x47)),
							new ColorPickerColor("English Ivy 505-6", Color.FromArgb(0x63, 0x71, 0x60)),
							new ColorPickerColor("Pine Forest 505-7", Color.FromArgb(0x3C, 0x48, 0x38)),
							new ColorPickerColor("Steel City 506-4", Color.FromArgb(0xAB, 0xB2, 0xB0)),
							new ColorPickerColor("Distant Thunder 506-5", Color.FromArgb(0x7E, 0x88, 0x86)),
							new ColorPickerColor("Ash Mist 507-1", Color.FromArgb(0xDE, 0xDE, 0xD7)),
							new ColorPickerColor("Pale Smoke 507-2", Color.FromArgb(0xD9, 0xDA, 0xD2)),
							new ColorPickerColor("Silver Sword 507-3", Color.FromArgb(0xCE, 0xD0, 0xCA)),
							new ColorPickerColor("Georgian Silver 507-4", Color.FromArgb(0xA6, 0xAC, 0xA6)),
							new ColorPickerColor("Ash Essence 508-1", Color.FromArgb(0xE2, 0xE2, 0xD9)),
							new ColorPickerColor("Hosta Flower 508-3", Color.FromArgb(0xDE, 0xDC, 0xD1)),
							new ColorPickerColor("Tempered Steel 508-4", Color.FromArgb(0xA9, 0xB2, 0xAC)),
							new ColorPickerColor("Gray Heron 508-5", Color.FromArgb(0x81, 0x88, 0x7F)),
							new ColorPickerColor("Gunmetal Gray 508-6", Color.FromArgb(0x5C, 0x62, 0x59)),
							new ColorPickerColor("Charcoal Smoke 508-7", Color.FromArgb(0x41, 0x45, 0x3A)),
							new ColorPickerColor("Garlic Clove 510-1", Color.FromArgb(0xE9, 0xE6, 0xD9)),
							new ColorPickerColor("Balsam 510-4", Color.FromArgb(0xB9, 0xBD, 0xAE)),
							new ColorPickerColor("Green Tea Leaf 510-5", Color.FromArgb(0x8B, 0x90, 0x7E)),
							new ColorPickerColor("Castle Stone 510-7", Color.FromArgb(0x4A, 0x4C, 0x3C)),
							new ColorPickerColor("Macaroon Cream 511-1", Color.FromArgb(0xED, 0xE7, 0xD5)),
							new ColorPickerColor("Turtle Dove 511-4", Color.FromArgb(0xC1, 0xC0, 0xB4)),
							new ColorPickerColor("Autumn Gray 511-6", Color.FromArgb(0x79, 0x75, 0x63)),
							new ColorPickerColor("Fisherman's Net 512-4", Color.FromArgb(0xD1, 0xCA, 0xB5)),
							new ColorPickerColor("Olive Gray 512-5", Color.FromArgb(0xAA, 0x9D, 0x83)),
							new ColorPickerColor("Rattan Palm 512-6", Color.FromArgb(0x88, 0x7B, 0x60)),
							new ColorPickerColor("Grapevine 512-7", Color.FromArgb(0x58, 0x4C, 0x38)),
							new ColorPickerColor("Mother Of Pearl 513-1", Color.FromArgb(0xF2, 0xEC, 0xDB)),
							new ColorPickerColor("Whiskers 513-4", Color.FromArgb(0xCD, 0xC7, 0xBB)),
							new ColorPickerColor("Dark Ash 513-6", Color.FromArgb(0x85, 0x79, 0x6A)),
							new ColorPickerColor("Coffee Bean 513-7", Color.FromArgb(0x59, 0x4C, 0x3C)),
							new ColorPickerColor("Off White 516-1", Color.FromArgb(0xE6, 0xE0, 0xCF)),
							new ColorPickerColor("Antique White 516-2", Color.FromArgb(0xDE, 0xD5, 0xC2)),
							new ColorPickerColor("Tear Drop 517-2", Color.FromArgb(0xDE, 0xDD, 0xD5)),
							new ColorPickerColor("Fog 517-3", Color.FromArgb(0xD5, 0xD4, 0xCC)),
							new ColorPickerColor("Gray Stone 517-4", Color.FromArgb(0xB2, 0xB2, 0xAD)),
							new ColorPickerColor("Phoenix Fossil 517-5", Color.FromArgb(0x86, 0x88, 0x84)),
							new ColorPickerColor("Aria 518-2", Color.FromArgb(0xE1, 0xE2, 0xDE)),
							new ColorPickerColor("Thin Ice 518-3", Color.FromArgb(0xD5, 0xD8, 0xD5)),
							new ColorPickerColor("Dover Gray 518-5", Color.FromArgb(0x7C, 0x7C, 0x7B)),
							new ColorPickerColor("Seagull 520-3", Color.FromArgb(0xDD, 0xDB, 0xD2)),
							new ColorPickerColor("Slate Pebble 520-4", Color.FromArgb(0xAE, 0xA6, 0xA1)),
							new ColorPickerColor("Dark Granite 520-7", Color.FromArgb(0x47, 0x3C, 0x38)),
							new ColorPickerColor("Eiffel Tower 521-5", Color.FromArgb(0x91, 0x85, 0x78)),
							new ColorPickerColor("Granite 521-6", Color.FromArgb(0x6B, 0x5F, 0x55)),
							new ColorPickerColor("Star Anise 521-7", Color.FromArgb(0x55, 0x47, 0x3C)),
							new ColorPickerColor("Earl Gray 522-5", Color.FromArgb(0x9E, 0x8D, 0x7F)),
							new ColorPickerColor("Frangelica 523-4", Color.FromArgb(0xD9, 0xC7, 0xB7)),
							new ColorPickerColor("Country Beige 525-2", Color.FromArgb(0xEA, 0xE4, 0xD8)),
							new ColorPickerColor("Tanglewood 525-5", Color.FromArgb(0x9C, 0x86, 0x7A)),
							new ColorPickerColor("Derby Brown 525-6", Color.FromArgb(0x81, 0x68, 0x5C)),
							new ColorPickerColor("Fudge 525-7", Color.FromArgb(0x59, 0x41, 0x35)),
							new ColorPickerColor("Pearl Hat Pin 526-3", Color.FromArgb(0xDE, 0xD9, 0xD0)),
							new ColorPickerColor("Thumper 526-4", Color.FromArgb(0xB6, 0xA4, 0x9A)),
							new ColorPickerColor("Ranch Mink 526-5", Color.FromArgb(0x8D, 0x79, 0x6F)),
							new ColorPickerColor("Fond Memory 527-4", Color.FromArgb(0xC3, 0xB5, 0xAE)),
							new ColorPickerColor("Woodchuck 528-6", Color.FromArgb(0x84, 0x6A, 0x62)),
							new ColorPickerColor("Spiced Wine 528-7", Color.FromArgb(0x5C, 0x41, 0x3A)),
							new ColorPickerColor("Granite Boulder 529-6", Color.FromArgb(0x78, 0x66, 0x62)),
							new ColorPickerColor("Silver Feather 530-1", Color.FromArgb(0xEA, 0xE9, 0xE2)),
							new ColorPickerColor("Arctic Cotton 530-2", Color.FromArgb(0xE3, 0xE0, 0xD9)),
							new ColorPickerColor("Rose Mist 531-2", Color.FromArgb(0xE8, 0xE1, 0xDB)),
							new ColorPickerColor("Pewter Mug 531-5", Color.FromArgb(0x83, 0x7A, 0x79)),
							new ColorPickerColor("Black Elegance 531-7", Color.FromArgb(0x48, 0x41, 0x43)),
							new ColorPickerColor("Rose Dust 532-1", Color.FromArgb(0xEC, 0xE8, 0xE2)),
							new ColorPickerColor("Chalfonte 532-2", Color.FromArgb(0xEB, 0xE6, 0xE0)),
							new ColorPickerColor("Albatross 532-3", Color.FromArgb(0xDE, 0xD6, 0xD1)),
							new ColorPickerColor("Jack Rabbit 532-4", Color.FromArgb(0xB8, 0xAB, 0xA8)),
							new ColorPickerColor("Gray Violet 532-5", Color.FromArgb(0x93, 0x86, 0x85)),
							new ColorPickerColor("Rose Stain 536-4", Color.FromArgb(0xCD, 0xB0, 0xB2)),
							new ColorPickerColor("Coffee Rose 536-5", Color.FromArgb(0x9F, 0x80, 0x83)),
							new ColorPickerColor("Chocolate Sparkle 536-6", Color.FromArgb(0x80, 0x63, 0x66)),
							new ColorPickerColor("Milk and Cookies 537-3", Color.FromArgb(0xE7, 0xDE, 0xD9)),
							new ColorPickerColor("Ancestral 537-4", Color.FromArgb(0xCC, 0xBB, 0xBB)),
							new ColorPickerColor("Coffee Custard 537-5", Color.FromArgb(0xA3, 0x93, 0x93)),
							new ColorPickerColor("Choo Choo 537-6", Color.FromArgb(0x7B, 0x6B, 0x6D)),
							new ColorPickerColor("High Society 539-4", Color.FromArgb(0xC4, 0xB1, 0xB8)),
							new ColorPickerColor("Gothic Amethyst 539-5", Color.FromArgb(0x99, 0x83, 0x89)),
							new ColorPickerColor("Spring Thaw 541-3", Color.FromArgb(0xD4, 0xD8, 0xD6)),
							new ColorPickerColor("Silver Charm 541-4", Color.FromArgb(0xA6, 0xAA, 0xAC)),
							new ColorPickerColor("Victorian Pewter 541-5", Color.FromArgb(0x7A, 0x7B, 0x7E)),
							new ColorPickerColor("Gray Flannel 541-6", Color.FromArgb(0x50, 0x51, 0x56)),
							new ColorPickerColor("Napoleon 541-7", Color.FromArgb(0x38, 0x3A, 0x41)),
							new ColorPickerColor("Snow Storm 542-1", Color.FromArgb(0xEB, 0xEB, 0xE7)),
							new ColorPickerColor("Powderfresh 542-2", Color.FromArgb(0xE8, 0xE7, 0xE3)),
							new ColorPickerColor("Winter Morn 542-3", Color.FromArgb(0xE7, 0xE6, 0xE7)),
							new ColorPickerColor("Ash Grove 542-4", Color.FromArgb(0xB2, 0xAD, 0xB7)),
							new ColorPickerColor("Tin Lizzie 542-5", Color.FromArgb(0x88, 0x84, 0x8F)),
							new ColorPickerColor("Silverado 542-6", Color.FromArgb(0x60, 0x5C, 0x68)),
							new ColorPickerColor("Little Dipper 544-2", Color.FromArgb(0xE0, 0xE3, 0xE4)),
							new ColorPickerColor("Glistening Gray 545-4", Color.FromArgb(0xAA, 0xAD, 0xB5)),
							new ColorPickerColor("Flannel Pajamas 545-5", Color.FromArgb(0x83, 0x86, 0x90)),
							new ColorPickerColor("Alley Cat 545-6", Color.FromArgb(0x5C, 0x60, 0x6B)),
							new ColorPickerColor("Black Flame 545-7", Color.FromArgb(0x3E, 0x45, 0x51)),
							new ColorPickerColor("Calla Lily 546-1", Color.FromArgb(0xE1, 0xE7, 0xE7)),
							new ColorPickerColor("Blue Beard 546-4", Color.FromArgb(0xA3, 0xAF, 0xBB)),
							new ColorPickerColor("Prophetic Sea 546-5", Color.FromArgb(0x77, 0x84, 0x92)),
							new ColorPickerColor("Admiralty 546-7", Color.FromArgb(0x38, 0x47, 0x58)),
							new ColorPickerColor("Windswept 547-1", Color.FromArgb(0xE3, 0xEA, 0xE9)),
							new ColorPickerColor("City Lights 549-2", Color.FromArgb(0xDC, 0xE4, 0xE5)),
							new ColorPickerColor("Blue Dolphin 549-4", Color.FromArgb(0xB5, 0xC0, 0xC8)),
							new ColorPickerColor("Quicksilver 549-5", Color.FromArgb(0x8B, 0x98, 0xA0)),
							new ColorPickerColor("Sheffield Gray 549-6", Color.FromArgb(0x60, 0x6D, 0x77)),
							new ColorPickerColor("Summer Shower 550-1", Color.FromArgb(0xDE, 0xE9, 0xE7)),
							new ColorPickerColor("Prelude 550-2", Color.FromArgb(0xDD, 0xEA, 0xE8)),
							new ColorPickerColor("Simply Elegant 550-3", Color.FromArgb(0xC8, 0xDB, 0xE0)),
							new ColorPickerColor("Canyon Blue 550-6", Color.FromArgb(0x56, 0x75, 0x85)),
							new ColorPickerColor("Aloof 551-2", Color.FromArgb(0xD0, 0xE0, 0xE0)),
							new ColorPickerColor("Dartmoor Mist 551-3", Color.FromArgb(0xC8, 0xD9, 0xDB)),
							new ColorPickerColor("Ice Castles 552-1", Color.FromArgb(0xE6, 0xEC, 0xE8)),
							new ColorPickerColor("Blue Smoke 552-3", Color.FromArgb(0xD3, 0xDD, 0xDC)),
							new ColorPickerColor("Chalky Blue 552-5", Color.FromArgb(0x72, 0x87, 0x8F)),
							new ColorPickerColor("Silent Night 552-6", Color.FromArgb(0x4A, 0x5F, 0x67)),
							new ColorPickerColor("Evening Mist 554-1", Color.FromArgb(0xDF, 0xE6, 0xE3)),
							new ColorPickerColor("Dewdrop 554-2", Color.FromArgb(0xD9, 0xE0, 0xDB)),
							new ColorPickerColor("April Sky 554-3", Color.FromArgb(0xCF, 0xD9, 0xD7)),
							new ColorPickerColor("Smoke Screen 554-4", Color.FromArgb(0xA6, 0xB0, 0xB1)),
							new ColorPickerColor("Feldspar 554-5", Color.FromArgb(0x7C, 0x87, 0x8A)),
							new ColorPickerColor("Lava Gray 554-6", Color.FromArgb(0x56, 0x5F, 0x63)),
							new ColorPickerColor("Midnight Hour 554-7", Color.FromArgb(0x32, 0x3E, 0x43)),
							new ColorPickerColor("Icy Bay 555-1", Color.FromArgb(0xDD, 0xE3, 0xDE)),
							new ColorPickerColor("Inverness Gray 555-2", Color.FromArgb(0xD9, 0xE0, 0xDB)),
							new ColorPickerColor("Tinsel 555-3", Color.FromArgb(0xCC, 0xD5, 0xD4)),
							new ColorPickerColor("Gray Frost 555-4", Color.FromArgb(0xB2, 0xBB, 0xBB)),
							new ColorPickerColor("Steeple Gray 555-5", Color.FromArgb(0x86, 0x8E, 0x8F)),
							new ColorPickerColor("Volcanic Ash 555-6", Color.FromArgb(0x66, 0x6D, 0x6F)),
							new ColorPickerColor("Black Forest 555-7", Color.FromArgb(0x3C, 0x45, 0x47)),
							new ColorPickerColor("Morning Song 556-1", Color.FromArgb(0xE0, 0xE9, 0xE3)),
							new ColorPickerColor("Honesty 556-2", Color.FromArgb(0xDA, 0xE9, 0xE4)),
							new ColorPickerColor("Pristine Petal 556-3", Color.FromArgb(0xD1, 0xDE, 0xDA)),
							new ColorPickerColor("Misty Surf 556-4", Color.FromArgb(0xB0, 0xC3, 0xC1)),
							new ColorPickerColor("Aqua Smoke 556-5", Color.FromArgb(0x85, 0x97, 0x97)),
							new ColorPickerColor("Blue Blood 556-6", Color.FromArgb(0x63, 0x76, 0x77)),
							new ColorPickerColor("Mountain Pine 556-7", Color.FromArgb(0x35, 0x4A, 0x4C)),
						}),
				}),
*/
			#endregion
		};

		#endregion

		#region Types

		private struct ColorPickerColor
		{
			#region Fields

			private string _name;
			private Color _value;

			#endregion

			#region Methods

			public ColorPickerColor(
				string name,
				Color value)
			{
				this._name = name;
				this._value = value;
			}

			#endregion

			#region Properties

			public string Name
			{
				get { return this._name; }
			}

			public Color Value
			{
				get { return this._value; }
			}

			#endregion
		}

		private struct ColorPickerColorGroup
		{
			#region Fields

			private string _name;
			private ColorPickerColor[] _colors;

			#endregion

			#region Methods

			public ColorPickerColorGroup(
				string name,
				ColorPickerColor[] colors)
			{
				this._name = name;
				this._colors = colors;
			}

			#endregion

			#region Properties

			public string Name
			{
				get { return this._name; }
			}

			public IEnumerable<ColorPickerColor> Colors
			{
				get { return this._colors; }
			}

			#endregion
		}

		private struct ColorPickerColorPalette
		{
			#region Fields

			private string _name;
			private string _code;
			private ColorPickerColorGroup[] _groups;

			#endregion

			#region Methods

			public ColorPickerColorPalette(
				string name,
				string code,
				ColorPickerColorGroup[] groups)
			{
				this._name = name;
				this._code = code;
				this._groups = groups;
			}

			#endregion

			#region Properties

			public string Name
			{
				get { return this._name; }
			}

			public string Code
			{
				get { return this._code; }
			}

			public IEnumerable<ColorPickerColorGroup> Groups
			{
				get { return this._groups; }
			}

			#endregion
		}

		#endregion

		#region Fields

		private string _palette = string.Empty;
		private Color _color = Color.Transparent;
		private Color _colorInitial = Color.Transparent;
		private ColorHSL _colorPictureHSL_L = new ColorHSL();
		private List<ListView> _listViews = new List<ListView>();
		private bool _updatingColor;

		#endregion

		#region Methods

		#region Constructors

		public FormColorPicker()
		{
			InitializeComponent();

			this._mapHSL_HS.SelectionMin = new PointF(ColorHSL.HMin, ColorHSL.SMin);
			this._mapHSL_HS.SelectionMax = new PointF(ColorHSL.HMax, ColorHSL.SMax);

			this._mapHSL_L.SelectionMin = new PointF(0.0F, ColorHSL.LMin);
			this._mapHSL_L.SelectionMax = new PointF(1.0F, ColorHSL.LMax);

			this._tabPalette.SuspendLayout();

			foreach (ColorPickerColorPalette palette in _palletes)
			{
				ListView listView = new ListView();

				this._listViews.Add(listView);

				listView.Activation = ItemActivation.OneClick;
				listView.Alignment = ListViewAlignment.SnapToGrid;
				listView.Dock = DockStyle.Fill;
				listView.FullRowSelect = true;
				listView.HotTracking = true;
				listView.HoverSelection = true;
				listView.Location = new Point(0, 0);
				listView.Margin = new Padding(0);
				listView.MultiSelect = false;
				listView.OwnerDraw = true;
				listView.ShowItemToolTips = true;
				listView.TileSize = new Size(32, 24);
				listView.UseCompatibleStateImageBehavior = false;
				listView.View = View.Tile;
				listView.MouseClick += this.ListView_MouseClick;
				listView.DrawItem += this.ListView_DrawItem;
				listView.SelectedIndexChanged += this.ListView_SelectedIndexChanged;
				listView.Tag = palette.Code;

				LoadColorSet(listView, palette);

				TabPage tabPage = new TabPage(palette.Name);

				tabPage.Margin = new Padding(0);
				tabPage.UseVisualStyleBackColor = true;
				tabPage.Controls.Add(listView);
				tabPage.Tag = listView;

				this._tabPalette.TabPages.Add(tabPage);
			}

			this._tabPalette.ResumeLayout(true);
		}

		#endregion

		#region Loading

		private static void LoadColorSet(ListView listView, ColorPickerColorPalette palette)
		{
			foreach (ColorPickerColorGroup colorGroup in palette.Groups)
			{
				ListViewGroup lvGroup = listView.Groups.Add(colorGroup.Name, colorGroup.Name);

				foreach (ColorPickerColor color in colorGroup.Colors)
				{
					ListViewItem lvItem = new ListViewItem(lvGroup);

					lvItem.Tag = color.Value;
					lvItem.ToolTipText = color.Name;

					listView.Items.Add(lvItem);
				}
			}
		}

		#endregion

		#region Form

		protected override void OnDeactivate(EventArgs e)
		{
			base.OnDeactivate(e);
			base.DialogResult = DialogResult.Cancel;
			base.Close();
		}

		#endregion

		#region TabPage Palette

		private void ListView_DrawItem(object sender, DrawListViewItemEventArgs e)
		{
			ListView listView = (ListView)sender;

			if ((e.ItemIndex >= 0) && (e.ItemIndex < listView.Items.Count))
			{
				ListViewItem lvItem = e.Item;
				Color vlItemColor = (Color)lvItem.Tag;
				Rectangle rectContent = e.Bounds;
				Rectangle rectFrame = e.Bounds;
				Brush brushItemColor = new SolidBrush(vlItemColor);
				Pen penForeColor = new Pen(listView.ForeColor, 2.0f);

				if (vlItemColor.ToArgb() == this._colorInitial.ToArgb())
				{
					rectContent.Inflate(-3, -3);
					e.Graphics.DrawRectangle(penForeColor, rectContent);
					e.Graphics.FillRectangle(brushItemColor, rectContent);

					rectFrame.Inflate(1, 1);
					e.Graphics.DrawRectangle(penForeColor, rectFrame);
				}
				else
				{
					rectContent.Inflate(-3, -3);
					e.Graphics.DrawRectangle(penForeColor, rectContent);
					e.Graphics.FillRectangle(brushItemColor, rectContent);
				}

				e.DrawFocusRectangle();
			}
		}

		private void ListView_SelectedIndexChanged(object sender, EventArgs e)
		{
			ListView listView = (ListView)sender;

			if (listView.SelectedIndices.Count == 1)
			{
				this._palette = (string)listView.Tag;

				ColorRGB colorRGB = ColorRGB.FromColor((Color)listView.SelectedItems[0].Tag);
				ColorHSL colorHSL = ColorSpaceConverter.HSLFromRGB(colorRGB);
				ColorYUV colorYUV = ColorSpaceConverter.YUVFromRGB(colorRGB);

				UpdateRGB(colorRGB);
				UpdateHSL(colorHSL);
				UpdateYUV(colorYUV);

				UpdatePictureHSL_HS(colorHSL);
				UpdatePictureHSL_L(colorHSL);
				UpdateColorPreview(colorRGB);
			}
		}

		private void ListView_MouseClick(object sender, MouseEventArgs e)
		{
			ListView listView = (ListView)sender;

			if (listView.SelectedIndices.Count == 1)
			{
				this._palette = (string)listView.Tag;

				ColorRGB colorRGB = ColorRGB.FromColor((Color)listView.SelectedItems[0].Tag);
				ColorHSL colorHSL = ColorSpaceConverter.HSLFromRGB(colorRGB);
				ColorYUV colorYUV = ColorSpaceConverter.YUVFromRGB(colorRGB);

				UpdateRGB(colorRGB);
				UpdateHSL(colorHSL);
				UpdateYUV(colorYUV);

				base.DialogResult = DialogResult.OK;
				base.Close();
			}
		}

		#endregion

		#region TabPage Custom

		private static float GetCustomColorValue(NumericUpDown numeric, float min, float max)
		{
			return
				((float)((numeric.Value - numeric.Minimum) / (numeric.Maximum - numeric.Minimum))) *
				(max - min) + min;
		}

		private static void SetCustomColorValue(NumericUpDown numeric, float min, float max, float value)
		{
			numeric.Value =
				((decimal)((value - min) / (max - min))) *
				(numeric.Maximum - numeric.Minimum) + numeric.Minimum;
		}

		private void UpdateColorPreview(ColorRGB colorRGB)
		{
			this._color = colorRGB.ToColor();

			string colorText =
				string.Format(
					CultureInfo.InvariantCulture,
					"RGB: {0:X2}{1:X2}{2:X2} ({0}; {1}; {2})",
					this._color.R,
					this._color.G,
					this._color.B);

			this._labelColorFore.ForeColor = this._color;
			this._labelColorFore.Text = colorText;
			this._labelColorFore.Refresh();
			this._labelColorBack.BackColor = this._color;
			this._labelColorBack.Text = colorText;
			this._labelColorBack.Refresh();
		}

		private void UpdatePictureHSL_HS(ColorHSL colorHSL)
		{
			if (colorHSL.L > 0.0F)
			{
				this._mapHSL_HS.SelectionPosition = new PointF(colorHSL.H, colorHSL.S);
			}
			else
			{
				this._mapHSL_HS.SelectionPosition = new PointF(0.0F, 0.0F);
			}

			this._mapHSL_L.SelectionPosition = new PointF(0.0F, ColorHSL.LMax - colorHSL.L);
		}

		private void UpdatePictureHSL_L(ColorHSL colorHSL)
		{
			if (
				(Math.Abs(this._colorPictureHSL_L.H - colorHSL.H) > ((ColorHSL.HMax - ColorHSL.HMin) / 1000.0F)) ||
				(Math.Abs(this._colorPictureHSL_L.S - colorHSL.S) > ((ColorHSL.SMax - ColorHSL.SMin) / 1000.0F)) ||
				(Math.Abs(this._colorPictureHSL_L.L - colorHSL.L) > ((ColorHSL.LMax - ColorHSL.LMin) / 1000.0F)))
			{
				this._colorPictureHSL_L = colorHSL;

				Bitmap bitmap = new Bitmap(this._mapHSL_L.ClientSize.Width / 4, 256, PixelFormat.Format24bppRgb);
				BitmapData bitmapData = bitmap.LockBits(
					new Rectangle(0, 0, bitmap.Width, bitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format24bppRgb);

				for (int y = 0; y < bitmap.Height; y++)
				{
					colorHSL.L = ((float)((bitmap.Height - 1) - y)) / bitmap.Height;

					ColorRGB colorRGB = ColorSpaceConverter.RGBFromHSL(colorHSL);

					for (int x = 0; x < bitmap.Width; x++)
					{
						Color color = colorRGB.ToColor();
						int offset = bitmapData.Stride * y + 3 * x;

						Marshal.WriteByte(bitmapData.Scan0, offset + 0, color.B);
						Marshal.WriteByte(bitmapData.Scan0, offset + 1, color.G);
						Marshal.WriteByte(bitmapData.Scan0, offset + 2, color.R);
					}
				}

				bitmap.UnlockBits(bitmapData);

				this._mapHSL_L.Image = bitmap;
			}
		}

		private void UpdateRGB(ColorRGB colorRGB)
		{
			SetCustomColorValue(this._numericRGB_R, ColorRGB.RMin, ColorRGB.RMax, colorRGB.R);
			SetCustomColorValue(this._numericRGB_G, ColorRGB.GMin, ColorRGB.GMax, colorRGB.G);
			SetCustomColorValue(this._numericRGB_B, ColorRGB.BMin, ColorRGB.BMax, colorRGB.B);
		}

		private void UpdateHSL(ColorHSL colorHSL)
		{
			SetCustomColorValue(this._numericHSL_H, ColorHSL.HMin, ColorHSL.HMax, colorHSL.H);
			SetCustomColorValue(this._numericHSL_S, ColorHSL.SMin, ColorHSL.SMax, colorHSL.S);
			SetCustomColorValue(this._numericHSL_L, ColorHSL.LMin, ColorHSL.LMax, colorHSL.L);
		}

		private void UpdateYUV(ColorYUV colorYUV)
		{
			SetCustomColorValue(this._numericYUV_Y, ColorYUV.YMin, ColorYUV.YMax, colorYUV.Y);
			SetCustomColorValue(this._numericYUV_U, ColorYUV.UMin, ColorYUV.UMax, colorYUV.U);
			SetCustomColorValue(this._numericYUV_V, ColorYUV.VMin, ColorYUV.VMax, colorYUV.V);
		}

		private void PictureHSL_HS_SelectionPositionChanged(object sender, EventArgs e)
		{
			if (!this._updatingColor)
			{
				this._updatingColor = true;

				ColorHSL colorHSL = new ColorHSL(
					this._mapHSL_HS.SelectionPosition.X,
					this._mapHSL_HS.SelectionPosition.Y,
					GetCustomColorValue(this._numericHSL_L, ColorHSL.LMin, ColorHSL.LMax));
				ColorRGB colorRGB = ColorSpaceConverter.RGBFromHSL(colorHSL);
				ColorYUV colorYUV = ColorSpaceConverter.YUVFromRGB(colorRGB);

				UpdateRGB(colorRGB);
				UpdateHSL(colorHSL);
				UpdateYUV(colorYUV);

				UpdatePictureHSL_L(colorHSL);
				UpdateColorPreview(colorRGB);

				this._updatingColor = false;
			}
		}

		private void PictureHSL_L_SelectionPositionChanged(object sender, EventArgs e)
		{
			if (!this._updatingColor)
			{
				this._updatingColor = true;

				ColorHSL colorHSL = new ColorHSL(
					GetCustomColorValue(this._numericHSL_H, ColorHSL.LMin, ColorHSL.LMax),
					GetCustomColorValue(this._numericHSL_S, ColorHSL.LMin, ColorHSL.LMax),
					ColorHSL.LMax - this._mapHSL_L.SelectionPosition.Y);
				ColorRGB colorRGB = ColorSpaceConverter.RGBFromHSL(colorHSL);
				ColorYUV colorYUV = ColorSpaceConverter.YUVFromRGB(colorRGB);

				UpdateRGB(colorRGB);
				UpdateHSL(colorHSL);
				UpdateYUV(colorYUV);

				UpdatePictureHSL_HS(colorHSL);

				if (colorHSL.L == 0.0F)
				{
					UpdatePictureHSL_L(colorHSL);
				}

				UpdateColorPreview(colorRGB);

				this._updatingColor = false;
			}
		}

		private void NumericRGB_ValueChanged(object sender, EventArgs e)
		{
			if (!this._updatingColor)
			{
				this._updatingColor = true;

				ColorRGB colorRGB = new ColorRGB(
					GetCustomColorValue(this._numericRGB_R, ColorRGB.RMin, ColorRGB.RMax),
					GetCustomColorValue(this._numericRGB_G, ColorRGB.GMin, ColorRGB.GMax),
					GetCustomColorValue(this._numericRGB_B, ColorRGB.BMin, ColorRGB.BMax));

				ColorHSL colorHSL = ColorSpaceConverter.HSLFromRGB(colorRGB);
				ColorYUV colorYUV = ColorSpaceConverter.YUVFromRGB(colorRGB);

				UpdateHSL(colorHSL);
				UpdateYUV(colorYUV);
				UpdatePictureHSL_HS(colorHSL);
				UpdatePictureHSL_L(colorHSL);

				this._updatingColor = false;
			}
		}

		private void NumericHSL_ValueChanged(object sender, EventArgs e)
		{
			if (!this._updatingColor)
			{
				this._updatingColor = true;

				ColorHSL colorHSL = new ColorHSL(
					GetCustomColorValue(this._numericHSL_H, ColorHSL.HMin, ColorHSL.HMax),
					GetCustomColorValue(this._numericHSL_S, ColorHSL.SMin, ColorHSL.SMax),
					GetCustomColorValue(this._numericHSL_L, ColorHSL.LMin, ColorHSL.LMax));
				ColorRGB colorRGB = ColorSpaceConverter.RGBFromHSL(colorHSL);
				ColorYUV colorYUV = ColorSpaceConverter.YUVFromRGB(colorRGB);

				UpdateRGB(colorRGB);
				UpdateYUV(colorYUV);
				UpdatePictureHSL_HS(colorHSL);
				UpdatePictureHSL_L(colorHSL);

				this._updatingColor = false;
			}
		}

		private void NumericYUV_ValueChanged(object sender, EventArgs e)
		{
			if (!this._updatingColor)
			{
				this._updatingColor = true;

				ColorYUV colorYUV = new ColorYUV(
					GetCustomColorValue(this._numericYUV_Y, ColorYUV.YMin, ColorYUV.YMax),
					GetCustomColorValue(this._numericYUV_U, ColorYUV.UMin, ColorYUV.UMax),
					GetCustomColorValue(this._numericYUV_V, ColorYUV.VMin, ColorYUV.VMax));
				ColorRGB colorRGB = ColorSpaceConverter.RGBFromYUV(colorYUV);
				ColorHSL colorHSL = ColorSpaceConverter.HSLFromRGB(colorRGB);

				UpdateRGB(colorRGB);
				UpdateHSL(colorHSL);
				UpdatePictureHSL_HS(colorHSL);
				UpdatePictureHSL_L(colorHSL);

				this._updatingColor = false;
			}
		}

		private void ButtonOK_Click(object sender, EventArgs e)
		{
			this._palette = string.Empty;
			base.DialogResult = DialogResult.OK;
			base.Close();
		}

		private void ButtonCancel_Click(object sender, EventArgs e)
		{
			base.DialogResult = DialogResult.Cancel;
			base.Close();
		}

		#endregion

		#endregion

		#region Properties

		public string Palette
		{
			get
			{
				return this._palette;
			}

			set
			{
				if (string.IsNullOrEmpty(value))
				{
					this._palette = string.Empty;
				}
				else
				{
					this._tabPalette.SelectedIndex = this._tabPalette.TabPages.Count - 1;

					for (int index = 0; index < this._tabPalette.TabPages.Count - 1; index++)
					{
						TabPage tabPage = this._tabPalette.TabPages[index];
						ListView listView = (ListView)tabPage.Tag;

						if (listView != null)
						{
							if (listView.Tag.ToString() == value)
							{
								this._tabPalette.SelectedIndex = index;
								this._palette = value;
								break;
							}
						}
					}
				}
			}
		}

		public Color Color
		{
			get { return this._color; }
		}

		public Color ColorInitial
		{
			get
			{
				return this._colorInitial;
			}

			set
			{
				this._colorInitial = value;

				ColorRGB colorRGB = ColorRGB.FromColor(value);
				ColorHSL colorHSL = ColorSpaceConverter.HSLFromRGB(colorRGB);
				ColorYUV colorYUV = ColorSpaceConverter.YUVFromRGB(colorRGB);

				UpdateRGB(colorRGB);
				UpdateHSL(colorHSL);
				UpdateYUV(colorYUV);
				UpdateColorPreview(colorRGB);
			}
		}

		#endregion

		#endregion

		#region Control

		protected override void OnKeyPress(KeyPressEventArgs e)
		{
			switch (e.KeyChar)
			{
				case '\x1B':
					base.DialogResult = DialogResult.Cancel;
					base.Close();
					break;
				case '\r':
					foreach (ListView listView in this._listViews)
					{
						if ((listView.Focused) && (listView.SelectedIndices.Count == 1))
						{
							this._palette = (string)listView.Tag;

							ColorRGB colorRGB = ColorRGB.FromColor((Color)listView.SelectedItems[0].Tag);
							ColorHSL colorHSL = ColorSpaceConverter.HSLFromRGB(colorRGB);
							ColorYUV colorYUV = ColorSpaceConverter.YUVFromRGB(colorRGB);

							UpdateRGB(colorRGB);
							UpdateHSL(colorHSL);
							UpdateYUV(colorYUV);

							base.DialogResult = DialogResult.OK;
							base.Close();
							break;
						}
					}
					break;
			}
		}

		#endregion
	}
}
/******************************************************************************/
/*                                END OF FILE                                 */
/******************************************************************************/
